      PROGRAM TLUSTY
C
C     Non-LTE model stellar atmospheres computer program
C
C ****************** Version TLUSTY194 ******************
C
C     Appropriate references for this version are:
C
C     Hubeny, I. 1988, Comput. Phys. Comm., 52, 103
C     Hubeny, I., Lanz, T. 1992, A&A, 262, 501
C     Hubeny, I., Lanz, T. 1995, ApJ, 439, 875
C
C     A User's Guide has been published in:
C
C     Hubeny, I., Lanz, T., Jeffery, C.S. 1994, in: Newsletter on
C        Analysis of Astronomical Spectra No 20, St. Andrews Univ.,
C        p. 30-42.
C
C     The User's Guide is available on the WWW server of CCP7:
C        ftp://ccp7.st-and.ac.uk/ccp7/ccp7_www.html
C
C *******************************************************
C
C     Version modified with respect to the last CCP7 version (Tlusty160):
C
C     - hybrid complete linearization/ALI method
C     - ODF treatment of line blanketing
C     - splitting of statistical equilibrium (SE) - ie. the set of SE
C       does not have to be solved as a whole (for all species together);
C       instead it may be solved for each species separately
C     - introduction of reference atom and levels; hydrogen does not have
C       to be a reference atom; one may calculate models without any
C       hydrogen
C     - chosen levels and/or whole ions may be kept in LTE, even if the
C       model is NLTE
C     - improved treatment of radiative equilibrium
C     - a more systematic and accurate treatment of overlapping hydrogen
C       lines
C     - better vectorization 
C
C     Version 160 was modified with respect to the published one in 
C     Comp.Phys.Comm. 52, 103 (1988):
C
C     - allowance for convection (mixing length)
C     - possibility of using the collisional-radiative switching 
C       procedure (Hummer and Voels)
C     - improved treatment of hydrogen Lyman and Balmer lines:
C       they may easily be considered with an approximate (but very
C       good) Stark+Doppler broadening; also opacity of high members
C       Lyman and Balmer series, plus corresponding pseudocontinuum
C       due to a lowering of ionization potential may easily be
C       included;
C       pseudocontinuum may be considered also for He II
C     - improved standard He I photoionization cross-sections and
C       collisional rates
C     - more user-friendly input of transition parameters and
C       albedos
C     - possibility of different number of fixed rates recalculations
C       after different global iterations of complete linearization
C     - improved Feautrier scheme (Hummer and Rybicki 1991)
C     - accelerated convergence procedures
C     - various minor modifications
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ALIPAR.FOR'
C
C
      OPEN(UNIT=91,STATUS='SCRATCH',FORM='UNFORMATTED')
      OPEN(UNIT=92,STATUS='SCRATCH',FORM='UNFORMATTED')
      OPEN(UNIT=93,STATUS='SCRATCH',FORM='UNFORMATTED')
C
C     Initialization
C
      INIT=1
      CALL START
      ITER=0
      LFIN=.FALSE.
      IF(NITER.EQ.0) LFIN=.TRUE.
C
C     Basic iteration loop of the hybrid CL/ALI method:
C
   10 ITER=ITER+1
C
C     1. Formal solution step
C
      CALL RESOLV
c     CALL TIMING(1,ITER)
      INIT=0
      IF(LFIN) GO TO 20
C
C     1b. Acceleration of convergence
C
      IF(IACC.GT.0) CALL ACCEL2
c     IF(LAC2) GO TO 10
C
C     2. Solution of the linearized equations
C
      IF(NN.GT.MSMX) THEN
         CALL SOLVE
      ELSE
         IF(IFALI.LE.7) THEN
            CALL SOLVES
          ELSE
            CALL SOLVE4
         END IF
      END IF
      CALL TIMING(2,ITER)
      GO TO 10
   20 CONTINUE
      STOP
      END

C
C
C     ****************************************************************
C
C

      BLOCK DATA
C     ==========
C
C     Hydrogenic oscillator strentghs
C
      INCLUDE 'IMPLIC.FOR'
C
      COMMON/OSCHYD/OSH(20,20)
C
      DATA ((OSH(I,J),I=1,20),J=1,16)/20*0.,
     * 0.4162,19*0.,7.910D-2,0.6407,18*0.,2.899D-2,0.1193,
     * 0.8421,17*0.,1.394D-2,4.467D-2,0.1506,1.038,16*0.,7.799D-3,
     * 2.209D-2,5.584D-2,0.1793,1.231,15*0.,4.814D-3,1.270D-2,2.768D-2,
     * 6.549D-2,0.2069,1.424,14*0.,3.183D-3,8.036D-3,1.604D-2,3.23D-2,
     * 7.448D-2,0.234,1.616,13*0.,2.216D-3,5.429D-3,1.023D-2,1.87D-2,
     * 3.645D-2,8.315D-2,0.2609,1.807,12*0.,1.605D-3,3.851D-3,6.98D-3,
     * 1.196D-2,2.104D-2,4.038D-2,9.163D-2,0.2876,1.999,11*0.,1.201D-3,
     * 2.835D-3,4.996D-3,8.187D-3,1.344D-2,2.32D-2,4.416D-2,0.1,0.3143,
     * 2.19,10*0.,9.214D-4,2.151D-3,3.711D-3,5.886D-3,9.209D-3,1.479D-2,
     * 2.525D-2,4.787D-2,0.1083,0.3408,2.381,9*0.,7.227D-4,1.672D-3,
     * 2.839D-3,4.393D-3,6.631D-3,1.012D-2,1.605D-2,2.724D-2,5.152D-2,
     * 0.1166,0.3673,2.572,8*0.,5.744D-4,1.326D-3,2.224D-3,3.375D-3,
     * 4.959D-3,7.289D-3,1.097D-2,1.726D-2,2.918D-2,5.513D-2,0.1248,
     * 0.3938,2.763,7*0.,4.686D-4,1.07D-3,1.776D-3,2.656D-3,3.821D-3,
     * 5.455D-3,7.891D-3,1.177D-2,1.843D-2,3.109D-2,5.872D-2,0.133,
     * 0.4202,2.954,6*0.,3.856D-4,8.764D-4,1.443D-3,2.131D-3,3.014D-3,
     * 4.207D-3,5.905D-3,8.456D-3,1.254D-2,1.958D-2,3.298D-2,6.228D-2,
     * 0.1412,0.4467,3.145,5*0./
      DATA ((OSH(I,J),I=1,20),J=17,20)/3.211D-4,
     * 7.270D-4,1.188D-3,1.739D-3,
     * 2.425D-3,3.324D-3,4.556D-3,6.323D-3,8.995D-3,.01328,.0207,.03486,
     * .06584,.1494,0.4731,3.336,4*0.,2.702D-4,6.099D-4,9.916D-4,
     * 1.439D-3,1.984D-3,2.679D-3,3.602D-3,4.877D-3,6.719D-3,9.515D-3,
     * 0.01402,.02182,.03672,.06938,.1575,.4995,3.527,3*0.,2.296D-4,
     * 5.167D-4,8.361D-4,1.204D-3,1.646D-3,2.196D-3,2.905D-3,3.856D-3,
     * 5.180D-3,7.099D-3,.01002,.01474,.02292,.03858,.07292,.1657,.5259,
     * 3.718,2*0.,1.967D-4,4.416D-4,7.118D-4,1.019D-3,1.382D-3,1.825D-3,
     * 2.383D-3,3.112D-3,4.094D-3,5.468D-3,7.468D-3,.01052,.01545,
     * .02402,.04043,.07644,0.1738,.5523,3.909,0./
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE START
C     ================
C
C     General input and initialization procedure
C
      mode=0
      read(1,*,err=10,end=10) mode
   10 continue
      if(mode.eq.0) then
         call initia
       else
         call start0
      end if
      return
      end
C
C
C     ****************************************************************
C
C

      SUBROUTINE START0
C     =================
C
C     "Old" input and initialization procedure
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (T15=1.D-15)
      PARAMETER (MPAG=MLEVEL/6+1) 
      CHARACTER*10 TYPLEV(MLEVEL)
      CHARACTER*4 TYPION(MION)
      CHARACTER*30 FIDATA(MION),FIODF1(MION),FIODF2(MION),FIBFCS(MION)
      CHARACTER*1 BLNK
      COMMON/OSCHYD/OSH(20,20)
      COMMON/PHOSET/S0(MLEVEL),ALF(MLEVEL),BET(MLEVEL),GAM(MLEVEL),
     *              IBF(MLEVEL)
      COMMON/VOIPAR/GAMR(MVOIGT),STARK1(MVOIGT),STARK2(MVOIGT),
     *              STARK3(MVOIGT),VDWH(MVOIGT)
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
      COMMON/PRINTP/NPGPOP,IIPR(6,MPAG),TYPLEV,TYPION
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      COMMON/TOPCS/CTOP(MFIT,MCROSS), !sigma=alog10(sigma/10^-18) of fit point
     +             XTOP(MFIT,MCROSS)  ! x = alog10(nu/nu0) of fit point 
      COMMON/DFEALI/DJMAX,NTRALI
      COMMON/EXTOTR/EXTOT
      COMMON/PPAPAR/IPOPST(MATOM),NTERST,ITERST,
     *              IACPPP,IACPP0,IACPPD,LACPPP
      COMMON/DMAXIP/DMAXIM
      COMMON/HEQAUX/PRD0,IHECOR
      COMMON/IONFIL/FIDATA,FIODF1,FIODF2,FIBFCS
      DATA BLNK/' '/
C
C ----------------------
C Basic input parameters
C ----------------------
C
C     TEFF, GRAV        - effective temperature, log g
C                       - if Teff < 0 - irradiation is considered
C                         (input of intensities later)
C     HMIX0      > 0    - value of mixing length in units of the scale height
C                = 0    - convection is suppressed, but adiabatic and
C                         radiative gradients are calculated and
C                         printed
C                < 0    - convection is suppressed and no gradients
C                         are calculated
C ---------------------- 
C
C     LTE        = true : an LTE model is going to be calculated
C                = false: a NLTE model is going to be calculated
C     LTGREY     = true : an LTE-grey model is calculated at the
C                         beginning as a starting approximation for
C                         complete linerarization
C                = false: initial model is read (procedure INPMOD)
C     LCHC       = true : the set of statistical equilibrium equations
C                         contains the charge conservation equation
C                = false: the set of statistical equilibrium equations
C                         contains the particle conservation equation
C                         and charge conservation is solved separately;
C                         this is usually recommendable option
C                  (for more details see procedures RATMAT and ELCOR)
C     ISPLIN     - mode of numerical representation of the radiative
C                  transfer equation:
C                = 0  - ordinary Feautrier (second order) scheme
C                = 1  - spline collocation
C                = 2  - Auer's Hermitian (fourth-order) scheme
C                = 3  - improved Feautrier scheme
C                       (Rybicki & Hummer 1991, A&A 245, 171.)
C                  If the ALI option with Rybicki-Hummer opeartor
C                  is considered (IFALI>0 - see below), ISPLIN must
C                  be set to 0 or 3 (otherwise the program stops)
C                = 5 or greater - Discontinuous Finite Element (DFE)
C                       scheme (Castor et al. ApJ 387, 561, 1992)
C     CHMAX      - overall convergence criterion (see next item); the
C                  iterations will stop if the maximum relative change
C                  (i.e. maximum for all quantities and all depths) is
C                  smaller that CHMAX
C
C ----------------------
C 
C     NITER      - maximum number of iterations of complete linearization
C                  (the iteration procedure can actually stop after less
C                  iterations if all relative changes of all model
C                  quantities in all depths are less than CHMAX)
C     ND         - number of depth points
C     NATOM      - number of "explicit atoms"
C                  Explicit atom is a chemical species some energy levels
C                  of which are considered explicitly, ie. their
C                  populations are determined by solving a corresponding
C                  statistical equilibrium equation.
C                  Each explicit atom is considered to be composed of
C                  one or more ionization states (chosen as those which
C                  are dominant at the temperature range under interest);
C                  the highest considered ionization state must be
C                  (obligatorily) represented as one-level ion.
C                < 0 - the switch IBFINT is set to 0 (default is IBFINT=1);
C                  IBFINT=1 means that photoionization cross-sections are 
C                  stored only for continuum frequencies, and are interpolated
C                  for line frequencies;
C                  IBFINT=0 means that cross-sections are stored for all
C                  frequencies
C     NION       - number of "explicit ions".
C                  Amongst explicit ions are counted all considered
C                  ionization states (including neutrals and/or
C                  negative ions, if appropriate) of all explicit atoms,
C                  except the highest (one-level) ionization states.
C     NLEVEL     - number of all "explicit levels", ie. all energy levels
C                  for which statistical equilibrium is solved.
C                  Highest ionization degrees of each explicit atom are
C                  now counted amongst explicit levels.
C                  The explicit levels are thus labeled by indices
C                  I = 1,NLEVEL; further input parameters (N0A,NKA in
C                  procedure STATE; NFIRST,NLAST,NNEXT here) specify
C                  to which atom and ion the given level I belongs.
C                < 0 - indicates that the rate matrix is not inverted as
C                      a whole; instead, one solves several separate sets
C                      of statistical equilibrium equations (with partial
C                      rate matrices) for each individual species.
C                      This is a useful option if there are many explicit
C                      levels.         
C                  Actual NLEVEL is set to ABS(NLEVEL).
C     IFALI      - switch to turn on the ALI option:
C                = 0 - no ALI; fixed transition treated as truly fixed;
C                > 0 - approximate lambda operator after Rybicki-Hummer
C                < 0 - approximate lambda operator after Olson-Kunasz;
C                      IFALI is then stored as ABS(IFALI)
C                >10 - switches on additional input 
C                      (preconditioning, acceleration, etc. see below),
C                      IFALI is then reset to IFALI-10
C                different values of ABS(IFALI) have the following meaning:
C                = 1 - no ALI; recalculation of the fixed rates in RESOLV
C                      is done even after the last lambda iteration for the
C                      explicit transitions
C                > 1 - ALI option; there are several possibilities:
C                      some of them useful only for testing; for calculations,
C                      the only practical options are 4 and 5
C                = 4,5 (and also 14,15) - diagonal operator -
C                       this is the RECOMMENDABLE OPTION !!
C                = 6 - (also 16) - tri-diagonal operator (not yet fully tested)
C     IFPOPR     - switch for POPulation Recalculation:
C                <= 0 - after a completed linearization, new populations
C                       (i.e. those obtained as n_new=n_old + delta n) are
C                       not used; instead, one uses new radiation field,
C                       calculates new radiative rates, and solves the
C                       statistical equilibrium for the new populations.
C                       Retained only for historical reasons (this was the
C                       procedure suggested by Auer and Mihalas); it is useful
C                       only in the full Complete Linearization mode.
C                 > 0 - populations directly coming from linearization are
C                       used 
C                 different values of ABS(IFPOPR) have the following meaning:
C                 < 4 - statistical equilibrium is linearized as
C                       n=A**(-1) * b which is the usual procedure
C                       (suggested by Auer and Mihalas);
C                 = 2 - in addition; populations are recalculated at the end
C                       of the formal solution.
C                       Useful only in the full Complete Linearization mode
C                 = 4 or 5 - statistical equilibrium is linearized as A*n=b 
C                       This is a HIGHLY RECOMMENDED option with version 185
C                       Better speaking, since the preconditioning is usually
C                       quite useful, the recommended option is 14 or 15
C                 > 5 - used for testing purposes; not supported any more
C
C     IATREF      - mode for treating the so-called reference atom and
C                   reference levels.
C                   Reference atom - the species to which all abundances
C                        are related (this does not have to be hydrogen!)
C                        The index of reference atom is stored in IATREF
C                   Reference level - the energy level for each species
C                        for which the statistical equilibrium equation is
C                        not written, instead, one considers the abundance
C                        definition equation (that was previously taken as
C                        the highest ionization degree)
C                        The indices of reference levels are stored in
C                        array NREF(IAT, IAT=1,NATOM)
C                  = 0 - IATREF is set to 1 (ie the first explicit species;
C                        again, this does not have to be hydrogen; one may
C                        calculate for instance pure-He models).
C                        NREF(IAT) is set to NKA(IAT), i.e. the highest
C                        ionization state
C                  > 0 - reference atom is the atom with index IATREF;
C                        NREF(IAT) is determined by the program to be the
C                        index of level with highest population for the
C                        species IAT
C                  < 0, but >-100 - then IATREF=IABS(IATREF), and
C                        NREF(IAT) is set to NKA(IAT), i.e. the highest
C                        ionization state
C                  < -100, but >-200 - then IATREF=-IATREF-100, and
C                        NREF(IAT) are read (see below)
C
C      NELSC       - mode of treating the electron scattering by the Feautrier
C                    scheme (it has no meaning for the DFE scheme, ISPLIN>4)
C                  = 0 - electron scattering source function is treated
C                        exactly; i.e. the single-frequency formal solution
C                        of the transfer equation contains an explicit
C                        angular coupling due to the J_nu dependence of the
C                        electron scattering source function;
C                  > 0 - electron scattering source function is treated as
C                        the thermal source function, i.e. it is given through
C                        the current mean intensity J_nu.
C                        This is NOT a recommended option, and is included
C                        for pedagogical and testing purposes.
C
C ----------------------
C  additional 3 lines of input in the case IFALI > 10. The default
C  values of the subsequent parameters which apply in the case these
C  three lines are not read are given below:
C
C     IFPREC       - flag for treating the preconditioning (Rybicki and
C                    Hummer 1992, A&A 262, 209) of the statistical
C                    equilibrium equations in the formal solution step.
C                    IFPREC=0 : no preconditioning
C                    IFPREC=1 : diagonal preconditioning (i.e. local)
C                    IFPREC>1 : tri-diagonal preconditioning (non-local)
C                   DEFAULT: IFPREC=1   (preconditioning)
C     IOVER        - flag for turning on the "line-overlapping" mode:
C                  = 0 - no overlapping allowed for (only one line may
C                        contribute to opacity at any single frequency)
C                  > 0 - general overlapping is allowed for
C                        This is a recommended option, but it may runs
C                        slower in case of heavy overlaps.
C                   DEFAULT: IOVER=1  (overlapping mode)
C     ITLAS        - flag for turning off laser lines, i.e. these for which 
C                    the absorption coefficient (= true absorption minus 
C                    stimulated emission) becomes negative.
C                    Turning off laser lines means that the line absorption and
C                    emission coefficient are set to zero at depths where the
C                    absorption coefficient would be negative.
C                    ITLAS has the meaning of the global iteration number 
C                    starting from which the laser lines are turned off
C                    (ITLAS=0 turns off laser lines from the very beginning)
C                   DEFAULT: ITLAS=100  (laser lines not turned off)
C     IELCOR       - introduced only for testing purposes.
C                    Flag for turning off calling subroutine ELCOR, i.e.
C                    an iterative update of electron density by solving a
C                    non-linear system of charge conservation + rate equations.
C                    IELCOR has the meaning of the global iteration number 
C                    starting from which the calling of ELCOR is turned off
C                    (IELCOR=0 turns off calling ELCOR from the very beginning)
C                   DEFAULT: IELCOR=100   (ELCOR always called)
C
C ----------------------
C     IACCP        - switch for Ng acceleration of the preconditioned formal
C                    solution:
C                   = 0 - no acceleration
C                   > 0 - acceleration is done first in the IACCP iteration
C                         of the formal solution, and is repeated every IACDP
C                         iterations
C                   Note: if IACCP > NLAMBD (total number of iterations of
C                         the formal solution - called, inaccurately,
C                         lambda iterations), then the acceleration is
C                         obviously not performed
C                   DEFAULT: IACCP=0   (no acceleration)
C     IACDP         - step for the Ng acceleration (see above)
C
C ----------------------
C     ILDER         - flag for controlling evaluation of derivatives of
C                     recombination rates with respect to temperature:
C                   = 0 - derivatives are calculated - recommended option
C                   > 0 - derivatives are set to zero
C                   DEFAULT: ILDER=0   
C     IBPOPE        - introduced for testing purposes only:
C                   = 0 - derivatives of the rows of rate equations with
C                         respect to the mean intensity in the CL (complete
C                         linearization) frequency points are not calculated
C                   > 0 - derivatives are calculated - RECOMMENDED OPTION
C                   DEFAULT: IBPOPE=1   
C     CHMAXT        - a parameter which enables to increase the number of
C                     iterations of the formal solution (the so-called lambda
C                     iterations) when the model is almost converged.
C                     If the maximum of absolute value sof relative changes of
C                     of temperature at all depths decreases below CHMAXT,
C                     the number of lambda iterations is set to NLAMT.
C                   DEFAULT: CHMAXT=0. 
C     NLAMT         - the reset number of lambda iterations - see above
C                   DEFAULT: NLAMT=1
C     
C
C ----------------------
C  additional line of input in the case -100>IATREF>-200 
C
C     NREF(IAT)    -  indices of reference levels 
C                     read only if -100>IATREF>-200 - see above
C
      CALL READBF
      READ(IBUFF,*) TEFF,GRAV,HMIX0
      READ(IBUFF,*) LTE,LTGREY,LCHC,ISPLIN,CHMAX
      nogray=0
      if(teff.lt.0) then
         teff=abs(teff)
         nogray=1
      end if
      INTRPL=0
      IF(GRAV.LT.0.) THEN
        INTRPL=-1
        GRAV=-GRAV
      ENDIF
      IBC=3
      IRDER=3
      IF(LTE) IRDER=0
      ILMCOR=1
      ILPSCT=0
      IF(TEFF.GT.6.D5) ILPSCT=1
      ILASCT=0
      IRTE=2
      IDLTE=MDEPTH+1
      DJMAX=1.e-3
      NTRALI=3
      IF(ISPLIN.LT.0) THEN
         IF(ISPLIN.LE.-5) THEN
            ISPLIN=-ISPLIN
          ELSE
            ISPLIN=0
         END IF
         READ(IBUFF,*,ERR=3) IBC,IRDER,ILMCOR,ILPSCT,ILASCT,
     *                       IRTE,IDLTE
         IF(IRTE.GE.2) READ(IBUFF,*) DJMAX,NTRALI
      END IF
    3 CONTINUE
      IFALI=5
      IFPOPR=4
      IATREF=1
      IFPREC=1
      IOVER=1
      ITLAS=100
      IELCOR=100
      IHECOR=0
      IACCP=0
      NELSC=0
      ILDER=0
      IBPOPE=1
      NLAMT=1
      CHMAXT=0.
      READ(IBUFF,*,ERR=4) NITER,ND,NATOM,NION,NLEVEL,
     *                    IFALI,IFPOPR,IATREF,NELSC
    4 CONTINUE
      IF(IFALI.GT.10) THEN
         IFALI=IFALI-10
         READ(IBUFF,*,err=5) IFPREC,IOVER,ITLAS,IELCOR,IHECOR
         READ(IBUFF,*,ERR=5) IACCP,IACDP
         READ(IBUFF,*,ERR=5) ILDER,IBPOPE,CHMAXT,NLAMT
      END IF
c
      IBFINT=1
      IF(NATOM.LT.0) THEN
	 IBFINT=0
	 NATOM=-NATOM
      END IF
c
    5 CONTINUE
      NTERST=0
      IACPPP=0
      IACPPD=0
      DO IAT=1,NATOM
         IPOPST(IAT)=0
      END DO
      IF(NION.LT.0) THEN
         NION=-NION
         READ(IBUFF,*,ERR=6) NTERST,IACPPP,IACPPD,
     *                       (IPOPST(IAT),IAT=1,NATOM),DMAXIM
      END IF
    6 CONTINUE
c
      irsplt=0
      if(nlevel.lt.0) irsplt=1
      nlevel=iabs(nlevel)
      if(irsplt.eq.1) lchc=.false.
      if(ifali.lt.0) then
         ifali=iabs(ifali)
         if(ifali.ge.2) jali=2
       else if(ifali.le.1) then
         jali=0
       else
         jali=1
         if(isplin.eq.1.or.isplin.eq.2) 
     *       call stop(' conflict in isplin - Rybicki-Hummer oper.')
      end if
C
      if(iatref.gt.100) then
         iatref=iatref-100
         modref=2
       else if(iatref.gt.0) then
         modref=1
       else if(iatref.eq.0) then
         iatref=1
         modref=0
       else if(iatref.gt.-100) then
         iatref=-iatref
         modref=0
       else if(iatref.gt.-200) then
         iatref=-iatref-100
         modref=-1
         READ(IBUFF,*) (nref(iat),iat=1,natom)
      end if
C
C     check the dimensions
C
      if(nd.gt.mdepth) call stop('nd too large')
      if(nlevel.gt.mlevel) call stop('nlevel too large')
      if(nlvexp.gt.mlvexp) call stop('nlvexp too large')
      if(natom.gt.matom) call stop('natom too large')
      if(nion.gt.mion) call stop('nion too large')
      if(isplin.lt.0) call stop('inconsistent isplin')
      if(iatref.gt.natom) call stop('conflict in iatref')
      if(ifprec.gt.1.and.mleve3.lt.nlevel) 
     *   call stop('mleve3 less than nlevel')
      if(isplin.ge.5.and.mfreq1.lt.nfreq) 
     *   call stop('mfreq1 less than nfreq')
      if(ifprec.gt.1.and.mtran3.lt.ntrans)
     *   call stop('mtran3 less than ntrans')
       if(ifali.ge.6.and.mlvex3.lt.nlvexp)
     *   call stop('mlvex3 less than nlvexp')
C
C     Initialize 1/i*i and 1/i*i*i
C
      DO 7 I=1,NLMX
        X=I
        XI2(I)=UN/(X*X)
        XI3(I)=XI2(I)/X
    7 CONTINUE
C
C ----------------------------
C Frequency points and weights
C ----------------------------
C
C     NJREAD     - number of "continuum" frequency points which are
C                  read. May be less than NFREQ - the actual number of
C                  frequencies, because the frequency points in lines
C                  can be calculated.
C ----------------------
C     FREQ(IJ)   - frequency (in sec**-1) of the IJ-th freq. point
C     W(IJ)      - corresponding frequency quadrature weight
C     WCH(IJ)    = 0 - if the point is an interior point of a Simpson
C                      integration interval;
C                > 0 - for a boundary point of any integration domain
C                      (i.e. all points except interior points of a
C                      Simpson integration). In this case the weight W(IJ)
C                      is given by a sum of two contributions - from the
C                      higher frequency side, and from the lower frequency
C                      side. WCH had then the meaning of the lower frequency
C                      side contribution.
C                  This quantity is used only in the non-overlapping mode
C                  (i.e. IOVER = 0), where the integration weight at the edge
C                  point of a continuum transition (program allows one or more
C                  transitions to have the same edge point),
C                  which appears in evaluating of radiative rates, is
C                  given by W(IJ)-WCH(IJ).
C                  In this mode, WCH is also used for setting the so-called
C                  subtraction weights (see CORRQW and CORRWM; detailed
C                  description is in Hubeny & Lanz 1995, ApJ 439, 875)
C     IJALI(IJ)  - basic switch of the hybrid CL/ALI method:
C                = 0 - the frequency point is explicitly linearized
C                > 0 - the frequency point is treated by ALI 
C                      (the so-called ALI point)
C                  Hint: typically, only a few points are to be linearized,
C                  usually those corresponding to most opaque frequencies
C                  (e.g. 2-4 points near the Lyman limit, etc.)
C
C ----------------------
C     NFFIX      - switch for globally changing the ALI switch IJALI 
C                = 0 - IJALI is read for continuum points (see above), 
C                      and is determined for each line transition (see below)
C                = 1 - IJALI is set to 1 (ALI) for all lines, regardless 
C                      of other input
C                = 2 - all points are ALI, regardless of other input
C                      (i.e. it overwrites IJALI(IJ) which were set up in the
C                      previous block of data, to IJALI(IJ)=1
C     IFSUB      - more or less an outdated parameter. It is only useful
C                  in the non-overlapping option, which is included
C                  for historical reasons anyway.
C                  In that case it has the meaning of a switch for
C                  globally changing ISUB - an instruction whether
C                  the subtraction of weights for a given transition
C                  is explicit or ALI (see also the input block for lines)
C                = 0 - ISUB is determined for all transitions
C                = 1 - ISUB for all lines reset to -1 (ALI subtraction)
C                = 2 - ISUB for all lines reaset to 0 (no explicit
C                      subtraction - WSUB formalism applies)
C                = 3 - ISUB for all lines reset to 1 (explicit subtraction)
C     IFLEV      - a switch for globally changing the mode of treating the
C                  linearization of atomic level populations
C                = 0 - the mode is specified by the input parameter IMODL
C                      (see the input block for levels), and is not changed
C                > 0 - the mode is reset to all levels except the highest
C                      ionization stage to IMODL=1, i.e. the updated LTE
C                      mode (see explanation in the input block of levels)
C           
C
      READ(IBUFF,*) NJREAD
      if(njread.gt.mfreq) call stop('njread too large')
C 
      IF(NJREAD.GT.0) THEN
        NFREQC=NJREAD
        DO 9 IJ=1,NJREAD
           WCH(IJ)=0.
           IJALI(IJ)=0
           IJX(IJ)=1
           READ(IBUFF,*,ERR=8) FREQ(IJ),W(IJ),WCH(IJ),IJALI(IJ)
    8      CONTINUE
    9   CONTINUE
      ELSE
        NFREQC=-NJREAD
        READ(IBUFF,*,ERR=10) FRCMAX,FRCMIN,FRLMAX,FRLMIN
        IF(FRLMIN.LE.0.) FRLMIN=FRCMIN
        IF(FRLMAX.LE.0.) FRLMAX=FRCMAX
      ENDIF
      IF(FRLMIN.LE.0.) FRLMIN=3.D12
      IF(FRLMAX.LE.0.) FRLMAX=FREQ(1)
C
   10 NFFIX=0
      IFSUB=0
      READ(IBUFF,*,ERR=11) NFFIX,IFSUB,IFLEV
      IF(IFLEV.NE.0) IRSPLT=0
      POPZER=1.D-20
      IF(IFLEV.LT.0) THEN
         IF(IFLEV.GT.-3) IFLEV=-IFLEV
         IF(IFLEV.LE.-3) IFLEV=0
         MODREF=1
         READ(IBUFF,*) POPZER
      END IF
   11 CONTINUE
      IF(NFFIX.EQ.2) THEN
         DO 12 IJ=1,NFREQC
            IJALI(IJ)=1
   12    CONTINUE
      END IF
C
      WRITE(6,601) TEFF,GRAV
      GRAV=EXP(2.3025851*GRAV)
C
C --------------------
C Turbulent velocities
C --------------------
C
C     ITB       =  0 - depth-independent turbulent velocity is assumed
C               >  0 - depth-dependent turbulent velocity
C     VTB       -  value of depth-independent turbulence
C                  (in km/sec or cm/sec, depending on the numerical value)
C ---------------------
C     VTURB(ID) -  values of depth-dependent turbulence
C                  (in km/sec or cm/sec, depending on the numerical value)
C                  (to be read ONLY if ITB > 0)
C
      READ(IBUFF,*) ITB,VTB
      IF(VTB.LT.1.E3) VTB=VTB*1.E5
      IF(ITB.EQ.0) THEN
         DO 15 ID=1,ND
   15       VTURB(ID)=VTB
       ELSE
         READ(IBUFF,*) (VTURB(ID),ID=1,ND)
         DO 17 ID=1,ND
            IF(VTURB(ID).LT.1.E3) VTURB(ID)=VTURB(ID)*1.E5
   17    CONTINUE
      END IF
C
C ----------------------------------------------------
C Input parameters for explicit and non-explicit atoms
C ----------------------------------------------------
C
C     Input parameters are read by procedure STATE
C     (see description there)
C
      CALL STATE(0,ID,X1,X2)
c      WRITE(6,602) YTOT,WMY,WMM
      DO 20 I=1,NLEVEL
         ILK(I)=0
         DO 20 J=1,NLEVEL
   20       ITRA(J,I)=0
C -------------------------------------
C Input of parameters for explicit ions
C -------------------------------------
C
C    NFIRST(I) -  index of the first explicit level of ion I;
C              < 0 - indicates that for the given ion one has a
C                    precalculated set of ODF's (opacity distribution
C                    functions). In this case, there is an additional
C                    record indicating two unit numbers where the ODF
C                    input is going to be read from (called INODF1(I)
C                    and INODF2(I);
C                    NFIRST is then set to ABS(NFIRST)
C    NLAST(I)  -  index of the last explicit level of ion I
C    NNEXT(I)  -  index of the ground state of the next higher
C                 ionization state;
C              < 0 - indicates that the given ion will be treated in LTE
C                    (even if the current model is NLTE).
C                    NNEXT is then set to ABS(NNEXT)
C    IZ(I)     -  effective charge ( = 1 for neutrals, 2 for once
C                 ionized, etc.)
C    IUPSUM(I) -  mode of evaluation of total population of higher,
C                 non-explicit, LTE energy levels of ion I - the
C                 so-called upper sum:
C              =  0 - calculated by means of the partition unction
C              >  0 - calculated as a sum of populations of hydrogenic
C                     levels starting with the quantum number next to
C                     the highest explicit level and ending with IUPSUM
C              <  0 - occupation probability form
C    ICUP(I)   -  mode of considering "modified collisional ionization
C                 rate", ie. allowing for collisional excitation into,
C                 and collisional deexcitation from, higher,
C                 non-explicit, LTE energy levels of the ion I
C              =  0 - this contribution is neglected
C              >  0 - calculated as a sum of contributions of
C                     rates into and from averaged (hydrogenic)
C                     levels starting with the quantum number next to
C                     the highest explicit level and ending with ICUP
C    MODEFF    -  mode of evaluating the free-free cross-section:
C              =  0 - free-free opacity is neglected
C              =  1 - hydrogenic cross-section with Gaunt factor
C                     set to unity
C              =  2 - hydrogenic cross-section with exact Gaunt
C                     factor
C              <  0 - non-standard expression, given by the user
C                     supplied procedure FFCROS
C    NFF       -  mode of considering "modified free-free" opacity,
C                 ie. allowing for the photoionization from higher,
C                 non-explicit, LTE energy levels of the ion I;
C              =  0 - this contribution is neglected
C              >  0 - main quantum number of the first non-explicit
C                     level
C    TYPION(I) -  character*4 string containing the name of the ion
C                 which appears in outputs (unit 6)
C
C ------------------------------
C     INODF1(I),INODF2(I) - unit numbers for the ODF input;
C                 read only in NFIRST(I) < 0  -  see above
C
      WRITE(6,603)
      DO 50 I=1,NION
         READ(IBUFF,*) NFIRST(I),NLAST(I),NNEXT(I),IZ(I),IUPSUM(I),
     *             ICUP(I),MODEFF,NFF,TYPION(I)
         IF(NFIRST(I).LT.0) THEN
            READ(IBUFF,*) INODF1(I),INODF2(I)
            FIODF1(I)=BLNK
            FIODF2(I)=BLNK
            FIBFCS(I)=BLNK
          ELSE
            INODF1(I)=0
            INODF2(I)=0
         END IF
         ILTION(I)=0
         IF(NNEXT(I).LT.0) THEN
            ILTION(I)=2
            NNEXT(I)=IABS(NNEXT(I))
         END IF
         IF(NLAST(I).LT.0) THEN
            ILTION(I)=1
            NLAST(I)=IABS(NLAST(I))
         END IF
         FF(I)=0.
         CHARG2(I)=IZ(I)*IZ(I)
         IF(NFF.GT.0) FF(I)=EH/H*IZ(I)*IZ(I)/NFF/NFF
         NFIRST(I)=IABS(NFIRST(I))
         N0I=NFIRST(I)
         N1I=NLAST(I)
         NKI=NNEXT(I)
         ITRA(NKI,NKI)=MODEFF
         DO 30 IAT=1,NATOM
            IF(N0I.GE.N0A(IAT).AND.NKI.LE.NKA(IAT)) IA=IAT
   30    CONTINUE
         DO 40 II=N0I,N1I
            IEL(II)=I
   40       IATM(II)=IA
         ILK(NKI)=I
         IATM(NKI)=IA
         IF(NKI.EQ.NKA(IA)) IEL(NKI)=I
         WRITE(6,604) I,TYPION(I),N0I,N1I,NKI,IZ(I),IUPSUM(I),ICUP(I),
     *                FF(I)
C
         IF(NUMAT(IA).EQ.1) THEN
            IATH=IA
            IF(IZ(I).EQ.1) IELH=I
            IF(IZ(I).EQ.0) IELHM=I
         END IF
         IF(NUMAT(IA).EQ.2) THEN
            IATHE=IA
            IF(IZ(I).EQ.1) IELHE1=I
            IF(IZ(I).EQ.2) IELHE2=I
         END IF
   50 CONTINUE
C
C -----------------------------------------
C Parameters for individual explicit levels
C -----------------------------------------
C
C     ENION(I)   - ionization energy of level I (with respect to the
C                  ground level of the next ionization state
C                  may be given either in ergs, eV, cm**-1,
C                  or as frequency (sec**-1)
C                  also, if ENION = 0., the program assigns the
C                  hydrogenic ionization energy, assuming that the
C                  principal quantum number is the order number of level
C                  within the corresponding ion;
C     G(I)       - statistical weight; the input G(I)=0. has the
C                  analogous meaning;
C     NQUANT(I)  - principal quantum number; again for NQUANT(I)=0 the
C                  program assigns the order number;
C                < 0 - indicates that the given level is kept in LTE
C                      (even if the model is NLTE);
C                      NQUANT is then set to ABS(NQUANT)
C     TYPLEV(I)  - character*10 string - spectroscopic identification
C                  (appears only in outputs), but may also be used as an
C                  identifier in case of using the pretabulated 
C                  Opacity Project photoionization cross-section data
C     IFWOP(I)   > 0 - occupation probability for level I
C                  is calculated, and is used consistently in statistical
C                  equilibrium, and for determining opacities/emissivities;
C                = 0 - occupation probability put to 1 (usual option);
C                < 0 - signals that the level is a so-called merged level
C                      (i.e. the Rydberg states lumped together)
C     FRODF(I)   - dummy (a former option, now obsolete)
C     IMODL(I)   - a mode of treating the population of level I:
C                = 0 - explicit level -- population is linearized;
C                = 1 - updated b-factor -- the b-factor is fixed in the
C                      linearization, and is updated in the formal solution;
C                =-1 - fixed b-factor -- the b-factor is fixed and is not 
C                      updated;
C                = 2 - updated population -- the population is fixed in the 
C                      linearization, and is updated in the formal solution;
C                =-2 - fixed population -- the population is fixed and is 
C                      not updated
C
      LBPFX=.TRUE.
      IMER=0
      WRITE(6,605)
      DO 60 I=1,NLEVEL
         IE=IEL(I)
         N0I=NFIRST(IE)
         NKI=NNEXT(IE)
         IQ=I-N0I+1
         X=IQ*IQ
         READ(IBUFF,*) 
     *   ENION(I),G(I),NQUANT(I),TYPLEV(I),ifwop(i),FRODF(I),IMODL(I)
         if(ifwop(i).lt.0.and.i.ne.nlast(ie)) 
     *      call stop('conflict in negative ifwop')
         IF(I.LT.NKI) THEN
            E=ABS(ENION(I))
            E0=E
            IF(E.EQ.0.) E0=EH*IZ(IE)*IZ(IE)/X
            IF(E.GT.1.D-7.AND.E.LT.100.) E0=1.6018D-12*E
            IF(E.GT.100..AND.E.LT.1.D7) E0=1.9857D-16*E
            IF(E.GT.1.D7) E0=H*E
            IF(ENION(I).GE.0.) THEN
              ENION(I)=E0
            ELSE
              ENION(I)=-E0
            ENDIF
            IF(G(I).EQ.0.) G(I)=2.D0*X
            IF(NQUANT(I).EQ.0) NQUANT(I)=IQ
            iltlev(i)=0
            if(nquant(i).lt.0) then
               iltlev(i)=1
               nquant(i)=iabs(nquant(i))
            end if
          ELSE
            if(modref.ge.0) nref(iatm(i))=nka(iatm(i))
            IF(G(I).EQ.0..AND.NKI.EQ.NKA(IATM(I))) G(I)=1.
         END IF
         if(ifwop(i).lt.0) then
            enion(i)=0.
            IMER=IMER+1
            IMRG(I)=IMER
            IIMER(IMER)=I
         endif
         WRITE(6,606) I,TYPLEV(I),TYPION(IE),ENION(I),G(I),NQUANT(I),
     *                IEL(I),ILK(I),IATM(I)
         LBPFX = LBPFX .AND. IMODL(I).EQ.0 .AND. IIFIX(IATM(I)).EQ.0
   60 CONTINUE
      CALL LEVSET
C
C     setup continuum frequencies (if they are not read)
C
      IF(NJREAD.LT.0) CALL INIFRC(0)
      IF(FRLMAX.LE.0.) FRLMAX=FREQ(1)
C
      lasv=.false.
      CALL HODFIN
C
C -------------------------------------
C Parameters for individual transitions
C -------------------------------------
C
C     For each transition considered in any option:
C
C     II      -  index of the lower level
C     JJ      -  index of the upper level
C
C             These indices may be submitted in two different ways:
C             1) as the "absolute" index, i.e. a level has an index
C                corresponding to the global numbering of all levels
C                considered (see also previous input block);
C             2) as a "relative" index (for instance to be able to
C                put for Lyman-alpha II=1, JJ=2, regardless of the
C                absolute indices of levels, which may obviously be
C                different from 1 and 2, respectively).
C                This is achieved by putting a record with 
C                II=0 and JJ=JJ0 > 0 (following parameters have no meaning);
C                which signals that in all the subsequent input (until an
C                analogous records is met); the input values of II and JJ
C                have the meaning of relative indices, JJ0 being the
C                reference index (offset); ie. the absolute indices are
C                II+JJ0-1 and JJ+J0-1.
C             A record with BOTH II and JJ equal to 0 signals the end of
C             input data for atomic transitions.
C
C     MODE    - mode of treating treating the radiative rates in the
C               transition.
C             =  0  -  detailed radiative balance (ie. radiative rates
C                      are not evaluated; but collisional rates are);
C             >  0  -  primarily linearized transition;
C             <  0  -  primarily ALI transition;
C             Note: the distinction "primarily linearized" or "primarily ALI"
C             does not have any meaning for continua; for lines it means
C             that the frequency points are standardly set to the 
C             corresponding type (lineariz. or ALI), but it may be changed
C             to the opposite by coding parameters IFR0 and IFR1 (see below)
C             The sign of MODE is a somewhat outdated feature - basically a 
C             leftover from the old times when one worked in terms of
C             "explicit" and "fixed option" transitions (see Hubeny 1988).
C             Since this option was replaced by the hybrid CL/ALI
C             scheme, many previous meanings of this flag disappeared.
C
C             special meaning for continua:
C             IABS(MODE) = 5 or 15  -  signals that the given continuum is
C                      supplemented by a pseudo-continuum (i.e. a dissolved
C                      part of a correspponding spectral series converging
C                      to the given edge;
C                      In this case, there is one additional record
C                      immediately following the present one, which
C                      specifies the minimum frequency to which the
C                      pseudo-continuum is considered
C             special meaning for lines:
C             IABS(MODE) = 1 - normal line
C             IABS(MODE) = 2 - ODF corresponding to the transition from
C                              a normal to the merged level (i.e. all high
C                              members of a spectral series);
C                              an ODF is calculated by TLUSTY
C                              Typically used only for H (or He II)
C             IABS(MODE) = 3 - normal ODF, representing a superline
C                              an ODF is calculated by a separate program,
C                              and is communicated to TLUSTY by input files
C             IABS(MODE) = 9 or 19  -  a contribution of the transition to
C                      the radiation pressure is set to zero
C                      (this option represents a means of avoiding
C                      instabilities - numerical as well as physical - at the
C                      top of an atmosphere due to some strong lines);
C        Finally, if MODE>100 (or MODE<-100), the next immediate input
C        record contains frequency or wavelength of the transition; MODE
C        is immediately changed to MODE-100 (or MODE+100 for negative MODE).
C        This option is useful if one needs a different frequency of the
C        transition than that correspondiong to a difference of level energies
C        (for instance for avoiding an overlap of a normal and an ODF line)
C
C     IFANCY  -  different meaning for lines and continua:
C       lines -  for = 0  -  Doppler profile
C                for = 1 or -1 -  Voigt profile
C                for = 2 or -2  - Stark (+ Doppler) profile
C                                 for hydrogenic lines;
C                for ge 10 - non-standard expression, given by a user
C                            supplied subroutine (PROFSP)
C             >  0  - the absorption profile for the last frequency point
C                     is taken to be 0
C             <  0  - the absorption profile for the last frequency point
C                     is evaluated exactly
C       cont. -  for = 0  -  hydrogenic photoionization cross-section,
C                            with Gaunt factor set to 1;
C                    = 1  -  hydrogenic cross-section with the exact
C                            Gaunt factor;
C                    = 2  -  cross-section by a Peach-type
C                            expression (see SIGK);
C                    = 3  -  modified Peach-type expression (called
C                            Henry-type  -  see SIGK);
C                    = 4  -  Butler's fit formula (polynomial fits to
C                            the Opacity Project results);
C                    = 5  -  cross-section from Verner tables;
C                    = 7  -  hydrogenic cross-section with the Gaunt
C                            factor from Klaus Werner;
C                    = 9  -  Opacity Project data stored in a special file.
C                            They are pretabulated and stored in file
C                            RBF.DAT
C                   > 49 and < 100 - special cross-section for
C                            a superlevel, precalculated.
c                            IFANCY then indicates the input unit number
C                            from which the data are read
C                   >100  -  Opacity Project data, immediately following
C                            the given record. There are IFANCY-100 data
C                            points.
C                for < 0  -  non-standard expression, given by a user
C                            supplied subroutine (SPSIGK)
C
C          for H- : for any IFANCY > 0  - standard expression
C          for He I:
C             IFANCY = 11 or = 13  -  Opacity Project cross section
C                                     Seaton-Ferney's cubic fits
C             IFANCY = 11  means that the multiplicity S=1 (singlet)
C             IFANCY = 13  means that the multiplicity S=3 (triplet)
C             IFANCY = 10  cross section, based on Opacity Project,
C                          but appropriately averaged for an averaged level
C                          (see explanation in SBFHE1)
C             IFANCY = 21  Cross-section for ground state from Koester
C                          fit (A&A 149, 423)
C
C     ICOL -  mode of evaluating collisional rates
C          for ge.0 -  means that collisional rate is evaluated by
C                      some standard expression, already coded in the
C                      program - for details see procedures COLIS,
C                      COLH, and COLHE;
C          for < 0  -  non-standard expression, given by a user supplied
C                      procedure (CSPEC)
C
C          Meaning of ICOL for all species, excluding hydrogen and helium:
C
C          a) for ionization
C          ICOL = 0 - Seatons formula ; here the value of the photo-
C                     ionization cross section at the threshold is
C                     transmitted in array OSC0
C               = 1 - Allen's formula; again, OSC0 has the meaning of
C                     the necessary multiplicative parameter
C               = 2 - the so-called SIMPLE1 mode - see subroutine COLIS
C               = 3 - the so-called SIMPLE2 mode - see subroutine COLIS
C          b) for excitation
C          ICOL = 0 - Van Regemorter formula, with standard g(bar)=0.25
C               = 1 - Van Regemorter formula, with "exact" g(bar);
C                     g(bar) is transmitted in CPAR
C               = 2 - the so-called SIMPLE1 mode - see subroutine COLIS
C               = 3 - the so-called SIMPLE2 mode - see subroutine COLIS
C          ICOL = 4  - Eissner-Seaton formula - see subroutine COLIS
C          
C          Meaning of ICOL for He I and He II:
C
C          ICOL =  0  - approximate expressions taken from Mihalas, Heasley
C                       and Auer, NCAR-TN/STR-104 (1975) 
C
C          For He I bound-bound transitions, the following standard
C          possibilities are also available:
C
C          ICOL =  1, 2, or 3  - much more accurate Storey's rates,
C                       subroutine written by D.G.Hummer (COLLHE).
C                       This procedure can be used only for transitions
C                       between states with n = 1, 2, 3, 4.
C          ICOL =  1  - means that a given transition is a transition
C                       between non-averaged (l,s) states. In this case,
C                       labeling of the He I energy levels must agree
C                       with that given in subroutine COLLHE, ie. states
C                       have to be labeled sequentially in order of
C                       increasing frequency.
C          ICOL =  2  - means that a given transition is a transition 
C                       between a non-averaged (l,s) lower state and an
C                       averaged upper state.
C          ICOL =  3  - means that a given transition is a transition 
C                       between two averaged states.
C
C          Meaning of ICOL for hydrogen:
C          a) for continua - ICOL ge. 0 :
C             Mihalas, Heasley, Auer NCAR-TN/STR-104 (1975) data for n>=10;
C             improved fit formulae from Klaus Werner for n<10.
C          b) for all b-b transitions except Lyman-alpha:
C             ICOL ge. 0 - approximate expressions taken from Mihalas, 
C                          Heasley, and Auer, NCAR-TN/STR-104 (1975) 
C          c) for 1 - 2 transition:
C                         = 0 - standard theoretical formula
C                         = 1 - experimental fit (formula quoted in
C                               Mihalas et al.)
C                         = 2 - formula by Crandall et al (procedure
C                                   CEH12)
C
C     IFRQ0,IFRQ1   -  CAUTION: the meaning was changed with respect to
C                      the previous version.
C                   -  It has no meaning for continuum transition.
C                   -  For line transitions, their non-zero values 
C                      signal a change for the mode of treatment frequency
C                      points (i.e. ALI or linearized) between indices
C                      IFRQ0 and IFRQ1 (internal indices for a line, 
C                      starting with 1). For instance, if the given
C                      transition is primarily an ALI one (MODE<0), then
C                      points between IFRQ0 and IFRQ1 will be taken
C                      as linearized.
C
C     OSC           -  has different meaning for lines and continua:
C                   -  for line transitions: oscillator strength,
C                      if OSC=0, the program assigns a scaled
C                      hydrogenic oscillator strength;
C                   -  for continuum transitions: first collision
C                      parameter (see COLIS for details)
C     CPARAM        -  second collision parameter (see COLIS for details)
C
C     These input parameters are stored at the following arrays
C     (ITR being the index of the given transition)
C
C     II     in  ILOW(ITR)
C     JJ     in  IUP(ITR)
C     MODE   in  INDEXP(ITR)
C     IFANCY in  IPROF(ITR) for line transitions,
C            in  IBF(IC) for bound-free transitions, IC is the index of
C                        a given b-f transition in the special numbering
C                        of b-f transitions (set up by the code)
C     ICOLIS in  ICOL(ITR)
C     IFRQ0  in  IFR0(ITR)
C     IFRQ1  in  IFR1(ITR)
C     OSC    in  OSC0(ITR)
C     CPARAM in  CPAR(ITR)
C
      ITR=0
      IC=0
      IL=0
      IP=0
      NLASTE=NFREQC
      NFREQ=NFREQC
      II0=0
      ILLIM=0
  100 CONTINUE
      READ(IBUFF,*) II,JJ,MODE,IFANCY,ICOLIS,IFRQ0,IFRQ1,OSC,CPARAM
      IF(II.EQ.0) THEN
         IF(JJ.EQ.0) GO TO 120
         II0=JJ-1 
         ILLIM=MODE+II0
         GO TO 100
      END IF
      IFFR0=0
      IF(IABS(MODE).GT.100) THEN
         IF(MODE.GT.0) MODE=MODE-100
         IF(MODE.LT.0) MODE=MODE+100
         IFFR0=1
         READ(IBUFF,*) FR0INP
         IF(FR0INP.LT.1.E10) FR0INP=2.997925D18/FR0INP
      END IF
      ITR=ITR+1
      IF(IABS(MODE).EQ.5 .OR. IABS(MODE).EQ.15) THEN
         READ(IBUFF,*) FR0PC(ITR)
      END IF
      II=II+II0
      JJ=JJ+II0
      ILOW(ITR)=II
      IUP(ITR)=JJ
      INDEXP(ITR)=MODE
      OSC0(ITR)=OSC
      ICOL(ITR)=ICOLIS
      CPAR(ITR)=CPARAM
      LALI(ITR)=.FALSE.
      LEXP(ITR)=.FALSE.
      ISUB(ITR)=0
      IF(ITRA(II,JJ).EQ.0) THEN
         ITRA(II,JJ)=ITR
       ELSE
         ICOL(ITR)=99
      END IF
      IE=IEL(II)
      N0I=NFIRST(IE)
      NKI=NNEXT(IE)
      IF(JJ.LT.NKI) THEN
         LINE(ITR)=.TRUE.
         IFR0(ITR)=IFRQ0
         IFR1(ITR)=IFRQ1
         FR0(ITR)=(ENION(II)-ENION(JJ))/H
         IF(IFFR0.EQ.1) FR0(ITR)=FR0INP
         IF(OSC.EQ.0.) THEN
           IF (MODE.NE.3.AND.MODE.NE.4) THEN
            GH=2.D0*NQUANT(II)*NQUANT(II)
            OSC0(ITR)=OSH(NQUANT(II),NQUANT(JJ))*G(II)/GH
           ENDIF
           IF (ifwop(jj).lt.0) THEN
            OSC0(ITR)=0.
            JJ0=NQUANT(JJ-1)
            J20=MIN(20,NLMX)
            IF(J20.GE.JJ0) THEN
            DO 102 JTR=JJ0,J20
              OSC0(ITR)=OSC0(ITR)+OSH(NQUANT(II),JTR)
  102       CONTINUE
            END IF
            IF(NLMX.GT.20) THEN
            XII=NQUANT(II)*NQUANT(II)
            SUF=0.
            DO 103 JTR=21,NLMX
               XJ=JTR
               XJJ=XJ*XJ
               XJTR=XJ/(XJJ-XII)
               SUF=SUF+XJTR*XJTR*XJTR
  103       CONTINUE
            XITR=(400.-XII)/20.
            OSC0(ITR)=OSC0(ITR)+OSH(NQUANT(II),20)*SUF*XITR*XITR*XITR
            END IF
           ENDIF
         END IF
       ELSE
         LINE(ITR)=.FALSE.
         IFC0(ITR)=IFRQ0
         IFC1(ITR)=IFRQ1
         FR0(ITR)=(ENION(II)-ENION(JJ)+ENION(NKI))/H
         IF(IFFR0.EQ.1) FR0(ITR)=FR0INP
c        IF(MODE.EQ.0) GO TO 100
C
C   -----------------------------------------------------
C   Additional input parameters for continuum transitions
C   -----------------------------------------------------
C
C     Only for IFANCY = 2, 3, or 4
C     S0, ALF, BET, GAM  - parameters for evaluation the
C         photoionization cross-section
C
         IC=IC+1
         NTRANC=IC
         IF(IFANCY.GE.2.AND.IFANCY.LE.4) THEN
            READ(IBUFF,*) S0(IC),ALF(IC),BET(IC),GAM(IC)
         END IF
C
C   -----------------------------------------------------
C   Additional input parameters for continuum transitions -TOPBASE DATA
C   -----------------------------------------------------
C
C     Only for IFANCY > 100 there are IFANCY-100 fit points
C
C     XTOP(MFIT,MCROSS) -  x = alog10(nu/nu0) of a fit point
C     CTOP(MFIT,MCROSS) -  sigma = alog10(sigma/10^-18) of a fit point
C
C     there are IFANCY-100 fit points
C
         IF(IFANCY.GT.100) THEN
            NFIT=IFANCY-100
            IF(NFIT.GT.MFIT) call stop(' nfit too large (TOPBASE fits)')
            READ(IBUFF,*) (XTOP(IFIT,IC),IFIT=1,NFIT)
            READ(IBUFF,*) (CTOP(IFIT,IC),IFIT=1,NFIT)
         END IF
         IBF(IC)=IFANCY
         if(ifancy.gt.49.and.ifancy.lt.100) lasv=.true.
         ITRA(JJ,II)=IC
         ITRBF(IC)=ITR
         IF(MODE.GT.0) THEN
C
C     determination of the first (IFR0) and the last (IFR1) frequency
C     for each explicit continuum - only if they were not already read
C
            IF(IFR0(ITR).LE.0) IFR0(ITR)=1
            IF(IFR1(ITR).GT.0) GO TO 100
            IF1=0
            FR01=FR0(ITR)
            IF(IABS(MODE).EQ.5 .OR. IABS(MODE).EQ.15) FR01=FR0PC(ITR)
            DO 105 IJ=1,NFREQC
               IF(FREQ(IJ).GE.FR01) IF1=IJ
  105       CONTINUE
            IFR1(ITR)=IF1
         END IF
      END IF
      IF(MODE.EQ.0) GO TO 100
      IF(IABS(MODE).EQ.2) GO TO 111
      IF(IABS(MODE).EQ.3.OR.IABS(MODE).EQ.4) THEN
C
C        change the status of treatment of superlines, 
C        if specified in the "offset-setting" record, or
C        because of too loo or too high frequency
C
         IF(II.LT.ILLIM.OR.FR0(ITR).LE.FRLMIN.OR.
     *      FR0(ITR).GE.FRLMAX) INDEXP(ITR)=0
         GO TO 100
      END IF
      IF(LINE(ITR)) GO TO 110
      GO TO 100
C
C  -----------------------------------------------------------
C  Additional input parameters for "clasical" line transitions
C   (i.e. those not represented by ODF's - ie ABS(MODE)=1)
C  -----------------------------------------------------------
C
C     LCOMP   -  mode of considering absorption profile:
C             =  false - depth-independent profile
C             =  true  - depth-dependent profile
C     INTMOD  -  mode of evaluating frequency points in the line
C             =  0  - means that frequency points and weights have
C                     already been read amongst the NJREAD or NFREAD
C                     frequencies;
C             ne 0  - frequency points and weights will be evaluated:
C             the meaning of the individual values:
C             =  1  - equidistant frequencies, trapezoidal integration
C             =  2  - equidistant frequencies, Simpson integration
C             =  3  - modified Simpson integration
C                     3-point Simpson integrations with each subsequent
C                     integration interval doubled, until the whole
C                     integration area is covered
C             =  4  - frequencies (in units of standard x) and weights
C                     (for integration over x) are read;
C
C      NF     -  number of frequency points in the line
C                (has the meaning only for INTMOD ne 0)
C
C      XMAX   =  0  - program sets up default XMAX=4.55
C             >  0  - means that the line is assumed symmetric around the
C                     center, frequency points are set up between x=0 and
C                     x=XMAX, where x is frequency difference from the
C                     line center in units of the standard Doppler width
C                     (standard Doppler width is the Doppler width
C                     calculated with standard temperature TSTD and
C                     standard turbulent velocity VTB);
C             <  0  - frequency points are set between x=XMAX and x=-XMAX
C
C  IMPORTANT NOTE: in the overlapping mode (IOVER > 0), all lines are
C  set by default to the full-profile mode. Thefore, if XMAX was
C  positive, it is reset to XMAX --> -XMAX, and NF --> 2*NF - 1
C
C     TSTD    >  0  - standard temperature (see above);
C             =  0  - the program assigns for the standard temperature the
C                     default value TSTD = 0.75*TEFF
C
  110 CONTINUE
      READ(IBUFF,*) LCOMP(ITR),INTMOD(ITR),NF,XMAX,TSTD
      IPROF(ITR)=IFANCY
      IF(IABS(IPROF(ITR)).EQ.1) THEN
	 IP=IP+1
         if(ip.gt.mvoigt) call stop('too many voigt profiles')
         ITRA(JJ,II)=IP
      END IF
C
C     if Voigt profile is assumed (ie. if IPROF = 1), an additional
C     input record is required which specifies an evaluation of the
C     relevant damping parameter - see procedure DOPGAM)
C
C      GAMR             - for > 0  - has the meaning of natural damping
C                                    parameter (=Einstein coefficient for
C                                    spontaneous emission)
C                             = 0  - classical natural damping assumed
C                             < 0  - damping is given by a non-standard,
C                                    user supplied procedure GAMSP
C      STARK1           -     = 0  - Stark broadening neglected
C                             < 0  - scaled classical expression
C                                    (ie gam = -STARK1 * classical Stark)
C                             > 0  - Stark broadening given by
C                                    n(el)*[STARK1*T**STARK2 + STARK3]
C      STARK2, STARK3   -   see above
C      VDWH             -   .le.0  - Van der Waals broadening neglected
C                             > 0  - scaled classical expression
C
      IF(IABS(IPROF(ITR)).EQ.1) THEN
         READ(IBUFF,*) GAMR(IP),STARK1(IP),STARK2(IP),STARK3(IP),
     *                 VDWH(IP) 
      END IF
C
C     change the status of treatment of lines, in virtue of:
C     a) too low frequency (FRLMIN); or
C     b) if specified in the "offset-setting" record
C
      IF(II.LT.ILLIM.OR.FR0(ITR).LE.FRLMIN.OR.FR0(ITR).GE.FRLMAX) THEN
         INDEXP(ITR)=0
         GO TO 100
      END IF
C      
      IF(TSTD.EQ.0.) TSTD=0.75*TEFF
      IF(XMAX.EQ.0.) XMAX=4.55
      if(iover.gt.0.and.xmax.gt.0.) then
         xmax=-xmax
         nf=2*nf-1
      end if
      CALL DOPGAM(ITR,1,TSTD,DOP,AGAM)
      INTM=INTMOD(ITR)
      IF(INTM.NE.0) IFR0(ITR)=NLASTE+1
      IF(INTM.NE.0) IFR1(ITR)=NLASTE+NF
      IF(INTM.NE.0) NLASTE=IFR1(ITR)
      IUNIT=IBUFF
      CALL LINSET(ITR,IUNIT,IFRQ0,IFRQ1,XMAX,DOP,AGAM)
      GO TO 100
C
C  -----------------------------------------------------------
C  Additional input parameters for a "merged superline" transition
C   (i.e. transition to a merged level, treated by means of an ODF
C     - i.e. for ABS(MODE)=2)
C  -----------------------------------------------------------
C
C     KDO(1),KDO(2),KDO(3),XDO(1),XDO(2),XDO(3),KDO(4) - 
C            have the following meaning:
C            The superline is represented by four frequency intervals.
C            Going away from the peak of ODF, the first interval is 
c            represented by a KDO(1)-point Simpson integration, with a 
C            distance XDO(1) fiducial Doppler widths between the points.
C            The same for the second and third interval.
C            The rest (the interval between the last point and the 
C            coresponding edge) is represented by a KDO(4)-point
C            Simpson integration.
C            The fiducial Doppler width is that corresponding to the
C            effective temperature and the standard turbulent velocity.
C
  111 NHOD=NHOD+1
      if(ifwop(jj).ge.0) call stop('Incorrect ifwop value')
      READ(IBUFF,*) (KDO(IHI,NHOD),IHI=1,3),(XDO(IHI,NHOD),IHI=1,3),
     *           KDO(4,NHOD) 
C
C     again, change status of treatment of lines if required
C     
      IF(II.LT.ILLIM.OR.FR0(ITR).LE.FRLMIN.OR.FR0(ITR).GE.FRLMAX) THEN
         INDEXP(ITR)=0
         GO TO 100
      END IF
      JNDODF(ITR)=NHOD
      GO TO 100
  120 CONTINUE
      NTRANS=ITR
      NFREQ=NLASTE
      NCON=IC
C
C     check the dimensions
C
      if(nd.gt.mdepth) call stop('nd too large')
      if(nfreqe.gt.mfrex) call stop('nfreq too large')
      if(nfreq.gt.mfreq) call stop('nfreq too large')
      if(nlevel.gt.mlevel) call stop('nlevel too large')
      if(ntrans.gt.mtrans) call stop('ntrans too large')
      if(natom.gt.matom) call stop('natom too large')
      if(nion.gt.mion) call stop('nion too large')
      if(nfreqc.gt.mfreqc) call stop ('nfreqc too large')
      if(ibfint.eq.0.and.nfreq.gt.mfreqc)
     *   call stop('mfreqc too small for no-interpol b-f c-s')
c
C  -----------------------------------------------------------
C     read input model
C
      IF(.NOT.LTGREY) CALL INPMOD
      CALL LINSET(0,0,0,0,0.,0.,0.)
C
C  -----------------------------------------------------------
C   There is no additional input for ODF transitions (i.e. ABS(MODE)=3)
C   from standard input; all information is taken from corresponding 
C   input files (subroutine ODFSET)
C  -----------------------------------------------------------
C
C     Input parameters for ODF lines
C
      DOPSTD=SQRT(TWO*BOLK*TEFF/HMASS+VTB*VTB)
      IF(NHOD.GT.0) CALL ODFHYS(DOPSTD)
      CALL ODFSET
C
C     maximum frequency in lines
C
      DO 125 IT=1,NTRANS
         IF(.NOT.LINE(IT)) GOTO 125
         FFMX=0.
         DO 126 IJ=IFR0(IT),IFR1(IT)
            FFMX=MAX(FFMX,FREQ(IJ))
  126    CONTINUE
         FRQMX(IT)=FFMX
  125 CONTINUE
C
C     select explicit continuum frequencies
C
      IF(NJREAD.LT.0) CALL INIFRC(1)
C
C     useful quantities for later use
C
      DO 140 IT=1,NTRANS
         LINEXP(IT)=(.NOT.LINE(IT).OR.INDEXP(IT).EQ.0)
         IF(.NOT.LINEXP(IT)) XPIF(IT)=PI4H/FR0(IT)
  140 CONTINUE
C
      CALL TRAINI
C
C     sorting frequencies & new weights
C
      DO 150 IJ=NFREQC+1,NFREQ
  150    IJX(IJ)=1
      IF(IOVER.GT.0) CALL SRTFRQ
C
C     final management of subtraction weights
C
      CALL CORRWM
C
C     print out some important transition parameters
C
      WRITE(6,607) NTRANS
      DO 130 IT=1,NTRANS
         IF(FR0(IT).GT.0.) THEN
           ALAM=2.997925D18/FR0(IT)
         ELSE
           ALAM=0.
         END IF
         WRITE(6,608) IT,ILOW(IT),IUP(IT),INDEXP(IT),ICOL(IT),
     *                IFR0(IT),IFR1(IT),OSC0(IT),FR0(IT),ALAM
  130 CONTINUE
      DO 131 IT=1,NTRANS
         FR15=FR0(IT)*T15
         BNTR(IT)=BN*FR15*FR15*FR15
         IF(FR0(IT).GT.0.) BNTRIN(IT)=UN/BNTR(IT)
  131 CONTINUE
C
C     --------------------------------------------------------
C     Evaluation of the photoionization cross-sections in both
C     explicit and fixed-option continuum transitions
C
C
C        array   CROSS(transition,frequency)
C
      NFREQB=NFREQ
      IF(IBFINT.GT.0) NFREQB=NFREQC
      if(lasv) call sigave(ibf)
      DO 220 ITR=1,NTRANS
         IF(LINE(ITR).OR.INDEXP(ITR).EQ.0) GO TO 220
	 IC=ITRA(IUP(ITR),ILOW(ITR))
         if(ibf(ic).gt.49.and.ibf(ic).lt.100) go to 220
         ISIK=0
         MODW=IABS(INDEXP(ITR))
         IF(MODW.EQ.5 .OR. MODW.EQ.15) ISIK=1
         DO 210 IJ=1,NFREQB
            BFCS(IC,IJ)=SIGK(FREQ(IJ),ITR,ISIK)
            IF(FREQ(IJ).LT.FR0PC(ITR)) BFCS(IC,IJ)=0.
            IF(IFWOP(ILOW(ITR)).LT.0) BFCS(IC,IJ)=1.E-30
  210    CONTINUE
  220 CONTINUE
C
C -----------------------------------------
C Input parameters for additional opacities
C -----------------------------------------
C
C     IOPADD  -  switch whether to calculate additional opacities
C             =  0  -  no additional opacities
C             >  0  -  some additional opacities, the user either
C                      employs existing expressions in OPADD, or add his
C                      own expressions into OPADD
C
C     for IOPADD > 0, there are the following existing possibilities
C                (for details, see OPADD); the relevant opacity is
C                evaluated if the values of the following switches are 
C                non-zero:
C -----------------------------------------
C     IRSCT   -  Rayleigh scattering
C     IOPHMI  -  H- opacity (both bound-free and free-free), assuming LTE.
C                Note that H- can be considered as one of explicit ions,
C                then the opacity is automatically calculated (in NLTE if
C                such model is calculated). In this case one must give
C                IOPHMI=0 in order not to calculate H- opacity twice.
C     IOPH2P  -  opacity of H2+  (molecular hydrogen ion)
C     IOPHE1  -  approximate, hydrogenic, opacity of neutral helium
C                given as a sum of bound-free transitions from averaged
C                levels with principal quantum numbers between that next
C                to the highest level considered explicitly and IOPHE1
C     IOPHE2  -  the same for ionized helium
C     IOPHL1  -  outdated option
C     IOPHL2  -  outdated option
C     IPHE2C  -  outdated option
C
      READ(IBUFF,*) IOPADD
      IF(IOPADD.GT.0) THEN
         READ(IBUFF,*) IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,IOPHL1,IOPHL2,
     *             IPHE2C
         WRITE(6,612) IOPADD,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,IOPHL1,
     *                IOPHL2,IPHE2C
C
C     procedure OPADD(-1,...) now sets up array CROSS that
C     contain relevant cross-sections
C
         DO 240 IJ=1,NFREQB
            CALL OPADD(-1,0,IJ,1)
  240    CONTINUE
      END IF
C
      IF(IOPHL1.NE.0 .OR. IOPHL2.NE.0) then
         if(ielh.le.0) call stop(' conflict in iophl1,2 - no hydrogen')
         CALL OPAHST
      end if
      if(iophe1.gt.0.and.ielhe1.le.0) call stop(' conflict in iophe1')
      if(iophe2.gt.0.and.ielhe2.le.0) call stop(' conflict in iophe2')
      if(iphe2c.gt.0.and.ielhe2.le.0) call stop(' conflict in iphe2c')
C
C ----------------------
C Input model atmosphere
C ----------------------
C     is controlled by procedure INPMOD, which reads input model
C     from unit 8;
C     there is no input of initial model atmosphere if LTGREY=true,
C     ie. if the initial model is calculated (in the LTE-grey approxim.)
C
C     INTRPL  -  switch indicating whether the input model has to be
C                interpolated to the present depth scale;
C                for details see procedure INPMOD
C     ICHANG  -  switch indicating whether the populations from the
C                input model have to be updated;
C                for details see procedure CHANGE
C
C     Important note: Procedure CHANGE also contains input from unit 5
C
      IF(.NOT.LTGREY) THEN
         READ(IBUFF,*) INTRPL,ICHANG
c         CALL INPMOD
         IF(ICHANG.NE.0) CALL CHANGE
       ELSE
C
C     If an initial LTE-grey model is to be calculated, call LTEGR
C     Important note: Procedure LTEGR also contains input from unit 5
C
         if(nogray.eq.0) CALL LTEGR
      END IF
C
C -----------------------------
C Driving parameters for RESOLV
C -----------------------------
C
C     NLAMBD     -    number of "lambda" iterations, i.e. the iterations
C                     of the formal solution
C     if NLAMBD > 0, there are NLAMBD pairs of in input records, 
C                    which are however now outdated (the records are kept
C                    here due to downward compatibility of input data)
C     if NLAMBD < 0, no pairs of input records, NLAMBD is set to -NLAMBD
C                    This is a recommended option.
C ------------------------------------------------------------------
C
      READ(IBUFF,*) NLAMBD
      IF(NLAMBD.GT.0) THEN
         DO 260 I=1,NLAMBD
            READ(IBUFF,*) IDUMMY
            READ(IBUFF,*) IDUMMY
  260    CONTINUE
       ELSE IF(NLAMBD.LT.0) THEN
         NLAMBD=-NLAMBD
      END IF
C
C --------------------------------------------------------------------
C Parameters specifying the overall organization and structure of the
C global matrices of complete linearization
C --------------------------------------------------------------------
C
C     ICLMAT  = 0  -  indicates the standard form of matrices and of
C                     the vector of unknown model parameters;
C                     the user need not care about the precise
C                     meaning of the following parameters;
C             - otherwise, some of the following parameters assume a
C               non-standard value
C
C     ICONV  = 0 - convection is neglected altogether (in linearization)
C            > 0 - number of additional model parameters needed for
C                  treating convection - usually = 1 (the only additional
C                  parameter is the logarithmic gradient DELTA)
C            =-1 - convection is taken into account, but is not
C                  linearized
C     IPRESS = 0 - total pressure is held fixed when evaluating
C                  derivatives of the convective flux 
C                  This is RECOMMENDED option in most cases
C            = 1 - derivatives wrt. total pressure are calculated
C     ITEMP  = 0 - temperature after a completed iteration of
C                  complete linearization is calculated as
C                  T(new)=T(old)+delta(T) (as it is done without
C                  convection)
C            = 1 - new temperature calculated through DELTA in the
C                  convection zone 
C            = 2 - new temperature calculated through DELTA everywhere
C            Recommended option is either 0 or 1
C     IPRINT - controls diagnostic output for models with convection
C            = 0 - no additional print, only final model printed
C            = 1 - convective flux and results of CONCOR printed
C                  after each iteration
C
C    The following parameters INHE,INRE,INPC,INSE,INMP each correspond
C    to one equation and one unknown model parameter, namely
C
C     INHE  -  hydrostatic equilibrium - total particle number density
C     INRE  -  radiative equilibrium - temperature
C     INPC  -  particle conservation - electron number density
C     INSE  -  statistical equilibrium - set of level populations
C     INMP  -  definition of mass density - fictitious massive particle
C              density (see Mihalas, 1978);
C              Note: INHE and INMP cannot be simultaneously non-zero.
C     INDL  -  logarithmic gradient of temperature (useful only
C              if convection is taken into account)
C
C     if INxx = 0, the corresponding equation is not solved, and the
C                  corresponding quantity is thus held fixed
C     if INxx > 0, the corresponding equation is solved, and the
C                  corresponding quantity is (NFREQE+INxx)-th component
C                  of the vector PSI of unknown model parameters
C                  (first NFREQE components are mean intensities of
C                  radiation in explicit frequencies)
C     NN    -  total number of unknown model parameters (ie. also the
C              size of basic matrices of complete linearization)
C     NDRE  -  defines a treatment of the radiative equilibrium equation:
C           =  0 or 1 - a superposition of the integral and the differential
C                       form is used;
C           =  0  - coefficients of the superposition are determined by the
C                   program. This is the recommended option!!
C                   The superposition is defined by coefficients
C                   REINT(ID) and REDIF(ID) (for the integral and
C                   differential parts, respectively.
C                   The default superposition is that
C                       REINT(ID)=1 for id.le ND-IDLST
C                                =0 elsewhere;
C                       REDIF(ID)=0 for Rosseland optical depths < TAUDIV
C                                =1 elsewhere;
C                    Default values of IDLST and TAUDIV are:
C                        IDLST=5
C                        TAUDIV=0.5
C                    To change these default values, see the next
C                    parameters on this line of input 
C                    (if these values are not there, the default applies)
C           =  1  - coefficients - REINT(ID),REDIF(ID) - are read from unit 95
C           >  1  - coefficients are step functions with the step at
C                   ID = NDRE (as in previous versions), i.e.
C              for depth points  ID=1,NDRE-1  - treated as integral
C                                               equation;
C              for depth points  ID=NDRE+1,ND - treated as differential
C                                               equation
C            If convection is taken into account, NDRE is calculated
C            as the point before the convection zone started
C
C     TAUDIV,IDLST - partitioning coefficients for the mode NDRE=0 - see above
C
C --------------------------------------------------------------------
C
C     LCHMAT -  a dummy input line (a former option, now outdated)
C
      READ(IBUFF,*) ICLMAT,ICONV,IPRESS,ITEMP,IPRINT
      IF(ICLMAT.EQ.0) THEN
         INHE=1
         INRE=2
         INPC=3
         INSE=4
         INMP=0
         INDL=0
         NN=NFREQE+NLVEXP+3
         IF(ICONV.GT.0) THEN
            INDL=3
            INPC=4
            INSE=5
            NN=NFREQE+NLVEXP+4
         END IF
         NDRE=0
         taudiv=0.5
         if(ifali.ge.6) taudiv=un
         idlst=5
         LCHMAT=.FALSE.
       ELSE
         READ(IBUFF,*,err=263) INHE,INRE,INPC,INSE,INMP,INDL,NDRE,
     *                        taudiv,idlst
         go to 264
  263    taudiv=0.5
         if(ifali.ge.6) taudiv=un
         idlst=5
  264    nn=nfreqe
         if(inhe.gt.0) nn=nn+1
         if(inre.gt.0) nn=nn+1
         if(inpc.gt.0) nn=nn+1
         if(inmp.gt.0) nn=nn+1
         if(indl.gt.0) nn=nn+1
         if(inse.gt.0) nn=nn+NLVEXP
         READ(IBUFF,*) 
         LCHMAT=.FALSE.
      END IF
c
C     change LCHC and IRSPLT in case that INPC=0
C    
      IF(INPC.EQ.0) THEN
         IRSPLT=1
         LCHC=.TRUE.
      END IF
C 
      if(ndre.eq.1) then
         do 265 id=1,nd
            read(ibuff,*) reint(id),redif(id)
  265    continue
       else if(ndre.gt.1) then
         do 266 id=1,nd
            if(id.lt.ndre) then
               reint(id)=1.
               redif(id)=0.
             else
               reint(id)=0.
               redif(id)=1.
            end if
  266    continue
      end if
C
C --------------------------------------------------------------------
C Acceleration parameters
C --------------------------------------------------------------------
C
C     IACC   - switch for acceleration procedure
C           <= 0 - acceleration switches off
C           <= 4 - Ng acceleration when ITER=7,10,...
C           >= 5 - Ng acceleration when ITER=IACC,IACC+3,...
C     IACD   - Ng acceleration is performed every IACD iterations
C              (if IACC>0); IACD>=4
C
C     ORELAX - over-relaxation factor
C     ITEK   - iteration after which Kantorovitch method is set up
C
C     if ITEK < 0, then read for all iterations a value KANT, where
C              KANT=0  - signifies a full complete linearization iteration;
C              KANT=1  - a Kantorovich iteration
C
      DO 268 ikt=1,NITER
        KANT(ikt)=0
  268 CONTINUE
      READ(IBUFF,*) IACC,IACD,ORELAX,ITEK
      IF(ORELAX.EQ.0.) ORELAX=UN
      IACC0=IACC-3
      LAC2=.FALSE.
      LRES2=.TRUE.
      IF(ITEK.LT.0) THEN
        READ(IBUFF,*) (KANT(ikt),ikt=1,NITER)
        IF(KANT(1).NE.0)
     *    CALL STOP(' *** Kantorovich in first iteration')
      ELSE IF(ITEK.GT.0) THEN
        DO 269 ikt=ITEK+1,NITER
          KANT(ikt)=1
  269   CONTINUE
       ENDIF
C
C ------------------------------------
C Wind-blanketing parameters - albedos; or irradiation intensity
C ------------------------------------
C
C     IWINBL  = 0  -  wind blanketing is not considered
C             > 0  -  wind blanketing is considered, basically as in
C                     Abbott and Hummer, Ap.J. 294, 286 (1985), slightly
C                     modified after Voels et al. (1988) to treat
C                     properly the angle-averaged albedos
C             < 0  -  wind blanketing is not considered, but irradiation
C
C     in the latter case, ALBE(IJ) are the albedos 
C     Specifically, ALBE are given by
C       integral over mu from 0 to 1 [intensity(taumin,-mu,freq) d mu] /
C       integral over mu from 0 to 1 [intensity(taumin,+mu,freq) d mu]
C 
C ------------------------------------
C    Input of (generally) frequency-dependent albedos is optional:
C   
C    For IWINBL = 1  - all values of ALBE are read;
C        IWINBL = 2  - not used any mor
C        IWINBL = 3  - condensed input of ALBE:
C                      Next line of input contains ALCON0,ALLIN0, where
C                      ALCON0 - standard (frequency-independent) albedo
C                               in continuum, except the HeII Lyman cont;
C                      ALCHE2 - standard (frequency-independent) albedo
C                               in the HeII Lyman continuum;
C                      ALLIN0 - standard (frequency-independent) albedo
C                               in lines.
C                      Next line(s) specify for which frequency points are
C                      these standard values modified:
C                      NUMMOD - number of (continuous) frequency regions
C                               where the albedo is non-standard;
C                               modifications can be done only for explicit
C                               frequencies).
C                      For NUMMOD>0, there are NUMMOD addition lines of 
C                      input, each containing:
C                      IJ1  - index of the first frequency point
C                      IJ2  - index of the last frequency point
C                      ALB  - value of albedo, by which is the standard
C                             value replaced for frequencies between
C                             IJ1 and IJ2 (including end points).
C
      EXTOT=0.
      READ(IBUFF,*) IWINBL
      IF(IWINBL.EQ.1) THEN
         READ(IBUFF,*) (ALBE(I),I=1,NFREQ)
       ELSE IF(IWINBL.EQ.2) THEN
         READ(IBUFF,*) (ALBE(I),I=1,NFREQ)
       ELSE IF(IWINBL.EQ.3) THEN
         READ(IBUFF,*) ALCON0,ALCHE2,ALLIN0
         DO 310 IJ=1,NFREQC
            ALBE(IJ)=ALCON0
            IF(FREQ(IJ).GT.1.315D16) ALBE(IJ)=ALCHE2
  310    CONTINUE
         DO 320 IJ=NFREQC+1,NFREQ
  320       ALBE(IJ)=ALLIN0
         READ(IBUFF,*) NUMMOD
         IF(NUMMOD.GT.0) THEN
            DO 360 I=1,NUMMOD
               READ(IBUFF,*) IJ1,IJ2,ALB
               DO 350 IJ=IJ1,IJ2
  350             ALBE(IJ)=ALB
  360       CONTINUE
         END IF
       ELSE IF(IWINBL.EQ.-1) THEN
         READ(IBUFF,*) TRAD,WDIL
         IF(WDIL.EQ.0.) WDIL=UN
         EXTOT=0.
         IF(TRAD.LE.0) THEN
            DO 361 IJ=1,NFREQ
               EXTRAD(IJ)=0.
  361       CONTINUE
          ELSE
            DO 362 IJ=1,NFREQ
               EXTRAD(IJ)=BNUE(IJ)/(EXP(HK*FREQ(IJ)/TRAD)-UN)*WDIL
               EXTOT=EXTOT+W(IJ)*EXTRAD(IJ)
  362       CONTINUE
         END IF
       ELSE IF(IWINBL.LT.-1) THEN
         DO 364 IJ=1,NFREQ
            READ(IBUFF,*) EXTRAD(IJ)
            EXTOT=EXTOT+W(IJ)*EXTRAD(IJ)
  364    CONTINUE
      END IF
      EXTOT=EXTOT/SIG4P/4.
      EXTOT0=WDIL*TRAD*TRAD*TRAD*TRAD
      write(6,496) extot0,extot
  496 format(' extot0,extot  ',1p2e12.3)
      IF(LTGREY.AND.IWINBL.LT.0) CALL LTEGR
C
C -----------------------------------------------------
C Format of printing level populations, and other output
C -----------------------------------------------------
C
C     NPGPOP  =  0  -  standard option (level populations are printed
C                      for 6 levels per page, consecutively by increasing
C                      index
C             <0, >-10  -  the same print of populations as for NPGPOP=0;
C                      but also unit 77 is created, equivalent to unit 7,
C                      but a new model is appended after each completed 
C                      iteration of CL
C             < -10 -  the same print of populations as for NPGPOP=0;
C                      but also unit 77 is created, equivalent to unit 7,
C                      but a new model is appended after each lambda 
C                      iteration in each CL iteration
C             >  0  -  non-standard option, in this case the array IIPR
C                      is read 
C
C      ------------------------------------
C      ((IIPR(I,J),I=1,6),J=1,NPGPOP) - means that level population (and
C                      the b-factor) of the level with index IIPR appears
C                      as an I-th column on the J-th page; IIPR zero
C                      leaves the corresponding column blank
C
      IPRIND=0
      READ(IBUFF,*) NPGPOP
      IF(NPGPOP.GT.0) THEN
         READ(IBUFF,*) ((IIPR(I,J),I=1,6),J=1,NPGPOP)
       ELSE
         IF(NPGPOP.LT.0) IPRIND=1
         IF(NPGPOP.LT.-10) IPRIND=2
         NPGPOP=(NLEVEL-1)/6+1
         DO 275 J=1,NPGPOP
            J1=6*(J-1)
            DO 275 I=1,6
               IIPR(I,J)=0
               IF(J1+I.LE.NLEVEL) IIPR(I,J)=J1+I
  275    CONTINUE
      END IF
C
C     ---------------------------------------------------------
C     Parameters for collisional-radiative switching
C     (see Hummer and Voels, Astron. Astrophys. 192, 279, 1988)
C     ---------------------------------------------------------
C
C     their procedure modified here by introducing depth-dependent
C     switching parameter CRSW(ID)
C
C     ICRSW   = 0  - collisional-radiative switching not considered
C             > 0  - collisional-radiative switching is considered; and
C             = 1  -  with depth-independent switching parameter
C             > 1  -  with dept-dependent switching parameter
C     SWPFAC  - initial CRSW = SWPFAC * min(collis.rate/rad.rate)
C                 if given as 0, program assigns default value 0.1
C     SWPLIM  - has the meaning: if CRSW > SWPLIM, then CRSW = 1
C                 if given as 0, program assigns default value 0.001
C     SWPINC  - CRSW(actual) = CRSW(previous) * SWPINC
C                 if given as 0, program assigns default value 3
C
      READ(IBUFF,*) ICRSW,SWPFAC,SWPLIM,SWPINC
      IF(ICRSW.EQ.0) THEN
         DO 280 ID=1,ND
  280       CRSW(ID)=UN
       ELSE
         IF(SWPFAC.EQ.0.) SWPFAC=1.D-1
         IF(SWPLIM.EQ.0.) SWPLIM=1.D-3
         IF(SWPINC.EQ.0.) SWPINC=3.D0
      END IF
C
  601 FORMAT(1H1,'*******************************'//
     *           ' M O D E L   A T M O S P H E R E'//
     *           ' *******************************'//
     * ' TEFF   =',F10.0/' LOG G  =',F10.2/)
  602 FORMAT(1H0//' YTOT   =',F11.5/
     * ' WMY    =',1PD15.5/' WMM    =',D15.5/)
  603 FORMAT(1H0//' EXPLICIT IONS INCLUDED'/
     *            ' ----------------------'//
     *  ' NO.  ION     N0    N1    NK    IZ  IUPSUM ICUP       FF'/)
  604 FORMAT(1H ,I3,2X,A4,6I6,1PD15.3)
  605 FORMAT(1H0//' EXPLICIT ENERGY LEVELS INCLUDED'/
     *            ' -------------------------------'//
     * ' NO.   LEVEL    ION   ION.EN.(ERG)        G   NQUANT',
     * '  IEL  ILK  IAT'/)
  606 FORMAT(1H ,I3,2X,A10,A4,1PD15.7,0PF10.2,4I5)
  607 FORMAT(1H0//' TRANSITION PARAMETERS (TOTAL OF',I6,' )'/
     *            ' ---------------------'//
     * '    ITR ILOW IUP INDEXP ICOL   IFR0   IFR1     OSC',
     * '        FR0',8X,' LAMBDA'/)
  608 FORMAT(1H ,I6,4I5,2I7,1P2D12.3,0PF12.3)
  609 FORMAT(1H0//' FREQUENCY POINTS AND WEIGHTS - EXPLICIT'/
     *            ' ---------------------------------------'//
     * '    IJ',7X,'FREQ',13X,'WEIGHT',11X,'PROF',10X,
     * 'WSUB'/)
  610 FORMAT(1H ,I5,1P2D17.8,D15.5,I5,D17.8)
  612 FORMAT(1H0,
     *'IOPADD,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,IOPHL1,IOPHL2,IPHE2C:'
     */9I5)
C
C     check the dimensions
C
      if(nd.gt.mdepth) call stop('nd too large')
      if(nn.gt.mtot) call stop('nn too large')
      if(nfreqe.gt.mfrex) call stop('nfreqe too large')
      if(nfreq.gt.mfreq) call stop('nfreq too large')
      if(nlevel.gt.mlevel) call stop('nlevel too large')
      if(ntrans.gt.mtrans) call stop('ntrans too large')
      if(natom.gt.matom) call stop('natom too large')
      if(nion.gt.mion) call stop('nion too large')
      if(lchmat.and.niter.gt.miter) call stop('niter too large')
      if(nlambd.gt.mlambd) call stop('nlambd too large')
      if(nhod.gt.mhod) call stop('nhod too large')
      if(isplin.lt.0) call stop('inconsistent isplin')
c
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE INITIA
C     =================
C
C     driver for input and initializations - "new" routine
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (T15=1.D-15)
      CHARACTER*20 FINSTD
      PARAMETER (MPAG=MLEVEL/6+1)
      CHARACTER*30 FIDATA(MION),FIODF1(MION),FIODF2(MION),FIBFCS(MION),
     *             FILEI
      CHARACTER*10 TYPLEV(MLEVEL)
      CHARACTER*4 TYPION(MION),TYPIOI
      COMMON/IONDAT/IATI(MION),IZI(MION),NLEVS(MION),NLLIM(MION)
      COMMON/IONFIL/FIDATA,FIODF1,FIODF2,FIBFCS
      COMMON/INUNIT/IUNIT
      COMMON/PHOSET/S0(MLEVEL),ALF(MLEVEL),BET(MLEVEL),GAM(MLEVEL),
     *              IBF(MLEVEL)
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
      COMMON/PRINTP/NPGPOP,IIPR(6,MPAG),TYPLEV,TYPION
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      COMMON/DFEALI/DJMAX,NTRALI
      COMMON/EXTOTR/EXTOT
      COMMON/STRPAR/IMER,ITR,IC,IL,IP,NLASTE,NHOD
      DIMENSION IGLE(18),IGMN(25),IGFE(26),IGNI(28)
      DATA IGLE/2,1,2,1,6,9,4,9,6,1,2,1,6,9,4,9,6,1/
      DATA IGMN/2,1,2,1,6,9,4,9,6,1,2,1,6,9,4,9,6,1,
     *          10,21,28,25,6,7,6/
      DATA IGFE/2,1,2,1,6,9,4,9,6,1,2,1,6,9,4,9,6,1,
     *          10,21,28,25,6,25,30,25/
      DATA IGNI/2,1,2,1,6,9,4,9,6,1,2,1,6,9,4,9,6,1,
     *          10,21,28,25,6,25,28,21,10,21/
C
C ----------------------
C Basic input parameters
C ----------------------
C
      CALL READBF
      READ(IBUFF,*) TEFF,GRAV
      READ(IBUFF,*) LTE,LTGREY
      READ(IBUFF,*) FINSTD
      CALL NSTPAR(FINSTD)
C
C
C ----------------------------
C Frequency points and weights
C ----------------------------
C
C     NFREAD     - number of "continuum" frequency points which are
C                  read. May be less than NFREQ - the actual number of
C                  frequencies, because the frequency points in lines
C                  can be calculated.
C     FREQ(IJ)   - frequency (in sec**-1) of the IJ-th freq. point
C     W(IJ)      - corresponding frequency quadrature weight
C
      READ(IBUFF,*) NFREAD
      NJREAD=NFREAD
      if(njread.gt.mfreq) call stop('njread too large')
C
      IF(NJREAD.LT.0) THEN
         NJREAD=-NJREAD
         NFREQC=NJREAD
         DO IJ=1,NJREAD
            WCH(IJ)=0.
            IJALI(IJ)=0
            IJX(IJ)=1
            READ(IBUFF,*) FREQ(IJ),W(IJ),WCH(IJ),IJALI(IJ)
         END DO   
       ELSE
         NFREQC=NJREAD
      END IF
C
   30 CONTINUE
      IF(NFFIX.EQ.2) THEN
         DO 12 IJ=1,NFREQC
            IJALI(IJ)=1
   12    CONTINUE
      END IF
C
      WRITE(6,601) TEFF,GRAV
      GRAV=EXP(2.3025851*GRAV)
C
C ----------------------------------------------------
C     Initialize 1/i*i, 1/i*i*i; and turbulent velocities
C ----------------------------------------------------
C
      DO I=1,NLMX
         X=I
         XI2(I)=UN/(X*X)
         XI3(I)=XI2(I)/X
      END DO
C
      IF(VTB.LT.1.E3) VTB=VTB*1.E5
      DO ID=1,ND
         VTURB(ID)=VTB
      END DO
C
C ----------------------------------------------------
C Input parameters for explicit and non-explicit atoms
C ----------------------------------------------------
C
C     Input parameters are read by procedure STATE
C     (see description there)
C
      CALL STATE2(0,ID,X1,X2)
      ID=1
      WRITE(6,602) YTOT(ID),WMY(ID),WMM(ID)
      DO I=1,NLEVEL
         ILK(I)=0
         DO J=1,NLEVEL
            ITRA(J,I)=0
         END DO
      END DO
C
C --------------------------------------------------------------
C Input of parameters for explicit ions, levels, and transitions
C --------------------------------------------------------------
C
      ILEV=0
      IATLST=0
      ION=0
      IA=0
      IUNIT=20
      NATOM=0
      WRITE(6,603)
   10 CONTINUE
      READ(IBUFF,*,END=20,ERR=20) IATII,IZII,NLEVSI,ILASTI,ILVLIN,
     *              NONSTD,TYPIOI,FILEI
      IF(ILASTI.EQ.0) THEN
         ION=ION+1
         IATI(ION)=IATII
         IZI(ION)=IZII
         NLEVS(ION)=NLEVSI
         TYPION(ION)=TYPIOI
         FIDATA(ION)=FILEI
         NLLIM(ION)=ILVLIN
         INODF1(ION)=0
         INODF2(ION)=0
         IUPSUM(ION)=0
         ICUP(ION)=16
         MODEFF=1
         NFF=0
         IF(IATI(ION).EQ.1.AND.IZI(ION).EQ.0) THEN 
            IUPSUM(ION)=-100
            MODEFF=2
         END IF
         IF(IATI(ION).EQ.2.AND.IZI(ION).EQ.1) THEN
            MODEFF=2
            ICUP(ION)=32
         END IF
         IF(NONSTD.GT.0) THEN
            READ(IBUFF,*) IUPSUM(ION),ICUP(ION),MODEFF,NFF
          ELSE IF(NONSTD.LT.0) THEN
            READ(IBUFF,*) INODF1(ION),INODF2(ION),FIODF1(ION),
     *                    FIODF2(ION),FIBFCS(ION)
            IF(INODF1(ION).EQ.0) THEN
               IUNIT=IUNIT+1
               INODF1(ION)=IUNIT
            END IF
            IF(INODF2(ION).EQ.0) THEN
               IUNIT=IUNIT+1
               INODF2(ION)=IUNIT
            END IF
            IF(FIBFCS(ION).NE.' ') THEN
               IUNIT=IUNIT+1
               INBFCS(ION)=IUNIT
            END IF
            IUPSUM(ION)=1
            ICUP(ION)=0
         END IF
C
         IF(IATI(ION).EQ.IATLST) THEN
            NFIRST(ION)=ILEV
          ELSE
            NFIRST(ION)=ILEV+1
            IATLST=IATI(ION)
            IA=IATEX(IATLST)
            N0A(IA)=NFIRST(ION)
            NATOM=MAX(NATOM,IA)
         END IF
         NLAST(ION)=NFIRST(ION)+NLEVS(ION)-1
         NNEXT(ION)=NLAST(ION)+1
         ILEV=NNEXT(ION)
         IZ(ION)=IZI(ION)+1 
         CHARG2(ION)=IZ(ION)*IZ(ION)
         IF(NFF.GT.0) FF(ION)=EH/H*IZ(ION)*IZ(ION)/NFF/NFF
C
         N0I=NFIRST(ION)
         N1I=NLAST(ION)
         NKI=NNEXT(ION)
         ITRA(NKI,NKI)=MODEFF
         DO II=N0I,N1I
            IEL(II)=ION
            IATM(II)=IA
         END DO
         ILK(NKI)=ION
         IATM(NKI)=IA
C
         IF(NUMAT(IA).EQ.1) THEN
            IATH=IA
            IF(IZ(ION).EQ.1) IELH=ION
            IF(IZ(ION).EQ.0) IELHM=ION
         END IF
         IF(NUMAT(IA).EQ.2) THEN
            IATHE=IA
            IF(IZ(ION).EQ.1) IELHE1=ION
            IF(IZ(ION).EQ.2) IELHE2=ION
         END IF
C         
         WRITE(6,604) ION,TYPION(ION),N0I,N1I,NKI,IZ(ION),
     *                IUPSUM(ION),ICUP(ION),FF(ION)
C   
       ELSE IF(ILASTI.GT.0) THEN
         ENION(ILEV)=0.
         G(ILEV)=ILASTI
         NQUANT(ILEV)=1
         TYPLEV(ILEV)=TYPIOI
         IMODL(ILEV)=0
         IFWOP(ILEV)=0
         IEL(ILEV)=ION
         NKA(IA)=NNEXT(ION)
         IF(ILASTI.EQ.1.AND.IATII.GT.IZII) THEN
            IF(IATII.LT.25) THEN
               G(ILEV)=IGLE(IATII-IZII)
             ELSE IF(IATII.EQ.25) THEN
               G(ILEV)=IGMN(IATII-IZII)
             ELSE IF(IATII.EQ.26) THEN
               G(ILEV)=IGFE(IATII-IZII)
             ELSE IF(IATII.EQ.28) THEN
               G(ILEV)=IGNI(IATII-IZII)
            ENDIF
         ENDIF
       ELSE
         GO TO 20
      END IF 
      GO TO 10    
   20 CONTINUE
      NION=ION
      NLEVEL=NKI
C     
      IMER=0
      ITR=0
      IC=0
      IL=0
      IP=0
      IF(NFREAD.LT.0) THEN
         NLASTE=NFREQC
         NFREQ=NFREQC
       ELSE
         NLASTE=0
         NFREQ=0
      END IF
      LBPFX=.TRUE.
C
      DO ION=1,NION
         CALL RDATA(ION) 
         NFF=NQUANT(NLAST(ION))+1
         IF(NFF.GT.0) FF(ION)=EH/H*IZ(ION)*IZ(ION)/NFF/NFF
      END DO
C
      NTRANS=ITR
      NFREQ=NLASTE
      NCON=IC
C
      CALL LEVSET
C
      WRITE(6,605)
      DO I=1,NLEVEL
         WRITE(6,606) I,TYPLEV(I),TYPION(IEL(I)),ENION(I),G(I),
     *                NQUANT(I),IEL(I),ILK(I),IATM(I),
     *                IMODL(I),ILTLEV(I),IIEXP(I),IIFOR(I)
      END DO
C
C ------------------------------------------------------
C     setup continuum frequencies (if they are not read)
C ------------------------------------------------------
C
      IF(NFREAD.GT.0) CALL INIFRD(0)
      IF(FRLMAX.LE.0.) FRLMAX=FREQ(1)
C
      lasv=.false.
      CALL HODFIN
C
C
C     check the dimensions
C
      if(nd.gt.mdepth) call stop('nd too large')
      if(nfreqe.gt.mfrex) call stop('nfreq too large')
      if(nfreq.gt.mfreq) call stop('nfreq too large')
      if(nlevel.gt.mlevel) call stop('nlevel too large')
      if(ntrans.gt.mtrans) call stop('ntrans too large')
      if(natom.gt.matom) call stop('natom too large')
      if(nion.gt.mion) call stop('nion too large')
      if(nfreqc.gt.mfreqc) call stop ('nfreqc too large')
      if(ibfint.eq.0.and.nfreq.gt.mfreqc)
     *   call stop('mfreqc too small for no-interpol b-f c-s')
c
C  -----------------------------------------------------------
C     read the input model
C  -----------------------------------------------------------
C
      IF(.NOT.LTGREY) THEN
         CALL INPMOD
         IF(ICHANG.NE.0) CALL CHANGE
      END IF
C
      CALL LINSET(0,0,0,0,0.,0.,0.)
C
C  -----------------------------------------------------------
C   There is no additional input for ODF transitions (i.e. ABS(MODE)=3)
C   from standard input; all information is taken from corresponding 
C   input files (subroutine ODFSET)
C  -----------------------------------------------------------
C
C     Input parameters for ODF lines
C
      DOPSTD=SQRT(TWO*BOLK*TEFF/HMASS+VTB*VTB)
      IF(NHOD.GT.0) CALL ODFHYS(DOPSTD)
      CALL ODFSET
C
C  -----------------------------------------------------------
C     maximum frequency in lines
C  -----------------------------------------------------------
C
      DO 125 IT=1,NTRANS
         IF(.NOT.LINE(IT)) GOTO 125
         FFMX=0.
         DO 126 IJ=IFR0(IT),IFR1(IT)
            FFMX=MAX(FFMX,FREQ(IJ))
  126    CONTINUE
         FRQMX(IT)=FFMX
  125 CONTINUE
C
C  -----------------------------------------------------------
C     select explicit continuum frequencies
C  -----------------------------------------------------------
C
      IF(NFREAD.GT.0) CALL INIFRD(1)
C
C  -----------------------------------------------------------
C     useful quantities for later use
C  -----------------------------------------------------------
C
      DO 140 IT=1,NTRANS
         LINEXP(IT)=(.NOT.LINE(IT).OR.INDEXP(IT).EQ.0)
         IF(.NOT.LINEXP(IT)) XPIF(IT)=PI4H/FR0(IT)
  140 CONTINUE
C
      CALL TRAINI
C
C  -----------------------------------------------------------
C     sorting frequencies & new weights
C  -----------------------------------------------------------
C
      DO IJ=NFREQC+1,NFREQ
         IJX(IJ)=1
      END DO
      IF(IOVER.GT.0) CALL SRTFRQ
C
C  -----------------------------------------------------------
C     final management of subtraction weights
C  -----------------------------------------------------------
C
      CALL CORRWM
C
C  -----------------------------------------------------------
C     print out some important transition parameters
C  -----------------------------------------------------------
C
      WRITE(6,607) NTRANS
      DO IT=1,NTRANS
         IF(FR0(IT).GT.0.) THEN
           ALAM=2.997925D18/FR0(IT)
         ELSE
           ALAM=0.
         END IF
         WRITE(6,608) IT,ILOW(IT),IUP(IT),INDEXP(IT),ICOL(IT),
     *                IFR0(IT),IFR1(IT),OSC0(IT),FR0(IT),ALAM
      END DO
      DO IT=1,NTRANS
         FR15=FR0(IT)*T15
         BNTR(IT)=BN*FR15*FR15*FR15
         IF(FR0(IT).GT.0.) BNTRIN(IT)=UN/BNTR(IT)
      END DO
C
C     --------------------------------------------------------
C     Evaluation of the photoionization cross-sections in both
C     explicit and fixed-option continuum transitions
C
C        array   CROSS(transition,frequency)
C     --------------------------------------------------------
C
      NFREQB=NFREQ
      IF(IBFINT.GT.0) NFREQB=NFREQC
      if(lasv) call sigave(ibf)
      DO 220 ITR=1,NTRANS
         IF(LINE(ITR).OR.INDEXP(ITR).EQ.0) GO TO 220
	 IC=ITRA(IUP(ITR),ILOW(ITR))
         if(ibf(ic).gt.49.and.ibf(ic).lt.100) go to 220
         ISIK=0
         MODW=IABS(INDEXP(ITR))
         IF(MODW.EQ.5 .OR. MODW.EQ.15) ISIK=1
         DO 210 IJ=1,NFREQB
            BFCS(IC,IJ)=SIGK(FREQ(IJ),ITR,ISIK)
            IF(FREQ(IJ).LT.FR0PC(ITR)) BFCS(IC,IJ)=0.
            IF(IFWOP(ILOW(ITR)).LT.0) BFCS(IC,IJ)=1.E-30
  210    CONTINUE
  220 CONTINUE
C
C -----------------------------------------
C Input parameters for additional opacities
C -----------------------------------------
C
      IF(IOPADD.GT.0) THEN
         WRITE(6,612) IOPADD,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,IOPHL1,
     *                IOPHL2,IPHE2C
C
C     procedure OPADD(-1,...) now sets up array CROSS that
C     contain relevant cross-sections
C
         DO 240 IJ=1,NFREQB
            CALL OPADD(-1,0,IJ,1)
  240    CONTINUE
      END IF
C
      IF(IOPHL1.NE.0 .OR. IOPHL2.NE.0) then
         if(ielh.le.0) call stop(' conflict in iophl1,2 - no hydrogen')
         CALL OPAHST
      end if
      if(iophe1.gt.0.and.ielhe1.le.0) call stop(' conflict in iophe1')
      if(iophe2.gt.0.and.ielhe2.le.0) call stop(' conflict in iophe2')
      if(iphe2c.gt.0.and.ielhe2.le.0) call stop(' conflict in iphe2c')
C
C  -----------------------------------------------------------
C     calculate the input LTE-grey model, if needed
C  -----------------------------------------------------------
C
      IF(LTGREY) CALL LTEGR2
C
C --------------------------------------------------------------------
C Parameters specifying the overall organization and structure of the
C global matrices of complete linearization
C --------------------------------------------------------------------
C
      IF(HMIX0.GT.0) THEN
         ICONV = 1
         INDL  = 3
         INPC  = 4
         INSE  = 5
      END IF
      nn=nfreqe
      if(inhe.gt.0) nn=nn+1
      if(inre.gt.0) nn=nn+1
      if(inpc.gt.0) nn=nn+1
      if(inmp.gt.0) nn=nn+1
      if(indl.gt.0) nn=nn+1
      if(inse.gt.0) nn=nn+NLVEXP
      LCHMAT=.FALSE.
      if(ifali.ge.6) taudiv=un
c
C     change LCHC and IRSPLT in case that INPC=0
C
      IF(INPC.EQ.0) THEN
         IRSPLT=1
         LCHC=.TRUE.
      END IF
C
C --------------------------------------------------------------------
C Acceleration parameters
C --------------------------------------------------------------------
C
C     if ITEK < 0, then read for all iterations a value KANT, where
C              KANT=0  - signifies a full complete linearization iteration;
C              KANT=1  - a Kantorovich iteration
C
      DO IKT=1,NITER
         KANT(IKT)=0
      END DO  
      IF(ORELAX.EQ.0.) ORELAX=UN
      IACC0=IACC-3
      LAC2=.FALSE.
      LRES2=.TRUE.
      IF(ITEK.LT.0) THEN
         READ(IBUFF,*) (KANT(IKT),IKT=1,NITER)
         IF(KANT(1).NE.0)
     *      CALL STOP(' *** Kantorovich in first iteration')
       ELSE IF(ITEK.GT.0) THEN
         DO IKT=ITEK+1,NITER
            KANT(IKT)=1
         END DO
      END IF
C
C ------------------------------------
C Wind-blanketing parameters - albedos; or irradiation intensity
C ------------------------------------
C
      IF(IWINBL.EQ.1) THEN
         READ(IBUFF,*) (ALBE(I),I=1,NFREQ)
       ELSE IF(IWINBL.EQ.2) THEN
         READ(IBUFF,*) (ALBE(I),I=1,NFREQ)
       ELSE IF(IWINBL.EQ.3) THEN
         READ(IBUFF,*) ALCON0,ALCHE2,ALLIN0
         DO 310 IJ=1,NFREQC
            ALBE(IJ)=ALCON0
            IF(FREQ(IJ).GT.1.315D16) ALBE(IJ)=ALCHE2
  310    CONTINUE
         DO 320 IJ=NFREQC+1,NFREQ
  320       ALBE(IJ)=ALLIN0
         READ(IBUFF,*) NUMMOD
         IF(NUMMOD.GT.0) THEN
            DO 360 I=1,NUMMOD
               READ(IBUFF,*) IJ1,IJ2,ALB
               DO 350 IJ=IJ1,IJ2
  350             ALBE(IJ)=ALB
  360       CONTINUE
         END IF
      END IF
C
C ------------------------------------
C External irradiation intensity
C ------------------------------------
C
      IF(TRAD.EQ.0) THEN
         DO IJ=1,NFREQ
            EXTRAD(IJ)=0.
         END DO
       ELSE IF(TRAD.GT.0) THEN
         DO IJ=1,NFREQ
            EXTRAD(IJ)=BNUE(IJ)/(EXP(HK*FREQ(IJ)/TRAD)-UN)*WDIL
            EXTOT=EXTOT+W(IJ)*EXTRAD(IJ)
         END DO
       ELSE 
         DO IJ=1,NFREQ
            READ(IBUFF,*) EXTRAD(IJ)
            EXTOT=EXTOT+W(IJ)*EXTRAD(IJ)
         END DO
      END IF
      EXTOT=EXTOT/SIG4P/4.
      EXTOT0=WDIL*TRAD*TRAD*TRAD*TRAD
      write(6,496) extot0,extot
  496 format(/' extot0,extot  ',1p2e12.3/)
C
C
C -----------------------------------------------------
C Format of printing level populations, and other output
C -----------------------------------------------------
C
C      ------------------------------------
C      ((IIPR(I,J),I=1,6),J=1,NPGPOP) - means that level population (and
C                      the b-factor) of the level with index IIPR appears
C                      as an I-th column on the J-th page; IIPR zero
C                      leaves the corresponding column blank
C
      IPRIND=0
      IF(NPGPOP.GT.0) THEN
         READ(IBUFF,*) ((IIPR(I,J),I=1,6),J=1,NPGPOP)
       ELSE
         IF(NPGPOP.LT.0) IPRIND=1
         IF(NPGPOP.LT.-10) IPRIND=2
         NPGPOP=(NLEVEL-1)/6+1
         DO 275 J=1,NPGPOP
            J1=6*(J-1)
            DO 275 I=1,6
               IIPR(I,J)=0
               IF(J1+I.LE.NLEVEL) IIPR(I,J)=J1+I
  275    CONTINUE
      END IF
C
      CALL NSTOUT
C
      if(nd.gt.mdepth) call stop('nd too large')
      if(nn.gt.mtot) call stop('nn too large')
      if(nfreqe.gt.mfrex) call stop('nfreqe too large')
      if(nfreq.gt.mfreq) call stop('nfreq too large')
      if(nlevel.gt.mlevel) call stop('nlevel too large')
      if(ntrans.gt.mtrans) call stop('ntrans too large')
      if(natom.gt.matom) call stop('natom too large')
      if(nion.gt.mion) call stop('nion too large')
      if(lchmat.and.niter.gt.miter) call stop('niter too large')
      if(nlambd.gt.mlambd) call stop('nlambd too large')
      if(nhod.gt.mhod) call stop('nhod too large')
      if(isplin.lt.0) call stop('inconsistent isplin')
C
  601 FORMAT(1H1,'*******************************'//
     *           ' M O D E L   A T M O S P H E R E'//
     *           ' *******************************'//
     * ' TEFF   =',F10.0/' LOG G  =',F10.2/)

c  602 FORMAT(1H0//' YTOT   =',F11.5/
c     * ' WMY    =',1PD15.5/' WMM    =',D15.5/)
  602 FORMAT(' YTOT WMY WMM ',F11.5,1P2D15.5)
  603 FORMAT(1H0//' EXPLICIT IONS INCLUDED'/
     *            ' ----------------------'//
     *  ' NO.  ION     N0    N1    NK    IZ  IUPSUM ICUP       FF'/)
  604 FORMAT(1H ,I3,2X,A4,6I6,1PD15.3)
  605 FORMAT(1H0//' EXPLICIT ENERGY LEVELS INCLUDED'/
     *            ' -------------------------------'//
     * ' NO.   LEVEL    ION   ION.EN.(ERG)        G   NQ',
     * '  IEL ILK IAT IMOD ILT IIE IIF'/)
  606 FORMAT(1H ,I3,2X,A10,A4,1PD15.7,0PF10.2,8I4)
  607 FORMAT(1H0//' TRANSITION PARAMETERS (TOTAL OF',I6,' )'/
     *            ' ---------------------'//
     * '    ITR ILOW IUP INDEXP ICOL   IFR0   IFR1     OSC',
     * '        FR0',8X,' LAMBDA'/)
  608 FORMAT(1H ,I6,4I5,2I7,1P2D12.3,0PF12.3)
  609 FORMAT(1H0//' FREQUENCY POINTS AND WEIGHTS - EXPLICIT'/
     *            ' ---------------------------------------'//
     * '    IJ',7X,'FREQ',13X,'WEIGHT',11X,'PROF',10X,
     * 'WSUB'/)
  610 FORMAT(1H ,I5,1P2D17.8,D15.5,I5,D17.8)
  612 FORMAT(1H0,
     *'IOPADD,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,IOPHL1,IOPHL2,IPHE2C:'
     */9I5)
C
      RETURN
      END
C     
C     
C ************************************************************************    
C     
C     
C  
      SUBROUTINE RDATA(ION)
C     =====================
C 
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (T15=1.D-15)
      PARAMETER (MPAG=MLEVEL/6+1) 
      CHARACTER*10 TYPLEV(MLEVEL)
      CHARACTER*4 TYPION(MION),TYPIOI
      CHARACTER*30 FIDATA(MION),FIODF1(MION),FIODF2(MION),FIBFCS(MION)
      CHARACTER*1 A
      COMMON/IONDAT/IATI(MION),IZI(MION),NLEVS(MION),NLLIM(MION)
      COMMON/IONFIL/FIDATA,FIODF1,FIODF2,FIBFCS
      COMMON/OSCHYD/OSH(20,20)
      COMMON/PHOSET/S0(MLEVEL),ALF(MLEVEL),BET(MLEVEL),GAM(MLEVEL),
     *              IBF(MLEVEL)
      COMMON/VOIPAR/GAMR(MVOIGT),STARK1(MVOIGT),STARK2(MVOIGT),
     *              STARK3(MVOIGT),VDWH(MVOIGT)
      COMMON/TOPCS/CTOP(MFIT,MCROSS), !sigma=alog10(sigma/10^-18) of fit point
     *             XTOP(MFIT,MCROSS)  ! x = alog10(nu/nu0) of fit point 
      COMMON/PRINTP/NPGPOP,IIPR(6,MPAG),TYPLEV,TYPION
      COMMON/INUNIT/IUNIT
      COMMON/STRPAR/IMER,ITR,IC,IL,IP,NLASTE,NHOD
C
      IUNIT=IUNIT+1
      OPEN(IUNIT,FILE=FIDATA(ION),STATUS='OLD')
C
C     read the first record - a label for the energy level input
C  
      READ(IUNIT,501) A
  501 FORMAT(A1)
C
C   -----------------------------------------------------
C   input parameters for explicit energy levels
C   -----------------------------------------------------
C  
      DO IL=1,NLEVS(ION)
         I=IL+NFIRST(ION)-1
         IE=IEL(I)
         N0I=NFIRST(IE)
         NKI=NNEXT(IE)
         IQ=I-N0I+1
         X=IQ*IQ
         READ(IUNIT,*) 
     *   ENION(I),G(I),NQUANT(I),TYPLEV(I),ifwop(i),FRODF(I),IMODL(I)
         if(ifwop(i).lt.0.and.i.ne.nlast(ie)) 
     *   call stop('conflict in negative ifwop')
         IF(I.LT.NKI) THEN
            E=ABS(ENION(I))
            E0=E
            IF(E.EQ.0.) E0=EH*IZ(IE)*IZ(IE)/X
            IF(E.GT.1.D-7.AND.E.LT.100.) E0=1.6018D-12*E
            IF(E.GT.100..AND.E.LT.1.D7) E0=1.9857D-16*E
            IF(E.GT.1.D7) E0=H*E
            IF(ENION(I).GE.0.) THEN
              ENION(I)=E0
            ELSE
              ENION(I)=-E0
            ENDIF
            IF(G(I).EQ.0.) G(I)=2.D0*X
            IF(NQUANT(I).EQ.0) NQUANT(I)=IQ
            iltlev(i)=0
            if(nquant(i).lt.0) then
               iltlev(i)=1
               nquant(i)=iabs(nquant(i))
            end if
          ELSE
            if(modref.ge.0) nref(iatm(i))=nka(iatm(i))
            IF(G(I).EQ.0..AND.NKI.EQ.NKA(IATM(I))) G(I)=1.
         END IF
         if(ifwop(i).lt.0) then
            enion(i)=0.
            IMER=IMER+1
            IMRG(I)=IMER
            IIMER(IMER)=I
         endif
         LBPFX = LBPFX .AND. IMODL(I).EQ.0 .AND. IIFIX(IATM(I)).EQ.0
      END DO
c
C ----------------------------------------------------------------------
C
C   skip lines if more levels than needed, and skip the continuum transition label
C
    5 READ(IUNIT,501) A
      IF(A.NE.'*') GO TO 5
      II0=NFIRST(ION)-1
      ILLIM=NLLIM(ION)+II0
      JCORR=0
C
C   -----------------------------------------------------
C   input parameters for continuum transitions
C   -----------------------------------------------------
C
   10 CONTINUE
      READ(IUNIT,*,END=20,ERR=15) II,JJ,MODE,IFANCY,ICOLIS,
     *                            IFRQ0,IFRQ1,OSC,CPARAM
      IFFR0=0
      IF(IABS(MODE).GT.100) THEN
         IF(MODE.GT.0) MODE=MODE-100
         IF(MODE.LT.0) MODE=MODE+100
         IFFR0=1
         READ(IUNIT,*) FR0INP
         IF(FR0INP.LT.1.E10) FR0INP=2.997925D18/FR0INP
      END IF
      ITR=ITR+1
      IF(IABS(MODE).EQ.5 .OR. IABS(MODE).EQ.15) THEN
         READ(IUNIT,*) FR0PC(ITR)
      END IF
      IF(II.EQ.1) JCORR=NLEVS(ION)+1-JJ
      II=II+II0
      JJ=JJ+II0+JCORR
      ILOW(ITR)=II
      IUP(ITR)=JJ
      INDEXP(ITR)=MODE
      OSC0(ITR)=OSC
      ICOL(ITR)=ICOLIS
      CPAR(ITR)=CPARAM
      LALI(ITR)=.FALSE.
      LEXP(ITR)=.FALSE.
      ISUB(ITR)=0
      IF(ITRA(II,JJ).EQ.0) THEN
         ITRA(II,JJ)=ITR
       ELSE
         ICOL(ITR)=99
      END IF
      IE=IEL(II)
      N0I=NFIRST(IE)
      NKI=NNEXT(IE)
      LINE(ITR)=.FALSE.
      IFC0(ITR)=IFRQ0
      IFC1(ITR)=IFRQ1
      FR0(ITR)=(ENION(II)-ENION(JJ)+ENION(NKI))/H
      IF(IFFR0.EQ.1) FR0(ITR)=FR0INP
C
C   -----------------------------------------------------
C   Additional input parameters for continuum transitions
C   -----------------------------------------------------
C
C     Only for IFANCY = 2, 3, or 4
C     S0, ALF, BET, GAM  - parameters for evaluation the
C         photoionization cross-section
C
      IC=IC+1
      NTRANC=IC
      IF(IFANCY.GE.2.AND.IFANCY.LE.4) THEN
         READ(IUNIT,*) S0(IC),ALF(IC),BET(IC),GAM(IC)
      END IF
C
C   -----------------------------------------------------
C   Additional input parameters for continuum transitions -TOPBASE DATA
C   -----------------------------------------------------
C
C     Only for IFANCY > 100 there are IFANCY-100 fit points
C
C     XTOP(MFIT,MCROSS) -  x = alog10(nu/nu0) of a fit point
C     CTOP(MFIT,MCROSS) -  sigma = alog10(sigma/10^-18) of a fit point
C
C     there are IFANCY-100 fit points
C
      IF(IFANCY.GT.100) THEN
         NFIT=IFANCY-100
         IF(NFIT.GT.MFIT) call stop(' nfit too large (TOPBASE fits)')
         READ(IUNIT,*) (XTOP(IFIT,IC),IFIT=1,NFIT)
         READ(IUNIT,*) (CTOP(IFIT,IC),IFIT=1,NFIT)
      END IF
      IBF(IC)=IFANCY
      if(ifancy.gt.49.and.ifancy.lt.100) lasv=.true.
      ITRA(JJ,II)=IC
      ITRBF(IC)=ITR
      IF(FR0(ITR).GT.0.) THEN
         ALAM=2.997925D18/FR0(ITR)
       ELSE
         ALAM=0.
      END IF
      IF(II.LT.NLAST(ION)) GO TO 10
   15 READ(IUNIT,501) A
      IF(A.NE.'*') GO TO 15
C
C  -----------------------------------------------------------
C  Input parameters for line transitions
C  -----------------------------------------------------------
C
   20 CONTINUE
      READ(IUNIT,*,END=30,ERR=30) II,JJ,MODE,IFANCY,ICOLIS,
     *                              IFRQ0,IFRQ1,OSC,CPARAM
      IFFR0=0
      IF(IABS(MODE).GT.100) THEN
         IF(MODE.GT.0) MODE=MODE-100
         IF(MODE.LT.0) MODE=MODE+100
         IFFR0=1
         READ(IUNIT,*) FR0INP
         IF(FR0INP.LT.1.E10) FR0INP=2.997925D18/FR0INP
      END IF
      IF(JJ.GT.NLEVS(ION)) THEN
         IF(IABS(MODE).EQ.2) THEN
            READ(IUNIT,*) K1,K2,K3,X1,X2,X3,K4
            GO TO 20
         END IF
         IF(IABS(MODE).EQ.1) READ(IUNIT,*) LCMP
         IF(IABS(IFANCY).EQ.1) READ(IUNIT,*) GAMRL
         GO TO 20
      END IF
      ITR=ITR+1
      II=II+II0
      JJ=JJ+II0
      ILOW(ITR)=II
      IUP(ITR)=JJ
      INDEXP(ITR)=MODE
      OSC0(ITR)=OSC
      ICOL(ITR)=ICOLIS
      CPAR(ITR)=CPARAM
      LALI(ITR)=.FALSE.
      LEXP(ITR)=.FALSE.
      ISUB(ITR)=0
      IF(ITRA(II,JJ).EQ.0) THEN
         ITRA(II,JJ)=ITR
       ELSE
         ICOL(ITR)=99
      END IF
      IE=IEL(II)
      N0I=NFIRST(IE)
      NKI=NNEXT(IE)
      LINE(ITR)=.TRUE.
      IFR0(ITR)=IFRQ0
      IFR1(ITR)=IFRQ1
      FR0(ITR)=(ENION(II)-ENION(JJ))/H
      IF(IFFR0.EQ.1) FR0(ITR)=FR0INP
      IF(OSC.EQ.0.) THEN
         IF(MODE.NE.3.AND.MODE.NE.4) THEN
            GH=2.D0*NQUANT(II)*NQUANT(II)
            OSC0(ITR)=OSH(NQUANT(II),NQUANT(JJ))*G(II)/GH
         ENDIF
         IF(ifwop(jj).lt.0) THEN
            OSC0(ITR)=0.
            JJ0=NQUANT(JJ-1)
            J20=MIN(20,NLMX)
            IF(J20.GE.JJ0) THEN
               DO JTR=JJ0,J20
                  OSC0(ITR)=OSC0(ITR)+OSH(NQUANT(II),JTR)
               END DO
            END IF
            IF(NLMX.GT.20) THEN
               XII=NQUANT(II)*NQUANT(II)
               SUF=0.
               DO JTR=21,NLMX
                  XJ=JTR
                  XJJ=XJ*XJ
                  XJTR=XJ/(XJJ-XII)
                  SUF=SUF+XJTR*XJTR*XJTR
               END DO
               XITR=(400.-XII)/20.
               OSC0(ITR)=OSC0(ITR)+OSH(NQUANT(II),20)*SUF*XITR*XITR*XITR
            END IF
         END IF
      END IF
C
      IF(FR0(ITR).GT.0.) THEN
         ALAM=2.997925D18/FR0(ITR)
       ELSE
         ALAM=0.
      END IF
      IF(MODE.EQ.0) GO TO 20
      IF(IABS(MODE).EQ.2) GO TO 25
C
C     change the status of treatment of superlines, 
C     if specified in the "offset-setting" record, or
C     because of too loo or too high frequency
C
      IF(IABS(MODE).EQ.3.OR.IABS(MODE).EQ.4) THEN
         IF(II.LT.ILLIM.OR.FR0(ITR).LE.FRLMIN.OR.
     *      FR0(ITR).GE.FRLMAX) INDEXP(ITR)=0
         GO TO 20
      END IF
C
C  -----------------------------------------------------------
C  Additional input parameters for "clasical" line transitions
C   (i.e. those not represented by ODF's - ie ABS(MODE)=1)
C  -----------------------------------------------------------
C
C     LCOMP   -  mode of considering absorption profile:
C             =  false - depth-independent profile
C             =  true  - depth-dependent profile
C     INTMOD  -  mode of evaluating frequency points in the line
C             =  0  - means that frequency points and weights have
C                     already been read amongst the NJREAD or NFREAD
C                     frequencies;
C             ne 0  - frequency points and weights will be evaluated:
C             the meaning of the individual values:
C             =  1  - equidistant frequencies, trapezoidal integration
C             =  2  - equidistant frequencies, Simpson integration
C             =  3  - modified Simpson integration
C                     3-point Simpson integrations with each subsequent
C                     integration interval doubled, until the whole
C                     integration area is covered
C             =  4  - frequencies (in units of standard x) and weights
C                     (for integration over x) are read;
C
C      NF     -  number of frequency points in the line
C                (has the meaning only for INTMOD ne 0)
C
C      XMAX   =  0  - program sets up default XMAX=4.55
C             >  0  - means that the line is assumed symmetric around the
C                     center, frequency points are set up between x=0 and
C                     x=XMAX, where x is frequency difference from the
C                     line center in units of the standard Doppler width
C                     (standard Doppler width is the Doppler width
C                     calculated with standard temperature TSTD and
C                     standard turbulent velocity VTB);
C             <  0  - frequency points are set between x=XMAX and x=-XMAX
C
C  IMPORTANT NOTE: in the overlapping mode (IOVER > 0), all lines are
C  set by default to the full-profile mode. Thefore, if XMAX was
C  positive, it is reset to XMAX --> -XMAX, and NF --> 2*NF - 1
C
C     TSTD    >  0  - standard temperature (see above);
C             =  0  - the program assigns for the standard temperature the
C                     default value TSTD = 0.75*TEFF
C
      READ(IUNIT,*) LCOMP(ITR),INTMOD(ITR),NF,XMAX,TSTD
      IPROF(ITR)=IFANCY
      IF(IABS(IPROF(ITR)).EQ.1) THEN
	 IP=IP+1
         if(ip.gt.mvoigt) call stop('too many voigt profiles')
         ITRA(JJ,II)=IP
      END IF
C
C     if Voigt profile is assumed (ie. if IPROF = 1), an additional
C     input record is required which specifies an evaluation of the
C     relevant damping parameter - see procedure DOPGAM)
C
C      GAMR             - for > 0  - has the meaning of natural damping
C                                    parameter (=Einstein coefficient for
C                                    spontaneous emission)
C                             = 0  - classical natural damping assumed
C                             < 0  - damping is given by a non-standard,
C                                    user supplied procedure GAMSP
C      STARK1           -     = 0  - Stark broadening neglected
C                             < 0  - scaled classical expression
C                                    (ie gam = -STARK1 * classical Stark)
C                             > 0  - Stark broadening given by
C                                    n(el)*[STARK1*T**STARK2 + STARK3]
C      STARK2, STARK3   -   see above
C      VDWH             -   .le.0  - Van der Waals broadening neglected
C                             > 0  - scaled classical expression
C
      IF(IABS(IPROF(ITR)).EQ.1) THEN
         READ(IUNIT,*) GAMR(IP),STARK1(IP),STARK2(IP),STARK3(IP),
     *                 VDWH(IP) 
      END IF
C
C     change the status of treatment of lines, in virtue of:
C     a) too low frequency (FRLMIN); or
C     b) if specified in the "offset-setting" record
C
      IF(II.LT.ILLIM.OR.FR0(ITR).LE.FRLMIN.OR.FR0(ITR).GE.FRLMAX) THEN
         INDEXP(ITR)=0
         GO TO 20
      END IF
C      
      IF(TSTD.EQ.0.) TSTD=0.75*TEFF
      IF(XMAX.EQ.0.) XMAX=4.55
      if(iover.gt.0.and.xmax.gt.0.) then
         xmax=-xmax
         nf=2*nf-1
      end if
      CALL DOPGAM(ITR,1,TSTD,DOP,AGAM)
      INTM=INTMOD(ITR)
      IF(INTM.NE.0) IFR0(ITR)=NLASTE+1
      IF(INTM.NE.0) IFR1(ITR)=NLASTE+NF
      IF(INTM.NE.0) NLASTE=IFR1(ITR)
      CALL LINSET(ITR,IUNIT,IFRQ0,IFRQ1,XMAX,DOP,AGAM)
      GO TO 20
C
C  -----------------------------------------------------------
C  Additional input parameters for a "merged superline" transition
C   (i.e. transition to a merged level, treated by means of an ODF
C     - i.e. for ABS(MODE)=2)
C  -----------------------------------------------------------
C
C     KDO(1),KDO(2),KDO(3),XDO(1),XDO(2),XDO(3),KDO(4) - 
C            have the following meaning:
C            The superline is represented by four frequency intervals.
C            Going away from the peak of ODF, the first interval is 
c            represented by a KDO(1)-point Simpson integration, with a 
C            distance XDO(1) fiducial Doppler widths between the points.
C            The same for the second and third interval.
C            The rest (the interval between the last point and the 
C            coresponding edge) is represented by a KDO(4)-point
C            Simpson integration.
C            The fiducial Doppler width is that corresponding to the
C            effective temperature and the standard turbulent velocity.
C
   25 NHOD=NHOD+1
      if(ifwop(jj).ge.0) call stop('Incorrect ifwop value')
      READ(IUNIT,*) (KDO(IHI,NHOD),IHI=1,3),(XDO(IHI,NHOD),IHI=1,3),
     *           KDO(4,NHOD) 
C
C     again, change status of treatment of lines if required
C     
      IF(II.LT.ILLIM.OR.FR0(ITR).LE.FRLMIN.OR.FR0(ITR).GE.FRLMAX) THEN
         INDEXP(ITR)=0
         GO TO 20
      END IF
      JNDODF(ITR)=NHOD
      IF(II.LT.NLAST(ION)) GO TO 20
   30 CONTINUE
C
      RETURN
      END
c
c
C    *****************************************************************
c
C
      SUBROUTINE NSTPAR(FINSTD)
C     ==========================
C
C     settiing up the default values of various input flags, and
C     input of non-standard values of various input flags and parameters
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
C 
      PARAMETER(MVAR=92)
      PARAMETER(INPFI=4)
      CHARACTER*(*) FINSTD
      CHARACTER*6 PVALUE(MVAR)
      CHARACTER*80 TEXT
      CHARACTER VARNAM(MVAR)*6
      CHARACTER*20 BLNK
      CHARACTER*6 BLNK6
      PARAMETER (MPAG=MLEVEL/6+1)
      CHARACTER*10 TYPLEV(MLEVEL)
      CHARACTER*4 TYPION(MION)
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
      COMMON/PRINTP/NPGPOP,IIPR(6,MPAG),TYPLEV,TYPION
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      COMMON/DFEALI/DJMAX,NTRALI
      COMMON/HEQAUX/PRD0,IHECOR
      COMMON/LTEGRP/TAUFIR,TAULAS,ABROS0,TSURF,ALBAVE,DION0,
     *              NDGREY,IDGREY
C
      DATA VARNAM /'ISPLIN','IRTE  ','IBC   ','ILMCOR','ILPSCT',
     *             'ILASCT','DJMAX ','NTRALI',
     *             'IFALI ','IFPOPR','JALI  ','IFRALI',
     *             'IFPREC','IELCOR','ICHC  ','IRSPLT','IATREF',
     *             'MODREF','IACPP ','IACPD ','IFLEV ','IDLTE ',
     *             'POPZER',
     *             'IOVER ','ITLAS ','IFSUB ','NITER ','NLAMBD',
     *             'ND    ',
     *             'NELSC ','IHECOR','IBFINT','IRDER ',
     *             'CHMAX ','ILDER ','IBPOPE','CHMAXT','NLAMT ',
     *             'INTRPL','ICHANG',
     *             'INHE  ','INRE  ','INPC  ','INSE  ','INMP  ',
     *             'INDL  ','NDRE  ','TAUDIV','IDLST ',
     *             'ICONV ','IPRESS','ITEMP ','IPRINT',
     *             'IOPADD','IRSCT ','IOPHMI','IOPH2P','IOPHE1',
     *             'IOPHE2',
     *             'IACC  ','IACD  ','ITEK  ','ORELAX',
     *             'IWINBL','NPGPOP',
     *             'ICRSW ','SWPFAC','SWPLIM','SWPINC',
     *             'TAUFIR','TAULAS','ABROS0','TSURF ','ALBAVE',
     *             'DION0 ','NDGREY','IDGREY','NCONIT','IPRING',
     *             'IHM   ','IH2   ','IH2P  ',
     *             'TRAD  ','WDIL  ',
     *             'HMIX0 ','VTB   ',
     *             'FRCMAX','FRCMIN','FRLMAX','FRLMIN',
     *             'DFTAIL','NFTAIL'/
C
      DATA PVALUE /'     0','     3','     3','     1','     0',
     *             '     0',' 1.D-3','     3',
     *             '     5','     4','     1','     0',
     *             '     1','   100','     0','     1','     1',
     *             '     1','     7','     4','     0','  1000',
     *             '1.D-20',
     *             '     1','   100','     0','    30','     2',
     *             '    70',
     *             '     0','     0','     1','     3',
     *             ' 1.D-3','     0','     1','     0','     1',
     *             '     0','     0',
     *             '     1','     2','     3','     4','     0',
     *             '     0','     0','   0.5','     5',
     *             '     0','     0','     0','     0',
     *             '     0','     0','     0','     0','     0',
     *             '     0',
     *             '     7','     4','     5','  1.D0',
     *             '     0','     0',
     *             '     0',' 1.D-1',' 1.D-3','  3.D0',
     *             ' 1.D-7',' 316.0','   0.4','    0.','    0.',
     *             '    1.','     0','     0','     0','     1',
     *             '     0','     0','     0',
     *             '    0.','    0.',
     *             '   -1.','    0.',
     *             '    0.',' 1.D12','    0.',' 1.D13',
     *             '  0.25','    21'/
C
      DATA BLNK/'                    '/,BLNK6/'      '/
C
      IF(FINSTD.NE.BLNK) 
     *   OPEN(UNIT=INPFI,NAME=FINSTD,STATUS='UNKNOWN')
      ISPLIN= 0
      IRTE  = 3
      IBC   = 3
      ILMCOR= 1
      ILPSCT= 0
      ILASCT= 0
      DJMAX = 1.E-3
      NTRALI= 3
C
      IFALI = 5
      IFPOPR= 4
      JALI  = 1
      IFRALI= 0
C
      IFPREC= 1
      IELCOR= 100
      ICHC  = 0
      IRSPLT= 1
      IATREF= 1
      MODREF= 1
      IACPP = 7
      IACDP = 4
      IFLEV = 0
      IDLTE = 1000
      POPZER= 1.D-20
C
      IOVER = 1
      ITLAS = 100
      IFSUB = 0
      NITER = 30
      NLAMBD= 2
      ND    = 70
C
      NELSC = 0
      IHECOR= 0
      IBFINT= 1
      IRDER = 3
C
      CHMAX = 1.D-3
      ILDER = 0
      IBPOPE= 1
      CHMAXT= 0
      NLAMT = 1
C
      INTRPL= 0
      ICHANG= 0
C
      INHE  = 1
      INRE  = 2
      INPC  = 3
      INSE  = 4
      INMP  = 0
      INDL  = 0
C
      NDRE  = 0
      TAUDIV= 0.5
      IDLST = 5
C
      ICONV = 0
      IPRESS= 0
      ITEMP = 0
      IPRINT= 0
C
      IOPADD= 0
      IRSCT = 0
      IOPHMI= 0
      IOPH2P= 0
      IOPHE1= 0
      IOPHE2= 0
C
      IACC  = 7
      IACD  = 4
      ITEK  = 5
      ORELAX= UN
C
      IWINBL= 0
      NPGPOP= 0
C
      ICRSW = 0
      SWPFAC= 1.D-1
      SWPLIM= 1.D-3
      SWPINC= 3.D0
      DO ID=1,MDEPTH
         CRSW(ID)=UN
      END DO
C
      TAUFIR= 1.D-7
      TAULAS= 3.16D2
      ABROS0= 0.4
      TSURF = 0.
      ALBAVE= 0.
      DION0 = 1.
      NDGREY= 0
      IDGREY= 0
      NCONIT= 0
      IHM   = 0
      IH2   = 0
      IH2P  = 0
C
      TRAD  = 0.
      WDIL  = 0.
C
      HMIX0 =-1.
      VTB   = 0.
C
      FRCMAX= 0.
      FRCMIN= 1.D12
      FRLMAX= 0.
      FRLMIN= 1.D13
      DFTAIL= 0.25
      NFTAIL= 21
C
      INDV=-1
C
C    go through the input file line by line
c
   10 CONTINUE
      K0=1
      READ(INPFI,500,END=70,ERR=70) TEXT
  500 FORMAT(A)
   20 CONTINUE
      CALL GETWRD(TEXT,K0,K1,K2)
      IF(K1.EQ.0) GO TO 60
      K0=K2+2
      IF(TEXT(K1:K2).EQ.'=') GO TO 20
      INDV=-INDV
      IF(INDV.EQ.1) THEN
         DO 40 I=1,MVAR
            IF(TEXT(K1:K2).EQ.VARNAM(I)(1:K2-K1+1)) GO TO 50
   40    CONTINUE
         CALL GETWRD(TEXT,K0,K1,K2)
         IF(K1.EQ.0) THEN
            K0=1
   45       READ(INPFI,500,END=70) TEXT
            CALL GETWRD(TEXT,K0,K1,K2)
            IF(K1.EQ.0) GO TO 45
         END IF
         K0=K2+2
         INDV=-INDV
         GO TO 20
   50    CONTINUE
         IVAR=I
       ELSE
         PVALUE(IVAR)=BLNK6
         PVALUE(IVAR)(6-K2+K1:6)=TEXT(K1:K2)
      END IF
      GO TO 20
   60 CONTINUE
      GO TO 10
   70 CONTINUE
C
      DO I=1,MVAR
         WRITE(84,684) PVALUE(I)
  684    FORMAT(1X,A)
      END DO
C
      CLOSE(UNIT=84)
      REWIND(84)
      READ(84,*) 
     *             ISPLIN,IRTE  ,IBC   ,ILMCOR,ILPSCT,
     *             ILASCT,DJMAX ,NTRALI,
     *             IFALI ,IFPOPR,JALI  ,IFRALI,
     *             IFPREC,IELCOR,ICHC  ,IRSPLT,IATREF,
     *             MODREF,IACPP ,IACPD ,IFLEV ,IDLTE,
     *             POPZER,
     *             IOVER ,ITLAS ,IFSUB ,NITER ,NLAMBD,ND,
     *             NELSC ,IHECOR,IBFINT,IRDER ,
     *             CHMAX ,ILDER ,IBPOPE,CHMAXT,NLAMT ,
     *             INTRPL,ICHANG,
     *             INHE  ,INRE  ,INPC  ,INSE  ,INMP  ,
     *             INDL  ,NDRE  ,TAUDIV,IDLST ,
     *             ICONV ,IPRESS,ITEMP ,IPRINT,
     *             IOPADD,IRSCT ,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *             IACC  ,IACD  ,ITEK  ,ORELAX,
     *             IWINBL,NPGPOP,
     *             ICRSW ,SWPFAC,SWPLIM,SWPINC,
     *             TAUFIR,TAULAS,ABROS0,TSURF ,ALBAVE,
     *             DION0 ,NDGREY,IDGREY,NCONIT,IPRING,
     *             IHM   ,IH2   ,IH2P  ,
     *             TRAD  ,WDIL  ,
     *             HMIX0 ,VTB   ,
     *             FRCMAX,FRCMIN,FRLMAX,FRLMIN,
     *             DFTAIL,NFTAIL
C      
      LCHC=.FALSE.
      IF(ICHC.EQ.1) LCHC=.TRUE.
      NFFIX=IFRALI
      IF(FRCMAX.EQ.0.) FRCMAX=6.25D11*TEFF
      IF(FRLMAX.EQ.0.) FRLMAX=FRCMAX
      IF(LTE.AND.NLAMBD.NE.0) NLAMBD=1
      RETURN
      END
C
C
C    ***************************************************************
C
C
      SUBROUTINE NSTOUT
C     =================
C
C     Diagnostic print of the input flags and parameters
c
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (MPAG=MLEVEL/6+1)
      CHARACTER*10 TYPLEV(MLEVEL)
      CHARACTER*4 TYPION(MION)
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
      COMMON/PRINTP/NPGPOP,IIPR(6,MPAG),TYPLEV,TYPION
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      COMMON/DFEALI/DJMAX,NTRALI
      COMMON/HEQAUX/PRD0,IHECOR
      COMMON/LTEGRP/TAUFIR,TAULAS,ABROS0,TSURF,ALBAVE,DION0,
     *              NDGREY,IDGREY
C
      write(6,602)
     *             ISPLIN,IRTE  ,IBC   ,ILMCOR,ILPSCT,
     *             ILASCT,DJMAX ,NTRALI,
     *             IFALI ,IFPOPR,JALI  ,IFRALI,
     *             IFPREC,IELCOR,ICHC  ,IRSPLT,IATREF,
     *             MODREF,IACPP ,IACPD ,IFLEV, IDLTE,
     *             POPZER,
     *             IOVER ,ITLAS ,IFSUB ,NITER ,NLAMBD,ND,
     *             NELSC ,IHECOR,IBFINT,IRDER ,
     *             CHMAX ,ILDER ,IBPOPE,CHMAXT,NLAMT
      write(6,603) INTRPL,ICHANG,
     *             INHE  ,INRE  ,INPC  ,INSE  ,INMP  ,
     *             INDL  ,NDRE  ,TAUDIV,IDLST ,
     *             ICONV ,IPRESS,ITEMP ,IPRINT,
     *             IOPADD,IRSCT ,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *             IACC  ,IACD  ,ITEK  ,ORELAX,
     *             IWINBL,NPGPOP,
     *             ICRSW ,SWPFAC,SWPLIM,SWPINC
      write(6,604) TRAD  ,WDIL  ,
     *             HMIX0 ,VTB   ,
     *             FRCMAX,FRCMIN,FRLMAX,FRLMIN,
     *             DFTAIL,NFTAIL
      IF(LTGREY)   WRITE(6,605)
     *             TAUFIR,TAULAS,ABROS0,TSURF ,ALBAVE,
     *             DION0 ,NDGREY,IDGREY,NCONIT,IPRING,
     *             IHM   ,IH2   ,IH2P
c
  602 FORMAT('ISPLIN=',I5,2X,'IRTE  =',I5,2X,'IBC   =',I5,2X,
     *       'ILMCOR=',I5,2X,'ILPSCT=',I5,2X/
     *       'ILASCT=',I5,2X,'DJMAX =',F5.3,2X,'NTRALI=',I5,2X/
     *       'IFALI =',I5,2X,'IFPOPR=',I5,2X,'JALI  =',I5,2X,
     *       'IFRALI=',I5,2X/
     *       'IFPREC=',I5,2X,'IELCOR=',I5,2X,'ICHC  =',I5,2X,
     *       'IRSPLT=',I5,2X,'IATREF=',I5,2X/
     *       'MODREF=',I5,2X,'IACPP =',I5,2X,'IACPD =',I5,2X,
     *       'IFLEV =',I5,2X,'IDLTE =',I5,2X/
     *       'POPZER=',1PE6.0,1X/
     *       'IOVER =',I5,2X,'ITLAS =',I5,2X,'IFSUB =',I5,2X/
     *       'NITER =',I5,2X,'NLAMBD=',I5,2X,'ND    =',I5/
     *       'NELSC =',I5,2X,'IHECOR=',I5,2X,'IBFINT=',I5,2X,
     *       'IRDER =',I5,2X,'CHMAX =',F5.3,2X/
     *       'ILDER =',I5,2X,'IBPOPE=',I5,2X,
     *       'CHMAXT=',F5.3,2X,'NLAMT =',I5,2X)
  603 FORMAT('INTRPL=',I5,2X,'ICHANG=',I5,2X/
     *       'INHE  =',I5,2X,'INRE  =',I5,2X,'INPC  =',I5,2X,
     *       'INSE  =',I5,2X,'INMP  =',I5,2X/
     *       'INDL  =',I5,2X,'NDRE  =',I5,2X,'TAUDIV=',F5.3,2X,
     *       'IDLST =',I5,2X/
     *       'ICONV =',I5,2X,'IPRESS=',I5,2X,'ITEMP =',I5,2X,
     *       'IPRINT=',I5,2X/
     *       'IOPADD=',I5,2X,'IRSCT =',I5,2X,'IOPHMI=',I5,2X,
     *       'IOPH2P=',I5,2X,'IOPHE1=',I5,2X/'IOPHE2=',I5,2X/
     *       'IACC  =',I5,2X,'IACD  =',I5,2X,'ITEK  =',I5,2X,
     *       'ORELAX=',F5.3,2X/
     *       'IWINBL=',I5,2X,'NPGPOP=',I5,2X/
     *       'ICRSW =',I5,2X,'SWPFAC=',F5.3,2X,'SWPLIM=',F5.3,2X,
     *       'SWPINC=',F5.3,2X/)
  604 FORMAT('TRAD  =',F8.0,2X,'WDIL  =',F5.3/
     *       'HMIX0 =',F5.1,2X,'VTB   =',F8.0/
     *       'FRCMAX=',1PE6.0,1X,'FRCMIN=',1PE6.0,1X,
     *       'FRLMAX=',1PE6.0,1X,'FRLMIN=',1PE6.0,1X/
     *       'DFTAIL=',0PF5.3,2X,'NFTAIL=',I5/)
  605 FORMAT('TAUFIR=',1PE6.0,1X,'TAULAS=',0PF5.1,2X,'ABROS0=',F5.3,2X,
     *       'TSURF =',F5.3,2X,'ALBAVE=',2X,F5.3/
     *       'DION0 =',F5.3,2X,'NDGREY=',I5,2X,'IDGREY=',I5,2X,
     *       'NCONIT=',I5,2X,'IPRING=',I5,2X/
     *       'IHM   =',I5,2X,'IH2   =',I5,2X,'IH2P  =',I5,2X/)
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE GETWRD(TEXT,K0,K1,K2)
C
C  FINDS NEXT WORD IN TEXT FROM INDEX K0. NEXT WORD IS TEXT(K1:K2)
C  THE NEXT WORD STARTS AT THE FIRST ALPHANUMERIC CHARACTER AT K0
C  OR AFTER. IT ENDS WITH THE LAST ALPHANUMERIC CHARACTER IN A ROW
C  FROM THE START
C
C  TAKEN FROM MULTI - M. CARLSSON (1976)
C
C     INCLUDE 'IMPLIC.FOR'
      PARAMETER (MSEPAR=7)
      CHARACTER*(*) TEXT
      CHARACTER SEPAR(MSEPAR)
      DATA SEPAR/' ','(',')','=','*','/',','/
C
      K1=0
      DO 400 I=K0,LEN(TEXT)
        IF(K1.EQ.0) THEN
          DO 100 J=1,MSEPAR
            IF(TEXT(I:I).EQ.SEPAR(J)) GOTO 200
  100     CONTINUE
          K1=I
C
C  NOT START OF WORD
C
  200     CONTINUE
        ELSE
          DO 300 J=1,MSEPAR
            IF(TEXT(I:I).EQ.SEPAR(J)) GOTO 500
  300     CONTINUE
        ENDIF
  400 CONTINUE
C
C  NO NEW WORD. RETURN K1=K2=0
C
      K1=0
      K2=0
      GOTO 999
C
C  NEW WORD IN TEXT(K1:I-1)
C
  500 CONTINUE
      K2=I-1
C
  999 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE STATE(MODE,ID,T,ANE)
C     ===============================
C
C     For MODE=0 - initialization of the basic parameters for the
C                  chemical species
C     for MODE=2 - solves the set of LTE Saha equations to determine
C                  the total charge due to ionization of non-explicit
C                  chemical species
C     for MODE=3 - similar as MODE=2, but also with derivatives wrt
C                  temperature and electron density (called from BPOP,
C                  ie. from the linearization step)
C     for MODE=1 - similar as MODE=2, but the total charge is evaluated
C                  summing the contributions of both non-explicit and
C                  explicit chemical species (called from LTEGR)
C
C     Input for MODE > 0:
C
C        T   - temperature
C        ANE - electron density
C
C     Output for MODE > 0 (through COMMON/STATEP
C
C        Q    - total charge, relative to the reference atom
C        QM   - charge of H- (in LTE), evaluated only if H- is not
C               explicit ion, and if desired (if IHM=1)
C        DQT  - derivative of Q wrt temperature
C        DQN  - derivative of Q wrt electron density
C        DQM  - derivative of QM wrt temperature
C        ENER - internal energy (of all species in all considered
C               ionization stages (needed only for evaluating
C               thermodynamic derivatoves if convection is considerd)
C        QREF - total charge due to the reference species
C        DQTR - derivative of QREF wrt temperature
C        DQNR - derivative of QREF wrt electron density
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      DIMENSION AMAS(30),TYPAT(30),ABND(30),D(4,30),E(4,10),
     *          abnref(mdepth),O(4,20)
      dimension xio(9,30),eio(9,10),oio(9,20)
      dimension ffi(30),pfstu(30),pfstt(30),pfstn(30),entot(30)
      dimension idat(30),uu(30,17)
      dimension u6(6),u7(7),u8(8),u10(10),u11(11)
      dimension u12(12),u13(13),u14(14),u16(16),u18(18),u20(20)
      dimension u24(24),u25(25),u26(26),u28(28)
      COMMON/PFSTDS/PFSTD(30,30),MODPF(30)
      COMMON/RRVALS/RR(30,30),ABNDD(30,MDEPTH),ENEV(30,30),IONIZ(30),
     *              IREF,LGR(30),LRM(30)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      PARAMETER (TH0=5.0404D3,XMX0=2.154D4,THL0=2.3025851,FI0=3.6113D1,
     *           TRHA=1.5D0,c1qm=1.0353d-16,c2qm=8762.9)
C
C    Standard atomic constants for first 30 species:
C
C            Element Atomic  Solar    Std.
C                    weight abundance highest 
C                                     ionization stage 
      DATA O/'  H ', 1.008, 1.00E 0, 2.,
     *       '  HE', 4.003, 1.00D-1, 3.,
     *       '  LI', 6.941, 3.9D-12, 3.,
     *       '  BE', 9.012, 1.1D-11, 3.,
     *       '  B ',10.810, 6.2D-10, 4.,
     *       '  C ',12.011, 3.68D-4, 5.,
     *       '  N ',14.007, 1.14D-4, 5.,
     *       '  O ',16.000, 6.70D-4, 5.,
     *       '  F ',18.918, 3.63D-8, 4.,
     *       '  NE',20.179, 2.79D-5, 4.,
     *       '  NA',22.990, 1.72D-6, 4.,
     *       ' MG ',24.305, 3.43D-5, 4.,
     *       ' AL ',26.982, 2.49D-6, 4.,
     *       ' SI ',28.086, 3.51D-5, 5.,
     *       ' P  ',30.974, 2.67D-7, 5.,
     *       ' S  ',32.060, 1.61D-5, 5.,
     *       ' CL ',35.453, 4.42D-7, 5.,
     *       ' AR ',39.948, 4.42D-6, 5.,
     *       ' K  ',39.098, 1.11D-7, 5.,
     *       ' CA ',40.080, 2.12D-6, 5./
      DATA E/' SC ',44.956, 1.16D-9, 5.,
     *       ' TI ',47.900, 5.44D-8, 5.,
     *       ' V  ',50.941, 1.25D-8, 5.,
     *       ' CR ',51.996, 4.96D-7, 5.,
     *       ' MN ',54.938, 1.57D-7, 5.,
     *       ' FE ',55.847, 2.49D-5, 5.,
     *       ' CO ',58.933, 3.13D-8, 5.,
     *       ' NI ',58.700, 1.89D-6, 5.,
     *       ' CU ',63.546, 2.79D-8, 5.,
     *       ' ZN ',65.380, 2.60D-8, 5./
C
C     Ionization potentials for first 30 species:
C
C     Element Ionization potentials (eV) 
C              I     II      III     IV       V     VI     VII    VIII
C
      data oio/'  H ',13.595, 0. , 0. , 0. ,  0.  ,  0.  ,  0.  ,  0.  ,
     *'  HE',24.580, 54.400,  0.   ,  0.   ,  0.  ,  0.  ,  0.  ,  0.  ,
     *'  LI', 5.392, 75.619,122.451,  0.   ,  0.  ,  0.  ,  0.  ,  0.  ,
     *'  BE', 9.322, 18.206,153.850,217.713,  0.  ,  0.  ,  0.  ,  0.  ,
     *'  B ', 8.296, 25.149, 37.920,259.298,340.22,  0.  ,  0.  ,  0.  ,
     *'  C ',11.264, 24.376, 47.864, 64.476,391.99,489.98,  0.  ,  0.  ,
     *'  N ',14.530, 29.593, 47.426, 77.450, 97.86,551.93,667.03,  0.  ,
     *'  O ',13.614, 35.108, 54.886, 77.394,113.87,138.08,739.11,871.39,
     *'  F ',17.418, 34.980, 62.646, 87.140,114.21,157.12,185.14,953.6 ,
     *'  NE',21.559, 41.070, 63.500, 97.020,126.30,157.91,207.21,239.0 ,
     *'  NA', 5.138, 47.290, 71.650, 98.880,138.37,172.09,208.44,264.16,
     *'  MG', 7.664, 15.030, 80.120,102.290,141.23,186.49,224.9 ,265.96, 
     *' AL ', 5.984, 18.823, 28.440,119.960,153.77,190.42,241.38,284.53, 
     *' SI ', 8.151, 16.350, 33.460, 45.140,166.73,205.11,246.41,303.07, 
     *' P  ',10.484, 19.720, 30.156, 51.354, 65.01,220.41,263.31,309.26,
     *' S  ',10.357, 23.400, 35.000, 47.290, 72.50, 88.03,280.99,328.8 ,
     *' CL ',12.970, 23.800, 39.900, 53.500, 67.80, 96.7 ,114.27,348.3 ,
     *' AR ',15.755, 27.620, 40.900, 59.790, 75.00, 91.3 ,124.0 ,143.46,
     *' K  ', 4.339, 31.810, 46.000, 60.900, 82.6 , 99.7 ,118.0 ,155.0 ,
     *' CA ', 6.111, 11.870, 51.210, 67.700, 84.39,109.0 ,128.0 ,147.0 /
      data eio/' SC ',6.560,12.890,24.750,73.900,92.0,111.1,138.0,158.7,
     *' TI ', 6.830, 13.630, 28.140, 43.240, 99.8 ,120.0 ,140.8 ,168.5 ,
     *' V  ', 6.740, 14.200, 29.700, 48.000, 65.2 ,128.9 ,151.0 ,173.7 ,
     *' CR ', 6.763, 16.490, 30.950, 49.600, 73.0 , 90.6 ,161.1 ,184.7 ,
     *' MN ', 7.432, 15.640, 33.690, 53.000, 76.0 , 97.0 ,119.24,196.46,
     *' FE ', 7.870, 16.183, 30.652, 54.800, 75.0 , 99.1 ,125.0 ,151.06,
     *' CO ', 7.860, 17.060, 33.490, 51.300, 79.5 ,102.0 ,129.0 ,157.0 ,
     *' NI ', 7.635, 18.168, 35.170, 54.900, 75.5 ,108.0 ,133.0 ,162.0 ,
     *' CU ', 7.726, 20.292, 36.830, 55.200, 79.9 ,103.0 ,139.0 ,166.0 ,
     *' ZN ', 9.394, 17.964, 39.722, 59.400, 82.6 ,108.0 ,134.0 ,174.0 /
C
C     data for additional ionization potentials for the Opacity
C     project species (IDAT sets the internal OP indexing)
C
      data idat   / 1, 2, 0, 0, 0, 3, 4, 5, 0, 6,
     *              7, 8, 9,10, 0,11, 0,12, 0,13,
     *              0, 0, 0,14,15,16, 0,17,0,0/ 
      data u10/173.93,330.391,511.8,783.3,1018.,1273.8,1671.792,
     *         1928.462,9645.005,10986.876/
      data u11/41.449,381.395,577.8,797.8,1116.2,1388.5,1681.5,2130.8,
     *         2418.7,11817.061,13297.676/
      data u12/61.671,121.268,646.41,881.1,1139.4,1504.3,1814.3,2144.7,
     *         2645.2,2964.4,14210.261,15829.951/
      data u13/48.278,151.86,229.446,967.8,1239.8,1536.3,1947.3,2295.4,
     *         2663.4,3214.8,3565.6,16825.022,18584.138/
      data u14/65.748,131.838,270.139,364.093,1345.1,1653.9,1988.4,
     *         2445.3,2831.9,3237.8,3839.8,4222.4,19661.693,21560.63/
      data u16/83.558,188.2,280.9,381.541,586.2,710.184,2265.9,2647.4,
     *         3057.7,3606.1,4071.4,4554.3,5255.9,5703.6,26002.663,
     *         28182.535/
      data u18/127.11,222.848,328.6,482.4,605.1,734.04,1002.73,1157.08,
     *         3407.3,3860.9,4347.,4986.6,5533.8,6095.5,6894.2,7404.4,
     *         33237.173,35699.936/
      data u20/49.306,95.752,410.642,542.6,681.6,877.4,1026.,1187.6,
     *         1520.64,1704.047,4774.,5301.,5861.,6595.,7215.,7860.,
     *         8770.,9338.,41366.,44177.41/
      data u24/54.576,132.966,249.7,396.5,560.2,731.02,1291.9,1490.,
     *         1688.,1971.,2184.,2404.,2862.,3098.52,8151.,8850.,
     *         9560.,10480.,11260.,12070.,13180.,13882.,60344.,63675.9/
      data u25/59.959,126.145,271.55,413.,584.,771.1,961.44,1569.,
     *         1789.,2003.,2307.,2536.,2771.,3250.,3509.82,9152.,
     *         9872.,10620.,11590.,12410.,13260.,14420.,15162.,
     *         65660.,69137.4/
      data u26/63.737,130.563,247.22,442.,605.,799.,1008.,1218.38,
     *         1884.,2114.,2341.,2668.,2912.,3163.,3686.,3946.82,
     *         10180.,10985.,11850.,12708.,13620.,14510.,15797.,
     *         16500.,71203.,74829.6/
      data u28/61.6,146.542,283.8,443.,613.5,870.,1070.,1310.,1560.,
     *         1812.,2589.,2840.,3100.,3470.,3740.,4020.,4606.,
     *         4896.2,12430.,13290.,14160.,15280.,16220.,17190.,
     *         18510.,19351.,82984.,86909.4/
C
      EQUIVALENCE (D(1,1),O(1,1))
      EQUIVALENCE (D(1,21),E(1,1))
      EQUIVALENCE (xio(1,1),Oio(1,1))
      EQUIVALENCE (xio(1,21),Eio(1,1))
      equivalence (u6(1),uu(1,3)),(u7(1),uu(1,4)),(u8(1),uu(1,5))
      equivalence (u10(1),uu(1,6)),(u11(1),uu(1,7)),(u12(1),uu(1,8))
      equivalence (u13(1),uu(1,9)),(u14(1),uu(1,10)),(u16(1),uu(1,11))
      equivalence (u18(1),uu(1,12)),(u20(1),uu(1,13)),(u24(1),uu(1,14))
      equivalence (u25(1),uu(1,15)),(u26(1),uu(1,16)),(u28(1),uu(1,17))
C
      IF(MODE.NE.0) GO TO 50
C
C     For MODE=0, STATE serves as an auxiliary procedure for START
C     Input of basic parameters for individual chemical species
C
C     An element (hydrogen through zinc) can be considered in one of
C     the three following options:
C     1. explicitly - some of energy levels of some of its ionization
C                     states are considered explicitly, ie. their
C                     populations are determined by solving statistical
C                     equilibrium
C     2. implicitly - the atom is assumed not to contribute to
C                     opacity; but is allowed to contribute to the
C                     total number of particles and to the total charge;
C                     the latter is evaluated assuming LTE ionization
C                     balance, ie. by solving a set of Saha equations
C     3. not considered at all
C
C     Input:
C     NATOMS -  the highest atomic number of an element that is
C               considered (explicitly or non-explicitly)
C            <  0 - then NATOMS=-NATOMS, and all Opacity Project
C                   species are treated as with MODPF>0, i.e.
C                   their partition functions are evaluated from
C                   the Opacity Project ionization fractions
C
C     For each element from 1 (hydrogen) to NATOMS, the following
C     parameters:
C
C     MA     =  0  - if the element is not considered (option 3)
C            =  1  - if the element is non-explicit (option 2)
C            =  2  - if the element is explicit (option 1)
C     NA0,NAK - have the meaning only for MA=2; indicate that the
C               explicit energy levels of the present species have
C               the indices between NA0 and NAK (NAK is thus the index
C               of the highest ionization state, which is represented
C               as one-level ion).
C     ION    -  has the meaning for MA=1 only;
C               if ION=0, standard number of ionization degrees is
C                         considered
C                         (counting the neutral state also; so for 
C                         instance to treat all stages of He requires
C                         ION=3, which is a default anyhow).
C               if ION>0, then ION ionization degrees is considered
C     MODPF  -  mode of evaluation of partition functions
C            =  0  -  standard evaluation (see procedure PARTF)
C            >  0  -  partition functions evaluated from the
C                     Opacity Project ionization fraction tables
C            <  0  -  non-standard evaluation, by user supplied
C                     procedure PFSPEC
C     ABN    -  if ABN=0, solar abundance is assumed (given above;
C                         abundance here is assumed as relative
C                         to hydrogen by number
C               if ABN>0, non-solar abundance ABN is assumed; in an
C                         arbitrary scale
C               if ABN<0, non-solar abundance ABN is assumed;
C                         (-ABN times the solar value)
C     PFS    -  see above
C
      READ(IBUFF,*) NATOMS
      WRITE(6,600)
      IAT=0
      IREF=0
      IFOPPF=0
      IF(NATOMS.LT.0) THEN
         NATOMS=-NATOMS
         IFOPPF=2
      END IF
C
      DO 20 I=1,NATOMS
         LGR(I)=.TRUE.
         LRM(I)=.TRUE.
         IATEX(I)=-1
         READ(IBUFF,*) MA,NA0,NAK,ION,MODPF(I),ABN
         IF(MA.EQ.0) GO TO 20
         TYPAT(I)=D(1,I)
         AMAS(I)=D(2,I)
         ABND(I)=D(3,I)
         IONIZ(I)=D(4,I)
C
C        increase the standard highest ionization for Teff larger
C        than 50000 K for N, O, Ne, and Fe
C
         IF(TEFF.GT.5.D4) THEN
            IF(I.EQ.7) IONIZ(I)=6
            IF(I.EQ.8) IONIZ(I)=7
            IF(I.EQ.10) IONIZ(I)=9
            IF(I.EQ.26) IONIZ(I)=9
         END IF
C
         DO 10 J=1,8
   10       ENEV(I,J)=xio(1+J,I)
         LGR(I)=.FALSE.
         IF(MODPF(I).GT.0) IFOPPF=1
         IF(IFOPPF.EQ.2.and.idat(i).gt.0) MODPF(I)=1
         if(modpf(i).gt.0) then
            if(idat(i).eq.0) modpf(i)=0
            if(modpf(i).gt.0) then
               ioniz(i)=i+1
               if(i.ge.10) then
                  do j=9,i
                     enev(i,j)=uu(j,idat(i))*0.1239529d0
                  end do
               end if
            end if
         end if
         IF(ABN.GT.0) ABND(I)=ABN
         IF(ABN.LT.0) ABND(I)=ABS(ABN)*D(3,I)
         IF(ION.NE.0) IONIZ(I)=ION
         IF(ABN.GT.1.E6) THEN
            READ(IBUFF,*) (ABNDD(I,ID),ID=1,ND)
          ELSE
            DO ID=1,ND
               ABNDD(I,ID)=ABND(I)
            END DO
         END IF
         IF(MA.EQ.1) THEN
            LRM(I)=.FALSE.
            IATEX(I)=0
          ELSE
            IAT=IAT+1
            IIFIX(IAT)=0
            IF(MA.EQ.-2) IIFIX(IAT)=1
            IATEX(I)=IAT
            IF(IAT.EQ.IATREF) THEN
               IREF=I
               DO ID=1,ND
                  ABNREF(ID)=ABNDD(I,ID)
               END DO
            END IF
C
C           store parameters for explicit atoms
C
            AMASS(IAT)=AMAS(I)*HMASS
            NUMAT(IAT)=I
            N0A(IAT)=NA0
            NKA(IAT)=NAK
         END IF
   20 CONTINUE
C
C     renormalize abundances to have the standard element abundance
C     equal to unity
C
      DO 30 I=1,NATOMS
         IAT=IATEX(I)
         IF(IAT.LT.0) GO TO 30
         DO ID=1,ND
            ABNDD(I,ID)=ABNDD(I,ID)/ABNREF(ID)
            YTOT(ID)=YTOT(ID)+ABNDD(I,ID)
            WMY(ID)=WMY(ID)+ABNDD(I,ID)*AMAS(I)
         END DO
         ABNR=ABND(I)/D(3,I)
         IF(IAT.EQ.0) THEN
            WRITE(6,601) I,TYPAT(I),ABND(I),ABNR
          ELSE
            DO ID=1,ND
               ABUND(IAT,ID)=ABNDD(I,ID)
            END DO 
            WRITE(6,602) I,TYPAT(I),ABND(I),ABNR,IAT,N0A(IAT),NKA(IAT)
         END IF
   30 CONTINUE
      DO ID=1,ND
         WMM(ID)=WMY(ID)*HMASS/YTOT(ID)
      END DO
c
c     initialization of the Opacity Project ionization fractions
c     (if required)
c
      if(ifoppf.gt.0) call opfrac(0,0,t,ane,pf,fra)
c
  600 FORMAT(1H0//' CHEMICAL ELEMENTS INCLUDED'/
     *            ' --------------------------'//
     * ' NUMBER  ELEMENT           ABUNDANCE'/1H ,16X,
     * 'A=N(ELEM)/N(H)  A/A(SOLAR)'/)
  601 FORMAT(1H ,I4,3X,A5,1P2E14.2)
  602 FORMAT(1H ,I4,3X,A5,1P2E14.2,3X,
     *       'EXPLICIT: IAT=',I3,'  N0A=',I3,'  NKA=',I3)
      RETURN
C
   50 TLN=LOG(T)*TRHA
      TK=BOLK*T
      THET=TH0/T
      THL=THL0*THET
      XMX=XMX0*SQRT(SQRT(T/ANE))
      DCH=EH/(XMX*XMX*TK)
      Q=0.
      QM=0.
      DQT=0.
      DQN=0.
      DQM=0.
      ENER=0.
      DO 70 I=1,NATOMS
         IF(MODE.GT.1.AND.LRM(I).OR.MODE.EQ.1.AND.LGR(I)) GO TO 70
         ION=IONIZ(I)
         DRQT=0.
         DRQN=0.
         DRST=0.
         DRSN=0.
         DFT=0.
         DFN=0.
         ENTOT(1)=0.
         RS=UN
         CALL PARTF(I,1,T,ANE,XMX,UM,DUTM,DUNM)
         pfstu(1)=um
         pfstt(1)=dutm
         pfstn(1)=dunm
         JMAX=1
         DO 55 J=2,ION
            J1=J-1
            DCHT=DCH*J1
            TE=ENEV(I,J1)*THL
            ENTOT(J)=ENTOT(J1)+TE
            FI=FI0+TLN-TE+DCHT
            X=J
            XMAX=XMX*SQRT(X)
            CALL PARTF(I,J,T,ANE,XMAX,U,DUT,DUN)
            pfstu(j)=u
            pfstt(j)=dut
            pfstn(j)=dun
            FFI(J)=0.
            IF(FI.GT.-20.) FFI(J)=EXP(FI)*U/UM/ANE
            IF(FFI(J).GT.UN) JMAX=J
            UM=U
   55    CONTINUE
         RQ=JMAX-1
         RI=ENTOT(JMAX)
         RE=PFSTT(JMAX)/PFSTU(JMAX)*T
         if(jmax.lt.ion) then
         R=UN
         DO 60 J=JMAX+1,ION
            J1=J-1
            DCHT=DCH*J1
            TE=ENEV(I,J1)*THL
            R=R*FFI(J)
	    RR(I,J)=R/pfstu(j)
            RS=RS+R
            RQ=RQ+J1*R
            RI=RI+R*ENTOT(J)
            RE=RE+R*PFSTT(J)/PFSTU(J)*T
            DFIT=pfstt(j)/pfstu(j)-pfstt(j1)/pfstu(j1)
     .           +(TRHA+TE-TRHA*DCHT)/T
            DFIN=pfstn(j)/pfstu(j)-pfstn(j1)/pfstu(j1)
     .           +(HALF*DCHT-UN)/ANE
            DFT=DFT+DFIT
            DFN=DFN+DFIN
            DFIT=DFT*R
            DFIN=DFN*R
            DRST=DRST+DFIT
            DRSN=DRSN+DFIN
            DRQT=DRQT+J1*DFIT
            DRQN=DRQN+J1*DFIN
   60    CONTINUE
         endif
         if(jmax.gt.1) then
         R=UN
         DFT=0.
         DFN=0.
         jmin=min(4,jmax-1)
         DO 61 JJ=1,JMIN
            J=JMAX-JJ
            J1=J-1
            JP1=J+1
            DCHT=DCH*J
            TE=ENEV(I,J)*THL
            R=R/FFI(JP1)
	    RR(I,J)=R/pfstu(j)
            RS=RS+R
            RQ=RQ+J1*R
            RI=RI+R*ENTOT(J)
            RE=RE+R*PFSTT(J)/PFSTU(J)*T
            DFIT=pfstt(jp1)/pfstu(jp1)-pfstt(j)/pfstu(j)
     .           +(TRHA+TE-TRHA*DCHT)/T
            DFIN=pfstn(jp1)/pfstu(jp1)-pfstn(j)/pfstu(j)
     .           +(HALF*DCHT-UN)/ANE
            DFT=DFT-DFIT
            DFN=DFN-DFIN
            DFIT=DFT*R
            DFIN=DFN*R
            DRST=DRST+DFIT
            DRSN=DRSN+DFIN
            DRQT=DRQT+J1*DFIT
            DRQN=DRQN+J1*DFIN
   61    CONTINUE
         endif
         X=RQ/RS
         ABND(I)=ABNDD(I,ID)
         X1=ABND(I)/RS
	 RR(I,JMAX)=X1
	 DO 65 J=1,ION
           IF(J.NE.JMAX) RR(I,J)=RR(I,J)*X1
   65    CONTINUE
	 RR(I,JMAX)=RR(I,JMAX)/PFSTU(JMAX)
         if(i.gt.2) ENER=ENER+(RI+RE)*X1
         IF(I.EQ.IREF) THEN
            QREF=X*ABND(I)
            DQTR=(DRQT-X*DRST)*X1
            DQNR=(DRQN-X*DRSN)*X1
          ELSE
            Q=X*ABND(I)+Q
            DQT=DQT+(DRQT-X*DRST)*X1
            DQN=DQN+(DRQN-X*DRSN)*X1
         END IF
   70 CONTINUE
C
C     Negative hydrogen ion
C
      IF(IHM.EQ.1) THEN
         Tinv=un/t
C        QM=C1QM*Tinv/SQRT(T)*EXP(C2QM*Tinv)*ANE  !! incorrect?
         QM=C1QM*Tinv/SQRT(T)*EXP(C2QM*Tinv)
         DQM=-QM*Tinv*(TRHA+C2QM*Tinv)
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE STATE2(MODE,ID,T,ANE)
C     ================================
C
C     For MODE=0 - initialization of the basic parameters for the
C                  chemical species
C     for MODE=2 - solves the set of LTE Saha equations to determine
C                  the total charge due to ionization of non-explicit
C                  chemical species
C     for MODE=3 - similar as MODE=2, but also with derivatives wrt
C                  temperature and electron density (called from BPOP,
C                  ie. from the linearization step)
C     for MODE=1 - similar as MODE=2, but the total charge is evaluated
C                  summing the contributions of both non-explicit and
C                  explicit chemical species (called from LTEGR)
C
C     Input for MODE > 0:
C
C        T   - temperature
C        ANE - electron density
C
C     Output for MODE > 0 (through COMMON/STATEPw

C
C        Q    - total charge, relative to the reference atom
C        QM   - charge of H- (in LTE), evaluated only if H- is not
C               explicit ion, and if desired (if IHM=1)
C        DQT  - derivative of Q wrt temperature
C        DQN  - derivative of Q wrt electron density
C        DQM  - derivative of QM wrt temperature
C        ENER - internal energy (of all species in all considered
C               ionization stages (needed only for evaluating
C               thermodynamic derivatoves if convection is considerd)
C        QREF - total charge due to the reference species
C        DQTR - derivative of QREF wrt temperature
C        DQNR - derivative of QREF wrt electron density
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      DIMENSION AMAS(30),TYPAT(30),ABND(30),D(4,30),E(4,10),
     *          abnref(mdepth),O(4,20)
      dimension xio(9,30),eio(9,10),oio(9,20)
      dimension ffi(30),pfstu(30),pfstt(30),pfstn(30),entot(30)
      dimension idat(30),uu(30,17)
      dimension u6(6),u7(7),u8(8),u10(10),u11(11)
      dimension u12(12),u13(13),u14(14),u16(16),u18(18),u20(20)
      dimension u24(24),u25(25),u26(26),u28(28)
      COMMON/PFSTDS/PFSTD(30,30),MODPF(30)
      COMMON/RRVALS/RR(30,30),ABNDD(30,MDEPTH),ENEV(30,30),IONIZ(30),
     *              IREF,LGR(30),LRM(30)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      PARAMETER (TH0=5.0404D3,XMX0=2.154D4,THL0=2.3025851,FI0=3.6113D1,
     *           TRHA=1.5D0,c1qm=1.0353d-16,c2qm=8762.9)
C
C    Standard atomic constants for first 30 species:
C
C            Element Atomic  Solar    Std.
C                    weight abundance highest 
C                                     ionization stage 
      DATA O/'  H ', 1.008, 1.00E 0, 2.,
     *       '  HE', 4.003, 1.00D-1, 3.,
     *       '  LI', 6.941, 3.9D-12, 3.,
     *       '  BE', 9.012, 1.1D-11, 3.,
     *       '  B ',10.810, 6.2D-10, 4.,
     *       '  C ',12.011, 3.68D-4, 5.,
     *       '  N ',14.007, 1.14D-4, 5.,
     *       '  O ',16.000, 6.70D-4, 5.,
     *       '  F ',18.918, 3.63D-8, 4.,
     *       '  NE',20.179, 2.79D-5, 4.,
     *       '  NA',22.990, 1.72D-6, 4.,
     *       ' MG ',24.305, 3.43D-5, 4.,
     *       ' AL ',26.982, 2.49D-6, 4.,
     *       ' SI ',28.086, 3.51D-5, 5.,
     *       ' P  ',30.974, 2.67D-7, 5.,
     *       ' S  ',32.060, 1.61D-5, 5.,
     *       ' CL ',35.453, 4.42D-7, 5.,
     *       ' AR ',39.948, 4.42D-6, 5.,
     *       ' K  ',39.098, 1.11D-7, 5.,
     *       ' CA ',40.080, 2.12D-6, 5./
      DATA E/' SC ',44.956, 1.16D-9, 5.,
     *       ' TI ',47.900, 5.44D-8, 5.,
     *       ' V  ',50.941, 1.25D-8, 5.,
     *       ' CR ',51.996, 4.96D-7, 5.,
     *       ' MN ',54.938, 1.57D-7, 5.,
     *       ' FE ',55.847, 2.49D-5, 5.,
     *       ' CO ',58.933, 3.13D-8, 5.,
     *       ' NI ',58.700, 1.89D-6, 5.,
     *       ' CU ',63.546, 2.79D-8, 5.,
     *       ' ZN ',65.380, 2.60D-8, 5./
C
C     Ionization potentials for first 30 species:
C
C     Element Ionization potentials (eV) 
C              I     II      III     IV       V     VI     VII    VIII
C
      data oio/'  H ',13.595, 0. , 0. , 0. ,  0.  ,  0.  ,  0.  ,  0.  ,
     *'  HE',24.580, 54.400,  0.   ,  0.   ,  0.  ,  0.  ,  0.  ,  0.  ,
     *'  LI', 5.392, 75.619,122.451,  0.   ,  0.  ,  0.  ,  0.  ,  0.  ,
     *'  BE', 9.322, 18.206,153.850,217.713,  0.  ,  0.  ,  0.  ,  0.  ,
     *'  B ', 8.296, 25.149, 37.920,259.298,340.22,  0.  ,  0.  ,  0.  ,
     *'  C ',11.264, 24.376, 47.864, 64.476,391.99,489.98,  0.  ,  0.  ,
     *'  N ',14.530, 29.593, 47.426, 77.450, 97.86,551.93,667.03,  0.  ,
     *'  O ',13.614, 35.108, 54.886, 77.394,113.87,138.08,739.11,871.39,
     *'  F ',17.418, 34.980, 62.646, 87.140,114.21,157.12,185.14,953.6 ,
     *'  NE',21.559, 41.070, 63.500, 97.020,126.30,157.91,207.21,239.0 ,
     *'  NA', 5.138, 47.290, 71.650, 98.880,138.37,172.09,208.44,264.16,
     *'  MG', 7.664, 15.030, 80.120,102.290,141.23,186.49,224.9 ,265.96, 
     *' AL ', 5.984, 18.823, 28.440,119.960,153.77,190.42,241.38,284.53, 
     *' SI ', 8.151, 16.350, 33.460, 45.140,166.73,205.11,246.41,303.07, 
     *' P  ',10.484, 19.720, 30.156, 51.354, 65.01,220.41,263.31,309.26,
     *' S  ',10.357, 23.400, 35.000, 47.290, 72.50, 88.03,280.99,328.8 ,
     *' CL ',12.970, 23.800, 39.900, 53.500, 67.80, 96.7 ,114.27,348.3 ,
     *' AR ',15.755, 27.620, 40.900, 59.790, 75.00, 91.3 ,124.0 ,143.46,
     *' K  ', 4.339, 31.810, 46.000, 60.900, 82.6 , 99.7 ,118.0 ,155.0 ,
     *' CA ', 6.111, 11.870, 51.210, 67.700, 84.39,109.0 ,128.0 ,147.0 /
      data eio/' SC ',6.560,12.890,24.750,73.900,92.0,111.1,138.0,158.7,
     *' TI ', 6.830, 13.630, 28.140, 43.240, 99.8 ,120.0 ,140.8 ,168.5 ,
     *' V  ', 6.740, 14.200, 29.700, 48.000, 65.2 ,128.9 ,151.0 ,173.7 ,
     *' CR ', 6.763, 16.490, 30.950, 49.600, 73.0 , 90.6 ,161.1 ,184.7 ,
     *' MN ', 7.432, 15.640, 33.690, 53.000, 76.0 , 97.0 ,119.24,196.46,
     *' FE ', 7.870, 16.183, 30.652, 54.800, 75.0 , 99.1 ,125.0 ,151.06,
     *' CO ', 7.860, 17.060, 33.490, 51.300, 79.5 ,102.0 ,129.0 ,157.0 ,
     *' NI ', 7.635, 18.168, 35.170, 54.900, 75.5 ,108.0 ,133.0 ,162.0 ,
     *' CU ', 7.726, 20.292, 36.830, 55.200, 79.9 ,103.0 ,139.0 ,166.0 ,
     *' ZN ', 9.394, 17.964, 39.722, 59.400, 82.6 ,108.0 ,134.0 ,174.0 /
C
C     data for additional ionization potentials for the Opacity
C     project species (IDAT sets the internal OP indexing)
C
      data idat   / 1, 2, 0, 0, 0, 3, 4, 5, 0, 6,
     *              7, 8, 9,10, 0,11, 0,12, 0,13,
     *              0, 0, 0,14,15,16, 0,17,0,0/ 
      data u10/173.93,330.391,511.8,783.3,1018.,1273.8,1671.792,
     *         1928.462,9645.005,10986.876/
      data u11/41.449,381.395,577.8,797.8,1116.2,1388.5,1681.5,2130.8,
     *         2418.7,11817.061,13297.676/
      data u12/61.671,121.268,646.41,881.1,1139.4,1504.3,1814.3,2144.7,
     *         2645.2,2964.4,14210.261,15829.951/
      data u13/48.278,151.86,229.446,967.8,1239.8,1536.3,1947.3,2295.4,
     *         2663.4,3214.8,3565.6,16825.022,18584.138/
      data u14/65.748,131.838,270.139,364.093,1345.1,1653.9,1988.4,
     *         2445.3,2831.9,3237.8,3839.8,4222.4,19661.693,21560.63/
      data u16/83.558,188.2,280.9,381.541,586.2,710.184,2265.9,2647.4,
     *         3057.7,3606.1,4071.4,4554.3,5255.9,5703.6,26002.663,
     *         28182.535/
      data u18/127.11,222.848,328.6,482.4,605.1,734.04,1002.73,1157.08,
     *         3407.3,3860.9,4347.,4986.6,5533.8,6095.5,6894.2,7404.4,
     *         33237.173,35699.936/
      data u20/49.306,95.752,410.642,542.6,681.6,877.4,1026.,1187.6,
     *         1520.64,1704.047,4774.,5301.,5861.,6595.,7215.,7860.,
     *         8770.,9338.,41366.,44177.41/
      data u24/54.576,132.966,249.7,396.5,560.2,731.02,1291.9,1490.,
     *         1688.,1971.,2184.,2404.,2862.,3098.52,8151.,8850.,
     *         9560.,10480.,11260.,12070.,13180.,13882.,60344.,63675.9/
      data u25/59.959,126.145,271.55,413.,584.,771.1,961.44,1569.,
     *         1789.,2003.,2307.,2536.,2771.,3250.,3509.82,9152.,
     *         9872.,10620.,11590.,12410.,13260.,14420.,15162.,
     *         65660.,69137.4/
      data u26/63.737,130.563,247.22,442.,605.,799.,1008.,1218.38,
     *         1884.,2114.,2341.,2668.,2912.,3163.,3686.,3946.82,
     *         10180.,10985.,11850.,12708.,13620.,14510.,15797.,
     *         16500.,71203.,74829.6/
      data u28/61.6,146.542,283.8,443.,613.5,870.,1070.,1310.,1560.,
     *         1812.,2589.,2840.,3100.,3470.,3740.,4020.,4606.,
     *         4896.2,12430.,13290.,14160.,15280.,16220.,17190.,
     *         18510.,19351.,82984.,86909.4/
C
      EQUIVALENCE (D(1,1),O(1,1))
      EQUIVALENCE (D(1,21),E(1,1))
      EQUIVALENCE (xio(1,1),Oio(1,1))
      EQUIVALENCE (xio(1,21),Eio(1,1))
      equivalence (u6(1),uu(1,3)),(u7(1),uu(1,4)),(u8(1),uu(1,5))
      equivalence (u10(1),uu(1,6)),(u11(1),uu(1,7)),(u12(1),uu(1,8))
      equivalence (u13(1),uu(1,9)),(u14(1),uu(1,10)),(u16(1),uu(1,11))
      equivalence (u18(1),uu(1,12)),(u20(1),uu(1,13)),(u24(1),uu(1,14))
      equivalence (u25(1),uu(1,15)),(u26(1),uu(1,16)),(u28(1),uu(1,17))
C
      IF(MODE.NE.0) GO TO 50
C
C     For MODE=0, STATE serves as an auxiliary procedure for START
C     Input of basic parameters for individual chemical species
C
C     An element (hydrogen through zinc) can be considered in one of
C     the three following options:
C     1. explicitly - some of energy levels of some of its ionization
C                     states are considered explicitly, ie. their
C                     populations are determined by solving statistical
C                     equilibrium
C     2. implicitly - the atom is assumed not to contribute to
C                     opacity; but is allowed to contribute to the
C                     total number of particles and to the total charge;
C                     the latter is evaluated assuming LTE ionization
C                     balance, ie. by solving a set of Saha equations
C     3. not considered at all
C
C     Input:
C     NATOMS -  the highest atomic number of an element that is
C               considered (explicitly or non-explicitly)
C            <  0 - then NATOMS=-NATOMS, and all Opacity Project
C                   species are treated as with MODPF>0, i.e.
C                   their partition functions are evaluated from
C                   the Opacity Project ionization fractions
C
C     For each element from 1 (hydrogen) to NATOMS, the following
C     parameters:
C
C     MA     =  0  - if the element is not considered (option 3)
C            =  1  - if the element is non-explicit (option 2)
C            =  2  - if the element is explicit (option 1)
C     ABN    -  if ABN=0, solar abundance is assumed (given above;
C                         abundance here is assumed as relative
C                         to hydrogen by number
C               if ABN>0, non-solar abundance ABN is assumed; in an
C                         arbitrary scale
C               if ABN<0, non-solar abundance ABN is assumed;
C                         (-ABN times the solar value)
C               if ABN>1.e6, depth-dpendent abundance (additional input)
C
      READ(IBUFF,*) NATOMS
      WRITE(6,600)
      IAT=0
      IREF=0
      IFOPPF=0
      IF(NATOMS.LT.0) THEN
         NATOMS=-NATOMS
         IFOPPF=2
      END IF
C
      DO 20 I=1,NATOMS
         LGR(I)=.TRUE.
         LRM(I)=.TRUE.
         IATEX(I)=-1
         READ(IBUFF,*) MA,ABN,MODPF0
         IF(MA.EQ.0) GO TO 20
         TYPAT(I)=D(1,I)
         AMAS(I)=D(2,I)
         ABND(I)=D(3,I)
         IONIZ(I)=D(4,I)
         MODPF(I)=MODPF0
C
C        increase the standard highest ionization for Teff larger
C        than 50000 K for N, O, Ne, and Fe
C
         IF(TEFF.GT.5.D4) THEN
            IF(I.EQ.7) IONIZ(I)=6
            IF(I.EQ.8) IONIZ(I)=7
            IF(I.EQ.10) IONIZ(I)=9
            IF(I.EQ.26) IONIZ(I)=9
         END IF
C
         DO 10 J=1,8
   10       ENEV(I,J)=xio(1+J,I)
         LGR(I)=.FALSE.
         IF(MODPF(I).GT.0) IFOPPF=1
         IF(IFOPPF.EQ.2.and.idat(i).gt.0) MODPF(I)=1
         if(modpf(i).gt.0) then
            if(idat(i).eq.0) modpf(i)=0
            if(modpf(i).gt.0) then
               ioniz(i)=i+1
               if(i.ge.10) then
                  do j=9,i
                     enev(i,j)=uu(j,idat(i))*0.1239529d0
                  end do
               end if
            end if
         end if
         IF(ABN.GT.0) ABND(I)=ABN
         IF(ABN.LT.0) ABND(I)=ABS(ABN)*D(3,I)
         IF(ION.NE.0) IONIZ(I)=ION
         IF(ABN.GT.1.E6) THEN
            READ(IBUFF,*) (ABNDD(I,ID),ID=1,ND)
          ELSE
            DO ID=1,ND
               ABNDD(I,ID)=ABND(I)
            END DO
         END IF
         IF(MA.EQ.1) THEN
            LRM(I)=.FALSE.
            IATEX(I)=0
          ELSE
            IAT=IAT+1
            IIFIX(IAT)=0
            IF(MA.EQ.-2) IIFIX(IAT)=1
            IATEX(I)=IAT
            IF(IAT.EQ.IATREF) THEN
               IREF=I
               DO ID=1,ND
                  ABNREF(ID)=ABNDD(I,ID)
               END DO
            END IF
C
C           store parameters for explicit atoms
C
            AMASS(IAT)=AMAS(I)*HMASS
            NUMAT(IAT)=I
         END IF
   20 CONTINUE
C
C     renormalize abundances to have the standard element abundance
C     equal to unity
C
      DO 30 I=1,NATOMS
         IAT=IATEX(I)
         IF(IAT.LT.0) GO TO 30
         DO ID=1,ND
            ABNDD(I,ID)=ABNDD(I,ID)/ABNREF(ID)
            YTOT(ID)=YTOT(ID)+ABNDD(I,ID)
            WMY(ID)=WMY(ID)+ABNDD(I,ID)*AMAS(I)
         END DO
         ABNR=ABND(I)/D(3,I)
         IF(IAT.EQ.0) THEN
            WRITE(6,601) I,TYPAT(I),ABND(I),ABNR
          ELSE
            DO ID=1,ND
               ABUND(IAT,ID)=ABNDD(I,ID)
            END DO 
            WRITE(6,602) I,TYPAT(I),ABND(I),ABNR,IAT
         END IF
   30 CONTINUE
      DO ID=1,ND
         WMM(ID)=WMY(ID)*HMASS/YTOT(ID)
      END DO
c
c     initialization of the Opacity Project ionization fractions
c     (if required)
c
      if(ifoppf.gt.0) call opfrac(0,0,t,ane,pf,fra)
c
  600 FORMAT(1H0//' CHEMICAL ELEMENTS INCLUDED'/
     *            ' --------------------------'//
     * ' NUMBER  ELEMENT           ABUNDANCE'/1H ,16X,
     * 'A=N(ELEM)/N(H)  A/A(SOLAR)'/)
  601 FORMAT(1H ,I4,3X,A5,1P2E14.2)
  602 FORMAT(1H ,I4,3X,A5,1P2E14.2,3X,
     *       'EXPLICIT: IAT=',I3)
      RETURN
C
   50 TLN=LOG(T)*TRHA
      TK=BOLK*T
      THET=TH0/T
      THL=THL0*THET
      XMX=XMX0*SQRT(SQRT(T/ANE))
      DCH=EH/(XMX*XMX*TK)
      Q=0.
      QM=0.
      DQT=0.
      DQN=0.
      DQM=0.
      ENER=0.
      DO 70 I=1,NATOMS
         IF(MODE.GT.1.AND.LRM(I).OR.MODE.EQ.1.AND.LGR(I)) GO TO 70
         ION=IONIZ(I)
         DRQT=0.
         DRQN=0.
         DRST=0.
         DRSN=0.
         DFT=0.
         DFN=0.
         ENTOT(1)=0.
         RS=UN
         CALL PARTF(I,1,T,ANE,XMX,UM,DUTM,DUNM)
         pfstu(1)=um
         pfstt(1)=dutm
         pfstn(1)=dunm
         JMAX=1
         DO 55 J=2,ION
            J1=J-1
            DCHT=DCH*J1
            TE=ENEV(I,J1)*THL
            ENTOT(J)=ENTOT(J1)+TE
            FI=FI0+TLN-TE+DCHT
            X=J
            XMAX=XMX*SQRT(X)
            CALL PARTF(I,J,T,ANE,XMAX,U,DUT,DUN)
            pfstu(j)=u
            pfstt(j)=dut
            pfstn(j)=dun
            FFI(J)=0.
            IF(FI.GT.-20.) FFI(J)=EXP(FI)*U/UM/ANE
            IF(FFI(J).GT.UN) JMAX=J
            UM=U
   55    CONTINUE
         RQ=JMAX-1
         RI=ENTOT(JMAX)
         RE=PFSTT(JMAX)/PFSTU(JMAX)*T
         if(jmax.lt.ion) then
         R=UN
         DO 60 J=JMAX+1,ION
            J1=J-1
            DCHT=DCH*J1
            TE=ENEV(I,J1)*THL
            R=R*FFI(J)
	    RR(I,J)=R/pfstu(j)
            RS=RS+R
            RQ=RQ+J1*R
            RI=RI+R*ENTOT(J)
            RE=RE+R*PFSTT(J)/PFSTU(J)*T
            DFIT=pfstt(j)/pfstu(j)-pfstt(j1)/pfstu(j1)
     .           +(TRHA+TE-TRHA*DCHT)/T
            DFIN=pfstn(j)/pfstu(j)-pfstn(j1)/pfstu(j1)
     .           +(HALF*DCHT-UN)/ANE
            DFT=DFT+DFIT
            DFN=DFN+DFIN
            DFIT=DFT*R
            DFIN=DFN*R
            DRST=DRST+DFIT
            DRSN=DRSN+DFIN
            DRQT=DRQT+J1*DFIT
            DRQN=DRQN+J1*DFIN
   60    CONTINUE
         endif
         if(jmax.gt.1) then
         R=UN
         DFT=0.
         DFN=0.
         jmin=min(4,jmax-1)
         DO 61 JJ=1,JMIN
            J=JMAX-JJ
            J1=J-1
            JP1=J+1
            DCHT=DCH*J
            TE=ENEV(I,J)*THL
            R=R/FFI(JP1)
	    RR(I,J)=R/pfstu(j)
            RS=RS+R
            RQ=RQ+J1*R
            RI=RI+R*ENTOT(J)
            RE=RE+R*PFSTT(J)/PFSTU(J)*T
            DFIT=pfstt(jp1)/pfstu(jp1)-pfstt(j)/pfstu(j)
     .           +(TRHA+TE-TRHA*DCHT)/T
            DFIN=pfstn(jp1)/pfstu(jp1)-pfstn(j)/pfstu(j)
     .           +(HALF*DCHT-UN)/ANE
            DFT=DFT-DFIT
            DFN=DFN-DFIN
            DFIT=DFT*R
            DFIN=DFN*R
            DRST=DRST+DFIT
            DRSN=DRSN+DFIN
            DRQT=DRQT+J1*DFIT
            DRQN=DRQN+J1*DFIN
   61    CONTINUE
         endif
         X=RQ/RS
         ABND(I)=ABNDD(I,ID)
         X1=ABND(I)/RS
	 RR(I,JMAX)=X1
	 DO 65 J=1,ION
           IF(J.NE.JMAX) RR(I,J)=RR(I,J)*X1
   65    CONTINUE
	 RR(I,JMAX)=RR(I,JMAX)/PFSTU(JMAX)
         if(i.gt.2) ENER=ENER+(RI+RE)*X1
         IF(I.EQ.IREF) THEN
            QREF=X*ABND(I)
            DQTR=(DRQT-X*DRST)*X1
            DQNR=(DRQN-X*DRSN)*X1
          ELSE
            Q=X*ABND(I)+Q
            DQT=DQT+(DRQT-X*DRST)*X1
            DQN=DQN+(DRQN-X*DRSN)*X1
         END IF
   70 CONTINUE
C
C     Negative hydrogen ion
C
      IF(IHM.EQ.1) THEN
         tinv=un/t
         QM=C1QM*Tinv/SQRT(T)*EXP(C2QM*Tinv)
         DQM=-QM*Tinv*(TRHA+C2QM*Tinv)
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
      SUBROUTINE INPMOD
C     =================
C
C     Read an initial model atmosphere from unit 8
C     File 8 contains:
C      1. NDPTH -  number of depth points in which the initial model is
C                  given (if not equal to ND, routine interpolates
C                  automatically to the set DM by linear interpolation
C                  in log(DM)
C         NUMPAR - number of input model parameters in each depth
C                  = 3 for LTE model - ie. N, T, N(electron);
C                  > 3 for NLTE model)
C      2. DEPTH(ID),ID=1,NDPTH - mass-depth points for the input model
C      3. for each depth:
C                 T   - temperature
C                 ANE - electron density
C                 RHO - mass density
C                 level populations - only for NLTE input model
C                       Number of input level populations need not be
C                       equal to NLEVEL; in that case the procedure
C                       CHANGE is called from START to calculate the
C                       remaining level populations
C
C     Note: The output file 7, which is created by this program
C           (procedure OUTPUT) has the same structure as file 8
C           and may thus be used as input to another run of the
C           program
C
C     INTRPL - switch indicating whether (and, if so, how) interpolate
C              the initial model if the depth scales for the input model
C              and the present depth scale are different
C            = 0  -  no interpolation, scale DEPTH replaces DM;
C                    i.e. DEPTH will be used as DM, regardless of which
C                    values of DM were read in subroutine START
C            > 0  -  polynomial interpolation of the (INTRPL-1)th order
C
C     If INTRPL > 0, there is an additional input from unit 8, namely
C     new depth scale DM, the one which will be used in the present run
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (MINPUT=MLEVEL+4)
      COMMON POPUL0(MLEVEL,MDEPTH),ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       TEMP0(MDEPTH),ELEC0(MDEPTH),DENS0(MDEPTH),PPL0(MDEPTH),
     *       DEPTH(MDEPTH),PPL(MDEPTH),POPLTE(MLEVEL),X(MINPUT)
      COMMON/STDPAR/ELSTD,IDSTD
      COMMON/NLVINP/NLEV0
C
      LCHC0=LCHC
      LCHC=.TRUE.
      LTE0=LTE
      LTE=.TRUE.
      IF(INTRPL.GE.0) THEN
        READ(8,*) NDPTH,NUMPAR
        IF(NDPTH.LE.0) CALL STOP(' ERROR in Unit 8')
        READ(8,*) (DEPTH(I),I=1,NDPTH)
        IF(NUMPAR.EQ.3) CALL LEVSET
        DO 30 ID=1,NDPTH
           READ(8,*) (X(I),I=1,NUMPAR)
           TEMP0(ID)=X(1)
           ELEC0(ID)=X(2)
           DENS0(ID)=X(3)
           IF(NUMPAR.GT.3) THEN
              NLEV0=NUMPAR-3
              DO 10 I=1,NLEV0
   10            POPUL0(I,ID)=X(3+I)
            ELSE
              NLEV0=NLEVEL
              TEMP(ID)=X(1)
              ELEC(ID)=X(2)
              DENS(ID)=X(3)
              CALL WNSTOR(ID)
              CALL SABOLF(ID)
              CALL STEQEQ(ID,POPLTE,0)
              DO 20 I=1,NLEV0
   20            POPUL0(I,ID)=POPLTE(I)
           END IF
   30   CONTINUE
        READ(8,*,END=31,ERR=31) INTRPL
      ELSE
        CALL KURUCZ(NDPTH)
        NUMPAR=3
        IF(ND.NE.NDPTH .AND. INTRPL.EQ.0)
     *    CALL STOP(' Incompatible number of depth points - KURUCZ')
      END IF
   31 LCHC=LCHC0
      LTE=LTE0
      IF(NUMPAR.EQ.3) CALL LEVSET
C
C     if INTRPL = 0  -  no interpolation
C        i.e. scale DEPTH replaces DM
C
      IF(INTRPL.EQ.0) THEN
         IDSTD=0
         DO 40 ID=1,ND
            DM(ID)=DEPTH(ID)
            TEMP(ID)=TEMP0(ID)
            ELEC(ID)=ELEC0(ID)
            DENS(ID)=DENS0(ID)
            CALL WNSTOR(ID)
            IF(TEMP(ID).LT.TEFF) IDSTD=ID
            DO 40 I=1,NLEVEL
               POPUL(I,ID)=POPUL0(I,ID)
   40    CONTINUE
         ELSTD=ELEC(IDSTD)
         RETURN
       ELSE
C
C    if INTRPL > 0  - Interpolation from log(DEPTH) to log(DM)
C       INTRPL = 1 or 2 - linear interpolation,
C              = 3  -  parabolic interpolation,
C              = 4  -  cubic interpolation, etc.
C
C       DM(ID)     - mass-depth variable (ie. column mass in g*cm**-2)
C                    at the depth point ID
C
         READ(8,*) (DM(ID),ID=1,ND)
         NTRPL=INTRPL
         CALL INTERP(DEPTH,TEMP0,DM,TEMP,NDPTH,ND,NTRPL,1,0)
         CALL INTERP(DEPTH,ELEC0,DM,ELEC,NDPTH,ND,NTRPL,1,1)
         CALL INTERP(DEPTH,DENS0,DM,DENS,NDPTH,ND,NTRPL,1,1)
         DO 70 I=1,NLEV0
            DO 50 ID=1,NDPTH
   50          PPL0(ID)=POPUL0(I,ID)
            CALL INTERP(DEPTH,PPL0,DM,PPL,NDPTH,ND,NTRPL,1,1)
            DO 60 ID=1,ND
               CALL WNSTOR(ID)
   60          POPUL(I,ID)=PPL(ID)
   70    CONTINUE
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE KURUCZ(NDPTH)
C     ========================
C
C     Read an initial model atmosphere from unit 8
C      in Kurucz ATLAS' format
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (MINPUT=7)
      CHARACTER KUR*15
      DIMENSION KKFIX0(MLEVEL)
      COMMON POPUL0(MLEVEL,MDEPTH),ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       TEMP0(MDEPTH),ELEC0(MDEPTH),DENS0(MDEPTH),PPL0(MDEPTH),
     *       DEPTH(MDEPTH),PPL(MDEPTH),POPLTE(MLEVEL),X(MINPUT)
      COMMON/NLVINP/NLEV0
C
      do iat=1,natom
        kkfix0(iat)=iifix(iat)
        iifix(iat)=0
      end do
C
      READ(8,801) KUR,GRAVK
      READ(KUR,802) TEFFK
  801 FORMAT(A15,6X,F8.5)
  802 FORMAT(4X,F8.0)
C
      IF(KUR(1:4).NE.'TEFF')
     *  CALL STOP(' Unit 8 is NOT a Kurucz model as expected')
      IF(ABS(TEFFK-TEFF).GT.50.)
     *  CALL STOP(' Teff not corresponding to Kurucz model')
      IF(ABS(GRAVK-LOG10(GRAV)).GT.0.02)
     *  CALL STOP(' Gravity not corresponding to Kurucz model')
C
      DO WHILE(KUR(1:9).NE.'READ DECK')
        READ(8,'(A15)') KUR
      END DO
      READ(KUR,803) NDPTH
  803 FORMAT(10X,I3)
      NDPTH=NDPTH-1
      NUMPAR=3
      CALL LEVSET
      NLEV0=NLEVEL
      READ(8,*) TTT
      DO ID=1,NDPTH
        READ(8,*) (X(I),I=1,MINPUT)
        DEPTH(ID)=X(1)
        TEMP0(ID)=X(2)
        ELEC0(ID)=X(4)
        AN=X(3)/BOLK/TEMP0(ID)
        DENS0(ID)=WMM(ID)*(AN-ELEC0(ID))
        TEMP(ID)=TEMP0(ID)
        ELEC(ID)=ELEC0(ID)
        DENS(ID)=DENS0(ID)
        CALL WNSTOR(ID)
        CALL SABOLF(ID)
        CALL STEQEQ(ID,POPLTE,0)
        DO I=1,NLEV0
          POPUL0(I,ID)=POPLTE(I)
        END DO
      END DO
C
      INTRPL=0
      DO WHILE(KUR(1:5).NE.'BEGIN')
        READ(8,'(A15)',END=100,ERR=100) KUR
      END DO
      READ(8,*,END=100,ERR=100) INTRPL
C
  100 do iat=1,natom
        iifix(iat)=kkfix0(iat)
      end do
C
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE CHANGE
C     =================
C
C     This procedure controls an evaluation of initial level
C     populations in case where the system of explicit levels
C     (ie. the choice of explicit level, their numbering, or their
C     total number) is not consistent with that for the input level
C     populations read by procedure INPMOD.
C     Obviously, this procedure need be used only for NLTE input models.
C
C     ICHANG > 0 - general change of populations as described below
C            < 0 - a simplified change; no additional input
C                In this case, the program assumes that the levels present
C                in the input model remain in the new model, with the
C                same indices, while one only adds additional levels (all
C                (all assumed having LTE populations) with indices 
C                subsequently higher.
C
C     Input from unit 5:
C     For each explicit level, II=1,NLEVEL, the following parameters:
C      IOLD   -  NE.0 - means that population of this level is
C                       contained in the set of input populations;
C                       IOLD is then its index in the "old" (i.e. input)
C                       numbering.
C                       All the subsequent parameters have no meaning
C                       in this case.
C             -  EQ.0 - means that this level has no equivalent in the
C                       set of "old" levels. Population of this level
C                       has thus to be evaluated.
C      MODE   -    indicates how the population is evaluated:
C             = 0  - population is equal to the population of the "old"
C                    level with index ISIOLD, multiplied by REL;
C             = 1  - population assumed to be LTE, with respect to the
C                    first state of the next ionization degree whose
C                    population must be contained in the set of "old"
C                    (ie. input) populations, with index NXTOLD in the
C                    "old" numbering.
C                    The population determined of this way may further
C                    be multiplied by REL.
C             = 2  - population determined assuming that the b-factor
C                    (defined as the ratio between the NLTE and
C                    LTE population) is the same as the b-factor of
C                    the level ISINEW (in the present numbering). The
C                    level ISINEW must have the equivalent in the "old"
C                    set; its index in the "old" set is ISIOLD, and the
C                    index of the first state of the next ionization
C                    degree, in the "old" numbering, is NXTSIO.
C                    The population determined of this way may further
C                    be multiplied by REL.
C             = 3  - level corresponds to an ion or atom which was not
C                    explicit in the old system; population is assumed
C                    to be LTE.
C      NXTOLD  -  see above
C      ISINEW  -  see above
C      ISIOLD  -  see above
C      NXTSIO  -  see above
C      REL     -  population multiplier - see above
C                 if REL=0, the program sets up REL=1
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/NLVINP/NLEV0
      COMMON POPUL0(MLEVEL,MDEPTH),POPULL(MLEVEL,MDEPTH),
     *       ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),POPL(MLEVEL)
C
      PARAMETER (S = 2.0706D-16)
      IF(ICHANG.GE.1) THEN
      IFESE=0
      DO 100 II=1,NLEVEL
         READ(IBUFF,*) IOLD,MODE,NXTOLD,ISINEW,ISIOLD,NXTSIO,REL
         IF(REL.EQ.0.) REL=1.
	 IF(MODE.GE.3) IFESE=IFESE+1
         DO 90 ID=1,ND
            IF(IOLD.EQ.0) GO TO 10
            POPUL0(II,ID)=POPUL(IOLD,ID)
            GO TO 90
   10       IF(MODE.NE.0) GO TO 20
            POPUL0(II,ID)=POPUL(ISIOLD,ID)*REL
            GO TO 90
   20       T=TEMP(ID)
            ANE=ELEC(ID)
	    IF(MODE.GE.3) GO TO 40
            NXTNEW=NNEXT(IEL(II))
            SB=S/T/SQRT(T)*G(II)/G(NXTNEW)*EXP(ENION(II)/T/BOLK)
            IF(MODE.GT.1) GO TO 30
            POPUL0(II,ID)=SB*ANE*POPUL(NXTOLD,ID)*REL
            GO TO 90
   30       KK=ISINEW
            KNEXT=NNEXT(IEL(KK))
            SBK=S/T/SQRT(T)*G(KK)/G(KNEXT)*EXP(ENION(KK)/T/BOLK)
            POPUL0(II,ID)=SB/SBK*POPUL(NXTOLD,ID)/POPUL(NXTSIO,ID)*
     *                    POPUL(ISIOLD,ID)*REL
	    GO TO 90
   40       IF(IFESE.EQ.1) THEN
               LTE0=LTE
	       LTE=.TRUE.
               CALL LEVSET
	       CALL SABOLF(ID)
	       CALL STEQEQ(ID,POPL,0)
	       DO 50 III=1,NLEVEL
   50             POPULL(III,ID)=POPL(III)
	       LTE=LTE0
               CALL LEVSET
	    END IF
	    POPUL0(II,ID)=POPULL(II,ID)
   90    CONTINUE
  100 CONTINUE
      DO 110 I=1,NLEVEL
         DO 110 ID=1,ND
            POPUL(I,ID)=POPUL0(I,ID)
  110 CONTINUE
C
C     simplified change - no additional input (the case ICHANG < 0)
C
      ELSE
      LTE0=LTE
      LTE=.TRUE.
      CALL LEVSET
      DO 200 ID=1,ND
	 CALL SABOLF(ID)
	 CALL STEQEQ(ID,POPL,0)
         DO 120 II=NLEV0+1,NLEVEL
            POPUL(II,ID)=POPL(II)
  120    CONTINUE
  200 CONTINUE
      LTE=LTE0
      CALL LEVSET
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE RESOLV
C     =================
C
C     Control procedure for the formal solution, i.e. all calculations
C     between two consecutive iterations of complete linearization
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ARRAY1.FOR'
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/FILES/PSY0(MTOT,MDEPTH),PSY1(MTOT,MDEPTH),
     * PSY2(MTOT,MDEPTH),PSY3(MTOT,MDEPTH)
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
C
C     Initialization - procedure INILAM:
C
      CALL INILAM
      LFINE=LFIN.AND.NITER.GT.0
      IF(NLAMBD.LE.0) GO TO 80
      IF(LFINE) NLAMBD=1
      LAC2P=.FALSE.
      IACC0P=IACCP-3
C
C     Set of NLAMBD procedures, called overall "lambda" iterations
C     (not to be confused with ordinary lambda iterations)
C     Each "lambda" iteration contains:
C
      DO 70 ILAM=1,NLAMBD
         CALL OPAINI(ILAM)
C
C     Radiative rates in all transitions, in all depths
C
         IF(IFPREC.EQ.0) THEN
            IF(IOVER.EQ.0) THEN
               CALL RATES
             ELSE
               CALL RATES1
            END IF
          ELSE IF(IFPREC.EQ.1) THEN
            IF(IOVER.EQ.0) THEN
               CALL RATESP
             ELSE
               CALL RATSP1
            END IF
          ELSE
            IF(IOVER.EQ.0) THEN
               CALL RATESP
             ELSE
               CALL RATSP3
            END IF
         END IF
C
C        Initialisation (if INIT=1), or updating (if INIT=0) of the
C        collisional-radiative switching parameters
C        (only if this option is chosen , i.e. if ICRSW>0)
C
         IF(ILAM.EQ.1) CALL SWITCH(INIT)
C
C ****** C) Third part - evaluation of the new populations,
C                 using all the previously calculated radiative rates
C           This part also contains, if desired (if LCHC=true),
C                 recalculation of the electron density, by solving
C                 a non-linear set of statistical equilibrium +
C                 particle conservation + charge conservation equations
C                 (procedure ELCOR)
C
         IF(IFPREC.LE.1) THEN
            DO 60 ID=1,ND
               CALL STEQEQ(ID,POP,0)
               CALL NEWPOP(ID,POP)
               IF(.NOT.LCHC.and.iter.lt.ielcor) CALL ELCOR(ID)
   60       CONTINUE
          ELSE
            CALL STEQ3
         END IF
C
         if(iprind.eq.2) call output
C
C ****** D)  Fourth part - acceleration of convergence
C
         IF(IACCP.GT.0) CALL ACCELP
C
   70 CONTINUE
   80 IF(NDRE.LE.0.AND.ITER.EQ.1.OR.LFIN) CALL ROSSTD(0,0)
C
C     in case of convection is considered: call pzeval to evaluate pressures
C     and the logarithmic gradient of the total pressure
c
      IF(HMIX0.GE.0.) CALL PZEVAL
      if(iter.le.1) call radpre(0)
      CALL TIMING(1,ITER)
      ipng=1
      if(iacd.gt.0) ipng=mod((iter-iacc),iacd)
      if(ipng.eq.0 .and. iter.ge.iacc .and. lres2) goto 90
C
C     evaluate necessary ALI aprameters
C
      CALL OPAINI(0)
      IF(KANT(ITER).EQ.1.OR.LFINE) THEN
         IF(IOVER.EQ.0) THEN
            CALL ALISKT
          ELSE
            CALL ALISK1
         END IF
       ELSE
         IF(IOVER.EQ.0) THEN
            CALL ALISET
          ELSE
            IF(IRDER.EQ.0) THEN
               CALL ALIST1
             ELSE
               IF(IFALI.LE.5) THEN
                  CALL ALIST2
                ELSE
                  CALL ALIST3
               END IF
            END IF
         END IF
      END IF
c
C     if IFPOPR=2 - evaluate new populations
C
      if(ifpopr.eq.2) then
         DO 110 ID=1,ND
            CALL STEQEQ(ID,POP,1)
            IF(.NOT.LCHC.and.iter.lt.ielcor) CALL ELCOR(ID)
  110    CONTINUE
      END IF
   90 CONTINUE
C
C     Output of condensed model atmosphere to file 7
C     This file can serve as input of initial model atmosphere for
C     another run of the program.
C
      CALL OUTPUT
C
C     Output of computed model atmosphere - standard output file
C
      IF(LFINE) THEN
         CALL OUTPRI(1,1,1)
         RETURN
      END IF
C
C     The final part - to store previously calculated
C     mean intensities  - RADEX(IJ,ID),
C     and other model parameters (total number density, temperature,
C     electron density, and populations) for
C     further use in procedure SOLVE (actual complete linearization)
C
      IF(NFREQE.GT.0) THEN
      DO 120 IJ=1,NFREQE
         DO 120 ID=1,ND
            PSY0(IJ,ID)=RADEX(IJ,ID)
  120 CONTINUE
      END IF
      DO 150 ID=1,ND
         DO 130 II=1,NLVEXP
            PSY0(NFREQE+INSE+II-1,ID)=POPUL(INDLEV(II),ID)
  130    CONTINUE
         IF(INRE.NE.0) PSY0(NFREQE+INRE,ID)=TEMP(ID)
         IF(INPC.NE.0) PSY0(NFREQE+INPC,ID)=ELEC(ID)
         IF(INHE.NE.0) PSY0(NFREQE+INHE,ID)=DENS(ID)/WMM(ID)
     *                                     +ELEC(ID)
         IF(INMP.NE.0) PSY0(NFREQE+INMP,ID)=DENS(ID)/HMASS
         IF(INDL.NE.0) PSY0(NFREQE+INDL,ID)=DELTA(ID)
  150 CONTINUE
C
C     Output of condensed model atmosphere to file 7
C     This file can serve as input of initial model atmosphere for
C     another run of the program.
C
      CALL OUTPUT
C
C     Output of computed model atmosphere - standard output file
C
      IF(LFIN) CALL OUTPRI(1,1,1)
C  
      IF(INIT.EQ.1) THEN               
      DO ID=1,ND
         WRITE(6,601) ID,(NREFS(I,ID),I=1,NATOM)
      END DO
  600 FORMAT(//' REFERENCE LEVEL INDICES AS FUNCTIONS OF DEPTH'/)
  601 FORMAT(' ID=',I3,2X,15I4)
      END IF
C           
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE INILAM
C     =================
C
C     Auxiliary procedure for RESOLV
C     initialization of model parameters for further use in RESOLV
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (PCK=4.19168946D-10)
      DIMENSION COL(MTRANS),DENS0(MDEPTH),SBW(MLEVEL),XE(MDEPTH)
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/FILES/PSY0(MTOT,MDEPTH),PSY1(MTOT,MDEPTH),
     * PSY2(MTOT,MDEPTH),PSY3(MTOT,MDEPTH)
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      COMMON/CHARFX/QFIX(MDEPTH)
      COMMON/HEQAUX/PRD0,IHECOR
C
      IF(INIT.EQ.1) THEN
C
C     Before first iteration of complete linearization, ie. just after
C     initialization of basic parameters and reading the initial model
C     atmosphere - i.e. if INIT=1:
C     only evaluation of collisional rates for all transitions, at all
C     depths
C
         CALL TDPINI
         DO 20 ID=1,ND
            CALL WNSTOR(ID)
            FCOOL(ID)=0.
            FPRD(ID)=0.
            call sabolf(id)
c         
c           intialize b-factors
c
            DO 2 I=1,NLEVEL
               BFAC(I,ID)=UN      
               SBW(I)=ELEC(ID)*SBF(I)*WOP(I,ID)
    2       CONTINUE  
            IF(.NOT.LTE) THEN        
            DO 4 ION=1,NION
               DO 4 I=NFIRST(ION),NLAST(ION)
                  IF(POPUL(NNEXT(ION),ID).GT.0.)
     *            BFAC(I,ID)=POPUL(I,ID)/(POPUL(NNEXT(ION),ID)*SBW(I))
    4       CONTINUE      
            END IF
c
c           fully fixed part of the total charge
c
            qfix(id)=0.
            do 5 i=1,nlevel
               if(iiexp(i).ne.0.or.iifor(i).ne.0) go to 5
               ch=iz(iel(i))-1
               il=ilk(i)
               if(il.gt.0) ch=iz(il)+(iz(il)-1)*usum(il)*elec(id)
               qfix(id)=qfix(id)+ch*popul(i,id)
    5       continue
c
c           collisional rates
c
            IF(LTE) GO TO 20
            CALL COLIS(ID,TEMP(ID),COL)
            DO 10 I=1,NTRANS
   10          COLRAT(I,ID)=COL(I)
   20    CONTINUE
         CALL ODFMER
c        call elsct(0)
C
C        in case of ISPLIN>4 - (i.e. the DFE solver for transfer equation)
C        evaluate Planck function for the first estimate of RAD for
C        the electron scattering source function
C
         IF(ISPLIN.GE.5) THEN
            DO 30 ID=1,ND
               DO 30 IJ=1,NFREQ
                  RAD(IJ,ID)=BNUE(IJ)/(EXP(HKT1(ID)*FREQ(IJ))-UN)
   30       CONTINUE
         END IF
         RETURN
      END IF
C
C     Immediately after a completed iteration of complete linearization:
C
      PRAD=0.
      DO 40 ID=1,ND
C
C        save some old quantities
C     
c         DOLD(ID)=DENS(ID)
c         EOLD(ID)=ELEC(ID)
         AOLD=DENS(ID)/WMM(ID)+ELEC(ID)
         XE(ID)=UN-ELEC(ID)/AOLD
C
C     Depth by depth:
C        set up individual arrays of model parameters, namely:
C        temperature - TEMP (if temperature is one of variables,
C                      ie. if radiative equilibrium equation is solved)
C
         IF(INRE.NE.0) TEMP(ID)=PSY0(NFREQE+INRE,ID)
C
C        electron density - ELEC (if ELEC is one of variables, ie. if
C                       particle conservation equation is solved)
C
         IF(INPC.NE.0) ELEC(ID)=PSY0(NFREQE+INPC,ID)
C
C        density - DENS, calculated by means of total particle number
C                  density and electron density (if total particle
C                  number density is one of variables, ie. if hydrostatic
C                  equilibrium equation is solved - INHE.GT.0)
C
         DENS0(ID)=DENS(ID)
         IF(INHE.NE.0) DENS(ID)=WMM(ID)*(PSY0(NFREQE+INHE,ID)-ELEC(ID))
C
C        or again density, but calculated by means of the so called
C             fictitious massive particle density, if this is one of
C             variables - INMP.GT.0)
C
         IF(INMP.NE.0) DENS(ID)=HMASS*PSY0(NFREQE+INMP,ID)
C
C     the temperature logarithmic gradient delta
C
         IF(INDL.NE.0) DELTA(ID)=PSY0(NFREQE+INDL,ID)
   40 CONTINUE
C
C
C     if needed, formal solution of the hydrostatic equilibrium to get
C     new total particle density, and possibly electron density
C
      IF(IHECOR.EQ.1) THEN
         DO ID=1,ND
            PTUR=WMM(ID)*HALF*VTURB(ID)*VTURB(ID)
            AN=(DM(ID)*GRAV+PRD0-PRADT(ID)+PTUR*ELEC(ID))/
     *         (BOLK*TEMP(ID)+PTUR)
            DENS(ID)=WMM(ID)*(AN-ELEC(ID))
         END DO
       ELSE IF(IHECOR.EQ.2) THEN
         DO ID=1,ND
            PTUR=WMM(ID)*HALF*VTURB(ID)*VTURB(ID)
            AN=(DM(ID)*GRAV+PRD0-PRADT(ID))/(BOLK*TEMP(ID)+XE(ID)*PTUR)
            ELEC(ID)=(UN-XE(ID))*AN
            DENS(ID)=WMM(ID)*(AN-ELEC(ID))
         END DO
      END IF
C
C     other depth-dependent quantities
C
      DO 60 ID=1,ND
C
C      gas pressure and occupation probabilities
C
         PGS(ID)=(DENS(ID)/WMM(ID)+ELEC(ID))*BOLK*TEMP(ID)
         CALL WNSTOR(ID)
C
C     Evaluation of collision rates for the new temperature
C
         IF(LTE) GO TO 60
         call sabolf(id)
         CALL COLIS(ID,TEMP(ID),COL)
         DO 50 I=1,NTRANS
   50       COLRAT(I,ID)=COL(I)
   60 CONTINUE
C
      CALL CONCOR
      CALL TDPINI
C------------------------
C
C     new populations
C
C     1. first possibility (useful for pure CL) --
C     new populations, ie. those calculated directly
C     by the complete linearization (as old population + corresponding
C     corrections) are not used.
C     Instead, new populations are calculated from the new radiation
C     field basically by lambda iteration, ie.
C     a) evaluation of new radiative rates for explicit transitions
C        (radiative rates in fixed-option transitions are not updated
C        at this step)
C
      if(ifpopr.le.0.or.lte) then
         IF(.NOT.LTE) CALL RATES
C
C     b) depth by depth evaluation of new populations by solving
C        statistical equilibrium equation with previously evaluated
C        rates, together with constraints - precisely as in RESOLV
C
         DO 110 ID=1,ND
            CALL SABOLF(ID)
            CALL STEQEQ(ID,POP,1)
            IF(.NOT.LCHC.and.iter.lt.ielcor) CALL ELCOR(ID)
  110    CONTINUE
          if(iprind.eq.2) call output
c
C     2. second possibility -- populations from linearization corrections
C
      ELSE
         DO 140 ID=1,ND
            DO 120 I=1,NLVEXP
               POPUL(INDLEV(I),ID)=PSY0(NFREQE+INSE+I-1,ID)
  120       CONTINUE
c            CALL SABOLF(ID)
c            CALL REFLEV(ID,0)
            DO 130 I=1,NLEVEL
               if(iifix(iatm(i)).eq.1) goto 130
               II=IIEXP(I)
               IF(II.LE.0) THEN
                  III=ILTREF(I,ID)
                  POPUL(I,ID)=POPUL(III,ID)*SBPSI(I,ID)
               END IF
  130       CONTINUE
  140    CONTINUE
      END IF
      CALL ODFMER
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE ROSSTD(IJ,IJLAST)
C     ============================
C
C     Rosseland mean opacity
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ALIPAR.FOR'
      DIMENSION TAUR(MDEPTH),SUMDPL(MDEPTH)
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/OPMEAN/ABROSD(MDEPTH),ABPLAD(MDEPTH)
      COMMON/TAURSS/TROSS(MDEPTH)
C
C     for IJ > 0 - contribution from the frequency IJ to the Rosseland
C                  opacity integrals
C
      IF(IJ.GT.0) THEN
         IF(IJ.EQ.1) THEN
            DO ID=1,ND
               ABROSD(ID)=0.
               SUMDPL(ID)=0.
            END DO
         END IF
         DO ID=1,ND
            PLAN=XKFB(ID)/XKF1(ID)*W(IJ)
            DPLAN=PLAN/XKF1(ID)*FREQ(IJ)*HKT21(ID)
            ABROSD(ID)=ABROSD(ID)+DPLAN/ABSO1(ID)
            SUMDPL(ID)=SUMDPL(ID)+DPLAN
         END DO
         IF(IJ.EQ.IJLAST) THEN
            DO ID=1,ND
               ABROSD(ID)=SUMDPL(ID)/ABROSD(ID)*DENS1(ID)
            END DO
         END IF
       ELSE   
C
C     for IJ=0 - evaluation of the Rosseland optical depth and
C                the radiative equilibrium division point
C
         ID=1
	 IDR=0
         TAUR(ID)=HALF*DEDM1*ABROSD(ID)*DENS(ID)
         DO ID=2,ND         
            DTAUR=DELDM(ID-1)*(ABROSD(ID)+ABROSD(ID-1))
   	    TAUR(ID)=TAUR(ID-1)+DTAUR
	    IF(TAUR(ID).LE.TAUDIV) IDR=ID
         END DO
C
C        in the last iteration, output of The Rosseland opacity and 
C        optical depth; skip the rest
C
         IF(LFIN) THEN
            DO ID=1,ND
               TROSS(ID)=TAUR(ID)
               WRITE(11,611) ID,DM(ID),TAUR(ID),ABROSD(ID),
     *                       TEMP(ID),ELEC(ID),DENS(ID)
            END DO
  611       FORMAT(I4,2X,1P6E12.4)
            RETURN
         END IF
C
C        determination of the radiative equilibrium parameters
C        REINT and REDIF;
C        REINT=1 for all ID .le. ND-idlst
C        REDIF=1 for Rosseland optical depth > taudiv  (taur.gt.taudiv)
C                typically 0.1 - 0.5;
C              - this value has been empirically shown to yield
C                the best convergence rate
C
	 DO 10 ID=1,ND
            REINT(ID)=UN
            REDIF(ID)=UN
            IF(ID.GT.ND-IDLST) REINT(ID)=0.
            IF(ID.LT.IDR) THEN
               REDIF(ID)=0.
               IF(MOD(NDRE,10).EQ.-1) THEN
                  REDIF(ID)=TAUR(ID)
                ELSE IF(MOD(NDRE,10).EQ.-2) THEN
                  REDIF(ID)=TAUR(ID)*TAUR(ID)
               END IF
            END IF
            IF(NDRE.LE.-10) THEN
               REDIF(ID)=REDIF(ID)/SIG4P/TEFF**4
               REINT(ID)=REINT(ID)/ABPLAD(ID)*DENS(ID)
            END IF
            write(6,602) id,redif(id),reint(id)
   10    CONTINUE
         ID=1
         REDIF(ID)=0.
         IF(MOD(NDRE,10).EQ.-5) REDIF(ID)=UN
         IF(NDRE.LE.-10) THEN 
            REDIF(ID)=REDIF(ID)/SIG4P/TEFF**4
            write(6,602) id,redif(id),reint(id)
         END IF
         NDRE=1
	 WRITE(6,601) IDR,IDR+1,ND-idlst
	 WRITE(10,601) IDR,IDR+1,ND-idlst
  601    FORMAT(1H0,'SCHEME OF RADIATIVE EQUIL. DETERMINED IN RESOLV'/
     *          ' ONLY INTEGRAL EQUATION FOR ID          <= ',I3/
     *          ' BOTH FOR                  ',I5,' <= ID <= ',I3/)
  602    format(i4,1p2e15.3)
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE NEWPOP(ID,POP1)
C     ==========================
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION POP1(MLEVEL),DPOP(MLEVEL),DPMAX(MDEPTH),
     *          SBW(MLEVEL)
C
      DPMAX(ID)=0.
      DO 10 I=1,NLEVEL
         IF(POPUL(I,ID).GT.0.) 
     *      DPOP(I)=(POP1(I)-POPUL(I,ID))/POPUL(I,ID)
         IF(ABS(DPOP(I)).GT.DPMAX(ID)) THEN
            DPMAX(ID)=ABS(DPOP(I))
            IMAX=I
         END IF
         POPUL(I,ID)=POP1(I) 
   10 CONTINUE   
      WRITE(18,601) ITER,ILAM,ID,DPMAX(ID),IMAX
  601 FORMAT(3I5,1PE10.2,I6)
C
C     array of b-factors
C
      DO 110 I=1,NLEVEL
         BFAC(I,ID)=UN      
         SBW(I)=ELEC(ID)*SBF(I)*WOP(I,ID)
  110 CONTINUE      
      DO 120 ION=1,NION
         DO 120 I=NFIRST(ION),NLAST(ION)
             IF(POPUL(NNEXT(ION),ID).GT.0.)
     *       BFAC(I,ID)=POPUL(I,ID)/(POPUL(NNEXT(ION),ID)*SBW(I))
  120 CONTINUE      
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE SWITCH(INITM)
C     ========================
C
C     Procedure for evaluating the collisional-radiative switching
C     parameter lambda(R), denoted here CRSW(ID);
C     Original procedure suggested by Hummer and Voels,
C     Astron. Astrophys. 192, 279, 1988, was modified here by a
C     possibility of considering depth dependent switching parameter CRSW
C
C     Parameters SWPFAC, SWPLIM, SWPINC, and ICRSW are input
C     parameters, having the meaning:
C
C     ICRSW   = 0  - collisional-radiative switching not considered
C             > 0  - collisional-radiative switching is considered
C     SWPFAC  - initial CRSW = SWPFAC * min(collis.rate/rad.rate)
C     SWPLIM  - has the meaning: if CRSW > SWPLIM, then CRSW = 1
C     SWPINC  - CRSW(actual) = CRSW(previous) * SWPINC
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      DIMENSION SWTCH(MDEPTH)
C
      IF(ICRSW.EQ.0) RETURN
      IF(INITM.EQ.0) GO TO 30
C
C     Before the first iteration of complete linearization
C     initialisation of the collisional-radiative switching
C     parameters, as described by Hummer and Voels,
C     Astron. Astrophys. 192, 279, 1988;
C     modified here by considering depth-dependent swiching
C     parameter lambda(R) - denoted here CRSW(ID)
C
      SWMIN=UN
      DO 20 ID=1,ND
	 SWTCH(ID)=UN
         DO 10 ITR=1,NTRANS
            C=COLRAT(ITR,ID)*ELEC(ID)
            IF(RRU(ITR,ID).EQ.0.) GO TO 10
C
C     upward rates
C
            SWU=C/RRU(ITR,ID)
C
C     downward rates
C
            IF(LINE(ITR)) THEN
               SWD=C*EXP(HK*FR0(ITR)/TEMP(ID))/RRD(ITR,ID)
             ELSE
               SWD=C/RRD(ITR,ID)
            END IF
C
C     minimum value
C
            IF(SWU.LT.SWTCH(ID)) SWTCH(ID)=SWU
            IF(SWD.LT.SWTCH(ID)) SWTCH(ID)=SWD
   10    CONTINUE
	 IF(SWTCH(ID).LT.SWMIN) SWMIN=SWTCH(ID)
   20 CONTINUE
      DO 25 ID=1,ND
	 IF(ICRSW.EQ.1) THEN
	    CRSW(ID)=SWMIN*SWPFAC
          ELSE
            CRSW(ID)=SWTCH(ID)*SWPFAC
         END IF
         IF(CRSW(ID).GT.SWPLIM) CRSW(ID)=UN
   25 CONTINUE
      WRITE(6,601) (CRSW(ID),ID=1,ND)
      RETURN
C
C     After second and further iterations of complete linearization
C     evaluation of new collisional-radiative switching parameters
C     by multiplication of the previous ones by a prechosen factor
C     SWPINC
C
   30 CONTINUE
      DO 40 ID=1,ND
         CRSW(ID)=CRSW(ID)*SWPINC
         IF(CRSW(ID).GT.SWPLIM) CRSW(ID)=UN
   40 CONTINUE
      WRITE(6,601) (CRSW(ID),ID=1,ND)
  601 FORMAT(1P8D10.3)
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE SABOLF(ID)
C     =====================
C
C     Saha-Boltzmann factors (SBF)
C     and "upper sums" - sum of Saha-Boltzmann factors for upper, LTE,
C     levels which are not included explicitly (USUM), and derivatives
C     wrt. temperature (T) and electron density (DUSUMN)
C
C     Input: ID  - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (UH=1.5)
      PARAMETER (CMAX=2.154D4,CCON=2.0706D-16)
      COMMON/UPSUMS/DUSUMT(MION),DUSUMN(MION)
C
C     DCHI - approximate lowering of ionization potential for neutrals
C      Actual lowering is DCHI*effective charge, and is considered only
C      if IUPSUM(ION).GT.0
C
      T=TEMP(ID)
      SQT=SQRT(T)
      ANE=ELEC(ID)
      STANE=SQRT(T/ANE)
      XMAX=CMAX*SQRT(STANE)
      TK=BOLK*T
      CON=CCON/T/SQT
C
C     Saha-Boltzmann factors
C
      DO 50 ION=1,NION
         QZ=IZ(ION)
         CFN=CON/G(NNEXT(ION))
         DCH=0.
         IUPS=IUPSUM(ION)
         SSBF=0.
         DSSBFT=0.
         USUM(ION)=0.
         DUSUMT(ION)=0.
         DUSUMN(ION)=0.
         nlst=nlast(ion)
         if(ifwop(nlst).ge.0) then
             nl1up=nquant(nlst)+1
          else
             nl1up=nquant(nlst)
         end if
         DO 10 II=NFIRST(ION),NLAST(ION)
            if(ifwop(ii).lt.0) then
               E=EH*QZ*QZ/TK
               SUM=0.
               DO 5 J=nl1up,NLMX
                  XJ=J
                  XI=J*J
                  X=E/XI
                  FI=XI*EXP(X)*WNHINT(J,ID)
                  SUM=SUM+FI
    5          CONTINUE
               g(ii)=sum*two
               gmer(imrg(ii),id)=g(ii)
            end if
            SB=CFN*G(II)*EXP(ENION(II)/TK)
            SBF(II)=SB
            SSBF=SSBF+SB
            DSBF(II)=-(UH+ENION(II)/TK)/T
            DSSBFT=DSSBFT+SB*DSBF(II)
   10    CONTINUE
C
C     Upper sums
C
         if(ifwop(nlst).lt.0) go to 50
         if(iups.eq.0) then
C
C     1. More exact approach - using (exact) partition functions
C
         IAT=NUMAT(IATM(NFIRST(ION)))
         XMX=XMAX*SQRT(QZ)
         CALL PARTF(IAT,IZ(ION),T,ANE,XMX,U,DUT,DUN)
         EE=ENION(NFIRST(ION))/TK
         CFE=CFN*EXP(EE)
         USUM(ION)=CFE*U-SSBF
         DUSUMT(ION)=CFE*(DUT-U/T*(UH+EE))-DSSBFT
         DUSUMN(ION)=CFE*DUN
         IF(USUM(ION).LT.0.) THEN 
            USUM(ION)=0.
            DUSUMT(ION)=0.
            DUSUMN(ION)=0.
         END IF
C
C     2. Approximate approach - summation over fixed number of upper
C        levels, assumed hydrogenic (ie. their ionization energy and
C        statistical weight hydrogenic)
C
         else if(iups.gt.0) then
         SUM=0.
         DSUM=0.
         E=EH*QZ*QZ/TK
         DO 30 J=NQUANT(NLAST(ION))+1,IUPS
            XI=J*J
            X=E/XI
            FI=XI*EXP(X)
            SUM=SUM+FI
            DSUM=DSUM-FI*(UH+X)/T
   30    CONTINUE
         USUM(ION)=SUM*CON*TWO
         DUSUMT(ION)=DSUM*CON*TWO
         DUSUMN(ION)=0.
C
c        3. occupation probability form
c
         else 
         SUM=0.
         DSUM=0.
         E=EH*QZ*QZ/TK
         DO 40 J=NQUANT(NLAST(ION))+1,NLMX
            XJ=J
            XI=J*J
            X=E/XI
            FI=XI*EXP(X)*WNHINT(J,ID)
            SUM=SUM+FI
            DSUM=DSUM-FI*(UH+X)/T
   40    CONTINUE
         USUM(ION)=SUM*CON*TWO
         DUSUMT(ION)=DSUM*CON*TWO
         DUSUMN(ION)=0.
         end if
   50 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C
C

      SUBROUTINE OPACF1(IJ)
C     =====================
C
C     Absorption, emission, and scattering coefficients
C     at frequency IJ and for all depths
C
C     Input: IJ   opacity and emissivity is calculated for the
C                 frequency points with index IJ
C     Output: ABSO1 -  array of absorption coefficient
C             EMIS1 -  array of emission coefficient
C             SCAT1 -  array of scattering coefficient (all scattering
C                       mechanisms except electron scattering)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (CFF1=1.3727D-25)
      COMMON/OPACAD/ABAD,EMAD,SCAD,DAT,DAN,DET,DEN,DST,DSN,DDN(MLEVEL)
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
C
C     initialize
c
      DO ID=1,ND
         ABSO1(ID)=ELSCAT(ID)
         EMIS1(ID)=0.
         SCAT1(ID)=ELSCAT(ID)
      END DO
C
C     basic frequency- and depth-dependent quantities
C
      FR=FREQ(IJ)
      FRINV=UN/FR
      FR3INV=FRINV*FRINV*FRINV
      DO 10 ID=1,ND
         XKF(ID)=EXP(-HKT1(ID)*FR)
         XKF1(ID)=UN-XKF(ID)
         XKFB(ID)=XKF(ID)*BNUE(IJ)
   10 CONTINUE
C
C ********  1. bound-free contribution
C
      DO 30 IBF=1,NTRANC
         ITR=ITRBF(IBF)
         SG=CROSS(IBF,IJ)
         IF(SG.LE.0.) GO TO 30
         II=ILOW(ITR)
         JJ=IUP(ITR)
         IZZ=IZ(IEL(II))
         IMER=IMRG(II)
         DO 20 ID=1,ND
            SGD=SG
            IF(MCDW(ITR).GT.0) THEN
               CALL DWNFR1(FR,FR0(ITR),ID,IZZ,DW1)
               DWF1(MCDW(ITR),ID)=DW1
               SGD=SG*DW1
            END IF
            IF(IFWOP(II).LT.0) THEN
               CALL SGMER1(FRINV,FR3INV,IMER,ID,SGME1)
               SGMG(IMER,ID)=SGME1
               SGD=SGME1
            END IF
            EMISBF=SGD*EMTRA(ITR,ID)
            ABSO1(ID)=ABSO1(ID)+SGD*ABTRA(ITR,ID)
            EMIS1(ID)=EMIS1(ID)+EMISBF
   20    CONTINUE
   30 CONTINUE
C
C ******** 2. free-free contribution
C
      DO 40 ION=1,NION
         IT=ITRA(NNEXT(ION),NNEXT(ION))
C
C        hydrogenic with Gaunt factor = 1
C
         IF(IT.EQ.1) THEN
            DO ID=1,ND
               SF1=SFF3(ION,ID)*FR3INV
               SF2=SFF2(ION,ID)
               IF(FR.LT.FF(ION)) SF2=UN/XKF(ID)
               ABSOFF=SF1*SF2
               ABSO1(ID)=ABSO1(ID)+ABSOFF
               EMIS1(ID)=EMIS1(ID)+ABSOFF
            END DO
C
C         hydrogenic with exact Gaunt factor 
C
          ELSE IF(IT.EQ.2) THEN
            DO ID=1,ND
               SF1=SFF3(ION,ID)*FR3INV
               SF2=SFF2(ION,ID)
               IF(FR.LT.FF(ION)) SF2=UN/XKF(ID)
               CALL GFREE1(ID,FR,CHARG2(ION),GFR)
               SF2=SF2-UN+GFR
               ABSOFF=SF1*SF2
               ABSO1(ID)=ABSO1(ID)+ABSOFF
               EMIS1(ID)=EMIS1(ID)+ABSOFF
            END DO
C
C         H minus free-free opacity
C
          ELSE IF(IT.EQ.3) THEN
            DO ID=1,ND
               ABSOFF=(CFF1+CFFT(ID)*FRINV)*CFFN(ID)*FRINV
               ABSO1(ID)=ABSO1(ID)+ABSOFF
               EMIS1(ID)=EMIS1(ID)+ABSOFF
            END DO
C
C         special evaluation of the cross-section
C
          ELSE IF(IT.LT.0) THEN
            DO ID=1,ND
               ABSOFF=FFCROS(ION,IT,TEMP(ID),FR)*
     *                POPUL(NNEXT(ION),ID)*ELEC(ID)
               ABSO1(ID)=ABSO1(ID)+ABSOFF
               EMIS1(ID)=EMIS1(ID)+ABSOFF
            END DO
         END IF
   40 CONTINUE
C
C     ********  3. - additional continuum opacity (OPADD)
C
      IF(IOPADD.NE.0) THEN
         DO ID=1,ND
            CALL OPADD(0,ICALL,IJ,ID)
            IF(ICALL.EQ.1) ICALL=0
            ABSO1(ID)=ABSO1(ID)+ABAD
            EMIS1(ID)=EMIS1(ID)+EMAD
            SCAT1(ID)=SCAT1(ID)+SCAD
         END DO
      END IF
C
C ********  4. - opacity and emissivity in lines
C
      IF(IJLIN(IJ).GT.0) THEN
C
C     the "primary" line at the given frequency
C
         ITR=IJLIN(IJ)
         DO 50 ID=1,ND
            SG=PRFLIN(IJ,ID)
            ABSO1(ID)=ABSO1(ID)+SG*ABTRA(ITR,ID)
            EMIS1(ID)=EMIS1(ID)+SG*EMTRA(ITR,ID)
   50    CONTINUE
      ENDIF
      IF(NLINES(IJ).LE.0) GO TO 200
C
C     the "overlapping" lines at the given frequency
C
      DO 100 ILINT=1,NLINES(IJ)
         ITR=ITRLIN(ILINT,IJ)
         IJ0=IFR0(ITR)
         DO 60 IJT=IJ0,IFR1(ITR)
            IF(FREQ(IJT).LE.FR) THEN
               IJ0=IJT
               GO TO 70
            END IF
   60    CONTINUE
   70    IJ1=IJ0-1
         A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))
         A2=UN-A1
         DO 80 ID=1,ND
            SG=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
            ABSO1(ID)=ABSO1(ID)+SG*ABTRA(ITR,ID)
            EMIS1(ID)=EMIS1(ID)+SG*EMTRA(ITR,ID)
   80    CONTINUE
  100 CONTINUE
  200 CONTINUE
C
C     ----------------------------
C     total opacity and emissivity
C     ----------------------------
C
      DO ID=1,ND
         ABSO1(ID)=ABSO1(ID)-EMIS1(ID)*XKF(ID)
         EMIS1(ID)=EMIS1(ID)*XKFB(ID)
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE STEQEQ(ID,POP1,MODE)
C     ===============================
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/POPSTR/POPP(MLEVEL),
     *       POPP1(MLEVEL),POPP2(MLEVEL),POPP3(MLEVEL)  
      COMMON/PPAPAR/IPOPST(MATOM),NTERST,ITERST,
     *              IACPPP,IACPP0,IACPPD,LACPPP
      DIMENSION A(MLEVEL,MLEVEL),B(MLEVEL),POP0(MLEVEL)
      DIMENSION AP(MLEVEL,MLEVEL),BP(MLEVEL),
     *          POP1(MLEVEL),SBW(MLEVEL)
C
C     evaluation of the global rate matrix
C
      CALL SABOLF(ID)
      CALL RATMAT(ID,IIFOR,1,A,B)
  600 format(' i',2i4)
  601 format(1p15d9.1)
  602 format('      non-zero element, i,j:',2i5,1pe9.1)
C
C     new populations - solution of the rate equations
C
C     a) either by inverting the global rate matrix (if IRSPLT=0)
C
      IF(IRSPLT.EQ.0) THEN
         CALL LINEQS(A,B,POP0,NLVFOR,MLEVEL)
C
C     b) or by inverting several partial rate matrices for the
C        individual chemical species
C
       ELSE
         DO 50 IAT=1,NATOM
            IF(IIFIX(IAT).EQ.1) GO TO 50
            N1=N0A(IAT)
            NK=NKA(IAT)
            N1=IIFOR(N1)
            NK=IIFOR(NK)
            IF(N1.LE.0) THEN
               DO 1 I=N0A(IAT),NKA(IAT)
                  N1=IIFOR(I)
                  IF(IIFOR(I).GT.0) GO TO 2
    1          CONTINUE
    2          CONTINUE
            END IF
            IF(N1.LE.0) GO TO 50
            NLP=NK-N1+1
            DO 20 I=N1,NK
               DO 10 J=N1,NK
                  AP(I-N1+1,J-N1+1)=A(I,J)
   10          CONTINUE
               BP(I-N1+1)=B(I)
   20       CONTINUE
            CALL LINEQS(AP,BP,POPP,NLP,MLEVEL)
            DO 30 I=N1,NK
                POP0(I)=POPP(I-N1+1)
   30       CONTINUE
   50    CONTINUE
      END IF
C
C     array of new populations
C
      DO 60 I=1,NLEVEL
         SBW(I)=ELEC(ID)*SBF(I)*WOP(I,ID)
         II=IIFOR(I)
         IF(II.GT.0) THEN
            POP1(I)=POP0(II)
          ELSE IF(II.EQ.0) THEN
            POP1(I)=POPUL(I,ID)
          ELSE
            III=IIFOR(ILTREF(I,ID))
            POP1(I)=SBPSI(I,ID)*POP0(III) 
         END IF
   60 CONTINUE
C
C     set up the parameter IPZERO indicating that a population is "small"
C     and will subsequently be set to zero
C
      DO 70 IAT=1,NATOM
         POPM=0.
         DO I=N0A(IAT),NKA(IAT)
            IF(POP1(I).GT.POPM) POPM=POP1(I)
            IPZERO(I,ID)=0
         END DO
         DO I=N0A(IAT),NKA(IAT)
            IF(POP1(I)/POPM.LT.POPZER) THEN
               POP1(I)=0.
               IPZERO(I,ID)=1
            END IF
         END DO
   70 CONTINUE
C
C     if required (MODE=1), set up the global array POPUL
C          
      IF(MODE.NE.1) RETURN
      DO 100 I=1,NLEVEL
  100    POPUL(I,ID)=POP1(I)
C
C     array of b-factors
C
      DO 110 I=1,NLEVEL
         BFAC(I,ID)=UN      
  110 CONTINUE      
      DO 120 ION=1,NION
         DO 120 I=NFIRST(ION),NLAST(ION)
             IF(POPUL(NNEXT(ION),ID).GT.0..AND.IPZERO(I,ID).EQ.0)
     *       BFAC(I,ID)=POPUL(I,ID)/(POPUL(NNEXT(ION),ID)*SBW(I))
  120 CONTINUE  
c
c     test output - to be removed !!!!
c      call ratmat(ID,IIEXP,0,a,b) 
c      call lineqs(a,b,popp1,nlvexp,mlevel)   
c      do i=1,nlevel
c         II=IIEXP(I)
c         IF(II.LE.0) THEN
c            III=iiexp(ILTREF(I,ID))
c            popp2(i)=popp1(III)*SBPSI(I,ID)
c          else
c            popp2(i)=popp1(ii)
c         END IF
c      end do
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE STEQ3
C     ================
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      COMMON/POPSTR/POPP(MLEVEL),
     *       POPP1(MLEVEL),POPP2(MLEVEL),POPP3(MLEVEL)  
      COMMON/PPAPAR/IPOPST(MATOM),NTERST,ITERST,
     *              IACPPP,IACPP0,IACPPD,LACPPP
      DIMENSION BB(MLEVE3,MLEVE3),VL(MLEVE3),POP0(MLEVE3)
      DIMENSION AA(MLEVE3,MLEVE3),CC(MLEVE3,MLEVE3),
     *          D(MLEVE3,MLEVE3,MDEPTH),ANU(MLEVE3,MDEPTH),
     *          POP1(MLEVE3),SBW(MLEVE3),DPMAX(MDEPTH),
     *          DPOP(MLEVE3)
C        
      DO 10 ID=1,ND
         CALL SABOLF(ID)
         CALL RATMAT(ID,IIFOR,1,BB,VL)
         CALL RATOFF(ID,IIFOR,AA,CC)
         NLP=NLVFOR
C              
         IF(ID.EQ.1) THEN
            CALL MATINV(BB,NLP,MLEVEL)
            DO I=1,NLP
               SUML=0.
               DO J=1,NLP
                  SUML=SUML+BB(I,J)*VL(J)
                  SUMD=0.
                  DO K=1,NLP
                     SUMD=SUMD+BB(I,K)*CC(K,J)
                  END DO
                  D(I,J,ID)=SUMD
               END DO
               ANU(I,ID)=SUML
            END DO
          ELSE
            DO I=1,NLP
               SUMA=0.
               DO J=1,NLP
                  SUMA=SUMA+AA(I,J)*ANU(J,ID-1)
                  SUMB=0.
                  DO K=1,NLP
                     SUMB=SUMB+AA(I,K)*D(K,J,ID-1)
                  END DO
                  BB(I,J)=BB(I,J)-SUMB
               END DO
               VL(I)=VL(I)+SUMA
            END DO
            CALL MATINV(BB,NLP,MLEVEL)
            DO I=1,NLP
               SUML=0.
               DO J=1,NLP
                  SUML=SUML+BB(I,J)*VL(J)
                  SUMD=0.
                  DO K=1,NLP
                     SUMD=SUMD+BB(I,K)*CC(K,J)
                  END DO
                  D(I,J,ID)=SUMD
               END DO
               ANU(I,ID)=SUML
            END DO
         END IF
   10 CONTINUE
C             
      DO 200 ID=ND,1,-1
         IF(ID.EQ.ND) THEN
            DO I=1,NLP
               POP0(I)=ANU(I,ID)
            END DO
          ELSE
            DO I=1,NLP
               SUMD=0.
               DO J=1,NLP
                  SUMD=SUMD+D(I,J,ID)*ANU(J,ID+1)
               END DO
               ANU(I,ID)=ANU(I,ID)+SUMD
               POP0(I)=ANU(I,ID)
            END DO
         END IF
C
C        array of new populations
C
         DPMAX(ID)=0.
         CALL SABOLF(ID)
         DO 60 I=1,NLEVEL
            SBW(I)=ELEC(ID)*SBF(I)*WOP(I,ID)
            II=IIFOR(I)
            IF(II.GT.0) THEN
               POP1(I)=POP0(II)
             ELSE IF(II.EQ.0) THEN
               POP1(I)=POPUL(I,ID)
             ELSE
               III=IIFOR(ILTREF(I,ID))
               POP1(I)=SBPSI(I,ID)*POP0(III) 
            END IF
            IF(POPUL(I,ID).GT.0.) 
     *         DPOP(I)=(POP1(I)-POPUL(I,ID))/POPUL(I,ID)
            IF(ABS(DPOP(I)).GT.DPMAX(ID)) THEN
               DPMAX(ID)=ABS(DPOP(I))
               IMAX=I
            END IF
            POPUL(I,ID)=POP1(I)
   60    CONTINUE
         WRITE(18,601) ITER,ILAM,ID,DPMAX(ID),IMAX
  601    FORMAT(3I5,1PE10.2,I6)
C
C        array of b-factors
C
         DO 110 I=1,NLEVEL
            BFAC(I,ID)=UN      
  110    CONTINUE      
         DO 120 ION=1,NION
            DO 120 I=NFIRST(ION),NLAST(ION)
                IF(POPUL(NNEXT(ION),ID).GT.0.)
     *          BFAC(I,ID)=POPUL(I,ID)/(POPUL(NNEXT(ION),ID)*SBW(I))
  120    CONTINUE
  200 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE RATMAT(ID,IICAL,IMODE,A,B)
C     =====================================
C
C     Evaluation of the rate matrix
C
C     More specifically, the set of statistical equilibrium + constraint
C     equations (see further) is represented as
C
C         A  *  vector of populations  =  B
C
C     (see Mihalas, 1978) pp.138-139 for details
C
C     Constraint equations:
C     1. the rows corresponding to the last explicit level of each atom
C        (ie. a one-level ion representing the highest ionization degree
C        of the given atom) are the abundance definition equations
C     2. the row corresponding to H+ (ionized hydrogen - ie.protons) is
C        a) either charge conservation equation - if LCHC = true;
C           in this case the formulation is equivalent to that of
C           Mihalas;
C        b) or the total particle number conservation equation
C           if LCHC = false ;
C
C     Input: ID - depth index
C     Input transmitted by COMMON blocks:
C            RRU - array of upward radiative rates in all transitions
C            RRD - array of downward radiative rates in all transitions
C            COLRAT - array of collisional rates (only upward) in all
C                     transitions
C            SBF - array of Saha-Boltzmann factors
C            USUM - array of upper sums
C
C     Output: A - rate matrix
C             B - the right-hand-side vector
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/CHARFX/QFIX(MDEPTH)
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      DIMENSION A(MLEVEL,MLEVEL),B(MLEVEL)
      DIMENSION AIJ(MTRANS),AJI(MTRANS)
      DIMENSION SBW(MLEVEL),IICAL(MLEVEL)
      DIMENSION LLTE(MLEVEL)
      DATA ICOMP /0/
C
      T=TEMP(ID)
      ANE=ELEC(ID)
      HKT=HK/T
      TK=HKT/H
      DO 10 I=1,NLEVEL
         B(I)=0.
         SBW(I)=ANE*SBF(I)*WOP(I,ID)
         ILT=ILTION(IEL(I))
         LLT=ILT.EQ.1.AND.IMODE.EQ.0
         LLTE(I)=LLT.OR.LTE.OR.ILTLEV(I).GE.1.OR.ILT.GE.2
         LLTE(I)=LLTE(I).OR.ID.GE.IDLTE
         DO 10 J=1,NLEVEL
            A(J,I)=0.
   10 CONTINUE
C
C     determine reference levels
C
      CALL REFLEV(ID,IMODE)
C
C ******* First part - the rows corresponding to true statistical
C                      equilibrium equations
C                      i.e. NLTE rate equations
C
C     1) simple expression in the case of LTE
C
      DO 30 IAT=1,NATOM
         IF(IIFIX(IAT).EQ.1) GO TO 30
         NREFI=NREF(IAT)
         DO 20 I=N0A(IAT),NKA(IAT)
            II=IICAL(I)
            IF(I.EQ.NREFI.OR.II.LE.0.OR..NOT.LLTE(I)) GO TO 20
               A(II,II)=UN
               N=IICAL(ILTREF(I,ID))
               A(II,N)=-SBPSI(I,ID)
   20    CONTINUE
   30 CONTINUE
C
C     2) NLTE rate equation
C
       IF(.NOT.LTE) THEN
         DO 40 ITR=1,NTRANS
            I=ILOW(ITR)
            IF(IIFIX(IATM(I)).EQ.1) GO TO 40
            J=IUP(ITR)
            NKE=NNEXT(IEL(I))
            IF(LINE(ITR)) THEN
C
C              bound-bound contribution 
C
               AIJ(ITR)=(COLRAT(ITR,ID)*ANE+RRD(ITR,ID))*
     *                  G(I)/G(J)*EXP(HKT*FR0(ITR))*WOP(I,ID)
             ELSE
C
C              Bound-free contribution
C              Quantity CORR is a factor which allows the bound-free
C              transition to end at an excited state
C
               CORR=UN
               IF(NKE.NE.J) CORR=G(NKE)/G(J)*
     *            EXP((ENION(NKE)-ENION(J))*TK)
               AIJ(ITR)=(COLRAT(ITR,ID)*ANE+RRD(ITR,ID))*SBW(I)*CORR
            END IF
            AJI(ITR)=(COLRAT(ITR,ID)*ANE+RRU(ITR,ID))*WOP(J,ID)
   40    CONTINUE
C
C      Elements of the rate matrix
C
         DO 41 ITR=1,NTRANS
            I=ILOW(ITR)
            IF(IIFIX(IATM(I)).EQ.1) GO TO 41
            NREFI=NREF(IATM(I))
            J=IUP(ITR)
            II=IICAL(I)
            JJ=IICAL(J)
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 41
            IF(I.NE.NREFI.AND.II.GT.0.AND..NOT.LLTE(I)) THEN
               A(II,II)=A(II,II)+AJI(ITR)
C              IF(IPZERO(J,ID).EQ.0) THEN
               IF(JJ.GT.0) THEN
                  A(II,JJ)=A(II,JJ)-AIJ(ITR)
                ELSE
                  JJJ=IICAL(ILTREF(J,ID))
                  A(II,JJJ)=A(II,JJJ)-AIJ(ITR)*SBPSI(J,ID)
c              END IF
               END IF
            END IF
            IF(J.NE.NREFI.AND.JJ.GT.0.AND..NOT.LLTE(J)) THEN
               A(JJ,JJ)=A(JJ,JJ)+AIJ(ITR)
C              IF(IPZERO(I,ID).EQ.0) THEN
               IF(II.GT.0) THEN
                  A(JJ,II)=A(JJ,II)-AJI(ITR)
                ELSE
                  III=IICAL(ILTREF(I,ID))
                  A(JJ,III)=A(JJ,III)-AJI(ITR)*SBPSI(I,ID)
               END IF
C              END IF
            END IF
   41    CONTINUE
         END IF
C
C        reset the rate matrix elements for "small" populations
C
         DO 42 I=1,NLEVEL
            II=IICAL(I)
            IF(II.LE.0) GO TO 42
            IF(IPZERO(I,ID).EQ.0) GO TO 42
            DO 43 J=1,NLEVEL
               A(II,J)=0.
   43       CONTINUE
            A(II,II)=1.
   42    CONTINUE
C
C ******** Second part - abundance definition equations
C
      DO 100 IAT=1,NATOM
         IF(IAT.EQ.IATREF.AND.IRSPLT.EQ.0) GO TO 100
         IF(IIFIX(IAT).EQ.1) GO TO 100
         NREFII=IICAL(NREF(IAT))
         IF(NREFII.LE.0) GO TO 100
         DO 80 I=N0A(IAT),NKA(IAT)
            IL=ILK(I)
            II=IICAL(I)
            IF(II.GT.0) THEN
               A(NREFII,II)=A(NREFII,II)+UN
               IF(IL.NE.0) A(NREFII,II)=A(NREFII,II)+ANE*USUM(IL)
             ELSE
               III=IICAL(ILTREF(I,ID))
               A(NREFII,III)=A(NREFII,III)+SBPSI(I,ID)
            END IF
   80    CONTINUE
         IF(IRSPLT.EQ.0) THEN
            DO 90 I=N0A(IATREF),NKA(IATREF)
               IL=ILK(I)
               II=IICAL(I)
               IF(II.GT.0) THEN
                  A(NREFII,II)=A(NREFII,II)-ABUND(IAT,ID)
                  IF(IL.GT.0) A(NREFII,II)=A(NREFII,II)-
     *                        ABUND(IAT,ID)*ANE*USUM(IL)
                ELSE
                  III=IICAL(ILTREF(I,ID))
                  A(NREFII,III)=A(NREFII,III)-SBPSI(I,ID)*ABUND(IAT,ID)
               END IF
   90       CONTINUE
          ELSE
            B(NREFII)=B(NREFII)+
     *                DENS(ID)/WMM(ID)/YTOT(ID)*ABUND(IAT,ID)
         END IF
  100 CONTINUE
C
      IF(IRSPLT.GT.0) RETURN
C
C ******** Third part:
C          either charge conservation equation - if LCHC=true
C
      NKH=IICAL(NREF(IATREF))
      IF(LCHC) THEN
C
C        here STATE determines Q, which is total charge due to
C         non-explicit atoms, relative to total hydrogen number
C         density; and QM - contribution of the charge of H- if this
C         is not considered as an explicit ion
C
         CALL STATE(2,ID,T,ANE)
         DO 120 IAT=1,NATOM
            IF(IIFIX(IAT).EQ.1) GO TO 120
            DO 110 I=N0A(IAT),NKA(IAT)
               IL=ILK(I)
               II=IICAL(I)
               CH=IZ(IEL(I))-1
               IF(IL.GT.0) CH=IZ(IL)+(IZ(IL)-1)*USUM(IL)*ANE
               IF(II.GT.0) THEN
                  A(NKH,II)=A(NKH,II)+CH
                ELSE
                  III=IICAL(ILTREF(I,ID))
                  A(NKH,III)=A(NKH,III)+CH*SBPSI(I,ID)
               END IF
  110       CONTINUE
  120    CONTINUE
         B(NKH)=B(NKH)+ANE-QFIX(ID)-
     *          Q*DENS(ID)/WMM(ID)/YTOT(ID)*ABUND(IATREF,ID)
       ELSE
C
C ********  or particle conservation equation, if LCHC=false
C
         DO 140 I=N0A(IATREF),NKA(IATREF)
            IL=ILK(I)
            II=IICAL(I)
            IF(II.GT.0) THEN
               A(NKH,II)=A(NKH,II)+UN
               IF(IL.GT.0) A(NKH,II)=A(NKH,II)+ANE*USUM(IL)
             ELSE
               III=IICAL(ILTREF(I,ID))
               A(NKH,III)=A(NKH,III)+SBPSI(I,ID)
            END IF
  140    CONTINUE
         B(NKH)=B(NKH)+DENS(ID)/WMM(ID)/YTOT(ID)*ABUND(IATREF,ID)
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE RATOFF(ID,IICAL,AOFF,COFF)  
C     =====================================
C
C     Evaluation of the "off-diagonal" block rate matrix 
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/CHARFX/QFIX(MDEPTH)
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      DIMENSION AOFF(MLEVE3,MLEVE3),COFF(MLEVE3,MLEVE3)
      DIMENSION AIJ(MTRANS),CIJ(MTRANS)
      DIMENSION SBW(MLEVEL),IICAL(MLEVEL)
      DIMENSION LLTE(MLEVEL)
      DATA ICOMP /0/
C
      T=TEMP(ID)
      ANE=ELEC(ID)
      HKT=HK/T
      TK=HKT/H
      DO 10 I=1,NLEVEL
         ILT=ILTION(IEL(I))
         LLT=ILT.EQ.1
         LLTE(I)=LLT.OR.LTE.OR.ILTLEV(I).GE.1.OR.ILT.GE.2
         LLTE(I)=LLTE(I).OR.ID.GE.IDLTE
         DO 10 J=1,NLEVEL
            AOFF(J,I)=0.
            COFF(J,I)=0.
   10 CONTINUE
C
         DO 40 ITR=1,NTRANS
            I=ILOW(ITR)
            J=IUP(ITR)
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 40
            AIJ(ITR)=0.
            CIJ(ITR)=0.
            IF(IIFIX(IATM(I)).EQ.1) GO TO 40
            NKE=NNEXT(IEL(I))
            IF(ID.GT.1) THEN
               AIJ(ITR)=RDDM(ITR,ID)*EMTRA(ITR,ID-1)/POPUL(J,ID-1)
            END IF
            IF(ID.LT.ND) THEN
               CIJ(ITR)=RDDP(ITR,ID)*EMTRA(ITR,ID+1)/POPUL(J,ID+1)
            END IF
   40    CONTINUE
C
C      Elements of the rate matrix
C
         DO 41 ITR=1,NTRANS
            I=ILOW(ITR)
            J=IUP(ITR)
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 41
            IF(IIFIX(IATM(I)).EQ.1) GO TO 41
            NREFI=NREF(IATM(I))
            II=IICAL(I)
            JJ=IICAL(J)
            IF(I.NE.NREFI.AND.II.GT.0.AND..NOT.LLTE(I)) THEN
               IF(JJ.GT.0) THEN
                  AOFF(II,JJ)=AOFF(II,JJ)-AIJ(ITR)
                  COFF(II,JJ)=COFF(II,JJ)-CIJ(ITR)
                ELSE
                  JJJ=IICAL(ILTREF(J,ID))
                  AOFF(II,JJJ)=AOFF(II,JJJ)-AIJ(ITR)*SBPSI(J,ID)
                  COFF(II,JJJ)=COFF(II,JJJ)-CIJ(ITR)*SBPSI(J,ID)
               END IF
            END IF
            IF(J.NE.NREFI.AND.JJ.GT.0.AND..NOT.LLTE(J)) THEN
               AOFF(JJ,JJ)=AOFF(JJ,JJ)+AIJ(ITR)
               COFF(JJ,JJ)=COFF(JJ,JJ)+CIJ(ITR)
            END IF
   41    CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE ELCOR(ID)
C     ====================
C
C     Procedure for a reevaluation of the electron number density
C     from the charge conservation equation in the formal solution
C     step (RESOLV)
C     This procedure is called only if LCHC=false, ie. if the charge
C     conservation equation is not part of the rate matrix
C
C     Input:  ID - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      COMMON/CHARFX/QFIX(MDEPTH)
      DIMENSION AJ(MLEVEL)
C
      T=TEMP(ID)
      ANE=ELEC(ID)
C
C     basic iteration loop for solving simultaneously a non-linear set
C     of statistical equilibrium, charge conservation, and particle
C     conservation equations
C
      KKK=0
    1 KKK=KKK+1
      AN=DENS(ID)/WMM(ID)+ANE
C
C     here STATE determines Q, which is total charge due to
C     non-explicit atoms, relative to total hydrogen number
C     density; and QM - contribution of the charge of H- if this
C     is not considered as an explicit ion
C
      CALL STATE(2,ID,T,ANE)
C
C     first set up charge conservation in the form
C
C       AJ  *  vector of populations  =  electron density
C
C     Note: vector AJ is exactly the same as the NKH-th row of rate
C           matrix with option LCHC=true.
C           NKH is NKA(IATH), ie. the level index corresponding to
C           ionized hydrogen
C
      RHS=QFIX(ID)+Q*(AN-ANE)/YTOT(ID)*ABUND(IATREF,ID)
      DO 30 IAT=1,NATOM
         IF(IIFIX(IAT).EQ.1) GO TO 30
         DO 20 I=N0A(IAT),NKA(IAT)
            IL=ILK(I)
            CH=IZ(IEL(I))-1
            IF(IL.GT.0) CH=IZ(IL)+(IZ(IL)-1)*USUM(IL)*ANE
            IF(IIFOR(I).NE.0) RHS=RHS+CH*POPUL(I,ID)
   20    CONTINUE
   30 CONTINUE
C
C     new electron density 
C
      RHS=HALF*(ANE+RHS)
      ELEC(ID)=RHS
      DENS(ID)=WMM(ID)*(AN-ELEC(ID))
      RELANE=(RHS-ANE)/ANE
      ANE=RHS
C
C     second part of the iteration loop - recalculation of all
C     populations with new electron density
C
      CALL STEQEQ(ID,POP,1)
C
C     convergence criterion for electron density
C
      IF(ABS(RELANE).LE.1.D-6) THEN
         CALL WNSTOR(ID)
         RETURN
      ENDIF
C
C     if convergence is not achieved
C
      IF(KKK.LT.10) GO TO 1
      WRITE(6,601) ID,RELANE
      WRITE(10,601) ID,RELANE
  601 FORMAT('0 SLOW CONVERGENCE OF ELCOR  ID =',I4,'  REL =',1PD10.3/)
      CALL WNSTOR(ID)
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE REFLEV(ID,IMODE)
C     ===========================
C
C     determination of the global reference level, and
C     determination of the LTE reference levels and corresponding
C     quantities
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
C
C     determination of the reference level (if required)
C
      IF(MODREF.GE.1) THEN
         IF(IMODE.EQ.1.AND.(ITER.LE.1.OR.KANT(ITER).EQ.0)) THEN
            DO 20 IAT=1,NATOM
               PMAX=0.
               IREF=0
               DO 10 I=N0A(IAT),NKA(IAT)
                  IF(POPUL(I,ID).LT.PMAX) GO TO 10
                  PMAX=POPUL(I,ID)
                  IREF=I
   10          CONTINUE
               IF(IREF.NE.NKA(IAT).AND.IREF.NE.NFIRST(IEL(IREF))) 
     *            IREF=NFIRST(IEL(IREF))
               IF(MODREF.EQ.2.AND.IREF.LT.NFIRST(IEL(NKA(IAT))))
     *            IREF=NFIRST(IEL(NKA(IAT)))
               NREF(IAT)=IREF
               NREFS(IAT,ID)=NREF(IAT)
   20       CONTINUE
          ELSE
            DO 30 IAT=1,NATOM
               NREF(IAT)=NREFS(IAT,ID)
   30       CONTINUE
         END IF
       ELSE
         DO IAT=1,NATOM
            NREFS(IAT,ID)=NREF(IAT)
         END DO
      END IF
C
C     determination of the LTE reference levels and corresponding
C     quantities
C
      ELEC1(ID)=UN/ELEC(ID)
      DO 100 IAT=1,NATOM
         IFSUP=0
         IREF=NREF(IAT)
         DO 50 I=N0A(IAT),NKA(IAT)
c            IF(ILTLEV(I).GE.1.OR.ILTION(IEL(I)).GE.1) BFAC(I,ID)=UN
C
C           generalized LTE reference level formalism
C
            IF(IABS(IMODL(I)).EQ.1.OR.IABS(IMODL(I)).EQ.2) THEN
               IN=NNEXT(IEL(I))
               IF(I.LT.IREF.OR.POPUL(I,ID).LT.POPUL(IN,ID)) THEN
                  ILTREF(I,ID)=IN
                  SBPSI(I,ID)=SBF(I)*ELEC(ID)*WOP(I,ID)*BFAC(I,ID)
                  DSBPST(I,ID)=DSBF(I)
                  DSBPSN(I,ID)=ELEC1(ID)
                ELSE IF(I.GT.IREF) THEN
                  I1=NFIRST(IEL(I))
                  ILTREF(I,ID)=I1
                  SBPSI(I,ID)=SBF(I)*WOP(I,ID)*BFAC(I,ID)/
     *                        (SBF(I1)*WOP(I1,ID)*BFAC(I1,ID))
                  DSBPST(I,ID)=DSBF(I)-DSBF(I1)
                  DSBPSN(I,ID)=0.
               END IF
C
C            original scheme
C
             ELSE IF(IABS(IMODL(I)).EQ.3.OR.IMODL(I).EQ.0) THEN
               ILTREF(I,ID)=NNEXT(IEL(I))
               SBPSI(I,ID)=SBF(I)*ELEC(ID)*WOP(I,ID)*BFAC(I,ID)
               DSBPST(I,ID)=DSBF(I)
               DSBPSN(I,ID)=ELEC1(ID)
             ELSE IF(IABS(IMODL(I)).EQ.5) THEN
               IFSUP=1
            END IF
   50    CONTINUE
C             
C        super-reference level formalism
C    
         LRF=LTE.AND.IREF.NE.NKA(IAT)        
         IF(IFSUP.EQ.1.OR.LRF) THEN
         XA=UN
         XT=0.
         XN=0.
         ILTREF(IREF,ID)=IREF
         IF(IREF.GT.N0A(IAT)) THEN
         DO 60 I=IREF-1,N0A(IAT),-1
            ILTREF(I,ID)=IREF
            SBPSI(I,ID)=XA*SBF(I)*ELEC(ID)*BFAC(I,ID)*WOP(I,ID)
            DSBPST(I,ID)=XT+DSBF(I)
            DSBPSN(I,ID)=XN+ELEC1(ID)
            IF(I.EQ.NFIRST(IEL(I))) THEN
               XA=SBPSI(I,ID)
               XT=DSBPST(I,ID)
               XN=DSBPSN(I,ID)
            END IF
   60    CONTINUE
         END IF
C
         XA=UN
         XT=0.
         XN=0.
         IF(IREF.LT.NKA(IAT)) THEN
         IF(IREF.EQ.NLAST(IEL(IREF))) THEN
            XA=UN/(SBF(IREF)*BFAC(IREF,ID)*WOP(IREF,ID)*ELEC(ID))
            XT=-DSBF(IREF)
            XN=-ELEC1(ID)
         END IF
         DO 70 I=IREF+1,NKA(IAT)-1
            ILTREF(I,ID)=IREF
            I1=NFIRST(IEL(I))
            SBB1=UN/(SBF(I1)*BFAC(I1,ID)*WOP(I1,ID))
            SBPSI(I,ID)=XA*SBF(I)*BFAC(I,ID)*WOP(I,ID)*SBB1
            DSBPST(I,ID)=XT+DSBF(I)-DSBF(I1)
            DSBPSN(I,ID)=XN
            IF(I.EQ.NLAST(IEL(I))) THEN
               XA=XA*SBB1*ELEC1(ID)
               XT=XT-DSBF(I1)
               XN=XN-ELEC1(ID)
            END IF
   70    CONTINUE
         I=NKA(IAT)
         ILTREF(I,ID)=IREF
         SBPSI(I,ID)=XA
         DSBPST(I,ID)=XT
         DSBPSN(I,ID)=XN
         END IF
         END IF
C
C        different meaning of SBPSI for IMODL=2 or 6, i.e. for levels
C        with fixed ratio of population to that of reference level
C
C        if(lte) go to 100
         DO I=N0A(IAT),NKA(IAT)
            IF(IABS(IMODL(I)).EQ.2.OR.IABS(IMODL(I)).EQ.6) THEN
               IF(IABS(IMODL(I)).EQ.6) ILTREF(I,ID)=IREF
               IF(POPUL(I,ID).EQ.0..OR.POPUL(ILTREF(I,ID),ID).EQ.0.) 
     *            THEN
                  IN=NNEXT(IEL(I))
                  IF(I.LT.IREF.OR.POPUL(I,ID).LT.POPUL(IN,ID)) THEN
                     ILTREF(I,ID)=IN
                     SBPSI(I,ID)=SBF(I)*ELEC(ID)*WOP(I,ID)*BFAC(I,ID)
                    ELSE IF(I.GT.IREF) THEN
                     I1=NFIRST(IEL(I))
                     ILTREF(I,ID)=I1
                     SBPSI(I,ID)=SBF(I)*WOP(I,ID)*BFAC(I,ID)/
     *                           (SBF(I1)*WOP(I1,ID)*BFAC(I1,ID))
                  END IF
                ELSE
                  SBPSI(I,ID)=POPUL(I,ID)/POPUL(ILTREF(I,ID),ID)
               END IF
               DSBPST(I,ID)=0.
               DSBPSN(I,ID)=0.
            END IF
         END DO
  100 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE RATES
C     ================
C
C     Evaluation of  radiative rates
C
C     Output (transferred by COMMON blocks):
C            RRU(IT,ID) - upward radiative rate in transition IT and
C                         depth ID
C            RRD(IT,ID) - analogously the downward rate; more precisely
C                         the exact downward rate is given by:
C               lines   : RRD * stat.weight(lower)/stat.weight(upper)
C               continua: RRD * n(elec) * Saha-Boltzmann factor
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL)
C
C     zero the rates
C
      DO ID=1,ND
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
         END DO
      END DO
C
C
C     ---------------------
C     Continuum transitions
C     ---------------------
C
      LROSS=NDRE.LE.0.AND.ITER.EQ.1.OR.LFIN
      IF(HMIX0.GT.0.) LROSS=.TRUE.
      DO 30 IJ=1,NFREQC
         FR=FREQ(IJ)
         CALL OPACF1(IJ)
         CALL RTEFR1(IJ)
         IF(LROSS) CALL ROSSTD(IJ,NFREQC)
C
         DO 20 ID=1,ND
            EHK=EXP(-HKT1(ID)*FR)
            EHKF=EHK*FR
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
C
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
                 W0=W0EC(IJ)
                 IF(IFR1(ITR).EQ.IJ) W0=W0EC(IJ)-WCHCE(IJ)
                 SGW0=SG*W0
               ELSE
                 SG=SGMG(IMER,ID)
                 IF(SG.LE.0.) GO TO 10
                 W0=W0E(IJ)
                 IF(IFR1(ITR).EQ.IJ)
     *              W0=W0E(IJ)*(W(IJ)-WCH(IJ))/W(IJ)
                 SGW0=SG*W0
               ENDIF
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RAD1(ID)+BNUE(IJ))*EHK
   10       CONTINUE
   20    CONTINUE
   30 CONTINUE
C            
C     ----------------
C     Line transitions 
C     ----------------
C             
      DO 200 ITR=1,NTRANS
         IF(LINEXP(ITR)) GO TO 200
         I=ILOW(ITR)
         J=IUP(ITR)
         I0=IFR0(ITR)
         I1=IFR1(ITR)
         NFL=I1-I0+1
         DO 120 IJ=I0,I1
            IJL=IJ-I0+1
            CALL OPACF1(IJ)
            CALL RTEFR1(IJ)
            DO 110 ID=1,ND
               RADL(IJL,ID)=RAD1(ID)
               DO ICDW=1,NCDW
                  DWFL(IJL,ID,ICDW)=DWF1(ICDW,ID)
               ENDDO
  110       CONTINUE
  120    CONTINUE
C
         DO 190 ID=1,ND
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 190
            SRRU=0.
            SRRD=0.
            DO 130 IJ=I0,I1
               IJL=IJ-I0+1
               SGW=PRFLIN(IJ,ID)*W(IJ)
               SRRU=SRRU+SGW*RADL(IJL,ID)
               SRRD=SRRD+SGW*(RADL(IJL,ID)+BNTR(ITR))
  130       CONTINUE
            RRU(ITR,ID)=SRRU*XPIF(ITR)
            RRD(ITR,ID)=SRRD*XPIF(ITR)
c                
c          correction to the continuum rates
c                
            DO 140 IJL=1,NFL
               IJ=IJL+I0-1
               EHKL(IJL)=EXP(-HKT1(ID)*FREQ(IJ))
  140       CONTINUE
C
            DO 180 IBF=1,NTRANC
               ITRC=ITRBF(IBF)
c              IF(FRQMX(ITR).LT.FREQ(IFR1(ITRC))) GO TO 180
               II=ILOW(ITRC)
               IF(IFWOP(II).LT.0) GO TO 180
               SRRU=0.
               SRRD=0.
               DO 160 IJ=I0,I1
                  IJL=IJ-I0+1
                  SG=CROSS(IBF,IJ)
                  IF(SG.LE.0.) GO TO 160
                  IF(MCDW(ITRC).GE.1) SG=SG*DWFL(IJL,ID,MCDW(ITRC))
                  SGW0=SG*W0E(IJ)
                  SRRU=SRRU+SGW0*RADL(IJL,ID)
                  SRRD=SRRD+SGW0*(RADL(IJL,ID)+BNUE(IJ))*EHKL(IJL)
  160          CONTINUE
               RRU(ITRC,ID)=RRU(ITRC,ID)+SRRU
               RRD(ITRC,ID)=RRD(ITRC,ID)+SRRD
  180       CONTINUE
  190    CONTINUE
  200 CONTINUE
C
      DO ITR=1,NTRANS
         DO ID=1,ND
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
         IF(LINE(ITR)) THEN
            DO ID=1,ND
               RRD(ITR,ID)=RRD(ITR,ID)*EXP(-HKT1(ID)*FR0(ITR))
            END DO
         END IF
      END DO
      RETURN
      END

C
C
C
C     ****************************************************************
C

      SUBROUTINE RATESP
C     =================
C
C     Evaluation of "preconditioned" radiative rates
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL)
C
C     zero the rates
C
      DO ID=1,ND
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
         END DO
      END DO
C
C     ---------------------
C     Continuum transitions
C     ---------------------
C
      LROSS=NDRE.LE.0.AND.ITER.EQ.1.OR.LFIN
      IF(HMIX0.GT.0.) LROSS=.TRUE.
      DO 30 IJ=1,NFREQC
         FR=FREQ(IJ)
         CALL OPACF1(IJ)
         CALL RTEFR1(IJ)
         IF(LROSS) CALL ROSSTD(IJ,NFREQC)
C
         DO 20 ID=1,ND
            EHK=EXP(-HKT1(ID)*FR)
            EHKF=EHK*FR
            ALA=ALI1(ID)/ABSO1(ID)
            if(ilpsct.eq.0) then
               ALA=ALI1(ID)/(ABSO1(ID)-ELSCAT(ID))
            else
               ALA=ALI1(ID)/ABSO1(ID)
            end if
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
C
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
                 W0=W0EC(IJ)
                 IF(IFR1(ITR).EQ.IJ) W0=W0EC(IJ)-WCHCE(IJ)
                 SGW0=SG*W0
               ELSE
                 SG=SGMG(IMER,ID)
                 IF(SG.LE.0.) GO TO 10
                 W0=W0E(IJ)
                 IF(IFR1(ITR).EQ.IJ)
     *              W0=W0E(IJ)*(W(IJ)-WCH(IJ))/W(IJ)
                 SGW0=SG*W0
               ENDIF
               ABSOC=ABTRA(ITR,ID)-EMTRA(ITR,ID)*EHK
               SLIN=BNUE(IJ)*EMTRA(ITR,ID)*EHK/ABSOC
               RLAM=ABSOC*SG*ALA
               RADRES=RAD1(ID)-RLAM*SLIN
               BNURES=BNUE(IJ)*(UN-RLAM)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RADRES+BNURES)*EHK
   10       CONTINUE
   20    CONTINUE
   30 CONTINUE
C            
C     ----------------
C     Line transitions 
C     ----------------
C             
      DO 200 ITR=1,NTRANS
         IF(LINEXP(ITR)) GO TO 200
         I=ILOW(ITR)
         J=IUP(ITR)
         I0=IFR0(ITR)
         I1=IFR1(ITR)
         NFL=I1-I0+1
         DO 120 IJ=I0,I1
            IJL=IJ-I0+1
            CALL OPACF1(IJ)
            CALL RTEFR1(IJ)
            DO 110 ID=1,ND
               RADL(IJL,ID)=RAD1(ID)
               ABSALI(IJL,ID)=ALI1(ID)/ABSO1(ID)
               DO ICDW=1,NCDW
                  DWFL(IJL,ID,ICDW)=DWF1(ICDW,ID)
               ENDDO
  110       CONTINUE
  120    CONTINUE
C
         DO 190 ID=1,ND
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 190
            POPIL=ABTRA(ITR,ID)
            POPJL=EMTRA(ITR,ID)*EXP(-HKT1(ID)*FR0(ITR))
            POPIJ=POPIL-POPJL
            IF(POPIJ.NE.0.) SLIN=BNTR(ITR)*POPJL/POPIJ
            SRRU=0.
            SRRD=0.
            DO 130 IJ=I0,I1
               IJL=IJ-I0+1
               SGW=PRFLIN(IJ,ID)*W(IJ)
               RLAM=PRFLIN(IJ,ID)*POPIJ*ABSALI(IJL,ID)
               RADRES=RADL(IJL,ID)-RLAM*SLIN
               BNURES=BNTR(ITR)*(UN-RLAM)
               SRRU=SRRU+SGW*RADRES
               SRRD=SRRD+SGW*(RADRES+BNURES)
  130       CONTINUE
            RRU(ITR,ID)=SRRU*XPIF(ITR)
            RRD(ITR,ID)=SRRD*XPIF(ITR)
c                
c          correction to the continuum rates
c                
            DO 140 IJL=1,NFL
               IJ=IJL+I0-1
               EHKL(IJL)=EXP(-HKT1(ID)*FREQ(IJ))
  140       CONTINUE
C
            DO 180 IBF=1,NTRANC
               ITRC=ITRBF(IBF)
c              IF(FRQMX(ITR).LT.FREQ(IFR1(ITRC))) GO TO 180
               II=ILOW(ITRC)
               IF(IFWOP(II).LT.0) GO TO 180
               SRRU=0.
               SRRD=0.
               DO 160 IJ=I0,I1
                  IJL=IJ-I0+1
                  SG=CROSS(IBF,IJ)
                  IF(SG.LE.0.) GO TO 160
                  IF(MCDW(ITRC).GE.1) SG=SG*DWFL(IJL,ID,MCDW(ITRC))
                  SGW0=SG*W0E(IJ)
                  ABSOC=ABTRA(ITRC,ID)-EMTRA(ITRC,ID)*EHKL(IJL)
                  SLIN=BNUE(IJ)*EMTRA(ITRC,ID)*EHKL(IJL)/ABSOC
                  RLAM=ABSOC*SG*ABSALI(IJL,ID)
                  RADRES=RADL(IJL,ID)-RLAM*SLIN
                  BNURES=BNUE(IJ)*(UN-RLAM)
                  SRRU=SRRU+SGW0*RADRES
                  SRRD=SRRD+SGW0*(RADRES+BNURES)*EHKL(IJL)
  160          CONTINUE
               RRU(ITRC,ID)=RRU(ITRC,ID)+SRRU
               RRD(ITRC,ID)=RRD(ITRC,ID)+SRRD
  180       CONTINUE
  190    CONTINUE
  200 CONTINUE
C
      DO ITR=1,NTRANS
         DO ID=1,ND
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
         IF(LINE(ITR)) THEN
            DO ID=1,ND
               RRD(ITR,ID)=RRD(ITR,ID)*EXP(-HKT1(ID)*FR0(ITR))
            END DO
         END IF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C
C
      SUBROUTINE ALISET
C     =================
C
C     Evaluation of all necessary ALI parameters + radiative rates
C     (the routine is analogous to RATES)
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),EHKLF(MFREQL)
C
C     zero the rates and other quantities (subr. NULL)
C
      DO ID=1,ND
         REIT(ID)=0.
         REIN(ID)=0.
         AREIT(ID)=0.
         AREIN(ID)=0.
         CREIT(ID)=0.
         CREIN(ID)=0.
         REDT(ID)=0.
         REDTM(ID)=0.
         REDN(ID)=0.
         REDNM(ID)=0.
         REDX(ID)=0.
         REDXM(ID)=0.
         HEIT(ID)=0.
         HEITM(ID)=0.
         HEIN(ID)=0.
         HEINM(ID)=0.
         FCOOLI(ID)=0.
         FLFIX(ID)=0.
         FLEXP(ID)=0.
         FPRD(ID)=0.
         DO II=1,NLVEXP
            HEIP(II,ID)=0.
            REIP(II,ID)=0.
            AREIP(II,ID)=0.
            CREIP(II,ID)=0.
            REDP(II,ID)=0.
            REDPM(II,ID)=0.
            HEIPM(II,ID)=0.
         END DO
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
            DRDT(ITR,ID)=0.
         END DO
      END DO
C
C
C     ---------------------
C     Continuum transitions
C     ---------------------
C
      DO 30 IJ=1,NFREQC
         FR=FREQ(IJ)
         CALL OPACFD(IJ)
         CALL RTEFR1(IJ)
         CALL ALIFR1(IJ)
C
         DO 20 ID=1,ND
            EHK=EXP(-HKT1(ID)*FR)
            EHKF=EHK*FR
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               LDER=IIEXP(II).GT.0
               if(ilder.gt.0) lder=.false.
C
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
                 W0=W0EC(IJ)
                 IF(IFR1(ITR).EQ.IJ) W0=W0EC(IJ)-WCHCE(IJ)
                 SGW0=SG*W0
               ELSE
                 SG=SGMG(IMER,ID)
                 IF(SG.LE.0.) GO TO 10
                 W0=W0E(IJ)
                 IF(IFR1(ITR).EQ.IJ)
     *              W0=W0E(IJ)*(W(IJ)-WCH(IJ))/W(IJ)
                 SGW0=SG*W0
               ENDIF
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RAD1(ID)+BNUE(IJ))*EHK
               IF(LDER) DRDT(ITR,ID)=DRDT(ITR,ID)+
     *                  SGW0*(RAD1(ID)+BNUE(IJ))*EHKF
   10       CONTINUE
   20    CONTINUE
   30 CONTINUE
C            
C     ----------------
C     Line transitions 
C     ----------------
C             
      DO 200 ITR=1,NTRANS
         IF(LINEXP(ITR)) GO TO 200
         I=ILOW(ITR)
         J=IUP(ITR)
         I0=IFR0(ITR)
         I1=IFR1(ITR)
         NFL=I1-I0+1
         DO 120 IJ=I0,I1
            IJL=IJ-I0+1
            CALL OPACFD(IJ)
            CALL RTEFR1(IJ)
            CALL ALIFR1(IJ)
C
            DO 110 ID=1,ND
               RADL(IJL,ID)=RAD1(ID)
               DO ICDW=1,NCDW
                  DWFL(IJL,ID,ICDW)=DWF1(ICDW,ID)
               ENDDO
  110       CONTINUE
  120    CONTINUE
C
         DO 190 ID=1,ND
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 190
            SRRU=0.
            SRRD=0.
            DO 130 IJ=I0,I1
               IJL=IJ-I0+1
               SGW=PRFLIN(IJ,ID)*W(IJ)
               SRRU=SRRU+SGW*RADL(IJL,ID)
               SRRD=SRRD+SGW*(RADL(IJL,ID)+BNTR(ITR))
  130       CONTINUE
            RRU(ITR,ID)=SRRU*XPIF(ITR)
            RRD(ITR,ID)=SRRD*XPIF(ITR)
c                
c          correction to the continuum rates
c                
            DO 140 IJL=1,NFL
               IJ=IJL+I0-1
               EHKL(IJL)=EXP(-HKT1(ID)*FREQ(IJ))
               EHKLF(IJL)=EHKL(IJL)*FREQ(IJ)
  140       CONTINUE
C
            DO 180 IBF=1,NTRANC
               ITRC=ITRBF(IBF)
c              IF(FRQMX(ITR).LT.FREQ(IFR1(ITRC))) GO TO 180
               II=ILOW(ITRC)
               IF(IFWOP(II).LT.0) GO TO 180
               LDER=IIEXP(II).GT.0.AND.ILDER.EQ.0
               SRRU=0.
               SRRD=0.
               SDRT=0.
               DO 160 IJ=I0,I1
                  IJL=IJ-I0+1
                  SG=CROSS(IBF,IJ)
                  IF(SG.LE.0.) GO TO 160
                  IF(MCDW(ITRC).GE.1) SG=SG*DWFL(IJL,ID,MCDW(ITRC))
                  SGW0=SG*W0E(IJ)
                  SRRU=SRRU+SGW0*RADL(IJL,ID)
                  SRRD=SRRD+SGW0*(RADL(IJL,ID)+BNUE(IJ))*EHKL(IJL)
                  SDRT=SDRT+SGW0*(RADL(IJL,ID)+BNUE(IJ))*EHKLF(IJL)
  160          CONTINUE
               RRU(ITRC,ID)=RRU(ITRC,ID)+SRRU
               RRD(ITRC,ID)=RRD(ITRC,ID)+SRRD
               IF(LDER) DRDT(ITRC,ID)=DRDT(ITRC,ID)+SDRT
  180       CONTINUE
  190    CONTINUE
  200 CONTINUE
C
C     multiply some quantities by frequency-independent constants
C
      DO ID=1,ND
         REDX(ID)=REDX(ID)*WMM(ID)*DENS1(ID)*DENS1(ID)
         IF(ID.GT.1) REDXM(ID)=REDXM(ID)*WMM(ID)*
     *                         DENS1(ID-1)*DENS1(ID-1)
         FCOOL(ID)=REINT(ID)*FCOOLI(ID)-REDIF(ID)*FLFIX(ID)
         DO ITR=1,NTRANC
            DRDT(ITR,ID)=DRDT(ITR,ID)*HKT21(ID)*CRSW(ID)
         END DO
      END DO
C
      DO ITR=1,NTRANS
         DO ID=1,ND
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
         IF(LINE(ITR)) THEN
            DO ID=1,ND
               RRD(ITR,ID)=RRD(ITR,ID)*EXP(-HKT1(ID)*FR0(ITR))
               DRDT(ITR,ID)=RRD(ITR,ID)*HKT21(ID)*FR0(ITR)
            END DO
         END IF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE ALISKT
C     =================
C
C     Simplified routine ALISET for Kantorovich iteration
C
C     Evaluation of all necessary ALI parameters + radiative rates
C     (the routine is analogous to RATES)
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL)
C
C     zero the rates and other quantities (subr. NULL)
C
      DO ID=1,ND
         FCOOLI(ID)=0.
         FLFIX(ID)=0.
         FPRD(ID)=0.
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
         END DO
      END DO
C
C
C     ---------------------
C     Continuum transitions
C     ---------------------
C
      DO 30 IJ=1,NFREQC
         FR=FREQ(IJ)
         CALL OPACF1(IJ)
         IF(IJEX(IJ).GT.0) THEN
            IJE=IJEX(IJ)
            DO ID=1,ND
               ABSOEX(IJE,ID)=ABSO1(ID)
               EMISEX(IJE,ID)=EMIS1(ID)
               SCATEX(IJE,ID)=SCAT1(ID)
            END DO
         END IF
         CALL RTEFR1(IJ)
         CALL ALIFRK(IJ)
C
         DO 20 ID=1,ND
            EHK=EXP(-HKT1(ID)*FR)
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
                 W0=W0EC(IJ)
                 IF(IFR1(ITR).EQ.IJ) W0=W0EC(IJ)-WCHCE(IJ)
                 SGW0=SG*W0
               ELSE
                 SG=SGMG(IMER,ID)
                 IF(SG.LE.0.) GO TO 10
                 W0=W0E(IJ)
                 IF(IFR1(ITR).EQ.IJ)
     *              W0=W0E(IJ)*(W(IJ)-WCH(IJ))/W(IJ)
                 SGW0=SG*W0
               ENDIF
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RAD1(ID)+BNUE(IJ))*EHK
   10       CONTINUE
   20    CONTINUE
   30 CONTINUE
C            
C     ----------------
C     Line transitions 
C     ----------------
C             
      DO 200 ITR=1,NTRANS
         IF(LINEXP(ITR)) GO TO 200
         I=ILOW(ITR)
         J=IUP(ITR)
         I0=IFR0(ITR)
         I1=IFR1(ITR)
         NFL=I1-I0+1
         DO 120 IJ=I0,I1
            IJL=IJ-I0+1
            CALL OPACF1(IJ)
            IF(IJEX(IJ).GT.0) THEN
               IJE=IJEX(IJ)
               DO ID=1,ND
                  ABSOEX(IJE,ID)=ABSO1(ID)
                  EMISEX(IJE,ID)=EMIS1(ID)
                  SCATEX(IJE,ID)=SCAT1(ID)
               END DO
            END IF
            CALL RTEFR1(IJ)
            CALL ALIFRK(IJ)
C
            DO 110 ID=1,ND
               RADL(IJL,ID)=RAD1(ID)
               DO ICDW=1,NCDW
                  DWFL(IJL,ID,ICDW)=DWF1(ICDW,ID)
               ENDDO
  110       CONTINUE
  120    CONTINUE
C
         DO 190 ID=1,ND
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 190
            SRRU=0.
            SRRD=0.
            DO 130 IJ=I0,I1
               IJL=IJ-I0+1
               SGW=PRFLIN(IJ,ID)*W(IJ)
               SRRU=SRRU+SGW*RADL(IJL,ID)
               SRRD=SRRD+SGW*(RADL(IJL,ID)+BNTR(ITR))
  130       CONTINUE
            RRU(ITR,ID)=SRRU*XPIF(ITR)
            RRD(ITR,ID)=SRRD*XPIF(ITR)
c                
c          correction to the continuum rates
c                
            DO 140 IJL=1,NFL
               IJ=IJL+I0-1
               EHKL(IJL)=EXP(-HKT1(ID)*FREQ(IJ))
  140       CONTINUE
C
            DO 180 IBF=1,NTRANC
               ITRC=ITRBF(IBF)
c              IF(FRQMX(ITR).LT.FREQ(IFR1(ITRC))) GO TO 180
               II=ILOW(ITRC)
               IF(IFWOP(II).LT.0) GO TO 180
               SRRU=0.
               SRRD=0.
               DO 160 IJ=I0,I1
                  IJL=IJ-I0+1
                  SG=CROSS(IBF,IJ)
                  IF(SG.LE.0.) GO TO 160
                  IF(MCDW(ITRC).GE.1) SG=SG*DWFL(IJL,ID,MCDW(ITRC))
                  SGW0=SG*W0E(IJ)
                  SRRU=SRRU+SGW0*RADL(IJL,ID)
                  SRRD=SRRD+SGW0*(RADL(IJL,ID)+BNUE(IJ))*EHKL(IJL)
  160          CONTINUE
               RRU(ITRC,ID)=RRU(ITRC,ID)+SRRU
               RRD(ITRC,ID)=RRD(ITRC,ID)+SRRD
  180       CONTINUE
  190    CONTINUE
  200 CONTINUE
C
C     multiply some quantities by frequency-independent constants
C
      DO ID=1,ND
         FCOOL(ID)=REINT(ID)*FCOOLI(ID)-REDIF(ID)*FLFIX(ID)
      END DO
      DO ITR=1,NTRANS
         DO ID=1,ND
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
         IF(LINE(ITR)) THEN
            DO ID=1,ND
               RRD(ITR,ID)=RRD(ITR,ID)*EXP(-HKT1(ID)*FR0(ITR))
            END DO
         END IF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE RATES1
C     =================
C
C     Evaluation of  radiative rates
C     -- a variant for the full overlap case
C
C     Output (transferred by COMMON blocks):
C            RRU(IT,ID) - upward radiative rate in transition IT and
C                         depth ID
C            RRD(IT,ID) - analogously the downward rate; more precisely
C                         the exact downward rate is given by:
C               lines   : RRD * stat.weight(lower)/stat.weight(upper)
C               continua: RRD * n(elec) * Saha-Boltzmann factor
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),RBNE(MDEPTH)
C
C     zero the rates
C
      DO ID=1,ND
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
         END DO
      END DO
C
      LROSS=NDRE.LE.0.AND.ITER.EQ.1.OR.LFIN
      IF(HMIX0.GT.0.) LROSS=.TRUE.
      DO 100 IJ=1,NFREQ
         IF(IJX(IJ).EQ.-1) GOTO 100
         FR=FREQ(IJ)
         W0=W0E(IJ)
         CALL OPACF1(IJ)
         CALL RTEFR1(IJ)
         IF(LROSS) CALL ROSSTD(IJ,NFREQ)
C
C        ---------------------
C        Continuum transitions
C        ---------------------
C
         DO 20 ID=1,ND
            RBNE(ID)=(RAD1(ID)+BNUE(IJ))*EXP(-HKT1(ID)*FR)
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               LDER=IIEXP(II).GT.0
               if(ilder.gt.0) lder=.false.
C
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
               ELSE
                 SG=SGMG(IMER,ID)
               ENDIF
               SGW0=SG*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNE(ID)
   10       CONTINUE
   20    CONTINUE
C            
C        ----------------
C        Line transitions 
C        ----------------
C             
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            DO 50 ID=1,ND
               SGW=PRFLIN(IJ,ID)*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW*RBNE(ID)
   50       CONTINUE
         ENDIF
         IF(NLINES(IJ).LE.0) GO TO 100
C
C        the "overlapping" lines at the given frequency
C
         DO 90 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            IJ0=IFR0(ITR)
            DO 60 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 70
               END IF
   60       CONTINUE
   70       IJ1=IJ0-1
            A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))*W0
            A2=W0-A1
            DO 80 ID=1,ND
               SGW=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW*RBNE(ID)
   80       CONTINUE
   90    CONTINUE
  100 CONTINUE
C
      DO ID=1,ND
        IF(CRSW(ID).NE.UN) THEN
         DO ITR=1,NTRANS
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
        END IF
      END DO
      RETURN
      END
C
C
C
C     ****************************************************************
C

      SUBROUTINE RATSP1
C     =================
C
C     Evaluation of "preconditioned" radiative rates
C     -- a variant for the full overlap case
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),EHK(MDEPTH)
C
C     zero the rates
C
      DO ID=1,ND
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
         END DO
      END DO
C
      LROSS=NDRE.LE.0.AND.ITER.EQ.1.OR.LFIN
      IF(HMIX0.GT.0.) LROSS=.TRUE.
      DO 100 IJ=1,NFREQ
         IF(IJX(IJ).EQ.-1) GOTO 100
         FR=FREQ(IJ)
         W0=W0E(IJ)
         CALL OPACF1(IJ)
         CALL RTEFR1(IJ)
         IF(LROSS) CALL ROSSTD(IJ,NFREQ)
         if(iter.le.1) call radpre(ij)
C
C        ---------------------
C        Continuum transitions
C        ---------------------
C
         DO 20 ID=1,ND
            EHK(ID)=EXP(-HKT1(ID)*FR)
            if(ilpsct.eq.0) then
               ALAB(ID)=ALI1(ID)/(ABSO1(ID)-ELSCAT(ID))
            else
               ALAB(ID)=ALI1(ID)/ABSO1(ID)
            end if
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
C
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
               ELSE
                 SG=SGMG(IMER,ID)
               ENDIF
               SGW0=SG*W0
               RLAM=SG*ALAB(ID)
               ELIN=EMTRA(ITR,ID)*EHK(ID)
               RADRES=RAD1(ID)-RLAM*BNUE(IJ)*ELIN
               BNURES=BNUE(IJ)*(UN-RLAM*(ABTRA(ITR,ID)-ELIN))
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RADRES+BNURES)*EHK(ID)
   10       CONTINUE
   20    CONTINUE
C            
C        ----------------
C        Line transitions 
C        ----------------
C             
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            DO 50 ID=1,ND
               SG=PRFLIN(IJ,ID)
               SGW0=SG*W0
               RLAM=SG*ALAB(ID)
               ELIN=EMTRA(ITR,ID)*EHK(ID)
               RADRES=RAD1(ID)-RLAM*BNUE(IJ)*ELIN
               BNURES=BNUE(IJ)*(UN-RLAM*(ABTRA(ITR,ID)-ELIN))
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RADRES+BNURES)*EHK(ID)
   50       CONTINUE
         ENDIF
         IF(NLINES(IJ).LE.0) GO TO 100
C
C        the "overlapping" lines at the given frequency
C
         DO 90 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            IJ0=IFR0(ITR)
            DO 60 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 70
               END IF
   60       CONTINUE
   70       IJ1=IJ0-1
            A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))
            A2=UN-A1
            DO 80 ID=1,ND
               SG=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
               SGW0=SG*W0
               RLAM=SG*ALAB(ID)
               ELIN=EMTRA(ITR,ID)*EHK(ID)
               RADRES=RAD1(ID)-RLAM*BNUE(IJ)*ELIN
               BNURES=BNUE(IJ)*(UN-RLAM*(ABTRA(ITR,ID)-ELIN))
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RADRES+BNURES)*EHK(ID)
   80       CONTINUE
   90    CONTINUE
  100 CONTINUE
C
      DO ID=1,ND
        IF(CRSW(ID).NE.UN) THEN
         DO ITR=1,NTRANS
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
        END IF
      END DO
      RETURN
      END

C
C
C
C     ****************************************************************
C

      SUBROUTINE RATSP3
C     =================
C
C     Evaluation of "preconditioned" radiative rates
C     -- a variant for the full overlap case
C     and for 3-diagonal operator 
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHK(MDEPTH),ALABM(MDEPTH),ALABP(MDEPTH)
C
C     zero the rates
C
      DO ID=1,ND
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
            RDDM(ITR,ID)=0.
            RDDP(ITR,ID)=0.
         END DO
      END DO
C
      LROSS=NDRE.LE.0.AND.ITER.EQ.1.OR.LFIN
      IF(HMIX0.GT.0.) LROSS=.TRUE.
      DO 100 IJ=1,NFREQ
         IF(IJX(IJ).EQ.-1) GOTO 100
         FR=FREQ(IJ)
         W0=W0E(IJ)
         CALL OPACF1(IJ)
         CALL RTEFR1(IJ)
         IF(LROSS) CALL ROSSTD(IJ,NFREQ)
C
C        ---------------------
C        Continuum transitions
C        ---------------------
C
         DO ID=1,ND 
            EHK(ID)=EXP(-HKT1(ID)*FR)
            if(ilpsct.eq.0) then
            ALAB(ID)=ALI1(ID)/(ABSO1(ID)-ELSCAT(ID))
            IF(ID.GT.1) ALABM(ID)=ALIM1(ID)/(ABSO1(ID-1)-ELSCAT(ID-1)) 
            IF(ID.LT.ND) ALABP(ID)=ALIP1(ID)/(ABSO1(ID+1)-ELSCAT(ID+1)) 
            else
            ALAB(ID)=ALI1(ID)/ABSO1(ID)
            IF(ID.GT.1) ALABM(ID)=ALIM1(ID)/ABSO1(ID-1) 
            IF(ID.LT.ND) ALABP(ID)=ALIP1(ID)/ABSO1(ID+1)
            end if 
         END DO
C 
         DO 20 ID=1,ND
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
C
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
               ELSE
                 SG=SGMG(IMER,ID)
               ENDIF
               SGW0=SG*W0
               RLAM=SG*ALAB(ID)
               ELIN=EMTRA(ITR,ID)*EHK(ID)
               CHTRA=ABTRA(ITR,ID)-ELIN 
C 
C              lower off-diagonal rate
C
               RADREM=0. 
               IF(ID.GT.1) THEN
               IF(IFWOP(II).GE.0) THEN
                 SGM=CROSS(IBF,IJ)
                 IF(ICDW.GE.1) SGM=SGM*DWF1(ICDW,ID-1)
               ELSE
                 SGM=SGMG(IMER,ID-1)
               ENDIF
               ELIM=SGM*ALABM(ID)*BNUE(IJ)*EHK(ID-1) 
               RADREM=ELIM*EMTRA(ITR,ID-1)
               RDDM(ITR,ID)=RDDM(ITR,ID)+SGW0*ELIM*CHTRA
               END IF
C
C              upper off-diagonal rate
C 
               RADREP=0. 
               IF(ID.LT.ND) THEN
               IF(IFWOP(II).GE.0) THEN
                 SGP=CROSS(IBF,IJ)
                 IF(ICDW.GE.1) SGP=SGP*DWF1(ICDW,ID+1)
               ELSE
                 SGP=SGMG(IMER,ID+1)
               ENDIF
               ELIP=SGP*ALABP(ID)*BNUE(IJ)*EHK(ID+1) 
               RADREP=ELIP*EMTRA(ITR,ID+1)
               RDDP(ITR,ID)=RDDP(ITR,ID)+SGW0*ELIP*CHTRA
               END IF
C
C              diagonal rate
C 
               RADRES=RAD1(ID)-RLAM*BNUE(IJ)*ELIN-RADREM-RADREP 
               BNURES=BNUE(IJ)*(UN-RLAM*CHTRA)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RADRES+BNURES)*EHK(ID)
   10       CONTINUE
   20    CONTINUE
C            
C        ----------------
C        Line transitions 
C        ----------------
C             
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            DO 50 ID=1,ND
               SG=PRFLIN(IJ,ID)
               SGW0=SG*W0
               RLAM=SG*ALAB(ID)
               ELIN=EMTRA(ITR,ID)*EHK(ID)
               CHTRA=ABTRA(ITR,ID)-ELIN 
C 
C              lower off-diagonal rate
C
               RADREM=0. 
               IF(ID.GT.1) THEN
               SGM=PRFLIN(IJ,ID-1) 
               ELIM=SGM*ALABM(ID)*BNUE(IJ)*EHK(ID-1) 
               RADREM=ELIM*EMTRA(ITR,ID-1)
               RDDM(ITR,ID)=RDDM(ITR,ID)+SGW0*ELIM*CHTRA
               END IF
C
C              upper off-diagonal rate
C 
               RADREP=0. 
               IF(ID.LT.ND) THEN
               SGP=PRFLIN(IJ,ID+1) 
               ELIP=SGP*ALABP(ID)*BNUE(IJ)*EHK(ID+1) 
               RADREP=ELIP*EMTRA(ITR,ID+1)
               RDDP(ITR,ID)=RDDP(ITR,ID)+SGW0*ELIP*CHTRA
               END IF
C
C              diagonal rate
C 
               RADRES=RAD1(ID)-RLAM*BNUE(IJ)*ELIN-RADREM-RADREP 
               BNURES=BNUE(IJ)*(UN-RLAM*CHTRA)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RADRES+BNURES)*EHK(ID)
   50       CONTINUE
         ENDIF
         IF(NLINES(IJ).LE.0) GO TO 100
C
C        the "overlapping" lines at the given frequency
C
         DO 90 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            IJ0=IFR0(ITR)
            DO 60 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 70
               END IF
   60       CONTINUE
   70       IJ1=IJ0-1
            A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))
            A2=UN-A1
            DO 80 ID=1,ND
               SG=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
               SGW0=SG*W0
               RLAM=SG*ALAB(ID)
               ELIN=EMTRA(ITR,ID)*EHK(ID)
               CHTRA=ABTRA(ITR,ID)-ELIN 
C 
C              lower off-diagonal rate
C
               RADREM=0. 
               IF(ID.GT.1) THEN
               SGM=A1*PRFLIN(IJ1,ID-1)+A2*PRFLIN(IJ0,ID-1) 
               ELIM=SGM*ALABM(ID)*BNUE(IJ)*EHK(ID-1) 
               RADREM=ELIM*EMTRA(ITR,ID-1)
               RDDM(ITR,ID)=RDDM(ITR,ID)+SGW0*ELIM*CHTRA
               END IF
C
C              upper off-diagonal rate
C 
               RADREP=0. 
               IF(ID.LT.ND) THEN
               SGP=A1*PRFLIN(IJ1,ID+1)+A2*PRFLIN(IJ0,ID+1) 
               ELIP=SGP*ALABP(ID)*BNUE(IJ)*EHK(ID+1) 
               RADREP=ELIP*EMTRA(ITR,ID+1)
               RDDP(ITR,ID)=RDDP(ITR,ID)+SGW0*ELIP*CHTRA
               END IF
C
C              diagonal rate
C 
               RADRES=RAD1(ID)-RLAM*BNUE(IJ)*ELIN-RADREM-RADREP 
               BNURES=BNUE(IJ)*(UN-RLAM*CHTRA)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RADRES+BNURES)*EHK(ID)
   80       CONTINUE
   90    CONTINUE
  100 CONTINUE
C
      DO ID=1,ND
        IF(CRSW(ID).NE.UN) THEN
         DO ITR=1,NTRANS
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
            RDDM(ITR,ID)=RDDM(ITR,ID)*CRSW(ID)
            RDDP(ITR,ID)=RDDP(ITR,ID)*CRSW(ID)
         END DO
        ENDIF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C
C
      SUBROUTINE ALIST1
C     =================
C
C     Evaluation of all nexcessary ALI parameters + radiative rates
C     (the routine is analogous to RATES)
C     -- a variant for the full overlap case
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),EHKLF(MFREQL)
      DIMENSION EXX(MDEPTH),RBNU(MDEPTH),RBNUF(MDEPTH)
C
C     zero the rates and other quantities 
C
      DO ID=1,ND
         REIT(ID)=0.
         REIN(ID)=0.
         AREIT(ID)=0.
         AREIN(ID)=0.
         CREIT(ID)=0.
         CREIN(ID)=0.
         REDT(ID)=0.
         REDTM(ID)=0.
         REDTP(ID)=0.
         REDN(ID)=0.
         REDNM(ID)=0.
         REDNP(ID)=0.
         REDX(ID)=0.
         REDXM(ID)=0.
         HEIT(ID)=0.
         HEITM(ID)=0.
         HEITP(ID)=0.
         HEIN(ID)=0.
         HEINM(ID)=0.
         HEINP(ID)=0.
         EHET(ID)=0.
         EHEN(ID)=0.
         ERET(ID)=0.
         EREN(ID)=0.
         FCOOLI(ID)=0.
         FLFIX(ID)=0.
         FLEXP(ID)=0.
         FPRD(ID)=0.
         DO II=1,NLVEXP
            HEIP(II,ID)=0.
            REIP(II,ID)=0.
            AREIP(II,ID)=0.
            CREIP(II,ID)=0.
            REDP(II,ID)=0.
            REDPM(II,ID)=0.
            HEIPM(II,ID)=0.
            REDPP(II,ID)=0.
            HEIPP(II,ID)=0.
            EHEP(II,ID)=0.
            EREP(II,ID)=0.
         END DO
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
            DRDT(ITR,ID)=0.
         END DO
      END DO
C
      DO 100 IJ=1,NFREQ
         IF(IJX(IJ).EQ.-1) GO TO 100
         FR=FREQ(IJ)
         W0=W0E(IJ)
         CALL OPACFD(IJ)
         CALL RTEFR1(IJ)
         CALL ALIFR1(IJ)
C
C        ---------------------
C        Continuum transitions
C        ---------------------
C
         DO 20 ID=1,ND
            EXX(ID)=EXP(-HKT1(ID)*FR)
            RBNU(ID)=(RAD1(ID)+BNUE(IJ))*EXX(ID)
            RBNUF(ID)=RBNU(ID)*FR*HKT21(ID)
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               LDER=IIEXP(II).GT.0.AND.ILDER.EQ.0
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
               ELSE
                 SG=SGMG(IMER,ID)
               ENDIF
               SGW0=SG*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
c              IF(IIEXP(II).GT.0.AND.ILDER.EQ.0)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
   10       CONTINUE
   20    CONTINUE
C            
C        ----------------
C        Line transitions 
C        ----------------
C             
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            DO 50 ID=1,ND
               SGW0=PRFLIN(IJ,ID)*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
c              IF(LDER) DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
   50       CONTINUE
         ENDIF
         IF(NLINES(IJ).LE.0) GO TO 100
C
C        the "overlapping" lines at the given frequency
C
         DO 90 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            IJ0=IFR0(ITR)
            DO 60 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 70
               END IF
   60       CONTINUE
   70       IJ1=IJ0-1
            A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))*W0
            A2=W0-A1
            DO 80 ID=1,ND
               SGW0=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
c              IF(LDER) DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
   80       CONTINUE
   90    CONTINUE
  100 CONTINUE
C
C     multiply some quantities by frequency-independent constants
C
      DO ID=1,ND
         REDX(ID)=REDX(ID)*WMM(ID)*DENS1(ID)*DENS1(ID)
         IF(ID.GT.1) REDXM(ID)=REDXM(ID)*WMM(ID)*
     *                         DENS1(ID-1)*DENS1(ID-1)
         FCOOL(ID)=REINT(ID)*FCOOLI(ID)-REDIF(ID)*FLFIX(ID)
         IF(CRSW(ID).NE.UN) THEN
            DO ITR=1,NTRANS
               RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
               RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)*CRSW(ID)
            END DO
         END IF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C
C
      SUBROUTINE ALIST2
C     =================
C
C     Evaluation of all nexcessary ALI parameters + radiative rates
C     (the routine is analogous to RATES)
C     -- a variant for the full overlap case
C        and for derivatives of the rate matrix w.r.t. populations
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),EHKLF(MFREQL)
      DIMENSION EXX(MDEPTH),RBNU(MDEPTH),RBNUF(MDEPTH)
C
C     zero the rates and other quantities 
C
      DO ID=1,ND
         REIT(ID)=0.
         REIN(ID)=0.
         AREIT(ID)=0.
         AREIN(ID)=0.
         CREIT(ID)=0.
         CREIN(ID)=0.
         REDT(ID)=0.
         REDTM(ID)=0.
         REDTP(ID)=0.
         REDN(ID)=0.
         REDNM(ID)=0.
         REDNP(ID)=0.
         REDX(ID)=0.
         REDXM(ID)=0.
         HEIT(ID)=0.
         HEITM(ID)=0.
         HEITP(ID)=0.
         HEIN(ID)=0.
         HEINM(ID)=0.
         HEINP(ID)=0.
         EHET(ID)=0.
         EHEN(ID)=0.
         ERET(ID)=0.
         EREN(ID)=0.
         FCOOLI(ID)=0.
         FLFIX(ID)=0.
         FLEXP(ID)=0.
         FPRD(ID)=0.
         DO II=1,NLVEXP
            HEIP(II,ID)=0.
            REIP(II,ID)=0.
            AREIP(II,ID)=0.
            CREIP(II,ID)=0.
            REDP(II,ID)=0.
            REDPM(II,ID)=0.
            HEIPM(II,ID)=0.
            REDPP(II,ID)=0.
            HEIPP(II,ID)=0.
            EHEP(II,ID)=0.
            EREP(II,ID)=0.
            APT(II,ID)=0.
            APN(II,ID)=0.
            DO JJ=1,NLVEXP
               APP(JJ,II,ID)=0.
            END DO
         END DO
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
            DRDT(ITR,ID)=0.
         END DO
      END DO
C
      DO 100 IJ=1,NFREQ
         IF(IJX(IJ).EQ.-1) GO TO 100
         FR=FREQ(IJ)
         W0=W0E(IJ)
         LRDER=IRDER.GT.0.AND.IJALI(IJ).GT.0
         CALL OPACFD(IJ)
         CALL RTEFR1(IJ)
         CALL ALIFR1(IJ)
C             
C        ---------------------
C        Continuum transitions
C        ---------------------
C          
         DO 20 ID=1,ND
            EXX(ID)=EXP(-HKT1(ID)*FR)
            RBNU(ID)=(RAD1(ID)+BNUE(IJ))*EXX(ID)
            RBNUF(ID)=RBNU(ID)*FR*HKT21(ID)
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               LDER=IIEXP(II).GT.0.AND.ILDER.EQ.0
               IF(IFWOP(II).GE.0) THEN
                  IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
                ELSE
                  SG=SGMG(IMER,ID)
               ENDIF
               SGW0=SG*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
C              IF(LDER) DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
               IF(LRDER) THEN
                  APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EXX(ID))*SGW0
                  IE=IIEXP(II)
                  JJ=IUP(ITR)
                  JE=IIEXP(JJ)
                  NREFI=NREFS(IATM(II),ID)
                  IF(IE.GT.0.AND.II.NE.NREFI.AND.ILTLEV(II).LE.0) THEN
                     IF(IRDER.NE.2) THEN
                       APT(IE,ID)=APT(IE,ID)+APFR*DSFDT(ID)
                       APN(IE,ID)=APN(IE,ID)+APFR*DSFDN(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                       DO KK=1,NLVEXP
                          APP(KK,IE,ID)=APP(KK,IE,ID)+APFR*DSFDP(KK,ID)
                       END DO
                     END IF
                  END IF
                  IF(JE.GT.0.AND.JJ.NE.NREFI.AND.ILTLEV(JJ).LE.0.
     *               AND.IABS(IMODL(II)).NE.4) THEN
                     IF(IRDER.NE.2) THEN
                        APT(JE,ID)=APT(JE,ID)-APFR*DSFDT(ID)
                        APN(JE,ID)=APN(JE,ID)-APFR*DSFDN(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                        DO KK=1,NLVEXP
                           APP(KK,JE,ID)=APP(KK,JE,ID)-APFR*DSFDP(KK,ID)
                        END DO
                     END IF
                  END IF
               END IF
   10       CONTINUE
   20    CONTINUE
C            
C        ----------------
C        Line transitions 
C        ----------------
C             
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            II=ILOW(ITR)
            JJ=IUP(ITR)
            IE=IIEXP(II)
            JE=IIEXP(JJ)
            DO 50 ID=1,ND
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 50
               SGW0=PRFLIN(IJ,ID)*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
C              IF(LDER) DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
               IF(LRDER) THEN
                  APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EXX(ID))*SGW0
                  NREFI=NREFS(IATM(II),ID)
                  IF(IE.GT.0.AND.II.NE.NREFI.AND.ILTLEV(II).LE.0) THEN
                     IF(IRDER.NE.2) THEN
                       APT(IE,ID)=APT(IE,ID)+APFR*DSFDT(ID)
                       APN(IE,ID)=APN(IE,ID)+APFR*DSFDN(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                       DO KK=1,NLVEXP
                          APP(KK,IE,ID)=APP(KK,IE,ID)+APFR*DSFDP(KK,ID)
                       END DO
                     END IF
                  END IF
                  IF(JE.GT.0.AND.JJ.NE.NREFI.AND.ILTLEV(JJ).LE.0.
     *               AND.IABS(IMODL(II)).NE.4) THEN
                     IF(IRDER.NE.2) THEN
                        APT(JE,ID)=APT(JE,ID)-APFR*DSFDT(ID)
                        APN(JE,ID)=APN(JE,ID)-APFR*DSFDN(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                        DO KK=1,NLVEXP
                           APP(KK,JE,ID)=APP(KK,JE,ID)-APFR*DSFDP(KK,ID)
                        END DO
                     END IF
                  END IF
               END IF
   50       CONTINUE
   55       CONTINUE
         ENDIF
         IF(NLINES(IJ).LE.0) GO TO 100
C
C        the "overlapping" lines at the given frequency
C
         DO 90 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            II=ILOW(ITR)
            JJ=IUP(ITR)
            IE=IIEXP(II)
            JE=IIEXP(JJ)
            IJ0=IFR0(ITR)
            DO 60 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 70
               END IF
   60       CONTINUE
   70       IJ1=IJ0-1
            A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))*W0
            A2=W0-A1
            DO 80 ID=1,ND
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 80
               SGW0=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
C              IF(LDER) DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
               IF(LRDER) THEN
                  APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EXX(ID))*SGW0
                  NREFI=NREFS(IATM(II),ID)
                  IF(IE.GT.0.AND.II.NE.NREFI.AND.ILTLEV(II).LE.0) THEN
                     IF(IRDER.NE.2) THEN
                       APT(IE,ID)=APT(IE,ID)+APFR*DSFDT(ID)
                       APN(IE,ID)=APN(IE,ID)+APFR*DSFDN(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                       DO KK=1,NLVEXP
                          APP(KK,IE,ID)=APP(KK,IE,ID)+APFR*DSFDP(KK,ID)
                       END DO
                     END IF
                  END IF
                  IF(JE.GT.0.AND.JJ.NE.NREFI.AND.ILTLEV(JJ).LE.0.
     *               AND.IABS(IMODL(II)).NE.4) THEN
                     IF(IRDER.NE.2) THEN
                        APT(JE,ID)=APT(JE,ID)-APFR*DSFDT(ID)
                        APN(JE,ID)=APN(JE,ID)-APFR*DSFDN(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                        DO KK=1,NLVEXP
                           APP(KK,JE,ID)=APP(KK,JE,ID)-APFR*DSFDP(KK,ID)
                        END DO
                     END IF
                  END IF
               END IF
   80       CONTINUE
   90    CONTINUE
  100 CONTINUE
C
C     multiply some quantities by frequency-independent constants
C
      DO ID=1,ND
        REDX(ID)=REDX(ID)*WMM(ID)*DENS1(ID)*DENS1(ID)
        IF(ID.GT.1) REDXM(ID)=REDXM(ID)*WMM(ID)*
     *                        DENS1(ID-1)*DENS1(ID-1)
        FCOOL(ID)=REINT(ID)*FCOOLI(ID)-REDIF(ID)*FLFIX(ID)
        IF(CRSW(ID).NE.UN) THEN
           DO ITR=1,NTRANS
              RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
              RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
              DRDT(ITR,ID)=DRDT(ITR,ID)*CRSW(ID)
           END DO
           IF(LRDER) THEN
           DO II=1,NLVEXP
              APT(II,ID)=APT(II,ID)*CRSW(ID)
              APN(II,ID)=APN(II,ID)*CRSW(ID)
              DO JJ=1,NLVEXP
                 APP(JJ,II,ID)=APP(JJ,II,ID)*CRSW(ID)
              END DO
           END DO
           END IF
        END IF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C
C
      SUBROUTINE ALIST3
C     =================
C
C     Evaluation of all necessary ALI parameters + radiative rates
C     (the routine is analogous to RATES)
C     -- a variant for the full overlap case
C        and for derivatives of the rate matrix w.r.t. populations;
C     tridiagonal operator
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),EHKLF(MFREQL)
      DIMENSION EXX(MDEPTH),RBNU(MDEPTH),RBNUF(MDEPTH)
C
C     zero the rates and other quantities 
C
      DO ID=1,ND
         REIT(ID)=0.
         REIN(ID)=0.
         AREIT(ID)=0.
         AREIN(ID)=0.
         CREIT(ID)=0.
         CREIN(ID)=0.
         REDT(ID)=0.
         REDTM(ID)=0.
         REDTP(ID)=0.
         REDN(ID)=0.
         REDNM(ID)=0.
         REDNP(ID)=0.
         REDX(ID)=0.
         REDXM(ID)=0.
         HEIT(ID)=0.
         HEITM(ID)=0.
         HEITP(ID)=0.
         HEIN(ID)=0.
         HEINM(ID)=0.
         HEINP(ID)=0.
         EHET(ID)=0.
         EHEN(ID)=0.
         ERET(ID)=0.
         EREN(ID)=0.
         FCOOLI(ID)=0.
         FLFIX(ID)=0.
         FLEXP(ID)=0.
         FPRD(ID)=0.
         DO II=1,NLVEXP
            HEIP(II,ID)=0.
            REIP(II,ID)=0.
            AREIP(II,ID)=0.
            CREIP(II,ID)=0.
            REDP(II,ID)=0.
            REDPM(II,ID)=0.
            HEIPM(II,ID)=0.
            REDPP(II,ID)=0.
            HEIPP(II,ID)=0.
            EHEP(II,ID)=0.
            EREP(II,ID)=0.
            APT(II,ID)=0.
            APN(II,ID)=0.
            AAPT(II,ID)=0.
            AAPN(II,ID)=0.
            CAPT(II,ID)=0.
            CAPN(II,ID)=0.
            DO JJ=1,NLVEXP
               APP(JJ,II,ID)=0.
               AAPP(JJ,II,ID)=0.
               CAPP(JJ,II,ID)=0.
            END DO
         END DO
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
            DRDT(ITR,ID)=0.
         END DO
      END DO
C
      DO 100 IJ=1,NFREQ
         IF(IJX(IJ).EQ.-1) GO TO 100
         FR=FREQ(IJ)
         W0=W0E(IJ)
         LRDER=IRDER.GT.0.AND.IJALI(IJ).GT.0
         CALL OPACFD(IJ)
         CALL RTEFR1(IJ)
         CALL ALIFR1(IJ)
C             
C        ---------------------
C        Continuum transitions
C        ---------------------
C          
         DO 20 ID=1,ND
            EXX(ID)=EXP(-HKT1(ID)*FR)
            RBNU(ID)=(RAD1(ID)+BNUE(IJ))*EXX(ID)
            RBNUF(ID)=RBNU(ID)*FR*HKT21(ID)
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               LDER=IIEXP(II).GT.0.AND.ILDER.EQ.0
               IF(IFWOP(II).GE.0) THEN
                  IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
                ELSE
                  SG=SGMG(IMER,ID)
               ENDIF
               SGW0=SG*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
               IF(LRDER) THEN
                  APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EXX(ID))*SGW0
                  IE=IIEXP(II)
                  JJ=IUP(ITR)
                  JE=IIEXP(JJ)
                  NREFI=NREFS(IATM(II),ID)
                  IF(IE.GT.0.AND.II.NE.NREFI.AND.ILTLEV(II).LE.0) THEN
                     IF(IRDER.NE.2) THEN
                       APT(IE,ID)=APT(IE,ID)+APFR*DSFDT(ID)
                       APN(IE,ID)=APN(IE,ID)+APFR*DSFDN(ID)
                       AAPT(IE,ID)=AAPT(IE,ID)+APFR*DSFDTM(ID)
                       AAPN(IE,ID)=AAPN(IE,ID)+APFR*DSFDNM(ID)
                       CAPT(IE,ID)=CAPT(IE,ID)+APFR*DSFDTP(ID)
                       CAPN(IE,ID)=CAPN(IE,ID)+APFR*DSFDNP(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                       DO KK=1,NLVEXP
                          APP(KK,IE,ID)=APP(KK,IE,ID)+APFR*DSFDP(KK,ID)
                          AAPP(KK,IE,ID)=AAPP(KK,IE,ID)+
     *                                   APFR*DSFDPM(KK,ID)
                          CAPP(KK,IE,ID)=CAPP(KK,IE,ID)+
     *                                   APFR*DSFDPP(KK,ID)
                       END DO
                     END IF
                  END IF
                  IF(JE.GT.0.AND.JJ.NE.NREFI.AND.ILTLEV(JJ).LE.0.
     *               AND.IABS(IMODL(II)).NE.4) THEN
                     IF(IRDER.NE.2) THEN
                        APT(JE,ID)=APT(JE,ID)-APFR*DSFDT(ID)
                        APN(JE,ID)=APN(JE,ID)-APFR*DSFDN(ID)
                        AAPT(JE,ID)=AAPT(JE,ID)-APFR*DSFDTM(ID)
                        AAPN(JE,ID)=AAPN(JE,ID)-APFR*DSFDNM(ID)
                        CAPT(JE,ID)=CAPT(JE,ID)-APFR*DSFDTP(ID)
                        CAPN(JE,ID)=CAPN(JE,ID)-APFR*DSFDNP(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                        DO KK=1,NLVEXP
                           APP(KK,JE,ID)=APP(KK,JE,ID)-APFR*DSFDP(KK,ID)
                           AAPP(KK,JE,ID)=AAPP(KK,JE,ID)-
     *                                    APFR*DSFDPM(KK,ID)
                           CAPP(KK,JE,ID)=CAPP(KK,JE,ID)-
     *                                    APFR*DSFDPP(KK,ID)
                        END DO
                     END IF
                  END IF
               END IF
   10       CONTINUE
   20    CONTINUE
C            
C        ----------------
C        Line transitions 
C        ----------------
C             
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            II=ILOW(ITR)
            JJ=IUP(ITR)
            IE=IIEXP(II)
            JE=IIEXP(JJ)
            DO 50 ID=1,ND
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 50
               SGW0=PRFLIN(IJ,ID)*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
               IF(LRDER) THEN
                  APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EXX(ID))*SGW0
                  NREFI=NREFS(IATM(II),ID)
                  IF(IE.GT.0.AND.II.NE.NREFI.AND.ILTLEV(II).LE.0) THEN
                     IF(IRDER.NE.2) THEN
                       APT(IE,ID)=APT(IE,ID)+APFR*DSFDT(ID)
                       APN(IE,ID)=APN(IE,ID)+APFR*DSFDN(ID)
                       AAPT(IE,ID)=AAPT(IE,ID)+APFR*DSFDTM(ID)
                       AAPN(IE,ID)=AAPN(IE,ID)+APFR*DSFDNM(ID)
                       CAPT(IE,ID)=CAPT(IE,ID)+APFR*DSFDTP(ID)
                       CAPN(IE,ID)=CAPN(IE,ID)+APFR*DSFDNP(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                       DO KK=1,NLVEXP
                          APP(KK,IE,ID)=APP(KK,IE,ID)+APFR*DSFDP(KK,ID)
                          AAPP(KK,IE,ID)=AAPP(KK,IE,ID)+
     *                                   APFR*DSFDPM(KK,ID)
                          CAPP(KK,IE,ID)=CAPP(KK,IE,ID)+
     *                                   APFR*DSFDPP(KK,ID)
                       END DO
                     END IF
                  END IF
                  IF(JE.GT.0.AND.JJ.NE.NREFI.AND.ILTLEV(JJ).LE.0.
     *               AND.IABS(IMODL(II)).NE.4) THEN
                     IF(IRDER.NE.2) THEN
                        APT(JE,ID)=APT(JE,ID)-APFR*DSFDT(ID)
                        APN(JE,ID)=APN(JE,ID)-APFR*DSFDN(ID)
                        AAPT(JE,ID)=AAPT(JE,ID)-APFR*DSFDTM(ID)
                        AAPN(JE,ID)=AAPN(JE,ID)-APFR*DSFDNM(ID)
                        CAPT(JE,ID)=CAPT(JE,ID)-APFR*DSFDTP(ID)
                        CAPN(JE,ID)=CAPN(JE,ID)-APFR*DSFDNP(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                        DO KK=1,NLVEXP
                           APP(KK,JE,ID)=APP(KK,JE,ID)-APFR*DSFDP(KK,ID)
                           AAPP(KK,JE,ID)=AAPP(KK,JE,ID)-
     *                                    APFR*DSFDPM(KK,ID)
                           CAPP(KK,JE,ID)=CAPP(KK,JE,ID)-
     *                                    APFR*DSFDPP(KK,ID)
                        END DO
                     END IF
                  END IF
               END IF
   50       CONTINUE
   55       CONTINUE
         ENDIF
         IF(NLINES(IJ).LE.0) GO TO 100
C
C        the "overlapping" lines at the given frequency
C
         DO 90 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            II=ILOW(ITR)
            JJ=IUP(ITR)
            IE=IIEXP(II)
            JE=IIEXP(JJ)
            IJ0=IFR0(ITR)
            DO 60 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 70
               END IF
   60       CONTINUE
   70       IJ1=IJ0-1
            A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))*W0
            A2=W0-A1
            DO 80 ID=1,ND
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 80
               SGW0=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
               DRDT(ITR,ID)=DRDT(ITR,ID)+SGW0*RBNUF(ID)
               IF(LRDER) THEN
                  APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EXX(ID))*SGW0
                  NREFI=NREFS(IATM(II),ID)
                  IF(IE.GT.0.AND.II.NE.NREFI.AND.ILTLEV(II).LE.0) THEN
                     IF(IRDER.NE.2) THEN
                       APT(IE,ID)=APT(IE,ID)+APFR*DSFDT(ID)
                       APN(IE,ID)=APN(IE,ID)+APFR*DSFDN(ID)
                       AAPT(IE,ID)=AAPT(IE,ID)+APFR*DSFDTM(ID)
                       AAPN(IE,ID)=AAPN(IE,ID)+APFR*DSFDNM(ID)
                       CAPT(IE,ID)=CAPT(IE,ID)+APFR*DSFDTP(ID)
                       CAPN(IE,ID)=CAPN(IE,ID)+APFR*DSFDNP(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                       DO KK=1,NLVEXP
                          APP(KK,IE,ID)=APP(KK,IE,ID)+APFR*DSFDP(KK,ID)
                          AAPP(KK,IE,ID)=AAPP(KK,IE,ID)+
     *                                   APFR*DSFDPM(KK,ID)
                          CAPP(KK,IE,ID)=CAPP(KK,IE,ID)+
     *                                   APFR*DSFDPP(KK,ID)
                       END DO
                     END IF
                  END IF
                  IF(JE.GT.0.AND.JJ.NE.NREFI.AND.ILTLEV(JJ).LE.0.
     *               AND.IABS(IMODL(II)).NE.4) THEN
                     IF(IRDER.NE.2) THEN
                        APT(JE,ID)=APT(JE,ID)-APFR*DSFDT(ID)
                        APN(JE,ID)=APN(JE,ID)-APFR*DSFDN(ID)
                        AAPT(JE,ID)=AAPT(JE,ID)-APFR*DSFDTM(ID)
                        AAPN(JE,ID)=AAPN(JE,ID)-APFR*DSFDNM(ID)
                        CAPT(JE,ID)=CAPT(JE,ID)-APFR*DSFDTP(ID)
                        CAPN(JE,ID)=CAPN(JE,ID)-APFR*DSFDNP(ID)
                     END IF
                     IF(IRDER.GT.1) THEN
                        DO KK=1,NLVEXP
                           APP(KK,JE,ID)=APP(KK,JE,ID)-APFR*DSFDP(KK,ID)
                           AAPP(KK,JE,ID)=AAPP(KK,JE,ID)-
     *                                    APFR*DSFDPM(KK,ID)
                           CAPP(KK,JE,ID)=CAPP(KK,JE,ID)-
     *                                    APFR*DSFDPP(KK,ID)
                        END DO
                     END IF
                  END IF
               END IF
   80       CONTINUE
   90    CONTINUE
  100 CONTINUE
C
C     multiply some quantities by frequency-independent constants
C
      DO ID=1,ND
        REDX(ID)=REDX(ID)*WMM(ID)*DENS1(ID)*DENS1(ID)
        IF(ID.GT.1) REDXM(ID)=REDXM(ID)*WMM(ID)*
     *                        DENS1(ID-1)*DENS1(ID-1)
        FCOOL(ID)=REINT(ID)*FCOOLI(ID)-REDIF(ID)*FLFIX(ID)
        IF(CRSW(ID).NE.UN) THEN
           DO ITR=1,NTRANS
              RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
              RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
              DRDT(ITR,ID)=DRDT(ITR,ID)*CRSW(ID)
           END DO
           IF(LRDER) THEN
           DO II=1,NLVEXP
              APT(II,ID)=APT(II,ID)*CRSW(ID)
              APN(II,ID)=APN(II,ID)*CRSW(ID)
              AAPT(II,ID)=AAPT(II,ID)*CRSW(ID)
              AAPN(II,ID)=AAPN(II,ID)*CRSW(ID)
              CAPT(II,ID)=CAPT(II,ID)*CRSW(ID)
              CAPN(II,ID)=CAPN(II,ID)*CRSW(ID)
              DO JJ=1,NLVEXP
                 APP(JJ,II,ID)=APP(JJ,II,ID)*CRSW(ID)
                 AAPP(JJ,II,ID)=AAPP(JJ,II,ID)*CRSW(ID)
                 CAPP(JJ,II,ID)=CAPP(JJ,II,ID)*CRSW(ID)
              END DO
           END DO
           END IF
        END IF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE ALISK1
C     =================
C
C     Simplified routine ALISET for Kantorovich iteration
C     -- a variant for the full overlap case
C
C     Evaluation of all nexcessary ALI parameters + radiative rates
C     (the routine is analogous to RATES)
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),RBNU(MDEPTH)
C
C     zero the rates and other quantities (subr. NULL)
C
      DO ID=1,ND
         FCOOLI(ID)=0.
         FLFIX(ID)=0.
         FPRD(ID)=0.
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
         END DO
      END DO
C
      DO 100 IJ=1,NFREQ
         IF(IJX(IJ).EQ.-1) GO TO 100
         FR=FREQ(IJ)
         W0=W0E(IJ)
         CALL OPACF1(IJ)
         IF(IJEX(IJ).GT.0) THEN
            IJE=IJEX(IJ)
            DO ID=1,ND
               ABSOEX(IJE,ID)=ABSO1(ID)
               EMISEX(IJE,ID)=EMIS1(ID)
               SCATEX(IJE,ID)=SCAT1(ID)
            END DO
         END IF
         CALL RTEFR1(IJ)
         CALL ALIFRK(IJ)
C
C        ---------------------
C        Continuum transitions
C        ---------------------
C
         DO 20 ID=1,ND
            RBNU(ID)=(RAD1(ID)+BNUE(IJ))*EXP(-HKT1(ID)*FR)
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
               ELSE
                 SG=SGMG(IMER,ID)
               ENDIF
               SGW0=SG*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
   10       CONTINUE
   20    CONTINUE
C            
C        ----------------
C        Line transitions 
C        ----------------
C             
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            DO 50 ID=1,ND
               SGW0=PRFLIN(IJ,ID)*W0
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
   50       CONTINUE
         ENDIF
         IF(NLINES(IJ).LE.0) GO TO 100
C
C        the "overlapping" lines at the given frequency
C
         DO 90 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            IJ0=IFR0(ITR)
            DO 60 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 70
               END IF
   60       CONTINUE
   70       IJ1=IJ0-1
            A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))*W0
            A2=W0-A1
            DO 80 ID=1,ND
               SGW0=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RAD1(ID)
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*RBNU(ID)
   80       CONTINUE
   90    CONTINUE
  100 CONTINUE
C
C     multiply some quantities by frequency-independent constants
C
      DO ID=1,ND
         FCOOL(ID)=REINT(ID)*FCOOLI(ID)-REDIF(ID)*FLFIX(ID)
         IF(CRSW(ID).NE.UN) THEN
         DO ITR=1,NTRANS
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
         END IF
      END DO
      RETURN
      END

C
C
C     ****************************************************************
C
C

      SUBROUTINE ALISEP
C     =================
C
C     Evaluation of all necessary ALI parameters + radiative rates
C     in the preconditioned formalism
C     (the routine is analogous to RATESP)
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),EHKLF(MFREQL)
C
C     zero the rates and other quantities
C
      DO ID=1,ND
         REIT(ID)=0.
         REIN(ID)=0.
         AREIT(ID)=0.
         AREIN(ID)=0.
         CREIT(ID)=0.
         CREIN(ID)=0.
         REDT(ID)=0.
         REDTM(ID)=0.
         REDN(ID)=0.
         REDNM(ID)=0.
         REDX(ID)=0.
         REDXM(ID)=0.
         HEIT(ID)=0.
         HEITM(ID)=0.
         HEIN(ID)=0.
         HEINM(ID)=0.
         FCOOLI(ID)=0.
         FLFIX(ID)=0.
         FLEXP(ID)=0.
         FPRD(ID)=0.
         DO II=1,NLVEXP
            HEIP(II,ID)=0.
            REIP(II,ID)=0.
            AREIP(II,ID)=0.
            CREIP(II,ID)=0.
            REDP(II,ID)=0.
            REDPM(II,ID)=0.
            HEIPM(II,ID)=0.
         END DO
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
            DRDT(ITR,ID)=0.
         END DO
      END DO
C
C     ---------------------
C     Continuum transitions
C     ---------------------
C
      LROSS=NDRE.LE.0.AND.ITER.EQ.1.OR.LFIN
      IF(HMIX0.GT.0.) LROSS=.TRUE.
      DO 30 IJ=1,NFREQC
         FR=FREQ(IJ)
         CALL OPACFD(IJ)
         CALL RTEFR1(IJ)
         IF(LROSS) CALL ROSSTD(IJ,NFREQC)
         CALL ALIFR1(IJ)
C
         DO 20 ID=1,ND
            EHK=EXP(-HKT1(ID)*FR)
            EHKF=EHK*FR
            ALA=ALI1(ID)/ABSO1(ID)
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               LDER=IIEXP(II).GT.0.AND.ILDER.EQ.0
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
                 W0=W0EC(IJ)
                 IF(IFR1(ITR).EQ.IJ) W0=W0EC(IJ)-WCHCE(IJ)
                 SGW0=SG*W0
               ELSE
                 SG=SGMG(IMER,ID)
                 IF(SG.LE.0.) GO TO 10
                 W0=W0E(IJ)
                 IF(IFR1(ITR).EQ.IJ)
     *              W0=W0E(IJ)*(W(IJ)-WCH(IJ))/W(IJ)
                 SGW0=SG*W0
               ENDIF
               ABSOC=ABTRA(ITR,ID)-EMTRA(ITR,ID)*EHK
               SLIN=BNUE(IJ)*EMTRA(ITR,ID)*EHK/ABSOC
               RLAM=ABSOC*SG*ALA
               RADRES=RAD1(ID)-RLAM*SLIN
               BNURES=BNUE(IJ)*(UN-RLAM)
               SRBNU=SGW0*(RADRES+BNURES)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SRBNU*EHK
               IF(LDER) DRDT(ITR,ID)=DRDT(ITR,ID)+SRBNU*EHKF
   10       CONTINUE
   20    CONTINUE
   30 CONTINUE
C            
C     ----------------
C     Line transitions 
C     ----------------
C             
      DO 200 ITR=1,NTRANS
         IF(LINEXP(ITR)) GO TO 200
         I=ILOW(ITR)
         J=IUP(ITR)
         I0=IFR0(ITR)
         I1=IFR1(ITR)
         NFL=I1-I0+1
         DO 120 IJ=I0,I1
            IJL=IJ-I0+1
            CALL OPACFD(IJ)
            CALL RTEFR1(IJ)
            CALL ALIFR1(IJ)
C
            DO 110 ID=1,ND
               RADL(IJL,ID)=RAD1(ID)
               ABSALI(IJL,ID)=ALI1(ID)/ABSO1(ID)
               DO ICDW=1,NCDW
                  DWFL(IJL,ID,ICDW)=DWF1(ICDW,ID)
               ENDDO
  110       CONTINUE
  120    CONTINUE
C
         DO 190 ID=1,ND
            POPIL=ABTRA(ITR,ID)
            POPJL=EMTRA(ITR,ID)*EXP(-HKT1(ID)*FR0(ITR))
            POPIJ=POPIL-POPJL
            IF(POPIJ.NE.0.) SLIN=BNTR(ITR)*POPJL/POPIJ
            SRRU=0.
            SRRD=0.
            DO 130 IJ=I0,I1
               IJL=IJ-I0+1
               SGW=PRFLIN(IJ,ID)*W(IJ)
               RLAM=PRFLIN(IJ,ID)*POPIJ*ABSALI(IJL,ID)
               RADRES=RADL(IJL,ID)-RLAM*SLIN
               BNURES=BNTR(ITR)*(UN-RLAM)
               SRRU=SRRU+SGW*RADRES
               SRRD=SRRD+SGW*(RADRES+BNURES)
  130       CONTINUE
            RRU(ITR,ID)=SRRU*XPIF(ITR)
            RRD(ITR,ID)=SRRD*XPIF(ITR)
c                
c          correction to the continuum rates
c                
            DO 140 IJL=1,NFL
               IJ=IJL+I0-1
               EHKL(IJL)=EXP(-HKT1(ID)*FREQ(IJ))
               EHKLF(IJL)=EHKL(IJL)*FREQ(IJ)
  140       CONTINUE
C
            DO 180 IBF=1,NTRANC
               ITRC=ITRBF(IBF)
c              IF(FRQMX(ITR).LT.FREQ(IFR1(ITRC))) GO TO 180
               II=ILOW(ITRC)
               IF(IFWOP(II).LT.0) GO TO 180
               LDER=IIEXP(II).GT.0
               if(ilder.gt.0) lder=.false.
               SRRU=0.
               SRRD=0.
               SDRT=0.
               DO 160 IJ=I0,I1
                  IJL=IJ-I0+1
                  SG=CROSS(IBF,IJ)
                  IF(SG.LE.0.) GO TO 160
                  IF(MCDW(ITRC).GE.1) SG=SG*DWFL(IJL,ID,MCDW(ITRC))
                  SGW0=SG*W0E(IJ)
                  ABSOC=ABTRA(ITRC,ID)-EMTRA(ITRC,ID)*EHKL(IJL)
                  SLIN=BNUE(IJ)*EMTRA(ITRC,ID)*EHKL(IJL)/ABSOC
                  RLAM=ABSOC*SG*ABSALI(IJL,ID)
                  RADRES=RADL(IJL,ID)-RLAM*SLIN
                  BNURES=BNUE(IJ)*(UN-RLAM)
                  SRRU=SRRU+SGW0*RADRES
                  SRRD=SRRD+SGW0*(RADRES+BNURES)*EHKL(IJL)
                  SRBNU=SGW0*(RADRES+BNURES)
                  SDRT=SDRT+SRBNU*EHKLF(IJL)
  160          CONTINUE
               RRU(ITRC,ID)=RRU(ITRC,ID)+SRRU
               RRD(ITRC,ID)=RRD(ITRC,ID)+SRRD
               IF(LDER) DRDT(ITR,ID)=DRDT(ITR,ID)+SDRT
  180       CONTINUE
  190    CONTINUE
  200 CONTINUE
C
C     multiply some quantities by frequency-independent constants
C
      DO ID=1,ND
         REDX(ID)=REDX(ID)*WMM(ID)*DENS1(ID)*DENS1(ID)
         IF(ID.GT.1) REDXM(ID)=REDXM(ID)*WMM(ID)*
     *                         DENS1(ID-1)*DENS1(ID-1)
         IF(IFALI.LT.3) RETURN
         FCOOL(ID)=REINT(ID)*FCOOLI(ID)-REDIF(ID)*FLFIX(ID)
         DO IBF=1,NTRANC
            DRDT(ITR,ID)=DRDT(ITR,ID)*HKT21(ID)*CRSW(ID)
         END DO
      END DO
      DO ITR=1,NTRANS
         DO ID=1,ND
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
         IF(LINE(ITR)) THEN
            DO ID=1,ND
               RRD(ITR,ID)=RRD(ITR,ID)*EXP(-HKT1(ID)*FR0(ITR))
            END DO
         END IF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE ALISKP
C     =================
C
C     Simplified routine ALISEP for Kantorovich iteration

C     Evaluation of all nexcessary ALI parameters + radiative rates
C     in the preconditioned version
C     (the routine is analogous to RATESP)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION EHKL(MFREQL),EHKLF(MFREQL)
C
C     zero the rates and other quantities
C
      DO ID=1,ND
         FCOOLI(ID)=0.
         FLFIX(ID)=0.
         FPRD(ID)=0.
         DO ITR=1,NTRANS
            RRU(ITR,ID)=0.
            RRD(ITR,ID)=0.
         END DO
      END DO
C
C     ---------------------
C     Continuum transitions
C     ---------------------
C
      DO 30 IJ=1,NFREQC
         FR=FREQ(IJ)
         CALL OPACF1(IJ)
         IF(IJEX(IJ).GT.0) THEN
            IJE=IJEX(IJ)
            DO ID=1,ND
               ABSOEX(IJE,ID)=ABSO1(ID)
               EMISEX(IJE,ID)=EMIS1(ID)
               SCATEX(IJE,ID)=SCAT1(ID)
            END DO
         END IF
         CALL RTEFR1(IJ)
         CALL ALIFRK(IJ)
C
         DO 20 ID=1,ND
            EHK=EXP(-HKT1(ID)*FR)
            ALA=ALI1(ID)/ABSO1(ID)
            DO 10 IBF=1,NTRANC
               ITR=ITRBF(IBF)
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 10
               II=ILOW(ITR)
               JJ=IUP(ITR)
               IF(IPZERO(II,ID).NE.0.OR.IPZERO(JJ,ID).NE.0) GO TO 10
	       JC=ITRA(JJ,II)
               ICDW=MCDW(ITR)
               IMER=IMRG(II)
               IF(IFWOP(II).GE.0) THEN
                 IF(ICDW.GE.1) SG=SG*DWF1(ICDW,ID)
                 W0=W0EC(IJ)
                 IF(IFR1(ITR).EQ.IJ) W0=W0EC(IJ)-WCHCE(IJ)
                 SGW0=SG*W0
               ELSE
                 SG=SGMG(IMER,ID)
                 IF(SG.LE.0.) GO TO 10
                 W0=W0E(IJ)
                 IF(IFR1(ITR).EQ.IJ)
     *              W0=W0E(IJ)*(W(IJ)-WCH(IJ))/W(IJ)
                 SGW0=SG*W0
               ENDIF
               ABSOC=ABTRA(ITR,ID)-EMTRA(ITR,ID)*EHK
               SLIN=BNUE(IJ)*EMTRA(ITR,ID)*EHK/ABSOC
               RLAM=ABSOC*SG*ALA
               RADRES=RAD1(ID)-RLAM*SLIN
               BNURES=BNUE(IJ)*(UN-RLAM)
               RRU(ITR,ID)=RRU(ITR,ID)+SGW0*RADRES
               RRD(ITR,ID)=RRD(ITR,ID)+SGW0*(RADRES+BNURES)*EHK
   10       CONTINUE
   20    CONTINUE
   30 CONTINUE
C            
C     ----------------
C     Line transitions 
C     ----------------
C             
      DO 200 ITR=1,NTRANS
         IF(LINEXP(ITR)) GO TO 200
         I=ILOW(ITR)
         J=IUP(ITR)
         I0=IFR0(ITR)
         I1=IFR1(ITR)
         NFL=I1-I0+1
         DO 120 IJ=I0,I1
            IJL=IJ-I0+1
            CALL OPACF1(IJ)
            IF(IJEX(IJ).GT.0) THEN
               IJE=IJEX(IJ)
               DO ID=1,ND
                  ABSOEX(IJE,ID)=ABSO1(ID)
                  EMISEX(IJE,ID)=EMIS1(ID)
                  SCATEX(IJE,ID)=SCAT1(ID)
               END DO
            END IF
            CALL RTEFR1(IJ)
            CALL ALIFRK(IJ)
            DO 110 ID=1,ND
               RADL(IJL,ID)=RAD1(ID)
               ABSALI(IJL,ID)=ALI1(ID)/ABSO1(ID)
               DO ICDW=1,NCDW
                  DWFL(IJL,ID,ICDW)=DWF1(ICDW,ID)
               ENDDO
  110       CONTINUE
  120    CONTINUE
C
         DO 190 ID=1,ND
            POPIL=ABTRA(ITR,ID)
            POPJL=EMTRA(ITR,ID)
            POPIJ=POPIL-POPJL
            SLIN=BNTR(ITR)*POPJL/POPIJ
            IF(POPIJ.NE.0.) SLIN=BNTR(ITR)*POPJL/POPIJ
            SRRU=0.
            SRRD=0.
            DO 130 IJ=I0,I1
               IJL=IJ-I0+1
               SGW=PRFLIN(IJ,ID)*W(IJ)
               RLAM=PRFLIN(IJ,ID)*POPIJ*ABSALI(IJL,ID)
               RADRES=RADL(IJL,ID)-RLAM*SLIN
               BNURES=BNTR(ITR)*(UN-RLAM)
               SRRU=SRRU+SGW*RADRES
               SRRD=SRRD+SGW*(RADRES+BNURES)
  130       CONTINUE
            RRU(ITR,ID)=SRRU*XPIF(ITR)
            RRD(ITR,ID)=SRRD*XPIF(ITR)
c                
c          correction to the continuum rates
c                
            DO 140 IJL=1,NFL
               IJ=IJL+I0-1
               EHKL(IJL)=EXP(-HKT1(ID)*FREQ(IJ))
  140       CONTINUE
C
            DO 180 IBF=1,NTRANC
               ITRC=ITRBF(IBF)
c              IF(FRQMX(ITR).LT.FREQ(IFR1(ITRC))) GO TO 180
               II=ILOW(ITRC)
               IF(IFWOP(II).LT.0) GO TO 180
               SRRU=0.
               SRRD=0.
               DO 160 IJ=I0,I1
                  IJL=IJ-I0+1
                  SG=CROSS(IBF,IJ)
                  IF(SG.LE.0.) GO TO 160
                  IF(MCDW(ITRC).GE.1) SG=SG*DWFL(IJL,ID,MCDW(ITRC))
                  SGW0=SG*W0E(IJ)
                  ABSOC=ABTRA(ITRC,ID)-EMTRA(ITRC,ID)*EHKL(IJL)
                  SLIN=BNUE(IJ)*EMTRA(ITRC,ID)*EHKL(IJL)/ABSOC
                  RLAM=ABSOC*SG*ABSALI(IJL,ID)
                  RADRES=RADL(IJL,ID)-RLAM*SLIN
                  BNURES=BNUE(IJ)*(UN-RLAM)
                  SRRU=SRRU+SGW0*RADRES
                  SRRD=SRRD+SGW0*(RADRES+BNURES)*EHKL(IJL)
  160          CONTINUE
               RRU(ITRC,ID)=RRU(ITRC,ID)+SRRU
               RRD(ITRC,ID)=RRD(ITRC,ID)+SRRD
  180       CONTINUE
  190    CONTINUE
  200 CONTINUE
C
C     multiply some quantities by frequency-independent constants
C
      DO ID=1,ND
         FCOOL(ID)=REINT(ID)*FCOOLI(ID)-REDIF(ID)*FLFIX(ID)
      END DO
      DO ITR=1,NTRANS
         DO ID=1,ND
            RRU(ITR,ID)=RRU(ITR,ID)*CRSW(ID)
            RRD(ITR,ID)=RRD(ITR,ID)*CRSW(ID)
         END DO
         IF(LINE(ITR)) THEN
            DO ID=1,ND
               RRD(ITR,ID)=RRD(ITR,ID)*EXP(-HKT1(ID)*FR0(ITR))
            END DO
         END IF
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE DOPGAM(ITR,ID,T,DOP,AGAM)
C     ====================================
C
C     Doppler width and the Voigt damping parameter for the line ITR
C
C     Input:
C      ITR   - index of transition
C      ID    - depth index
C      T     - temperature
C     Output:
C      DOP   - Doppler width
C      AGAM  - total damping parameter (in units of Doppler widths;
C              ie. = gam/4pi/DOP, where gam is the "physical" damping
C              parameter expresed in circular frequencies)
C
C     Damping parameter is calculated only for transitions with
C     |IPROF(ITR)| = 1 (ie. those for which either Voigt or some non-
C     standard profile is assumed)
C     Determination of AGAM:
C     is controlled by input parameters transmitted by COMMON/VOIPAR:
C
C      GAMR(IP)         - for > 0  - has the meaning of natural damping
C                                    parameter (=Einstein coefficient for
C                                    spontaneous emission)
C                             = 0  - classical natural damping assumed
C                             < 0  - damping is given by a non-standard,
C                                    user supplied procedure GAMSP
C      STARK1(IP)       -     = 0  - Stark broadening neglected
C                             < 0  - scaled classical expression
C                                    (ie gam = -STARK1 * classical Stark)
C                             > 0  - Stark broadening given by
C                                    n(el)*[STARK1*T**STARK2 + STARK3]
C      STRAK2, STARK3   -   see above
C      VDWH(IP)         -   .le.0  - Van der Waals broadening neglected
C                             > 0  - scaled classical expression
C
C      the corresponding index IP is given by ITRA(IUP(ITR),ILOW(ITR))
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (BOL2=2.76108D-16, CIN=UN/2.997925D10)
      COMMON/VOIPAR/GAMR(MVOIGT),STARK1(MVOIGT),STARK2(MVOIGT),
     *              STARK3(MVOIGT),VDWH(MVOIGT)
C
      J=IUP(ITR)
      IAT=IATM(J)
      FR=FR0(ITR)
      IE=IEL(J)
      AM=BOL2/AMASS(IAT)*T
      AGAM=0.
C
C     Doppler width
C
      DOP=FR*CIN*SQRT(AM+VTURB(ID)*VTURB(ID))
C
C     -----------------
C     damping parameter - only for IPROF = 1
C
      IF(IABS(IPROF(ITR)).NE.1) RETURN
      IP=ITRA(J,ILOW(ITR))
      ANE=ELEC(ID)
      IF(GAMR(IP)) 30,20,10
C
C     Natural (radiation) broadening
C
   10 AGAM=GAMR(IP)
      GO TO 40
   20 AGAM=2.47342D-8*FR*FR
      GO TO 40
C
C     Non-standard expression - for the total damping parameter, not
C     only for radiation damping
C
   30 CALL GAMSP(ITR,T,ANE,AGAM)
      GO TO 100
C
C     Stark broadening
C
   40 IF(STARK1(IP)) 50,70,60
   50 ANFF=IZ(IE)*IZ(IE)*EH/ENION(J)
      AGAM=AGAM-1.D-8*ANFF**2.5*ANE*STARK1(IP)
      GO TO 70
   60 AGAM=AGAM+ANE*(STARK1(IP)*T**STARK2(IP)+STARK3(IP))
C
C     Van der Waals broadening
C
   70 IF(VDWH(IP).LE.0.) GO TO 100
      AH=DENS(ID)/WMM(ID)/YTOT(ID)
      AHE=AH*0.1
      IF(IELHE1.NE.0) AHE=AHE*10.*ABUND(IATHE,ID)
      AGAM=AGAM+T**0.3*(AH+0.42*AHE)*VDWH(IP)
C
C     Total damping parameter in units of Doppler widths
C
  100 AGAM=AGAM/DOP/12.566370614
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE GAMSP(ITR,T,ANE,AGAM)
C     ================================
C
C     Non-standard expression for the damping parameter  -
C     a user-supplied procedure
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      AGAM=0.
      RETURN
      END
C
C
C     ****************************************************************
C
C
      FUNCTION PROFIL(FR,A,DOP,ITR,IP,ID)
C     ===================================
C
C     Standard absorption profile - normalized to unity
C
C     Input:
C     FR  - frequency
C     A   - Voigt damping parameter
C     DOP - Doppler width
C     ITR - transition index
C     ID  - depth index
C
C     Profile is evaluated differently for different IP=IPROF(ITR):
C     IP = 0   -  Doppler profile
C     IP = 1   -  Voigt profile
C     IP = 2   -  approximate Stark (+ Doppler) profile for hydrogen lines;
C                 however, the routine is called with IP=2 only from
C                 START, i.e. for the initialization
C     IP > 9   -  non-standard profile, given by a user-supplied
C                 procedure PROFSP
C
C     V - frequency displacement from the line center in units of
C         Doppler width
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/STRAUX/BETAD,ADH,DIVH
      COMMON/STDPAR/ELSTD,IDSTD
      PARAMETER (PISQ=1.77245385090551D0,PISQ1=UN/PISQ)
C
      PROFIL=0.
      V=(FR-FR0(ITR))/DOP
      IPA=IABS(IP)
      IF(IPA.EQ.0) THEN
         IF(V.LE.13.) PROFIL=EXP(-V*V)*PISQ1
       ELSE IF(IPA.EQ.1) THEN
         PROFIL=VOIGT(V,A)*PISQ1
       ELSE IF(IPA.EQ.2) THEN
         IF(ID.GT.0) THEN
            ANE=ELEC(ID)
          ELSE
            ANE=1.e9*grav
         END IF
         F00=1.25D-9*EXP(0.666666667*LOG(ANE))
         II=NQUANT(ILOW(ITR))
         JJ=NQUANT(IUP(ITR))
         CALL STARK0(II,JJ,XKIJ,WL0,FIJ)
         FXK=F00*XKIJ
         DBETA=WL0*WL0/2.997925D18/FXK
         BETAD=DOP*DBETA
         CALL DIVSTR
         BETA=DBETA*ABS(FR-FR0(ITR))
         SG=STARKA(BETA)*BETAD
         PROFIL=SG
       ELSE IF(IPA.GT.10) THEN
         PROFIL=PROFSP(FR,A,DOP,ITR,ID)
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      FUNCTION VOIGT(V,AGAM)
C     ======================
C
C     Voigt function
C     Procedure after Matta and Reichel, 1971, Math.Comp. 25, 339.
C
      INCLUDE 'IMPLIC.FOR'
      DIMENSION HN(12),EN(12)
      PARAMETER (PI=3.141592653589793D0, M=12, HH=0.5D0, UN=1.D0)
      PARAMETER (PISQ=1.77245385090551D0,PISQ1=UN/PISQ)
      DATA ICOMP /0/
      SAVE EN,HN,PH,HP,ICOMP
C
C     Initialization of auxiliary quantities
C
      IF(ICOMP.NE.0) GO TO 20
      HP=HH*PISQ1
      PH=PI/HH
      DO 10 I=1,M
         XI=I
         U=XI*XI*HH*HH
         EN(I)=EXP(-U)
   10    HN(I)=4.D0*U
      ICOMP=1
   20 CONTINUE
C
C     Main term
C
      AGAM1=UN/AGAM
      X=V*AGAM1
      T=0.25D0*AGAM1*AGAM1
      X2=X*X
      X4=4.D0*X2
      S1=UN+X2
      S2=UN-X2
      U0=0.
      DO 30 I=1,M
         S0=HN(I)*T
         U=EN(I)/((S2+S0)*(S2+S0)+X4)
         U0=U0+U*(S1+S0)
   30 CONTINUE
      S2=UN/S1
      U0=HP*(S2+2.D0*U0)
C
C     Correction term
C
      IF(T.LT.0.25D0/PH/PH) GO TO 50
      U=X/2.D0/T
      A=COS(U)
      B=SIN(U)
      TSQ1=UN/SQRT(T)
      S1=PH*TSQ1
      S2=S1*X
      C=EXP(-S1)-COS(S2)
      D=SIN(S2)
      T4=0.25D0/T
      U=EXP(-X2*T4-S1+T4)*PISQ*TSQ1/(C*C+D*D)
      U0=U0+U*(A*C-B*D)
   50 VOIGT=U0*AGAM1*PISQ1
      RETURN
      END
C
C
C     ****************************************************************
C
C
      FUNCTION PROFSP(FR,A,DOP,ITR,ID)
C     ================================
C
C     Non-standard absorption profile - normalized to unity;
C     a user supplied procedure
C
C     Input:
C     FR  - frequency
C     A   - Voigt damping parameter
C     DOP - Doppler width
C     ITR - transition index
C     ID  - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
C
      PROFSP=0.
      IP=IPROF(ITR)
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE LINSET(ITR,IUNIT,IFRQ0,IFRQ1,XMAX,DOP,AGAM)
C     ======================================================
C
C     Set up frequency points and weights for a line
C     Auxiliary procedure for START
C
C     Input:
C      ITR    -  index of the transition
C      INMOD  -  mode of evaluating frequency points in the line
C             =  0  - means that frequency points and weights have
C                     already been read;
C             ne 0  - frequency points and weights will be evaluated:
C             the meaning of the individual values:
C             =  1  - equidistant frequencies, trapezoidal integration
C             =  2  - equidistant frequencies, Simpson integration
C             =  3  - modified Simpson integration, i.e. a series of
C                     3-point Simpson integrations with each subsequent
C                     integration interval doubled, until the whole
C                     integartion area is covered
C             =  4  - frequencies (in units of standard x) and weights
C                     (for integration over x) are read;
C
C      XMAX   >  0  - means that the line is assumed symmetric around the
C                     center, frequency points are set up between x=0 and
C                     x=XMAX, where x is frequency difference from the
C                     line center in units of the standard Doppler width
C             <  0  - frequency points are set between x=XMAX and x=-XMAX
C      DOP    -  Doppler width
C      AGAM   -  damping parameter (for lines with Voigt or non-standard
C                profile only)
C
C     Output (to COMMON/FRQEXP)
C      FREQ   -  array of frequencies
C                Note that LINSET calculates values of frequencies only
C                for frequency points between IFR0(ITR) and IFR1(ITR).
C      W      -  corresponding integration weights
C      PROF   -  corresponding values of absorption profile
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/STDPAR/ELSTD,IDSTD
      COMMON/STRAUX/BETAD,ADH,DIVH
      PARAMETER (BOL2=2.76108D-16, CIN=UN/2.997925D10, OS0=0.02654,
     *           F0C=1.25D-9, TTW=2./3., PISQ1=UN/1.77245385090551D0,
     *           C18IN=UN/CAS)
      DIMENSION X(MFREQL),W0(MFREQL)
C
      IF(ITR.EQ.0) GO TO 200
      INMOD=INTMOD(ITR)
      IF(INMOD.EQ.0) RETURN
      INMOD0=MOD(IABS(INMOD),10)
      IJ0=IFR0(ITR)
      IJ1=IFR1(ITR)
      N=IJ1-IJ0+1
      X0=0.
      IF(XMAX.LT.0) X0=XMAX
      M=(N-1)/2
C
      X(1)=0.
      W0(1)=UN
      IF(N.LE.1) GO TO 100
C
      IF(INMOD0.LE.2) THEN
C
C     Trapezoidal integration
C
         HH=ABS(X0+XMAX)/(N-1)
         DO 20 I=1,N
   20       X(I)=X0+(I-1)*HH
         IF(INMOD0.EQ.2) GO TO 40
         DO 30 I=1,N
   30       W0(I)=HH
         W0(1)=HALF *HH
         W0(N)=HALF *HH
         GO TO 100
C
C     Ordinary Simpson integration
C
   40    HH=HH/3.D0
         IF(MOD(N,2).NE.1) CALL STOP(' conflict in Simpson integr.')
         DO 50 I=1,M
            W0(2*I)=4.D0*HH
            W0(2*I+1)=2.D0*HH
   50    CONTINUE
         W0(1)=HH
         W0(N)=HH
       ELSE IF(INMOD0.EQ.3) THEN
C
C     Modified Simpson integration - a set of 3-point Simpson
C      integrations with continuosly increasing distance between the
C      integration points (schematically, distances between points are
C      h,h,2h,2h,4h,4h, etc.)
C
         TWI=UN
         MM=M
         IF(MOD(N,2).NE.1) CALL STOP('conflict in m.Simpson integr.')
         IF(XMAX.LT.0) MM=M/2
         DO 70 I=1,MM
            TWI=TWI*2.D0
            X(2*I+1)= TWI-UN
            X(2*I)=TWI-UN-TWI/4.D0
            W0(2*I)=2.D0*TWI
            W0(2*I+1)=1.5D0*TWI
   70    CONTINUE
         TWN=TWI-UN
         TWA=ABS(XMAX)/TWN
         HH=TWA/6.D0
         DO 80 I=1,MM
            X(2*I+1)=X(2*I+1)*TWA
            X(2*I)=X(2*I)*TWA
            W0(2*I)=W0(2*I)*HH
            W0(2*I+1)=W0(2*I+1)*HH
   80    CONTINUE
         W0(1)=HH
         W0(N)=TWI*HH/2.D0
         X(1)=0.
         IF(M.EQ.MM) GO TO 100
         IF(MOD(N,4).NE.1) CALL STOP('conflict in m.Simpson integr.')
         DO 90 I=1,M
            X(M+1+I)=X(I+1)
            W0(M+1+I)=W0(I+1)
   90    CONTINUE
         M2=2*(M+1)
         DO 91 I=1,M
            X(I)=-X(M2-I)
            W0(I)=W0(M2-I)
   91    CONTINUE
         X(M+1)=0.
         W0(M+1)=2.D0*HH
C
C        frequencies (in units of standard x) and weights
C        (for integration over x) are read;
C
       ELSE IF(INMOD0.EQ.4) THEN
         READ(IUNIT,*) (X(I),I=1,N),(W0(I),I=1,N)
      END IF
C
C     For all types of integration:
C     set up arrays  FR,WW,PRF
C
  100 S=OSC0(ITR)*0.02654D0
      IP0=IPROF(ITR)
      IP=IABS(IP0)
      DO 110 I=1,N
         FREQ(I+IJ0-1)=FR0(ITR)-DOP*X(I)
         W(I+IJ0-1)=DOP*W0(I)
         PROF(I+IJ0-1)=PROFIL(FREQ(I+IJ0-1),AGAM,DOP,ITR,IP,0)*S/DOP
  110 CONTINUE
C
C     for IPROF(ITR) ge 0  -  endpoint(s) of the line profile are forced to
C                             have zero cross-section
C
      IF(IP0.GE.0) THEN
         IF(XMAX.LT.0.) THEN
            PROF(IJ0)=0.
            PROF(IJ1)=0.
          ELSE
            PROF(IJ1)=0.
         END IF
      END IF
C
C     Recalculation of quadrature weights in order to enforce exact
C     normalization of the integral (absorption profile * weights)
C
      SUM=0.
      DO 115 I=1,N
         SUM=SUM+PROF(I+IJ0-1)*W(I+IJ0-1)
  115 CONTINUE
      SUM=S/SUM
      DO 120 I=1,N
  120    W(I+IJ0-1)=W(I+IJ0-1)*SUM
C
C     reset the switch INTMOD
C
      IF(IPROF(ITR).GE.0) THEN
         IF(XMAX.LT.0.) THEN
            INTMOD(ITR)=-2
          ELSE
            INTMOD(ITR)=-1
         END IF
       ELSE
         IF(XMAX.LT.0.) THEN
            INTMOD(ITR)=2
          ELSE
            INTMOD(ITR)=1
         END IF
      END IF
      IF(ABS(INMOD).GE.10) INTMOD(ITR)=0
C
C     set up WSUB - subtraction weight, 
C                   which is used only in the non-overlapping option
C
      IF(XMAX.GT.0.) THEN
         WSUB(IJ1)=TWO*DOP*XMAX
       ELSE
         WSUB(IJ0)=-DOP*XMAX
         WSUB(IJ1)=WSUB(IJ0)
      END IF
      IF(INDEXP(ITR).NE.0) THEN
        CALL IJALIS(ITR,IFRQ0,IFRQ1)
        IF(IOVER.EQ.0) CALL CORRQW(ITR)
      ENDIF
      RETURN
C
  200 CONTINUE
      DO 220 IT=1,NTRANS
         IF(.NOT.LINE(IT)) GO TO 220
         IP=IABS(IPROF(IT))
         IF(IP.NE.2) GO TO 220
         IAT=IATM(ILOW(IT))
         AM=BOL2/AMASS(IAT)*TEFF
         DOPP=FR0(IT)*CIN*SQRT(AM+VTB*VTB)
         DOP1=UN/DOPP
	 ANE=ELSTD
	 IF(ANE.LE.0.) ANE=1.E14
         F00=F0C*EXP(TTW*LOG(ANE))
	 II=NQUANT(ILOW(IT))
	 JJ=NQUANT(IUP(IT))
	 CALL STARK0(II,JJ,XKIJ,WL0,FIJ)
	 FXK=F00*XKIJ
	 DBETA=WL0*WL0*C18IN/FXK
	 BETAD=DOPP*DBETA
	 FID=OS0*FIJ*DBETA
         FID0=OS0*(OSC0(IT)-FIJ)*DOP1*PISQ1
	 CALL DIVSTR
         DO 210 IJ=IFR0(IT),IFR1(IT)
	    BETA=DBETA*ABS(FREQ(IJ)-FR0(IT))
	    SG=STARKA(BETA)*FID
            SG0=0.
            V=(FREQ(IJ)-FR0(IT))*DOP1
            IF(ABS(V).LE.13.) SG0=EXP(-V*V)*FID0
	    PROF(IJ)=SG+SG0
  210    CONTINUE
  220 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE LINPRO(ITR,ID,PRF)
C     =============================
C
C     Line profile coefficient for the line ITR;
C     for "classical" lines (i.e. those which are not represented by ODF's).
C     It is either calculated for each depth (if LCOMP=true), or is
C     just taken as constant, depth-independent quantity, already
C     calculated in START and stored in array PROF
C
C     Input:  ITR    - index of transition
C             ID     - depth index
C     Output: PRF    - array of absorption profile
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      PARAMETER (BOL2=2.76108D-16, CIN=UN/2.997925D10, OS0=0.02654,
     *           F0C=1.25D-9, TTW=2./3., PISQ1=UN/1.77245385090551D0,
     *           C18IN=UN/CAS)
      COMMON/STRAUX/BETAD,ADH,DIVH
      DIMENSION PRF(MFREQL)
C
      MODE=IABS(INDEXP(ITR))
      IJ0=IFR0(ITR)
      IJ1=IFR1(ITR)
      INTM0=INTMOD(ITR)
      INTM=IABS(INTM0)
      IP=IABS(IPROF(ITR))
C
C     Doppler width
C
      IF(LCOMP(ITR).OR.IP.EQ.2) THEN
         IAT=IATM(ILOW(ITR))
         AM=BOL2/AMASS(IAT)*TEMP(ID)
         DOP=FR0(ITR)*CIN*SQRT(AM+VTURB(ID)*VTURB(ID))
         DOP1=UN/DOP
      END IF
      IF(LCOMP(ITR).AND.IP.EQ.1.OR.IP.LT.0) THEN
         CALL DOPGAM(ITR,ID,TEMP(ID),DOP,AGAM)
         DOP1=UN/DOP
      END IF
      S=OSC0(ITR)*OS0
      XNORM=PISQ1*S*DOP1
C
      DO 10 IJ=IJ0,IJ1
         PRF(IJ-IJ0+1)=PROF(IJ)
   10 CONTINUE
      IF(.NOT.LCOMP(ITR)) RETURN
c
c     evaluation of the depth-dependent profile
C
      IF(IP.EQ.0) THEN
         DO 20 IJ=IJ0,IJ1
            V=(FREQ(IJ)-FR0(ITR))*DOP1
            IF(ABS(V).LE.13.) PRF(IJ-IJ0+1)=EXP(-V*V)*XNORM
   20    CONTINUE
       ELSE IF(IP.EQ.1) THEN
         DO 30 IJ=IJ0,IJ1
            V=(FREQ(IJ)-FR0(ITR))*DOP1
            PRF(IJ-IJ0+1)=VOIGT(V,A)*XNORM
   30    CONTINUE
       ELSE IF(IP.EQ.2) THEN
C
C     for IP=2 - approximate Stark profile for hydrogen lines, 
C                evaluate necessary frequency-independent quantities 
C
	 ANE=ELEC(ID)
         F00=F0C*EXP(TTW*LOG(ANE))
	 II=NQUANT(ILOW(ITR))
	 JJ=NQUANT(IUP(ITR))
	 CALL STARK0(II,JJ,XKIJ,WL0,FIJ)
	 FXK=F00*XKIJ
	 DBETA=WL0*WL0*C18IN/FXK
	 BETAD=DOP*DBETA
	 FID=OS0*FIJ*DBETA
         FID0=OS0*(OSC0(ITR)-FIJ)*DOP1*PISQ1
	 CALL DIVSTR
C
C        loop over frequencies
C
         DO 40 IJ=IJ0,IJ1
	    BETA=DBETA*ABS(FREQ(IJ)-FR0(ITR))
	    SG=STARKA(BETA)*FID
            SG0=0.
            V=(FREQ(IJ)-FR0(ITR))*DOP1
            IF(ABS(V).LE.13.) SG0=EXP(-V*V)*FID0
	    PRF(IJ-IJ0+1)=SG+SG0
   40    CONTINUE
       ELSE IF(IP.GE.10) THEN
         DO 50 IJ=IJ0,IJ1
            PRF(IJ-IJ0+1)=PROFSP(FREQ(IJ),AGAM,DOP,ITR,ID)
   50    CONTINUE
      END IF
C
C     if required, force the profile at the endpoints to zero
C
      IF(INTM0.EQ.-1) THEN
         PRF(IJ1-IJ0+1)=0.
       ELSE IF(INTM0.EQ.-2) THEN
         PRF(1)=0.
         PRF(IJ1-IJ0+1)=0.
      END IF
      RETURN
      END
C
C
C
C     ****************************************************************
C
C
      SUBROUTINE RTEnew
C     =================
C
C     Solution of the radiative transfer equation - frequency by
C     frequency - for the known source function.
C     Determination of the radiation field and variable Eddington
C     factors.
C
C     The numerical method used:
C     improved Feautrier scheme (Rybicki & Hummer 1991, A&A 245, 171.)
C
C     The overall matrix system is solved by the standard
C     Gaussian elimination, analogous to that described in SOLVE
C     (auxiliary matrix D is called ALF in SOLVE; auxiliary vector ANU
C     is called BET in SOLVE)
C
C     Output transmitted through COMMON/SURFAC
C      FLUX   - surface flux (precisely, second moment H), for all
C               frequencies
C      FH     - second Eddington factor (at the surface only)
C      Q0     - integral over angles [1 - exp(taumin/amu)]*amu*d(amu),
C               where amu is the cosine of angle, and taumin is the
C               optical depth at the first depth point
C      U0     - derivative of Q0 wrt taumin;
C               for "fixed" frequencies, U0 has the meaning of
C               absorption coefficient * second moment H
C               ( a quantity needed for lower boundary condition of the
C               hydrostatic equilibrium equation, specifically for
C               accounting for an effect of fixed-option transitions)
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      PARAMETER (NMU=3)
      PARAMETER (SIXTH=UN/6.D0,
     *           THIRD=UN/3.D0)
      DIMENSION DT(MDEPTH),ST0(MDEPTH),AB0(MDEPTH),
     *          RDJ(MDEPTH),RDK(MDEPTH),FKK(MDEPTH),
     *          AMU(NMU),WTMU(NMU),
     *          ALI0(MDEPTH),
     *          A(MDEPTH),B(MDEPTH),C(MDEPTH),D(MDEPTH),
     *          E(MDEPTH),F(MDEPTH),U(MDEPTH),Z(MDEPTH),
     *          ALRH(MDEPTH),ALRM(MDEPTH),ALRP(MDEPTH)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
C
C     angle points (AMU) and angular integration weights (WTMU)
C
      DATA AMU/.887298334620742D0,.5D0,.112701665379258D0/,
     * WTMU/.277777777777778D0,.444444444444444D0,.277777777777778D0/
C
      ISPL=ISPLIN
      IF(ISPLIN.GE.5.AND.IJALI(IJ).GE.0) THEN
         IF(IRTE.EQ.0) THEN
            CALL RTEDF1(IJ)
          ELSE
            CALL RTEDF2(IJ)
         END IF
         ISPLIN=ISPL
         RETURN
      END IF
C
      FR=FREQ(IJ)
C
C     total source function
C  
      DO 20 ID=1,ND
         AB0(ID)=ABSO1(ID)
         ST0(ID)=(EMIS1(ID)+SCAT1(ID)*EMEL1(ID)*RAD1(ID))/
     *           AB0(ID)
         RDJ(ID)=0.
         RDK(ID)=0.
         ALRH(ID)=0.
   20 CONTINUE
      AH=0.
C
C     optical depth scale
C
      DO 30 ID=1,ND-1
         DT(ID)=DELDM(ID)*(AB0(ID+1)*DENS1(ID+1)+AB0(ID)*DENS1(ID))
   30 CONTINUE
C
      U0=0.
      QQ0=0.
      US0=0.
      TAUMIN=AB0(1)*DEDM1
C
C     Allowance for wind blanketing
C
      ALB1=0.
      IF(IWINBL.GT.0) ALB1=TWO*ALBE(IJ)/(UN+ALBE(IJ))
C
C     quantities for the lower boundary condition
C
      FR15=FR*1.D-15
      BNU=BN*FR15*FR15*FR15
      PLAND=BNU/(EXP(HK*FR/TEMP(ND))-UN)
      DPLAN=BNU/(EXP(HK*FR/TEMP(ND-1))-UN)
      DPLAN=(PLAND-DPLAN)/DT(ND-1)
c
c     loop over angle poits
c
      DO 100 I=1,NMU
         AMU2=AMU(I)*AMU(I)*WTMU(I)
C
C     ************** Forward elimination
C
C        Upper boundary condition
C
         ID=1
         P0=0.
         EX=UN
C
C        allowance for non-zero optical depth at the first depth point
C
         IF(IWINBL.EQ.0) THEN
            TAMM=TAUMIN/AMU(I)
            EX=EXP(-TAMM)
            P0=UN-EX
            QQ0=QQ0+P0*AMU(I)*WTMU(I)
            U0=U0+EX*WTMU(I)
            US0=US0+P0/TAMM*WTMU(I)
         END IF
C
         DTP1=AMU(I)/DT(ID)
         CC=TWO*DTP1
         C(ID)=CC*DTP1
         B(ID)=UN+CC+C(ID)
         A(ID)=0.
         VL=ST0(ID)*(UN+CC*P0)
         F(ID)=(B(ID)-C(ID))/C(ID)
         D(ID)=UN/(UN+F(ID))
         Z(ID)=VL/B(ID)
C
C     Normal depth points  1 < ID < ND
C
         DO 60 ID=2,ND-1
            DTM1=DT(ID-1)/AMU(I)
            DTP1=DT(ID)/AMU(I)
            DT0=TWO/(DTM1+DTP1)
            A(ID)=DT0/DTM1
            C(ID)=DT0/DTP1
            B(ID)=UN+A(ID)+C(ID)
            VL=ST0(ID)
            F(ID)=(B(ID)-C(ID)-A(ID)*(UN-F(ID-1)*D(ID-1)))/C(ID)
            D(ID)=UN/(UN+F(ID))
            Z(ID)=(VL+A(ID)*Z(ID-1))*D(ID)/C(ID)
   60    CONTINUE
C
C        Lower boundary condition
C
         ID=ND
         DTM1=AMU(I)/DT(ID-1)
         AA=TWO*DTM1
         A(ID)=AA*DTM1
         B(ID)=UN+AA+A(ID)
         C(ID)=0.
         VL=ST0(ID)+AA*(PLAND+AMU(I)*DPLAN)
         Z(ID)=(VL+A(ID)*Z(ID-1))/(B(ID)-A(ID)*D(ID-1))
C
C ***************** Backsolution
C
         U(ND)=Z(ND)
         E(ND)=A(ND)
         RDJ(ND)=RDJ(ND)+WTMU(I)*U(ND)
         RDK(ND)=RDK(ND)+AMU2*U(ND)
         DO 70 ID=ND-1,1,-1
            U(ID)=U(ID+1)*D(ID)+Z(ID)
            RDJ(ID)=RDJ(ID)+WTMU(I)*U(ID)
            RDK(ID)=RDK(ID)+AMU2*U(ID)
C            E(ID)=A(ID)/(B(ID)-C(ID)*E(ID+1))
C            ALRH(ID)=ALRH(ID)+WTMU(I)*D(ID)/C(ID)/(UN-D(ID)*E(ID+1))
   70    CONTINUE
         AH=AH+AMU(I)*WTMU(I)*U(1)
c
c     end of the loop over angle points
c
  100 CONTINUE
C
C     solution of the transfer equation
C     Variables:
C     RDJ     - mean intensity
C     FKK     - Eddington factor f(K) = K/J
C     FH      - the "surface" Eddington factor
C
      DO 110 ID=1,ND
         FKK(ID)=RDK(ID)/RDJ(ID)
  110 CONTINUE
      FLUX(IJ)=AH
      FH(IJ)=AH/RDJ(1)-HALF*ALB1
C
C ********************
C
C     Again solution of the transfer equation, now with Eddington
C     FKK and FH determined above, to insure strict consistency of the
C     radiation field and Eddington factors
C
C     Upper boundary condition
C
      ID=1
      DTP1=DT(ID)
      CC=TWO/DTP1
      GA=CC/DTP1
      C(ID)=GA*FKK(ID+1)
      B(ID)=UN+CC*FH(IJ)+GA*FKK(ID)
      A(ID)=0.
      VL=ST0(ID)*(UN+CC*QQ0)
      F(ID)=(B(ID)-C(ID))/C(ID)
      D(ID)=UN/(UN+F(ID))
      Z(ID)=VL/B(ID)
C
C     Normal depth points
C
      DO 280 ID=2,ND-1
         DTM1=DTP1
         DTP1=DT(ID)
         DT0=TWO/(DTP1+DTM1)
         AL=DT0/DTM1
         GA=DT0/DTP1
         A(ID)=AL*FKK(ID-1)
         C(ID)=GA*FKK(ID+1)
         B(ID)=(AL+GA)*FKK(ID)+UN
         VL=ST0(ID)
         F(ID)=(B(ID)-C(ID)-A(ID)*(UN-F(ID-1)*D(ID-1)))/C(ID)
         D(ID)=UN/(UN+F(ID))
         Z(ID)=(VL+A(ID)*Z(ID-1))*D(ID)/C(ID)
  280 CONTINUE
C
C     Lower boundary condition
C
      ID=ND
      AA=TWO/DT(ID-1)
      AL=AA/DT(ID-1)
      A(ID)=AL*FKK(ID-1)
      B(ID)=UN+AA*HALF+AL*FKK(ID)
      C(ID)=0.
      VL=ST0(ID)+AA*(HALF*PLAND+THIRD*DPLAN)
      Z(ID)=(VL+A(ID)*Z(ID-1))/(B(ID)-A(ID)*D(ID-1))
      RAD1(ID)=Z(ID)
      FAK1(ID)=FKK(ND)
      E(ID)=A(ID)
C
C     Backsolution
C
      DO 290 ID=ND-1,1,-1
         E(ID)=A(ID)/(B(ID)-C(ID)*E(ID+1))
         RAD1(ID)=Z(ID)+D(ID)*RAD1(ID+1)
         FAK1(ID)=FKK(ID)
         ALRH(ID)=D(ID)/C(ID)/(UN-D(ID)*E(ID+1))
         ALRM(ID)=0.
         ALRP(ID)=0.
  290 CONTINUE
      FLUX(IJ)=FH(IJ)*RAD1(1)
c
C        evaluate approximate Lambda operator
C
C        a) Rybicki-Hummer Lambda^star operator (diagonal)
C           (for JALI = 1)
C
c         if(iter.ge.10) go to 500
         IF(IFALI.GT.5) THEN
           DO 301 ID=1,ND
              ALIM1(ID)=0.
              ALIP1(ID)=0.
  301      CONTINUE
         ENDIF
         IF(JALI.EQ.1) THEN
         DO 310 ID=1,ND 
            ALI1(ID)=ALRH(ID)
  310    CONTINUE
         ali1(nd-1)=rad1(nd-1)/st0(nd-1)
         ali1(nd)=rad1(nd)/st0(nd)
C
C        for IFALI>5:
C        tridiagonal Rybicki-Hummer operator (off-diagonal terms)
C
         IF(IFALI.GE.6) THEN
         ALIP1(1)=ALRH(2)*D(1)
         DO 340 ID=2,ND-1 
            ALIM1(ID)=ALRH(ID-1)*E(ID)
            ALIP1(ID)=ALRH(ID+1)*D(ID)
  340    CONTINUE
         ALIM1(ND)=ALRH(ND-1)*E(ND)
         ALIM1(nd)=0.
         ALIM1(nd-1)=0.
         ALIP1(nd)=0.
         ALIP1(nd-1)=0.
         END IF
c
C        b) diagonal Olson-Kunasz Lambda^star operator, 
C           (for JALI = 2)
C
         ELSE IF(JALI.EQ.2) THEN
         DO 360 ID=1,ND-1
            ALI0(ID)=0.
            DO 350 I=1,NMU
               X=DT(ID)/AMU(I)
               ALI0(ID)=ALI0(ID)+(UN-EXP(-X))/X*WTMU(I)
  350       CONTINUE
  360    CONTINUE
         DO 370 ID=2,ND-1
            ALI1(ID)=UN-HALF*(ALI0(ID)+ALI0(ID-1))
  370    CONTINUE
         ALI1(1)=UN-HALF*(ALI0(1)+US0)
         ALI1(ND)=UN-ALI0(ND-1)
         ali1(nd-1)=rad1(nd-1)/st0(nd-1)
         ali1(nd)=rad1(nd)/st0(nd)
	 END IF
      ISPLIN=ISPL
C
C     store quantities for explicit (linearized) frequencies
C
      IF(IJEX(IJ).LE.0) RETURN
      IJE=IJEX(IJ)
      DO ID=1,ND
         RADEX(IJE,ID)=RAD1(ID)
         FAKEX(IJE,ID)=FAK1(ID)
      END DO
      Q0(IJE)=QQ0
      UU0(IJE)=U0
      RETURN
      END
C
C
C     ****************************************************************
C
C
 
      FUNCTION SIGK(FR,ITR,MODE)
C     ==========================
C
C     Photoionization cross-sections
C
C     Input: FR  -  frequency
C            ITR -  index of the transition
C            MODE-  =0 - cross-section equal to zero longward of edge
C                   >0 - cross-section non-zero (extrapolated) longward
C                        of edge
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      PARAMETER (SIH0=2.815D29, E10=2.3025851)
      PARAMETER (MPAG=MLEVEL/6+1)
      CHARACTER*10 TYPLEV(MLEVEL)
      CHARACTER*4 TYPION(MION)
      COMMON/PHOSET/S0(MLEVEL),ALF(MLEVEL),BET(MLEVEL),GAM(MLEVEL),
     *              IBF(MLEVEL)
      COMMON/PRINTP/NPGPOP,IIPR(6,MPAG),TYPLEV,TYPION
      COMMON/TOPCS/CTOP(MFIT,MCROSS), ! sigma=alog10(sigma/10^-18) of fit point
     +             XTOP(MFIT,MCROSS)  ! x = alog10(nu/nu0) of fit point 
      DIMENSION XFIT(MFIT) , ! local array containing x     for OP data
     +          SFIT(MFIT)   ! local array containing sigma for OP data
C
      PEACH(X,S,A,B)  =A*X**S*(B+X*(1.-B))*1.D-18
      HENRY(X,S,A,B,C)=A*X**S*(C+X*(B-2.*C+X*(1.+C-B)))*1.D-18
C
      SIGK=0.
      IF(MODE.EQ.0.AND.FR.LT.FR0(ITR)) RETURN
C
C     IC is the index of the given transition in the special numbering
C        of continua (given by ITRA(IUP,ILOW))
C     IBF(IC) is the switch controlling the mode of evaluation of the
C        cross-section:
C      = 0  hydrogenic cross-section, with Gaunt factor set to 1
C      = 1  hydrogenic cross-section with exact Gaunt factor
C      = 2  Peach-type expression (see function PEACH)
C      = 3  Henry-type expression (see function HENRY)
C      = 4  Butler fit formula (polynomial fits to the OP results);
C      = 5  Verner fit formula (OP results & HDS calculations at high
C              energies); ONLY for GROUND states;
C      = 7  hydrogenic cross-section with Gaunt factor after Werner
C      = 9  Opacity project fits (routine TOPBAS - interpolations)
C      > 100 - cross-sections extracted form TOPBASE, for several points
C           In this case, IBF-100 is the number of points
C      < 0  non-standard, user supplied expression (user should update
C           subroutine SPSIGK)
C
C      for H- : for any IBF > 0  - standard expression
C      for He I:
C       for IBF = 11 or = 13  -  Opacity Project cross section
C                Seaton-Ferney's cubic fits, Hummer's procedure (HEPHOT)
C           IBF = 11  means that the multiplicity S=1 (singlet)
C           IBF = 13  means that the multiplicity S=3 (triplet)
C       for IBF = 10  - cross section, based on Opacity Project, but
C                       appropriately averaged for an averaged level
C                       (see explanation in SBFHE1)
C       for IBF = 21  Koester's fit (A&A 149, 423) for He I ground state
C                     photoionization
C
C
      IC=ITRA(IUP(ITR),ILOW(ITR))
      IB=IBF(IC)
      II=ILOW(ITR)
      IQ=NQUANT(II)
      IE=IEL(ILOW(ITR))
      IF(IB.LT.0) GO TO 60
      IF(IE.EQ.IELHM) GO TO 40
      IF(IE.EQ.IELHE1.AND.IB.GE.10.AND.IB.LE.13) GO TO 50
      IF(IE.EQ.IELHE1.AND.IB.EQ.21) GO TO 55
      CH=IZ(IE)*IZ(IE)
      IQ5=IQ*IQ*IQ*IQ*IQ
C
      IF(IB.EQ.0) THEN
C
C        hydrogenic expression (for IBF = 0)
C
         SIGK=SIH0/FR/FR/FR*CH*CH/IQ5
C
C        exact hydrogenic - with Gaunt factor (for IBF=1)
C
       ELSE IF(IB.EQ.1) THEN
         SIGK=SIH0/FR/FR/FR*CH*CH/IQ5
         SIGK=SIGK*GAUNT(IQ,FR/CH)
       ELSE IF(IB.EQ.2) THEN
C
C        Peach-type formula (for IBF=2)
C   
         FREL=FR0(ITR)/FR
         IF(GAM(IC).GT.0) THEN
            IF(GAM(IC).LT.1.E6) THEN
              FR00=2.997925E18/GAM(IC)
            ELSE
              FR00=GAM(IC)
            END IF
            IF(FR.LT.FR00) RETURN        
         FREL=FR00/FR
         END IF 
         SIGK=PEACH (FREL,S0(IC),ALF(IC),BET(IC))
       ELSE IF(IB.EQ.3) THEN
C
C        Henry-type formula (for IBF=3)
C
         FREL=FR0(ITR)/FR
         SIGK=HENRY(FREL,S0(IC),ALF(IC),BET(IC),GAM(IC))
C
C        Butler expression
C
       ELSE IF(IB.EQ.4) THEN
         FREL=FR0(ITR)/FR
         XL=LOG(FREL)
         SL=S0(IC)+XL*(ALF(IC)+XL*BET(IC))
         SIGK=EXP(SL)
C
C        Verner expression
C
       ELSE IF(IB.EQ.5) THEN
         SIGK=VERNER(FR,ITR)
C
C     exact hydrogenic - with Gaunt factor from K Werner (for IBF=7)
C
       ELSE IF(IB.EQ.7) THEN
         IQ5=IQ*IQ*IQ*IQ*IQ
         SIGK=SIH0/(FR*FR*FR)*CH*CH/IQ5*GNTK(IQ,FR/CH)
C
C     selected Opacity Project data (for IBF=9)
C     (c.-s. evaluated by routine TOPBAS which needs an input file RBF.DAT)
C
       ELSE IF(IB.EQ.9) THEN
         SIGK=TOPBAS(FR,FR0(ITR),TYPLEV(II))
C
C     other Opacity Project data (for IBF>100)
C     (c.-s. evaluated by interpolating from direct input data)
C
       ELSE IF(IB.GT.100) THEN
         NFIT=IB-100
         X = LOG10(FR/FR0(ITR))
         SIGM=0.
         IF(X.GE.XTOP(1,IC)) THEN
           DO IFIT = 1,NFIT
              XFIT(IFIT) = XTOP(IFIT,IC)
              SFIT(IFIT) = CTOP(IFIT,IC)
           END DO
           SIGM  = YLINTP (X,XFIT,SFIT,NFIT,MFIT)
           SIGM  = 1.D-18*EXP(E10*SIGM)
         ENDIF
         SIGK=SIGM
      END IF
      RETURN
C
C     special expression for H-
C
   40 SIGK=SBFHMI(FR)
      RETURN
C
C     He I cross-sections
C
   50 SIGK=SBFHE1(II,IB,FR)
      RETURN
   55 X=LOG(CAS/FR)
      SIGK=EXP(-58.229+X*(4.3965-X*0.22134))/G(II)
      RETURN
C
C     non-standard, user supplied form of cross-section (for IBF < 0)
C
   60 CALL SPSIGK(ITR,IB,FR,SIGSP)
      SIGK=SIGSP
      RETURN
      END
C
C
C     ****************************************************************
C
C
      FUNCTION VERNER(FR,ITR)
C     =======================
C
C     Photoionization cross-sections for ground states
C       of atoms and ions. Analytical fits from:
C           Verner D.A. et al. 1996, ApJ 465
C           Verner & Yakovlev 1995, A&AS 109, 125
C
C     10-July-1996: Version for H to Si, S, Ar, Ca and Fe.
C          No test on threshold energy as given by Verner et al.
C          to avoid inconsistencies with limits FR0 as read by Tlusty
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
C
      PARAMETER (HHEV=H/1.6022D-12,T18=1.D-18,MVER=105)
      DIMENSION S0(MVER),E0(MVER),EMX(MVER)
      DIMENSION Y0(MVER),Y1(MVER),YW(MVER),YA(MVER),PV(MVER)
      DIMENSION S95(MVER),E95(MVER),Y95(MVER),YW95(MVER),P95(MVER)
      DIMENSION IV0(14)
C
      DATA IV0/0,1,3,6,10,15,21,28,36,45,55,66,78,91/
      DATA S0/5.475D4,9.492D2,1.369D4,6.245D1,3.201D2,6.083D3,2.932D5,
     +        2.678D2,5.458D2,3.422D3,5.466D0,1.859D4,5.393D1,2.846D2,
     +        2.190D3,5.027D2,8.709D0,1.539D4,1.068D2,2.344D2,1.521D3,
     +        8.235D2,1.944D0,9.375D-1,1.690D4,8.376D1,1.519D2,1.117D3,
     +        1.745D3,5.967D1,6.753D2,8.659D-1,1.642D4,6.864D1,1.329D2,
     +        8.554D2,3.803D3,8.013D1,1.541D2,3.165D0,4.690D-1,1.157D4,
     +        6.930D1,1.039D2,6.759D2,4.287D3,1.583D3,5.708D0,1.685D3,
     +        2.430D0,9.854D-1,1.198D4,5.631D1,6.695D1,5.475D2,1.601D0,
     +        1.04D3,1.885D3,2.33D0,2.346D1,7.101D1,1.609D0,7.215D3,
     +        4.729D1,3.995D1,4.525D2,1.372D8,3.278D0,5.377D2,1.394D3,
     +        1.728D0,2.185D0,3.104D0,6.344D-2,9.008D2,4.427D1,6.14D1,
     +        3.802D2,7.195D0,6.948D-2,4.915D0,1.513D1,2.925D2,1.962D-2,
     +        1.889D2,2.35D-1,4.982D-1,1.774D4,3.388D1,4.036D1,3.239D2,
     +        2.506D1,4.14D0,5.79D-4,6.083D0,8.863D-1,7.293D1,6.68D-2,
     +        3.477D-1,1.465D-1,1.95D-1,1.992D4,2.539D1,4.754D1,2.793D2/
      DATA E0/0.4298,13.61,1.72,3.107,20.06,3.871,9.539,1.181,17.6,
     +        6.879,0.5213,2.869,1.041,33.36,10.75,2.144,0.4058,4.614,
     +        3.506,46.24,15.48,4.034,0.06128,0.242,5.494,4.471,69.43,
     +        21.08,1.24,1.386,0.1723,0.2044,2.854,7.824,87.09,27.54,
     +        12.97,1.763,2.542,0.7744,0.7286,4.008,2.563,113.1,34.85,
     +        4.87,12.47,0.7753,5.566,1.248,1.499,4.888,10.03,158.6,
     +        43.04,6.139,8.203,10.69,0.669,5.408,48.46,2.096,153.5,
     +        13.91,226.8,52.11,11.97,8.139,10.86,29.12,0.9762,1.711,
     +        3.57,0.4884,34.82,14.52,204.2,62.03,13.81,0.2048,10.27,
     +        3.13,24.14,0.3483,2.636,0.4866,1.842,8.044,23.55,273.8,
     +        72.81,23.17,2.556,0.1659,1.288,0.7761,63.05,0.3277,0.7655,
     +        0.3343,0.8787,12.05,35.6,275.2,84.47/
      DATA EMX/5.D4,5.D4,5.D4,64.39,5.D4,5.D4,11.93,12.99,5.D4,5.D4,
     +         194.0,209.8,227.4,5.D4,5.D4,291.0,307.6,328.9,352.2,
     +         5.D4,5.D4,404.8,423.6,447.3,475.3,504.3,5.D4,5.D4,
     +         538.0,558.1,584.0,614.4,649.1,683.7,5.D4,5.D4,694.0,
     +         712.2,739.2,770.9,809.1,850.2,890.5,5.D4,5.D4,870.1,
     +         883.1,913.1,948.0,987.3,1031.,1078.,1125.,5.D4,5.D4,
     +         38.14,1074.,1110.,1143.,1185.,1230.,1281.,1335.,1386.,
     +         5.D4,5.D4,54.9,65.69,1317.,1356.,1400.,1449.,1501.,
     +         1558.,1618.,1675.,5.D4,5.D4,80.4,89.97,102.6,1588.,
     +         1634.,1688.,1739.,1799.,1862.,1929.,1992.,5.D4,5.D4,
     +         106.,118.6,131.1,146.6,1887.,1946.,2001.,2058.,2125.,
     +         2194.,2268.,2336.,5.D4,5.D4/
      DATA Y0/0.,0.4434,0.,0.,0.,0.,8.278D-4,0.,0.,0.,1.319D1,4.96D-3,
     +        0.,0.,0.,1.133,4.929D1,4.378D-3,0.,0.,0.,8.598D-1,4.28D2,
     +        1.877D2,6.415D-3,0.,0.,0.,8.698,2.131D1,3.839D-3,3.328D2,
     +        3.036D-2,0.,0.,0.,1.701D-4,1.715D1,1.641D1,9.531D1,
     +        1.506D2,2.071D-2,0.,0.,0.,4.236D-2,1.52,7.654D1,5.149,
     +        9.169D1,1.042D2,2.536D-2,0.,0.,0.,0.,3.375,3.725,1.383D2,
     +        2.204D1,9.603D-4,9.94D1,0.1667,0.,0.,0.,0.,0.,4.86,
     +        9.402D-1,1.276D2,1.007D2,5.452D1,5.348D2,5.444,0.,0.,0.,
     +        2.041D-1,9.149D1,0.,3.994D1,3.495,5.675D-2,3.552D1,
     +        5.704D2,1.719D2,2.538D-2,0.,0.,0.,1.672D-5,6.634,9.613D1,
     +        0.,2.009D2,1.115,1.149D-2,3.85D2,1.036D3,4.528D2,1.99D-2,
     +        0.,0.,0./
      DATA Y1/0.,2.136,0.,0.,0.,0.,1.269D-2,0.,0.,0.,4.556,3.4D-2,0.,
     +        0.,0.,1.607,3.234,2.528D-2,0.,0.,0.,2.325,2.03D1,3.999,
     +        1.937D-2,0.,0.,0.,1.271D-1,1.503D-2,4.569D-1,4.285D1,
     +        5.554D-2,0.,0.,0.,1.345D-2,7.724D-1,5.124,9.781,2.574D-1,
     +        3.998D-2,0.,0.,0.,5.873,1.084D-1,2.023,6.687,3.702D-1,
     +        1.435,4.417D-2,0.,0.,0.,0.,4.01,0.2279,4.26,0.7577,
     +        6.378D-3,3.278,1.766D-2,0.,0.,0.,0.,0.,3.722,0.1135,
     +        3.979,1.729,2.078,3.997D-3,7.918D-2,0.,0.,0.,0.4753,
     +        0.6565,0.,4.803,0.2701,0.2768,8.223D-3,0.158,6.595,
     +        1.203D-2,0.,0.,0.,0.4207,0.1272,0.6442,0.,4.537,8.051D-2,
     +        0.6396,8.999D-2,0.2936,1.015,1.007D-2,0.,0.,0./
      DATA YW/0.,2.039,0.,0.,0.,0.,3.655D-1,0.,0.,0.,1.887D1,3.503,0.,
     +        0.,0.,9.157D-2,2.093,5.922,0.,0.,0.,9.097D-2,1.043D1,
     +        1.85,7.904,0.,0.,0.,7.589D-2,1.934D-2,1.191D-1,3.143,
     +        2.836D1,0.,0.,0.,2.17D-3,5.103D-1,1.115,6.812,2.57D-1,
     +        2.411D1,0.,0.,0.,2.434D-1,6.558D-2,4.633D-1,8.29D-3,
     +        6.855D-1,1.656,2.811D1,0.,0.,0.,0.,2.328,8.579D-2,0.7365,
     +        0.9275,1.285D-2,1.895,0.9121,0.,0.,0.,0.2805,0.,2.604,
     +        4.326D-2,0.809,0.6325,1.422,0.6666,2.751,0.,0.,0.,0.3166,
     +        0.4615,0.,5.342,0.1,8.839,1.836,0.2773,0.6945,29.53,0.,
     +        0.,0.,0.2837,1.57,0.8626,0.,1.303,2.989D-3,3.28,1.476D-3,
     +        1.646,0.4489,2.392D1,0.,0.,0./
      DATA YA/3.288D1,1.469D0,3.288D1,1.501D1,7.391D0,3.288D1,4.301D-1,
     +        5.645D0,1.719D1,3.288D1,8.618D0,1.783D0,1.767D1,2.163D1,
     +        3.288D1,6.216D1,1.261D2,1.737D0,1.436D1,2.183D1,3.288D1,
     +        8.033D1,8.163D2,2.788D2,1.714D0,3.297D1,2.627D1,3.288D1,
     +        3.784D0,3.175D1,3.852D2,4.931D2,1.792D0,3.210D1,2.535D1,
     +        3.288D1,2.587D0,1.667D1,5.742D1,1.099D2,1.400D2,1.848D0,
     +        7.547D1,2.657D1,3.288D1,5.798D0,3.935D0,6.725D1,6.409D2,
     +        1.066D2,1.350D2,1.788D0,3.628D1,3.352D1,3.288D1,6.148D3,
     +        8.259D0,3.613D0,1.205D2,2.913D1,3.945D1,2.473D2,3.886D-1,
     +        3.889D1,5.315D1,3.288D1,2.228D-1,4.341D7,9.779D0,2.895D0,
     +        9.184D1,9.350D1,6.060D1,5.085D2,1.823D0,3.826D1,2.778D1,
     +        3.288D1,1.621D3,5.675D2,1.990D6,1.674D1,6.973D0,1.856D1,
     +        1.338D2,7.216D2,2.568D2,1.653D0,3.432D1,3.567D1,3.288D1,
     +        2.057D1,1.337D1,1.474D2,1.356D6,1.541D2,1.558D2,4.132D1,
     +        3.733D2,1.404D3,7.461D2,1.582D0,3.307D1,2.848D1,3.288D1/
      DATA PV/2.963,31.88,2.963,4.895,2.916,2.963,10.52,11.7,3.157,
     +        2.963,17.28,16.18,9.54,2.624,2.963,5.101,8.578,15.93,
     +        7.457,2.581,2.963,3.928,8.773,9.156,17.06,6.003,2.315,
     +        2.963,17.64,8.943,6.822,8.785,26.47,5.495,2.336,2.963,
     +        7.275,10.5,6.614,9.203,9.718,24.46,6.448,2.255,2.963,
     +        8.355,7.81,10.05,3.056,8.999,8.836,25.5,5.585,2.002,
     +        2.963,3.839,7.362,9.803,9.714,8.26,2.832,7.681,8.476,
     +        5.265,1.678,2.963,15.74,3.61,7.117,6.487,10.06,9.202,
     +        8.857,9.385,14.44,5.46,2.161,2.963,3.642,9.049,3.477,
     +        11.8,6.724,20.84,6.204,8.659,8.406,26.55,5.085,1.915,
     +        2.963,3.546,11.91,13.36,3.353,9.98,2.4,16.06,8.986,
     +        8.503,8.302,24.25,4.728,2.135,2.963/
      DATA S95/5.475D4,4.47D3,1.369D4,1.564D2,3.201D2,6.083D3,1.306D2,
     +         9.796D1,5.458D2,3.422D3,9.698D1,1.037D2,8.605D1,2.846D2,
     +         2.19D3,7.421D1,6.649D1,8.067D1,8.111D1,2.344D2,1.521D3,
     +         4.748D1,5.002D1,5.235D1,7.046D1,7.304D1,1.519D2,1.117D3,
     +         3.237D1,3.584D1,3.939D1,4.123D1,5.735D1,6.029D1,1.329D2,
     +         8.554D2,2.295D1,4.798D1,3.144D1,5.302D1,3.680D1,4.668D1,
     +         4.890D1,1.039D2,6.759D2,1.664D1,2.783D1,2.943D1,3.027D1,
     +         3.097D1,3.232D1,5.011D1,3.719D1,6.695D1,5.475D2,2.486D2,
     +         1.889D1,3.466D1,2.551D1,3.753D1,2.654D1,2.745D1,3.613D1,
     +         3.850D1,3.995D1,4.525D2,2.023D2,2.049D2,1.877D1,2.212D1,
     +         2.350D1,3.278D1,3.407D1,2.721D1,3.699D1,3.290D1,6.140D1,
     +         3.802D2,1.735D2,1.842D2,1.990D2,2.494D1,2.572D1,2.786D1,
     +         2.653D1,2.989D1,2.857D1,3.223D1,3.272D1,4.036D1,3.239D2,
     +         1.532D2,1.832D2,2.197D2,2.326D2,3.100D1,2.398D1,2.474D1,
     +         3.135D1,2.641D1,3.343D1,2.832D1,3.880D1,4.754D1,2.793D2/
      DATA E95/0.4298,5.996,1.72,27.4,20.06,3.871,40.93,47.59,17.6,
     +         6.879,61.55,59.84,65.92,33.36,10.75,86.55,91.13,83.7,
     +         84.12,46.24,15.48,127.,124.2,122.,107.,106.,69.43,21.08,
     +         177.4,169.,162.,159.3,137.7,135.4,87.09,27.54,239.,166.,
     +         205.5,160.3,192.5,173.7,171.1,113.1,34.85,314.4,245.2,
     +         238.7,236.,234.3,230.8,188.6,219.3,158.6,43.04,36.55,
     +         326.6,242.6,284.1,236.,280.6,277.7,246.4,240.6,226.8,
     +         52.11,49.37,49.4,360.7,334.5,325.9,277.7,274.2,308.2,
     +         267.1,285.8,204.2,62.03,64.45,61.54,60.16,348.2,343.2,
     +         326.8,339.6,319.3,330.5,311.7,313.7,273.8,72.81,78.08,
     +         71.54,66.52,64.82,346.,379.9,375.9,344.2,367.9,335.6,
     +         359.9,315.,275.2,84.47/
      DATA Y95/3.288D1,2.199D0,3.288D1,3.382D1,7.391D0,3.288D1,1.212D2,
     +         1.166D3,1.719D1,3.288D1,7.354D1,7.915D1,1.906D2,2.163D1,
     +         3.288D1,5.498D1,9.609D1,7.471D1,7.459D1,2.183D1,3.288D1,
     +         1.380D2,9.100D1,9.428D1,5.342D1,5.547D1,2.627D1,3.288D1,
     +         3.812D2,1.894D2,1.104D2,1.141D2,5.486D1,5.682D1,2.535D1,
     +         3.288D1,1.257D3,5.000D1,1.230D2,5.000D1,7.933D1,5.876D1,
     +         6.137D1,2.657D1,3.288D1,2.042D5,6.075D1,6.525D1,6.734D1,
     +         6.907D1,7.167D1,5.000D1,8.181D1,3.352D1,3.288D1,3.222D2,
     +         2.527D2,5.000D1,6.977D1,5.000D1,7.237D1,7.512D1,4.968D1,
     +         5.198D1,5.315D1,3.288D1,1.079D4,4.112D4,1.401D2,7.147D1,
     +         6.173D1,5.000D1,5.000D1,5.108D1,5.000D1,5.384D1,2.778D1,
     +         3.288D1,1.131D4,2.404D3,5.082D2,3.260D1,3.376D1,5.000D1,
     +         3.478D1,5.000D1,3.660D1,5.000D1,4.295D1,3.567D1,3.288D1,
     +         5.765D6,3.537D2,1.169D2,1.188D2,1.979D1,5.000D1,5.000D1,
     +         2.260D1,5.000D1,2.487D1,5.000D1,3.034D1,2.848D1,3.288D1/
      DATA YW95/55*0.,0.1465,10*0.,1.463D-2,2.223D-5,10*0.,2.337D-2,
     +          7.839D-3,2.016D-2,10*0.,2.774D-4,2.87D-4,8.658D-4,
     +          8.417D-4,10*0./
      DATA P95/2.963,6.098,2.963,1.49,2.916,2.963,1.348,1.022,3.157,
     +         2.963,1.438,1.436,1.21,2.624,2.963,1.503,1.338,1.442,
     +         1.428,2.581,2.963,1.252,1.335,1.335,1.552,1.538,2.315,
     +         2.963,1.083,1.185,1.289,1.287,1.540,1.533,2.336,2.963,
     +         0.9638,1.65,1.263,1.65,1.377,1.511,1.501,2.255,2.963,
     +         0.845,1.42,1.424,1.422,1.416,1.411,1.65,1.396,2.002,
     +         2.963,3.57,1.12,1.65,1.414,1.65,1.405,1.397,1.579,
     +         1.575,1.678,2.963,2.96,2.995,1.238,1.413,1.464,1.65,
     +         1.65,1.539,1.65,1.56,2.161,2.963,2.762,2.92,3.11,1.732,
     +         1.73,1.65,1.721,1.65,1.712,1.65,1.676,1.915,2.963,2.639,
     +         3.133,3.529,3.547,2.094,1.65,1.65,2.02,1.65,1.982,1.65,
     +         1.925,2.135,2.963/
C
      VERNER=0.
      E=HHEV*FR
C
      II=ILOW(ITR)
      N1=NFIRST(IEL(II))
      IF(II.NE.N1) THEN
        WRITE(10,*) 'Level II=',II
        CALL STOP('Verner fits only for ground states')
      END IF
      IAT=NUMAT(IATM(II))
      IZZ=IZ(IEL(II))
      IF(IAT.GT.14) GO TO 10
      IVER=IV0(IAT)+IZZ
C
C     1996 Expression
C
      IF(E.LT.EMX(IVER)) THEN
        XX=E/E0(IVER)-Y0(IVER)
        YY=SQRT(XX*XX+Y1(IVER)*Y1(IVER))
        AA=(XX-UN)*(XX-UN)+YW(IVER)*YW(IVER)
        BB=YY**(HALF*PV(IVER)-5.5)
        CC=(UN+SQRT(YY/YA(IVER)))**PV(IVER)
        FY=AA*BB/CC
        VERNER=S0(IVER)*T18*FY
      ELSE
C
C     1995 Expression for high energies
C          (ionization of inner shell electron)
C
        YY=E/E95(IVER)
        XL=0.
        IF((IAT-IZZ).GE.10) XL=UN
        Q=HALF*P95(IVER)-5.5-XL
        AA=(YY-UN)*(YY-UN)+YW95(IVER)*YW95(IVER)
        BB=YY**Q
        CC=(UN+SQRT(YY/Y95(IVER)))**P95(IVER)
        FY=AA*BB/CC
        VERNER=S95(IVER)*T18*FY
      END IF
      RETURN
C
C     Heavier elements
C
   10 IF(IAT.EQ.26) THEN
        VERNER=VERN26(E,IZZ)
      ELSE IF(IAT.EQ.16) THEN
        VERNER=VERN16(E,IZZ)
      ELSE IF(IAT.EQ.18) THEN
        VERNER=VERN18(E,IZZ)
      ELSE IF(IAT.EQ.20) THEN
        VERNER=VERN20(E,IZZ)
      ELSE
        WRITE(10,*) ' IAT, IZZ',IAT,IZZ
        CALL STOP('VERNER - No data for this element')
      ENDIF
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
      FUNCTION VERN26(E,IZZ)
C     ======================
C
C     Photoionization cross-sections for ground states
C       of all Fe ions.
C           Verner D.A. et al. 1996, ApJ 465
C           Verner & Yakovlev 1995, A&AS 109, 125
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      PARAMETER (T18=1.D-18,MVER=26)
      DIMENSION S0(MVER),E0(MVER),EMX(MVER)
      DIMENSION Y0(MVER),Y1(MVER),YW(MVER),YA(MVER),PV(MVER)
      DIMENSION S95(MVER),E95(MVER),Y95(MVER),YW95(MVER),P95(MVER)
C
      DATA S0/3.062D-1,4.365D3,6.107D0,3.653D2,1.523D-3,5.259D-1,2.42D4,
     +        1.979D1,2.687D1,6.470D1,3.281D0,1.738D0,2.791D-3,1.454D-1,
     +        2.108D2,1.207D1,1.452D0,2.388D0,6.066D-5,4.455D-1,1.098D1,
     +        7.204D-2,2.580D4,1.276D1,1.195D1,8.099D1/
      DATA E0/0.05461,0.1761,0.1698,25.44,0.7256,2.656,5.059,0.07098,
     +        6.741,68.86,8.284,6.295,0.1317,0.8509,0.05555,28.73,
     +        0.3444,31.9,7.519D-4,20.11,9.243,9.713,45.75,73.26,
     +        1057.,293.2/
      DATA EMX/66.,76.17,87.05,106.7,128.8,152.7,178.3,205.5,921.1,
     +         959.,998.3,1039.,1081.,1125.,1181.,1216.,7651.,7769.,
     +         7918.,8041.,8184.,8350.,8484.,8638.,5.D4,5.D4/
      DATA Y0/1.382D2,9.272D1,1.76D2,0.,8.871D1,3.361D1,0.4546,2.542D3,
     +        2.494D1,1.19D-5,2.971D1,4.671D1,2.17D3,4.505D2,2.706D-4,
     +        0.,2.891D1,3.805D1,1.915D6,6.847D1,4.446D1,1.702D2,
     +        3.582D-2,0.,0.,0./
      DATA Y1/0.2481,1.075D2,1.847D1,0.,5.28D-2,3.743D-3,2.683D1,
     +        4.672D2,8.251,6.57D-3,0.522,0.1425,6.852D-3,2.504,
     +        1.628,0.,3.404,0.4805,3.14D1,3.989,3.512,4.263,8.712D-3,
     +        0.,0.,0./
      DATA YW/2.069D1,1.141D1,8.698,0.5602,5.064D1,1.558D1,2.516D-3,
     +        2.158D2,2.387D-4,2.778D-4,0.3279,0.3096,0.6938,0.4937,
     +        1.885D-3,0.,1.264,2.902D-2,4.398,2.757,1.748,9.551D-3,
     +        2.723D1,0.,0.,0./
      DATA YA/2.671D7,6.298D3,1.555D3,8.913D0,3.736D1,1.450D1,4.850D4,
     +        1.745D4,1.807D2,2.062D1,5.360D1,1.130D2,2.487D3,1.239D3,
     +        2.045D4,5.150D2,3.960D2,2.186D1,1.606D6,4.236D1,7.637D1,
     +        1.853D2,1.358D0,4.914D1,5.769D1,3.288D1/
      DATA PV/7.923,5.204,8.055,6.538,17.67,16.32,2.374,6.75,6.29,
     +        4.111,8.571,8.037,9.791,8.066,6.033,3.846,10.13,9.589,
     +        8.813,9.724,7.962,8.843,26.04,4.941,1.718,2.963/
      DATA S95/6.298D1,4.624D1,4.422D1,4.81D1,5.143D1,5.246D1,5.21D1,
     +         5.336D1,2.205D2,2.392D2,2.449D2,3.325D2,3.316D2,3.367D2,
     +         1.496D2,3.383D2,1.15D1,8.327D0,1.155D1,8.619D0,8.773D0,
     +         1.181D1,9.098D1,1.157D1,1.195D1,8.099D1/
      DATA E95/76.3,77.5,77.77,76.25,72.73,72.6,74.33,75.56,171.5,
     +         164.7,163.2,138.3,139.2,138.7,213.6,139.6,1067.,1249.,
     +         1068.,1235.,1228.,1066.,1215.,1087.,1057.,293.2/
      DATA Y95/1.479D1,2.155D1,2.336D1,2.286D1,2.428D1,2.751D1,3.306D1,
     +         3.855D1,5.298D1,5.276D1,5.452D1,5.09D1,5.237D1,5.279D1,
     +         7.000D1,5.459D1,3.412D1,5.000D1,3.578D1,5.000D1,5.000D1,
     +         4.116D1,5.000D1,5.086D1,5.769D1,3.288D1/
      DATA YW95/0.2646,0.2599,0.2557,0.2449,0.1365,0.02105,0.02404,
     +          0.02667,1.508D-5,1.574D-5,1.594D-4,1.114D-5,1.107D-5,
     +          1.111D-5,0.1,1.179D-5,10*0./ 
      DATA P95/7.672,7.138,7.017,7.043,7.028,6.823,6.509,6.265,4.154,
     +         4.204,4.187,4.446,4.41,4.407,3.7,4.366,1.922,1.65,1.895,
     +         1.65,1.65,1.827,1.65,1.722,1.718,2.963/
C
      VERN26=0.
      IVER=IZZ
C
C     1996 Expression
C
      IF(E.LT.EMX(IVER)) THEN
        XX=E/E0(IVER)-Y0(IVER)
        YY=SQRT(XX*XX+Y1(IVER)*Y1(IVER))
        AA=(XX-UN)*(XX-UN)+YW(IVER)*YW(IVER)
        BB=YY**(HALF*PV(IVER)-5.5)
        CC=(UN+SQRT(YY/YA(IVER)))**PV(IVER)
        FY=AA*BB/CC
        VERN26=S0(IVER)*T18*FY
      ELSE
C
C     1995 Expression for high energies
C          (ionization of inner shell electron)
C
        YY=E/E95(IVER)
        XL=0.
        IF(IZZ.LE.16) XL=UN
        Q=HALF*P95(IVER)-5.5-XL
        AA=(YY-UN)*(YY-UN)+YW95(IVER)*YW95(IVER)
        BB=YY**Q
        CC=(UN+SQRT(YY/Y95(IVER)))**P95(IVER)
        FY=AA*BB/CC
        VERN26=S95(IVER)*T18*FY
      END IF
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
      FUNCTION VERN16(E,IZZ)
C     ======================
C
C     Photoionization cross-sections for ground states
C       of all Sulfur ions.
C           Verner D.A. et al. 1996, ApJ 465
C           Verner & Yakovlev 1995, A&AS 109, 125
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      PARAMETER (T18=1.D-18,MVER=16)
      DIMENSION S0(MVER),E0(MVER),EMX(MVER)
      DIMENSION Y0(MVER),Y1(MVER),YW(MVER),YA(MVER),PV(MVER)
      DIMENSION S95(MVER),E95(MVER),Y95(MVER),YW95(MVER),P95(MVER)
C
      DATA S0/4.564D4,3.136D2,6.666D0,2.606D0,5.072D-4,9.139D0,5.703D-1,
     +        3.161D1,9.646D3,5.364D1,1.275D1,3.49D-1,2.294D4,2.555D1,
     +        2.453D1,2.139D2/
      DATA E0/18.08,8.787,2.027,2.173,0.1713,14.13,0.3757,14.62,0.1526,
     +        10.4,6.485,2.443,14.74,33.1,439.,110.4/
      DATA EMX/170.,184.6,199.5,216.4,235.,255.7,2569.,2641.,2705.,
     +         2782.,2859.,2941.,3029.,3107.,5.D4,5.D4/
      DATA Y0/0.9935,2.782,15.68,19.75,94.24,0.,222.2,18.69,1.615D-3,
     +        17.75,34.26,227.9,2.203D-2,0.,0.,0./
      DATA Y1/0.2486,0.1788,9.421,3.361,0.6265,0.,4.606,0.3037,0.4049,
     +        1.663,0.137,1.172,1.073D-2,0.,0.,0./
      DATA YW/0.6385,0.7354,4.109,1.863,0.788,0.,1.503,1.153D-3,1.492,
     +        2.31,1.678,0.7033,27.38,0.,0.,0./
      DATA YA/1.,3.442,54.54,66.41,198.6,1656.,146.,16.11,1438.,36.41,
     +        65.83,541.1,1.529,38.21,44.05,32.88/
      DATA PV/13.61,12.81,8.611,8.655,13.07,3.626,11.35,8.642,5.977,
     +        7.09,7.692,7.769,25.68,5.037,1.765,2.963/
      DATA S95/1.883D2,1.896D2,1.780D2,2.037D2,2.919D2,4.712D2,1.916D1,
     +         1.931D1,1.946D1,2.041D1,2.101D1,2.087D1,2.233D1,2.293D1,
     +         2.453D1,2.139D2/
      DATA E95/91.52,90.58,92.46,87.44,74.11,57.47,495.2,489.1,493.7,
     +         480.2,475.8,482.8,466.9,466.7,439.,110.4/
      DATA Y95/71.93,75.38,149.8,93.1,48.64,36.1,35.55,50.,35.68,50.,
     +         50.,37.42,50.,44.59,44.05,32.88/
      DATA YW95/0.2485,0.2934,0.02142,9.497D-3,0.02785,0.0248,10*0./
      DATA P95/3.633,3.635,3.319,3.565,4.142,4.742,1.742,1.65,1.737,
     +         1.65,1.65,1.72,1.65,1.668,1.765,2.963/
C
      VERN16=0.
      IVER=IZZ
C
C     1996 Expression
C
      IF(E.LT.EMX(IVER)) THEN
        XX=E/E0(IVER)-Y0(IVER)
        YY=SQRT(XX*XX+Y1(IVER)*Y1(IVER))
        AA=(XX-UN)*(XX-UN)+YW(IVER)*YW(IVER)
        BB=YY**(HALF*PV(IVER)-5.5)
        CC=(UN+SQRT(YY/YA(IVER)))**PV(IVER)
        FY=AA*BB/CC
        VERN16=S0(IVER)*T18*FY
      ELSE
C
C     1995 Expression for high energies
C          (ionization of inner shell electron)
C
        YY=E/E95(IVER)
        XL=0.
        IF(IZZ.LE.6) XL=UN
        Q=HALF*P95(IVER)-5.5-XL
        AA=(YY-UN)*(YY-UN)+YW95(IVER)*YW95(IVER)
        BB=YY**Q
        CC=(UN+SQRT(YY/Y95(IVER)))**P95(IVER)
        FY=AA*BB/CC
        VERN16=S95(IVER)*T18*FY
      END IF
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
      FUNCTION VERN18(E,IZZ)
C     ======================
C
C     Photoionization cross-sections for ground states
C       of all Argon ions.
C           Verner D.A. et al. 1996, ApJ 465
C           Verner & Yakovlev 1995, A&AS 109, 125
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      PARAMETER (T18=1.D-18,MVER=18)
      DIMENSION S0(MVER),E0(MVER),EMX(MVER)
      DIMENSION Y0(MVER),Y1(MVER),YW(MVER),YA(MVER),PV(MVER)
      DIMENSION S95(MVER),E95(MVER),Y95(MVER),YW95(MVER),P95(MVER)
C
      DATA S0/2.106D1,2.503D1,3.58D1,2.035D1,9.946D0,1.080D0,3.693D0,
     +        3.295D1,8.279D-1,8.204D0,1.76D3,7.018D-1,2.459D-2,
     +        4.997D-2,2.571D4,2.135D1,3.108D1,1.69D2/
      DATA E0/17.09,24.94,14.17,6.953,10.31,0.544,0.02966,3.844,
     +        0.1926,10.4,0.1257,5.31,0.3209,1.557,18.88,41.54,
     +        446.8,139.9/
      DATA EMX/249.2,266.2,280.1,298.7,320.,342.6,366.7,392.5,3361.,
     +         3446.,3523.,3613.,3702.,3798.,3898.,3988.,5.D4,5.D4/
      DATA Y0/1.688,0.9299,2.384,7.501,6.406,1.7D2,4.383D-4,0.,38.14,
     +        38.04,3.286D-3,1.099D2,2.068D3,4.552D2,2.445D-2,0.,0.,0./
      DATA Y1/0.8943,0.7195,1.794,0.1806,3.659D-3,15.87,2.513,0.,4.649,
     +        0.639,0.3226,0.2202,21.13,6.459,1.054D-2,0.,0.,0./
      DATA YW/0.4185,0.5108,0.6316,0.8842,0.4885,11.07,1.363D-2,0.,
     +        1.434,9.203D-4,1.975,0.4987,0.6692,0.2938,29.09,0.,0.,0./
      DATA YA/2.645D2,1.272D2,3.776D1,1.4D1,7.444D1,9.419D2,9.951D3,
     +        7.082D2,2.392D2,1.495D1,1.579D3,1.001D2,2.285D3,5.031D2,
     +        1.475D0,4.118D1,3.039D1,3.288D1/
      DATA PV/4.796,4.288,5.742,9.595,6.261,7.582,7.313,4.645,11.21,
     +        11.15,6.714,8.939,8.81,8.966,26.34,4.945,2.092,2.963/
      DATA S95/8.372D1,1.937D2,2.281D2,2.007D2,2.474D2,2.786D2,3.204D2,
     +         4.198D2,2.931D1,1.585D1,2.796D1,1.666D1,2.888D1,2.874D1,
     +         2.883D1,3.003D1,3.108D1,1.69D2/
      DATA E95/164.7,108.5,102.5,107.3,98.35,92.33,85.63,73.68,467.9,
     +         612.6,478.9,602.8,473.1,474.9,475.6,468.,466.8,139.9/
      DATA Y95/54.52,70.,43.8,70.,42.84,42.2,42.3,44.19,17.44,50.,
     +         19.17,50.,20.42,22.35,26.15,28.54,30.39,32.88/
      DATA YW95/0.627,0.1,7.167D-3,0.1,7.283D-3,7.408D-3,7.258D-3,
     +          7.712D-3,10*0./
      DATA P95/3.328,3.7,4.046,3.7,4.125,4.227,4.329,4.492,2.362,
     +         1.65,2.271,1.65,2.234,2.171,2.074,2.037,2.092,2.963/
C
      VERN18=0.
      IVER=IZZ
C
C     1996 Expression
C
      IF(E.LT.EMX(IVER)) THEN
        XX=E/E0(IVER)-Y0(IVER)
        YY=SQRT(XX*XX+Y1(IVER)*Y1(IVER))
        AA=(XX-UN)*(XX-UN)+YW(IVER)*YW(IVER)
        BB=YY**(HALF*PV(IVER)-5.5)
        CC=(UN+SQRT(YY/YA(IVER)))**PV(IVER)
        FY=AA*BB/CC
        VERN18=S0(IVER)*T18*FY
      ELSE
C
C     1995 Expression for high energies
C          (ionization of inner shell electron)
C
        YY=E/E95(IVER)
        XL=0.
        IF(IZZ.LE.8) XL=UN
        Q=HALF*P95(IVER)-5.5-XL
        AA=(YY-UN)*(YY-UN)+YW95(IVER)*YW95(IVER)
        BB=YY**Q
        CC=(UN+SQRT(YY/Y95(IVER)))**P95(IVER)
        FY=AA*BB/CC
        VERN18=S95(IVER)*T18*FY
      END IF
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
      FUNCTION VERN20(E,IZZ)
C     ======================
C
C     Photoionization cross-sections for ground states
C       of all Calcium ions.
C           Verner D.A. et al. 1996, ApJ 465
C           Verner & Yakovlev 1995, A&AS 109, 125
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      PARAMETER (T18=1.D-18,MVER=20)
      DIMENSION S0(MVER),E0(MVER),EMX(MVER)
      DIMENSION Y0(MVER),Y1(MVER),YW(MVER),YA(MVER),PV(MVER)
      DIMENSION S95(MVER),E95(MVER),Y95(MVER),YW95(MVER),P95(MVER)
C
      DATA S0/5.37D5,1.064D7,3.815D1,7.736D0,1.523D-1,7.642D1,4.76D-1,
     +        6.641D-1,2.076D2,1.437D1,9.384D-1,1.227D1,1.849D3,
     +        1.116D0,5.513D1,1.293D0,2.028D4,1.105D1,1.936D1,1.369D2/
      DATA E0/12.78,15.53,24.36,4.255,0.6882,9.515,0.808,1.366,0.0552,
     +        16.05,0.2288,23.45,10.08,9.98,130.9,4.293,26.18,94.72,
     +        629.7,172.9/
      DATA EMX/34.43,40.9,373.1,394.4,417.5,442.3,468.7,496.7,527.,
     +         556.9,4265.,4362.,4453.,4555.,4659.,4767.,4880.,4982.,
     +         5.D4,5.D4/
      DATA Y0/1.012D-3,2.161D-3,1.802,14.67,121.,4.829,148.7,103.9,
     +        2.826D-4,0.,24.78,24.17,6.138D-3,71.04,1.833D-2,0.9363,
     +        2.402D-2,0.,0.,0./
      DATA Y1/1.851D-2,6.706D-2,1.233,3.298D-2,3.876,5.824,1.283,3.329,
     +        1.657,0.,3.1,0.5469,69.31,5.311,0.9359,4.589D-2,9.323D-3,
     +        0.,0.,0./
      DATA YW/0.4477,0.6453,0.3126,1.369,8.277,2.471,0.572,0.2806,
     +        1.843D-3,0.,1.39,6.842D-4,241.,3.879,9.084D-2,3.461D-5,
     +        28.03,0.,0.,0./
      DATA YA/0.3162,0.779,293.1,13.55,150.2,89.73,368.2,318.8,1.79D4,
     +        698.9,254.9,13.12,1.792D4,59.18,382.8,16.91,1.456,38.18,
     +        39.21,32.88/
      DATA PV/12.42,21.3,3.944,12.36,10.61,5.141,8.634,8.138,5.893,
     +        3.857,11.03,9.771,2.868,9.005,2.023,14.38,25.6,4.192,
     +        1.862,2.963/
      DATA S95/9.017D1,7.314D1,1.945D2,1.542D2,1.622D2,1.855D2,2.181D2,
     +         2.788D2,1.934D2,6.616D2,1.547D1,1.324D1,1.57D1,1.384D1,
     +         1.417D1,1.665D1,1.486D1,1.82D1,1.936D1,1.369D2/
      DATA E95/44.87,44.98,126.,141.3,138.4,130.3,120.8,107.,129.3,
     +         65.11,701.,750.3,698.9,739.6,734.2,686.2,723.5,664.,
     +         629.7,172.9/
      DATA Y95/14.65,18.98,68.19,99.06,88.11,69.93,58.16,47.68,70.,
     +         43.71,31.97,50.,32.18,50.,50.,34.43,50.,39.79,39.21,
     +         32.88/
      DATA YW95/0.2754,0.2735,4.791D-4,1.107D-3,4.384D-4,1.4D-5,
     +          4.346D-6,4.591D-6,0.1,7.881D-6,10*0./
      DATA P95/7.498,7.152,3.77,3.446,3.521,3.707,3.907,4.2,3.7,4.937,
     +         1.858,1.65,1.851,1.65,1.65,1.823,1.65,1.777,1.862,
     +         2.963/
C
      VERN20=0.
      IVER=IZZ
C
C     1996 Expression
C
      IF(E.LT.EMX(IVER)) THEN
        XX=E/E0(IVER)-Y0(IVER)
        YY=SQRT(XX*XX+Y1(IVER)*Y1(IVER))
        AA=(XX-UN)*(XX-UN)+YW(IVER)*YW(IVER)
        BB=YY**(HALF*PV(IVER)-5.5)
        CC=(UN+SQRT(YY/YA(IVER)))**PV(IVER)
        FY=AA*BB/CC
        VERN20=S0(IVER)*T18*FY
      ELSE
C
C     1995 Expression for high energies
C          (ionization of inner shell electron)
C
        YY=E/E95(IVER)
        XL=0.
        IF(IZZ.LE.10) XL=UN
        Q=HALF*P95(IVER)-5.5-XL
        AA=(YY-UN)*(YY-UN)+YW95(IVER)*YW95(IVER)
        BB=YY**Q
        CC=(UN+SQRT(YY/Y95(IVER)))**P95(IVER)
        FY=AA*BB/CC
        VERN20=S95(IVER)*T18*FY
      END IF
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
 
      FUNCTION GAUNT(I,FR)
C     ====================
C
C     Hydrogenic bound-free Gaunt factor for the principal quantum
C     number I and frequency FR
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (UN=1.)
      DIMENSION CGT(7,10),X(7)
      DATA CGT/0.,12.803223,-5.5759888,1.2302628,-2.9094219D-3,
     *  7.3993579D-6,-8.7356966D-9,-2.0244141,2.1325684,-1.2709045,
     *  1.1595421,-2.0735860D-3,2.7033384D-6,0.,-0.23387146,
     *  0.52471924,-0.55936432,1.1450949,-1.9366592D-3,2.3572356D-6,
     *  0.,-5.4418565D-2,0.19683564,-0.31190730,1.1306695,
     *  -1.3482273D-3,-4.6949424D-6,2.3548636D-8,-8.9182854D-3,
     *  5.5545091D-2,-0.16051018,1.1190904,-1.0401085D-3,
     *  -6.9943488D-6,2.8496742D-8,-5.5303574D-3,4.1921183D-2,
     *  -0.13075417,1.1168376,-8.9466573D-4,-8.8393133D-6,
     *  3.4696768D-8,-2.2752881D-3,2.3350812D-2,-9.5441161D-2,
     *  1.1128632,-7.4833260D-4,-1.0244504D-5,3.8595771D-8,
     *  -9.7200274D-4,1.3298411D-2,-7.1010560D-2,1.1093137,
     *  -6.2619148D-4,-1.1342068D-5,4.1477731D-8,-4.9576163D-4,
     *  8.5139736D-3,-5.6046560D-2,1.1078717,-5.4837392D-4,
     *  -1.2157943D-5,4.3796716D-8,-2.9467046D-4,6.1516856D-3,
     *  -4.7326370D-2,1.1052734,-4.4341570D-4,-1.3235905D-5,
     *  4.7003140D-8/
C
      IF(I.LE.10) THEN
        X(5)=FR/2.99793D14
        X(6)=X(5)*X(5)
        X(7)=X(6)*X(5)
        X(4)=UN
        X(3)=UN/X(5)
        X(2)=X(3)*X(3)
        X(1)=X(2)*X(3)
        GAUNT=0.
        DO 10 J=1,7
          GAUNT=GAUNT+CGT(J,I)*X(J)
   10   CONTINUE
      ELSE
        GAUNT=1.
      ENDIF
      RETURN
      END
C
C
C     ****************************************************************
C
C
 
      FUNCTION GNTK(I,FR)
C     ===================
C
C     Hydrogenic bound-free Gaunt factor for the principal quantum
C     number I and frequency FR (from Klaus Werner)
C
      INCLUDE 'IMPLIC.FOR'
      GAUNT=1.
      IF(I.GT.3) GO TO 16
      Y=1./FR      
      GO TO (1,2,3),I
    1 GNTK=0.9916+Y*(2.71852D13-Y*2.26846D30)
      GO TO 16
    2 GNTK=1.1050-Y*(2.37490D14-Y*4.07677D28)
      GO TO 16
    3 GNTK=1.1010-Y*(0.98632D14-Y*1.03540D28)
   16 RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE SPSIGK(ITR,IB,FR,SIGSP)
C     ==================================
C
C     Non-standard evaluation of the photoionization cross-sections
C     Basically user-suppled procedure; here are some examples
C
      INCLUDE 'IMPLIC.FOR'
      SIGSP=0.
C
C     Special formula for the He I ground state
C
      IF(IB.EQ.-201) SIGSP=7.3D-18*EXP(1.373-2.311E-16*FR)
C
C     Special formula for the averaged <n=2> level of He I
C
      IF(IB.EQ.-202) SIGSP=SGHE12(FR)
C
C     Carbon ground configuration levels 2p2 1D and 1S
C
      IF(IB.EQ.-602.OR.IB.EQ.-603) THEN
         CALL CARBON(IB,FR,SG)
         SIGSP=SG
      END IF
C
C     Hidalgo (Ap.J. 153, 981, 1968) photoionization data
C
      IF(IB.LE.-101.AND.IB.GE.-137) SIGSP=HIDALG(IB,FR)
C
C     Reilman and Manson (Ap.J. Suppl. 40, 815, 1979) photoionization data
C
      IF(IB.LE.-301.AND.IB.GE.-337) SIGSP=REIMAN(IB,FR)
      RETURN
      END
C
C
C     *********************************************************************
C
C

      FUNCTION HIDALG(IB,FR)
C     ======================
C
C     Read table of wavelengths and photo-ionization cross-sections
C     from Hidalgo (1968, Ap. J., 153, 981) for the species indicated by IB
C     (Hidalgo's number = INDEX = -IB-100).
C     Compute linearly interpolated value of the cross-section 
C     at the frequency FR.
C
C     (At the moment, only a few transitions are considered)
C
      INCLUDE 'IMPLIC.FOR'
      DIMENSION WL1(20),WL2(20),WL(20),SIG0(20,24),SIGS(20)
C
      DATA WL1 /
     *  39.1, 80.9, 97.6,100.1,104.3,107.2,108.7,111.9,113.6,115.4,
     * 117.1,119.0,124.8,126.9,129.1,131.3,133.6,136.0,138.5,141.1/
      DATA WL2 /
     *  68.5, 80.9,100.1,120.9,158.8,165.7,177.3,190.6,200.7,206.2,
     * 211.9,218.0,224.5,231.3,246.3,5*0./
      DATA SIG0 /
     * 120*0.,
     *.0460,.2400,.3500,.3700,.4000,.4300,.4400,.4600,.4700,.4900, 
     *.5000,.5200,.5700,.6200, 6*0.,
     * 80*0.,
     *.0092,.1000,.1900,.2100,.2300,.2500,.2600,.2900,.3000,.3200,
     *.3400,.3500,.4100,.4300,.4500,.4800,.5000,.5300,.5600,.5900,
     * 20*0.,
     *.3400,.4600,.6300,.7700,.9100,1.080, 14*0.,
     * 20*0.,
     *.0064,.1100,.2200,.4100,.9400,1.000,1.300,1.600, 12*0.,
     * 80*0.,
     *.0370,.0650,.1300,.2400,.5500,.6300,.7700,.9500,1.100,1.250,
     * 10*0.,
     * 20*0.,
     * 20*0.,
     *.0220,.0390,.0800,.1500,.3500,.4000,.4900,.6200,.7200,.7800,
     *.8500,.9300,1.020,
     * 7*0./
      SAVE WL1,WL2,SIG0
C

      INDEX=-IB-100
      NUM=20  
      IF(INDEX.GE.13.AND.INDEX.LE.27) NUM=15
      DO 10 I=1,NUM
         IF(INDEX.LT.13) WL(I)=WL1(I)
         IF(INDEX.GE.13) WL(I)=WL2(I)
         SIGS(I)=SIG0(I,INDEX)
   10 CONTINUE
C 
      WLAM=2.997925D18/FR
      IL=1
      IR=NUM
      DO 50 I=1,NUM-1
        IF(WLAM.GE.WL(I).AND.WLAM.LE.WL(I+1)) THEN
          IL=I
          IR=I+1
          GO TO 60
        ENDIF
 50   CONTINUE
C
C     LINEAR INTERPOLATION:
C
 60   SIGM=(SIGS(IR)-SIGS(IL))*(WLAM-WL(IL))/(WL(IR)-WL(IL))
     *      + SIGS(IL)
C
C     IF OUTSIDE WAVELENGTH RANGE SET TO FIRST(LAST) VALUE:
C
       IF(WLAM.LE.WL(1)) SIGM=SIGS(1)
       IF(WLAM.GE.WL(NUM)) SIGM=SIGS(NUM)
C
C     IF LAST NON-ZERO SIG VALUES, NO INTERPOLATION:
C
c       IF(SIGS(IR).EQ.0.) SIGM=SIGS(IL)   
C
      HIDALG=SIGM*1.D-18
      RETURN
      END
C
C
C     *********************************************************************
C
C

      FUNCTION REIMAN(IB,FR)
C     ======================
C
C     Read table of photon energies and photo-ionization cross-sections
C     from Reilman & Manson (1979, Ap. J. Suppl., 40, 815) for the species 
C     indicated by IB
C    
C     Compute linearly interpolated value of the cross-section 
C     at the frequency FR.
C
C     (At the moment, only a few transitions are considered)
C
      INCLUDE 'IMPLIC.FOR'
      DIMENSION HEV(30),F0(30),SIG0(30,2),SIGS(30)
C
      DATA HEV /
     * 130.,160.,190.,210.,240.,270.,300.,330.,360.,390.,
     * 420.,450.,480.,510.,540.,570.,600.,630.,660.,690.,
     * 720.,750.,780.,810.,840.,870.,900.,930.,960.,990./
      DATA SIG0 /
     * 3*0.,     4.422D-1, 3.478D-1, 
     * 2.794D-1, 2.286D-1, 1.899D-1, 1.598D-1, 1.360D-1,
     * 1.169D-1, 1.013D-1, 8.845D-2, 7.776D-2, 6.877D-2,
     * 6.114D-2, 5.463D-2, 4.904D-2, 4.419D-2, 3.998D-2,
     * 3.629D-2, 3.305D-2, 3.019D-2, 2.766D-2, 2.540D-2,
     * 2.339D-2, 2.158D-2, 1.996D-2, 1.850D-2, 1.718D-2,
     * 4*0.,     1.981D-1, 1.584D-1, 
     * 1.290D-1, 1.066D-1, 8.932D-2, 7.567D-2, 6.475D-2,
     * 5.589D-2, 4.862D-2, 4.259D-2, 3.754D-2, 3.329D-2,
     * 2.966D-2, 2.656D-2, 2.388D-2, 2.157D-2, 1.954D-2,
     * 1.777D-2, 1.621D-2, 1.484D-2, 1.362D-2, 1.253D-2,
     * 1.155D-2, 1.067D-2, 9.888D-3, 9.179D-3/
      SAVE HEV,SIG0
C
      INDEX=-IB-300
      NUM=30  
      DO 10 I=1,NUM
         F0(I)=HEV(I)*2.418573D14
         SIGS(I)=SIG0(I,INDEX)
   10 CONTINUE
C 
      IL=1
      IR=NUM
      DO 50 I=1,NUM-1
        IF(FR.GE.F0(I).AND.FR.LE.F0(I+1)) THEN
          IL=I
          IR=I+1
          GO TO 60
        ENDIF
 50   CONTINUE
C
C     LINEAR INTERPOLATION:
C
 60   SIGM=(SIGS(IR)-SIGS(IL))*(FR-F0(IL))/(F0(IR)-F0(IL))
     *      + SIGS(IL)
C
C     IF OUTSIDE WAVELENGTH RANGE SET TO FIRST(LAST) VALUE:
C
       IF(FR.LE.F0(1)) SIGM=SIGS(1)
       IF(FR.GE.F0(NUM)) SIGM=SIGS(NUM)
C
C     IF LAST NON-ZERO SIG VALUES, NO INTERPOLATION:
C
c       IF(SIGS(IR).EQ.0.) SIGM=SIGS(IL)   
C
      REIMAN=SIGM*1.D-18
      RETURN
      END
C
C
C     ****************************************************************
C
C
 
      SUBROUTINE CARBON(IB,FR,SG)
C     ===========================
C
C     Photoionization cross-section for neutral carbon 2p1D and 2p1S
C     levels (G.B.Taylor - private communication)
C
      INCLUDE 'IMPLIC.FOR'
      DIMENSION FR2(34),SG2(34),FR3(45),SG3(45)
      DATA FR2/ 0.74, 0.75, 0.76, 0.77, 0.78, 0.79, 0.80, 0.81, 0.82,
     *                0.83,       0.85, 0.86, 0.87, 0.88, 0.89, 0.90,
     *          0.91, 0.92, 0.93, 0.94, 0.95, 0.96, 0.97, 0.98, 0.99,
     *          1.00, 1.10, 1.20, 1.30, 1.45, 1.50, 1.60, 1.80, 2./
      DATA SG2/ 12.04, 12.03, 12.09, 12.26, 12.60, 13.24, 14.36, 16.24,
     *          19.28, 23.94, 37.41, 42.88, 44.76, 43.41, 40.46, 37.19,
     *          34.26, 31.82, 29.96, 28.57, 27.68, 27.37, 27.84, 29.69,
     *          34.45, 46.35, 13.80, 11.54, 10.40,  8.96,  8.54,  7.47,
     *           6.53,  5.66/
      DATA FR3/ 0.66, 0.68, 0.70, 0.72, 0.74, 0.76, 0.78, 0.80, 0.82,
     *          0.84, 0.86, 0.864,0.866,0.868,0.87, 0.874,0.876,0.88,
     *          0.882,0.884,0.886,0.888,0.89 ,0.894,0.896,0.898,0.90,
     *          0.904,0.908,0.910,0.920,0.94, 0.98, 1.00, 1.10, 1.20,
     *          1.26, 1.34, 1.36, 1.40, 1.46, 1.60, 1.70, 1.80, 2./
      DATA SG3/ 13.94, 13.29, 12.56, 11.73, 10.82, 10.18,  8.62,  7.27,
     *           5.74,  4.14,  4.61,  5.92,  6.94,  8.34, 10.21, 16.12,
     *          20.64, 34.56, 44.82, 57.71, 73.09, 89.99,106.38,127.08,
     *         128.38,124.44,117.17, 99.32, 82.95, 76.05, 52.65, 33.23,
     *          21.29, 18.69, 12.62, 11.44,  9.77,  7.53, 10.47,  9.65,
     *          10.19,  7.28,  6.70,  6.11,  4.96/
      PARAMETER (FR0=3.28805D15, NC2=34, NC3=45)
      SAVE FR2,SG2,FR3,SG3
C
      F=FR/FR0
      IF(IB.NE.-602) GO TO 25
      J=2
      IF(F.LE.FR2(1)) GO TO 20
      DO 10 I=2,NC2
         J=I
         IF(F.GT.FR2(I-1).AND.F.LE.FR2(I)) GO TO 20
   10 CONTINUE
   20 SG=(F-FR2(J-1))/(FR2(J)-FR2(J-1))*(SG2(J)-SG2(J-1))+SG2(J-1)
      SG=SG*1.D-18
   25 IF(IB.NE.-603) GO TO 50
      J=2
      IF(F.LE.FR3(1)) GO TO 40
      DO 30 I=2,NC3
         J=I
         IF(F.GT.FR3(I-1).AND.F.LE.FR3(I)) GO TO 40
   30 CONTINUE
   40 SG=(F-FR3(J-1))/(FR3(J)-FR3(J-1))*(SG3(J)-SG3(J-1))+SG3(J-1)
      SG=SG*1.D-18
   50 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C

      FUNCTION SGHE12(FR)
C     ===================
C
C     Special formula for the photoionization cross-section from the
C     averaged <n=2> level of He I
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (C1=3.D0,C2=9.D0,C3=1.6D1,T15=1.D-15,
     * A1=6.45105D-18,A2=3.02D-19,A3=9.9847D-18,A4=1.1763673D-17,
     * A5=3.63662D-19,A6=-2.783D2,A7=1.488D1,A8=-2.311D-1,
     * E1=3.5D0,E2=3.6D0,E3=1.91D0,E4=2.9D0,E5=3.3D0)
C
      X=FR*T15
      XX=LOG(FR)
      SGHE12=(C1*(A1/X**E1+A2/X**E2)+A3/X**E3+C2*(A4/X**E4+A5/X**E5)+
     *       C1*EXP(A6+XX*(A7+XX*A8)))/C3
      RETURN
      END
C
C
C     ****************************************************************
C
C

      FUNCTION SBFHE1(II,IB,FR)
C     =========================
C
C     Calculates photoionization cross sections of neutral helium
C     from states with n = 1, 2, 3, 4.
C
C     The levels are either non-averaged (l,s) states, or some
C     averaged levels.
C     The program allows only two standard possibilities of
C     constructing averaged levels:
C     i)  all states within given principal quantum number n (>1) are
C         lumped together
C     ii) all siglet states for given n, and all triplet states for
C         given n are lumped together separately (there are thus two
C         explicit levels for a given n)
C
C     The cross sections are calculated using appropriate averages
C     of the Opacity Project cross sections, calculated by procedure
C     HEPHOT
C
C     Input parameters:
C      II    - index of the lower level (in the numbering of explicit
C              levels)
C      IB    - photoionization switch IBF for the given transition
C            = 10  -  means that the given transition is from an
C                     averaged level
C            = 11  -  the given transition is from non-averaged
C                     singlet state
C            = 13  -  the given transition is from non-averaged
C                     triplet state
C      FR    - frequency
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
C
      NI=NQUANT(II)
      IGI=INT(G(II)+0.01)
      IS=IB-10
C
C     ----------------------------------------------------------------
C     IB=11 or 13  - photoionization from an non-averaged (l,s) level
C     ----------------------------------------------------------------
C
      IF(IS.EQ.1.OR.IS.EQ.3) THEN
         IL=(IGI/IS-1)/2
         SBFHE1=HEPHOT(IS,IL,NI,FR)
      END IF
C
C     ----------------------------------------------------------------
C     IS=10 - photoionization from an averaged level
C     ----------------------------------------------------------------
C
      IF(IS.EQ.0) THEN
         IF(NI.EQ.2) THEN
C
C ********    photoionization from an averaged level with n=2
C
            IF(IGI.EQ.4) THEN
C
C      a) lower level is an averaged singlet state
C
               SBFHE1=(HEPHOT(1,0,2,FR)+3.D0*HEPHOT(1,1,2,FR))/9.D0
            ELSE IF(IGI.EQ.12) THEN
C
C      b) lower level is an averaged triplet state
C
               SBFHE1=(HEPHOT(3,0,2,FR)+3.D0*HEPHOT(3,1,2,FR))/9.D0
            ELSE IF(IGI.EQ.16) THEN
C
C      c) lower level is an average of both singlet and triplet states
C
               SBFHE1=(HEPHOT(1,0,2,FR)+3.D0*(HEPHOT(1,1,2,FR)+
     *                HEPHOT(3,0,2,FR))+9.D0*HEPHOT(3,1,2,FR))/1.6D1
            ELSE
               GO TO 10
            END IF
C
C
C ********    photoionization from an averaged level with n=3
C
         ELSE IF(NI.EQ.3) THEN
            IF(IGI.EQ.9) THEN
C
C      a) lower level is an averaged singlet state
C
               SBFHE1=(HEPHOT(1,0,3,FR)+3.D0*HEPHOT(1,1,3,FR)+
     *                5.D0*HEPHOT(1,2,3,FR))/9.D0
            ELSE IF(IGI.EQ.27) THEN
C
C      b) lower level is an averaged triplet state
C
               SBFHE1=(HEPHOT(3,0,3,FR)+3.D0*HEPHOT(3,1,3,FR)+
     *                5.D0*HEPHOT(3,2,3,FR))/9.D0
            ELSE IF(IGI.EQ.36) THEN
C
C      c) lower level is an average of both singlet and triplet states
C
               SBFHE1=(HEPHOT(1,0,3,FR)+3.D0*HEPHOT(1,1,3,FR)+
     *                5.D0*HEPHOT(1,2,3,FR)+
     *                3.D0*HEPHOT(3,0,3,FR)+9.D0*HEPHOT(3,1,3,FR)+
     *                15.D0*HEPHOT(3,2,3,FR))/3.6D0
            ELSE
               GO TO 10
            END IF
         ELSE IF(NI.EQ.4) THEN
C
C ********    photoionization from an averaged level with n=4
C
            IF(IGI.EQ.16) THEN
C
C      a) lower level is an averaged singlet state
C
               SBFHE1=(HEPHOT(1,0,4,FR)+3.D0*HEPHOT(1,1,4,FR)+
     *                 5.D0*HEPHOT(1,2,4,FR)+
     *                 7.D0*HEPHOT(1,3,4,FR))/1.6D1
            ELSE IF(IGI.EQ.48) THEN
C
C      b) lower level is an averaged triplet state
C
               SBFHE1=(HEPHOT(3,0,4,FR)+3.D0*HEPHOT(3,1,4,FR)+
     *                 5.D0*HEPHOT(3,2,4,FR)+
     *                 7.D0*HEPHOT(3,3,4,FR))/1.6D1
            ELSE IF(IGI.EQ.64) THEN
C
C      c) lower level is an average of both singlet and triplet states
C
               SBFHE1=(HEPHOT(1,0,4,FR)+3.D0*HEPHOT(1,1,4,FR)+
     *                 5.D0*HEPHOT(1,2,4,FR)+
     *                 7.D0*HEPHOT(1,3,4,FR)+
     *                 3.D0*HEPHOT(3,0,4,FR)+
     *                 9.D0*HEPHOT(3,1,4,FR)+
     *                 15.D0*HEPHOT(3,2,4,FR)+
     *                 21.D0*HEPHOT(3,3,4,FR))/6.4D1
            ELSE
               GO TO 10
            END IF
         ELSE
            GO TO 10
         END IF
      END IF
      RETURN
   10 WRITE(6,601) NI,IGI,IS
      WRITE(10,601) NI,IGI,IS
  601 FORMAT(1H0/' INCONSISTENT INPUT TO PROCEDURE SBFHE1'/
     * ' QUANTUM NUMBER =',I3,'  STATISTICAL WEIGHT',I4,'  S=',I3)
      STOP
      END
C
C
C     ****************************************************************
C
C

      FUNCTION HEPHOT(S,L,N,FREQ)
C     ===========================
C
C
C
C
C           EVALUATES HE I PHOTOIONIZATION CROSS SECTION USING SEATON
C           FERNLEY'S CUBIC FITS TO THE OPACITY PROJECT CROSS SECTIONS
C           UP TO SOME ENERGY "EFITM" IN THE RESONANCE-FREE ZONE.  BEYOND
C           THIS ENERGY LINEAR FITS TO LOG SIGMA IN LOG (E/E0) ARE USED.
C           THIS EXTRAPOLATION SHOULD BE USED UP TO THE BEGINNING OF THE
C           RESONANCE ZONE "XMAX", BUT AT PRESENT IT IS USED THROUGH IT.
C           BY CHANGING A FEW LINES THAT ARE PRESENTLY COMMENTED OUT,
C           FOR ENERGIES IN THE RESONANCE ZONE A VALUE OF 1/100 OF THE
C           THRESHOLD CROSS SECTION IS USED -- THIS IS PURELY AD HOC AND
C           ONLY A TEMPORARY MEASURE.  OBVIOUSLY ANY OTHER VALUE OR FUNCTIONAL
C           FORM CAN BE INSERTED HERE.
C
C           CALLING SEQUENCE INCLUDES:
C                S = MULTIPLICITY, EITHER 1 OR 3
C                L = ANGULAR MOMENTUM, 0, 1, OR 2;
C                    for L > 2 - hydrogenic expresion
C                FREQ = FREQUENCY
C
C           DGH JUNE 1988 JILA, slightly modified by I.H.
C
      INCLUDE 'IMPLIC.FOR'
      INTEGER S,L,SS,LL
      PARAMETER (TENM18=1.D-18, FRH=3.28805D15, TWO=2.D0,
     *           TENLG=2.302585093, PHOT0=2.815D29)
      DIMENSION COEF(4,53),IST(3,2),N0(3,2),
     *          FL0(53),A(53),B(53),XMAX(53),XFITM(53)
C
      DATA IST/1,36,20,11,45,28/
      DATA N0/1,2,3,2,2,3/
C
      DATA FL0/
     . 2.521D-01,-5.381D-01,-9.139D-01,-1.175D+00,-1.375D+00,-1.537D+00,
     .-1.674D+00,-1.792D+00,-1.896D+00,-1.989D+00,-4.555D-01,-8.622D-01,
     .-1.137D+00,-1.345D+00,-1.512D+00,-1.653D+00,-1.774D+00,-1.880D+00,
     .-1.974D+00,-9.538D-01,-1.204D+00,-1.398D+00,-1.556D+00,-1.690D+00,
     .-1.806D+00,-1.909D+00,-2.000D+00,-9.537D-01,-1.204D+00,-1.398D+00,
     .-1.556D+00,-1.690D+00,-1.806D+00,-1.909D+00,-2.000D+00,-6.065D-01,
     .-9.578D-01,-1.207D+00,-1.400D+00,-1.558D+00,-1.692D+00,-1.808D+00,
     .-1.910D+00,-2.002D+00,-5.749D-01,-9.352D-01,-1.190D+00,-1.386D+00,
     .-1.547D+00,-1.682D+00,-1.799D+00,-1.902D+00,-1.995D+00/
C
      DATA XFITM/
     . 3.262D-01, 6.135D-01, 9.233D-01, 8.438D-01, 1.020D+00, 1.169D+00,
     . 1.298D+00, 1.411D+00, 1.512D+00, 1.602D+00, 7.228D-01, 1.076D+00,
     . 1.206D+00, 1.404D+00, 1.481D+00, 1.464D+00, 1.581D+00, 1.685D+00,
     . 1.777D+00, 9.586D-01, 1.187D+00, 1.371D+00, 1.524D+00, 1.740D+00,
     . 1.854D+00, 1.955D+00, 2.046D+00, 9.585D-01, 1.041D+00, 1.371D+00,
     . 1.608D+00, 1.739D+00, 1.768D+00, 1.869D+00, 1.803D+00, 7.360D-01,
     . 1.041D+00, 1.272D+00, 1.457D+00, 1.611D+00, 1.741D+00, 1.855D+00,
     . 1.870D+00, 1.804D+00, 9.302D-01, 1.144D+00, 1.028D+00, 1.210D+00,
     . 1.362D+00, 1.646D+00, 1.761D+00, 1.863D+00, 1.954D+00/
C
      DATA A/
     . 6.95319D-01, 1.13101D+00, 1.36313D+00, 1.51684D+00, 1.64767D+00,
     . 1.75643D+00, 1.84458D+00, 1.87243D+00, 1.85628D+00, 1.90889D+00,
     . 9.01802D-01, 1.25389D+00, 1.39033D+00, 1.55226D+00, 1.60658D+00,
     . 1.65930D+00, 1.68855D+00, 1.62477D+00, 1.66726D+00, 1.83599D+00,
     . 2.50403D+00, 3.08564D+00, 3.56545D+00, 4.25922D+00, 4.61346D+00,
     . 4.91417D+00, 5.19211D+00, 1.74181D+00, 2.25756D+00, 2.95625D+00,
     . 3.65899D+00, 4.04397D+00, 4.13410D+00, 4.43538D+00, 4.19583D+00,
     . 1.79027D+00, 2.23543D+00, 2.63942D+00, 3.02461D+00, 3.35018D+00,
     . 3.62067D+00, 3.85218D+00, 3.76689D+00, 3.49318D+00, 1.16294D+00,
     . 1.86467D+00, 2.02110D+00, 2.24231D+00, 2.44240D+00, 2.76594D+00,
     . 2.93230D+00, 3.08109D+00, 3.21069D+00/
C
      DATA B/
     .-1.29000D+00,-2.15771D+00,-2.13263D+00,-2.10272D+00,-2.10861D+00,
     .-2.11507D+00,-2.11710D+00,-2.08531D+00,-2.03296D+00,-2.03441D+00,
     .-1.85905D+00,-2.04057D+00,-2.02189D+00,-2.05930D+00,-2.03403D+00,
     .-2.02071D+00,-1.99956D+00,-1.92851D+00,-1.92905D+00,-4.58608D+00,
     .-4.40022D+00,-4.39154D+00,-4.39676D+00,-4.57631D+00,-4.57120D+00,
     .-4.56188D+00,-4.55915D+00,-4.41218D+00,-4.12940D+00,-4.24401D+00,
     .-4.40783D+00,-4.39930D+00,-4.25981D+00,-4.26804D+00,-4.00419D+00,
     .-4.47251D+00,-3.87960D+00,-3.71668D+00,-3.68461D+00,-3.67173D+00,
     .-3.65991D+00,-3.64968D+00,-3.48666D+00,-3.23985D+00,-2.95758D+00,
     .-3.07110D+00,-2.87157D+00,-2.83137D+00,-2.82132D+00,-2.91084D+00,
     .-2.91159D+00,-2.91336D+00,-2.91296D+00/
C
      DATA XMAX/
     . 3.84251D-01, 9.89081D-01, 1.34758D+00, 1.60093D+00, 1.79766D+00,
     . 1.95867D+00, 2.09411D+00, 2.21217D+00, 2.31555D+00, 2.40796D+00,
     . 8.54216D-01, 1.25914D+00, 1.52406D+00, 1.74001D+00, 1.91064D+00,
     . 2.05087D+00, 2.17074D+00, 2.27614D+00, 2.37017D+00, 1.38737D+00,
     . 1.63108D+00, 1.82209D+00, 1.97849D+00, 2.11121D+00, 2.26666D+00,
     . 2.38139D+00, 2.47258D+00, 1.36716D+00, 1.60889D+00, 1.86129D+00,
     . 2.01803D+00, 2.15086D+00, 2.26617D+00, 2.36801D+00, 2.45919D+00,
     . 9.86915D-01, 1.35996D+00, 1.60525D+00, 1.81878D+00, 1.97653D+00,
     . 2.10895D+00, 2.22480D+00, 2.32641D+00, 2.47969D+00, 1.06043D+00,
     . 1.39913D+00, 1.64584D+00, 1.85192D+00, 2.01031D+00, 2.15231D+00,
     . 2.26861D+00, 2.37124D+00, 2.46305D+00/
C
      DATA ((COEF(I,J),I=1,4),J=1,10)/
     . 8.734D-01,-1.545D+00,-1.093D+00, 5.918D-01, 9.771D-01,-1.567D+00,
     .-4.739D-01,-1.302D-01, 1.174D+00,-1.638D+00,-2.831D-01,-3.281D-02,
     . 1.324D+00,-1.692D+00,-2.916D-01, 9.027D-02, 1.445D+00,-1.761D+00,
     .-1.902D-01, 4.401D-02, 1.546D+00,-1.817D+00,-1.278D-01, 2.293D-02,
     . 1.635D+00,-1.864D+00,-8.252D-02, 9.854D-03, 1.712D+00,-1.903D+00,
     .-5.206D-02, 2.892D-03, 1.782D+00,-1.936D+00,-2.952D-02,-1.405D-03,
     . 1.845D+00,-1.964D+00,-1.152D-02,-4.487D-03/
      DATA ((COEF(I,J),I=1,4),J=11,19)/
     . 7.377D-01,-9.327D-01,-1.466D+00, 6.891D-01, 9.031D-01,-1.157D+00,
     .-7.151D-01, 1.832D-01, 1.031D+00,-1.313D+00,-4.517D-01, 9.207D-02,
     . 1.135D+00,-1.441D+00,-2.724D-01, 3.105D-02, 1.225D+00,-1.536D+00,
     .-1.725D-01, 7.191D-03, 1.302D+00,-1.602D+00,-1.300D-01, 7.345D-03,
     . 1.372D+00,-1.664D+00,-8.204D-02,-1.643D-03, 1.434D+00,-1.715D+00,
     .-4.646D-02,-7.456D-03, 1.491D+00,-1.760D+00,-1.838D-02,-1.152D-02/
      DATA ((COEF(I,J),I=1,4),J=20,27)/
     . 1.258D+00,-3.442D+00,-4.731D-01,-9.522D-02, 1.553D+00,-2.781D+00,
     .-6.841D-01,-4.083D-03, 1.727D+00,-2.494D+00,-5.785D-01,-6.015D-02,
     . 1.853D+00,-2.347D+00,-4.611D-01,-9.615D-02, 1.955D+00,-2.273D+00,
     .-3.457D-01,-1.245D-01, 2.041D+00,-2.226D+00,-2.669D-01,-1.344D-01,
     . 2.115D+00,-2.200D+00,-1.999D-01,-1.410D-01, 2.182D+00,-2.188D+00,
     .-1.405D-01,-1.460D-01/
      DATA ((COEF(I,J),I=1,4),J=28,35)/
     . 1.267D+00,-3.417D+00,-5.038D-01,-1.797D-02, 1.565D+00,-2.781D+00,
     .-6.497D-01,-5.979D-03, 1.741D+00,-2.479D+00,-6.099D-01,-2.227D-02,
     . 1.870D+00,-2.336D+00,-4.899D-01,-6.616D-02, 1.973D+00,-2.253D+00,
     .-3.972D-01,-8.729D-02, 2.061D+00,-2.212D+00,-3.072D-01,-1.060D-01,
     . 2.137D+00,-2.189D+00,-2.352D-01,-1.171D-01, 2.205D+00,-2.186D+00,
     .-1.621D-01,-1.296D-01/
      DATA ((COEF(I,J),I=1,4),J=36,44)/
     . 1.129D+00,-3.149D+00,-1.910D-01,-5.244D-01, 1.431D+00,-2.511D+00,
     .-3.710D-01,-1.933D-01, 1.620D+00,-2.303D+00,-3.045D-01,-1.391D-01,
     . 1.763D+00,-2.235D+00,-1.829D-01,-1.491D-01, 1.879D+00,-2.215D+00,
     .-9.003D-02,-1.537D-01, 1.978D+00,-2.213D+00,-2.066D-02,-1.541D-01,
     . 2.064D+00,-2.220D+00, 3.258D-02,-1.527D-01, 2.140D+00,-2.225D+00,
     . 6.311D-02,-1.455D-01, 2.208D+00,-2.229D+00, 7.977D-02,-1.357D-01/
      DATA ((COEF(I,J),I=1,4),J=45,53)/
     . 1.204D+00,-2.809D+00,-3.094D-01, 1.100D-01, 1.455D+00,-2.254D+00,
     .-4.795D-01, 6.872D-02, 1.619D+00,-2.109D+00,-3.357D-01,-2.532D-02,
     . 1.747D+00,-2.065D+00,-2.317D-01,-5.224D-02, 1.853D+00,-2.058D+00,
     .-1.517D-01,-6.647D-02, 1.943D+00,-2.055D+00,-1.158D-01,-6.081D-02,
     . 2.023D+00,-2.070D+00,-6.470D-02,-6.800D-02, 2.095D+00,-2.088D+00,
     .-2.357D-02,-7.250D-02, 2.160D+00,-2.107D+00, 1.065D-02,-7.542D-02/
C
      SAVE COEF,IST,N0,FL0,A,B,XMAX,XFITM
C
      IF(L.GT.2) GO TO 20
C
C          SELECT BEGINNING AND END OF COEFFICIENTS
C
      SS=(S+1)/2
      LL=L+1
      NSL0=N0(LL,SS)
      I=IST(LL,SS)+N-NSL0
C
C          EVALUATE CROSS SECTION
C
      FL=LOG10(FREQ/FRH)
      X=FL-FL0(I)
      IF(X.GE.-0.001D0) THEN
         IF(X.LT.XFITM(I)) THEN
            P=COEF(4,I)
            DO 10 K=1,3
               P=X*P+COEF(4-K,I)
   10       CONTINUE
            HEPHOT=TENM18*EXP(TENLG*P)
          ELSE
C           OTHERWISE REMOVE INSTRUCTION AND 3 FOLLOWING "C"
C         ELSE IF(X.LT.XMAX(I)) THEN
            HEPHOT=TENM18*EXP(TENLG*(A(I)+B(I)*X))
C         ELSE
C           HEPHOT=TENM18*EXP(TENLG*(COEF(1,I)-TWO))
          END IF
       ELSE
          HEPHOT=0.
      END IF
      RETURN
C
C     Hydrogenic expression for L > 2
C      [multiplied by relative population of state (s,l,n), ie.
C       by  stat.weight(s,l)/stat.weight(n)]
C
   20 GN=TWO*N*N
      HEPHOT=PHOT0/FREQ/FREQ/FREQ/N**5*(2*L+1)*S/GN
      RETURN
      END
C
C
C     ******************************************************************
C
C
      FUNCTION TOPBAS(FREQ,FREQ0,TYPLV)
C     ==================================
C
C     Procedure calculates the photo-ionisation cross section SIGMA in 
C     [cm^2] at frequency FREQ. FREQ0 is the threshold frequency from
C     level I of ion KI. Threshold cross-sections will be of the order
C     of the numerical value of 10^-18.
C     Opacity-Project (OP) interpolation fit formula
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER    (E10=2.3025851)
      PARAMETER    (MMAXOP = 200,! maximum number of levels in OP data
     +              MOP    =  15 )! maximum number of fit points per level
      CHARACTER*10  IDLVOP(MMAXOP) ! level identifyer Opacity-Project data
      CHARACTER*10  TYPLV
      LOGICAL*2     LOPREA   
      COMMON /TOPB/ SOP(MOP,MMAXOP) ,! sigma = alog10(sigma/10^-18) of fit point
     +              XOP(MOP,MMAXOP) ,! x = alog10(nu/nu0) of fit point 
     +              NOP(MMAXOP)     ,! number of fit points for current level
     +              NTOTOP          ,! total number of levels in OP data
     +              IDLVOP         ,! level identifyer Opacity-Project data
     +              LOPREA   ! .T. OP data read in; .F. OP data not yer read in
      DIMENSION XFIT(MOP) ,! local array containing x     for OP data
     +          SFIT(MOP)  ! local array containing sigma for OP data
C
      TOPBAS=0.
C
C     Read OP data if not yet done
C
      IF (.NOT.LOPREA) CALL OPDATA
      X = LOG10(FREQ/FREQ0)
      DO IOP = 1,NTOTOP
         IF (IDLVOP(IOP).EQ.TYPLV) THEN
C           level has been detected in OP-data file
            IF (NOP(IOP).LE.0) GO TO 20
            DO IFIT = 1,NOP(IOP)
               XFIT(IFIT) = XOP(IFIT,IOP)
               SFIT(IFIT) = SOP(IFIT,IOP)
            END DO
            SIGM  = YLINTP (X,XFIT,SFIT,NOP(IOP),MOP)
            SIGM  = 1.D-18*EXP(E10*SIGM)
            TOPBAS=SIGM
            GO TO 10
         END IF
      END DO
   10 RETURN
C     Level is not found ,or no data for this level, in RBF.DAT
   20 WRITE (61,100) TYPLV    
  100 FORMAT ('SIGMA.......: OP DATA NOT AVAILABLE FOR LEVEL ',A10)
      RETURN
      END
C

C     ******************************************************************
C
C
      SUBROUTINE OPDATA
C     =================
C
C     Procedure reads photo-ionization cross sections fit coefficients
C     based on Opacity-Project (OP) data from file RBF.DAT
C     Data, as stored, requires linear interpolation.
C
C     Meaning of global variables:
C        NTOTOP    = total number of levels in Opacity Project data
C        IDLVOP() = level identifyer of current level
C        NOP()     = number of fit points for current level
C        XOP(,)    = x     = alog10(nu/nu0)       of fit point
C        SOP(,)    = sigma = alog10(sigma/10^-18) of fit point 
C      
      INCLUDE 'IMPLIC.FOR'
      PARAMETER    (MMAXOP = 200,! maximum number of levels in OP data
     +              MOP    =  15 )! maximum number of fit points per level
      CHARACTER*10  IDLVOP(MMAXOP) ! level identifyer Opacity-Project data
      LOGICAL*2     LOPREA   
      COMMON /TOPB/ SOP(MOP,MMAXOP) ,! sigma = alog10(sigma/10^-18) of fit point
     +              XOP(MOP,MMAXOP) ,! x = alog10(nu/nu0) of fit point 
     +              NOP(MMAXOP)     ,! number of fit points for current level
     +              NTOTOP          ,! total number of levels in OP data
     +              IDLVOP         ,! level identifyer Opacity-Project data
     +              LOPREA   ! .T. OP data read in; .F. OP data not yer read in
      CHARACTER*4 IONID
C
      OPEN (UNIT=40,FILE='RBF.DAT',STATUS='OLD')
C     Skip header
      DO IREAD = 1, 21
         READ (40,*)
      END DO
      IOP = 0
C         = initialize sequential level index op Opacity Project data 
C     Read number of elements in file
      READ (40,*) NEOP
      DO IEOP = 1, NEOP
C        Skip element name header
         DO IREAD = 1, 3
            READ (40,*)
         END DO
C        Read number of ionization stages of current element in  file
         READ (40,*) NIOP
         DO IIOP = 1, NIOP
C           Read ion identifyer, atomic & electron number, # of levels 
C           for current ion
            READ (40,*) IONID, IATOM_OP, IELEC_OP, NLEVEL_OP
            DO ILOP = 1, NLEVEL_OP
C              Increase sequential level index of Opacity Project data
               IOP = IOP+1
C              Read level identifyer and number of sigma fit points
               READ (40,*) IDLVOP(IOP), NOP(IOP)
C              Read normalized log10 frequency and log10 cross section values
               DO IS = 1, NOP(IOP)
                  READ (40,*) INDEX, XOP(IS,IOP), SOP(IS,IOP)
               END DO
            END DO
         END DO
      END DO
      NTOTOP  = IOP
C             = total number of levels in Opacity Project data
      LOPREA  = .TRUE.
C             = set flag as data has been read in
C
      RETURN
      END
C
C
C
C     ******************************************************************
C
C
      FUNCTION YLINTP (XINT,X,Y,N,NTOT)
C     =================================
C
C     linear interpolation routine. Determines YINT = Y(XINT) from 
C     grid Y(X) with N points and dimension NTOT. 
C
      INCLUDE 'IMPLIC.FOR'
      DIMENSION X(NTOT),Y(NTOT)
C
C     bisection (see Numerical Recipes par 3.4 page 90)
      JL = 0
      JU = N+1
10    IF (JU-JL.GT.1) THEN
         JM = (JU+JL)/2
         IF ((X(N).GT.X(1)).EQV.(XINT.GT.X(JM))) THEN
            JL = JM
         ELSE
            JU = JM
         END IF
         GO TO 10
      END IF
      J = JL
      IF (J.EQ.N) J = J-1
      IF (J.EQ.0) J = J+1
CM42 version: enable extrapolation
cccc  IF (J.EQ.0.OR.J.EQ.N) CALL ERRPRI 
cccc +                      ('YLININTERP: EXTRAPOLATION')
C
      RC         = (Y(J+1)-Y(J))/(X(J+1)-X(J))
      YLINTP = RC*(XINT-X(J))+Y(J)
C
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE GFREE0(ID)
C     =====================
C
C     depth-dependent quantities for the hydrogenic free-free Gaunt factor
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (THET0=5.0404D3,
     *           THET1=UN/THET0,
     *           A0=1.0823d0,
     *           B0=2.98D-2,
     *           C0=6.70D-3,
     *           D0=1.12D-2,
     *           A1=3.9999187d-3,
     *           B1=-7.8622889d-5,
     *           C1=1.070192d0,
     *           A2=6.4628601d-2,
     *           B2=-6.1953813d-4,
     *           C2=2.6061249d-1,
     *           A3=3.7542343d-2,
     *           B3=1.3983474d-5,
     *           C3=5.7917786d-1,
     *           A4=3.4169006d-1,
     *           B4=1.1852264d-2,
     *           XMIN=0.2D0,
     *           XMINI=UN/XMIN,
     *           THMIN=4.0D-2)
C
      T=TEMP(ID)
      THET=UN/MAX(THET0/T,THMIN)
      GF0(ID)=(A0+B0*THET)
      GF1(ID)=(A1+B1*THET)*THET+C1
      GF2(ID)=(A2+B2*THET)*THET+C2
      GF3(ID)=(A3+B3*THET)*THET+C3
      GF4(ID)=A4+B4*THET
      GF5(ID)=C0+D0*THET
      GF6(ID)=GF0(ID)+GF5(ID)*XMINI
C
C     auxiliary quantities for derivatives
C
      THT=THET0/T
      IF(THT.GE.THMIN) THEN 
         THET=THT
         GF0D(ID)=B0*THET1
         GF1D(ID)=(A1+B1*THET*TWO)*THET1
         GF2D(ID)=(A2+B2*THET*TWO)*THET1
         GF3D(ID)=(A3+B3*THET*TWO)*THET1
         GF4D(ID)=B4*THET1
         GF5D(ID)=D0*THET1
         GF6D(ID)=GF0D(ID)+GF5D(ID)*XMINI
       ELSE
         GF0D(ID)=0.
         GF1D(ID)=0.
         GF2D(ID)=0.
         GF3D(ID)=0.
         GF4D(ID)=0.
         GF5D(ID)=0.
         GF6D(ID)=0.
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE GFREE1(ID,FR,CH,GFR)
C     ===============================
C
C     Hydrogenic free-free Gaunt factor, for depth ID,
C     frequency FR, and charge CH
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (C14=2.99793D14,
     *           XMIN=0.2D0,
     *           XMINI=UN/XMIN)
C
      X=C14*CH/FR
      IF(X.LT.UN) THEN
         GFR=((GF4(ID)*X-GF3(ID))*X+GF2(ID))*X+GF1(ID)
       ELSE IF(X.LT.XMINI) THEN
         GFR=GF0(ID)+GF5(ID)*X
       ELSE
         GFR=GF6(ID)
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE GFREED(ID,FR,CH,GFR,GFRD)
C     ====================================
C
C     Hydrogenic free-free Gaunt factor (GFR) and its derivative (GFRD)
C     for depth ID, frequency FR, and charge CH
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (C14=2.997925D14,
     *           XMIN=0.2D0,
     *           XMINI=UN/XMIN)
C
      X=C14*CH/FR
      IF(X.LT.UN) THEN
         GFR=((GF4(ID)*X-GF3(ID))*X+GF2(ID))*X+GF1(ID)
         GFRD=((GF4D(ID)*X-GF3D(ID))*X+GF2D(ID))*X+GF1D(ID)
       ELSE IF(X.LT.XMINI) THEN
         GFR=GF0(ID)+GF5(ID)*X
         GFRD=GF0D(ID)+GF5D(ID)*X
       ELSE
         GFR=GF6(ID)
         GFRD=GF6D(ID)
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      FUNCTION FFCROS(IEL,IFR,T,FR)
C     ==================================
C
C     Non-standard evaluation of free-free cross section;
C     a user supplied procedure
C
      INCLUDE 'IMPLIC.FOR'
      FFCROS=0.
      RETURN
      END
C
C
C     ****************************************************************
C
C

      FUNCTION SBFHMI(FR)
C     ===================
C
C     Bound-free cross-section for H- (negative hydrogen ion)
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (UN=1.D0)
      SBFHMI=0.
      FR0=1.8259D14
      IF(FR.LT.FR0) RETURN
      IF(FR.LT.2.111D14) GO TO 10
      X=2.997925D15/FR
      SBFHMI=(6.80133D-3+X*(1.78708D-1+X*(1.6479D-1+X*(-2.04842D-2+X*
     *        5.95244D-4))))*1.D-17
      RETURN
   10 X=2.997925D15*(UN/FR0-UN/FR)
      SBFHMI=(2.69818D-1+X*(2.2019D-1+X*(-4.11288D-2+X*2.73236D-3)))
     1       *X*1.D-17
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE COLIS(ID,T,COL)
C     ==========================
C
C     Driving procedure for evaluation of the collisional rates
C
C     Input:  T   - temperature
C     Output: COL - array of quantities proportional to the
C                   collisional rates in all transitions
C                   for a given temperature (ie. at a given depth)
C                   Precisely,  COL(IT) * n(el)  is the upward
C                   collisional rate in the transition IT
C
C     Procedure COLIS calls procedures COLH and COLHE for evaluating
C     the collisional rates in hydrogen and helium,
C     and itself evaluates collisional rates for other species
C
C     Evaluation is controlled by input parameter ICOL(ITR).
C     Meaning of ICOL for all species, excluding hydrogen and helium:
C
C      a) for ionization
C         ICOL = 0  - Seatons formula ; here the value of the photo-
C                     ionization cross section at the threshold is
C                     transmitted in array OSC0
C              = 1  - Allen's formula; again, OSC0 has the meaning of
C                     the necessary multiplicative parameter
C              = 2  - the so-called SIMPLE1 mode - see below
C              = 3  - the so-called SIMPLE2 mode - see below
C      b) for excitation
C         ICOL = 0  - Van Regemorter formula, with standard g(bar)=0.25
C              = 1  - Van Regemorter formula, with "exact" g(bar)
C              = 2  - the so-called SIMPLE1 mode - see below
C              = 3  - the so-called SIMPLE2 mode - see below
C              = 4  - Eissner-Seaton formula - see below
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (EXPIA1=-0.57721566,EXPIA2=0.99999193,
     *           EXPIA3=-0.24991055,EXPIA4=0.05519968,
     *           EXPIA5=-0.00976004,EXPIA6=0.00107857,
     *           EXPIB1=0.2677734343,EXPIB2=8.6347608925,
     *           EXPIB3=18.059016973,EXPIB4=8.5733287401,
     *           EXPIC1=3.9584969228,EXPIC2=21.0996530827,
     *           EXPIC3=25.6329561486,EXPIC4=9.5733223454)
      DIMENSION COL(MTRANS)
      COMMON/OSCHYD/OSH(20,20)
C
      CREGER(X,U,A,GG)=19.7363*X*EXP(-U)/U*GG*A
      CSEATN(X,U,A)=1.55D13*X/ABS(U)*EXP(-U)*A
      CALLEN(X,U,A)=X*A*EXP(-U)/U/U
      CSMPL1(X,U,A)=5.465D-11*X*EXP(-U)*A
      CSMPL2(X,U,A)=5.465D-11*X*EXP(-U)*A*(1.+U)
      CUPSX(X,U,A)=8.631D-6/X*EXP(-U)*A
C
    1 DO 10 I=1,NTRANS
   10    COL(I)=0.
      HKT=HK/T
      SRT=SQRT(T)
      T32=UN/T/SRT
      TK=HKT/H
      CSTD=0.25
C
C     Call procedures COLH and COLHE for hydrogen and helium
C
      IF(IATH.NE.0) CALL COLH(ID,T,COL)
      IF(IATHE.NE.0) CALL COLHE(ID,T,COL)
C
C     Loop over all explicit species, excluding hydrogen and helium
C
      DO 100 IAT=1,NATOM
      IF(IAT.EQ.IATH.OR.IAT.EQ.IATHE) GO TO 100
      N0I=N0A(IAT)
      NKI=NKA(IAT)
      DO 50 I=N0I,NKI-1
         IE=IEL(I)
         DO 40 J=I+1,NKI
            IT=ITRA(I,J)
            IF(IT.EQ.0) GO TO 40
            IC=ICOL(IT)
            C1=OSC0(IT)
            C2=CPAR(IT)
            U0=FR0(IT)*HKT
            IF(LINE(IT)) GO TO 30
C
C ********* Collisional ionization
C
            IF(IC.EQ.0) THEN
               COL(IT)=CSEATN(UN/SRT,U0,C1)
             ELSE IF(IC.EQ.1) THEN
               COL(IT)=CALLEN(T32,U0,C1)
             ELSE IF(IC.EQ.2) THEN
               COL(IT)=CSMPL1(SRT,U0,C1)
             ELSE IF(IC.EQ.3) THEN
               COL(IT)=CSMPL2(SRT,U0,C1)
             ELSE IF(IC.LT.0) THEN
               CALL CSPEC(I,J,IC,C1,C2,U0,T,COL(IT))
            END IF
C
C      collisional excitations from level I to higher, non-explicit
C      levels are lumped into the collisional ionization rate
C      (the so-called modified collision ionization rate)
C
            N0Q=NQUANT(NLAST(IE))+1
            N1Q=ICUP(IE)
            IF(N1Q.EQ.0) GO TO 40
            IQ=NQUANT(I)
            REL=G(I)/2./IQ/IQ
            DO 20 JQ=N0Q,N1Q
               XJ=JQ
               U0=(ENION(I)-EH/XJ/XJ)*TK
               IF(JQ.LE.20) CC1=OSH(IQ,JQ)*REL
               IF(JQ.GT.20) CC1=OSH(IQ,20)*(20./XJ)**3*REL
               GG=CSTD
               IF(U0.LE.UN) THEN
         EXPIU0=-LOG(U0)+EXPIA1+U0*(EXPIA2+U0*(EXPIA3+U0*(EXPIA4+
     *            U0*(EXPIA5+U0*EXPIA6))))
               ELSE
         EXPIU0=EXP(-U0)*((EXPIB1+U0*(EXPIB2+U0*(EXPIB3+
     *           U0*(EXPIB4+U0))))/(EXPIC1+U0*(EXPIC2+
     *           U0*(EXPIC3+U0*(EXPIC4+U0)))))/U0
               END IF
               GG0=0.276*EXP(U0)*EXPIU0
               IF(GG0.GT.GG) GG=GG0
               COL(IT)=COL(IT)+CREGER(T32,U0,CC1,GG)
   20       CONTINUE
            GO TO 40
C
C ********* Collisional excitation
C
   30       CONTINUE
            IF(IC.LE.1) THEN
               GG=CSTD
               IF(IC.EQ.1) GG=C2
               IF(U0.LE.UN) THEN
         EXPIU0=-LOG(U0)+EXPIA1+U0*(EXPIA2+U0*(EXPIA3+U0*(EXPIA4+
     *            U0*(EXPIA5+U0*EXPIA6))))
               ELSE
         EXPIU0=EXP(-U0)*((EXPIB1+U0*(EXPIB2+U0*(EXPIB3+
     *           U0*(EXPIB4+U0))))/(EXPIC1+U0*(EXPIC2+
     *           U0*(EXPIC3+U0*(EXPIC4+U0)))))/U0
               END IF
               GG0=0.276*EXP(U0)*EXPIU0
               IF(GG0.GT.GG) GG=GG0
               COL(IT)=CREGER(T32,U0,C1,GG)
             ELSE IF(IC.EQ.2) THEN
               COL(IT)=CSMPL1(SRT,U0,C1*C2)
             ELSE IF(IC.EQ.3) THEN
               COL(IT)=CSMPL2(SRT,U0,C2)
             ELSE IF(IC.EQ.4) THEN
               COL(IT)=CUPSX(SRT,U0,C2/G(I))
             ELSE IF(IC.LT.0) THEN
               CALL CSPEC(I,J,IC,C1,C2,U0,T,COL(IT))
            END IF
   40    CONTINUE
   50 CONTINUE
  100 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE COLH(ID,T,COL)
C     =========================
C
C     Hydrogen collision rates
C
C     All standard expressions  are taken from Mihalas, Heasley, and
C     Auer, NCAR-TN/STR-104 (1975)
C
C     New expressions (also from Mihalas) for collisional ionization
C     for first 10 levels taken from Klaus Werner.
C
C     Meaning of ICOL:
C      a) for ionization - .ge.0  - standard expression
C                           <  0  - non-standard, user suplied formula
C      b) for 1 - 2 transition =0 - standard theoretical formula
C                             = 1 - experimental fit (formula quted in
C                                   Mihalas et al.)
C                             = 2 - formula by Crandall et al (procedure
C                                   CEH12)
C      c) for all other line transitions
C                           .ge.0 - standard expression
C                            <  0 - non-standard, user supplie formula
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (CC0   =   5.465D-11,
     *           CEX1  = -30.20581,
     *           CEX2  =   3.8608704,
     *           CEX3  = 305.63574,
     *           CI1   =   0.3,
     *           CI2   =   0.435,
     *           CA1   =   5.444416D7,
     *           CA2   =  -2.8185937D4,
     *           CA3   =  19.987261,
     *           CA4   =  -5.8906298D-5,
     *           CB1   =   1.3935312D3,
     *           CB2   =  -1.6805859D2,
     *           CB3   =  -2.539D3,
     *           CC1   =   2.0684609D3,
     *           CC2   =  -3.341582D2,
     *           CC3   =  -7.6440625D3,
     *           CD1   =   3.2174844D3,
     *           CD2   =  -5.5882422D2,
     *           CD3   =  -6.86325D3,
     *           CE1   =   5.759125D3,
     *           CE2   =  81.75,
     *           CE3   =  -1.5163D3,
     *           CF1   =   1.461475D4,
     *           CF2   = 393.4,
     *           CF3   =  -4.8284D3,
     *           ALF0  =   1.8,
     *           ALF1  =   0.4,
     *           BET0  =   3.0,
     *           BET1  =   1.2,
     *           O148  =   0.148,
     *           CHMI  =   5.59D-15)
      PARAMETER (EXPIA1=-0.57721566,EXPIA2=0.99999193,
     *           EXPIA3=-0.24991055,EXPIA4=0.05519968,
     *           EXPIA5=-0.00976004,EXPIA6=0.00107857,
     *           EXPIB1=0.2677734343,EXPIB2=8.6347608925,
     *           EXPIB3=18.059016973,EXPIB4=8.5733287401,
     *           EXPIC1=3.9584969228,EXPIC2=21.0996530827,
     *           EXPIC3=25.6329561486,EXPIC4=9.5733223454)
      DIMENSION COL(MTRANS),A(6,10)
      COMMON/OSCHYD/OSH(20,20)
      DATA ((A(I,J),J=1,10),I=1,6) /
     * -86.7633398, 2632.8369 , 7478.9556 ,-4202.8442 ,-47995.930    ,
     * -120942.89 ,-202300.81 ,-261373.03 ,-266337.91 ,-192293.20    ,
     * 100.919188 ,-2738.7485 ,-8495.4590 , 1937.3763 , 45825.371    ,
     *  122209.39 , 211928.67 , 285044.75 , 309455.47 , 258802.22    ,
     * -45.7813807, 1121.3976 , 3794.6826 , 340.35764 ,-16617.055    ,
     * -47390.313 ,-84973.688 ,-117833.95 ,-133243.61 ,-120363.95    ,
     * 10.1978559 ,-224.30670 ,-822.83636 ,-290.10489 , 2905.7393    ,
     *  8944.6025 , 16556.992 , 23544.543 , 27419.742 , 26002.143    ,
     * -1.11223557, 21.923729 , 86.619110 , 48.840523 ,-246.99014    ,
     * -828.41028 ,-1581.2722 ,-2297.9321 ,-2738.1743 ,-2686.4087    ,
     *  .0474198818,-.83974838,-3.5534720 ,-2.6097214 , 8.1972208    ,
     *  30.267115 , 59.521984 , 88.178680 , 107.05288 , 107.73775    /
C
C     experimental fit formula for Lyman-alpha
C
      CEXP12(CC,X,U)=CC*EXP(-U)*(CEX1+CEX2*X+CEX3/X/X)
C
      HKT=HK/T
      CT=CC0*SQRT(T)
      TK=HKT/H
      X=LOG10(T)
      X2=X*X
      X3=X*X2
      X4=X2*X2
      X5=X3*X2
      N0HN=NFIRST(IELH)
      N1H=NLAST(IELH)
      NKH=NKA(IATH)
      N0Q=NQUANT(N1H)+1
      N1Q=ICUP(IELH)
      DO 200 II=N0HN,N1H
         I=II-N0HN+1
         IT=ITRA(II,NKH)
         IF(IT.EQ.0) GO TO 100
C
C *************** Collisional ionization
C
         IC=ICOL(IT)
         U0=FR0(IT)*HKT
         IF(IC.LT.0) GO TO 90
         if(ifwop(ii).lt.0) go to 95
         GAM=I*I*I
c        IF(I.GT.7) GO TO 80
c        GO TO (10,20,30,40,50,60,70),I
c  10    GAM=CI1*X-CI2
c        GO TO 80
c  20    GAM=(CA1/T+CA2)/T+CA3+CA4*T
c        GO TO 80
c  30    GAM=CB1+CB2*X+CB3/X
c        GO TO 80
c  40    GAM=CC1+CC2*X+CC3/X/X
c        GO TO 80
c  50    GAM=CD1+CD2*X+CD3/X/X
c        GO TO 80
c  60    GAM=CE1+(CE2*X+CE3)*X
c        GO TO 80
c  70    GAM=CF1+(CF2*X+CF3)*X
         IF(I.GT.10) GO TO 80
         GAM=A(1,I)+A(2,I)*X+A(3,I)*X2+A(4,I)*X3
     *      +A(5,I)*X4+A(6,I)*X5
   80    COL(IT)=CT*EXP(-U0)*GAM
         GO TO 100
C
C      non-standard (user supplied) formula
C
   90    CALL CSPEC(II,NKH,IC,OSC0(IT),CPAR(IT),U0,T,COL(IT))
         go to 100
c
c      ionization from the merged state
c
   95    sum1=0.
         sum2=0.
         ehk=eh/tk
         n00q=nquant(n1h-1)+1
         n11q=nlmx
         do 96 img=n00q,n11q
            xi=img
            xii=xi*xi
            sum1=sum1+xii*xii*xi*wnhint(img,id)
            sum2=sum2+xii*wnhint(img,id)*exp(ehk/xii)
   96    continue
         col(it)=ct*sum1/sum2
         go to 200
C
C ***************** Collisional excitation
C
  100    I1=I+1
         XI=I
         VI=XI*XI
         ALF=ALF0-ALF1/VI
         BET=BET0-BET1/XI
         NHL=N1H-N0HN+1
         IF(N1Q.GT.0) NHL=N1Q
         N1HC=N1H
         IF(IFWOP(N1H).LT.0) THEN
            NHL=NLMX
            N1HC=N1H-1
         END IF
         CSUM=0.
         IF(I1.GT.NHL) GO TO 200
         DO 190 J=I1,NHL
            XJ=J
            VJ=XJ*XJ
            IC=0
            JJ=J+N0HN-1
            IF(JJ.GT.N1HC) GO TO 150
            ICT=ITRA(II,JJ)
            IF(ICT.EQ.0) GO TO 190
            IC=ICOL(ICT)
            U0=FR0(ICT)*HKT
            E=U0/EH/TK
            C1=OSC0(ICT)
            IF(IC) 110,160,120
  110       CALL CSPEC(II,JJ,IC,C1,CPAR(ICT),U0,T,COL(ICT))
            GO TO 190
  120       IF(IC-1) 130,130,140
  130       COL(ICT)=CEXP12(CT,X,U0)
            GO TO 190
  140       COL(ICT)=CEH12(T)
            GO TO 190
C
C      collisional excitations from level I to higher, non-explicit
C      levels are lumped into the collisional ionization rate
C      (the so-called modified collision ionization rate)
C
  150       CONTINUE
            E=UN/VI-UN/VJ
            U0=EH*E*TK
            IF(J.LE.20) C1=OSH(I,J)
            IF(J.GT.20) THEN
               C1=OSH(I,20)*((400.-VI)/20.*XJ/(VJ-VI))**3
            end if
C
C      Standard formula for the collisional excitation rate - used for
C      rates in explicit transitions as well as for evaluation of the
C      modified collisional rate
C
  160       CS=4.*CT*C1/E/E
            EX=EXP(-U0)
            IF(U0.LE.UN) THEN
              E1=-LOG(U0)+EXPIA1+U0*(EXPIA2+U0*(EXPIA3+U0*(EXPIA4+
     *            U0*(EXPIA5+U0*EXPIA6))))
            ELSE
              E1=EXP(-U0)*((EXPIB1+U0*(EXPIB2+U0*(EXPIB3+
     *           U0*(EXPIB4+U0))))/(EXPIC1+U0*(EXPIC2+
     *           U0*(EXPIC3+U0*(EXPIC4+U0)))))/U0
            END IF
            E5=E1
            DO 170 IX=1,4
  170          E5=(EX-U0*E5)/IX
            CS=CS*U0*(E1+O148*U0*E5)
            IF(J-I.NE.1) CS=CS*(BET+TWO*(ALF-BET)/(XJ-XI))
C      End of the standard formula
C
            IF(JJ.GT.N1HC) THEN
               CSUM=CSUM+CS
             ELSE
               COL(ICT)=CS
            END IF
  190    CONTINUE
         IF(IT.NE.0.AND.N1Q.GT.0) COL(IT)=COL(IT)+CSUM
         ITH=ITRA(II,N1H)
         IF(IFWOP(N1H).LT.0.AND.ITH.GT.0) COL(ITH)=CSUM
  200 CONTINUE
C
C     special standard formula for collisional ionization of H-
C
      IF(IELHM.EQ.0) RETURN
      IT=ITRA(NFIRST(IELHM),N0HN)
      IF(IT.EQ.0) RETURN
      IC=ICOL(IT)
      IF(IC) 220,210,210
  210 COL(IT)=CHMI*T*SQRT(T)
      RETURN
C
C     if desired, non-standard, user supplied, formula for H-
C
  220 U0=ENION(NFIRST(IELHM))*TK
      CALL CSPEC(NFIRST(IELHM),N0HN,IC,OSC0(IT),CPAR(IT),U0,T,COL(IT))
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE COLHE(ID,T,COL)
C     ==========================
C
C     Helium (both neutral and ionized) collision rates
C
C     Meaning of ICOL: for all kinds of transitions and for both HeI
C     and HeII:
C      ICOL =  0  - approximate expressions taken from Mihalas, Heasley
C                   and Auer, NCAR-TN/STR-104 (1975) - for He I and II
C
C     For He I bound-bound transitions, the following standard
C     possibilities are also available:
C
C      ICOL =  1, 2, or 3  - much more accurate Storey's rates,
C                   subroutine written by D.G.Hummer (COLLHE).
C                   This procedure can be used only for transitions
C                   between states with n = 1, 2, 3, 4.
C      ICOL =  1  - means that a given transition is a transition
C                   between non-averaged (l,s) states. In this case,
C                   labeling of the He I energy levels must agree
C                   with that given in subroutine COLLHE, ie. states
C                   have to be labeled sequentially in order of
C                   increasing frequency.
C      ICOL =  2  - means that a given transition is a transition between
C                   a non-averaged (l,s) lower state and averaged upper
C                   state.
C      ICOL =  3  - means that a given transition is a transition between
C                   two averaged states.
C      Note:
C     The program allows only two standard possibilities of
C     constructing averaged levels of He I:
C     i)  all states within given principal quantum number n (>1) are
C         lumped together
C     ii) all siglet states for given n, and all triplet states for
C         given n are lumped together separately (there are thus two
C         explicit levels for a given n).
C     If the user wants to use another averaging, he had to take care
C     of appropriate averaged collisional rates himself (by updating
C     subroutine CSPEC)
C
C
C      ICOL <  0  - non-standard, user supplied formula
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      DIMENSION COL(MTRANS)
      DIMENSION FHE1(16),G0(3),G1(3),G2(3),G3(3)
      COMMON/OSCHYD/OSH(20,20)
      COMMON/HECRAT/COLHE1(19,19)
      PARAMETER (EXPIA1=-0.57721566,EXPIA2=0.99999193,
     *           EXPIA3=-0.24991055,EXPIA4=0.05519968,
     *           EXPIA5=-0.00976004,EXPIA6=0.00107857,
     *           EXPIB1=0.2677734343,EXPIB2=8.6347608925,
     *           EXPIB3=18.059016973,EXPIB4=8.5733287401,
     *           EXPIC1=3.9584969228,EXPIC2=21.0996530827,
     *           EXPIC3=25.6329561486,EXPIC4=9.5733223454)
      DATA FHE1/0.,2.75D-1,7.29D-2,2.96D-2,1.48  D-2,8.5D-3,5.3D-3,
     *          3.5D-3,2.5D-3,1.8D-3,1.5D-3,1.2D-3,9.4D-4,7.5D-4,
     *          6.1D-4,5.3D-4/
      DATA G0/ 7.3399521D-2,  1.7252867,      8.6335087    /,
     *     G1/-1.4592763D-7,  2.0944117D-6,   2.7575544D-5 /,
     *     G2/ 7.6621299D5,   5.4254879D6,    6.6395519D6  /,
     *     G3/ 2.3775439D2,   2.2177891D3,    5.20725D3    /
      SAVE FHE1,G0,G1,G2,G3
C
      HKT=HK/T
      TK=HKT/H
      SRT=SQRT(T)
      CT=5.465D-11*SRT
      CT1=5.4499487/T/SRT
C
C     --------------
C     Neutral helium
C     --------------
C
      IF(IELHE1.EQ.0) GO TO 60
      ICALL=0
      N0I=NFIRST(IELHE1)
      N1I=NLAST(IELHE1)
      NKI=NNEXT(IELHE1)
      N0Q=NQUANT(NLAST(IELHE1))+1
      N1Q=ICUP(IELHE1)
      DO 50 II=N0I,N1I
         IT=ITRA(II,NKI)
         IF(IT.EQ.0) GO TO 10
C
C ******** Collisional ionization
C
         IC=ICOL(IT)
         C1=OSC0(IT)
         C2=CPAR(IT)
         U0=ENION(II)*TK
         IF(IC.GE.0) THEN
            U1=U0+0.27
            U2=(U0+3.43)/(U0+1.43)**3
            IF(U0.LE.UN) THEN
         EXPIU0=-LOG(U0)+EXPIA1+U0*(EXPIA2+U0*(EXPIA3+U0*(EXPIA4+
     *            U0*(EXPIA5+U0*EXPIA6))))
            ELSE
         EXPIU0=EXP(-U0)*((EXPIB1+U0*(EXPIB2+U0*(EXPIB3+
     *           U0*(EXPIB4+U0))))/(EXPIC1+U0*(EXPIC2+
     *           U0*(EXPIC3+U0*(EXPIC4+U0)))))/U0
            END IF
            IF(U1.LE.UN) THEN
         EXPIU1=-LOG(U1)+EXPIA1+U1*(EXPIA2+U1*(EXPIA3+U1*(EXPIA4+
     *            U1*(EXPIA5+U1*EXPIA6))))
            ELSE
         EXPIU1=EXP(-U1)*((EXPIB1+U1*(EXPIB2+U1*(EXPIB3+
     *           U1*(EXPIB4+U1))))/(EXPIC1+U1*(EXPIC2+
     *           U1*(EXPIC3+U1*(EXPIC4+U1)))))/U1
            END IF
            COL(IT)=CT*C1*U0*(EXPIU0-U0*(0.728*EXPIU1/U1+
     *              0.189*EXP(-U0)*U2))
          ELSE
            CALL CSPEC(II,NKI,IC,C1,C2,U0,T,COL(IT))
         END IF
   10    IF(II.GE.N1I) GO TO 30
C
C ********* Collisional excitation
C
         DO 20 JJ=II+1,N1I
            ICT=ITRA(II,JJ)
            IF(ICT.EQ.0) GO TO 20
            IC=ICOL(ICT)
            C1=OSC0(ICT)
            C2=CPAR(ICT)
            U0=FR0(ICT)*HKT
            IF(IC.EQ.0) THEN
C
C   *** ICOL = 0    Formula used by Mihalas, Heasley, and Auer
C
               IF(U0.LE.UN) THEN
            EX=-LOG(U0)+EXPIA1+U0*(EXPIA2+U0*(EXPIA3+U0*(EXPIA4+
     *            U0*(EXPIA5+U0*EXPIA6))))
               ELSE
            EX=EXP(-U0)*((EXPIB1+U0*(EXPIB2+U0*(EXPIB3+
     *           U0*(EXPIB4+U0))))/(EXPIC1+U0*(EXPIC2+
     *           U0*(EXPIC3+U0*(EXPIC4+U0)))))/U0
               END IF
               IF(II.EQ.N0I) THEN
C
C          excitation from the ground state
C
                  COL(ICT)=CT1*EX/U0*C1
                ELSE
C
C          transitions between excited states
C
                  U1=U0+0.2
                  IF(U1.LE.UN) THEN
          EXPIU1=-LOG(U1)+EXPIA1+U1*(EXPIA2+U1*(EXPIA3+U1*(EXPIA4+
     *            U1*(EXPIA5+U1*EXPIA6))))
                  ELSE
          EXPIU1=EXP(-U1)*((EXPIB1+U1*(EXPIB2+U1*(EXPIB3+
     *           U1*(EXPIB4+U1))))/(EXPIC1+U1*(EXPIC2+
     *           U1*(EXPIC3+U1*(EXPIC4+U1)))))/U1
                  END IF
                  COL(ICT)=CT1/U0*(EX-U0/U1*0.81873*EXPIU1)*C1
               END IF
             ELSE IF(IC.EQ.1) THEN
C
C   *** ICOL = 1   Storey - Hummer collisional rates between
C                  non-averaged states
C                  (Note: procedure COLLHE, which calculates all rates,
C                  is called only once)
C
               IF(ICALL.EQ.0) CALL COLLHE(T,COLHE1)
               ICALL=1
               COL(ICT)=COLHE1(II-N0I+1,JJ-N0I+1)
             ELSE IF(IC.EQ.2.OR.IC.EQ.3) THEN
C
C   *** ICOL = 2 or 3  Storey - Hummer collisional rates between
C                      averaged states
C
               IF(ICALL.EQ.0) CALL COLLHE(T,COLHE1)
               ICALL=1
               COL(ICT)=CHEAV(II,JJ,IC)
             ELSE IF(IC.LT.0) THEN
C
C       Non-standard, user supplied formula
C
               CALL CSPEC(II,JJ,IC,C1,CPAR(ICT),U0,T,COL(ICT))
            END IF
   20    CONTINUE
C
C      collisional excitations from level II to higher, non-explicit
C      levels are lumped into the collisional ionization rate
C      (the so-called modified collision ionization rate);
C      the individual rates are calculated by expressions used by
C      Mihalas, Heasley, and Auer
C
   30    IF(N1Q.EQ.0.OR.IT.EQ.0) GO TO 50
         I=NQUANT(II)
         REL=G(II)/2./I/I
         DO 40 J=N0Q,N1Q
            XJ=J
            U0=(ENION(II)-EH/XJ/XJ)*TK
            IF(I.EQ.1) THEN
               GAM=0.
               C1=FHE1(J)
             ELSE
               C1=OSH(I,J)*REL
               U1=U0+0.2
               IF(U1.LE.UN) THEN
          EXPIU1=-LOG(U1)+EXPIA1+U1*(EXPIA2+U1*(EXPIA3+U1*(EXPIA4+
     *            U1*(EXPIA5+U1*EXPIA6))))
               ELSE
          EXPIU1=EXP(-U1)*((EXPIB1+U1*(EXPIB2+U1*(EXPIB3+
     *           U1*(EXPIB4+U1))))/(EXPIC1+U1*(EXPIC2+
     *           U1*(EXPIC3+U1*(EXPIC4+U1)))))/U1
               END IF
               GAM=U0/U1*0.81873*EXPIU1
            END IF
            IF(U0.LE.UN) THEN
          EXPIU0=-LOG(U0)+EXPIA1+U0*(EXPIA2+U0*(EXPIA3+U0*(EXPIA4+
     *            U0*(EXPIA5+U0*EXPIA6))))
            ELSE
          EXPIU0=EXP(-U0)*((EXPIB1+U0*(EXPIB2+U0*(EXPIB3+
     *           U0*(EXPIB4+U0))))/(EXPIC1+U0*(EXPIC2+
     *           U0*(EXPIC3+U0*(EXPIC4+U0)))))/U0
            END IF
            COL(IT)=COL(IT)+CT1/U0*C1*(EXPIU0-GAM)
   40    CONTINUE
   50 CONTINUE
C
C     --------------
C     Ionized helium
C     --------------
C
   60 IF(IELHE2.EQ.0) RETURN
      N0I=NFIRST(IELHE2)
      N1I=NLAST(IELHE2)
      NKI=NNEXT(IELHE2)
      N0Q=NQUANT(NLAST(IELHE2))+1
      N1Q=ICUP(IELHE2)
      X=LOG10(T)
      X2=X*X
      CT2=3.7036489/T/SRT
C
      DO 200 II=N0I,N1I
         I=II-N0I+1
         IT=ITRA(II,NKI)
         IF(IT.EQ.0) GO TO 100
C
C ********* Collisional ionization
C
         IC=ICOL(IT)
	 U0=FR0(IT)*HKT
	 IF(IC.GE.0) THEN
            IF(I.LE.3) THEN
               GAM=G0(I)-G1(I)*T+(G2(I)/T-G3(I))/T
             ELSE IF(I.EQ.4) THEN
               GAM=-95.23828+(62.656249-8.1454078*X)*X
	     ELSE IF(I.EQ.5) THEN
               GAM=472.99219-74.144287*X-1869.6562/X2
	     ELSE IF(I.EQ.6) THEN
               GAM=825.17186-134.23096*X-2739.4375/X2
	     ELSE IF(I.EQ.7) THEN
               GAM=1181.3516-200.71191*X-2810.7812/X2
	     ELSE IF(I.EQ.8) THEN
               GAM=1440.1016-259.75781*X-1283.5625/X2
	     ELSE IF(I.EQ.9) THEN
               GAM=2492.1250-624.84375*X+30.101562*X2
	     ELSE IF(I.EQ.10) THEN
               GAM=4663.3129-1390.1250*X+97.671874*X2
             ELSE
               GAM=I*I*I
            END IF
            COL(IT)=CT*EXP(-U0)*GAM
          ELSE
            CALL CSPEC(II,NKI,IC,OSC0(IT),CPAR(IT),U0,T,COL(IT))
         END IF
C
  100    I1=I+1
         XI=I
         VI=XI*XI
         NHL=N1I-N0I+1
         IF(N1Q.GT.0) NHL=N1Q
         IF(I1.GT.NHL) GO TO 200
C
C ********** collisional excitation
C
C     both explicit transitions as well as contributions to the
C     modified collisional ionization rate
C
         DO 150 J=I1,NHL
            JJ=J+N0I-1
            IC=0
            IF(JJ.GT.N1I) GO TO 110
            ICT=ITRA(II,JJ)
            IF(ICT.EQ.0) GO TO 150
            IC=ICOL(ICT)
  110       XJ=J
            VJ=XJ*XJ
            U0=ENION(N0I)*(1./VI-1./VJ)*TK
            IF(J.LE.20) C1=OSH(I,J)
            IF(J.GT.20) C1=OSH(I,20)*(20./XJ)**3
            IF(IC.LT.0) GO TO 120
            GAM=XI-(XI-1.)/(XJ-XI)
            IF(GAM.GT.XJ-XI) GAM=XJ-XI
            IF(I.GT.1) GAM=GAM*1.1
            IF(U0.LE.UN) THEN
          EXPIU0=-LOG(U0)+EXPIA1+U0*(EXPIA2+U0*(EXPIA3+U0*(EXPIA4+
     *            U0*(EXPIA5+U0*EXPIA6))))
            ELSE
          EXPIU0=EXP(-U0)*((EXPIB1+U0*(EXPIB2+U0*(EXPIB3+
     *           U0*(EXPIB4+U0))))/(EXPIC1+U0*(EXPIC2+
     *           U0*(EXPIC3+U0*(EXPIC4+U0)))))/U0
            END IF
            CS=CT2/U0*C1*(0.693*EXP(-U0)+EXPIU0)*GAM
            GO TO 130
  120       CALL CSPEC(II,JJ,IC,C1,CPAR(ICT),U0,T,COL(ICT))
            GO TO 150
  130       IF(JJ.GT.N1I) GO TO 140
            COL(ICT)=CS
            GO TO 150
  140       IF(IT.NE.0) COL(IT)=COL(IT)+CS
  150    CONTINUE
  200 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      FUNCTION CEH12(T)
C     =================
C
C     Special formula for collisional rate in hydrogen Lyman-alpha
C     transition
C     After Crandall et al. Ap.J. 191, 789 (1974)
C
      INCLUDE 'IMPLIC.FOR'
      DIMENSION A(6),B(8)
      PARAMETER (C=-118353.41)
      DATA A/ 2.579997D-10, -1.629166D-10,  7.713069D-11,
     *       -2.668768D-11,  6.642513D-12, -9.422885D-13/
      SAVE A
c
      DO 10 I=1,8
   10    B(I)=0.
      X=LOG10(T)-4.
      DO 20 I=1,6
         J=7-I
         B(J)=2.*X*B(J+1)-B(J+2)+A(J)
   20 CONTINUE
      CEH12=2.4*SQRT(T)*(B(1)-B(3))*EXP(C/T)
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE CSPEC(I,J,IC,OS,CP,U0,T,CS)
C     ======================================
C
C     Non-standard evaluation of collision rates
C     Basically user-suppled procedure; here is an example
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      COMMON/HECRAT/COLHE1(19,19)
C
      CS=0.
      XR=-1.68D0
      IF(IC.EQ.-1) CS=2.16*U0**XR/T/SQRT(T)*EXP(-U0)*OS
      RETURN
      END
C
C
C     ****************************************************************
C
C


      SUBROUTINE COLLHE(TEMP,COLHE1)
C     ==============================
C
C          GENERATES COLLISIONAL RATE COEFFICIENTS AMONG THE 19 STATES OF
C          HELIUM WITH N = 1, 2, 3, AND 4, USING RATES EVALUATED FROM THE
C          CROSS SECTIONS CALCULATED BY BERRINGTON AND KINGSTON (J. PHYS.B. 20,
C          6631(1987)).  COLLISIONAL RATE COEFFICIENTS HAVE BEEN EVALUATED
C          NUMERICALLY BY P.J.STOREY FROM THE UNPUBLISHED COMPUTER OUTPUT
C          FILES OF BERRINGTON AND KINGSTON.
C
C          THE STATES INCLUDED IN THE CALCULATION ARE LABELLED SEQUENTIALLY
C          IN ORDER OF INCREASING ENERGY:
C                           1          1 SING S
C                           2          2 TRIP S
C                           3          2 SING S
C                           4          2 TRIP P
C                           5          2 SING P
C                           6          3 TRIP S
C                           7          3 SING S
C                           8          3 TRIP P
C                           9          3 TRIP D
C                          10          3 SING D
C                          11          3 SING P
C                          12          4 TRIP S
C                          13          4 SING S
C                          14          4 TRIP P
C                          15          4 TRIP D
C                          16          4 SING D
C                          17          4 TRIP F
C                          18          4 SING F
C                          19          4 SING P
C
C          THIS ORDERING DIFFERS SLIGHTLY FROM THAT OF BERRINGTON AND KINGSTON,
C          IN WHICH 15 AND 16, AND 17 AND 18, WERE INTERCHANGED.
C
C          THE INTRINSIC ACCURACY OF TRANSITIONS AMONG STATES WITH N = 1, 2,
C          AND 3 IS EXPECTED TO BE CONSIDERABLY BETTER THAN THOSE WITH N = 4
C          AS DISCUSSED BY BERRINGTON AND KINGSTON.  THE FITTING ACCURACY IS
C          EVERYWHERE BETTER THAN 2%.  THE ENERGIES OF THE LEVELS ARE TAKEN
C          FROM W. C. MARTIN, PHYS. CHEM. REF. DATA., VOL.2, 257 (1973) AND
C          ARE GIVEN IN ELECTRON VOLTS.  (BOLTZMANN'S CONSTANT = 8.62E-5)
C
C          FIRST REVISED VERSION: D.G.HUMMER, MAY 1988, JILA
C          slightly modified by I.H.,  July 1988
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (UN=1.D0,
     *           C1=3.849485D0,
     *           C2=8.49485002D-1,
     *           N=19)
      CHARACTER*14 IDEN
      DIMENSION IDEN(19),ENER(19),COLHE1(19,19),
     *          NSTART(172),A(929),B(10),STWT(19)
C
      DATA ENER/ 0.0D0,19.8198D0,20.6160D0,20.96432D0,21.2182D0,
     .22.7187D0,22.9206D0,23.00731D0,23.0739D0,23.0743D0,23.0873D0,
     .23.5942D0,23.6738D0,23.7081D0,23.7363D0,23.7366D0,23.7373D0,
     .23.7373D0,23.7423D0/
C
      DATA STWT/1.0D0,3.0D0,1.0D0,9.0D0,3.0D0,3.0D0,1.0D0,9.0D0,
     .1.5D1,5.0D0,3.0D0,3.0D0,1.0D0,9.0D0,1.5D1,5.0D0,2.1D1,
     .7.0D0,3.0D0/
C
      DATA IDEN/'HE I 1 SING S','HE I 2 TRIP S','HE I 2 SING S',
     .'HE I 2 TRIP P','HE I 2 SING P','HE I 3 TRIP S','HE I 3 SING S',
     .'HE I 3 TRIP P','HE I 3 TRIP D','HE I 3 SING D','HE I 3 SING P',
     .'HE I 4 TRIP S','HE I 4 SING S','HE I 4 TRIP P','HE I 4 TRIP D',
     .'HE I 4 SING D','HE I 4 TRIP F','HE I 4 SING F','HE I 4 SING P'/
C
      DATA NSTART/
     .  1,  6, 11, 16, 20, 28, 32, 40, 44, 52, 57, 62, 67, 72, 77, 82,
     . 88, 92, 98,104,110,114,120,125,129,135,139,147,151,157,164,170,
     .177,183,190,195,202,208,213,220,225,232,236,243,247,251,260,266,
     .273,278,285,290,300,304,309,316,320,324,329,333,338,343,347,352,
     .357,362,367,372,376,382,386,391,395,401,405,410,414,421,425,431,
     .435,440,445,449,454,459,465,470,475,480,487,491,497,503,508,515,
     .520,525,530,536,542,547,552,559,564,571,576,581,587,592,598,603,
     .608,613,617,623,630,635,642,646,650,655,660,666,671,677,683,689,
     .695,702,707,713,718,723,728,732,737,741,745,750,754,759,765,771,
     .777,782,789,796,801,805,810,815,819,824,831,837,844,850,856,861,
     .868,873,877,882,890,895,905,909,913,920,925,930/
C
      DATA (A(I),I=1,95)/
     . 1.7339D-07, 2.7997D-08,-1.3812D-08, 2.6639D-09, 1.7776D-09,
     . 2.9820D-07, 7.5210D-08,-3.5975D-09, 3.2270D-09, 1.5245D-09,
     . 1.5601D-05, 1.5340D-06,-2.2122D-06,-1.1073D-07, 1.9249D-07,
     . 2.3682D-08, 1.0638D-08, 2.0959D-09, 2.8381D-10, 3.0497D-05,
     . 1.9252D-05, 6.3109D-06, 6.9098D-07,-2.8039D-07,-2.1128D-07,
     .-1.2192D-07,-4.4417D-08, 1.3896D-06, 1.5715D-07,-8.4358D-08,
     .-2.8800D-08, 5.9599D-08, 3.4756D-08, 1.2183D-08, 3.7999D-09,
     . 8.5500D-10,-3.9428D-10,-5.3999D-10,-2.2962D-10, 2.2510D-06,
     . 6.1436D-07,-1.2437D-07,-7.1718D-08, 5.9026D-05, 3.8150D-05,
     . 1.1426D-05, 9.2886D-07,-6.4827D-07,-4.4270D-07,-1.8611D-07,
     .-5.6403D-08, 5.8752D-06, 2.5167D-06, 2.0787D-07,-2.3353D-07,
     .-8.9900D-08, 5.6334D-08, 2.9313D-09, 1.7775D-09, 5.5494D-10,
     .-5.8914D-10, 8.1939D-06, 2.4014D-07, 3.8681D-07, 2.8446D-07,
     .-6.1936D-08, 1.8173D-06,-4.7530D-07,-6.6432D-08, 5.4898D-08,
     .-1.2377D-08, 2.0732D-05, 6.5991D-07, 1.5840D-06, 4.9920D-07,
     .-1.8332D-07, 3.2273D-06,-4.9880D-07,-2.4929D-07, 1.1964D-07,
     .-2.1996D-08, 9.7096D-08, 1.3557D-08, 7.4404D-09, 1.3858D-09,
     .-1.3778D-09,-8.4885D-10, 3.5068D-06,-5.0675D-07,-1.2252D-07,
     . 6.0514D-08, 7.0524D-06, 1.4454D-06, 8.3966D-07, 2.7203D-07/
      DATA (A(I),I=96,190)/
     .-2.3854D-08,-8.6693D-08, 7.1193D-06, 8.3111D-09,-9.3916D-07,
     .-2.7944D-08, 1.7803D-07,-3.9216D-08, 9.2760D-06, 2.4761D-06,
     . 1.0095D-06, 3.4039D-07,-8.6900D-08,-1.1156D-07, 2.5036D-05,
     .-5.7791D-06,-1.8197D-06, 1.0630D-06, 9.8746D-09, 3.1048D-09,
     . 1.2172D-09, 1.8411D-10,-1.5835D-10,-1.4443D-10, 1.9240D-06,
     . 1.5012D-07, 7.9741D-08, 2.9323D-08,-1.2796D-08, 5.0457D-07,
     .-5.5146D-08,-2.7506D-08, 1.4341D-09, 1.3321D-05, 2.6960D-06,
     . 2.8059D-07, 1.7548D-08,-2.3623D-07,-1.4619D-07, 1.5294D-06,
     .-9.4925D-08,-1.1347D-07, 8.1980D-09, 1.3324D-04, 7.8068D-05,
     . 2.9238D-05, 4.2718D-06,-1.6556D-06,-1.4529D-06,-8.6046D-07,
     .-2.1062D-07, 2.8510D-06,-4.7936D-07,-2.4042D-07, 6.2333D-08,
     . 1.3493D-09, 3.5113D-10,-4.7269D-11, 2.4872D-11,-9.7136D-12,
     .-1.4217D-11, 1.5680D-06, 8.9272D-07, 2.8313D-07, 5.2456D-08,
     .-2.3404D-08,-2.7304D-08,-8.4539D-09, 1.7967D-07, 6.2133D-08,
     .-3.2814D-09,-3.7299D-09,-1.9587D-09,-1.6685D-09, 1.2707D-05,
     . 7.5029D-06, 2.8330D-06, 6.7478D-07,-1.5012D-07,-2.3916D-07,
     .-8.9594D-08, 7.6160D-07, 2.0422D-07,-2.5881D-08,-1.7624D-08,
     .-8.3518D-09,-4.1743D-09, 4.6044D-05, 2.2425D-05, 3.3079D-06,
     . 3.6752D-07,-8.4476D-08,-4.8455D-07,-2.5391D-07, 7.0824D-07/
      DATA (A(I),I=191,285)/
     . 1.4917D-07,-1.2005D-07,-1.6983D-08, 1.1545D-08, 7.2866D-04,
     . 3.8907D-04, 9.6365D-05, 2.3153D-05, 1.8462D-07,-9.0627D-06,
     .-5.4332D-06, 1.0458D-08, 1.6430D-09, 5.6453D-10, 2.0276D-10,
     .-1.6501D-10,-1.0656D-10, 5.2198D-07, 4.2898D-08,-1.0332D-08,
     .-5.6754D-09,-7.1960D-09, 3.0390D-06, 1.5385D-06, 6.2110D-07,
     . 1.4111D-07,-3.7415D-08,-4.8395D-08,-1.7219D-08, 2.7227D-06,
     . 1.4381D-07,-5.5030D-08,-2.2192D-10,-2.8583D-08, 1.8417D-05,
     . 9.3860D-06, 3.9999D-06, 1.0424D-06,-2.1337D-07,-3.3271D-07,
     .-1.2684D-07, 4.6063D-06,-6.7185D-07,-2.5830D-07, 2.9976D-08,
     . 7.8012D-05, 3.4921D-05, 8.3012D-06, 1.5643D-06,-4.3892D-07,
     .-9.5318D-07,-4.6534D-07, 1.7584D-05,-2.8817D-06,-1.0081D-06,
     . 1.3259D-07, 1.8561D-05, 2.6602D-06,-1.7243D-06,-3.5533D-07,
     . 2.3315D-08, 1.6239D-08, 7.3739D-09, 1.9570D-09,-2.5667D-10,
     .-5.8101D-10,-2.3947D-10, 4.5821D-12, 4.5024D-11, 3.8796D-07,
     . 1.1239D-07,-2.9033D-08,-5.7237D-09, 2.4186D-09,-2.9670D-09,
     . 1.0887D-06, 4.3737D-07, 8.3753D-08, 3.6771D-08, 1.6545D-09,
     .-1.3849D-08,-5.8855D-09, 2.1028D-06, 4.3725D-07,-1.7621D-07,
     .-3.0743D-08, 1.7119D-08, 8.4698D-06, 4.5395D-06, 1.5774D-06,
     . 4.1647D-07,-7.9865D-08,-1.6003D-07,-6.0171D-08, 3.1692D-06/
      DATA (A(I),I=286,380)/
     . 3.6965D-07,-4.9333D-07,-2.8856D-08, 4.2819D-08, 2.0492D-04,
     . 1.3705D-04, 4.0972D-05, 2.9565D-06,-1.4061D-06,-1.8775D-06,
     .-1.6306D-06,-3.6380D-07, 3.1213D-07, 2.0912D-07, 1.1965D-05,
     . 9.9088D-07,-1.3565D-06,-2.3128D-07, 1.1379D-05, 2.6632D-06,
     .-1.6877D-06,-4.0541D-07, 9.8921D-08, 5.9262D-03, 3.2332D-03,
     . 9.2954D-04, 1.6597D-04,-3.7972D-05,-7.1646D-05,-4.0073D-05,
     . 2.5789D-08, 5.2124D-09,-2.2517D-10,-7.9400D-10, 3.3181D-06,
     . 1.9117D-07, 1.0299D-07,-7.7443D-08, 2.2953D-06,-1.0879D-06,
     . 3.4368D-07,-1.1230D-07, 1.6826D-08, 6.9774D-06, 9.2721D-07,
     .-1.8983D-08,-1.6068D-07, 1.5843D-06,-3.5616D-08,-1.1262D-07,
     .-4.3051D-08, 8.8140D-09, 3.4244D-05, 8.0163D-06, 2.9703D-06,
     .-1.6480D-07,-6.3282D-07, 2.3791D-06,-4.9305D-07,-1.7122D-07,
     . 1.2147D-07, 7.2131D-05, 1.7699D-05, 8.4281D-06,-9.3966D-07,
     .-6.8048D-07, 5.3785D-05, 6.9745D-06,-2.3554D-06,-7.2141D-07,
     . 3.8501D-07, 4.4794D-06,-6.2435D-07,-2.7340D-07, 2.3127D-08,
     . 4.2314D-08, 3.3784D-06,-5.1356D-07,-2.4321D-07,-6.2698D-09,
     . 3.0812D-08, 5.6419D-08, 1.6889D-08, 2.7037D-09,-1.7433D-09,
     .-9.4507D-10, 1.9714D-06, 4.2743D-08,-7.4114D-08,-2.9304D-08,
     . 2.8973D-06, 1.0079D-06, 2.9561D-07,-6.2977D-09,-4.5782D-08/
      DATA (A(I),I=381,475)/
     .-2.4331D-08, 4.3284D-06, 2.8398D-07,-3.2705D-07,-1.5079D-07,
     . 5.2686D-06, 1.6539D-06, 3.6079D-07,-1.1589D-07,-5.4904D-08,
     . 7.0939D-06,-1.9534D-06, 3.2391D-08, 5.5702D-08, 3.9072D-05,
     . 1.7299D-05, 5.1530D-06,-6.0911D-07,-1.2652D-06,-4.6507D-07,
     . 1.1506D-05,-2.0422D-06,-6.1195D-07, 5.8641D-08, 1.0150D-05,
     .-4.6977D-07,-6.9446D-07, 2.2516D-08, 1.1109D-07, 3.6715D-05,
     . 5.6186D-06,-3.2309D-06,-1.6403D-06, 4.1051D-05, 2.3335D-05,
     . 7.8106D-06, 2.0279D-07,-8.1139D-07,-3.7619D-07,-1.4982D-07,
     . 2.5621D-05,-1.0798D-05, 1.4607D-06, 3.2421D-07, 6.5478D-09,
     . 2.7233D-09, 3.8646D-10,-1.9143D-10,-1.0483D-10,-4.8664D-11,
     . 1.0698D-06, 2.7752D-07,-2.6636D-08,-3.7583D-08, 2.6989D-07,
     . 3.0325D-08,-2.4613D-08,-1.0828D-08, 2.2522D-09, 8.0777D-06,
     . 2.2558D-06, 7.4760D-08,-2.4140D-07,-5.4292D-08, 8.8362D-07,
     . 9.5045D-08,-5.0543D-08,-1.9134D-08, 1.1284D-05, 4.0659D-06,
     . 7.6246D-07,-9.0394D-08,-8.7408D-08, 1.2192D-06,-2.0362D-07,
     .-1.0700D-07, 2.1990D-08, 1.2519D-08, 5.2758D-05, 2.0175D-05,
     . 3.6690D-06,-4.9014D-07,-7.0474D-07,-3.9931D-07, 4.7638D-05,
     . 1.5546D-05, 6.2294D-07,-1.3428D-06,-1.8177D-07, 4.1203D-06,
     .-4.9340D-07,-3.0198D-07,-1.5902D-08, 2.8567D-08, 2.2397D-06/
      DATA (A(I),I=476,570)/
     .-2.1306D-08,-1.7897D-07, 2.8178D-08, 4.1722D-08, 2.0594D-04,
     . 1.2838D-04, 5.8219D-05, 1.1735D-05,-6.9778D-06,-6.2486D-06,
     .-1.3800D-06, 2.9170D-06,-7.0833D-07,-1.2673D-07, 4.5069D-08,
     . 1.0974D-09, 4.7908D-10, 3.0294D-11,-7.0815D-11,-1.2039D-11,
     . 5.6357D-12, 8.1274D-07, 4.5158D-07, 1.1655D-07,-2.1481D-08,
     .-2.2493D-08,-6.5884D-09, 9.8696D-08, 3.6499D-08, 1.4768D-09,
     .-8.0141D-09,-2.8147D-09, 5.8287D-06, 3.2469D-06, 9.4927D-07,
     .-7.2550D-08,-1.4231D-07,-5.8408D-08,-1.4468D-08, 4.6070D-07,
     . 1.5956D-07,-3.2311D-09,-3.3487D-08,-7.9013D-09, 4.0092D-06,
     . 1.2195D-06, 1.1192D-07,-1.3870D-07,-6.2194D-08, 5.4918D-07,
     .-5.1133D-08,-5.4844D-08,-3.1054D-09, 6.1049D-09, 2.4833D-05,
     . 1.1280D-05, 2.2680D-06,-4.4637D-07,-2.8942D-07,-1.2382D-07,
     . 6.8223D-05, 3.6505D-05, 7.8792D-06,-1.7946D-06,-1.4925D-06,
     .-4.6018D-07, 2.5677D-06,-7.2440D-08,-2.2441D-07,-4.1769D-08,
     . 2.3833D-08, 1.1885D-06, 4.0457D-09,-1.2273D-07,-2.3271D-08,
     . 1.5149D-08, 9.1912D-05, 5.6621D-05, 1.4338D-05,-2.6072D-06,
     .-2.3688D-06,-6.2490D-07,-1.4386D-07, 1.0821D-06,-1.6304D-07,
     .-7.9756D-08,-4.9881D-09, 9.9527D-09, 1.5288D-03, 9.8517D-04,
     . 3.4966D-04, 3.4238D-05,-4.3610D-05,-3.2734D-05,-8.3676D-06/
      DATA (A(I),I=571,665)/
     . 9.6630D-09, 3.2976D-09, 2.1545D-10,-3.3938D-10,-1.4397D-10,
     . 3.4140D-07, 7.6536D-08,-2.2485D-08,-1.8244D-08,-2.0611D-09,
     . 1.3128D-06, 6.4165D-07, 1.4728D-07,-2.7600D-08,-3.0125D-08,
     .-1.0320D-08, 1.6295D-06, 3.3697D-07,-5.7547D-08,-7.3134D-08,
     .-1.5301D-08, 8.0375D-06, 4.0695D-06, 1.1441D-06,-5.8423D-08,
     .-1.6488D-07,-7.5920D-08, 2.1173D-06,-1.9184D-07,-2.4986D-07,
     . 7.2656D-09, 2.7560D-08, 7.9040D-06, 3.3963D-06, 4.8082D-07,
     .-3.1619D-07,-1.6501D-07, 5.8017D-06,-5.5173D-07,-6.3613D-07,
     . 1.9633D-08, 7.8681D-08, 9.4568D-06, 9.8227D-08,-7.8983D-07,
     .-2.0343D-07, 7.0351D-05, 3.6017D-05, 8.0504D-06,-1.7276D-06,
     .-1.5329D-06,-4.7799D-07, 3.5263D-05, 1.8639D-05, 4.2070D-06,
     .-4.2643D-07,-5.2726D-07,-2.5481D-07,-1.0144D-07, 4.4613D-06,
     .-8.7802D-07,-3.9633D-07, 6.7953D-08, 4.1539D-08, 1.8780D-04,
     . 1.0547D-04, 2.0983D-05,-3.7076D-06,-3.7090D-06,-1.5934D-06,
     .-2.6146D-07, 1.6384D-05,-3.3765D-06,-7.8390D-07, 1.2382D-07,
     . 1.5748D-05,-9.4352D-07,-4.8936D-07,-4.9967D-07, 3.8177D-10,
     . 9.6781D-11,-1.9622D-11,-1.7859D-11,-4.4781D-12, 3.0310D-07,
     . 1.1193D-07,-5.4059D-09,-1.4249D-08,-1.9549D-09, 4.7664D-08,
     . 8.5684D-09,-5.0948D-09,-3.1313D-09, 4.9557D-10, 4.2702D-10/
      DATA (A(I),I=666,760)/
     . 2.3433D-06, 8.3824D-07, 2.6585D-08,-7.5944D-08,-1.5093D-08,
     . 1.8775D-07, 2.7614D-08,-2.1193D-08,-1.0264D-08, 1.9660D-09,
     . 1.1637D-09, 7.7890D-06, 3.2901D-06,-2.9753D-07,-5.3153D-07,
     .-5.2098D-08, 3.3947D-08, 5.5937D-07, 5.1363D-08,-8.5390D-08,
     .-2.2580D-08, 1.0857D-08, 3.5157D-09, 4.1303D-05, 2.1947D-05,
     . 3.9480D-06,-1.4234D-06,-9.0220D-07,-2.4485D-07, 1.1031D-04,
     . 6.6726D-05, 1.9581D-05,-4.3637D-07,-2.8911D-06,-1.4332D-06,
     .-3.2332D-07, 3.0922D-06, 2.0624D-07,-4.0771D-07,-1.0402D-07,
     . 4.8807D-08, 1.2491D-06, 2.1187D-07,-1.5951D-07,-7.2537D-08,
     . 1.4035D-08, 9.6514D-09, 2.5146D-05,-9.4490D-08,-3.4103D-06,
     . 2.4020D-07, 2.9564D-07, 6.6566D-07,-3.0772D-08,-1.0055D-07,
     .-3.1082D-09, 1.4897D-08, 7.7189D-04, 3.3710D-04, 3.9258D-05,
     .-1.5271D-05,-6.0652D-06, 2.1986D-04,-3.3135D-05,-8.5682D-06,
     .-9.2988D-07, 3.8085D-06,-1.3259D-07,-3.8517D-07,-5.7276D-08,
     . 3.7226D-08, 2.0825D-09, 1.6707D-10,-1.2443D-10,-3.9117D-11,
     . 1.0068D-07, 6.2278D-09,-5.9169D-09,-3.9535D-09, 5.3334D-07,
     . 2.2022D-07, 1.4522D-08,-2.8440D-08,-8.1406D-09, 6.0187D-07,
     . 5.8318D-08,-2.7772D-08,-2.6639D-08, 3.0929D-06, 1.0648D-06,
     . 1.2317D-07,-9.9233D-08,-4.0796D-08, 1.5217D-06, 8.3095D-08/
      DATA (A(I),I=761,855)/
     .-1.9819D-07,-6.0417D-08, 2.9553D-08, 9.0905D-09, 8.6651D-06,
     . 3.9125D-06,-2.2871D-07,-6.4651D-07,-7.4440D-08, 4.3543D-08,
     . 4.4505D-06, 2.7350D-07,-4.6428D-07,-2.2293D-07, 5.5275D-08,
     . 3.6505D-08, 8.3471D-06, 5.0215D-07,-1.1189D-06,-2.5404D-07,
     . 1.3175D-07, 1.1687D-04, 7.0674D-05, 2.0219D-05,-1.2136D-06,
     .-3.2299D-06,-1.3981D-06,-2.8271D-07, 3.7298D-05, 2.2054D-05,
     . 5.5510D-06,-9.1515D-07,-1.0832D-06,-3.6657D-07,-4.8581D-08,
     . 2.2506D-06,-2.9469D-07,-2.1641D-07,-3.9630D-08, 5.2879D-08,
     . 2.7894D-05,-3.9312D-06,-2.6413D-06, 5.5848D-07, 8.7829D-06,
     .-1.4800D-06,-7.0982D-07,-2.1645D-08, 1.3258D-07, 1.2248D-05,
     .-1.4244D-06,-7.6696D-07,-1.5029D-07, 6.8467D-08, 1.9392D-04,
     .-2.6676D-05,-8.0536D-06,-6.3609D-07, 2.3074D-05, 7.2856D-07,
     .-2.3751D-06,-5.8464D-07, 1.3685D-07, 1.9646D-08, 1.2918D-08,
     . 4.6924D-09, 5.1593D-10,-4.4715D-10,-3.4091D-10,-9.7254D-11,
     . 3.4117D-07, 1.2731D-07,-2.2300D-08,-2.5297D-08,-1.1877D-10,
     . 2.3436D-09, 9.4463D-07, 4.9464D-07, 8.9138D-08,-2.2158D-08,
     .-1.2008D-08,-4.3715D-09,-2.8719D-09, 1.4091D-06, 4.8184D-07,
     .-9.6135D-08,-1.0945D-07,-9.3174D-09, 9.0166D-09, 4.3747D-06,
     . 2.1053D-06, 1.7702D-07,-3.0956D-07,-1.2902D-07,-8.9418D-09/
      DATA (A(I),I=856,929)/
     . 1.4297D-06, 7.7612D-08,-1.7188D-07, 6.4348D-10, 1.1572D-08,
     . 7.4828D-06, 4.0177D-06, 1.0890D-06, 8.6393D-08,-5.4701D-08,
     .-5.7656D-08,-3.3030D-08, 5.0379D-06, 1.2861D-07,-7.1313D-07,
     .-3.3703D-09, 7.9367D-08, 6.2235D-06, 9.8196D-07,-4.6416D-07,
     .-2.3278D-07, 2.9519D-05, 1.5275D-05, 3.1448D-06,-6.4571D-07,
     .-3.4445D-07, 3.6508D-05, 2.0524D-05, 3.1203D-06,-2.3606D-06,
     .-1.5012D-06,-2.5807D-07, 1.3807D-07, 9.9656D-08, 3.1628D-06,
     .-5.6361D-08,-3.8861D-07,-1.0385D-09, 2.9930D-08, 3.1351D-04,
     . 2.3774D-04, 1.0342D-04, 1.2429D-05,-1.4107D-05,-9.2597D-06,
     .-1.7338D-06, 8.4903D-07, 7.3795D-07, 2.2065D-07, 1.2134D-05,
     . 4.6651D-07,-9.3659D-07,-2.7485D-07, 1.2462D-05, 6.2559D-07,
     .-6.1634D-07,-5.1080D-07, 1.2493D-02, 8.2057D-03, 2.9562D-03,
     . 3.3090D-04,-3.7349D-04,-2.8886D-04,-7.4606D-05, 1.2726D-05,
     .-1.6480D-07,-1.5006D-06,-1.1181D-07, 1.4846D-07, 8.7160D-03,
     . 4.2652D-03, 5.2455D-04,-2.6363D-04,-7.9836D-05/
      SAVE ENER,NSTART,A,STWT
C
C          SET ALL ELEMENTS OF COLHE1 = 0
C
           DO 10 I=1,N
              DO 10 J=1,N
   10            COLHE1(I,J)=0.
C
C        EVALUATE GAMMAS FOR REQUESTED LEVELS
C
      XXX=2.D0*(LOG10(TEMP)-C1)/C2
      TFAC=UN/SQRT(TEMP)
C
C          LOOPS OVER LEVELS
C
      DO 30 IL=1,N-1
           DO 30 IU=IL+1,N
                J=((IU*IU-3*IU+4)/2)+IL-1
                N1=NSTART(J)
                NF=NSTART(J+1)-1
                NT=NF-N1+1
                NTM2=NT-2
C
C          CLENSHAW SUMMATION
C
                B(NT)=A(NF)
                B(NT-1)=XXX*B(NT)+A(NF-1)
                IR=NTM2
                JJ=NF-2
                DO 20 J=1,NTM2
                     B(IR)=XXX*B(IR+1)-B(IR+2)+A(JJ)
                     IR=IR-1
                     JJ=JJ-1
   20           CONTINUE
                COLHE1(IU,IL)=(B(1)-B(3))*TFAC
                X=(ENER(IL)-ENER(IU))/8.62D-5/TEMP
                COLHE1(IL,IU)=COLHE1(IU,IL)*STWT(IU)/STWT(IL)*EXP(X)
   30  CONTINUE
       RETURN
       END
C
C
C     ****************************************************************
C
C
      FUNCTION CHEAV(II,JJ,IC)
C     ========================
C
C     Calculates collisional excitation rates of neutral helium
C     between states with n= 1, 2, 3, 4; with either the upper state
C     alone, or both upper and lower states are some averaged states
C     The program allows only two standard possibilities of
C     constructing averaged levels:
C     i)  all states within given principal quantum number n (>1) are
C         lumped together
C     ii) all siglet states for given n, and all triplet states for
C         given n are lumped together separately (there are thus two
C         explicit levels for a given n)
C
C     The rates are calculated using appropriate summations and/or
C     averages of the Storey-Hummer rates (calculated by procedure
C     COLLHE and stored in array COLHE1)
C
C     Input parameters:
C      II,JJ - indices of the lower and the upper level (in the
C              numbering of the explicit levels)
C      IC    - collisional switch ICOL for the given transition
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      COMMON/HECRAT/COLHE1(19,19)
C
      NI=NQUANT(II)
      NJ=NQUANT(JJ)
      IGI=INT(G(II)+0.01)
      IGJ=INT(G(JJ)+0.01)
C
C     ----------------------------------------------------------------
C     IC=2 - transition from an (l,s) lower level to an averaged upper
C            level
C     ----------------------------------------------------------------
C
      IF(IC.EQ.2) THEN
         I=II-NFIRST(IELHE1)+1
         CHEAV=CHEAVJ(I,NJ,IGJ)
      END IF
C
C     ----------------------------------------------------------------
C     IC=3 - transition from an averaged lower level to an averaged
C            upper level
C     ----------------------------------------------------------------
C
      IF(IC.EQ.3) THEN
         IF(NI.EQ.2) THEN
C
C ********    transitions from an averaged level with n=2
C
            IF(IGI.EQ.4) THEN
C
C      a) lower level is an averaged singlet state
C
               CHEAV=(CHEAVJ(3,NJ,IGJ)+3.D0*CHEAVJ(5,NJ,IGJ))/4.D0
            ELSE IF(IGI.EQ.12) THEN
C
C      b) lower level is an averaged triplet state
C
               CHEAV=(CHEAVJ(2,NJ,IGJ)+3.D0*CHEAVJ(4,NJ,IGJ))/4.D0
            ELSE IF(IGI.EQ.16) THEN
C
C      c) lower level is an average of both singlet and triplet states
C
               CHEAV=(CHEAVJ(3,NJ,IGJ)+3.D0*(CHEAVJ(5,NJ,IGJ)+
     *                CHEAVJ(2,NJ,IGJ))+9.D0*CHEAVJ(4,NJ,IGJ))/1.6D1
            ELSE
               GO TO 10
            END IF
C
C
C ********    transitions from an averaged level with n=3
C
         ELSE IF(NI.EQ.3) THEN
            IF(IGI.EQ.9) THEN
C
C      a) lower level is an averaged singlet state
C
               CHEAV=(CHEAVJ(7,NJ,IGJ)+3.D0*CHEAVJ(11,NJ,IGJ)+
     *                5.D0*CHEAVJ(10,NJ,IGJ))/9.D0
            ELSE IF(IGI.EQ.27) THEN
C
C      b) lower level is an averaged triplet state
C
               CHEAV=(CHEAVJ(6,NJ,IGJ)+3.D0*CHEAVJ(8,NJ,IGJ)+
     *                5.D0*CHEAVJ(9,NJ,IGJ))/9.D0
            ELSE IF(IGI.EQ.36) THEN
C
C      c) lower level is an average of both singlet and triplet states
C
               CHEAV=(CHEAVJ(7,NJ,IGJ)+3.D0*CHEAVJ(11,NJ,IGJ)+
     *                5.D0*CHEAVJ(10,NJ,IGJ)+
     *                3.D0*CHEAVJ(6,NJ,IGJ)+9.D0*CHEAVJ(8,NJ,IGJ)+
     *                1.5D1*CHEAVJ(9,NJ,IGJ))/3.6D1
            ELSE
               GO TO 10
            END IF
C
C ********    transitions from an averaged level with n=4
C
         ELSE IF(NI.EQ.4) THEN
            IF(IGI.EQ.16) THEN
C
C      a) lower level is an averaged singlet state
C
               CHEAV=(CHEAVJ(13,NJ,IGJ)+
     *                3.D0*CHEAVJ(19,NJ,IGJ)+
     *                5.D0*CHEAVJ(16,NJ,IGJ)+
     *                7.D0*CHEAVJ(18,NJ,IGJ))/1.6D1
            ELSE IF(IGI.EQ.48) THEN
C
C      b) lower level is an averaged triplet state
C
               CHEAV=(CHEAVJ(12,NJ,IGJ)+
     *                3.D0*CHEAVJ(14,NJ,IGJ)+
     *                5.D0*CHEAVJ(15,NJ,IGJ)+
     *                7.D0*CHEAVJ(17,NJ,IGJ))/1.6D1
            ELSE IF(IGI.EQ.64) THEN
C
C      c) lower level is an average of both singlet and triplet states
C
               CHEAV=(CHEAVJ(13,NJ,IGJ)+
     *                3.D0*CHEAVJ(19,NJ,IGJ)+
     *                5.D0*CHEAVJ(16,NJ,IGJ)+
     *                7.D0*CHEAVJ(18,NJ,IGJ)+
     *                3.D0*CHEAVJ(12,NJ,IGJ)+
     *                9.D0*CHEAVJ(14,NJ,IGJ)+
     *                15.D0*CHEAVJ(15,NJ,IGJ)+
     *                21.D0*CHEAVJ(17,NJ,IGJ))/6.4D1
            ELSE
               GO TO 10
            END IF
         ELSE
            GO TO 10
         END IF
      END IF
      RETURN
   10 WRITE(6,601) NI,NJ,IGI,IGJ
      WRITE(10,601) NI,NJ,IGI,IGJ
  601 FORMAT(1H0/' INCONSISTENT INPUT TO PROCEDURE CHEAV'/
     * ' QUANTUM NUMBERS =',2I3,'    STATISTICAL WEIGHTS',2I4)
      STOP
      END
C
C
C     ****************************************************************
C
C

      FUNCTION CHEAVJ(I,NJ,IGJ)
C     =========================
C
C     Calculates collisional excitation rates from a non-averaged (l,s)
C     state of He I, with n=1, 2, 3,  to some averaged state
C     with n = 2, 3, 4.
C
C     The rates are calculated using appropriate summations of the
C     Storey-Hummer rates (calculated by procedure COLLHE, and stored
C     in array COLHE1)
C
C     Input:
C     I   -  index of the lower state, using the ordering defined in
C            COLLHE, ie. I=1 for 1 sing S, I=2 for 2 trip S, etc.
C     NJ  - principal quantum number of the (averaged) upper level
C     IGJ - statistical weight of the upper level
C
      INCLUDE 'IMPLIC.FOR'
      COMMON/HECRAT/COLHE1(19,19)
C
C -----------------------------------------------------
C ********    transitions to an averaged level with n=2
C -----------------------------------------------------
C
      IF(NJ.EQ.2) THEN
         IF(IGJ.EQ.4) THEN
C
C      a) upper level is an averaged singlet state
C
            CHEAVJ=COLHE1(1,3)+COLHE1(1,5)
         ELSE IF(IGJ.EQ.12) THEN
C
C      b) upper level is an averaged triplet state
C
            CHEAVJ=COLHE1(1,2)+COLHE1(1,4)
         ELSE IF(IGJ.EQ.16) THEN
C
C      c) upper level is an average of both siglet and triplet states
C
            CHEAVJ=COLHE1(1,3)+COLHE1(1,5)+COLHE1(1,2)+COLHE1(1,4)
         ELSE
            GO TO 10
         END IF
C
C -----------------------------------------------------
C ********    transitions to an averaged level with n=3
C -----------------------------------------------------
C
       ELSE IF(NJ.EQ.3) THEN
         IF(IGJ.EQ.9) THEN
C
C      a) upper level is an averaged singlet state
C
            CHEAVJ=COLHE1(I,7)+COLHE1(I,11)+COLHE1(I,10)
         ELSE IF(IGJ.EQ.27) THEN
C
C      b) upper level is an averaged triplet state
C
            CHEAVJ=COLHE1(I,6)+COLHE1(I,8)+COLHE1(I,9)
         ELSE IF(IGJ.EQ.36) THEN
C
C      c) upper level is an average of both siglet and triplet states
C
            CHEAVJ=COLHE1(I,7)+COLHE1(I,11)+COLHE1(I,10)+
     *             COLHE1(I,6)+COLHE1(I,8)+COLHE1(I,9)
         ELSE
            GO TO 10
         END IF
C
C -----------------------------------------------------
C ********    transitions to an averaged level with n=4
C -----------------------------------------------------
C
       ELSE IF(NJ.EQ.4) THEN
         IF(IGJ.EQ.16) THEN
C
C      a) upper level is an averaged singlet state
C
            CHEAVJ=COLHE1(I,13)+COLHE1(I,19)+COLHE1(I,16)+
     *             COLHE1(I,18)
         ELSE IF(IGJ.EQ.48) THEN
C
C      b) upper level is an averaged triplet state
C
            CHEAVJ=COLHE1(I,12)+COLHE1(I,14)+COLHE1(I,15)+
     *             COLHE1(I,17)
         ELSE IF(IGJ.EQ.64) THEN
C
C      c) upper level is an average of both siglet and triplet states
C
            CHEAVJ=COLHE1(I,13)+COLHE1(I,19)+COLHE1(I,16)+
     *             COLHE1(I,18)+COLHE1(I,12)+COLHE1(I,14)+
     *             COLHE1(I,15)+COLHE1(I,17)
         ELSE
            GO TO 10
         END IF
       ELSE
         GO TO 10
      END IF
      RETURN
   10 WRITE(6,601) NJ,IGJ
      WRITE(10,601) NJ,IGJ
  601 FORMAT(1H0/' INCONSISTENT INPUT TO PROCEDURE CHEAVJ'/
     * ' QUANTUM NUMBER =',I3,'    STATISTICAL WEIGHT',2I4)
      STOP
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE MATINV(A,N,NR)
C     =========================
C
C     Matrix inversion
C      by LU decomposition
C
C      A  -  matrix of actual size (N x N) and maximum size (NR x NR)
C            to be inverted;
C      Inversion is accomplished in place and the original matrix is
C      replaced by its inverse
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      DIMENSION A(NR,NR)
C
C  --------- Cray version ------------------
C
c      parameter(mt2=2*mtot)
c      dimension scr(mt2)
c      call minv(a,n,nr,scr,det,1e-135,0,1)
c      return
c      end
c  _________________________________________
C
C  For Cray - all what is needed is to comment out the rest of subroutine
C  Note:
C  Obviously, the normal version works for Cray as well, but MINV is a
C  routine devised specifically for Cray and is therefore optimalized.
C  Numerical experience shows that the execution time when using MINV
C  is about 75 - 80 % of the time using ordinary MATINV.
C
C     Normal version follows:
C
      DO 50 I=2,N
         IM1=I-1
         DO 20 J=1,IM1
            JM1=J-1
            DIV=A(J,J)
            SUM=0.
            IF(JM1.LT.1) GO TO 20
            DO 10 K=1,JM1
   10          SUM=SUM+A(I,K)*A(K,J)
   20       A(I,J)=(A(I,J)-SUM)/DIV
         DO 40 J=I,N
            SUM=0.
            DO 30 K=1,IM1
   30          SUM=SUM+A(I,K)*A(K,J)
   40       A(I,J)=A(I,J)-SUM
   50 CONTINUE
      DO 80 II=2,N
         I=N+2-II
         IM1=I-1
         IF(IM1.LT.1) GO TO 80
         DO 70 JJ=1,IM1
            J=I-JJ
            JP1=J+1
            SUM=0.
            IF(JP1.GT.IM1) GO TO 70
        DO 60 K=JP1,IM1
   60          SUM=SUM+A(I,K)*A(K,J)
   70    A(I,J)=-A(I,J)-SUM
   80 CONTINUE
      DO 110 II=1,N
         I=N+1-II
         DIV=A(I,I)
         IP1=I+1
         IF(IP1.GT.N) GO TO 110
         DO 100 JJ=IP1,N
            J=N+IP1-JJ
            SUM=0.
            DO 90 K=IP1,J
   90          SUM=SUM+A(I,K)*A(K,J)
            A(I,J)=-SUM/DIV
  100    CONTINUE
  110 A(I,I)=1.0D0/A(I,I)
C 
      DO 240 I=1,N
         DO 230 J=1,N
            K0=I
            IF(J.GE.I) GO TO 220
            SUM=0.
  200       DO 210 K=K0,N
  210          SUM=SUM+A(I,K)*A(K,J)
            GO TO 230
  220       K0=J
            SUM=A(I,K0)
            IF(K0.EQ.N) GO TO 230
            K0=K0+1
            GO TO 200
  230    A(I,J)=SUM
  240 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE MINV3(A)
C     ===================
C
C     Special routine for an invresion of a 3 x 3 matrix
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (UN=1.D0)
      DIMENSION A(3,3)
C
      A(2,1)=A(2,1)/A(1,1)
      A(2,2)=A(2,2)-A(2,1)*A(1,2)
      A(2,3)=A(2,3)-A(2,1)*A(1,3)
      A(3,1)=A(3,1)/A(1,1)
      A(3,2)=(A(3,2)-A(3,1)*A(1,2))/A(2,2)
      A(3,3)=A(3,3)-A(3,1)*A(1,3)-A(3,2)*A(2,3)
C
      A(3,2)=-A(3,2)
      A(3,1)=-A(3,1)-A(3,2)*A(2,1)
      A(2,1)=-A(2,1)
C
      A(3,3)=UN/A(3,3)
      A(2,3)=-A(2,3)*A(3,3)/A(2,2)
      A(2,2)=UN/A(2,2)
      A(1,3)=-(A(1,2)*A(2,3)+A(1,3)*A(3,3))/A(1,1)
      A(1,2)=-A(1,2)*A(2,2)/A(1,1)
      A(1,1)=UN/A(1,1)
C
      A(1,1)=A(1,1)+A(1,2)*A(2,1)+A(1,3)*A(3,1)
      A(1,2)=A(1,2)+A(1,3)*A(3,2)
      A(2,1)=A(2,2)*A(2,1)+A(2,3)*A(3,1)
      A(2,2)=A(2,2)+A(2,3)*A(3,2)
      A(3,1)=A(3,3)*A(3,1)
      A(3,2)=A(3,3)*A(3,2)
C
      RETURN
      END
C
C
C     ******************************************************************
C
C

      SUBROUTINE LINEQS(A,B,X,N,NR)
C     =============================
C
C     Solution of the linear system A*X=B
C     by Gaussian elimination with partial pivoting
C
C     Input: A  - matrix of the linear system, with actual size (N x N),
C                and maximum size (NR x NR)
C            B  - the rhs vector
C     Output: X - solution vector
C     Note that matrix A and vector B are destroyed here
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      DIMENSION A(NR,NR),B(NR),X(NR),D(MLEVEL),IP(MLEVEL)
      DO 70 I=1,N
         DO 10 J=1,N
   10       D(J)=A(J,I)
         IM1=I-1
         IF(IM1.LT.1) GO TO 40
         DO 30 J=1,IM1
            IT=IP(J)
            A(J,I)=D(IT)
            D(IT)=D(J)
            JP1=J+1
            DO 20 K=JP1,N
   20          D(K)=D(K)-A(K,J)*A(J,I)
   30    CONTINUE
   40    AM=ABS(D(I))
         IP(I)=I
         DO 50 K=I,N
            IF(AM.GE.ABS(D(K))) GO TO 50
            IP(I)=K
            AM=ABS(D(K))
   50    CONTINUE
         IT=IP(I)
         A(I,I)=D(IT)
         D(IT)=D(I)
         IP1=I+1
         IF(IP1.GT.N) GO TO 80
         DO 60 K=IP1,N
   60       A(K,I)=D(K)/A(I,I)
   70 CONTINUE
   80 DO 100 I=1,N
         IT=IP(I)
         X(I)=B(IT)
         B(IT)=B(I)
         IP1=I+1
         IF(IP1.GT.N) GO TO 110
         DO 90 J=IP1,N
   90       B(J)=B(J)-A(J,I)*X(I)
  100 CONTINUE
  110 DO 140 I=1,N
         K=N-I+1
         SUM=0.
         KP1=K+1
         IF(KP1.GT.N) GO TO 130
         DO 120 J=KP1,N
  120       SUM=SUM+A(K,J)*X(J)
  130    X(K)=(X(K)-SUM)/A(K,K)
  140 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      FUNCTION EXPINT(X)
C     ==================
C
C     First exponential integral function E1(X)
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (A1  =  -0.57721566,
     *           A2  =   0.99999193,
     *           A3  =  -0.24991055,
     *           A4  =   0.05519968,
     *           A5  =  -0.00976004,
     *           A6  =   0.00107857,
     *           B1  =   0.2677734343,
     *           B2  =   8.6347608925,
     *           B3  =  18.059016973,
     *           B4  =   8.5733287401,
     *           C1  =   3.9584969228,
     *           C2  =  21.0996530827,
     *           C3  =  25.6329561486,
     *           C4  =   9.5733223454,
     *           UN  =   1.0)
C
      IF(X.LE.UN) THEN
         EXPINT=-LOG(X)+A1+X*(A2+X*(A3+X*(A4+X*(A5+X*A6))))
       ELSE
         EXPINT=EXP(-X)*((B1+X*(B2+X*(B3+X*(B4+X))))/
     *                   (C1+X*(C2+X*(C3+X*(C4+X)))))/X
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE INTERP(X,Y,XX,YY,NX,NXX,NPOL,ILOGX,ILOGY)
C     ====================================================
C
C     General interpolation procedure of the (NPOL-1)-th order
C
C     for  ILOGX = 1  logarithmic interpolation in X
C     for  ILOGY = 1  logarithmic interpolation in Y
C
C     Input:
C      X    - array of original x-coordinates
C      Y    - array of corresponding functional values Y=y(X)
C      NX   - number of elements in arrays X or Y
C      XX   - array of new x-coordinates (to which is to be
C             interpolated
C      NXX  - number of elements in array XX
C     Output:
C      YY   - interpolated functional values YY=y(XX)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      DIMENSION X(MDEPTH),Y(MDEPTH),XX(MDEPTH),YY(MDEPTH)
      IF(NPOL.LE.0.OR.NX.LE.0) GO TO 200
      IF(ILOGX.EQ.0) GO TO 30
      DO 10 I=1,NX
   10    X(I)=LOG10(X(I))
      DO 20 I=1,NXX
   20    XX(I)=LOG10(XX(I))
   30 IF(ILOGY.EQ.0) GO TO 50
      DO 40 I=1,NX
   40    Y(I)=LOG10(Y(I))
   50 NM=(NPOL+1)/2
      NM1=NM+1
      NUP=NX+NM1-NPOL
      DO 100 ID=1,NXX
         XXX=XX(ID)
         DO 60 I=NM1,NUP
            IF(XXX.LE.X(I)) GO TO 70
   60    CONTINUE
         I=NUP
   70    J=I-NM
         JJ=J+NPOL-1
         YYY=0.
         DO 90 K=J,JJ
            T=1.
            DO 80 M=J,JJ
               IF(K.EQ.M) GO TO 80
               T=T*(XXX-X(M))/(X(K)-X(M))
   80       CONTINUE
   90    YYY=Y(K)*T+YYY
         YY(ID)=YYY
  100 CONTINUE
      IF(ILOGX.EQ.0) GO TO 130
      DO 110 I=1,NX
  110    X(I)=EXP(X(I)*2.30258509299405)
      DO 120 I=1,NXX
  120    XX(I)=EXP(XX(I)*2.30258509299405)
  130 IF(ILOGY.EQ.0) RETURN
      DO 140 I=1,NX
  140    Y(I)=EXP(Y(I)*2.30258509299405)
      DO 150 I=1,NXX
  150    YY(I)=EXP(YY(I)*2.30258509299405)
      RETURN
  200    N=NX
         IF(NXX.GE.NX) N=NXX
      DO 210 I=1,N
         XX(I)=X(I)
  210    YY(I)=Y(I)
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE OUTPUT
C     =================
C
C     Output of computed model atmosphere on file 7
C     This file may be used as input file 8 (initial model atmosphere)
C     for a subsequent run of the program
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/PRINTS/IPRINT,IPRING,IPRIND

C
      NUMPAR=NLEVEL+3
      IF(LTE) NUMPAR=3
C
C     NUMPAR  - number of model parameters in each depth
C             = 3  for LTE model, ie. TEMP - temperature
C                                     ELEC - electron density
C                                     DENS - density
C             = 3+NLEVEL  for NLTE model, ie. the above + populations
C ---------------------------------------------------------------------
C      2. DM(ID),ID=1,ND - mass-depth points for the input model
C ---------------------------------------------------------------------
C      3. for each depth:
C                 T   - temperature
C                 ANE - electron density
C                 RHO - mass density
C                 level populations - only for NLTE input model
C
      REWIND 7
      WRITE(7,501) ND,NUMPAR
      WRITE(7,502) (DM(I),I=1,ND)
      DO 10 I=1,ND
         IF(LTE) THEN
            WRITE(7,503) TEMP(I),ELEC(I),DENS(I)
          ELSE
            WRITE(7,503) TEMP(I),ELEC(I),DENS(I),(POPUL(J,I),J=1,NLEVEL)
         END IF
   10 CONTINUE
      IF(IPRIND.GT.0) THEN
        WRITE(17,501) ND,NUMPAR
        WRITE(17,502) (DM(I),I=1,ND)
        DO 20 I=1,ND
          IF(LTE) THEN
           WRITE(17,503) TEMP(I),ELEC(I),DENS(I)
          ELSE
           WRITE(17,503) TEMP(I),ELEC(I),DENS(I),(POPUL(J,I),J=1,NLEVEL)
          END IF
   20   CONTINUE
      ENDIF
  501 FORMAT(2I5)
  502 FORMAT(1P8D10.3)
  503 FORMAT(1P5D15.6)
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE OUTPRI(KRAD,KMOD,KPOP)
C     =================================
C
C     Output on unit 6 (printer)
C
C     Input parameters:
C      KRAD  -  whether to print emergent radiation (1=yes; 0=no)
C      KMOD  -  whether to print model parameters (1=yes; 0=no)
C      KPOP  -  whether to print populations (1=yes; 0=no)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      PARAMETER (MPAG=MLEVEL/6+1) 
      DIMENSION IELEM(6),II0(6),PPR(6),BF(6)
      DIMENSION USUSTK(MDEPTH,MION),POPTOT(MDEPTH,MION)
      DIMENSION POPA(MATOM),RPEH(MATOM)
      CHARACTER*10 TYPLEV(MLEVEL),TP(6)
      CHARACTER*4 TYPION(MION),IOT(6)
      COMMON/PRINTP/NPGPOP,IIPR(6,MPAG),TYPLEV,TYPION
      COMMON/OPMEAN/ABROSD(MDEPTH),ABPLAD(MDEPTH)
      COMMON/TAURSS/TROSS(MDEPTH)
C
C ************    Print emergent radiation field, namely
C
C      IJ       - frequency index
C      FREQ(IJ) - value of frequency
C      ALAM     - wavelength (in Angstroms)
C      FLUX(IJ) - emergent flux, precisely the second moment H(freq)
C                 at the surface, in ergs/cm**2/s/sterad/Hz
C      FH(IJ)   - Eddington factor f(H), ie the ratio H/J, J is the
C                 mean intensity of radiation
C      FLL      - log H(freq)
C      FLAM     - log H(lambda); H(lambda) in ergs/cm**2/s/ster/angstrom
C
      WRITE(6,600) ITER-1
      IF(KRAD.EQ.0) GO TO 50
      GL=LOG10(GRAV)
      WRITE(6,601)
      TOTF=0.
      DO 20 IJ=1,NFREQ
         IJP=IJ
         IF(IOVER.GT.0) IJP=JIK(IJ)
         IF(IJX(IJP).EQ.-1) GOTO 20
         ALAM=2.997925D18/FREQ(IJP)
         IF(FLUX(IJP).GT.0.) FLL=LOG10(FLUX(IJP))
         FLAM=FLL+2.*LOG10(FREQ(IJP))-18.4768208D0
         TOTF=TOTF+FLUX(IJP)*W(IJP)
         WRITE(6,602) IJP,FREQ(IJP),ALAM,FLUX(IJP),FH(IJP),FLL,FLAM
         WRITE(13,602) IJP,FREQ(IJP),ALAM,FLUX(IJP),FH(IJP),FLL,FLAM
   20 CONTINUE
      WRITE(6,603) TOTF
C
C ************    Print basic model parameters, namely
C
C      ID       - depth index
C      DM(ID)   - mass-depth variable (in g/cm**2)
C      TEMP(ID) - temperature (in K)
C      ELEC(ID) - electron density (cm**-3)
C      AN       - total particle number density (cm**-3)
C      DENS(ID) - mass density (g/cm**3)
C      P        - total gas pressure (cgs)
C      GR       - radiative acceleration (cgs)
C      FLTOT(ID)- total (integrated over frequencies) radiative flux
C
   50 IF(KMOD.EQ.0) GO TO 100
      WRITE(6,611)
      DO 70 ID=1,ND
C
C     contributions from explicit (linearized) frequencies to the
C     flux and radiation pressure
C
         GRD=0.
         FLEX=0.
         IF(NFREQE.GT.0) THEN
         DO 60 IJ=1,NFREQE
            RAD0(IJ)=RADEX(IJ,ID)
            FK0(IJ)=FAKEX(IJ,ID)
            ABSO0(IJ)=ABSOEX(IJ,ID)
            IJT=IJFR(IJ)
            WD0=W(IJT)-WSUB(IJT)-WCTOTE(IJ)
            WD0C=WD0+WDC0(IJ)
            IF(ID.EQ.1) THEN
               FLUXW=FH(IJT)*RAD0(IJ)
               IF(.NOT.LSKIP(IJT)) GRD=GRD+WD0C*FLUXW*ABSO0(IJ)
               FLEX=FLEX+WD0*FLUXW
             ELSE
               RADM(IJ)=RADEX(IJ,ID-1)
               FKM(IJ)=FAKEX(IJ,ID-1)
               ABSOM(IJ)=ABSOEX(IJ,ID-1)
               FRD=FK0(IJ)*RAD0(IJ)-FKM(IJ)*RADM(IJ)
               IF(.NOT.LSKIP(IJFR(IJ))) GRD=GRD+WD0C*FRD
               DTAUM=(ABSO0(IJ)*DENS1(ID)+ABSOM(IJ)*DENS1(ID-1))*
     *                DELDM(ID-1)
               FLEX=FLEX+WD0*FRD/DTAUM
            END IF
   60    CONTINUE
         END IF
         GRAD(ID)=GRD+FPRD(ID)
         IF(ID.EQ.1) THEN
            GRAD(ID)=GRAD(ID)/DENS(ID)
          ELSE
            GRAD(ID)=GRAD(ID)/(DM(ID)-DM(ID-1))
         END IF
         FLTOT(ID)=FLEX
C
C        other quantities
C
         AN=DENS(ID)/WMM(ID)+ELEC(ID)
         P=AN*TEMP(ID)*BOLK
         IF(ID.LT.ND.AND.GRAD(ID).GT.0.)
     *      GR=LOG10(GRAD(ID)*4.1916825D-10)
         FLTO=FLTOT(ID)+FLFIX(ID)
         IF(ID.EQ.ND) FLTO=SIG4P*TEFF**4
         WRITE(6,612) ID,DM(ID),TROSS(ID),TEMP(ID),ELEC(ID),DENS(ID),P,
     *                GR,FLTOT(ID),FLFIX(ID),FLTO
   70 CONTINUE
C
C ************    Print populations and corresponding b-factors
C
C     In total, NPGPOP pages of outprint of populations are printed,
C     each page contains populations and b-factors for up to 6 levels.
C     The control parameter IIPR(I,J) (which is transmitted by
C     COMMON/PRINTP) says that on the I-th position on the J-th page
C     the population of the level with index IIPR is printed.
C
  100 IF(KPOP.EQ.0) RETURN
      DO 200 IPAGE=1,NPGPOP
         N=6
         I=1
  110    II=IIPR(I,IPAGE)
         IF(II.NE.0.AND.II.LE.NLEVEL) GO TO 120
         N=I-1
         GO TO 130
  120    IOT(I)=TYPION(IEL(II))
         TP(I)=TYPLEV(II)
         IELEM(I)=IEL(II)
         II0(I)=II
         I=I+1
         IF(I.LE.6) GO TO 110
c 130    WRITE(6,621) (IOT(I),TP(I),I=1,N)
  130    WRITE(6,621) (TP(I),I=1,N)
C
         DO 150 ID=1,ND
            call sabolf(id)
            do 141 i=1,nion
  141          usustk(id,i)=usum(i)
            DO 140 I=1,N
               II=II0(I)
               IE=IELEM(I)
               NKI=NNEXT(IE)
               PPR(I)=POPUL(II,ID)
               BF(I)=UN
               IF(II.EQ.NKI) GO TO 140
               plte=elec(id)*sbf(ii)*popul(nki,id)
               if(plte.gt.0.) bf(i)=ppr(i)/plte
  140       CONTINUE
         WRITE(6,623) ID,(PPR(I),BF(I),I=1,N)
  150    CONTINUE
  200 CONTINUE
C
C ************    Print some populations and upper sums
C
      DO 250 I=1,NION
         N0=NFIRST(I)
         N1=NNEXT(I)
         WRITE(6,630) TYPION(I),TYPLEV(N0),TYPLEV(N1)
         DO 260 ID=1,ND
            USUPR=ELEC(ID)*USUSTK(ID,I)*POPUL(N1,ID)
            POPTOT(ID,I)=USUPR
            DO 270 IL=N0,NLAST(I)
               POPTOT(ID,I)=POPTOT(ID,I)+POPUL(IL,ID)
  270       CONTINUE
            WRITE(6,631) ID,POPUL(N0,ID),USUPR,POPUL(N1,ID)
  260    CONTINUE
  250 CONTINUE
C
      DO 300 ID=1,ND
         XH=POPTOT(ID,IELH)+POPUL(NNEXT(IELH),ID)
         DO 310 IA=2,NATOM
            POPA(IA)=POPUL(NKA(IA),ID)
            DO 315 I=1,NION
               N0=NFIRST(I)
               IF(IATM(N0).NE.IA) GOTO 315
               POPA(IA)=POPA(IA)+POPTOT(ID,I)
  315       CONTINUE
            RPEH(IA)=POPA(IA)/XH
  310    CONTINUE
      WRITE(6,631) ID,XH,(POPA(IA),RPEH(IA),IA=2,4)
  300 CONTINUE
C
  600 FORMAT(1H1,' MODEL QUANTITIES IN',I3,'. ITERATION'//)
  601 FORMAT('   IJ',8X,'FREQ',11X,'LAMBDA',8X,'FLUX',9X,
     *       'FH',4X,'LOG(FNU)',3X,'LOG(FLAM)'/)
  602 FORMAT(1H ,I6,1PD16.8,0PF12.3,1PD13.4,0PF10.3,2F9.3)
  603 FORMAT(1H0,'TOTAL SURFACE FLUX',1PD15.8)
  611 FORMAT('1 ID    MASS',6X,'TAUROSS',5X,'TEMP',7X,'NE',9X,'DENS',
     *          8X,'P',6X,'LOG G(RAD)',2X,'EXP.FLUX',
     *          3X,'ALI FLUX',3X,'TOT.FLUX'/)
  612 FORMAT(1H ,I3,1P2D11.3,0PF10.1,1P6D11.3,3D13.5)
  621 FORMAT(1H1,'  ID  ',6(7X,A10,3X))
  622 FORMAT(1H ,4X,6('    POPUL     B-FAC  ')/)
  623 FORMAT(1H ,I4,6(1PD11.3,D10.3))
  630 FORMAT(1H1,'  ID  ',2X,A4,3X,A10,'     USUM  ',3X,A10)
  631 FORMAT(1H ,I4,6X,1P7D11.3)
  681 FORMAT(' EMERGENT FLUX FOR MODEL  TEFF=',F8.0,
     *       '  LOG G =',F5.2,'  LTE=',L1 //
     *       '   IJ',8X,'FREQ',11X,'LAMBDA',7X,'FLUX',9X,
     *       'FH',4X,'LOG(FNU)',3X,'LOG(FLAM)'/)
      RETURN
      END
C
C
C     ****************************************************************
C
C
C

      SUBROUTINE SOLVE
C     ================
C
C     Driving procedure for complete linearization
C     Solution of the system
C
C      A * del(PSI{ID-1}) + B * del(PSI{ID}) + C * del(PSI{ID+1}) = VECL
C
C     where PSI{ID} means vector PSI at depth ID,
C           del(PSI{ID}) linearization corrections to PSI;
C     Vector PSI is a vector composed of all unknown model parameters,
C     the choice and order of which is given by input parameters INHE,
C     INRE,INPC,INSE, and INMP (see START).
C     A,B,C are the so-called matrices of complete linearization
C     and VECL is the corressponding rhs vector, pertinent to depth ID.
C
C     The above block tridiagonal system is solved by the standard
C     Gaussian elimination, ie.
C
C       del(PSI{ID}) = BET{ID} - ALF{ID} * del{PSI{ID+1}),
C
C     (the so-called back-solution),
C     where the auxiliary matrix ALF and vector BET, at each depth,
C     are given by (the so-called forward elimination)
C
C       ALF{ID} = (B - A * ALF{ID-1})**-1 * C ,
C     and
C       BET{ID} = (B - A * ALF{ID-1})**-1 * (VECL - A * BET{ID-1})
C
C     Programming notes:
C     Although matrices A,B,C have size (NN x NN), only matrix B is
C     represented as array (MTOT x MTOT), MTOT being the maximum value
C     of NN; other matrices have large parts void, so that
C     A - is represented as array (MAROWS x MTOT), the maximum number of
C         rows of A is MFREX+2 (because only the transfer equations, the
C         hydrostatic equilibrium, and optionally radiative equilibrium
C         equations couple the depth point ID and ID-1)
C     C - is represented as array (MFREX x MCCOLS), ie.
C         the maximum number of rows is MFREX (only the transfer
C         equations couple the depth point ID and ID+1);
C         and since the upper square block (NFREQE x NFREQE), corresponding
C         to the radiative transfer equations, is diagonal, all the
C         transfer equations may be represemted as one column, thus
C         reducing the number of columns of C to number of constraint
C         equations + 1, ie. the maximum number to MCCOLS=MLEVEL+5.
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      DIMENSION ALF(MTOT,MTOT),BET(MTOT,MDEPTH),DPSI(MTOT)
      COMMON/FILES/PSY0(MTOT,MDEPTH),PSY1(MTOT,MDEPTH),
     * PSY2(MTOT,MDEPTH),PSY3(MTOT,MDEPTH)
      COMMON/ODFCHT/CHANT(MDEPTH)
      EQUIVALENCE (DPSI(1),Y2(1))
C
      REWIND 91
      REWIND 92
      REWIND 93
      N=NN
      M=NFREQE
      IF(IFALI.LE.5) THEN
         M1=NFREQE+2
         IF(ICONV.GT.0) M1=NFREQE+3
       ELSE
         M1=N
      END IF
C
C     --------------------------------
C     First part - forward elimination
C     --------------------------------
C
      LMKA=.FALSE.
      IF(ITER.LT.NITER) LMKA=KANT(ITER+1).EQ.1
      LASO=KANT(ITER).EQ.1
      DO 100 ID=1,ND
C
C        evaluate matrices A,B,C, and rhs vector VECL, corresponding
C        to depth ID
C
          CALL WNSTOR(ID)
         IF(.NOT.LASO) THEN
            CALL MATGEN(ID)
            IF(LMKA) WRITE(93) ((A(I,J),I=1,M1),J=1,N)
         ELSE
            CALL RHSGEN(ID)
            READ(93) ((A(I,J),I=1,M1),J=1,N)
         ENDIF
         IF(ID.GT.1) THEN
C
C           expression   VECL-A*BET
C
            DO 20 I=1,M1
               SUM=0.
               DO 10 J=1,N
                  SUM=SUM+A(I,J)*BET(J,ID-1)
   10          CONTINUE
               VECL(I)=VECL(I)-SUM
   20       CONTINUE
C
C           expression   B-A*ALF,  stored in array B
C
            IF(.NOT.LASO) THEN
               DO 40 I=1,M1
                  DO 40 J=1,N
                     SUM=0.
                     DO 30 K=1,N
                        SUM=SUM+A(I,K)*ALF(K,J)
   30                CONTINUE
                     B(I,J)=B(I,J)-SUM
   40          CONTINUE
            END IF
         END IF
C
C        expression  (B-A*ALF)**-1,  stored in array B
C
         IF(LASO) THEN
            READ(92) ((B(I,J),I=1,N),J=1,N)
         ELSE
            CALL MATINV(B,N,MTOT)
            IF(LMKA) WRITE(92) ((B(I,J),I=1,N),J=1,N)
         END IF
C
C        auxiliary vector BET = (B-A*ALF)**-1 * (L-A*BET)
C
         DO 50 I=1,N
            SUM=0.
            DO 45 J=1,N
   45          SUM=SUM+B(I,J)*VECL(J)
            BET(I,ID)=SUM
   50    CONTINUE
C
         IF(ID.LT.ND) THEN
C
C        auxiliary matrix   ALF = (B-A*ALF)**-1 * C
C
            IF(LASO) THEN
               IF(ID.LT.(ND-1)) THEN
                  READ(91) ALF(1,1)
               ELSE
                  READ(91) ((ALF(I,J),I=1,N),J=1,N)
               ENDIF
            ELSE
               DO 60 I=1,N
                  DO 60 J=1,M
                     ALF(I,J)=B(I,J)*C(J,J)
   60          CONTINUE
               DO 90 I=1,N
                  DO 80 J=M+1,N
                     SUM=0.
                     DO 70 K=1,M
   70                   SUM=SUM+B(I,K)*C(K,J)
                     IF(IFALI.GT.5.OR.ICONV.GE.3) THEN
                     DO 75 K=M+1,N
   75                   SUM=SUM+B(I,K)*C(K,J)
                     END IF
   80                ALF(I,J)=SUM
   90          CONTINUE
C
C           store the auxiliary matrix ALF
C
               WRITE(91) ((ALF(I,J),I=1,N),J=1,N)
            END IF
         END IF
  100 CONTINUE
C
C     --------------------------
C     Second part - backsolution
C     --------------------------
C
      DO 110 I=1,N
  110    VECL(I)=0.
      DO 200 IID=1,ND
         ID=ND-IID+1
         IF(ID.LT.ND) THEN
C
C           read old  PSI  
C
            DO 150 I=1,N
               PSI0(I)=PSY0(I,ID)
  150       CONTINUE
C
C           read auxiliary matrix ALF and vector BET
C
            BACKSPACE 91
            READ(91) ((ALF(I,J),I=1,N),J=1,N)
            BACKSPACE 91
C
C           expression  ALF * delta(PSI{previous depth}), stored 
C           in array VECL
C
            DO 130 I=1,N
               SUM=0.
               DO 120 J=1,N
  120             SUM=SUM+ALF(I,J)*DPSI(J)
               VECL(I)=SUM
  130       CONTINUE
         END IF
C
C        finally, evaluate delta(PSI) and corresponding relative changes
C        delta(PSI)/PSI
C
         DO 140 I=1,N
            DPSI(I)=BET(I,ID)-VECL(I)
            CHAN=0.
            IF(PSI0(I).GT.0.) CHAN=DPSI(I)/PSI0(I)
            BET(I,ID)=CHAN
C
C           over-relaxation
C
            IF(I.GE.NFREQE+INSE) CHAN=ORELAX*CHAN 
C
C           To avoid instabilities, relative changes of all quantities
C           are artificially limited not to exceed 0.2 (or -0.2)
C
            IF(CHAN.LE.-0.9) CHAN=-0.9
            IF(CHAN.GE.9.) CHAN=9.
            IF(INRE.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INRE) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INRE) CHAN=0.25
            END IF
            IF(INHE.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INHE) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INHE) CHAN=0.25
            END IF
            IF(INPC.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INPC) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INPC) CHAN=0.25
            END IF
C
C           new vector PSI
C
            PSI0(I)=PSI0(I)*(CHAN+UN)
  140    CONTINUE
         IF(INRE.GT.0) CHANT(ID)=BET(NFREQE+INRE,ID)
C
C        new vector PSI is stored
C
         DO 160 I=1,N
            PSY0(I,ID)=PSI0(I)
  160    CONTINUE
  200 CONTINUE
C

C     print out the relative changes of vector PSI
C
      CALL PRCHAN(BET,CHMX)
C
C     STOP if changes become too large
C
      IF(ITER.NE.1 .AND. ABS(CHMX).GT.1.D16) THEN
         WRITE(6,610) ITER,CHMX
         WRITE(10,610) ITER,CHMX
         STOP
      ENDIF
C
C     Set up Kantorovich method
C
         IF(LASO) THEN
            WRITE(6,600) ITER
            WRITE(10,600) ITER
         ENDIF
c     ENDIF
C
C     Finally, set up quantity LFIN that indicates whether or not
C     this iteration of complete linearization is the last one
C
      LFIN=ABS(CHMX).LE.CHMAX.OR.ITER.GE.NITER
      RETURN
  600 FORMAT(/,' **** KANTOROVICH acceleration: ITER',I4,//)
  610 FORMAT(/,' **** STOP in SOLVE after ITER',I4,/,
     * ' Max change:',1PE12.2)
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE SOLVES
C     =================
C
C     Same routine than SOLVE, but used for small systems of
C     equations (NN<=MSMX) to keep matrices in memory and to avoid I/O
C
C     Driving procedure for complete linearization
C     Solution of the system
C
C      A * del(PSI{ID-1}) + B * del(PSI{ID}) + C * del(PSI{ID+1}) = VECL
C
C     where PSI{ID} means vector PSI at depth ID,
C           del(PSI{ID}) linearization corrections to PSI;
C     Vector PSI is a vector composed of all unknown model parameters,
C     the choice and order of which is given by input parameters INHE,
C     INRE,INPC,INSE, and INMP (see START).
C     A,B,C are the so-called matrices of complete linearization
C     and VECL is the corressponding rhs vector, pertinent to depth ID.
C
C     The above block tridiagonal system is solved by the standard
C     Gaussian elimination, ie.
C
C       del(PSI{ID}) = BET{ID} - ALF{ID} * del{PSI{ID+1}),
C
C     (the so-called back-solution),
C     where the auxiliary matrix ALF and vector BET, at each depth,
C     are given by (the so-called forward elimination)
C
C       ALF{ID} = (B - A * ALF{ID-1})**-1 * C ,
C     and
C       BET{ID} = (B - A * ALF{ID-1})**-1 * (VECL - A * BET{ID-1})
C
C     Programming notes:
C     Although matrices A,B,C have size (NN x NN), only matrix B is
C     represented as array (MTOT x MTOT), MTOT being the maximum value
C     of NN; other matrices have large parts void, so that
C     A - is represented as array (MAROWS x MTOT), the maximum number of
C         rows of A is MFREX+2 (because only the transfer equations, the
C         hydrostatic equilibrium, and optionally radiative equilibrium
C         equations couple the depth point ID and ID-1)
C     C - is represented as array (MFREX x MCCOLS), ie.
C         the maximum number of rows is MFREX (only the transfer
C         equations couple the depth point ID and ID+1);
C         and since the upper square block (NFREQE x NFREQE), corresponding
C         to the radiative transfer equations, is diagonal, all the
C         transfer equations may be represemted as one column, thus
C         reducing the number of columns of C to number of constraint
C         equations + 1, ie. the maximum number to MCCOLS=MLEVEL+5.
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      DIMENSION ALF(MTOT,MTOT),BET(MTOT,MDEPTH),DPSI(MTOT)
      COMMON/FILES/PSY0(MTOT,MDEPTH),PSY1(MTOT,MDEPTH),
     * PSY2(MTOT,MDEPTH),PSY3(MTOT,MDEPTH)
      COMMON/ODFCHT/CHANT(MDEPTH)
      COMMON/STOMAT/STOA(MSMX,MSMX,MDEPTH),STOB(MSMX,MSMX,MDEPTH),
     * STOALF(MSMX,MSMX,MDEPTH)
      EQUIVALENCE (DPSI(1),Y2(1))
C
      N=NN
      M=NFREQE
      IF(IFALI.LE.5) THEN
         M1=NFREQE+2
         IF(ICONV.GT.0) M1=NFREQE+3
       ELSE
         M1=N
      END IF
C
C     --------------------------------
C     First part - forward elimination
C     --------------------------------
C
      LMKA=.FALSE.
      IF(ITER.LT.NITER) LMKA=KANT(ITER+1).EQ.1
      LASO=KANT(ITER).EQ.1
      DO 100 ID=1,ND
C
C        evaluate matrices A,B,C, and rhs vector VECL, corresponding
C        to depth ID
C
         CALL WNSTOR(ID)
         IF(.NOT.LASO) THEN
            CALL MATGEN(ID)
            IF(LMKA) THEN
              DO 500 J=1,N
                DO 500 I=1,M1
                  STOA(I,J,ID)=A(I,J)
  500         CONTINUE
            ENDIF
         ELSE
            CALL RHSGEN(ID)
            DO 510 J=1,N
              DO 510 I=1,M1
                A(I,J)=STOA(I,J,ID)
  510       CONTINUE
         ENDIF
         IF(ID.GT.1) THEN
C
C           expression   VECL-A*BET
C
            DO 20 I=1,M1
               SUM=0.
               DO 10 J=1,N
                  SUM=SUM+A(I,J)*BET(J,ID-1)
   10          CONTINUE
               VECL(I)=VECL(I)-SUM
   20       CONTINUE
C
C           expression   B-A*ALF,  stored in array B
C
            IF(.NOT.LASO) THEN
               DO 40 I=1,M1
                  DO 40 J=1,N
                     SUM=0.
                     DO 30 K=1,N
                        SUM=SUM+A(I,K)*ALF(K,J)
   30                CONTINUE
                     B(I,J)=B(I,J)-SUM
   40          CONTINUE
            END IF
         END IF
C
C        expression  (B-A*ALF)**-1,  stored in array B
C
         IF(LASO) THEN
            DO 520 J=1,N
              DO 520 I=1,N
                B(I,J)=STOB(I,J,ID)
  520       CONTINUE
         ELSE
            CALL MATINV(B,N,MTOT)
            IF(LMKA) THEN
              DO 530 J=1,N
                DO 530 I=1,N
                  STOB(I,J,ID)=B(I,J)
  530         CONTINUE
            ENDIF
         END IF
C
C        auxiliary vector BET = (B-A*ALF)**-1 * (L-A*BET)
C
         DO 50 I=1,N
            SUM=0.
            DO 45 J=1,N
   45          SUM=SUM+B(I,J)*VECL(J)
            BET(I,ID)=SUM
   50    CONTINUE
C
         IF(ID.LT.ND) THEN
C
C        auxiliary matrix   ALF = (B-A*ALF)**-1 * C
C
            IF(LASO) THEN
               DO 540 J=1,N
                 DO 540 I=1,N
                   ALF(I,J)=STOALF(I,J,ID)
  540          CONTINUE
            ELSE
               DO 60 I=1,N
                  DO 60 J=1,M
                     ALF(I,J)=B(I,J)*C(J,J)
   60          CONTINUE
               DO 90 I=1,N
                  DO 80 J=M+1,N
                     SUM=0.
                     DO 70 K=1,M
   70                   SUM=SUM+B(I,K)*C(K,J)
                     IF(IFALI.GT.5.OR.ICONV.GE.3) THEN
                     DO 75 K=M+1,N
   75                   SUM=SUM+B(I,K)*C(K,J)
                     END IF
   80                ALF(I,J)=SUM
   90          CONTINUE
C
C           store the auxiliary matrix ALF
C
               DO 550 J=1,N
                 DO 550 I=1,N
                   STOALF(I,J,ID)=ALF(I,J)
  550          CONTINUE
            END IF
         END IF
  100 CONTINUE
C
C     --------------------------
C     Second part - backsolution
C     --------------------------
C
      DO 110 I=1,N
  110    VECL(I)=0.
      DO 200 IID=1,ND
         ID=ND-IID+1
         IF(ID.LT.ND) THEN
C
C           read old  PSI  
C
            DO 150 I=1,N
               PSI0(I)=PSY0(I,ID)
  150       CONTINUE
C
C           read auxiliary matrix ALF and vector BET
C
            DO 560 J=1,N
              DO 560 I=1,N
                ALF(I,J)=STOALF(I,J,ID)
  560       CONTINUE
C
C           expression  ALF * delta(PSI{previous depth}), stored 
C           in array VECL
C
            DO 130 I=1,N
               SUM=0.
               DO 120 J=1,N
  120             SUM=SUM+ALF(I,J)*DPSI(J)
               VECL(I)=SUM
  130       CONTINUE
         END IF
C
C        finally, evaluate delta(PSI) and corresponding relative changes
C        delta(PSI)/PSI
C
         DO 140 I=1,N
            DPSI(I)=BET(I,ID)-VECL(I)
            CHAN=0.
            IF(PSI0(I).GT.0.) CHAN=DPSI(I)/PSI0(I)
            BET(I,ID)=CHAN
C
C           over-relaxation
C
            IF(I.GE.NFREQE+INSE) CHAN=ORELAX*CHAN 
C
C           To avoid instabilities, relative changes of all quantities
C           are artificially limited not to exceed 0.2 (or -0.2)
C
            IF(CHAN.LE.-0.9) CHAN=-0.9
            IF(CHAN.GE.9.) CHAN=9.
            IF(INRE.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INRE) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INRE) CHAN=0.25
            END IF
            IF(INHE.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INHE) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INHE) CHAN=0.25
            END IF
            IF(INPC.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INPC) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INPC) CHAN=0.25
            END IF
C
C           new vector PSI
C
            PSI0(I)=PSI0(I)*(CHAN+UN)
  140    CONTINUE
         IF(INRE.GT.0) CHANT(ID)=BET(NFREQE+INRE,ID)
C
C        new vector PSI is stored
C
         DO 160 I=1,N
            PSY0(I,ID)=PSI0(I)
  160    CONTINUE
  200 CONTINUE
C
C     print out the relative changes of vector PSI
C
      CALL PRCHAN(BET,CHMX)
C
C     STOP if changes become too large
C
      IF(ITER.NE.1 .AND. ABS(CHMX).GT.1.D16) THEN
         WRITE(6,610) ITER,CHMX
         WRITE(10,610) ITER,CHMX
         STOP
      ENDIF
C
C     Set up Kantorovich method
C
         IF(LASO) THEN
            WRITE(6,600) ITER
            WRITE(10,600) ITER
         ENDIF
c     ENDIF
C
C     Finally, set up quantity LFIN that indicates whether or not
C     this iteration of complete linearization is the last one
C
      LFIN=ABS(CHMX).LE.CHMAX.OR.ITER.GE.NITER
      RETURN
  600 FORMAT(/,' **** KANTOROVICH acceleration: ITER',I4,//)
  610 FORMAT(/,' **** STOP in SOLVE after ITER',I4,/,
     * ' Max change:',1PE12.2)
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE SOLVE4
C     =================
C
C     Modification of solve for a 4-band block matrix
C     Used for small systems of
C     equations (NN<=MSMX) to keep matrices in memory and to avoid I/O
C
C     Solution of the system
C
C      E * del(PSI{ID-2}) + A * del(PSI{ID-1}) + 
C      B * del(PSI{ID}) + C * del(PSI{ID+1}) = VECL
C
C     where PSI{ID} means vector PSI at depth ID,
C           del(PSI{ID}) linearization corrections to PSI;
C     Vector PSI is a vector composed of all unknown model parameters,
C     the choice and order of which is given by input parameters INHE,
C     INRE,INPC,INSE, and INMP (see START).
C     A,B,C are the so-called matrices of complete linearization
C     and VECL is the corressponding rhs vector, pertinent to depth ID.
C
C     The above block tridiagonal system is solved by the standard
C     Gaussian elimination, ie.
C
C       del(PSI{ID}) = BET{ID} - ALF'{ID} * del{PSI{ID+1}),
C
C     (the so-called back-solution),
C     where the auxiliary matrix ALF and vector BET, at each depth,
C     are given by (the so-called forward elimination)
C
C       ALF{ID} = (B - A * ALF'{ID-1})**-1 * C ,
C     and
C       BET{ID} = (B - A * ALF'{ID-1})**-1 * (VECL' - A * BET'{ID-1})
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      DIMENSION ALF(MTOT,MTOT),BET(MTOT,MDEPTH),DPSI(MTOT)
      COMMON/FILES/PSY0(MTOT,MDEPTH),PSY1(MTOT,MDEPTH),
     * PSY2(MTOT,MDEPTH),PSY3(MTOT,MDEPTH)
      COMMON/ODFCHT/CHANT(MDEPTH)
      COMMON/STOMAT/STOA(MSMX,MSMX,MDEPTH),STOB(MSMX,MSMX,MDEPTH),
     * STOALF(MSMX,MSMX,MDEPTH)
      EQUIVALENCE (DPSI(1),Y2(1))
C
      N=NN
      M=NFREQE
      IF(IFALI.LE.5) THEN
         M1=NFREQE+2
         IF(ICONV.GT.0) M1=NFREQE+3
       ELSE
         M1=N
      END IF
C
C     --------------------------------
C     First part - forward elimination
C     --------------------------------
C
      LMKA=.FALSE.
      IF(ITER.LT.NITER) LMKA=KANT(ITER+1).EQ.1
      LASO=KANT(ITER).EQ.1
      DO 100 ID=1,ND
C
C        evaluate matrices A,B,C, and rhs vector VECL, corresponding
C        to depth ID
C
         CALL WNSTOR(ID)
         IF(.NOT.LASO) THEN
            CALL MATGEN(ID)
            IF(LMKA) THEN
              DO 500 J=1,N
                DO 500 I=1,M1
                  STOA(I,J,ID)=A(I,J)
  500         CONTINUE
            ENDIF
         ELSE
            CALL RHSGEN(ID)
            DO 510 J=1,N
              DO 510 I=1,M1
                A(I,J)=STOA(I,J,ID)
  510       CONTINUE
         ENDIF
         IF(ID.GT.1) THEN
C
C           modified L and A
C
            IF(ID.GT.2) THEN
               DO I=1,N
                  SUM=0.
                  DO J=1,N
                     SUM=SUM+E(I,J)*BET(J,ID-2)
                     SUMA=0.
                     DO K=1,N
                        SUMA=SUMA+E(I,K)*STOALF(K,J,ID-2)
                     END DO
                     A(I,J)=A(I,J)-SUMA
                  END DO
                  VECL(I)=VECL(I)-SUM
               END DO
            END IF
C
C           expression   VECL'-A'*BET
C
            DO 20 I=1,M1
               SUM=0.
               DO 10 J=1,N
                  SUM=SUM+A(I,J)*BET(J,ID-1)
   10          CONTINUE
               VECL(I)=VECL(I)-SUM
   20       CONTINUE
C
C           expression   B-A*ALF,  stored in array B
C
            IF(.NOT.LASO) THEN
               DO 40 I=1,M1
                  DO 40 J=1,N
                     SUM=0.
                     DO 30 K=1,N
                        SUM=SUM+A(I,K)*ALF(K,J)
   30                CONTINUE
                     B(I,J)=B(I,J)-SUM
   40          CONTINUE
            END IF
         END IF
C
C        expression  (B-A'*ALF)**-1,  stored in array B
C
         IF(LASO) THEN
            DO 520 J=1,N
              DO 520 I=1,N
                B(I,J)=STOB(I,J,ID)
  520       CONTINUE
         ELSE
            CALL MATINV(B,N,MTOT)
            IF(LMKA) THEN
              DO 530 J=1,N
                DO 530 I=1,N
                  STOB(I,J,ID)=B(I,J)
  530         CONTINUE
            ENDIF
         END IF
C
C        auxiliary vector BET = (B-A'*ALF)**-1 * (L'-A'*BET)
C
         DO 50 I=1,N
            SUM=0.
            DO 45 J=1,N
   45          SUM=SUM+B(I,J)*VECL(J)
            BET(I,ID)=SUM
   50    CONTINUE
C
         IF(ID.LT.ND) THEN
C
C        auxiliary matrix   ALF = (B-A'*ALF)**-1 * C
C
            IF(LASO) THEN
               DO 540 J=1,N
                 DO 540 I=1,N
                   ALF(I,J)=STOALF(I,J,ID)
  540          CONTINUE
            ELSE
               DO 90 I=1,N
                  DO 80 J=1,N
                     SUM=0.
                     DO 70 K=1,N
   70                   SUM=SUM+B(I,K)*C(K,J)
   80                ALF(I,J)=SUM
   90          CONTINUE
C
C           store the auxiliary matrix ALF
C
               DO 550 J=1,N
                 DO 550 I=1,N
                   STOALF(I,J,ID)=ALF(I,J)
  550          CONTINUE
            END IF
         END IF
  100 CONTINUE
C
C     --------------------------
C     Second part - backsolution
C     --------------------------
C
      DO 110 I=1,N
  110    VECL(I)=0.
      DO 200 IID=1,ND
         ID=ND-IID+1
         IF(ID.LT.ND) THEN
C
C           read old  PSI  
C
            DO 150 I=1,N
               PSI0(I)=PSY0(I,ID)
  150       CONTINUE
C
C           read auxiliary matrix ALF and vector BET
C
            DO 560 J=1,N
              DO 560 I=1,N
                ALF(I,J)=STOALF(I,J,ID)
  560       CONTINUE
C
C           expression  ALF * delta(PSI{previous depth}), stored 
C           in array VECL
C
            DO 130 I=1,N
               SUM=0.
               DO 120 J=1,N
  120             SUM=SUM+ALF(I,J)*DPSI(J)
               VECL(I)=SUM
  130       CONTINUE
         END IF
C
C        finally, evaluate delta(PSI) and corresponding relative changes
C        delta(PSI)/PSI
C
         DO 140 I=1,N
            DPSI(I)=BET(I,ID)-VECL(I)
            CHAN=0.
            IF(PSI0(I).GT.0.) CHAN=DPSI(I)/PSI0(I)
            BET(I,ID)=CHAN
C
C           over-relaxation
C
            IF(I.GE.NFREQE+INSE) CHAN=ORELAX*CHAN 
C
C           To avoid instabilities, relative changes of all quantities
C           are artificially limited not to exceed 0.2 (or -0.2)
C
            IF(CHAN.LE.-0.9) CHAN=-0.9
            IF(CHAN.GE.9.) CHAN=9.
            IF(INRE.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INRE) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INRE) CHAN=0.25
            END IF
            IF(INHE.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INHE) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INHE) CHAN=0.25
            END IF
            IF(INPC.GT.0) THEN
               IF(CHAN.LE.-0.25.AND.I.EQ.NFREQE+INPC) CHAN=-0.25
               IF(CHAN.GT.0.25.AND.I.EQ.NFREQE+INPC) CHAN=0.25
            END IF
C
C           new vector PSI
C
            PSI0(I)=PSI0(I)*(CHAN+UN)
  140    CONTINUE
         IF(INRE.GT.0) CHANT(ID)=BET(NFREQE+INRE,ID)
C
C        new vector PSI is stored
C
         DO 160 I=1,N
            PSY0(I,ID)=PSI0(I)
  160    CONTINUE
  200 CONTINUE
C
C     print out the relative changes of vector PSI
C
      CALL PRCHAN(BET,CHMX)
C
C     STOP if changes become too large
C
      IF(ITER.NE.1 .AND. ABS(CHMX).GT.1.D16) THEN
         WRITE(6,610) ITER,CHMX
         WRITE(10,610) ITER,CHMX
         STOP
      ENDIF
C
C     Set up Kantorovich method
C
         IF(LASO) THEN
            WRITE(6,600) ITER
            WRITE(10,600) ITER
         ENDIF
c     ENDIF
C
C     Finally, set up quantity LFIN that indicates whether or not
C     this iteration of complete linearization is the last one
C
      LFIN=ABS(CHMX).LE.CHMAX.OR.ITER.GE.NITER
      RETURN
  600 FORMAT(/,' **** KANTOROVICH acceleration: ITER',I4,//)
  610 FORMAT(/,' **** STOP in SOLVE after ITER',I4,/,
     * ' Max change:',1PE12.2)
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE MATGEN(ID)
C     =====================
C
C     Auxiliary procedure for SOLVE
C     controls evaluation of matrices A,B, and C
C
C     Input:  ID - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      COMMON/FILES/PSY0(MTOT,MDEPTH),PSY1(MTOT,MDEPTH),
     *             PSY2(MTOT,MDEPTH),PSY3(MTOT,MDEPTH)
C
C
C     evaluation of the opacity, emissivity, scattering, and
C     their derivatives, at the current depth point ID;
C
         DO 10 I=1,NN
   10       PSI0(I)=PSY0(I,ID)
         IF(NFREQE.GT.0) THEN
         DO 20 IJ=1,NFREQE
            IJT=IJFR(IJ)
            WDEP0(IJ)=W(IJT)-WSUB(IJT)-WCTOTE(IJ)
            RAD0(IJ)=RADEX(IJ,ID)
            FK0(IJ)=FAKEX(IJ,ID)
            ABSO0(IJ)=ABSOEX(IJ,ID)
            EMIS0(IJ)=EMISEX(IJ,ID)
            SCAT0(IJ)=SCATEX(IJ,ID)
            DABT0(IJ)=DABTEX(IJ,ID)
            DEMT0(IJ)=DEMTEX(IJ,ID)
            DABN0(IJ)=DABNEX(IJ,ID)
            DEMN0(IJ)=DEMNEX(IJ,ID)
            DO 20 II=1,NLVEXP
               DRCH0(II,IJ)=DRCHEX(II,IJ,ID)
               DRET0(II,IJ)=DRETEX(II,IJ,ID)
   20    CONTINUE
         END IF
C
      IF(ID.GT.1) THEN
         DO 30 I=1,NN
   30       PSIM(I)=PSY0(I,ID-1)
         IF(NFREQE.GT.0) THEN
         DO 40 IJ=1,NFREQE
            IJT=IJFR(IJ)
            RADM(IJ)=RADEX(IJ,ID-1)
            FKM(IJ)=FAKEX(IJ,ID-1)
            ABSOM(IJ)=ABSOEX(IJ,ID-1)
            EMISM(IJ)=EMISEX(IJ,ID-1)
            SCATM(IJ)=SCATEX(IJ,ID-1)
            DABTM(IJ)=DABTEX(IJ,ID-1)
            DEMTM(IJ)=DEMTEX(IJ,ID-1)
            DABNM(IJ)=DABNEX(IJ,ID-1)
            DEMNM(IJ)=DEMNEX(IJ,ID-1)
            DO 40 II=1,NLVEXP
               DRCHM(II,IJ)=DRCHEX(II,IJ,ID-1)
               DRETM(II,IJ)=DRETEX(II,IJ,ID-1)
   40    CONTINUE
         END IF
      END IF
C
      IF(ID.LT.ND) THEN
         DO 50 I=1,NN
   50       PSIP(I)=PSY0(I,ID+1)
         IF(NFREQE.GT.0) THEN
         DO 60 IJ=1,NFREQE
            IJT=IJFR(IJ)
            RADP(IJ)=RADEX(IJ,ID+1)
            FKP(IJ)=FAKEX(IJ,ID+1)
            ABSOP(IJ)=ABSOEX(IJ,ID+1)
            EMISP(IJ)=EMISEX(IJ,ID+1)
            SCATP(IJ)=SCATEX(IJ,ID+1)
            DABTP(IJ)=DABTEX(IJ,ID+1)
            DEMTP(IJ)=DEMTEX(IJ,ID+1)
            DABNP(IJ)=DABNEX(IJ,ID+1)
            DEMNP(IJ)=DEMNEX(IJ,ID+1)
            DO 60 II=1,NLVEXP
               DRCHP(II,IJ)=DRCHEX(II,IJ,ID+1)
               DRETP(II,IJ)=DRETEX(II,IJ,ID+1)
   60    CONTINUE
         END IF
      END IF
C
C
C     ------------------------------------------------------------
C     Actual evaluation of matrices A,B,C, and the rhs vector VECL
C     ------------------------------------------------------------
C
C     First null arrays A,B,C,VECL
C
      DO 110 I=1,NN   
         VECL(I)=0.
         DO 110 J=1,NN   
            B(J,I)=0.
            A(J,I)=0.
            C(J,I)=0.
            E(J,I)=0.
  110 CONTINUE
C
C     for the sake of clarity, the matrices are evaluated by several
C     different subroutines
C
      CALL BRTE(ID)
      IF(INHE.NE.0) CALL BHE(ID)
      IF(INRE.NE.0) CALL BRE(ID)
      CALL SABOLF(ID)
      CALL BPOP(ID)
      CALL MATCON(ID)
      CALL EMAT(ID)
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE BRTE(ID)
C     ===================
C
C     The part of matrices A,B,C corresponding to the linearized
C     radiative transfer equation
C     i.e. the first NFREQE rows
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ARRAY1.FOR'
c     PARAMETER (SIXTH=0.166666666666667D0,
c    *           THIRD=0.333333333333333D0)
      PARAMETER (SIXTH=UN/6.D0,
     *           THIRD=UN/3.D0)
      COMMON/OPACAD/ABAD,EMAD,SCAD,DAT,DAN,DET,DEN,DST,DSN,DDN(MLEVEL)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
C
      IF(NFREQE.LE.0) RETURN
      ispl=isplin
      if(isplin.ge.5) isplin=isplin-5
      NHE=NFREQE+INHE
      NRE=NFREQE+INRE
      NPC=NFREQE+INPC
      NSE=NFREQE+INSE-1
      NMP=NFREQE+INMP
C
      GP=0.
      GN=UN
      IF(INMP.GT.0) THEN
         GP=UN/WMY(ID)
         GN=0.
      END IF
C
C
C     ----------------------------------------
C     For  ID = 1  -  upper boundary condition
C     ----------------------------------------
C
      IF(ID.GT.1) GO TO 50
      DDP=(DM(2)-DM(1))*HALF 
      DO 40 IJ=1,NFREQE
         IJT=IJFR(IJ)
         OMEG0=ABSO0(IJ)/DENS(ID)
         OMEGP=ABSOP(IJ)/DENS(ID+1)
         DZP=OMEG0+OMEGP
         DTAUP=DZP*DDP
         ALF1=(FK0(IJ)*RAD0(IJ)-FKP(IJ)*RADP(IJ))/DTAUP
         CHIEL0=SCAT0(IJ)
         CHIELP=SCATP(IJ)
         S0=(EMIS0(IJ)+CHIEL0*RAD0(IJ))/ABSO0(IJ)
         BS=HALF*DTAUP
         CS=0.
         C2=0.
         GAM2=0.
         BET2=0.
         SP=0.
         IF(MOD(ISPLIN,3).GT.0) THEN
C
C      Spline collocation and/or Hermitian method (ISPLIN=1 or 2) -
C      both give the same expression for the boundary conditions
C
            BS=DTAUP*THIRD
            CS=HALF *BS
            SP=(EMISP(IJ)+CHIELP*RADP(IJ))/ABSOP(IJ)
            C2=CS/ABSOP(IJ)
            GAM2=CS*(RADP(IJ)-SP)
         END IF
C
C      auxiliary quantities
C
         ALF2=BS*(RAD0(IJ)-S0)
         BET2=ALF2+GAM2
         X1=(ALF1-BET2)/DZP
         B2=(BS+Q0(IJ))/ABSO0(IJ)
         B1=X1/DENS(1)
         B1=B1+UU0(IJ)*S0*DM(1)/DENS(1)
         C1=X1/DENS(2)
C
C *** elements of the IJ-th row of matrices B and C
C
         RTN=OMEG0*WMM(1)*B1
         B(IJ,NHE)=-GN*RTN
         B(IJ,NMP)=-GP*RTN
         B1=B1-B2*S0
C
         RTNC=OMEGP*WMM(2)*C1
         C(IJ,NHE)=-GN*RTNC
         C(IJ,NMP)=-GP*RTNC
         C1=C1-C2*SP
C
         B(IJ,NRE)=B1*DABT0(IJ)+B2*(DEMT0(IJ)+DST*RAD0(IJ))
         C(IJ,NRE)=C1*DABTP(IJ)+C2*(DEMTP(IJ)+DST*RADP(IJ))
         B(IJ,NPC)=B1*DABN0(IJ)+
     *             B2*(DEMN0(IJ)+(DSN+SIGE)*RAD0(IJ))+
     *             GN*RTN
         C(IJ,NPC)=C1*DABNP(IJ)+
     *             C2*(DEMNP(IJ)+(DSN+SIGE)*RADP(IJ))+
     *             GN*RTNC
         DO 20 II=1,NLVEXP
             B(IJ,NSE+II)=B(IJ,NSE+II)+
     *                    B1*DRCH0(II,IJ)+B2*DRET0(II,IJ)
             C(IJ,NSE+II)=C(IJ,NSE+II)+
     *                    C1*DRCHP(II,IJ)+C2*DRETP(II,IJ)
   20    CONTINUE
         B(IJ,NFREQE)=0.
         B(IJ,IJ)=-FK0(IJ)/DTAUP-FH(IJT)-BS*(UN-CHIEL0/ABSO0(IJ))+
     *             Q0(IJ)*CHIEL0/ABSO0(IJ)
         C(IJ,NFREQE)=0.
         C(IJ,IJ)=FKP(IJ)/DTAUP-CS*(UN-CHIELP/ABSOP(IJ))
C
C *** the IJ-th element of the rhs vector
C
         VECL(IJ)=ALF1+BET2+FH(IJT)*RAD0(IJ)-S0*Q0(IJ)
         IF(IWINBL.LT.0) VECL(IJ)=VECL(IJ)-HALF*EXTRAD(IJT)
   40 CONTINUE
      isplin=ispl
      RETURN
C
C     ---------------------------------------
C     For  1 < ID < ND  -  normal depth point
C     ---------------------------------------
C
   50 DDM=(DM(ID)-DM(ID-1))*HALF 
      IF(ID.EQ.ND) GO TO 150
      DDP=(DM(ID+1)-DM(ID))*HALF 
      DO 130 IJ=1,NFREQE
         OMEG0=ABSO0(IJ)/DENS(ID)
         OMEGP=ABSOP(IJ)/DENS(ID+1)
         OMEGM=ABSOM(IJ)/DENS(ID-1)
         DZP=OMEG0+OMEGP
         DZM=OMEG0+OMEGM
         DTAUP=DZP*DDP
         DTAUM=DZM*DDM
         DTAU0=HALF *(DTAUP+DTAUM)
         FRD=FK0(IJ)*RAD0(IJ)
         ALF1=(FRD-FKP(IJ)*RADP(IJ))/DTAUP/DTAU0
         GAM1=(FRD-FKM(IJ)*RADM(IJ))/DTAUM/DTAU0
         BET1=ALF1+GAM1
         X1=HALF *BET1/DTAU0
         A1=(GAM1+X1*DTAUM)/DZM
         C1=(ALF1+X1*DTAUP)/DZP
         B1=(A1+C1)/DENS(ID)
         A1=A1/DENS(ID-1)
         C1=C1/DENS(ID+1)
         BS=UN
         CHIELM=SCATM(IJ)
         CHIEL0=SCAT0(IJ)
         CHIELP=SCATP(IJ)
         S0=(EMIS0(IJ)+CHIEL0*RAD0(IJ))/ABSO0(IJ)
         AS=0.
         CS=0.
         A2=0.
         C2=0.
         A3=0.
         C3=0.
         BET2=0.
         SM=0.
         SP=0.
         IF(MOD(ISPLIN,3).EQ.0) GO TO 60
         SM=(EMISM(IJ)+RADM(IJ)*CHIELM)/ABSOM(IJ)
         SP=(EMISP(IJ)+RADP(IJ)*CHIELP)/ABSOP(IJ)
         IF(ISPLIN.EQ.1) THEN
C
C     spline collocation (ISPLIN=1)
C
            AS=DTAUM/DTAU0*SIXTH
            CS=DTAUP/DTAU0*SIXTH
            BS=0.666666666666667D0
            ALF2=AS*(RADM(IJ)-SM)
            GAM2=CS*(RADP(IJ)-SP)
            BET2=ALF2+GAM2
            X =HALF *BET2/DTAU0
            A2=(GAM2-X*DTAUM)/DZM
            C2=(ALF2-X*DTAUP)/DZP
          ELSE
C
C     Hermitian method (ISPLIN=2)
C
            AS=DTAUP*DTAUP/DTAUM/DTAU0
            CS=DTAUM*DTAUM/DTAUP/DTAU0
            AL3=(RADP(IJ)-SP-RAD0(IJ)+S0)*SIXTH
            GA3=(RADM(IJ)-SM-RAD0(IJ)+S0)*SIXTH
            AV=AL3*CS
            CV=GA3*AS
            AS=(UN-HALF *AS)*SIXTH
            CS=(UN-HALF *CS)*SIXTH
            BS=UN-AS-CS
            X=(AV+CV)/DTAU0/4.D0
            A2=(X*DTAUM+HALF *CV-AV)/DZM
            C2=(X*DTAUP+HALF *AV-CV)/DZP
            BET2=AS*(RADM(IJ)-SM)+CS*(RADP(IJ)-SP)
         END IF
C
C     auxiliary quantities
C
         B1=B1-(A2+C2)/DENS(ID)
         A1=A1-A2/DENS(ID-1)
         C1=C1-C2/DENS(ID+1)
         A2=AS/ABSOM(IJ)
         C2=CS/ABSOP(IJ)
         A3=A2*SM
         C3=C2*SP
   60    B2=BS/ABSO0(IJ)
         B3=B2*S0
C
C *** elements of the IJ-th row of matrices A, B, and C
C
         RTNA=OMEGM*WMM(ID-1)*A1
         A(IJ,NHE)=-GN*RTNA
         A(IJ,NMP)=-GP*RTNA
         A1=A1-A3
C
         RTN=OMEG0*WMM(ID)*B1
         B(IJ,NHE)=-GN*RTN
         B(IJ,NMP)=-GP*RTN
         B1=B1-B3
C
         RTNC=OMEGP*WMM(ID+1)*C1
         C(IJ,NHE)=-GN*RTNC
         C(IJ,NMP)=-GP*RTNC
         C1=C1-C3
C
         A(IJ,NRE)= A1*DABTM(IJ)+A2*(DEMTM(IJ)+DST*RADM(IJ))
         B(IJ,NRE)= B1*DABT0(IJ)+B2*(DEMT0(IJ)+DST*RAD0(IJ))
         C(IJ,NRE)= C1*DABTP(IJ)+C2*(DEMTP(IJ)+DST*RADP(IJ))
         A(IJ,NPC)= A1*DABNM(IJ)+
     *              A2*(DEMNM(IJ)+(DSN+SIGE)*RADM(IJ))+
     *              GN*RTNA
         B(IJ,NPC)= B1*DABN0(IJ)+
     *              B2*(DEMN0(IJ)+(DSN+SIGE)*RAD0(IJ))+
     *              GN*RTN
         C(IJ,NPC)= C1*DABNP(IJ)+
     *              C2*(DEMNP(IJ)+(DSN+SIGE)*RADP(IJ))+
     *              GN*RTNC
         DO 70 II=1,NLVEXP
            A(IJ,NSE+II)=A(IJ,NSE+II)+
     *                   A1*DRCHM(II,IJ)+A2*DRETM(II,IJ)
            B(IJ,NSE+II)=B(IJ,NSE+II)+
     *                   B1*DRCH0(II,IJ)+B2*DRET0(II,IJ)
            C(IJ,NSE+II)=C(IJ,NSE+II)+
     *                   C1*DRCHP(II,IJ)+C2*DRETP(II,IJ)
   70    CONTINUE
         A(IJ,NFREQE)=0.
         A(IJ,IJ)=FKM(IJ)/DTAUM/DTAU0-AS*(UN-CHIELM/ABSOM(IJ))
         B(IJ,NFREQE)=0.
         B(IJ,IJ)=-FK0(IJ)/DTAU0*(UN/DTAUP+UN/DTAUM)-
     *             BS*(UN-CHIEL0/ABSO0(IJ))
         C(IJ,NFREQE)=0.
         C(IJ,IJ)=FKP(IJ)/DTAUP/DTAU0-CS*(UN-CHIELP/ABSOP(IJ))
C
C *** the IJ-th element of the rhs vector
C
         VECL(IJ)=BET1+BET2+BS*(RAD0(IJ)-S0)
  130 CONTINUE
      isplin=ispl
      RETURN
C
C   --------------------------------------
C   For ID=ND  -  lower boundary condition
C   --------------------------------------
C
  150 T=TEMP(ID)
      TM=TEMP(ID-1)
      HKT=HK/T
      HKTM=HK/TM
C
C     1. the case NDRE.le.ND, ie. radiative equilibrium equation at this
C        depth is treated as differential equation.
C
C     2. the case NDRE > ND, ie. radiative equilibrium equation at this
C        depth is treated as integral equation.
C        Boundary condition of the transfer equation is then complex,
C        because it has to incorporate the constraint of the total flux
C        equal to SIG4P*Teff**4 (integral form of the radiative
C        equation does not fix the total flux)
C
C        Details of this approach are presented in Mihalas, Heasley, Auer
C        NCAR-TN/STR-104 (1975).
C
C        This option is presented here for the sake of continuity with
C        previous work; otherwise the first option (NDRE.le.ND) is
C        recommended due to its better numerical properties
C
C     Auxiliary quantities for the second option
C
      IF(ID.LT.NDRE.AND.INRE.NE.0) THEN
      SUMB=0.
      SUMF=0.
      SUMDBT=0.
      SUMDFT=0.
      SUMDFN=0.
      DO 210 I=1,NLVEXP
  210    SUMDCH(I)=0.
C
      DO 220 IJ=1,NFREQE
         IJT=IJFR(IJ)
         FR=FREQ(IJT)
         WD=WDEP0(IJ)
         X=HKT*FR
         EX=EXP(X)
         PLAN=BN*(FR*1.D-15)**3/(EX-UN)
         DPLAN=PLAN*X/T/(UN-UN/EX)*WD
         SUMB=SUMB+(PLAN-RAD0(IJ))*WD
         SUMDBT=SUMDBT+DPLAN
         FI=DPLAN/ABSO0(IJ)
         SUMF=SUMF+FI
         U=(EX+UN)/(EX-UN)*X
         SUMDFT=SUMDFT+((U-TWO)/T-DABT0(IJ)/ABSO0(IJ))*FI
         FI=FI/ABSO0(IJ)
         SUMDFN=SUMDFN+FI*DABN0(IJ)
         DO 220 I=1,NLVEXP
            SUMDCH(I)=SUMDCH(I)+FI*DRCH0(I,IJ)
  220 CONTINUE
      FL=SIG4P*TEFF**4
      ZZ=(FL-HALF*SUMB)/SUMF
      END IF
C
C     auxiliary quantites for both options
C
      DO 300 IJ=1,NFREQE
         IJT=IJFR(IJ)
         CHIELM=SCATM(IJ)
         CHIEL0=SCAT0(IJ)
         OMEGM=ABSOM(IJ)/DENS(ID-1)
         OMEG0=ABSO0(IJ)/DENS(ID)
         DZM=OMEG0+OMEGM
         DTAUM=DZM*DDM
         FRD=FK0(IJ)*RAD0(IJ)-FKM(IJ)*RADM(IJ)
         GAM1=FRD/DTAUM
         A1=GAM1/DZM
         AS=0.
         BS=0.
         A2=0.
         B2=0.
         A3=0.
         B3=0.
         ALF2=0.
         BET2=0.
         GAM2=0.
C
C      Spline collocation and/or Hermitian method (ISPLIN=1 or 2) -
C      both give the same expression for the boundary conditions
C
C      Commented out here for consistency with RTE where the lower
C      boudary condition is not taken in the spline form
C
C        IF(ISPLIN.GT.0) THEN
C           BS=DTAUM*THIRD
C           AS=HALF *BS
C           S0=(EMIS0(IJ)+CHIEL0*RAD0(IJ))/ABSO0(IJ)
C           SM=(EMISM(IJ)+RADM(IJ)*CHIELM)/ABSOM(IJ)
C           ALF2=AS*(RADM(IJ)-SM)
C           GAM2=BS*(RAD0(IJ)-S0)
C           BET2=ALF2+GAM2
C           X1=BET2/DZM
C           A1=A1-X1
C           A2=AS/ABSOM(IJ)
C           B2=BS/ABSO0(IJ)
C           A3=A2*SM
C           B3=B2*S0
C        END IF
C
C        second-order boundary condition
C
         IF(IBC.GT.0) THEN
            BS=DTAUM*HALF
            S0=(EMIS0(IJ)+CHIEL0*RAD0(IJ))/ABSO0(IJ)
            GAM2=BS*(RAD0(IJ)-S0)
            BET2=GAM2
            X1=BET2/DZM
            A1=A1-X1
            B2=BS/ABSO0(IJ)
            B3=B2*S0
         END IF
C
C     auxiliary parameters
C
         FR=FREQ(IJT)
         FR15=FR*1.D-15
         X=HKT*FR
         EX=EXP(X)
         XM=HKTM*FR
         EXM=EXP(XM)
         PLAN=BN*FR15*FR15*FR15/(EX-UN)
         IF(INRE.EQ.0.OR.ID.GE.NDRE) THEN
            PLANM=BN*FR15*FR15*FR15/(EXM-UN)
            GAM3=(PLAN-PLANM)/DTAUM*THIRD
            A1=A1-GAM3/DZM
            GAM1=GAM1-GAM3
         END IF
         C1=A1
         A1=C1/DENS(ID-1)
         B1=C1/DENS(ID)
C
C *** elements of the IJ-th row of matrices A and B
C
         RTNA=OMEGM*WMM(ID-1)*A1
         A(IJ,NHE)=-GN*RTNA
         A(IJ,NMP)=-GP*RTNA
         A1=A1-A3
C
         RTN=OMEG0*WMM(ID)*B1
         B(IJ,NHE)=-GN*RTN
         B(IJ,NMP)=-GP*RTN
         B1=B1-B3
C
         IF(INRE.EQ.0.OR.ID.GE.NDRE) THEN
            DPLANM=PLANM*XM/TM/(UN-UN/EXM)
            A(IJ,NRE)=A1*DABTM(IJ)+A2*(DEMTM(IJ)+DST*RADM(IJ))-
     *              DPLANM/DTAUM*THIRD
            A(IJ,NPC)=A1*DABNM(IJ)+
     *              A2*(DEMNM(IJ)+(DSN+SIGE)*RADM(IJ))+
     *              GN*RTNA
            BB=HALF+THIRD/DTAUM
            DPLAN=PLAN*X/T/(UN-UN/EX)
            B(IJ,NRE)= B1*DABT0(IJ)+B2*(DEMT0(IJ)+DST*RAD0(IJ))+
     *                 BB*DPLAN
            B(IJ,NPC)= B1*DABN0(IJ)+
     *                 B2*(DEMN0(IJ)+(DSN+SIGE)*RAD0(IJ))+
     *                 GN*RTN
            DO 260 II=1,NLVEXP
               A(IJ,NSE+II)=A(IJ,NSE+II)+
     *                      A1*DRCHM(II,IJ)+A2*DRETM(II,IJ)
               B(IJ,NSE+II)=B(IJ,NSE+II)+
     *                      B1*DRCH0(II,IJ)+B2*DRET0(II,IJ)
  260       CONTINUE
            A(IJ,NFREQE)=0.
            A(IJ,IJ)=FKM(IJ)/DTAUM-AS*(UN-CHIELM/ABSOM(IJ))
            B(IJ,NFREQE)=0.
          ELSE
C        
C     The second option for the lower boudary condition
C       
            DPLAN=PLAN*X/T/(UN-UN/EX)
            FI=DPLAN/ABSO0(IJ)
            X1=FI*ZZ
            B1=B1-X1/ABSO0(IJ)
            GAM1=GAM1-X1
            A1=X1/SUMF
            B(IJ,NPC)= B1*DABN0(IJ)+
     *                 B2*(DEMN0(IJ)+(DSN+SIGE)*RAD0(IJ))+
     *                 GN*RTN+A1*SUMDFN
            DO 280 II=1,NLVEXP
               B(IJ,NSE+II)=B1*DRCH0(II,IJ)+B2*DRET0(II,IJ)+
     *                      A1*SUMDCH(II)
  280       CONTINUE
            U=(EX+UN)/(EX-UN)*X
            A3=((U-TWO)/T-SUMDBT/(TWO*FL-SUMB))*X1
            B(IJ,NRE)= B1*DABT0(IJ)+B2*(DEMT0(IJ)+DST*RAD0(IJ))+
     *                 A3+HALF*DPLAN-A1*SUMDFT
            A1=HALF *FI/SUMF
            DO 290 IJ1=1,NFREQE
  290          B(IJ,IJ1)=A1*WDEP0(IJ1)
         END IF
C
C *** the IJ-th element of the rhs vector
C
         IF(IBC.EQ.0) THEN
            B(IJ,IJ)=B(IJ,IJ)-FK0(IJ)/DTAUM-
     *               BS*(UN-CHIEL0/ABSO0(IJ))-HALF
            VECL(IJ)=GAM1+BET2-HALF*(PLAN-RAD0(IJ))
          ELSE
            B(IJ,IJ)=B(IJ,IJ)-FK0(IJ)/DTAUM-
     *               BS*(UN-CHIEL0/ABSO0(IJ))-FHD(IJT)
            VECL(IJ)=GAM1+BET2-HALF*PLAN+FHD(IJT)*RAD0(IJ)
         END IF
  300 CONTINUE
      isplin=ispl
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE BHE(ID)
C     ==================
C
C     The part of matrices A and B corresponding to the hydrostatic
C     equilibrium equation,
C     i.e. the (NFREQE+INHE)-th row;
C     and, if desired (INMP > 0), the part corresponding to the
C     definition equation for the fictitious massive particle density,
C     ie. the (NFREQE+INMP)-th row.
C
C     Input:  ID - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (PCK=4.19168946D-10)
C
      NHE=NFREQE+INHE
      NRE=NFREQE+INRE
      NPC=NFREQE+INPC
      NSE=NFREQE+INSE-1
C
C ***********  Linearized equation for the fictitious massive particle
C              density
C
      GP=0.
      GN=UN
      IF(INMP.GT.0) THEN
         NMP=NFREQE+INMP
         GP=UN/WMY(ID)
         GN=0.
         B(NMP,NMP)=-UN
         B(NMP,NHE)=WMY(ID)
         IF(INPC.GT.0) B(NMP,NPC)=-WMY(ID)
      END IF
C
C ***********  Linearized hydrostatic equilibrium
C
      HEXT=0.
      HEXN=0.
      GRD=0.
      FLUXW=0.
      IF(ID.GT.1) GO TO 50
C
C *** Upper boundary condition (ID=1)
C     Basically, linearized eq. (7-10) of Mihalas (1978)
C
      DO 1 I=1,NLVEXP
         HEX(I)=0.
    1 CONTINUE
C
      X1=PCK/DENS(ID)
      IF(NFREQE.GT.0) THEN
      DO 20 IJ=1,NFREQE
         IJT=IJFR(IJ)
         IF(LSKIP(IJT)) GO TO 20 
         FLUXW=(WDEP0(IJ)+WDC0(IJ))*FH(IJT)*RAD0(IJ)
         GRD=GRD+FLUXW*ABSO0(IJ)
         HEXN=HEXN+FLUXW*DABN0(IJ)
         HEXT=HEXT+FLUXW*DABT0(IJ)
         DO 10 I=1,NLVEXP
   10       HEX(I)=HEX(I)+FLUXW*DRCH0(I,IJ)
C
C
C     Columns corresponding to mean intensities
C
         B(NHE,IJ)=X1*WDEP0(IJ)*FH(IJT)*ABSO0(IJ)
   20 CONTINUE
      END IF
C
      RTN=X1*WMM(ID)/DENS(ID)*(GRD+FPRD(ID))
      VT0=HALF*VTURB(ID)*VTURB(ID)/DM(ID)*WMM(ID)
C
C     columns corresponding to total particle density, fictitious
C     massive particle density, temperature, and electron density,
C     respectively
C
      B(NHE,NHE)=BOLK*TEMP(ID)/DM(ID)-GN*(RTN-VT0)
      IF(INMP.GT.0) B(NHE,NFREQE+INMP)=GP*(VT0-RTN)
      IF(INRE.GT.0) THEN
         B(NHE,NRE)=BOLK*PSI0(NHE)/DM(1)+X1*(HEXT+HEIT(ID))
         C(NHE,NRE)=X1*HEITP(ID)
      END IF
      IF(INPC.GT.0) THEN
         B(NHE,NPC)=X1*(HEXN+HEIN(ID))+GN*(RTN-VT0)
         C(NHE,NPC)=X1*HEINP(ID)
      END IF
C
C     Columns corresponding to populations
C
      DO 30 II=1,NLVEXP
          B(NHE,NSE+II)=B(NHE,NSE+II)+X1*(HEX(II)+HEIP(II,ID))
          C(NHE,NSE+II)=C(NHE,NSE+II)+X1*HEIPP(II,ID)
   30 CONTINUE
C
C     The rhs vector also accounts for the total radiation pressure in
C     the fixed-option transitions (array FPRD, generated by FIXLIN)
C
      VECL(NHE)=GRAV-BOLK*TEMP(ID)*PSI0(NHE)/DM(ID)-
     *          X1*(GRD+FPRD(ID))-VT0/WMM(ID)*DENS(ID)
      RETURN
C
C *** Normal depth point (ID > 1)
C
C     Columns (for matrices A and B) corresponding to mean intensities
C
   50 IF(NFREQE.GT.0) THEN
      DO 60 IJ=1,NFREQE
         IF(LSKIP(IJFR(IJ))) GO TO 60
         WDC=WDC0(IJ)
         GRD=GRD+(FK0(IJ)*RAD0(IJ)-FKM(IJ)*RADM(IJ))*(WDEP0(IJ)+WDC)
         A(NHE,IJ)=-PCK*(WDEP0(IJ)+WDC)*FKM(IJ)
         B(NHE,IJ)=PCK*(WDEP0(IJ)+WDC)*FK0(IJ)
   60 CONTINUE
      END IF
C
      VT0=HALF*VTURB(ID)*VTURB(ID)*WMM(ID)
      VTM=HALF*VTURB(ID-1)*VTURB(ID-1)*WMM(ID-1)
C
C     columns corresponding to total particle density
C
      A(NHE,NHE)=-BOLK*TEMP(ID-1)-GN*VTM
      B(NHE,NHE)=BOLK*TEMP(ID)+GN*VT0
C
C     columns corresponding to temperature
C
      IF(INRE.GT.0) THEN
         A(NHE,NRE)=-BOLK*PSIM(NHE)+PCK*HEITM(ID)
         B(NHE,NRE)=BOLK*PSI0(NHE)+PCK*HEIT(ID)
         C(NHE,NRE)=PCK*HEITP(ID)
      END IF
C
C     columns corresponding to electron density
C
      IF(INPC.GT.0) THEN
         A(NHE,NPC)=GN*VTM+PCK*HEINM(ID)
         B(NHE,NPC)=-GN*VT0+PCK*HEIN(ID)
         C(NHE,NPC)=PCK*HEINP(ID)
      END IF
C
C     columns corresponding to NMP
C
      IF(INMP.GT.0) THEN
         A(NHE,NFREQE+INMP)=-GP*VTM
         B(NHE,NFREQE+INMP)=GP*VT0
      END IF
C
C     columns corresponding to populations
C
      DO 70 II=1,NLVEXP
         A(NHE,NSE+II)=A(NHE,NSE+II)+PCK*HEIPM(II,ID)
         B(NHE,NSE+II)=B(NHE,NSE+II)+PCK*HEIP(II,ID)
         C(NHE,NSE+II)=C(NHE,NSE+II)+PCK*HEIPP(II,ID)
   70 CONTINUE
C
C     the rhs vector
C     again, which accounts for the total radiation pressure in the
C     fixed-option transitions (array FPRD)
C
      VECL(NHE)=GRAV*(DM(ID)-DM(ID-1))-
     *          BOLK*(TEMP(ID)*PSI0(NHE)-TEMP(ID-1)*PSIM(NHE))-
     *          PCK*(GRD+FPRD(ID))-
     *          VT0/WMM(ID)*DENS(ID)+VTM/WMM(ID-1)*DENS(ID-1)
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE BRE(ID)
C     ==================
C
C     The part of matrices A and B corresponding to the radiative
C     equilibrium equation
C     i.e. the (NFREQE+INRE)-th row
C
C     Input:  ID  - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      DIMENSION REXB(MLEVEL)
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/RESDER/RSAT,RSBT,RSAN,RSBN,RSAX(MLEVEL),RSBX(MLEVEL)
      COMMON/OPACAD/ABAD,EMAD,SCAD,DAT,DAN,DET,DEN,DST,DSN,DDN(MLEVEL)
      EQUIVALENCE (REX(1),REXB(1))
C
      NRE=NFREQE+INRE
      NPC=NFREQE+INPC
      NSE=NFREQE+INSE-1
C
C     the rhs vector accounts for total net cooling in ALI
C     transitions (FCOOL)
C 
      VECL(NRE)=FCOOL(ID)
      if(reint(id).le.0) go to 100
C
C *********     integral equation part of the radiative
C     equilibrium equation
C
      BREPC=0.
      DO 10 I=1,NLVEXP
   10    REXB(I)=0.
      IF(NFREQE.GT.0) THEN
      DO 30 IJ=1,NFREQE
         BREPC=BREPC+((DABN0(IJ)-SIGE)*RAD0(IJ)-
     *         DEMN0(IJ))*WDEP0(IJ)
         DO 20 I=1,NLVEXP
            REXB(I)=REXB(I)+(DRCH0(I,IJ)*RAD0(IJ)-
     *             DRET0(I,IJ))*WDEP0(IJ)
   20    CONTINUE
         B(NRE,NRE)=B(NRE,NRE)+(DABT0(IJ)*RAD0(IJ)-
     *              DEMT0(IJ))*WDEP0(IJ)*reint(id)
         HEAT=ABSO0(IJ)-SCAT0(IJ)
         B(NRE,IJ)=WDEP0(IJ)*HEAT*reint(id)
         VECL(NRE)=VECL(NRE)-(HEAT*RAD0(IJ)-EMIS0(IJ))*WDEP0(IJ)*
     *             reint(id)
   30 CONTINUE
      END IF
C
C     corrections for ALI frequency points
C
      B(NRE,NRE)=B(NRE,NRE)+REIT(ID)*reint(id)
      IF(INPC.GT.0) B(NRE,NPC)=(BREPC+REIN(ID))*reint(id)
      IF(IFALI.GT.5) THEN
      A(NRE,NRE)=AREIT(ID)*reint(id)
      IF(INPC.GT.0) A(NRE,NPC)=AREIN(ID)*reint(id)
      C(NRE,NRE)=CREIT(ID)*reint(id)
      IF(INPC.GT.0) C(NRE,NPC)=CREIN(ID)*reint(id)
      END IF
C
      DO 40 II=1,NLVEXP
         B(NRE,NSE+II)=B(NRE,NSE+II)+(REXB(II)+REIP(II,ID))*reint(id)
   40 CONTINUE
      IF(IFALI.GT.5.AND.ID.GT.1) THEN
      DO 42 II=1,NLVEXP
         A(NRE,NSE+II)=A(NRE,NSE+II)+AREIP(II,ID)*reint(id)
   42 CONTINUE
      END IF
      IF(IFALI.GT.5.AND.ID.LT.ND) THEN
      DO 44 II=1,NLVEXP
         C(NRE,NSE+II)=C(NRE,NSE+II)+CREIP(II,ID)*reint(id)
   44 CONTINUE
      END IF
C
C *********    differential equation part of the
C     radiative equilibrium equation
C
  100 CONTINUE
      if(redif(id).eq.0) return
C
      VECL(NRE)=VECL(NRE)+SIG4P*TEFF**4*redif(id)
      if(id.eq.1) go to 200
C
      DDM=(DM(ID)-DM(ID-1))*HALF 
      AREN=0.
      BREN=0.
      AREPC=0.
      BREPC=0.
C
      GP=0.
      GN=UN
      IF(INMP.GT.0) THEN
         GP=UN/WMY(ID)
         GN=0.
      END IF
C
      DO 110 I=1,NLVEXP
         REXB(I)=0.
         REXA(I)=0.
  110 CONTINUE
C
      IF(NFREQE.GT.0) THEN
      DO 130 IJ=1,NFREQE
         OMEG0=ABSO0(IJ)*DENS1(ID)
         OMEGM=ABSOM(IJ)*DENS1(ID-1)
         DTAUM=(OMEG0+OMEGM)*DDM
         FRD=FK0(IJ)*RAD0(IJ)-FKM(IJ)*RADM(IJ)
         GAMR=FRD/DTAUM
         A1=GAMR/(OMEG0+OMEGM)
         A3R=A1*DENS1(ID-1)*WDEP0(IJ)
         B3R=A1*DENS1(ID)*WDEP0(IJ)
C
C     Corresponding elements of matrix A
C
         A(NRE,IJ)=-WDEP0(IJ)*FKM(IJ)/DTAUM*redif(id)
         RTR=OMEGM*WMM(ID-1)*A3R
         AREN=AREN+RTR*GN
         AREPC=AREPC-A3R*DABNM(IJ)-RTR*GN
         IF(INMP.NE.0) A(NRE,NFREQE+INMP)=A(NRE,NFREQE+INMP)+RTR*GP*
     *                                    redif(id)
         A(NRE,NRE)=A(NRE,NRE)-A3R*DABTM(IJ)*redif(id)
C
C     Corresponding elements of matrix B
C     Columns corresponding to mean intensities
C
         B(NRE,IJ)=B(NRE,IJ)+WDEP0(IJ)*FK0(IJ)/DTAUM*redif(id)
         RTR=OMEG0*WMM(ID)*B3R
         BREN=BREN+RTR*GN
         BREPC=BREPC-B3R*DABN0(IJ)-RTR*GN
         IF(INMP.NE.0) B(NRE,NFREQE+INMP)=B(NRE,NFREQE+INMP)+
     *                                    (RTR+REDX(ID))*GP*redif(id)
C
C     Column corresponding to temperature
C
         B(NRE,NRE)=B(NRE,NRE)-B3R*DABT0(IJ)*redif(id)
C
C     auxiliary vectors for columns corresponding to populations
C
      DO 120 I=1,NLVEXP
         REXA(I)=REXA(I)-A3R*DRCHM(I,IJ)
         REXB(I)=REXB(I)-B3R*DRCH0(I,IJ)
  120 CONTINUE
C
C     The rhs vector
C
         VECL(NRE)=VECL(NRE)-WDEP0(IJ)*GAMR*redif(id)
  130 CONTINUE
      END IF
C
C     Column corresponding to N (total particle number density)
C     for both A and B matrices
C
      IF(INHE.NE.0) THEN
         A(NRE,NFREQE+INHE)=(AREN+REDXM(ID))*redif(id)
         B(NRE,NFREQE+INHE)=B(NRE,NFREQE+INHE)+(BREN+REDX(ID))*redif(id)
      END IF
C
C     Column corresponding to temperature
C
      A(NRE,NRE)=A(NRE,NRE)+REDTM(ID)*REDIF(ID)
      B(NRE,NRE)=B(NRE,NRE)+REDT(ID)*REDIF(ID)
      C(NRE,NRE)=C(NRE,NRE)+REDTP(ID)*REDIF(ID)
C
C     Column corresponding to electron density (for matrices A and B)
C
      IF(INPC.NE.0) THEN
         A(NRE,NPC)=A(NRE,NPC)+(AREPC+REDNM(ID)-REDXM(ID))*redif(id)
         B(NRE,NPC)=B(NRE,NPC)+(BREPC+REDN(ID)-REDX(ID))*redif(id)
         C(NRE,NPC)=C(NRE,NPC)+REDNP(ID)*redif(id)
      END IF
C
C     Columns corresponding to populations (for matrices A and B)
C     Note that auxiliary arrays REXA and REX, which contain derivatives
C     of the absorption and emission coefficients wrt populations, have
C     been generated by BRTE
C
      DO 140 II=1,NLVEXP
         A(NRE,NSE+II)=A(NRE,NSE+II)+(REXA(II)+REDPM(II,ID))*redif(id)
         B(NRE,NSE+II)=B(NRE,NSE+II)+(REXB(II)+REDP(II,ID))*redif(id)
         C(NRE,NSE+II)=C(NRE,NSE+II)+REDPP(II,ID)*redif(id)
  140 CONTINUE
      RETURN
c     
C     upper boundary condition for the differential form
c
  200 CONTINUE
C
C     Columns corresponding to mean intensities; rhs vector
C
      IF(NFREQE.GT.0) THEN
      DO 230 IJ=1,NFREQE
         IJT=IJFR(IJ)
         WF=WDEP0(IJ)*FH(IJT)*REDIF(ID)
         B(NRE,IJ)=B(NRE,IJ)+WF
         VECL(NRE)=VECL(NRE)-WF*RAD0(IJ)
  230 CONTINUE
      END IF
C
C     Column corresponding to temperature
C
      B(NRE,NRE)=B(NRE,NRE)+REDT(ID)*REDIF(ID)
C
C     Column corresponding to electron density 
C
      IF(INPC.NE.0) THEN
         B(NRE,NPC)=B(NRE,NPC)+REDN(ID)*redif(id)
      END IF
C
C     Columns corresponding to populations
C
      DO 240 II=1,NLVEXP
            B(NRE,NSE+II)=B(NRE,NSE+II)+REDP(II,ID)*redif(id)
  240 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE BPOP(ID)
C     ===================
C
C     The part of matrix B corresponding to the statistical
C     equilibrium equations
C     i.e. the (NFREQE+INSE)-th thru (NFREQE+INSE+NLVEXP-1)-th rows;
C     and to the charge conservation equation, ie. the (NFREQE+INPC)-th
C     row
C
C     The formalism is similar to that described in Mihalas, Stellar
C     Atmospheres, 1978, pp. 143-145
C
C     Input: ID - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      COMMON/BPOCOM/ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       ATT(MLEVEL),ANN(MLEVEL)
C
      DO 20 I=1,NLVEXP
         ATT(I)=0.
         ANN(I)=0.
   20 CONTINUE
C
      CALL RATMAT(ID,IIEXP,0,ESEMAT,BESE)
      IF(IFPOPR.LE.3) CALL MATINV(ESEMAT,NLVEXP,MLEVEL)
C
C     Split BPOP in separate subroutines
C
      IF(.NOT.LTE.AND.IBPOPE.GT.0.AND.ID.LT.IDLTE) THEN
         IF(IOVER.EQ.0) THEN
            CALL BPOPE(ID)
          ELSE
            CALL BPOPE1(ID)
         END IF
      END IF
      IF(.NOT.LTE.AND.IRDER.GT.1.AND.ID.LT.IDLTE) THEN
          CALL BPOPF(ID)
      END IF
      CALL BPOPT(ID)
      IF(INPC.GT.0) CALL BPOPC(ID)
C
C     reset matrix elements for "small" populations
C
      DO 40 I=1,NLVEXP
         IF(IPZERO(INDLEV(I),ID).EQ.0) GO TO 40
         DO J=1,NLVEXP
            B(NFREQE+INSE-1+I,NFREQE+INSE-1+J)=0.
         END DO
         B(NFREQE+INSE-1+I,NFREQE+INSE-1+I)=1.
         VECL(NFREQE+INSE-1+I)=0.
   40 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE BPOPE(ID)
C     ====================
C
C     the part of B-matrix corresponding to the population rows and
C     the explicit frequency columns
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ARRAY1.FOR'
      DIMENSION AJIJ(MFREX,MLVEXP),EHKE(MFREX)
      COMMON/BPOCOM/ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       ATT(MLEVEL),ANN(MLEVEL)
C
      IF(NFREQE.LE.0) RETURN
      NSE=NFREQE+INSE-1
      DO I=1,NLVEXP
         DO IJE=1,NFREQE
            AJIJ(IJE,I)=0.
         END DO
      END DO
      HKT=HK/TEMP(ID)
      DO IJE=1,NFREQE
         EHKE(IJE)=EXP(-HKT1(ID)*FREQ(IJFR(IJE)))
      END DO
C
C     ---------------------
C     Continuum transitions
C     ---------------------
C
      DO 20 IJ=1,NFREQC
         IF(IJEX(IJ).LE.0) GO TO 20
         IJE=IJEX(IJ)
         FR=FREQ(IJ)
         FRINV=UN/FR
         FR3INV=FRINV*FRINV*FRINV
         DO 10 IBF=1,NTRANC
            ITR=ITRBF(IBF)
            SG=CROSS(IBF,IJ)
            IF(SG.LE.0.) GO TO 10
            I=ILOW(ITR)
            IF(ILTION(IEL(I)).GE.1.OR.IIFIX(IATM(I)).EQ.1) GO TO 10
            ICDW=MCDW(ITR)
            IMER=IMRG(I)
            II=IIEXP(I)
            J=IUP(ITR)
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 10
            JJ=IIEXP(J)
            NREFI=NREF(IATM(I))
            IF(IFWOP(I).GE.0) THEN
              IF(ICDW.GE.1) THEN
                 IZZ=IZ(IEL(I))
                 CALL DWNFR1(FR,FR0(ITR),ID,IZZ,DW1)
                 SG=SG*DW1
              END IF
              W0=W0EC(IJ)
              IF(IFR1(ITR).EQ.IJ) W0=W0EC(IJ)-WCHCE(IJ)
              SGW0=SG*W0
            ELSE
              CALL SGMER1(FRINV,FR3INV,IMER,ID,SGME1)
              SG=SGME1
              IF(SG.LE.0.) GO TO 10
              W0=W0E(IJ)
              IF(IFR1(ITR).EQ.IJ)
     *           W0=W0E(IJ)*(W(IJ)-WCH(IJ))/W(IJ)
              SGW0=SG*W0
            ENDIF
            APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EHKE(IJE))*SGW0 
            IF(II.GT.0.AND.I.NE.NREFI.AND.ILTLEV(I).LE.0) 
     *         AJIJ(IJE,II)=AJIJ(IJE,II)+APFR  
            IF(JJ.GT.0.AND.J.NE.NREFI.AND.ILTLEV(J).LE.0) 
     *         AJIJ(IJE,JJ)=AJIJ(IJE,JJ)-APFR  
   10    CONTINUE
   20 CONTINUE
C            
C     ----------------
C     Line transitions 
C     ----------------
C             
      DO 200 ITR=1,NTRANS
         IF(LINEXP(ITR)) GO TO 200
         IF(.NOT.LEXP(ITR)) GO TO 200
         I=ILOW(ITR)
         IF(ILTION(IEL(I)).GE.1.OR.IIFIX(IATM(I)).EQ.1) GO TO 200
         J=IUP(ITR)
         II=IIEXP(I)
         JJ=IIEXP(J)
         IF(II.LE.0.AND.JJ.LE.0) GO TO 200
         IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 200
         NREFI=NREF(IATM(I))
         I0=IFR0(ITR)
         I1=IFR1(ITR)
         NFL=I1-I0+1
         DO 110 IJ=I0,I1
            IF(IJEX(IJ).LE.0) GO TO 110
            IJL=IJ-I0+1
            DO ICDW=1,NCDW
               ITRC=ITRCDW(ICDW)
               IZZ=IZ(IEL(ILOW(ITRC)))
               CALL DWNFR1(FREQ(IJ),FR0(ITRC),ID,IZZ,DW1)
               DWFL(IJL,ID,ICDW)=DW1
            ENDDO
  110    CONTINUE
C
         POPIL=ABTRA(ITR,ID)
         POPJL=EMTRA(ITR,ID)
         POPIJ=POPIL-POPJL
         DO 130 IJ=I0,I1
            IF(IJEX(IJ).LE.0) GO TO 130
            IJE=IJEX(IJ)
            IJL=IJ-I0+1
            SGW=PRFLIN(IJ,ID)*W(IJ)
            APFR=POPIJ*SGW*XPIF(ITR)
            IF(II.GT.0.AND.I.NE.NREFI.AND.ILTLEV(I).LE.0) 
     *         AJIJ(IJE,II)=AJIJ(IJE,II)+APFR  
            IF(JJ.GT.0.AND.J.NE.NREFI.AND.ILTLEV(J).LE.0) 
     *         AJIJ(IJE,JJ)=AJIJ(IJE,JJ)-APFR  
  130    CONTINUE
c                
c        correction to the continuum rates
c                
         DO 180 IBF=1,NTRANC
            ITRC=ITRBF(IBF)
c           IF(FRQMX(ITR).LT.FREQ(IFR1(ITRC))) GO TO 180
            I=ILOW(ITRC)
            IF(IFWOP(I).LT.0) GO TO 180
            IF(ILTION(IEL(I)).GE.1.OR.IIFIX(IATM(I)).EQ.1) GO TO 180
            J=IUP(ITRC)
            II=IIEXP(I)
            JJ=IIEXP(J)
            IF(II.LE.0.AND.JJ.LE.0) GO TO 180
            NREFI=NREF(IATM(I))
            DO 160 IJ=I0,I1
               IF(IJEX(IJ).LE.0) GO TO 160
               IJE=IJEX(IJ)
               IJL=IJ-I0+1
               SG=CROSS(IBF,IJ)
               IF(SG.LE.0.) GO TO 160
               IF(MCDW(ITRC).GE.1) SG=SG*DWFL(IJL,ID,MCDW(ITRC))
               SGW0=SG*W0E(IJ)
               APFR=(ABTRA(ITRC,ID)-EMTRA(ITRC,ID)*EHKE(IJE))*SGW0 
               IF(II.GT.0.AND.I.NE.NREFI.AND.ILTLEV(I).LE.0) 
     *            AJIJ(IJE,II)=AJIJ(IJE,II)+APFR  
               IF(JJ.GT.0.AND.J.NE.NREFI.AND.ILTLEV(J).LE.0) 
     *            AJIJ(IJE,JJ)=AJIJ(IJE,JJ)-APFR  
  160       CONTINUE
  180    CONTINUE
  200 CONTINUE
C
C     elements of the B-matrix
C
      DO I=1,NLVEXP
         DO IJE=1,NFREQE
            IF(IFPOPR.LE.3) THEN
               SUM=0.
               DO J=1,NLVEXP
                  SUM=SUM-ESEMAT(I,J)*AJIJ(IJE,J)
               END DO    
             ELSE
               SUM=AJIJ(IJE,I)
            END IF
            B(NSE+I,IJE)=SUM*CRSW(ID)
         END DO
      END DO
      RETURN
      END
C
C
C
C     ****************************************************************
C
C

      SUBROUTINE BPOPE1(ID)
C     =====================
C
C     the part of B-matrix corresponding to the population rows and
C     the explicit frequency columns
C     -- a variant for the full overlap case
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      INCLUDE 'ARRAY1.FOR'
      DIMENSION AJIJ(MFREX,MLVEXP),EHKE(MFREX)
      COMMON/BPOCOM/ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       ATT(MLEVEL),ANN(MLEVEL)
C
      IF(NFREQE.LE.0) RETURN
      NSE=NFREQE+INSE-1
      DO I=1,NLVEXP
         DO IJE=1,NFREQE
            AJIJ(IJE,I)=0.
         END DO
      END DO
      HKT=HK/TEMP(ID)
      DO IJE=1,NFREQE
         EHKE(IJE)=EXP(-HKT1(ID)*FREQ(IJFR(IJE)))
      END DO
C
      DO 100 IJ=1,NFREQ
         IF(IJEX(IJ).LE.0) GO TO 100
         IF(IJX(IJ).EQ.-1) GOTO 100
         IJE=IJEX(IJ)
         FR=FREQ(IJ)
         FRINV=UN/FR
         FR3INV=FRINV*FRINV*FRINV
C
C        ---------------------
C        Continuum transitions
C        ---------------------
C
         DO 10 IBF=1,NTRANC
            ITR=ITRBF(IBF)
            SG=CROSS(IBF,IJ)
            IF(SG.LE.0.) GO TO 10
            I=ILOW(ITR)
            IF(ILTION(IEL(I)).GE.1.OR.IIFIX(IATM(I)).EQ.1) GO TO 10
            ICDW=MCDW(ITR)
            IMER=IMRG(I)
            II=IIEXP(I)
            J=IUP(ITR)
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 10
            JJ=IIEXP(J)
            NREFI=NREF(IATM(I))
            IF(IFWOP(I).GE.0) THEN
              IF(ICDW.GE.1) THEN
                 IZZ=IZ(IEL(I))
                 CALL DWNFR1(FR,FR0(ITR),ID,IZZ,DW1)
                 SG=SG*DW1
              END IF
            ELSE
              CALL SGMER1(FRINV,FR3INV,IMER,ID,SGME1)
              SG=SGME1
            ENDIF
            W0=W0E(IJ)
            SGW0=SG*W0
            APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EHKE(IJE))*SGW0 
            IF(II.GT.0.AND.I.NE.NREFI.AND.ILTLEV(I).LE.0) 
     *         AJIJ(IJE,II)=AJIJ(IJE,II)+APFR  
            IF(JJ.GT.0.AND.J.NE.NREFI.AND.ILTLEV(J).LE.0.
     *         and.iabs(imodl(i)).ne.4) 
     *         AJIJ(IJE,JJ)=AJIJ(IJE,JJ)-APFR  
   10    CONTINUE
C            
C        ----------------
C        Line transitions 
C        ----------------
C             
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            IF(LINEXP(ITR)) GO TO 20
            IF(.NOT.LEXP(ITR)) GO TO 20
            I=ILOW(ITR)
            IF(ILTION(IEL(I)).GE.1.OR.IIFIX(IATM(I)).EQ.1) GO TO 20
            J=IUP(ITR)
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 20
            II=IIEXP(I)
            JJ=IIEXP(J)
            IF(II.LE.0.AND.JJ.LE.0) GO TO 20
            NREFI=NREF(IATM(I))
            SGW=PRFLIN(IJ,ID)*W0E(IJ)
            APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EHKE(IJE))*SGW
            IF(II.GT.0.AND.I.NE.NREFI.AND.ILTLEV(I).LE.0) 
     *         AJIJ(IJE,II)=AJIJ(IJE,II)+APFR  
            IF(JJ.GT.0.AND.J.NE.NREFI.AND.ILTLEV(J).LE.0. 
     *         and.iabs(imodl(i)).ne.4)
     *         AJIJ(IJE,JJ)=AJIJ(IJE,JJ)-APFR  
         ENDIF
C
C        the "overlapping" lines at the given frequency
C
   20    IF(NLINES(IJ).LE.0) GO TO 100
         DO 50 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            IF(LINEXP(ITR)) GO TO 50
            I=ILOW(ITR)
            IF(ILTION(IEL(I)).GE.1.OR.IIFIX(IATM(I)).EQ.1) GO TO 50
            J=IUP(ITR)
            IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 50
            II=IIEXP(I)
            JJ=IIEXP(J)
            IF(II.LE.0.AND.JJ.LE.0) GO TO 50
            NREFI=NREF(IATM(I))
            IJ0=IFR0(ITR)
            DO 30 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 40
               END IF
   30       CONTINUE
   40       IJ1=IJ0-1
            X=W0E(IJ)/(FREQ(IJ1)-FREQ(IJ0))
            A1=(FR-FREQ(IJ0))*X
            A2=(FREQ(IJ1)-FR)*X
            SGW=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
            APFR=(ABTRA(ITR,ID)-EMTRA(ITR,ID)*EHKE(IJE))*SGW
            IF(II.GT.0.AND.I.NE.NREFI.AND.ILTLEV(I).LE.0) 
     *         AJIJ(IJE,II)=AJIJ(IJE,II)+APFR  
            IF(JJ.GT.0.AND.J.NE.NREFI.AND.ILTLEV(J).LE.0. 
     *         and.iabs(imodl(i)).ne.4)
     *         AJIJ(IJE,JJ)=AJIJ(IJE,JJ)-APFR  
   50    CONTINUE
  100 CONTINUE
C
C     elements of the B-matrix
C
      DO I=1,NLVEXP
         DO IJE=1,NFREQE
            IF(IFPOPR.LE.3) THEN
               SUM=0.
               DO J=1,NLVEXP
                  SUM=SUM-ESEMAT(I,J)*AJIJ(IJE,J)
               END DO    
             ELSE
               SUM=AJIJ(IJE,I)
            END IF
            B(NSE+I,IJE)=SUM*CRSW(ID)
         END DO
      END DO
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE BPOPF(ID)
C     =====================
C
C     the part of B-matrix corresponding to the population rows and
C     populations - i.e. derivatives of the ALI points intensities
C                   wrt. populations
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ODFPAR.FOR'
      COMMON/BPOCOM/ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       ATT(MLEVEL),ANN(MLEVEL)
C
      NSE=NFREQE+INSE-1
      NRE=NFREQE+INRE
      NPC=NFREQE+INPC
C
C     matrix B of complete linearization
C
      DO I=1,NLVEXP
         SUMT=0.
         SUMN=0.
         DO II=1,NLVEXP
            IF(IFPOPR.LE.3) THEN
            SUMT=SUMT-ESEMAT(I,II)*APT(II,ID)
            SUMN=SUMN-ESEMAT(I,II)*APN(II,ID)
            SUM=0.
            DO J=1,NLVEXP
               SUM=SUM-ESEMAT(I,J)*APP(II,J,ID)
            END DO
            ELSE
                SUM=APP(II,I,ID)
            END IF
            B(NSE+I,NSE+II)=B(NSE+I,NSE+II)+SUM
         END DO
         IF(IFPOPR.GT.3) THEN
            SUMT=APT(I,ID)
            SUMN=APN(I,ID)
         END IF
         IF(INRE.NE.0) B(NSE+I,NRE)=B(NSE+I,NRE)+SUMT
         IF(INPC.NE.0) B(NSE+I,NPC)=B(NSE+I,NPC)+SUMN
      END DO
      IF(CRSW(ID).NE.UN) THEN
         DO I=1,NLVEXP
            DO II=1,NLVEXP
               B(NSE+I,NSE+II)=B(NSE+I,NSE+II)*CRSW(ID)
            END DO
         END DO
      END IF
C
C     matrix A and C of complete linearization
C
      IF(IFALI.GE.6) THEN
      DO I=1,NLVEXP
         ASUMT=0.
         ASUMN=0.
         CSUMT=0.
         CSUMN=0.
         DO II=1,NLVEXP
            IF(IFPOPR.LE.3) THEN
            ASUMT=ASUMT-ESEMAT(I,II)*AAPT(II,ID)
            ASUMN=ASUMN-ESEMAT(I,II)*AAPN(II,ID)
            CSUMT=CSUMT-ESEMAT(I,II)*CAPT(II,ID)
            CSUMN=CSUMN-ESEMAT(I,II)*CAPN(II,ID)
            ASUM=0.
            CSUM=0.
            DO J=1,NLVEXP
               ASUM=ASUM-ESEMAT(I,J)*AAPP(II,J,ID)
               CSUM=CSUM-ESEMAT(I,J)*CAPP(II,J,ID)
            END DO
            ELSE
                ASUM=AAPP(II,I,ID)
                CSUM=CAPP(II,I,ID)
            END IF
            A(NSE+I,NSE+II)=ASUM
            C(NSE+I,NSE+II)=CSUM
         END DO
         IF(IFPOPR.GT.3) THEN
            ASUMT=AAPT(I,ID)
            ASUMN=AAPN(I,ID)
            CSUMT=CAPT(I,ID)
            CSUMN=CAPN(I,ID)
         END IF
         IF(INRE.NE.0) THEN
            A(NSE+I,NRE)=A(NSE+I,NRE)+ASUMT
            C(NSE+I,NRE)=C(NSE+I,NRE)+CSUMT
         END IF
         IF(INPC.NE.0) THEN
            A(NSE+I,NPC)=A(NSE+I,NPC)+ASUMN
            C(NSE+I,NPC)=C(NSE+I,NPC)+CSUMN
         END IF
      END DO
C
      IF(CRSW(ID).NE.UN) THEN
         DO I=1,NLVEXP
            DO II=1,NLVEXP
               A(NSE+I,NSE+II)=A(NSE+I,NSE+II)*CRSW(ID)
               C(NSE+I,NSE+II)=C(NSE+I,NSE+II)*CRSW(ID)
            END DO
         END DO
      END IF
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE BPOPT(ID)
C     ====================
C
C     the part of B-matrix corresponding to the population rows
C     and T and ne columns
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ODFPAR.FOR'
      PARAMETER (TRHA=1.5D0)
      PARAMETER (CCOR=0.09,SIXTH=UN/6.)
      DIMENSION DCOL(MTRANS),AM(MLEVEL)
      COMMON/UPSUMS/DUSUMT(MION),DUSUMN(MION)
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      COMMON/BPOCOM/ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       ATT(MLEVEL),ANN(MLEVEL)
C
      NSE=NFREQE+INSE-1
      IF(INRE.EQ.0.AND.INPC.EQ.0) GO TO 400
      IF(IELH.GT.0) N0HN=NFIRST(IELH)
      NKH=IIEXP(NREF(IATREF))
      T=TEMP(ID)
      ANE=ELEC(ID)
      HKT=HK/T
      TK=HKT/H
      ANMNE1=WMM(ID)*DENS1(ID)
      DO 10 I=1,NTRANS
         DCOL(I)=0.
   10 CONTINUE
      DO 20 I=1,NLEVEL
         AM(I)=0.
   20 CONTINUE
C
C     Derivatives of collisional rates wrt temperature - DCOL
C     Note that these derivatives are calculated numerically
C
      IF(.NOT.LTE.AND.INRE.GT.0.AND.ID.LT.IDLTE) THEN
         DELTAT=T*1.D-4
         CALL COLIS(ID,T+DELTAT,DCOL)
         DO 30 ITR=1,NTRANS
   30       DCOL(ITR)=(DCOL(ITR)-COLRAT(ITR,ID))/DELTAT
      END IF
C
C     Column corresponding to temperature and electron density, ie. the
C     (NFREQE+INRE)-th and (NFREQE+INPC)-t resach columns
C
C      ATT(I) - auxiliary vector = (derivative of rate matrix wrt
C              temperature) times (vector of populations)
C
C      ANN(I) - auxiliary vector = (derivative of rate matrix wrt
C              electron density) times (vector of populations)
C
C
C     a) contribution to AT and AN from true statistical equilibrium
C        equations (arising due to dependence of transition rates on
C        temperature and electron density);
C        derivatives contain the collisional-radiative switching
C        parameter CRSW
C
         IF(.NOT.LTE.AND.ID.LT.IDLTE) THEN
            DO 230 ITR=1,NTRANS
               I=ILOW(ITR)
               IF(ILTION(IEL(I)).GE.1.OR.IIFIX(IATM(I)).EQ.1) GO TO 230
               J=IUP(ITR)
               IF(IPZERO(I,ID).NE.0.OR.IPZERO(J,ID).NE.0) GO TO 230
               II=IIEXP(I)
               JJ=IIEXP(J)
               NREFI=NREF(IATM(I))
               IF(.NOT.LINE(ITR)) THEN
                  DLGT=-(TRHA+HKT*FR0(ITR))/T
                  DLGN=ELEC1(ID)
                ELSE
                  DLGT=-HKT*FR0(ITR)/T
                  DLGN=0.
               END IF
               POPI=ABTRA(ITR,ID)
               POPJ=EMTRA(ITR,ID)
               PJI=POPJ*(RRD(ITR,ID)+COLRAT(ITR,ID)*ANE)
               AVT=(POPI-POPJ)*DCOL(ITR)*ANE-PJI*DLGT-
     *              POPJ*DRDT(ITR,ID)
               AVN=(POPI-POPJ)*COLRAT(ITR,ID)-PJI*DLGN
C
               IF(I.NE.NREFI.AND.II.GT.0.AND.ILTLEV(I).LE.0) THEN
                  ATT(II)=ATT(II)+AVT
                  ANN(II)=ANN(II)+AVN
                  IF(JJ.LT.0) THEN
                     ATT(II)=ATT(II)-PJI*DSBPST(J,ID)
                     ANN(II)=ANN(II)-PJI*DSBPSN(J,ID)
                  END IF
               END IF
               IF(J.NE.NREFI.AND.JJ.GT.0.AND.ILTLEV(J).LE.0.
     *         and.iabs(imodl(i)).ne.4) THEN
                  ATT(JJ)=ATT(JJ)-AVT
                  ANN(JJ)=ANN(JJ)-AVN
                  IF(II.LT.0) THEN
                     PIJ=POPI*(RRU(ITR,ID)+ANE*COLRAT(ITR,ID))
                     ATT(JJ)=ATT(JJ)-PIJ*DSBPST(I,ID)
                     ANN(JJ)=ANN(JJ)-PIJ*DSBPSN(I,ID)
                  END IF
               END IF
  230       CONTINUE
         END IF
C
C      simple expressions in the case of LTE
C
      LLT=LTE.OR.ID.GE.IDLTE
      DO 260 IAT=1,NATOM
         DO 260 I=N0A(IAT),NKA(IAT)
            II=IIEXP(I)
            IF(II.LE.0.OR.I.EQ.NREF(IAT)) GO TO 260
            IF(LLT.OR.ILTION(IEL(I)).GE.1.OR.ILTLEV(I).GE.1) THEN
               ATT(II)=ATT(II)-POPUL(I,ID)*DSBPST(I,ID)
               ANN(II)=ANN(II)-POPUL(I,ID)*DSBPSN(I,ID)
            END IF
  260    CONTINUE
C
C
C      b) contribution to AT and AN (and AM - for total particle density)
C         from the abundance definition equations
C
      DO 300 IAT=1,NATOM
         IF(IAT.EQ.IATREF.AND.IRSPLT.EQ.0) GO TO 300
         IF(IIFIX(IAT).EQ.1) GO TO 300
         NREFII=IIEXP(NREF(IAT))
         IF(NREFII.LE.0) GO TO 300
         DO 280 I=N0A(IAT),NKA(IAT)
            IL=ILK(I)
            II=IIEXP(I)
            IF(IL.EQ.0) THEN
               IF(II.LT.0) THEN
                  ATT(NREFII)=ATT(NREFII)+POPUL(I,ID)*DSBPST(I,ID)
                  ANN(NREFII)=ANN(NREFII)+POPUL(I,ID)*DSBPSN(I,ID)
               END IF
             ELSE
               ATT(NREFII)=ATT(NREFII)+POPUL(I,ID)*DUSUMT(IL)*ANE
               ANN(NREFII)=ANN(NREFII)+
     *                     POPUL(I,ID)*(USUM(IL)+ANE*DUSUMN(IL))
            END IF
  280    CONTINUE
         IF(IRSPLT.EQ.0) THEN
            DO 290 I=N0A(IATREF),NKA(IATREF)
               II=IIEXP(I)
               IL=ILK(I)
               IF(IL.EQ.0) THEN
                  IF(II.LT.0) THEN
                     ATT(NREFII)=ATT(NREFII)-
     *                   ABUND(IAT,ID)*POPUL(I,ID)*DSBPST(I,ID)
                     ANN(NREFII)=ANN(NREFII)-
     *                   ABUND(IAT,ID)*POPUL(I,ID)*DSBPSN(I,ID)
                 END IF
                ELSE
                  ATT(NREFII)=ATT(NREFII)-
     *                ABUND(IAT,ID)*POPUL(I,ID)*DUSUMT(IL)*ANE
                  ANN(NREFII)=ANN(NREFII)-
     *                ABUND(IAT,ID)*POPUL(I,ID)*
     *                (USUM(IL)+DUSUMN(IL)*ANE)
               END IF
  290       CONTINUE
          ELSE
            ANN(NREFII)=ANN(NREFII)+UN/YTOT(ID)*ABUND(IAT,ID)
            AM(NREFII)=AM(NREFII)-UN/YTOT(ID)*ABUND(IAT,ID)
         END IF
  300 CONTINUE
      IF(IRSPLT.GT.0) GO TO 360
C
C     c) contribution to AT and AN from the row of rate matrix
C        corresponding to the particle or charge conservation
C        (equation corresponding to the NREF(IATREF)-row)
C
      IF(LCHC) THEN
C
C     if the equation is a charge conservation equation 
C
      CALL STATE(3,ID,T,ANE)
C
      APTT=0.
      APNN=0.
      APM=0.
      DO 330 IAT=1,NATOM
         IF(IIFIX(IAT).EQ.1) GO TO 330
         DO 320 I=N0A(IAT),NKA(IAT)
            II=IIEXP(I)
            IL=ILK(I)
            IF(IL.EQ.0) THEN
               CH=IZ(IEL(I))-1
               DCHT=0.
               DCHN=0.
             ELSE
               CH=IZ(IL)+(IZ(IL)-1)*USUM(IL)*ANE
               DCHT=(IZ(IL)-1)*ANE*DUSUMT(IL)*POPUL(I,ID)
               DCHN=(IZ(IL)-1)*(ANE*DUSUMN(IL)+USUM(IL))*POPUL(I,ID)
            END IF
            IF(II.GT.0) THEN
               APTT=APTT+DCHT
               APNN=APNN+DCHN
             ELSE IF(II.LT.0) THEN
               APTT=APTT+CH*POPUL(I,ID)*DSBPST(I,ID)
               APNN=APNN+CH*POPUL(I,ID)*DSBPSN(I,ID)
            END IF
  320    CONTINUE
  330 CONTINUE
C
      ATT(NKH)=ATT(NKH)+APTT
      ANN(NKH)=ANN(NKH)+APNN-UN
      AM(NKH)=AM(NKH)+APM
C
      ELSE
C
C     if the equation is particle conservation (LCHC=F)
C
      AVT=0.
      AVN=0.
      DO 350 I=N0A(IATREF),NKA(IATREF)
         II=IIEXP(I)
         IL=ILK(I)
         IF(IL.EQ.0) THEN
            IF(II.LT.0) THEN
               AVT=AVT+POPUL(I,ID)*DSBPST(I,ID)
               AVN=AVN+POPUL(I,ID)*DSBPSN(I,ID)
            END IF
          ELSE
            AVT=AVT+POPUL(I,ID)*DUSUMT(IL)*ANE
            AVN=AVN+POPUL(I,ID)*(ANE*DUSUMN(IL)+USUM(IL))
         END IF
  350 CONTINUE
      ATT(NKH)=ATT(NKH)+AVT
      ANN(NKH)=ANN(NKH)+AVN+UN/YTOT(ID)*ABUND(IATREF,ID)
      AM(NKH)=AM(NKH)-UN/YTOT(ID)*ABUND(IATREF,ID)
      END IF
C
C -----------------------
C     Having evaluated auxiliary vectors AT, AN, AM, we may now set up
C     the columns corresponding to temperature, el.density, and total
C     particle number density
C
  360 DO 390 I=1,NLVEXP
         IF(IFPOPR.LE.3) THEN
         AVT=0.
         AVN=0.
         AVM=0.
         DO 380 J=1,NLVEXP
            AVT=AVT-ESEMAT(I,J)*ATT(J)
            AVN=AVN-ESEMAT(I,J)*ANN(J)
            AVM=AVM-ESEMAT(I,J)*AM(J)
  380    CONTINUE
         ELSE
            AVT=ATT(I)
            AVN=ANN(I)
            AVM=AM(I)
         END IF
         IF(INHE.NE.0) B(NSE+I,NFREQE+INHE)=B(NSE+I,NFREQE+INHE)+AVM
         IF(INRE.NE.0) B(NSE+I,NFREQE+INRE)=B(NSE+I,NFREQE+INRE)+AVT
         IF(INPC.NE.0) B(NSE+I,NFREQE+INPC)=B(NSE+I,NFREQE+INPC)+AVN
  390 CONTINUE
C
C     Columns corresponding to populations
C
  400 CONTINUE
      IF(IFPOPR.LE.3) THEN
      DO 420 I=1,NLVEXP
         B(NSE+I,NSE+I)=B(NSE+I,NSE+I)-UN
         IF(IABS(IFPOPR).GE.3) THEN
         SUM=0.
         DO 410 J=1,NLVEXP
  410       SUM=SUM+ESEMAT(I,J)*BESE(J)
         VECL(NSE+I)=POPUL(INDLEV(I),ID)-SUM
         END IF
  420 CONTINUE
      ELSE IF(IFPOPR.LE.5) THEN
      DO 440 I=1,NLVEXP
         SUM=0.
         DO 430 J=1,NLVEXP
            SUM=SUM+ESEMAT(I,J)*POPUL(INDLEV(J),ID)
            B(NSE+I,NSE+J)=B(NSE+I,NSE+J)+ESEMAT(I,J)
  430    CONTINUE
         VECL(NSE+I)=BESE(I)-SUM
  440 CONTINUE
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE BPOPC(ID)
C     ====================
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ODFPAR.FOR'
      DIMENSION AJ(MLEVEL)
      COMMON/UPSUMS/DUSUMT(MION),DUSUMN(MION)
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      COMMON/BPOCOM/ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       ATT(MLEVEL),ANN(MLEVEL)
      COMMON/CHARFX/QFIX(MDEPTH)
C
      NSE=NFREQE+INSE-1
      NPC=NFREQE+INPC
      IF(IELH.GT.0) N0HN=NFIRST(IELH)
      NKH=NREF(IATREF)
      NKH=IIEXP(NKH)
      T=TEMP(ID)
      ANE=ELEC(ID)
      HKT=HK/T
      TK=HKT/H
      ANMNE1=WMM(ID)*DENS1(ID)
      DO 20 I=1,NLEVEL
         AJ(I)=0.
   20 CONTINUE
C
C *************************
C     remaining equation - (NFREQE+INPC)-th row
C     (either particle or charge conservation, depending on LCHC)
C *************************
C
      IF(LCHC) THEN
C
C     a) Linearized particle conservation
C
      SUMH=0.
      AVT=0.
      AVN=0.
      AVM=0.
      DO 420 I=N0A(IATREF),NKA(IATREF)
         IF(IPZERO(I,ID).NE.0) GO TO 420
         II=IIEXP(I)
         IL=ILK(I)
         IF(IL.EQ.0) THEN
            IF(II.GT.0) THEN
               B(NPC,NSE+II)=B(NPC,NSE+II)+UN
               SUMH=SUMH+POPUL(I,ID)
             ELSE
               SUMH=SUMH+POPUL(I,ID)
               AVT=AVT+POPUL(I,ID)*DSBPST(I,ID)
               AVN=AVN+POPUL(I,ID)*DSBPSN(I,ID)
            END IF
          ELSE
            SUMH=SUMH+(UN+ANE*USUM(IL))*POPUL(I,ID)
            B(NPC,NSE+II)=B(NPC,NSE+II)+UN+ANE*USUM(IL)
            AVT=AVT+POPUL(I,ID)*DUSUMT(IL)*ANE
            AVN=AVN+POPUL(I,ID)*(ANE*DUSUMN(IL)+USUM(IL))
         END IF
  420 CONTINUE
      IF(INHE.NE.0) B(NPC,NFREQE+INHE)=AVM-
     *              UN/YTOT(ID)*ABUND(IATREF,ID)
      IF(INRE.NE.0) B(NPC,NFREQE+INRE)=AVT
      B(NPC,NPC)=AVN+UN/YTOT(ID)*ABUND(IATREF,ID)
C
C     (NFREQE+INPC)-th element of the rhs vector VECL
C
      VECL(NPC)=DENS(ID)/WMM(ID)/YTOT(ID)*ABUND(IATREF,ID)-SUMH
C
      ELSE
C
C     b) Linearized charge conservation equation 
C
C     This part is very similar to procedure ELCOR (obviously);
C     array AJ has the meaning of coefficients of the charge conserv.
C     ie. charge conservation is written
C           AJ * (vector of populations) = electron density
C     then
C           APTT = (vector of populations) * (derivative AJ wrt temp)
C           APNN = (vector of populations) * (derivative AJ wrt n(el))
C           APM = (vector of populations) * (derivative AJ wrt N)
C
      IF(INPC.EQ.0) RETURN
      CALL STATE(3,ID,T,ANE)
      QQ=Q*ABUND(IATREF,ID)/YTOT(ID)
C
      APTT=0.
      APNN=0.
      APM=0.
      VPC=QFIX(ID)+QQ/ANMNE1
      DO 440 IAT=1,NATOM
         IF(IIFIX(IAT).EQ.1) GO TO 440
         DO 430 I=N0A(IAT),NKA(IAT)
            IF(IPZERO(I,ID).NE.0) GO TO 430
            IL=ILK(I)
            II=IIEXP(I)
            IF(IL.EQ.0) THEN
               CH=IZ(IEL(I))-1
               DCHT=0.
               DCHN=0.
             ELSE
               CH=IZ(IL)+(IZ(IL)-1)*USUM(IL)*ANE
               DCHT=(IZ(IL)-1)*ANE*DUSUMT(IL)*POPUL(I,ID)
               DCHN=(IZ(IL)-1)*(ANE*DUSUMN(IL)+USUM(IL))*POPUL(I,ID)
            END IF
            IF(II.GT.0) THEN
               AJ(II)=AJ(II)+CH
               APTT=APTT+DCHT
               APNN=APNN+DCHN
             ELSE
               III=IIEXP(ILTREF(I,ID))
               AJ(III)=AJ(III)+CH*SBPSI(I,ID)
               APTT=APTT+CH*POPUL(I,ID)*DSBPST(I,ID)
               APNN=APNN+CH*POPUL(I,ID)*DSBPSN(I,ID)
            END IF
  430    CONTINUE
  440 CONTINUE
C
C     (NFREQE+INPC)-th row of matrix B
C
      NPC=NFREQE+INPC
      QQQ=ABUND(IATREF,ID)/YTOT(ID)/ANMNE1
      IF(INHE.NE.0) B(NPC,NFREQE+INHE)=APM+QQ
      IF(INRE.NE.0) B(NPC,NFREQE+INRE)=APTT+QQQ*DQT
      B(NPC,NPC)=APNN-QQ-UN+QQQ*DQN
      DO 460 I=1,NLEVEL
         II=IIEXP(I)
         IF(II.LE.0) GO TO 460
         VPC=VPC+AJ(II)*POPUL(I,ID)
         B(NPC,NSE+II)=AJ(II)
  460 CONTINUE
C
C     (NFREQE+INPC)-th element of the rhs vector VECL
C
      VECL(NPC)=ANE-VPC
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE OPACFD(IJ)
C     =====================
C
C     Absorption and emission coefficients, and their derivatives
C
C     This procedure is very similar to OPACF1, the only differences is
C     the evaluation of derivatives
C
C     Input: 
C      IJ    - depth index
C     Output:
C      ABSO1  - array of absorption coefficient
C      EMIS1  - array of emission coefficient
C      SCAT1  - array of scattering coefficient
C      Dxxy  - array of derivatives of xx (=AB for absorption, =EM for
C              emission) coefficient wrt y (=T for temperature, =N for
C              electron density)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ITERAT.FOR'
      PARAMETER (CFF1=1.3727D-25)
      COMMON/OPACAD/ABAD,EMAD,SCAD,DAT,DAN,DET,DEN,DST,DSN,DDN(MLEVEL)
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
      DIMENSION DABP0(MLEVEL),DEMP0(MLEVEL)
C
C     initialize 
c
      DO ID=1,ND
         ABSO1(ID)=ELSCAT(ID)
         EMIS1(ID)=0.
         SCAT1(ID)=ELSCAT(ID)
         DABT1(ID)=0.
         DEMT1(ID)=0.
         DABN1(ID)=SIGE
         DEMN1(ID)=0.
         ABSFF(ID)=0.
         DABFT(ID)=0.
         DABFN(ID)=0.
         DO II=1,NLEVEL
            DABP1(II,ID)=0.
            DEMP1(II,ID)=0.
         END DO
      END DO
C
C     basic frequency- and depth-dependent quantities
C
      FR=FREQ(IJ)
      FRINV=UN/FR
      FR3INV=FRINV*FRINV*FRINV
      DO 10 ID=1,ND
         XKF(ID)=EXP(-HKT1(ID)*FR)
         XKF1(ID)=UN-XKF(ID)
         XKFB(ID)=XKF(ID)*BNUE(IJ)
   10 CONTINUE
C
C ********  1. bound-free contribution
C
      DO 30 IBF=1,NTRANC
         ITR=ITRBF(IBF)
         SG=CROSS(IBF,IJ)
         IF(SG.LE.0.) GO TO 30
         II=ILOW(ITR)
         JJ=IUP(ITR)
         IZZ=IZ(IEL(II))
         IMER=IMRG(II)
         DO 20 ID=1,ND
            SGD=SG
            IF(MCDW(ITR).GT.0) THEN
               CALL DWNFR1(FR,FR0(ITR),ID,IZZ,DW1)
               DWF1(MCDW(ITR),ID)=DW1
               SGD=SG*DW1
            END IF
            IF(IFWOP(II).LT.0) THEN
               CALL SGMER1(FRINV,FR3INV,IMER,ID,SGME1)
               SGMG(IMER,ID)=SGME1
               SGD=SGME1
            END IF
            EMISBF=SGD*EMTRA(ITR,ID)
            ABSO1(ID)=ABSO1(ID)+SGD*ABTRA(ITR,ID)
            EMIS1(ID)=EMIS1(ID)+EMISBF
            IF(IIEXP(II).NE.0.or.iabs(ilder).eq.1) THEN
               DEMT1(ID)=DEMT1(ID)+EMISBF*DEMLT(ITR,ID)
               DEMN1(ID)=DEMN1(ID)+EMISBF*ELEC1(ID)
            END IF
            DEMP1(JJ,ID)=DEMP1(JJ,ID)+EMISBF*POPINV(JJ,ID)
            if(ipzero(ii,id).eq.0) DABP1(II,ID)=DABP1(II,ID)+SGD
   20    CONTINUE
   30 CONTINUE
C
C ******** 2. free-free contribution
C
      DO 40 ION=1,NION
         II=NNEXT(ION)
         IT=ITRA(II,II)
C
C        hydrogenic (with Gaunt factor = 1 for IT=1; exact for IT=2)
C        (derivative of Gaunt factor wrt T is neglected)
C
         IF(IT.LE.2) THEN
            DO ID=1,ND
               SF1=SFF3(ION,ID)*FR3INV
               SF2=SFF2(ION,ID)
               DSF2=DSFF(ION,ID)
               IF(FR.LT.FF(ION)) THEN
                  SF2=UN/XKF(ID)
                  DSF2=(HKT1(ID)*FR+HALF)*TEMP1(ID)
               END IF
               IF(IT.EQ.2) THEN
                  CALL GFREE1(ID,FR,CHARG2(ION),GFR)
                  SF2=SF2-UN+GFR
                ELSE IF(IT.EQ.3) THEN
                  CALL GFREED(ID,FR,CHARG2(ION),GFR,DGFR)
                  SF2=SF2-UN+GFR
                  DSF2=DSF2-(DGFR-(GFR-UN)*TEMP1(ID)*HALF)/SF2
               END IF
               ABSOFF=SF1*SF2
               ABSFF(ID)=ABSFF(ID)+ABSOFF
               DABFT(ID)=DABFT(ID)-ABSOFF*DSF2
               DABFN(ID)=DABFN(ID)+ABSOFF*ELEC1(ID)
               DABPP=ABSOFF*POPINV(II,ID)
               DABP1(II,ID)=DABP1(II,ID)+DABPP
               DEMP1(II,ID)=DEMP1(II,ID)+DABPP
            END DO
C
C         H minus free-free opacity
C         (all derivatives are neglected)
C
          ELSE IF(IT.EQ.3) THEN
            DO ID=1,ND
               ABSOFF=(CFF1+CFFT(ID)*FRINV)*CFFN(ID)*FRINV
               ABSFF(ID)=ABSFF(ID)+ABSOFF
            END DO
C
C         special evaluation of the cross-section
C         (all derivatives are neglected)
C
          ELSE IF(IT.LT.0) THEN
            DO ID=1,ND
               ABSOFF=FFCROS(ION,IT,TEMP(ID),FR)*
     *                POPUL(NNEXT(ION),ID)*ELEC(ID)
               ABSFF(ID)=ABSFF(ID)+ABSOFF
            END DO
         END IF
   40 CONTINUE
C
C     ********  3. - additional opacity (OPADD)
C
      IF(IOPADD.NE.0) THEN
         DO ID=1,ND
            CALL OPADD(0,ICALL,IJ,ID)
            IF(ICALL.EQ.1) ICALL=0
            ABSO1(ID)=ABSO1(ID)+ABAD
            EMIS1(ID)=EMIS1(ID)+EMAD
            SCAT1(ID)=SCAT1(ID)+SCAD
            DABT1(ID)=DABT1(ID)+DAT
            DEMT1(ID)=DEMT1(ID)+DET
            DABN1(ID)=DABN1(ID)+DAN
            DEMN1(ID)=DEMN1(ID)+DEN
         END DO
      END IF
C
C     -----------------------
C     total continuum opacity
C     -----------------------
C
      DO ID=1,ND
         ABSO1(ID)=ABSO1(ID)+ABSFF(ID)
         DABT1(ID)=DABT1(ID)+DABFT(ID)
         DABN1(ID)=DABN1(ID)+DABFN(ID)
         EMIS1(ID)=EMIS1(ID)+ABSFF(ID)
         DEMT1(ID)=DEMT1(ID)+DABFT(ID)
         DEMN1(ID)=DEMN1(ID)+DABFN(ID)
      END DO
C
C ********  4. - opacity and emissivity in lines
C
      IF(IJLIN(IJ).GT.0) THEN
         LASER=ITER.GT.ITLAS
C
C     the "primary" line at the given frequency
C
         ITR=IJLIN(IJ)
         II=ILOW(ITR)
         JJ=IUP(ITR)
         DO 50 ID=1,ND
            SG=PRFLIN(IJ,ID)
            SGPI=SG*ABTRA(ITR,ID)
            IF(SGPI.LE.0.AND.LASER) GO TO 50
            SGPJ=SG*EMTRA(ITR,ID)
            ABSO1(ID)=ABSO1(ID)+SGPI
            EMIS1(ID)=EMIS1(ID)+SGPJ
            DEMT1(ID)=DEMT1(ID)+SGPJ*DEMLT(ITR,ID)
            DABP1(II,ID)=DABP1(II,ID)+SGPI*POPINV(II,ID)
            DEMP1(JJ,ID)=DEMP1(JJ,ID)+SGPJ*POPINV(JJ,ID)
   50    CONTINUE
      ENDIF
      IF(NLINES(IJ).LE.0) GO TO 200
C
C     the "overlapping" lines at the given frequency
C
      DO 100 ILINT=1,NLINES(IJ)
         ITR=ITRLIN(ILINT,IJ)
         II=ILOW(ITR)
         JJ=IUP(ITR)
         IJ0=IFR0(ITR)
         DO 60 IJT=IJ0,IFR1(ITR)
            IF(FREQ(IJT).LE.FR) THEN
               IJ0=IJT
               GO TO 70
            END IF
   60    CONTINUE
   70    IJ1=IJ0-1
         A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))
         A2=UN-A1
         DO 80 ID=1,ND
            SG=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
            SGPI=SG*ABTRA(ITR,ID)
            IF(SGPI.LE.0.AND.LASER) GO TO 80
            SGPJ=SG*EMTRA(ITR,ID)
            ABSO1(ID)=ABSO1(ID)+SGPI
            EMIS1(ID)=EMIS1(ID)+SGPJ
            DEMT1(ID)=DEMT1(ID)+SGPJ*DEMLT(ITR,ID)
            DABP1(II,ID)=DABP1(II,ID)+SGPI*POPINV(II,ID)
            DEMP1(JJ,ID)=DEMP1(JJ,ID)+SGPJ*POPINV(JJ,ID)
   80    CONTINUE
  100 CONTINUE
  200 CONTINUE
C
C     ------------------------------------------
C     total opacity, emissivity, and derivatives
C     ------------------------------------------
C
      DO ID=1,ND
         DEMT1(ID)=DEMT1(ID)+EMIS1(ID)*FR*HKT21(ID)
c
         ABSO1(ID)=ABSO1(ID)-EMIS1(ID)*XKF(ID) 
         DABN1(ID)=DABN1(ID)-DEMN1(ID)*XKF(ID)
         DABT1(ID)=DABT1(ID)-DEMT1(ID)*XKF(ID)
c
         EMIS1(ID)=EMIS1(ID)*XKFB(ID)
         DEMN1(ID)=DEMN1(ID)*XKFB(ID)
         DEMT1(ID)=DEMT1(ID)*XKFB(ID)
c
         DO II=1,NLEVEL
            DABP1(II,ID)=DABP1(II,ID)-DEMP1(II,ID)*XKF(ID)
            DEMP1(II,ID)=DEMP1(II,ID)*XKFB(ID)
         END DO
      END DO
C
C     ---------------------------------------------
C
C     derivatives in the linearized explicit levels
C
      IF(NLVEXP.LT.NLEVEL) THEN
      DO ID=1,ND
         DO I=1,NLVEXP
            DABP0(I)=0.
            DEMP0(I)=0.
         END DO
         DO I=1,NLEVEL
            II=IIEXP(I)
            IF(II.GT.0) THEN
               DABP0(II)=DABP0(II)+DABP1(I,ID)
               DEMP0(II)=DEMP0(II)+DEMP1(I,ID)
             ELSE IF(II.LT.0) THEN
               DABT1(ID)=DABT1(ID)+DABP1(I,ID)*PT(I,ID)
               DEMT1(ID)=DEMT1(ID)+DEMP1(I,ID)*PT(I,ID)
               DABN1(ID)=DABN1(ID)+DABP1(I,ID)*PN(I,ID)
               DEMN1(ID)=DEMN1(ID)+DEMP1(I,ID)*PN(I,ID)
               JJ=IIEXP(ILTREF(I,ID))
               DABP0(JJ)=DABP0(JJ)+DABP1(I,ID)*PP(I,ID)
               DEMP0(JJ)=DEMP0(JJ)+DEMP1(I,ID)*PP(I,ID)
             ELSE
               JJ=IIEXP(ILTREF(I,ID))
               if(jj.gt.0) then
               DABP0(JJ)=DABP0(JJ)+DABP1(I,ID)*PP(I,ID)
               DEMP0(JJ)=DEMP0(JJ)+DEMP1(I,ID)*PP(I,ID)
               endif
            END IF
         END DO
         DO II=1,NLVEXP
            DABP1(II,ID)=DABP0(II)
            DEMP1(II,ID)=DEMP0(II)
         END DO
      END DO
      END IF
C
C     store quantities for explicit (linearized) frequencies
C
      IF(IJEX(IJ).LE.0) RETURN
      IJE=IJEX(IJ)
      DO ID=1,ND
         ABSOEX(IJE,ID)=ABSO1(ID)
         EMISEX(IJE,ID)=EMIS1(ID)
         SCATEX(IJE,ID)=SCAT1(ID)
         DABTEX(IJE,ID)=DABT1(ID)
         DEMTEX(IJE,ID)=DEMT1(ID)
         DABNEX(IJE,ID)=DABN1(ID)
         DEMNEX(IJE,ID)=DEMN1(ID)
         DO II=1,NLVEXP
            DRCHEX(II,IJE,ID)=DABP1(II,ID)
            DRETEX(II,IJE,ID)=DEMP1(II,ID)
         END DO
      END DO
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE ALIFR1(IJ)
C     =====================
C
C     hydrostatic and radiative equilibrium quantities - 
C     derivatives of the total heating and cooling rates in the 
C     ALI points with respect to the
C     temperature, electron density, and populations
C     a variant for consistent tridiagonal operator
C
      INCLUDE 'IMPLIC.FOR'     
      INCLUDE 'BASICS.FOR'     
      INCLUDE 'ATOMIC.FOR'     
      INCLUDE 'MODELQ.FOR'     
      INCLUDE 'ALIPAR.FOR'     
      PARAMETER (T23=TWO/3.D0, T43=4.D0/3.D0)
      DIMENSION DSFP1(MLVEXP),DSFP1M(MLVEXP),DSFP1D(MLVEXP),
     *          DSFP1P(MLVEXP),DSFPMM(MLVEXP)
C
      if(ifali.le.1) return
      LNSKIP=.NOT.LSKIP(IJ)
      WW=WC(IJ)
C
      DSFT1M=0.
      DSFN1M=0.
      DO II=1,NLVEXP
         DSFP1M(II)=0.
      END DO
C
c     loop over depths
c
      DO 100 ID=1,ND
c
c ****1. special eqpressions for the first depth  -  id=1
c
         IF(ID.EQ.1) THEN
c
c        basic auxliliary quantities - derivatives of the source function
c        at depth ID and ID-1
c
            EMISIV=UN/EMIS1(ID)
            if(ilasct.eq.0) then
               ABST=UN/(ABSO1(ID)-ELSCAT(ID))
               S0=EMIS1(ID)*ABST
               DSFN1=S0*(DEMN1(ID)*EMISIV-(DABN1(ID)-SIGE)*ABST)
            else
               ABST=UN/ABSO1(ID)
               S0=EMIS1(ID)*ABST
               DSFN1=S0*(DEMN1(ID)*EMISIV-DABN1(ID)*ABST)
            end if
            DSFT1=S0*(DEMT1(ID)*EMISIV-DABT1(ID)*ABST)
            DO II=1,NLVEXP
               DSFP1(II)=S0*(DEMP1(II,ID)*EMISIV-DABP1(II,ID)*ABST)
            END DO
          ELSE
            DSFTMM=DSFT1M
            DSFNMM=DSFN1M
            DO II=1,NLVEXP
               DSFPMM(II)=DSFP1M(II)
            END DO
            DSFT1M=DSFT1
            DSFN1M=DSFN1
            DO II=1,NLVEXP
               DSFP1M(II)=DSFP1(II)
            END DO
            S0=S0P
            DSFT1=DSFT1P
            DSFN1=DSFN1P
            DO II=1,NLVEXP
               DSFP1(II)=DSFP1P(II)
            END DO
         END IF
c
c        basic auxliliary quantities - derivatives of the source function
c        at depth ID+1
c
         IF(ID.LT.ND) THEN
            EMISIP=UN/EMIS1(ID+1)
            if(ilasct.eq.0) then
               ABSTP=UN/(ABSO1(ID+1)-ELSCAT(ID+1))
               S0P=EMIS1(ID+1)*ABSTP
               DSFN1P=S0P*(DEMN1(ID+1)*EMISIP-
     *                (DABN1(ID+1)-SIGE)*ABSTP)
             else
               ABSTP=UN/ABSO1(ID+1)
               S0P=EMIS1(ID+1)*ABSTP
               DSFN1P=S0P*(DEMN1(ID+1)*EMISIP-DABN1(ID+1)*ABSTP)
            end if
            DSFT1P=S0P*(DEMT1(ID+1)*EMISIP-DABT1(ID+1)*ABSTP)
            DO II=1,NLVEXP
               DSFP1P(II)=S0P*(DEMP1(II,ID+1)*EMISIP-
     *                    DABP1(II,ID+1)*ABSTP)
            END DO
         END IF
C
C        improved lower boundary condition
C
         IF(ID.EQ.ND.AND.IBC.GT.0) THEN
            DT=UN/(DELDM(ID-1)*(ABSO1(ID)*DENS1(ID)+
     *                      ABSO1(ID-1)*DENS1(ID-1)))
            PLAD=XKFB(ID)/XKF1(ID)
            DBDT=PLAD/XKF1(ID)*HKT21(ID)*FREQ(IJ)*DT
            IF(IBC.EQ.1) THEN
               DSFT1=DSFT1+DBDT
             ELSE IF(IBC.GE.2) THEN
               PLAM=XKFB(ID-1)/XKF1(ID-1)
               TAU23=T23*DT
               TAU43=T43*DT
               D0=(PLAD*(UN+TAU43)-T43*PLAM*DT)*DT*DT
               RHD=DELDM(ID-1)/DENS(ID)
               E0=D0*RHD
               DSFT1=DSFT1+DBDT*(UN+TAU23)-E0*DABT1(ID)
               DSFN1=DSFN1-E0*(DABN1(ID)+ABSO1(ID)/DENS(ID)*WMM(ID))
               DO II=1,NLVEXP
                  DSFP1(II)=DSFP1(II)-E0*DABP1(II,ID)
               END DO
               IF(IBC.GE.3) THEN
                  DBDTM=PLAM/XKF1(ID-1)*HKT21(ID-1)*FREQ(IJ)*DT
                  RHD=DELDM(ID-1)/DENS(ID-1)
                  E0=D0*RHD
                  DSFT1D=-DBDTM*DT*T23-E0*DABT1(ID-1)
                  DSFN1D=-E0*(DABN1(ID-1)+
     *                    ABSO1(ID-1)/DENS(ID-1)*WMM(ID))
                  DO II=1,NLVEXP
                     DSFP1D(II)=-E0*DABP1(II,ID-1)
                  END DO
               END IF
            END IF
         END IF
C
         IF(IRDER.EQ.1.OR.IRDER.EQ.3) THEN
            DSFDT(ID)=DSFT1*ALI1(ID)
            DSFDN(ID)=DSFN1*ALI1(ID)
         END IF
         IF(IRDER.GT.1) THEN
            DO II=1,NLVEXP
               DSFDP(II,ID)=DSFP1(II)*ALI1(ID)
            END DO
         END IF
C
         IF(IFALI.GE.6) THEN
         IF(IRDER.EQ.1.OR.IRDER.EQ.3) THEN
            DSFDTM(ID)=DSFT1M*ALIM1(ID)
            DSFDNM(ID)=DSFN1M*ALIM1(ID)
            DSFDTP(ID)=DSFT1P*ALIP1(ID)
            DSFDNP(ID)=DSFN1P*ALIP1(ID)
         END IF
         IF(IRDER.GT.1) THEN
            DO II=1,NLVEXP
               DSFDPM(II,ID)=DSFP1M(II)*ALIM1(ID)
               DSFDPP(II,ID)=DSFP1P(II)*ALIP1(ID)
            END DO
         END IF
         END IF
c
c ****1. special eqpressions for the first depth  -  id=1
c
         IF(ID.EQ.1) THEN
c
c           hydrostatic equilibrium quantities
c
            WF=WW*FH(IJ)
            IF(LNSKIP) THEN
            FPRD(ID)=FPRD(ID)+WF*ABSO1(ID)*RAD1(ID)-
     *               WW*HALF*EXTRAD(IJ)*ABSO1(ID)
            E0=WF*RAD1(ID)
            D0=WF*ABSO1(ID)*ALI1(ID)
            HEIT(ID)=HEIT(ID)+D0*DSFT1+E0*DABT1(ID)
            HEIN(ID)=HEIN(ID)+D0*DSFN1+E0*DABN1(ID)
            DO II=1,NLVEXP
               HEIP(II,ID)=HEIP(II,ID)+D0*DSFP1(II)+E0*DABP1(II,ID)
            END DO
            IF(IFALI.GE.7) THEN
            D0P=WF*ABSO1(ID)*ALIP1(ID)
            HEITP(ID)=HEITP(ID)+D0P*DSFT1P
            HEINP(ID)=HEINP(ID)+D0P*DSFN1P
            DO II=1,NLVEXP
               HEIPP(II,ID)=HEIPP(II,ID)+D0P*DSFP1P(II)
            END DO
            END IF
            END IF
c
c           differential equation part of radiative equilibrium
c
            FLFIX(ID)=FLFIX(ID)+WF*RAD1(ID)-WW*HALF*EXTRAD(IJ)
            IF(REDIF(ID).GT.0.) THEN
               WF=WF*ALI1(ID)
               REDT(ID)=REDT(ID)+WF*DSFT1
               REDN(ID)=REDN(ID)+WF*DSFN1
               DO II=1,NLVEXP
                  REDP(II,ID)=REDP(II,ID)+WF*DSFP1(II)
               END DO
            END IF
c
         ELSE  
c
c ****2. expressions valid for ID>1
c
c        hydrostatic equilibrium equation
c
            IF(LNSKIP) THEN
            D0=WW*FAK1(ID)
            A0=WW*FAK1(ID-1)
            FPRD(ID)=FPRD(ID)+D0*RAD1(ID)-A0*RAD1(ID-1)
            F0=D0*ALIP1(ID)
            E0=D0*ALIM1(ID)-A0*ALI1(ID-1)
            D0=D0*ALI1(ID)-A0*ALIP1(ID-1)
            HEIT(ID)=HEIT(ID)+D0*DSFT1
            HEIN(ID)=HEIN(ID)+D0*DSFN1
            HEITM(ID)=HEITM(ID)+E0*DSFT1M
            HEINM(ID)=HEINM(ID)+E0*DSFN1M
            DO II=1,NLVEXP
               HEIP(II,ID)=HEIP(II,ID)+D0*DSFP1(II)
               HEIPM(II,ID)=HEIPM(II,ID)+E0*DSFP1M(II)
            END DO
            IF(IFALI.GE.7) THEN
            HEITP(ID)=HEITP(ID)+F0*DSFT1P
            HEINP(ID)=HEINP(ID)+F0*DSFN1P
            DO II=1,NLVEXP
               HEIPP(II,ID)=HEIPP(II,ID)+F0*DSFP1P(II)
            END DO
            A0M=A0*ALIM1(ID-1)
            EHET(ID)=EHET(ID)-A0M*DSFTMM
            EHEN(ID)=EHEN(ID)-A0M*DSFNMM
            DO II=1,NLVEXP
               EHEP(II,ID)=EHEP(II,ID)-A0M*DSFPMM(II)
            END DO
            END IF
               IF(ID.EQ.ND.AND.IBC.EQ.3) THEN
                  HEITM(ID)=HEITM(ID)-D0*DSFT1D
                  HEINM(ID)=HEINM(ID)-D0*DSFN1D
                  DO II=1,NLVEXP
                     HEIPM(II,ID)=HEIPM(II,ID)-D0*DSFP1D(II)
                  END DO
               END IF   
            END IF
C   
C        differential equation part of radiative equilibrium
C
            DDT=UN/(ABSO1(ID)*DENS1(ID)+
     *              ABSO1(ID-1)*DENS1(ID-1))
            DT=DDT/DELDM(ID-1)
            FL=(RAD1(ID)*FAK1(ID)-RAD1(ID-1)*FAK1(ID-1))*DT
            FLFIX(ID)=FLFIX(ID)+WW*FL
            IF(REDIF(ID).GT.0) THEN
               D0=WW*FAK1(ID)*DT
               A0=WW*FAK1(ID-1)*DT
               D0M=D0*ALIM1(ID)-A0*ALI1(ID-1)
               D0P=D0*ALIP1(ID)
               D0=D0*ALI1(ID)-A0*ALIP1(ID-1)
               E0=WW*FL*DDT
               REDX(ID)=REDX(ID)+E0*ABSO1(ID)
               REDXM(ID)=REDXM(ID)+E0*ABSO1(ID-1)
               E0M=E0*DENS1(ID-1)
               E0=E0*DENS1(ID)
               REDT(ID)=REDT(ID)+D0*DSFT1-E0*DABT1(ID)
               REDTM(ID)=REDTM(ID)+D0M*DSFT1M-E0M*DABT1(ID-1)
               REDN(ID)=REDN(ID)+D0*DSFN1-E0*DABN1(ID)
               REDNM(ID)=REDNM(ID)+D0M*DSFN1M-E0M*DABN1(ID-1)
               DO II=1,NLVEXP
                  REDP(II,ID)=REDP(II,ID)+D0*DSFP1(II)-E0*DABP1(II,ID)
                  REDPM(II,ID)=REDPM(II,ID)+D0M*DSFP1M(II)-
     *                         E0M*DABP1(II,ID-1)
               END DO
               IF(IFALI.GE.7) THEN
               REDTP(ID)=REDTP(ID)+D0P*DSFT1M
               REDNP(ID)=REDNP(ID)+D0P*DSFN1M
               DO II=1,NLVEXP
                   REDPP(II,ID)=REDPP(II,ID)+D0P*DSFP1P(II)
               END DO
               A0M=A0*ALIM1(ID-1)
               ERET(ID)=ERET(ID)-A0M*DSFTMM
               EREN(ID)=EREN(ID)-A0M*DSFNMM
               DO II=1,NLVEXP
                  EREP(II,ID)=EREP(II,ID)-A0M*DSFPMM(II)
               END DO
               END IF
               IF(ID.EQ.ND.AND.IBC.EQ.3) THEN
                  REDTM(ID)=REDTM(ID)+D0*DSFT1D
                  REDNM(ID)=REDNM(ID)+D0*DSFN1D
                  DO II=1,NLVEXP
                     REDPM(II,ID)=REDPM(II,ID)+D0*DSFP1D(II)
                  END DO
               END IF   
            END IF   
C
         END IF
c
c ****3. expressions valid for all depths
c
C        integral equation part of the radiative equilibrium 
C
         IF(REINT(ID).GT.0) THEN
            if(ilasct.eq.0) then
               ABST=ABSO1(ID)-ELSCAT(ID)
               WWK=WW*ABST
               FCOOLI(ID)=FCOOLI(ID)+WW*(EMIS1(ID)-ABST*RAD1(ID))
               D0=WW*(ALI1(ID)-UN)*ABST
               E0=WW*(RAD1(ID)-S0)
               REIN(ID)=REIN(ID)+D0*DSFN1+E0*(DABN1(ID)-SIGE)
             else
               SRH=SIGE*DENS1(ID)
               ABST=ABSO1(ID)
               ABSTE=ABST-ELSCAT(ID)
               WWK=WW*ABST
               FCOOLI(ID)=FCOOLI(ID)+WW*(EMIS1(ID)-ABSTE*RAD1(ID))
               D0=WW*(ABSTE*ALI1(ID)-ABST)
               E0=WW*(RAD1(ID)-S0)
               REIN(ID)=REIN(ID)+D0*DSFN1+E0*DABN1(ID)-
     *                  WW*SIGE*RAD1(ID)
            end if
            IF(ID.LT.ND.OR.IBC.EQ.0) THEN
               REIT(ID)=REIT(ID)+D0*DSFT1+E0*DABT1(ID)
             ELSE
               REIT(ID)=REIT(ID)+D0*(DSFT1-DBDT)+E0*DABT1(ID)+
     *                  ALI1(ID)/ABST*DBDT
            END IF
            DO II=1,NLVEXP
               REIP(II,ID)=REIP(II,ID)+D0*DSFP1(II)+E0*DABP1(II,ID)
            END DO
c
c           the following are needed only for tridiagonal Lambda^star
c
            IF(IFALI.GT.5) THEN
C
C              lower sub-diagonal band
C
               IF(ID.GT.1) THEN
               WWKA=WWK*ALIM1(ID)
               AREIT(ID)=AREIT(ID)+WWKA*DSFT1M
               AREIN(ID)=AREIN(ID)+WWKA*DSFN1M
               DO II=1,NLVEXP
                  AREIP(II,ID)=AREIP(II,ID)+WWKA*DSFP1M(II)
               END DO
               END IF
C
C              upper sub-diagonal band
C
               IF(ID.LT.ND) THEN
               WWKC=WWK*ALIP1(ID)
               CREIT(ID)=CREIT(ID)+WWKC*DSFT1P
               CREIN(ID)=CREIN(ID)+WWKC*DSFN1P
               DO II=1,NLVEXP
                  CREIP(II,ID)=CREIP(II,ID)+WWKC*DSFP1P(II)
               END DO
               END IF
            END IF
         END IF
  100 CONTINUE
      RETURN
      END
C
C
C
C     ****************************************************************
C
C

      SUBROUTINE ALIFRK(IJ)
C     =====================
C
C     Simplified routine ALIFR1 for a Kantorovich iteartion
C
C     hydrostatic and radiative equilibrium quantities - 
C     derivatives of the total heating and cooling rates in the 
C     ALI points with respect to the
C     temperature, electron density, and populations
C
      INCLUDE 'IMPLIC.FOR'     
      INCLUDE 'BASICS.FOR'     
      INCLUDE 'ATOMIC.FOR'     
      INCLUDE 'MODELQ.FOR'     
      INCLUDE 'ALIPAR.FOR'     
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
C
      if(ifali.le.1) return
      LNSKIP=.NOT.LSKIP(IJ)
      WW=WC(IJ)
C
c     loop over depths
c
      DO 100 ID=1,ND
c
c ****1. special eqpressions for the first depth  -  id=1
c
         IF(ID.EQ.1) THEN
c
c           hydrostatic equilibrium quantities
c
            WF=WW*(FH(IJ)*RAD1(ID)-HALF*EXTRAD(IJ))
            IF(LNSKIP) FPRD(ID)=FPRD(ID)+WF*ABSO1(ID)
c
c           differential equation part of radiative equilibrium
c
            FLFIX(ID)=FLFIX(ID)+WF
         ELSE  
c
c ****2. expressions valid for ID>1
c
c        hydrostatic equilibrium equation
c
            IF(LNSKIP) THEN
               FPRD(ID)=FPRD(ID)+WW*(RAD1(ID)*FAK1(ID)-
     *                               RAD1(ID-1)*FAK1(ID-1))
            END IF
C   
C        differential equation part of radiative equilibrium
C
            DDT=UN/(ABSO1(ID)*DENS1(ID)+
     *              ABSO1(ID-1)*DENS1(ID-1))
            DT=DDT/DELDM(ID-1)
            FL=(RAD1(ID)*FAK1(ID)-RAD1(ID-1)*FAK1(ID-1))*DT
            FLFIX(ID)=FLFIX(ID)+WW*FL
C
         END IF
c
c ****3. expressions valid for all depths
c
C        integral equation part of the radiative equilibrium 
C
         IF(REINT(ID).GT.0) THEN
            ABST=ABSO1(ID)-ELSCAT(ID)
            FCOOLI(ID)=FCOOLI(ID)+WW*(EMIS1(ID)-ABST*RAD1(ID))
         END IF
  100 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE EMAT(ID)
C     ===================
C
C     Auxiliary procedure for SOLVE
C     
C     sub-sub-diagonal band matrix E
C
C     Input:  ID - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (PCK=4.19168946D-10)
C
      IF(ID.LE.2) RETURN
      NSE=NFREQE+INSE-1
      IF(INHE.GT.0) THEN
      NHE=NFREQE+INHE
      IF(INRE.GT.0) E(NHE,NFREQE+INRE)=EHET(ID)*PCK
      IF(INPC.GT.0) E(NHE,NFREQE+INPC)=EHEN(ID)*PCK
      DO II=1,NLVEXP
         E(NHE,NSE+II)=EHEP(II,ID)*PCK
      END DO
      END IF
C
      IF(INRE.GT.0.AND.REDIF(ID).GT.0.) THEN
      NRE=NFREQE+INRE
      IF(INRE.GT.0) E(NRE,NRE)=ERET(ID)*REDIF(ID)
      IF(INPC.GT.0) E(NRE,NFREQE+INPC)=EREN(ID)*REDIF(ID)
      DO II=1,NLVEXP
         E(NRE,NSE+II)=EREP(II,ID)*REDIF(ID)
      END DO
      END IF
      RETURN
      END 
C
C
C     ****************************************************************
C
C
      SUBROUTINE RHSGEN(ID)
C     =====================
C
C     Auxiliary procedure for SOLVE
C     controls evaluation of rhs vector when
C     matrices A,B, and C are kept fixed.
C
C     Input:  ID - depth index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (PCK=4.19168946D-10,
     *           SIXTH=UN/6.D0,
     *           THIRD=UN/3.D0)
      COMMON/BPOCOM/ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       ATT(MLEVEL),ANN(MLEVEL)
      DIMENSION AJ(MLEVEL)
      COMMON/FILES/PSY0(MTOT,MDEPTH),PSY1(MTOT,MDEPTH),
     *             PSY2(MTOT,MDEPTH),PSY3(MTOT,MDEPTH)
      COMMON/OPACAD/ABAD,EMAD,SCAD,DAT,DAN,DET,DEN,DST,DSN,DDN(MLEVEL)
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/RESDER/RSAT,RSBT,RSAN,RSBN,RSAX(MLEVEL),RSBX(MLEVEL)
      COMMON/CUBCON/ACNV,BCNV,DEL,GRDADB,DELMDE,RHO,FLXTOT
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/OPMEAN/ABROSD(MDEPTH),ABPLAD(MDEPTH)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      COMMON/CHARFX/QFIX(MDEPTH)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
C
      ispl=isplin
      if(isplin.ge.5) isplin=isplin-5
C
C     evaluation of the opacity, emissivity, scattering, and
C     their derivatives, at the current depth point ID;
C
         DO 10 I=1,NN
   10       PSI0(I)=PSY0(I,ID)
         IF(NFREQE.GT.0) THEN
          DO 11 IJ=1,NFREQE
            IJT=IJFR(IJ)
            WDEP0(IJ)=W(IJT)-WSUB(IJT)-WCTOTE(IJ)
            RAD0(IJ)=RADEX(IJ,ID)
            FK0(IJ)=FAKEX(IJ,ID)
            ABSO0(IJ)=ABSOEX(IJ,ID)
            EMIS0(IJ)=EMISEX(IJ,ID)
            SCAT0(IJ)=SCATEX(IJ,ID)
   11     CONTINUE
         END IF
C
      IF(ID.GT.1) THEN
         DO 20 I=1,NN
   20       PSIM(I)=PSY0(I,ID-1)
         IF(NFREQE.GT.0) THEN
         DO 21 IJ=1,NFREQE
            IJT=IJFR(IJ)
            RADM(IJ)=RADEX(IJ,ID-1)
            FKM(IJ)=FAKEX(IJ,ID-1)
            ABSOM(IJ)=ABSOEX(IJ,ID-1)
            EMISM(IJ)=EMISEX(IJ,ID-1)
            SCATM(IJ)=SCATEX(IJ,ID-1)
   21    CONTINUE
         END IF
      END IF
C
      IF(ID.LT.ND) THEN
         DO 27 I=1,NN
   27       PSIP(I)=PSY0(I,ID+1)
         IF(NFREQE.GT.0) THEN
         DO 28 IJ=1,NFREQE
            IJT=IJFR(IJ)
            RADP(IJ)=RADEX(IJ,ID+1)
            FKP(IJ)=FAKEX(IJ,ID+1)
            ABSOP(IJ)=ABSOEX(IJ,ID+1)
            EMISP(IJ)=EMISEX(IJ,ID+1)
            SCATP(IJ)=SCATEX(IJ,ID+1)
   28    CONTINUE
         END IF
      END IF
c
C
C     ------------------------------------------------------------
C     Actual evaluation of the rhs vector VECL
C     ------------------------------------------------------------
C
      DO 30 I=1,NN
         VECL(I)=0.
   30 CONTINUE
C
      IF(NFREQE.LE.0) GO TO 100
C
C     -----------------------------------------------------------
C     1. Radiative transfer components
C     -----------------------------------------------------------
C
C     For  ID = 1  -  upper boundary condition
C
      IF(ID.GT.1) GO TO 50
      DDP=(DM(2)-DM(1))*HALF 
      DO 40 IJ=1,NFREQE
         IJT=IJFR(IJ)
         OMEG0=ABSO0(IJ)/DENS(ID)
         OMEGP=ABSOP(IJ)/DENS(ID+1)
         DZP=OMEG0+OMEGP
         DTAUP=DZP*DDP
         ALF1=(FK0(IJ)*RAD0(IJ)-FKP(IJ)*RADP(IJ))/DTAUP
         CHIEL0=SCAT0(IJ)
         CHIELP=SCATP(IJ)
         S0=(EMIS0(IJ)+CHIEL0*RAD0(IJ))/ABSO0(IJ)
         BS=HALF*DTAUP
         CS=0.
         C2=0.
         SP=0.
         BET2=0.
         GAM2=0.
         IF(ISPLIN.GT.0) THEN
C
C      Spline collocation and/or Hermitian method (ISPLIN=1 or 2) -
C      both give the same expression for the boundary conditions
C
            BS=DTAUP*THIRD
            CS=HALF*BS
            SP=(EMISP(IJ)+CHIELP*RADP(IJ))/ABSOP(IJ)
            C2=CS/ABSOP(IJ)
            GAM2=CS*(RADP(IJ)-SP)
         END IF
C0
C      auxiliary quantities
C
         ALF2=BS*(RAD0(IJ)-S0)
         BET2=ALF2+GAM2
C
C *** the IJ-th element of the rhs vector
C
         VECL(IJ)=ALF1+BET2+FH(IJT)*RAD0(IJ)-S0*Q0(IJ)
         IF(IWINBL.LT.0) VECL(IJ)=VECL(IJ)-HALF*EXTRAD(IJT)
C
   40 CONTINUE
      GO TO 100
C
C     For  1 < ID < ND  -  normal depth point
C
   50 DDM=(DM(ID)-DM(ID-1))*HALF 
      IF(ID.EQ.ND) GO TO 80
      DDP=(DM(ID+1)-DM(ID))*HALF 
      DO 60 IJ=1,NFREQE
         OMEG0=ABSO0(IJ)/DENS(ID)
         OMEGP=ABSOP(IJ)/DENS(ID+1)
         OMEGM=ABSOM(IJ)/DENS(ID-1)
         DZP=OMEG0+OMEGP
         DZM=OMEG0+OMEGM
         DTAUP=DZP*DDP
         DTAUM=DZM*DDM
         DTAU0=HALF*(DTAUP+DTAUM)
         FRD=FK0(IJ)*RAD0(IJ)
         ALF1=(FRD-FKP(IJ)*RADP(IJ))/DTAUP/DTAU0
         GAM1=(FRD-FKM(IJ)*RADM(IJ))/DTAUM/DTAU0
         BET1=ALF1+GAM1
         X1=HALF*BET1/DTAU0
         A1=(GAM1+X1*DTAUM)/DZM
         C1=(ALF1+X1*DTAUP)/DZP
         B1=(A1+C1)/DENS(ID)
         A1=A1/DENS(ID-1)
         C1=C1/DENS(ID+1)
         BS=UN
         CHIELM=SCATM(IJ)
         CHIEL0=SCAT0(IJ)
         CHIELP=SCATP(IJ)
         S0=(EMIS0(IJ)+CHIEL0*RAD0(IJ))/ABSO0(IJ)
         BET2=0.
         IF(ISPLIN.EQ.1) THEN
C
C     spline collocation (ISPLIN=1)
C
            AS=DTAUM/DTAU0*SIXTH
            CS=DTAUP/DTAU0*SIXTH
            BS=0.666666666666667D0
            SM=(EMISM(IJ)+RADM(IJ)*CHIELM)/ABSOM(IJ)
            SP=(EMISP(IJ)+RADP(IJ)*CHIELP)/ABSOP(IJ)
            ALF2=AS*(RADM(IJ)-SM)
            GAM2=CS*(RADP(IJ)-SP)
            BET2=ALF2+GAM2
          ELSE IF(ISPLIN.EQ.2) THEN
C
C     Hermitian method (ISPLIN=2)
C
            AS=DTAUP*DTAUP/DTAUM/DTAU0
            CS=DTAUM*DTAUM/DTAUP/DTAU0
            SM=(EMISM(IJ)+RADM(IJ)*CHIELM)/ABSOM(IJ)
            SP=(EMISP(IJ)+RADP(IJ)*CHIELP)/ABSOP(IJ)
            AS=(UN-HALF*AS)*SIXTH
            CS=(UN-HALF*CS)*SIXTH
            BS=UN-AS-CS
            BET2=AS*(RADM(IJ)-SM)+CS*(RADP(IJ)-SP)
         END IF
C
C *** the IJ-th element of the rhs vector
C
         VECL(IJ)=BET1+BET2+BS*(RAD0(IJ)-S0)
C
   60 CONTINUE
      GO TO 100
C
C   For ID=ND  -  lower boundary condition
C
   80 T=TEMP(ID)
      HKT=HK/T
C
C     1. the case NDRE.le.ND, ie. radiative equilibrium equation at this
C        depth is treated as differential equation.
C
C     2. the case NDRE > ND, ie. radiative equilibrium equation at this
C        depth is treated as integral equation.
C        Boundary condition of the transfer equation is then complex,
C        because it has to incorporate the constraint of the total flux
C        equal to SIG4P*Teff**4 (integral form of the radiative
C        equation does not fix the total flux)
C
C        Details of this approach are presented in Mihalas, Heasley, Auer
C        NCAR-TN/STR-104 (1975).
C
C        This option is presented here for the sake of continuity with
C        previous work; otherwise the first option (NDRE.le.ND) is
C        recommended due to its better numerical properties
C
C        auxiliary quantities for the second option
C
         IF(ID.LT.NDRE.AND.INRE.NE.0) THEN
         SUMB=0.
         SUMF=0.
         DO 90 IJ=1,NFREQE
            IJT=IJFR(IJ)
            FR=FREQ(IJT)
            FR15=FR*1.D-15
            WD=WDEP0(IJ)
            X=HKT*FR
            EX=EXP(X)
            PLAN=BN*FR15*FR15*FR15/(EX-UN)
            DPLAN=PLAN*X/T/(UN-UN/EX)*WD
            SUMB=SUMB+(PLAN-RAD0(IJ))*WD
            FI=DPLAN/ABSO0(IJ)
            SUMF=SUMF+FI
   90    CONTINUE
         FL=SIG4P*TEFF*TEFF*TEFF*TEFF
         ZZ=(FL-HALF*SUMB)/SUMF
         END IF
C
C        auxiliary quantities for both options
C
         DO 95 IJ=1,NFREQE
            IJT=IJFR(IJ)
            CHIELM=SCATM(IJ)
            CHIEL0=SCAT0(IJ)
            OMEGM=ABSOM(IJ)/DENS(ID-1)
            OMEG0=ABSO0(IJ)/DENS(ID)
            DZM=OMEG0+OMEGM
            DTAUM=DZM*DDM
            GAM1=(FK0(IJ)*RAD0(IJ)-FKM(IJ)*RADM(IJ))/DTAUM
            BET2=0.
C
C      Spline collocation and/or Hermitian method (ISPLIN=1 or 2) -
C      both give the same expression for the boundary conditions
C
C      Commented out here for consistency with RTE
C
C           AS=0.
C           BS=0.
C           ALF2=0.
C           GAM2=0.
C           IF(ISPLIN.GT.0) THEN
C              BS=DTAUM*THIRD
C              AS=HALF *BS
C              S0=(EMIS0(IJ)+CHIEL0*RAD0(IJ))/ABSO0(IJ)
C              SM=(EMISM(IJ)+RADM(IJ)*CHIELM)/ABSOM(IJ)
C              ALF2=AS*(RADM(IJ)-SM)
C              GAM2=BS*(RAD0(IJ)-S0)
C              BET2=ALF2+GAM2
C           END IF
C
C
C        second-order boundary condition
C
            IF(IBC.GT.0) THEN
               BS=DTAUM*HALF
               S0=(EMIS0(IJ)+CHIEL0*RAD0(IJ))/ABSO0(IJ)
               BET2=BS*(RAD0(IJ)-S0)
            END IF
C
            FR=FREQ(IJT)
            X=HKT*FR
            EX=EXP(X)
            PLAN=BN*(FR*1.D-15)**3/(EX-UN)
            IF(INRE.EQ.0.OR.ID.GE.NDRE) THEN
               DPLAN=BN*(FR*1.D-15)**3/(EXP(HK*FR/TEMP(ID-1))-UN)
               GAM1=GAM1-(PLAN-DPLAN)/DTAUM*THIRD
            ELSE
               DPLAN=PLAN*X/T/(UN-UN/EX)
               FI=DPLAN/ABSO0(IJ)
               X1=FI*ZZ
               GAM1=GAM1-X1
            END IF
C
C *** the IJ-th element of the rhs vector
C
            IF(IBC.EQ.0) THEN
               VECL(IJ)=GAM1+BET2-HALF*(PLAN-RAD0(IJ))
             ELSE
               VECL(IJ)=GAM1+BET2-HALF*PLAN+FHD(IJT)*RAD0(IJ)
            END IF
   95    CONTINUE
C
C     -----------------------------------------------------------
C     2. Hydrostatic equilibrium
C     -----------------------------------------------------------
C
  100 isplin=ispl
      IF(INHE.EQ.0) GO TO 200
      NHE=NFREQE+INHE
      GRD=0.
C
      IF(ID.EQ.1) THEN
C
C     Upper boundary condition (ID=1)
C     Basically, linearized eq. (7-10) of Mihalas (1978)
C
         IF(NFREQE.GT.0) THEN
         DO 110 IJ=1,NFREQE
            IJT=IJFR(IJ)
            IF(LSKIP(IJT)) GO TO 110 
            FLUXW=(WDEP0(IJ)+WDC0(IJ))*FH(IJT)*RAD0(IJ)
            GRD=GRD+FLUXW*ABSO0(IJ)
  110    CONTINUE
         END IF
C
         X1=PCK/DENS(ID)
         VT0=HALF*VTURB(ID)*VTURB(ID)/DM(ID)*WMM(ID)
C
C     The rhs vector also accounts for the total radiation pressure in
C     the fixed-option transitions (array FPRD, generated by FIXLIN)
C
         VECL(NHE)=GRAV-BOLK*TEMP(ID)*PSI0(NHE)/DM(ID)-
     *            X1*(GRD+FPRD(ID))-VT0/WMM(ID)*DENS(ID)
C
      ELSE
C
C     Normal depth point (ID > 1)
C
C
         IF(NFREQE.GT.0) THEN
         DO 150 IJ=1,NFREQE
            IF(LSKIP(IJFR(IJ))) GO TO 150
            GRD=GRD+(FK0(IJ)*RAD0(IJ)-FKM(IJ)*RADM(IJ))*
     *           (WDEP0(IJ)+WDC0(IJ))
  150    CONTINUE
         END IF
C
         VT0=HALF*VTURB(ID)*VTURB(ID)*WMM(ID)
         VTM=HALF*VTURB(ID-1)*VTURB(ID-1)*WMM(ID-1)
C
C     the rhs vector
C     again, which accounts for the total radiation pressure in the
C     fixed-option transitions (array FPRD, generated by FIXLIN)
C
         VECL(NHE)=GRAV*(DM(ID)-DM(ID-1))-
     *            BOLK*(TEMP(ID)*PSI0(NHE)-TEMP(ID-1)*PSIM(NHE))-
     *            PCK*(GRD+FPRD(ID))-
     *            VT0/WMM(ID)*DENS(ID)+VTM/WMM(ID-1)*DENS(ID-1)
      END IF
C
C     -----------------------------------------------------------
C     3. Radiative equilibrium
C     -----------------------------------------------------------
C
  200 IF(INRE.EQ.0) GO TO 250
      NRE=NFREQE+INRE
C
C     integral equation part of the radiative
C     equilibrium equation
C
C     the rhs vector accounts for total net cooling in ALI transitions
C
         VECL(NRE)=FCOOL(ID)
C
      if(reint(id).gt.0) then
         IF(NFREQE.GT.0) THEN
         DO 210 IJ=1,NFREQE
            HEAT=ABSO0(IJ)-SCAT0(IJ)
            VECL(NRE)=VECL(NRE)-
     *                (HEAT*RAD0(IJ)-EMIS0(IJ))*WDEP0(IJ)*reint(id)
  210    CONTINUE
         END IF
      end if
C
C
C     differential equation part of the
C     radiative equilibrium equation
C
      if(redif(id).gt.0) then
         VECL(NRE)=vecl(nre)+SIG4P*TEFF**4*redif(id)
         IF(ID.GT.1) THEN
         DDM=(DM(ID)-DM(ID-1))*HALF 
         IF(NFREQE.GT.0) THEN
         DO 220 IJ=1,NFREQE
            OMEG0=ABSO0(IJ)/DENS(ID)
            OMEGM=ABSOM(IJ)/DENS(ID-1)
            DTAUM=(OMEG0+OMEGM)*DDM
            FRD=FK0(IJ)*RAD0(IJ)-FKM(IJ)*RADM(IJ)
            GAMR=FRD/DTAUM
            VECL(NRE)=VECL(NRE)-WDEP0(IJ)*GAMR*redif(id)
  220    CONTINUE
         END IF
         ELSE
         IF(NFREQE.GT.0) THEN
         DO 225 IJ=1,NFREQE
            IJT=IJFR(IJ)
            VECL(NRE)=VECL(NRE)-WDEP0(IJ)*FH(IJT)*RAD0(IJ)*REDIF(ID)
  225    CONTINUE
         END IF
         END IF
       end if
C
C     -----------------------------------------------------------
C     4. Statistical equilibrium
C     -----------------------------------------------------------
C
  250 IF(IABS(IFPOPR).GE.3.and.ifpopr.le.5) THEN
         NSE=NFREQE+INSE-1
         CALL SABOLF(ID)
         CALL RATMAT(ID,IIEXP,0,ESEMAT,BESE)
         IF(IFPOPR.LE.3) THEN
            CALL MATINV(ESEMAT,NLVEXP,MLEVEL)
            DO 270 I=1,NLVEXP
               SUM=0.
               DO 260 J=1,NLVEXP
  260             SUM=SUM+ESEMAT(I,J)*BESE(J)
               VECL(NSE+I)=POPUL(INDLEV(I),ID)-SUM
  270       CONTINUE
          ELSE 
            DO 290 I=1,NLVEXP
               SUM=0.
               DO 280 J=1,NLVEXP
                  SUM=SUM+ESEMAT(I,J)*POPUL(INDLEV(J),ID)
  280          CONTINUE
               VECL(NSE+I)=BESE(I)-SUM
  290       CONTINUE
         END IF
      END IF
C
C     -----------------------------------------------------------
C     5. Charge or particle conservation (depending on LCHC)
C     -----------------------------------------------------------
C
      IF(INPC.EQ.0) GO TO 400
      NPC=NFREQE+INPC
      IF(LCHC) THEN
C
C     a) Linearized particle conservation (LCHC=true)
C
      SUMH=0.
      DO 310 I=N0A(IATREF),NKA(IATREF)
         IL=ILK(I)
         IF(IL.LE.0) THEN
            SUMH=SUMH+POPUL(I,ID)
          ELSE
            SUMH=SUMH+(UN+ANE*USUMS(IL,ID))*POPUL(I,ID)
         END IF
  310 CONTINUE
      VECL(NPC)=DENS(ID)/WMM(ID)/YTOT(ID)*ABUND(IATREF,ID)-SUMH
      ELSE
C
C     b) Linearized charge conservation equation (LCHC=false)
C
C     This part is very similar to procedure ELCOR (obviously);
C     array AJ has the meaning of coefficients of the charge conserv.
C     ie. charge conservation is written
C           AJ * (vector of populations) = electron density
C
      T=TEMP(ID)
      ANE=ELEC(ID)
      CALL STATE(2,ID,T,ANE)
C
      VPC=QFIX(ID)+Q*DENS(ID)/WMM(ID)/YTOT(ID)*ABUND(IATREF,ID)
      DO 340 IAT=1,NATOM
         IF(IIFIX(IAT).EQ.1) GO TO 340
         DO 330 I=N0A(IAT),NKA(IAT)
            IL=ILK(I)
            CH=IZ(IEL(I))-1
            IF(IL.GT.0) CH=IZ(IL)+(IZ(IL)-1)*ANE*USUMS(IL,ID)
            IF(IIFOR(I).NE.0) VPC=VPC+CH*POPUL(I,ID)
  330    CONTINUE
  340 CONTINUE
C
      VECL(NPC)=ANE-VPC
      END IF
C
C     -----------------------------------------------------------
C     6. Convection
C     -----------------------------------------------------------
C
  400 IF(HMIX0.LE.0.) RETURN
      NRE=NFREQE+INRE
      NDEL=NFREQE+INDL
C
C     Upper boundary condition (ID=1)
C
      ANEREL=ELEC(1)/(DENS(1)/WMM(1)+ELEC(1))
      IF(ID.EQ.1) THEN
         DELTA(ID)=0.
         FLXC(ID)=0.
      ELSE
C
C     Normal depth point 1 < ID < ND
C
         T=TEMP(ID)
         P=PTOTAL(ID)
         PG=PGS(ID)
         PRAD=P-PG-HALF*DENS(ID)*VTURB(ID)**2
         TM=TEMP(ID-1)
         PM=PTOTAL(ID-1)
         PGM=PGS(ID-1)
         PRADM=PM-PGM-HALF*DENS(ID-1)*VTURB(ID-1)**2
         T0=HALF*(T+TM)
         P0=HALF*(P+PM)
         PG0=HALF*(PG+PGM)
         PR0=HALF*(PRAD+PRADM)
         DLT=(T-TM)/(P-PM)*P0/T0
         DELTA(ID)=DLT
         IF(INDL.GT.0) VECL(NDEL)=DELTA(ID)-DLT
C
C        convective flux
C
         if(redif(id).gt.0) then
         CALL CONVEC(ID,T0,P0,PG0,PR0,ABROSD(ID),DLT,FLXCNV,VCON)
         FLXC(ID)=FLXCNV
         VECL(NRE)=VECL(NRE)-FLXC(ID)*redif(id)
         end if
C
         if(reint(id).gt.0.AND.ICONV.GT.2) then
         CALL CONVEC(ID,T,P,PG,PRAD,ABROSD(ID),DLT,FLXCNV,VCON)
         DLTM=DELTA(ID-1)
         CALL CONVEC(ID-1,TM,PM,PGM,PRADM,ABROSD(ID-1),DLTM,FLXCM,VCON)
         DELM=DM(ID)-DM(ID-1)
         RDELM=(DENS(ID)+DENS(ID-1))*HALF/DELM
         VECL(NRE)=VECL(NRE)-RDELM*(FLXCNV-FLXCM)*reint(id)
         end if
      END IF
C
      RETURN
      END
C
C
C
C     ****************************************************************
C
C

      SUBROUTINE PRCHAN(CHANG,CHM)
C     ============================
C
C     Diagnostic output of relative changes of vector PSI
C
C     Input:
C      CHANG - array of relative changes of vector PSI at depth ID
C      ID    - depth index
C     Output:
C      CHM   - maximum relative change of all unknowns at all depths
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'ITERAT.FOR'
      DIMENSION CHANG(MTOT,MDEPTH),CHANM(MDEPTH)
C
C     maximum relative change of all unknowns in depth ID
C
      CHMT=0.
      DO 50 ID=ND,1,-1
         CH=0.
         chrad=0.
         chpop=0.
         cht=0.
         che=0.
         DO 10 I=1,NN
            IF(ITER.EQ.1.AND.I.EQ.NFREQE+INDL) GO TO 10
            IF(ABS(CHANG(I,ID)).LT.ABS(CH)) GO TO 10
            CH=CHANG(I,ID)
   10    CONTINUE
         CHANM(ID)=CH
         IF(NFREQE.GT.0) THEN
         DO 11 I=1,NFREQE
            IF(ABS(CHANG(I,ID)).LT.ABS(CHRAD)) GO TO 11
            CHRAD=CHANG(I,ID)
            jjr=i
   11    CONTINUE
         END IF
         DO 12 I=NFREQE+INSE,NFREQE+INSE+NLVEXP-1
            IF(ABS(CHANG(I,ID)).LT.ABS(CHPOP)) GO TO 12
            CHpop=CHANG(I,ID)
            jjp=i-nfreqE-inse+1
   12    CONTINUE
         if(inre.gt.0) then
            cht=chang(nfreqe+inre,id)
            if(abs(cht).ge.abs(chmt)) chmt=abs(cht)
         end if
         if(inpc.gt.0) che=chang(nfreqe+inpc,id)
C
C     output onto file 9
C
         IF(ID.EQ.ND.AND.ITER.EQ.1) WRITE(9,800)
         WRITE(9,801) ITER,ID,cht,che,CHpop,CHrad,ch,jjp,jjr
c        WRITE(19,802) (CHANG(NFREQE+I,ID),I=1,NN-NFREQE)
  800    FORMAT(' RELATIVE CHANGES OF VECTOR PSI'/
     *   '  ITER  ID    TEMP        NE       POP     RAD      MAXIMUM',
     *   '   ilev   ifr',/)
  801    FORMAT(2I5,1P5D10.2,0p2i5)
  802    FORMAT(10X,1P7D10.2)
   50 CONTINUE
C
C     determination of the maximum relative change of all unknowns
C     at all depths
C
      CHM=0.
      DO 20 I=1,ND
         IF(ABS(CHANM(I)).GE.ABS(CHM)) CHM=CHANM(I)
   20 CONTINUE
C
      if(chmt.lt.chmaxt) nlambd=nlamt
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE OPADD(MODE,ICALL,IJ,ID)
C     ==================================
C
C     Additional opacities
C     This is basically user-supplied procedure; here are some more
C     important non-standard opacity sources, namely
C     Rayleigh scattering, H- opacity, H2+ opacity, and additional
C     opacity of He I and He II.
C     Inclusion of these opacities is contolled by switches transmitted
C     by COMMON/OPCKEY - see description in START.
C
C     Input parameters:
C     MODE  - controls the nature and the amount of calculations
C           = -1 - (OPADD called from START) evaluation of relevant
C                  depth-dependent quantities (usually photoionization
C                  cross-sections, but also possibly other), which are
C                  stored in array CROS
C           = 0  - evaluation of an additional opacity, emissivity, and
C                  scattering - for procedure OPACF1
C           = 1  - the same, but also derivatives wrt temperature,
C                  electron density, and level populations
C     IJ    - frequency index
C     ID    - depth index
C
C     Output: transmitted by COMMON/OPACAD:
C
C     ABAD  - absorption coefficient (at frequency point IJ and depth ID)
C     EMAD  - emission coefficient (at frequency point IJ and depth ID)
C     SCAD  - scattering coefficient (at frequency point IJ and depth ID)
C     Dxy   - derivatives of x (=A for Absorption, =E for Emission,
C             =S for scattering) coefficient wrt y (=T for temperature,
C             =N for electron density)
C     DDN(I)  - quantity proportional to a derivative of absorption (and
C               emission) coefficient wrt population of I-th level; ie.
C               d(abs)/d(pop) = DDN * [1-exp(-h*nu/kT)]
C               d(em)/d(pop) = DDN * (2h*nu**3/c**2)*exp(-h*nu/kT)
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/OPACAD/ABAD,EMAD,SCAD,DAT,DAN,DET,DEN,DST,DSN,DDN(MLEVEL)
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
      PARAMETER (FRRAY =  2.463D15,
     *           C18   =  2.997925D18,
     *           CR0   =  5.799D-13,
     *           CR1   =  1.422D-6,
     *           CR2   =  2.784D0,
     *           TENM4 =  1.0D-4,
     *           THM0  =  8.7629D3,
     *           SBHM  =  1.0353D-16,
     *           TRHA  =  1.5D0,
     *           SFF0  =  1.3727D-25,
     *           SFF1  =  4.3748D-10,
     *           SFFM2 =  -2.5993D-7,
     *           F0HE1 =  3.29D15,
     *           F0HE2 =  1.316D16,
     *           SBH0  =  4.1412D-16,
     *           SG01  =  2.815D-16,
     *           SG02  =  4.504D-15)
      SAVE T,DELTAT,ANE,HKT,T32,XHM,POPI,SB00
C
      AB0=0.
      AB1=0.
      DAB0=0.
      DAB1=0.
      ABAD=0.
      EMAD=0.
      SCAD=0.
      DBF=0.
      DFF=0.
      DAT=0.
      DET=0.
      DAN=0.
      DEN=0.
      DST=0.
      DSN=0.
      DO 10 I=1,NLEVEL
   10    DDN(I)=0.
C
      FR=FREQ(IJ)
      if(ielh.gt.0) then
      N0HN=NFIRST(IELH)
      NKH=NKA(IATH)
      end if
C
      IF(ICALL.GT.0) THEN
         T=TEMP(ID)
         DELTAT=TENM4*T
         ANE=ELEC(ID)
         HKT=HK/T
         T32=UN/T/SQRT(T)
         XHM=THM0/T
         if(ielh.gt.0) POPI=POPUL(N0HN,ID)
         SB00=SBHM*T32*EXP(XHM)*POPI*ANE
      END IF
C
      IT=NCON
C
C   -----------------------
C   HI  Rayleigh scattering
C   -----------------------
C
      IF(IRSCT.NE.0) THEN
      IT=IT+1
      IF(MODE.LT.0) THEN
         FRM=MIN(FR,FRRAY)
         X=(C18/FRM)**2
         if(it.gt.mcross) call stop('too many cross-sections in opadd')
         if(ielh.le.0) call stop('conflict in irstc - no hydrogen')
         BFCS(IT,IJ)=(CR0+(CR1+CR2/X)/X)/X/X
       ELSE
         ABAD=POPI*CROSS(IT,IJ)
         SCAD=ABAD
      END IF
      END IF
      IF(IOPHMI.NE.0) THEN
C
C   ----------------------------
C   H-  bound-free and free-free
C   ----------------------------
C     Note: IOPHMI must not by taken non-zero if H- is considered
C           explicitly, because H- opacity would be taken twice
C
      IT=IT+1
      IF(MODE.LT.0) THEN
         if(it.gt.mcross) call stop('too many cross-sections in opadd')
         if(ielh.le.0) call stop('conflict in iophmi - no hydrogen')
         BFCS(IT,IJ)=SBFHMI(FR)
       ELSE
          SB=SB00*CROSS(IT,IJ)
          SF=SFF0+(SFF1+SFFM2/T)/FR
          PFR=POPI/FR*ANE
          AB0=SB+SF*PFR  
          IF(POPI.GT.0.) DDN(N0HN)=DDN(N0HN)+(SB+SF*PFR)/POPI
          IF(MODE.EQ.1) THEN 
            DSF=SFFM2/T/FR
            DAB0=DAB0+SB*(TRHA+XHM)+DSF*PFR
          END IF
      END IF
      END IF

C
      IF(IOPH2P.GT.0) THEN
C
C   -----------------------------
C   H2+  bound-free and free-free
C   -----------------------------
C
      IT=IT+1
      IF(MODE.LT.0) THEN
         X=FR*1.D-15
         BFCS(IT,IJ)=(-7.342D-3+(-2.409+(1.028+(-4.23D-1+
     *        (1.224D-1-1.351D-2*X)*X)*X)*X)*X)*1.602D-12/BOLK
         IT=IT+1
         if(it.gt.mcross) call stop('too many cross-sections in opadd')
         if(ielh.le.0) call stop('conflict in ioph2p - no hydrogen')
         X=LOG(FR)
         BFCS(IT,IJ)=-3.0233D3+(3.7797D2+(-1.82496D1+(3.9207D-1-
     *                 3.1672D-3*X)*X)*X)*X
       ELSE
         X2=-CROSS(IT,IJ)/T+CROSS(IT+1,IJ)
         IT=IT+1
         SB=0.
         IF(X2.GT.-60.) SB=POPUL(N0HN,ID)*EXP(X2)*POPUL(NKH,ID)
         AB1=SB
         DAB1=SB*CROSS(IT-1,IJ)/T/T
         DDN(N0HN)=DDN(N0HN)+SB/POPI
         IF(POPUL(NKH,ID).GT.0) DDN(NKH)=DDN(NKH)+SB/POPUL(NKH,ID)
      END IF
      END IF
C
C   ----------------------
C   neutral helium opacity
C   ----------------------
C     approximate, hydrogenic, opacity of neutral helium
C     given as a sum of bound-free transitions from averaged
C     levels with principal quantum numbers between that next
C     to the highest level considered explicitly and IOPHE1
C
      IF(IOPHE1.GT.0) THEN
      NHE2=NNEXT(IELHE1)
      I1=NQUANT(NLAST(IELHE1))+1
      IF(MODE.LT.0) THEN
         SG0=SG01/(FR*1.D-15)**3
         DO 20 I=I1,IOPHE1
            IT=IT+1
         if(it.gt.mcross) call stop('too many cross-sections in opadd')
            FRI=F0HE1/I/I
            SG=0.
            IF(FR.GE.FRI) SG=SG0*GAUNT(I,FR)/I**5
            BFCS(IT,IJ)=SG
   20    CONTINUE
       ELSE
         SB=SBH0*T32
         DO 30 I=I1,IOPHE1
            IT=IT+1
            FRI=F0HE1/I/I
            IF(FR.LT.FRI) GO TO 30
            XI=HKT*FRI
            SG=SB*EXP(XI)*I*I*POPUL(NHE2,ID)*ANE*CROSS(IT,IJ)
            AB0=AB0+SG
            DAB0=DAB0+(TRHA+XI)*SG
            IF(POPUL(NHE2,ID).GT.0) DDN(NHE2)=DDN(NHE2)+
     *                             SG/POPUL(NHE2,ID)
   30    CONTINUE
      END IF
      END IF
C
C   --------------
C   ionized helium (analogously as for neutral helium)
C   --------------
C
      IF(IOPHE2.GT.0) THEN
      NHE3=NNEXT(IELHE2)
      I1=NQUANT(NLAST(IELHE2))+1
      IF(MODE.LT.0) THEN
         SG0=SG02/(FR*1.D-15)**3
         DO 40 I=I1,IOPHE2
            IT=IT+1
         if(it.gt.mcross) call stop('too many cross-sections in opadd')
            FRI=F0HE2/I/I
            SG=0.
            IF(FR.GE.FRI) SG=SG0*GAUNT(I,FR/4.D0)/I**5
            BFCS(IT,IJ)=SG
   40    CONTINUE
       ELSE
         SB=SBH0*T32
         DO 50 I=I1,IOPHE2
            IT=IT+1
            FRI=F0HE2/I/I
            IF(FR.LT.FRI) GO TO 50
            XI=HKT*FRI
            SG=SB*EXP(XI)*I*I*POPUL(NHE3,ID)*ANE*CROSS(IT,IJ)
            AB0=AB0+SG
            DAB0=DAB0+(TRHA+XI)*SG
            IF(POPUL(NHE3,ID).GT.0) DDN(NHE3)=DDN(NHE3)+
     *                              SG/POPUL(NHE3,ID)
   50    CONTINUE
      END IF
      END IF
C
C     ----------------------------------------------
C     The user may supply more opacity sources here:
C     ----------------------------------------------
C
C     Finally, actual absorption and emission coefficients and
C     their derivatives

      IF(MODE.LT.0) RETURN
      X=EXP(-HKT*FR)
      X1=UN-X
      FR15=FR*1.D-15
      BNX=BN*FR15*FR15*FR15*X
      ABAD=ABAD+X1*(AB0+AB1)
      EMAD=EMAD+BNX*(AB0+AB1)
      IF(MODE.EQ.1) THEN
         HKFT=HKT*FR/T
         DB=HKFT*AB0
         DAT=-(X1*DAB0/T+X*DB)+X1*DAB1-X*AB1*HKFT
         DET=BNX*(DB-DAB0/T+DAB1+AB1*HKFT)
         DAN=X1*AB0/ANE
         DEN=BNX*AB0/ANE
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C
C

      SUBROUTINE PARTF(IAT,IZI,T,ANE,XMAX,U,DUT,DUN)
C     ==============================================
C
C     Partition functions 
C     The standard evaluation is for hydrogen through zinc, for
C     neutrals and first four ionization degrees.
C     Basically after Traving, Baschek, and Holweger, Abhand. Hamburg.
C     Sternwarte. Band VIII, Nr. 1 (1966)
C
C     The routine calls special procedures for Fe and Ni; or
C     the values based on the tabulated Opacity Project ionization
C     fractions
C
C     Input:
C      IAT   - atomic number
C      IZI   - ionic charge (=1 for neutrals, =2 for once ionized, etc)
C      T     - temperature
C      ANE   - electron density
C      XMAX  - principal quantum number of the last bound level
C
C     Output:
C      U     - partition function
C      DUT   - derivative dU/dT
C      DUN   - derivative dU/d(ANE)
C
C     Quantities in COMMON/PFSTDS:
C      for MODPF(IAT) < 0  - non-standard, user supplied procedure
C                            for evaluating partition functions PFSPEC
C                     = 0  - standard expressions
C                            After Traving, Baschek, and Holweger, Abhand.
C                            Hamburg. Sternwarte, Band VIII, Nr. 1 (1966)
C                     > 0  - partition functions evaluated from the
C                            Opacity Project ionization fractions
C                            (by routine OPFRAC)
C      PFSTD(IAT,IZI) - see above
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (NIONS=123, NSS=222)
      PARAMETER (UN=1.D0, HALF=0.5D0, TWO=2.D0, TRHA=1.5D0,
     *           THIRD=UN/3.D0, SIXTH=UN/6.D0)
      REAL*4 AHH( 6),  ALB(12),  AB (11),  AC (19),  AN (30),  AO (49),
     *       AF (34),  ANN(23),  ANA(19),  AMG(15),  AAL(17),  ASI(23),
     *       AP (19),  AS (29),  ACL(28),  AAR(25),  AK (30),  ACA(17),
     *       ASC(24),  ATI(33),  AV (33),  ACR(29),  AMN(28),  AFE(35),
     *       ACO(29),  ANI(23),  ACU(20),  AZN(18)
      REAL*4 GHH( 6),  GLB(12),  GB (11),  GC (19),  GN (30),  GO (49),
     *       GF (34),  GNN(23),  GNA(19),  GMG(15),  GAL(17),  GSI(23),
     *       GP (19),  GS (29),  GCL(28),  GAR(25),  GK (30),  GCA(17),
     *       GSC(24),  GTI(33),  GV (33),  GCR(29),  GMN(28),  GFE(35),
     *       GCO(29),  GNI(23),  GCU(20),  GZN(18)
      REAL*4 XL1(99), XL2(123),  XL(222),
     *       CH1(66),  CH2(72),  CH3(55),  CH4(29),  CHION(222)
      REAL*4 ALF(678), GAM(678)
      INTEGER*2 II1(5,15),II2(5,15),INDEX0(5,30),
     *          IS1(53),IS2(70),IS(123),INDEXS(123),
     *          IM1(99),IM2(123),IM(222),INDEXM(222),
     *          IGP1(99),IGP2(123),IGPR(222),
     *          IG01(53),IG02(70),IG0(123)
      COMMON/PFSTDS/PFSTD(30,30),MODPF(30)
C
C    Internal numbering of ions; each line corresponds to one species,
C    starting from hydrogen.
C    0 means that the ion does not exist
C    negative number means that partition function is assumed constant
C    (ie even for MODPF=0) and equal to ABS(that value)
C
      DATA II1      /   1,  -1,   0,   0,   0,
     *                  2,   3,  -1,   0,   0,
     *                  4,   5,  -2,  -1,   0,
     *                  6,   7,  -1,  -2,  -1,
     *                  8,   9,  10,  -1,  -2,
     *                 11,  12,  13,  14,  -1,
     *                 15,  16,  17,  18,  19,
     *                 20,  21,  22,  23,  24,
     *                 25,  26,  27,  28,  -6,
     *                 29,  30,  31,  32,  -9,
     *                 33,  34,  35,  36,  -4,
     *                 37,  38,  39,  40,  -9,
     *                 41,  42,  43,  44,  -6,
     *                 45,  46,  47,  48,  -1,
     *                 49,  50,  51,  52,  53                         /
      DATA II2      /  54,  55,  56,  57,  58,
     *                 59,  60,  61,  62,  63,
     *                 64,  65,  66,  67,  68,
     *                 69,  70,  71,  72,  73,
     *                 74,  75,  76,  77,  -9,
     *                 78,  76,  80,  81,  82,
     *                 83,  84,  85,  86,  87,
     *                 88,  89,  90,  91,  92,
     *                 93,  94,  95,  96,  97,
     *                 98,  99, 100, 101, 102,
     *                103, 104, 105, 106, 107,
     *                108, 109, 110, 111, -25,
     *                112, 113, 114, 115,  -1,
     *                116, 117, 118, 119,  -1,
     *                120, 121, 122, 123,  -1                         /
C
C     Statistical weights of the ground states
C
      DATA IG01     /   2,
     *                  1,   2,
     *                  2,   1,
     *                  1,   2,
     *                  2,   1,   2,
     *                  1,   2,   1,   2,
     *                  4,   1,   2,   1,   2,
     *                  5,   4,   1,   2,   1,
     *                  4,   5,   4,   1,
     *                  1,   4,   5,   4,
     *                  2,   1,   4,   5,
     *                  1,   2,   1,   4,
     *                  2,   1,   2,   1,
     *                  1,   2,   1,   2,
     *                  4,   1,   2,   1,   2                         /
      DATA  IG02    /   5,   4,   1,   2,   1,
     *                  4,   5,   4,   1,   2,
     *                  1,   4,   5,   4,   1,
     *                  2,   1,   4,   5,   4,
     *                  1,   2,   1,   4,
     *                  4,   3,   4,   1,   4,
     *                  5,   4,   5,   4,   1,
     *                  4,   1,   4,   5,   4,
     *                  7,   6,   1,   4,   5,
     *                  6,   7,   6,   1,   4,
     *                  9,  10,   9,   6,   1,
     *                 10,   9,  10,  20,
     *                  9,   6,   9,  28,
     *                  2,   1,   6,  21,
     *                  1,   2,   1,  10                              /
C
      DATA IS1      /   1,
     *                  1,   1,
     *                  1,   1,
     *                  2,   1,
     *                  1,   2,   1,
     *                  1,   2,   2,   1,
     *                  2,   2,   3,   2,   1,
     *                  3,   4,   3,   5,   2,
     *                  2,   3,   4,   3,
     *                  2,   2,   3,   2,
     *                  1,   2,   2,   3,
     *                  1,   1,   2,   2,
     *                  2,   2,   1,   2,
     *                  1,   2,   2,   1,
     *                  2,   1,   1,   1,   1                         /
      DATA  IS2     /   3,   2,   1,   2,   2,
     *                  2,   3,   2,   1,   1,
     *                  2,   2,   3,   1,   1,
     *                  1,   2,   3,   3,   2,
     *                  2,   1,   2,   2,
     *                  3,   1,   1,   1,   1,
     *                  3,   2,   1,   1,   1,
     *                  2,   3,   1,   1,   1,
     *                  3,   2,   1,   1,   1,
     *                  3,   2,   1,   1,   1,
     *                  3,   2,   2,   1,   1,
     *                  4,   2,   1,   1,
     *                  2,   2,   1,   1,
     *                  3,   2,   1,   1,
     *                  3,   3,   1,   1                              /
C
      DATA IM1      /   2,
     *                  2,   2,
     *                  2,   2,
     *                  3,   2,   3,
     *                  3,   3,   2,   3,
     *                  4,   3,   3,   3,   3,   3,
     *                  3,   3,   4,   3,   3,   4,   2,   3,   2,   3,
     *                  4,   2,   2,   4,   2,   3,   3,   4,   4,   2,
     *                  3,   4,   2,   2,   2,   3,   3,
     *                  3,   3,   4,   2,   2,
     *                  4,   2,   3,   2,   5,   2,   2,
     *                  2,   2,   3,   2,   4,   2,   2,   4,   2,
     *                  2,   2,   2,   3,   2,   4,   2,   2,
     *                  3,   3,   2,   2,   3,   2,
     *                  3,   2,   3,   2,   3,   2,   2,
     *                  5,   4,   4,   4,   3,   3,
     *                  3,   2,   4,   4,   3,   3                    /
      DATA  IM2     /   4,   2,   2,   4,   2,   5,   4,   2,   3,   1,
     *                  3,   2,   5,   2,   2,   4,   2,   4,   4,
     *                  2,   2,   3,   2,   4,   2,   2,   4,   4,
     *                  3,   2,   3,   3,   2,   3,
     *                  4,   2,   2,   4,   2,
     *                  3,   2,   3,   2,   2,   3,   2,
     *                  4,   3,   3,   5,   4,   2,   3,
     *                  6,   4,   3,   6,   3,   5,   4,   2,
     *                  5,   3,   5,   4,   4,   4,   4,   4,
     *                  3,   3,   3,   4,   4,   4,   4,   4,
     *                  3,   2,   3,   4,   4,   4,   4,   4,
     *                  4,   4,   3,   5,   3,   4,   4,   4,   4,
     *                  5,   3,   3,   3,   5,   4,   5,   1,
     *                  6,   3,   5,   3,   5,   1,
     *                  2,   3,   3,   4,   3,   4,   1,
     *                  2,   2,   2,   3,   3,   2,   3,   1          /
C
      DATA IGP1     /   2,
     *                  4,   2,
     *                  2,   4,
     *                  4,  12,   2,
     *                  2,   4,  12,   2,
     *                 12,   2,  18,   4,  12,   2,
     *                 18,  10,  12,  24,   2,  18,   6,   4,  12,   2,
     *                  8,  20,  12,  18,  10,   2,  10,  12,  24,  20,
     *                  2,  18,   6,  18,  10,   4,  12,
     *                 18,  10,   8,  20,  12,
     *                 18,  10,   2,  10,  12,  24,  20,
     *                  8,   4,  18,  10,   8,  20,  12,  18,  10,
     *                  2,   8,   4,  18,  10,   8,  20,  12,
     *                  4,   2,   8,   4,  18,  10,
     *                  2,  18,   4,  12,   2,   8,   4,
     *                 12,   2,  18,   4,  12,   2,
     *                 18,  10,  12,   2,   4,   2                    /
      DATA  IGP2    /   8,  20,  12,  18,  10,  12,   2,  18,   4,  12,
     *                 18,  10,   8,  20,  12,  18,  10,  12,   2,
     *                  8,   4,  18,  10,   8,  20,  12,  18,  12,
     *                  2,   8,   4,  18,  10,   2,
     *                  8,  20,  12,  18,  10,
     *                  4,  20,   2,   8,   4,  18,  10,
     *                 30,  42,  18,  20,   2,  12,  18,
     *                 56,  56,  28,  42,  10,  20,   2,  12,
     *                 50,  70,  56,  72,  64,  42,  20,   2,
     *                 12,  60,  40,  50,  18,  56,  42,  20,
     *                 14,  10,  50,  12,  72,  50,  56,  42,
     *                 60,  56,  40,  50,  18,  12,  72,  50,  56,
     *                 42,  70,  42,  18,  56,  24,  50,  12,
     *                 20,  56,  42,  18,  56,  50,
     *                  2,  30,  10,  20,  56,  42,  56,
     *                  4,   8,  12,   2,  30,  10,  20,  42          /
C
      DATA XL1      /11.0,
     *                8.0,12.0,
     *                6.0, 6.0,
     *                6.0, 4.0, 8.0,
     *                9.0, 6.0, 4.0, 6.0,
     *                6.0, 6.0, 5.0, 6.1, 5.0, 6.0,
     *                6.1, 4.0, 5.0, 3.9, 6.0, 5.0, 4.0, 6.0, 6.3, 6.0,
     *                8.0, 6.0, 3.4, 6.0, 5.0, 3.9, 3.9, 6.0, 4.9, 4.0,
     *                5.9, 5.0, 4.9, 4.0, 4.0, 6.0, 6.0,
     *                4.0, 4.0, 5.0, 4.0, 4.0,
     *                5.0, 4.0, 3.9, 4.0, 5.0, 5.0, 4.0,
     *                6.0, 6.0, 5.0, 4.0, 3.9, 4.0, 4.0, 5.0, 5.0,
     *                7.0, 4.0, 4.0, 4.0, 4.0, 5.0, 5.0, 5.0,
     *                7.0, 7.0, 5.0, 5.0, 5.0, 5.0,
     *                7.0, 4.0, 7.0, 4.0, 7.0, 5.0, 5.0,
     *                6.1, 5.9, 5.0, 5.0, 5.0, 7.0,
     *                5.0, 5.0, 5.0, 7.0, 8.6, 8.0                    /
      DATA  XL2     / 6.0, 5.0, 5.0, 5.0, 5.0, 3.5, 5.0,14.4, 5.0, 4.0,
     *                6.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.2,
     *                6.0, 6.0, 5.1, 5.0, 5.0, 5.0, 5.0, 5.0, 4.0,
     *                7.0, 5.0, 5.0, 6.0, 6.0, 5.0,
     *                6.0, 5.0, 5.0, 3.6, 4.0,
     *                5.9, 6.0, 7.0, 5.0, 4.9, 5.0, 4.3,
     *                4.9, 4.9, 5.0, 5.0, 6.0, 4.6, 3.8,
     *                5.0, 4.7, 5.0, 5.0, 5.0, 5.0, 6.0, 4.8,
     *                5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0,11.2,
     *                5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.2,
     *                6.0, 5.0, 6.0, 7.0, 5.0, 5.0, 5.0, 5.0,
     *                5.0, 5.0, 5.0, 5.0, 5.0, 6.0, 5.0, 3.6, 3.8,
     *                5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 3.0,
     *                5.4, 5.0, 9.0, 5.0, 5.0, 3.0,
     *                8.0, 6.0, 5.0, 7.0, 5.0, 5.0, 2.9,
     *                8.0, 5.0, 5.0, 8.0, 5.0, 5.0, 5.0, 2.8          /
C
C
      DATA CH1      /  13.595 ,
     *                 24.580 ,  54.403 ,
     *                  5.390 , 75.619 ,
     *                  9.320 ,  13.278 ,  18.206 ,
     *                  8.296 ,  25.149 ,  31.146 ,  37.920 ,
     *                 11.256 ,  24.376 ,  30.868 ,  47.871 ,  55.873 ,
     *                 64.476 ,
     *                 14.529 ,  16.428 ,  29.593 ,  36.693 ,
     *                 47.426 ,  55.765 ,  63.626 ,  77.450 ,  87.445 ,
     *                 97.863 ,
     *                 13.614 ,  16.938 ,  18.630 ,
     *                 35.108 ,  37.621 ,  40.461 ,  42.584 ,
     *                 54.886 ,  63.733 ,  70.556 ,
     *                 77.394 ,  87.609 ,  97.077 , 103.911 , 106.116 ,
     *                113.873 , 125.863 ,
     *                 17.418 ,  20.009 ,  34.977 ,  39.204 ,  41.368 ,
     *                 62.646 ,  65.774 ,  69.282 ,  71.882 ,
     *                 87.139 ,  97.852 , 106.089 ,
     *                 21.559 ,  21.656 ,  41.071 ,  44.274 ,
     *                 63.729 ,  68.806 ,  71.434 ,  97.162 , 100.917 /
      DATA CH2      /   5.138 ,  47.290 ,  47.459 ,  71.647 ,  75.504 ,
     *                 98.880 , 104.778 , 107.864 ,
     *                  7.644 ,  15.031 ,  80.117 ,  80.393 ,
     *                109.294 , 113.799 ,
     *                  5.984 ,  10.634 ,  18.823 ,  25.496 ,
     *                 28.441 , 119.957 , 120.383 ,
     *                  8.149 ,  16.339 ,  22.894 ,
     *                 33.459 ,  42.333 ,  45.130 ,
     *                 10.474 ,  11.585 ,  19.720 ,
     *                 30.156 ,  51.354 ,  65.007 ,
     *                 10.357 ,  12.200 ,  13.401 ,  23.405 ,  24.807 ,
     *                 35.047 ,  47.292 ,  57.681 ,  72.474 ,  85.701 ,
     *                 13.014 ,  14.458 ,  23.798 ,  26.041 ,  27.501 ,
     *                 39.904 ,  41.610 ,  53.450 ,  67.801 ,
     *                 15.755 ,  15.933 ,  27.619 ,  29.355 ,
     *                 40.899 ,  42.407 ,  45.234 ,  59.793 ,  75.002 ,
     *                  4.339 ,  31.810 ,  32.079 ,
     *                 45.738 ,  47.768 ,  50.515 ,
     *                 60.897 ,  63.890 ,  65.849 ,  82.799 ,  85.150 /
      DATA CH3      /   6.111 ,   7.808 ,  11.868 ,
     *                 51.207 ,  51.596 ,  67.181 ,  69.536 ,
     *                  6.538 ,   7.147 ,   8.042 ,
     *                 12.891 ,  24.752 ,  74.090 ,  91.847 ,
     *                  6.818 ,  6.953 ,  7.411 ,
     *                 13.635 ,  14.685 ,  28.137 ,  43.236 , 100.083 ,
     *                  6.738 ,   7.101 ,  14.205 ,  15.670 ,  16.277 ,
     *                 29.748 ,  48.464 ,  65.198 ,
     *                  6.763 ,   8.285 ,   9.221 ,
     *                 16.493 ,  18.662 ,  30.950 ,  49.580 ,  73.093 ,
     *                  7.432 ,   8.606 ,   9.240 ,  15.636 ,  18.963 ,
     *                 33.690 ,  53.001 ,  76.006 ,
     *                  7.896 ,   8.195 ,   8.927 ,  16.178 ,  18.662 ,
     *                 30.640 ,  34.607 ,  56.001 ,  79.001           /
      DATA CH4      /   7.863 ,   8.378 ,   9.160 ,   9.519 ,
     *                 17.052 ,  18.958 ,  33.491 ,  53.001 ,
     *                  7.633 ,   8.793 ,  18.147 ,  20.233 ,  35.161 ,
     *                 56.025 ,
     *                  7.724 ,  10.532 ,  10.980 ,
     *                 20.286 ,  27.985 ,  36.826 ,  61.975 ,
     *                  9.391 ,  17.503 ,  17.166 ,
     *                 17.959 ,  27.757 ,  28.310 ,  39.701 ,  65.074 /
C
C     data for hydrogen and helium
C
      DATA AHH      /  20.4976, 747.5023,
     *                 28.1703, 527.8296,  22.2809, 987.7189          /
      DATA GHH      /  10.853 ,  13.342 ,
     *                 21.170 ,  24.125 ,  43.708 ,  53.542           /
C
C     data for lithium and beryllium
C
      DATA ALB      /   8.4915,  97.5015,  23.3299, 192.6701,
     *                  9.1849,  32.9263, 183.8887,  19.9563,  88.0437,
     *                  6.0478,  35.9723, 233.9798                    /
      DATA GLB      /   2.022 ,   4.604 ,  62.032 ,  72.624 ,
     *                  2.735 ,   6.774 ,   8.569 ,  10.750 ,  11.672 ,
     *                  3.967 ,  12.758 ,  16.692                     /
C
C     data for boron
C
      DATA AB       /   4.0086,  19.6741, 402.3110,
     *                  9.7257,  30.9262, 186.3466,  44.1629,  60.8371,
     *                  6.0084,  23.5767,  76.4149                    /
      DATA GB       /   0.002 ,   3.971 ,   7.882 ,
     *                  4.720 ,  13.477 ,  22.103 ,  23.056 ,  24.734 ,
     *                  6.000 ,  24.540 ,  32.300                     /
C
C     data for carbon
C
      DATA AC       /   8.0158,   5.8833,  33.7521, 595.3432,
     *                  4.0003,  17.0841,  82.9154,
     *                 15.9808,  48.2044, 435.8093,
     *                 10.0281,  15.7574, 186.2109,
     *                 15.4127,  55.9559, 243.6311,
     *                  6.0057,  23.5757,  76.4185                    /
      DATA GC       /   0.004 ,   1.359 ,   6.454 ,  10.376 ,
     *                  0.008 ,  16.546 ,  21.614 ,
     *                  5.688 ,  15.801 ,  26.269 ,
     *                  6.691 ,  25.034 ,  40.975 ,
     *                 17.604 ,  36.180 ,  47.133 ,
     *                  8.005 ,  40.804 ,  54.492                     /
C
C     data for nitrogen
C
      DATA AN       /  14.0499,  30.8008, 883.1443,
     *                 10.0000,  16.0000,  64.0000,
     *                  8.0462,   6.2669,  17.8696, 282.8084,
     *                  7.3751,  33.1390, 215.4829,
     *                  4.0003,  19.3533,  80.6462,
     *                 13.0998,  19.6425,  94.3035, 370.9539,
     *                 16.0000,  38.0000,
     *                 10.3289,  14.5021, 187.1624, 108.1615, 191.8383,
     *                  6.0044,  23.5612,  76.4344                    /
      DATA GN       /   2.554 ,   9.169 ,  13.651 ,
     *                 12.353 ,  13.784 ,  14.874 ,
     *                  0.014 ,   2.131 ,  15.745 ,  24.949 ,
     *                  6.376 ,  14.246 ,  29.465 ,
     *                  0.022 ,  31.259 ,  41.428 ,
     *                  7.212 ,  15.228 ,  34.387 ,  46.708 ,
     *                 46.475 ,  49.468 ,
     *                  8.693 ,  37.650 ,  65.479 ,  61.155 ,  79.196 ,
     *                  9.999 ,  60.991 ,  82.262                     /
C
C     data for oxygen
C
      DATA AO       /   4.0029,   5.3656,  36.2853,1044.3447,
     *                131.0217, 868.9779,  14.8533,  93.1466,
     *                 12.7843,   5.6828,  98.0919, 829.4396,
     *                 50.9878, 199.0120,   2.0000,   6.0000,  10.0000,
     *                 10.0000,  30.0000,  50.0000,
     *                  8.0703,   5.7144,  84.1156, 529.0927,
     *                  5.6609,  28.9355, 111.3620, 494.0413,
     *                 45.5249, 134.4751,
     *                  4.0003,  21.2937,  78.7058,
     *                 12.8293,  16.2730, 123.6578, 327.2396,
     *                 48.7883, 102.2117,  20.0060, 161.9903,
     *                 28.4184,  61.5816,
     *                 10.5563,  13.2950, 188.1390,
     *                 14.6560, 129.4922, 470.8512                    /
      DATA GO       /   0.022 ,   2.019 ,   9.812 ,  13.087 ,
     *                 13.804 ,  16.061 ,  14.293 ,  16.114 ,
     *                  3.472 ,   7.437 ,  22.579 ,  32.035 ,
     *                 27.774 ,  33.678 ,  28.118 ,  31.019 ,  34.204 ,
     *                 30.892 ,  33.189 ,  36.181 ,
     *                  0.032 ,   2.760 ,  35.328 ,  48.277 ,
     *                  7.662 ,  16.786 ,  42.657 ,  54.522 ,
     *                 50.204 ,  56.044 ,
     *                  0.048 ,  50.089 ,  66.604 ,
     *                  8.954 ,  18.031 ,  57.755 ,  72.594 ,
     *                 68.388 ,  82.397 ,  31.960 ,  76.876 ,
     *                 75.686 ,  80.388 ,
     *                 10.747 ,  52.323 ,  94.976 ,
     *                 27.405 ,  86.350 , 109.917                     /
C
C     data for fluor
C
      DATA AF       /   2.0001,  39.9012, 122.0986,
     *                 10.0000,  30.0000,  50.0000,
     *                  4.0199,   5.5741,  22.1839, 190.2179,
     *                 53.0383, 126.9616,  31.6894,  75.3105,
     *                 13.5014,   7.9936,  55.7981, 298.7039,
     *                 26.2496,  63.7503,   2.0000,   6.0000,  10.0000,
     *                 28.7150,  71.2850,
     *                  8.0153,   6.1931,  21.7287,  48.7780, 278.2782,
     *                178.5560, 421.4435,  51.7632,  95.2368          /
      DATA GF       /   0.050 ,  13.317 ,  15.692 ,
     *                 15.361 ,  17.128 ,  18.498 ,
     *                  0.048 ,   2.735 ,  20.079 ,  30.277 ,
     *                 27.548 ,  32.532 ,  30.391 ,  34.707 ,
     *                  4.479 ,  12.072 ,  31.662 ,  51.432 ,
     *                 44.283 ,  50.964 ,  46.193 ,  50.436 ,  54.880 ,
     *                 50.816 ,  57.479 ,
     *                  0.058 ,   3.434 ,  14.892 ,  37.472 ,  69.883 ,
     *                 67.810 ,  83.105 ,  72.435 ,  79.747           /
C
C     data for neon
C
      DATA ANN      /  34.5080, 365.4919,  16.5768, 183.4231,
     *                  2.0007,  89.5607, 380.4381,  26.4473,  63.5527,
     *                  4.0342,   5.6162,  11.5176,  72.8273,
     *                 48.5684, 131.4315,  31.1710,  76.8290,
     *                 14.0482,  13.3077,  52.7897, 467.8487,
     *                 54.2196, 195.7800                              /
      DATA GNN      /  17.796 ,  20.730 ,  17.879 ,  20.855 ,
     *                  0.097 ,  29.878 ,  37.221 ,  31.913 ,  37.551 ,
     *                  0.092 ,   3.424 ,  24.806 ,  46.616 ,
     *                 45.643 ,  54.147 ,  48.359 ,  57.420 ,
     *                  5.453 ,  18.560 ,  46.583 ,  80.101 ,
     *                 70.337 ,  85.789                               /
C
C     data for sodium
C
      DATA ANA      /  11.6348, 158.3593,
     *                 21.0453,  50.9546,  10.1389,  25.8611,
     *                  2.0019,  38.0569, 137.9398,  28.3106,  61.6893,
     *                  4.0334,   5.8560,  18.1786, 208.9142,
     *                 93.6895, 406.3095,  60.4276, 239.5719          /
      DATA GNA      /   2.400 ,   4.552 ,
     *                 34.367 ,  40.566 ,  34.676 ,  40.764 ,
     *                  0.170 ,  44.554 ,  57.142 ,  51.689 ,  60.576 ,
     *                  0.152 ,   4.260 ,  36.635 ,  83.254 ,
     *                 72.561 ,  89.475 ,  75.839 ,  92.582           /
C
C     data for magnesium
C
      DATA AMG      /  10.7445, 291.5057,  53.7488,
     *                  6.2270,  31.1291, 132.6438,
     *                 40.4379, 159.5618,  20.3845,  79.6154,
     *                  2.0007, 106.8977, 343.1010,  10.1326, 237.8581/
      DATA GMG      /   2.805 ,   6.777 ,   9.254 ,
     *                  4.459 ,   9.789 ,  13.137 ,
     *                 57.413 ,  71.252 ,  58.010 ,  71.660 ,
     *                  0.276 ,  74.440 ,  94.447 ,  54.472 ,  95.858 /
C
C     data for aluminium
C
      DATA AAL      /   4.0009,  11.7804, 142.2179,  13.6585,  96.3371,
     *                 10.0807,  49.5843, 285.3343,  14.6872,  59.3122,
     *                  6.3277,  29.5086, 134.1634,
     *                 46.3164, 153.6833,  22.9896,  77.0103          /
      DATA GAL      /   0.014 ,   3.841 ,   5.420 ,   3.727 ,   8.833 ,
     *                  4.749 ,  11.902 ,  16.719 ,  11.310 ,  18.268 ,
     *                  6.751 ,  16.681 ,  24.151 ,
     *                 83.551 , 104.787 ,  84.293 , 105.171           /
C
C     data for silicon
C
      DATA ASI      /   7.9658,   4.6762,   1.3512, 123.2267, 443.7797,
     *                  4.0000,   7.4186,  24.1754, 60.4060,
     *                 14.4695,  11.9721,  26.5062, 269.0521,
     *                  9.1793,   4.8766,  29.1442,  52.7998,
     *                 13.2674,  36.0417, 180.6910,
     *                  6.4839,  27.6851, 135.8301                    /
      DATA GSI      /   0.020 ,   0.752 ,   1.614 ,   5.831 ,   7.431 ,
     *                  0.036 ,   8.795 ,  11.208 ,  13.835 ,
     *                  5.418 ,   7.825 ,  14.440 ,  19.412 ,
     *                  6.572 ,  11.449 ,  18.424 ,  25.457 ,
     *                 15.682 ,  27.010 ,  34.599 ,
     *                  9.042 ,  24.101 ,  37.445                     /
C
C     data for phosphorus
C
      DATA AP       /  13.5211,  22.2130, 353.2583,  10.0000, 150.0000,
     *                  8.0241,   5.8085,  51.7542, 252.4002,
     *                  4.0021,  20.7985,  62.4194, 200.7786,
     *                 11.7414,  63.5124, 179.7420,
     *                  6.8835,  32.7777, 228.3366                    /
      DATA GP       /   1.514 ,   5.575 ,   9.247 ,   8.076 ,  10.735 ,
     *                  0.043 ,   1.212 ,   8.545 ,  15.525 ,
     *                  0.074 ,   7.674 ,  16.639 ,  25.118 ,
     *                  8.992 ,  24.473 ,  40.704 ,
     *                 11.464 ,  33.732 ,  55.455                     /
C
C     data for sulphur
C
      DATA AS       /   3.9615,   5.0780,  15.0944, 362.8588,
     *                 51.5995, 268.4002,  12.0000, 276.0000,
     *                 11.4377,   5.5126, 141.0009, 254.0478,
     *                 33.0518, 126.9479,
     *                  4.0707,   4.0637,   5.7245, 144.6376, 106.4909,
     *                  4.0011,  19.2813,  27.5990,  35.1179,
     *                 94.7454, 283.2486,
     *                 10.5474,  28.7137,  65.7378,  24.0000          /
      DATA GS       /   0.053 ,   1.121 ,   5.812 ,   9.425 ,
     *                  8.936 ,  11.277 ,   9.600 ,  12.551 ,
     *                  1.892 ,   3.646 ,  13.550 ,  19.376 ,
     *                 16.253 ,  21.062 ,
     *                  0.043 ,   0.123 ,   1.590 ,  13.712 ,  22.050 ,
     *                  0.118 ,   9.545 ,  18.179 ,  31.441 ,
     *                 30.664 ,  56.150 ,
     *                 10.704 ,  27.075 ,  50.599 ,  43.034           /
C
C     data for chlorine
C
      DATA ACL      /   2.0007,  62.5048, 669.4942,  29.0259, 130.9740,
     *                  3.9064,   0.3993,   5.3570,  60.3424, 119.9913,
     *                138.1567, 278.8418, 102.3681, 158.6314,
     *                 12.6089,   5.9527, 110.5635, 262.8715,
     *                 69.2035, 100.7960,
     *                  7.3458,   5.6638,  44.1256, 202.7846,
     *                  4.0037,  21.8663,  40.5363,  57.5919          /
      DATA GCL      /   0.110 ,   9.919 ,  12.280 ,  11.017 ,  13.532 ,
     *                  0.092 ,   0.581 ,   1.620 ,  13.121 ,  19.787 ,
     *                 16.365 ,  21.988 ,  18.065 ,  23.594 ,
     *                  2.358 ,   5.708 ,  19.084 ,  30.683 ,
     *                 24.880 ,  33.229 ,
     *                  0.102 ,   1.391 ,  14.709 ,  36.968 ,
     *                  0.185 ,  11.783 ,  25.653 ,  44.698           /
C
C     data for argon
C
      DATA AAR      /  43.6623, 324.3375,  20.8298, 163.1701,
     *                  2.0026, 137.4515, 258.5445,  62.8129, 149.1867,
     *                  4.0495,  14.4466,  46.8234, 124.6651,
     *                151.9828, 268.0157, 101.1302, 150.8691,
     *                 13.3718,   8.6528,  60.4614, 285.5072,
     *                  6.7655,   4.7684,  12.8631,  54.5260          /
      DATA GAR      /  12.638 ,  14.958 ,  12.833 ,  15.139 ,
     *                  0.178 ,  17.522 ,  23.584 ,  20.464 ,  25.150 ,
     *                  0.151 ,   1.561 ,  17.399 ,  30.871 ,
     *                 24.684 ,  33.978 ,  27.091 ,  36.481 ,
     *                  2.810 ,   8.877 ,  24.351 ,  44.489 ,
     *                  0.144 ,   1.160 ,  10.210 ,  27.178           /
C
C     data for potassium
C
      DATA AK       /  12.9782, 148.6673,   6.3493,
     *                 66.3444, 101.6553,   4.0001,  13.4465,  46.5534,
     *                  2.0171, 116.4767, 713.4965,  63.5907, 396.4079,
     *                  2.0000,  10.0000,  30.0000,
     *                  4.0702,   5.7791,  52.6795, 327.4539,
     *                 62.8604, 357.1331,  55.9337, 196.0646,
     *                 10.9275,   5.5398,  43.2761,  76.2560,
     *                 42.0000,  18.0000                              /
      DATA GK       /   1.871 ,   3.713 ,  18.172 ,
     *                 21.185 ,  27.705 ,   2.059 ,  23.709 ,  28.542 ,
     *                  0.273 ,  26.709 ,  39.640 ,  31.220 ,  41.865 ,
     *                 29.955 ,  37.557 ,  42.862 ,
     *                  0.228 ,   2.274 ,  21.703 ,  50.191 ,
     *                 32.145 ,  49.262 ,  34.155 ,  51.718 ,
     *                  3.043 ,   5.479 ,  20.547 ,  30.680 ,
     *                 36.275 ,  47.345                               /
C
C     data for calcium
C
      DATA ACA      /  18.2366,  27.5012, 149.2617,  94.5242, 705.4711,
     *                 11.8706,  14.0710, 106.0547,
     *                 57.2414, 110.7567,  29.8121,  54.1874,
     *                  2.0184,  97.5784, 282.3939, 209.1871, 252.8129/
      DATA GCA      /   2.050 ,   3.349 ,   5.321 ,   4.873 ,   7.017 ,
     *                  1.769 ,   5.109 ,   9.524 ,
     *                 27.271 ,  41.561 ,  29.172 ,  42.140 ,
     *                  0.394 ,  28.930 ,  52.618 ,  38.593 ,  49.646 /
C
C     data for scandium
C
      DATA ASC      /   6.0014,  83.1958,  67.3666, 329.4354,
     *                 44.0793, 169.9969, 533.9195,
     *                 34.1642, 124.8475, 228.9879,
     *                 11.9979,  16.9280,  28.4778,  82.0418, 234.5360,
     *                  6.0042,   2.7101,  13.9801,  65.3039,
     *                 12.0000,  12.0000,
     *                  2.0051,   2.9621,  29.0306                    /
      DATA GSC      /   0.021 ,   2.056 ,   3.551 ,   5.465 ,
     *                  1.535 ,   3.797 ,   6.203 ,
     *                  2.389 ,   4.858 ,   7.141 ,
     *                  0.011 ,   0.430 ,   1.156 ,   3.711 ,   8.863 ,
     *                  0.025 ,   3.499 ,  10.463 ,  18.606 ,
     *                 41.779 ,  57.217 ,
     *                  0.539 ,  24.442 ,  51.079                     /
C
C     data for titanium
C
      DATA ATI      /   7.0887,   8.9186,  17.5633, 206.6832, 438.5735,
     *                654.1721,
     *                 38.0462,  69.6271, 364.2845, 832.0408,
     *                 98.8562,  57.9934, 442.1498,
     *                 19.7843,  32.0637,  37.0895, 110.6682, 288.4946,
     *                521.8837,
     *                 10.0000,  34.0000, 120.0000,
     *                 16.1691,  22.3550,  24.1646,  83.5128, 222.7963,
     *                  6.0020,   4.6177,  25.2636,  52.1162,
     *                 12.0000,   8.0000                              /
      DATA GTI      /   0.021 ,   0.048 ,   1.029 ,   2.183 ,   4.109 ,
     *                  5.785 ,
     *                  0.846 ,   1.792 ,   3.836 ,   5.787 ,
     *                  2.561 ,   4.869 ,   6.340 ,
     *                  0.023 ,   0.124 ,   0.774 ,   1.810 ,   4.980 ,
     *                  9.585 ,
     *                  1.082 ,   4.928 ,  11.279 ,
     *                  0.041 ,   1.375 ,   4.768 ,  10.985 ,  19.769 ,
     *                  0.048 ,  11.577 ,  24.531 ,  36.489 ,
     *                 54.436 ,  75.373                               /
C
C     data for vanadium
C
      DATA AV       /  15.2627,  23.9869,  51.3053, 570.3384,1650.9417,
     *                162.2829, 298.8303, 908.8852,
     *                 23.6736,  37.1624,  86.8011, 300.7440, 864.5880,
     *                 57.8961,  79.4605, 214.9007, 864.7425,
     *                 61.8508,  64.0845, 192.8298, 718.2349,
     *                 23.8116,  68.2495, 135.0613, 536.7632,
     *                 15.9543,  22.5542,  71.4921, 248.9544,
     *                  6.0006,   5.8785,  50.5077,  97.6129          /
      DATA GV       /   0.026 ,   0.145 ,   0.718 ,   2.586 ,   5.458 ,
     *                  2.171 ,   4.153 ,   6.097 ,
     *                  0.009 ,   0.366 ,   1.504 ,   5.294 ,  10.126 ,
     *                  1.796 ,   2.353 ,   6.068 ,  12.269 ,
     *                  2.560 ,   3.674 ,   6.593 ,  12.880 ,
     *                  0.045 ,   1.684 ,   8.162 ,  21.262 ,
     *                  0.065 ,   1.746 ,  15.158 ,  33.141 ,
     *                  0.077 ,  21.229 ,  44.134 ,  60.203           /
C
C     data for chromium
C
      DATA ACR      /  30.1842,  79.2847, 149.5293,
     *                215.3696, 119.1974, 741.4321,
     *                184.9946,1352.5038, 784.4937,
     *                 46.6191, 160.1361, 488.0449, 657.1928,
     *                 47.1742, 267.0275, 441.1324, 150.6650,
     *                 24.3768, 122.8359, 285.5092, 794.1654,
     *                 24.2296,  75.0258, 172.9452, 543.6511,
     *                 15.9819,  17.6800,  95.2003, 225.0947          /
      DATA GCR      /   0.993 ,   3.070 ,   5.673 ,
     *                  3.339 ,   4.801 ,   7.198 ,
     *                  2.829 ,   4.990 ,   7.643 ,
     *                  1.645 ,   3.727 ,   7.181 ,  12.299 ,
     *                  2.902 ,   4.273 ,   8.569 ,  14.912 ,
     *                  0.047 ,   2.566 ,   9.441 ,  21.198 ,
     *                  0.078 ,   2.242 ,  15.638 ,  32.725 ,
     *                  0.103 ,   2.146 ,  26.153 ,  49.381           /
C
C     data for manganese
C
      DATA AMN      /  53.9107,  81.3931, 546.6945 ,
     *                144.1893, 407.8029,  45.6177, 298.4423,2410.9335,
     *                 22.6382,  93.8419, 183.9367, 907.5765,
     *                137.0409, 168.6783, 329.0287, 773.2513,
     *                 70.1925,  72.3372, 213.9512, 539.5165,
     *                 24.2373,  93.5415, 456.6167, 506.5484,
     *                 24.7687,  66.9896, 264.1853, 484.0161          /
      DATA GMN      /   2.527 ,   4.204 ,   6.602 ,
     *                  4.155 ,   7.321 ,   2.285 ,   5.631 ,   8.448 ,
     *                  1.496 ,   3.839 ,   7.751 ,  13.484 ,
     *                  3.681 ,   6.054 ,   9.934 ,  14.936 ,
     *                  3.531 ,   6.967 ,  15.222 ,  25.069 ,
     *                  0.071 ,   2.896 ,  20.725 ,  37.383 ,
     *                  0.126 ,   2.660 ,  28.528 ,  53.413           /
C
C     data for iron
C
      DATA AFE      /  14.4102,   2.7050, 421.6612, 940.1484,
     *                 36.2187,  22.8883, 239.5997, 825.2919,
     *                110.0242, 992.3040, 640.6715,
     *                 17.0494,  32.3783,  34.3184, 420.9626,1067.2064,
     *                154.0059, 462.1117, 329.8618,
     *                 15.7906,  47.1186, 279.9292, 692.1005,
     *                 91.0206, 206.3082, 706.9927, 836.6689,
     *                 40.0790,  27.6965,  28.2243,  18.0001,
     *                 24.0899,  89.6340,  51.5756, 241.6980          /
      DATA GFE      /   0.066 ,   0.339 ,   2.897 ,   6.585 ,
     *                  0.923 ,   1.679 ,   4.620 ,   7.053 ,
     *                  4.249 ,   5.875 ,   7.781 ,
     *                  0.062 ,   0.283 ,   1.504 ,   5.430 ,  11.210 ,
     *                  2.792 ,   7.627 ,  13.623 ,
     *                  0.077 ,   3.723 ,  12.137 ,  23.700 ,
     *                  2.688 ,   7.595 ,  15.444 ,  25.587 ,
     *                  3.982 ,   4.677 ,   6.453 ,  23.561 ,
     *                  0.102 ,   3.354 ,  22.954 ,  33.796           /
C
C     data for cobalt
C
      DATA ACO      /  11.9120,  20.4424,  28.3863, 132.5038, 600.7461,
     *                 33.3092, 237.4331, 977.2502,
     *                 55.5396, 318.8169, 619.6366,
     *                 32.6900,  83.8694, 107.4378,
     *                 11.2593,  38.2239,  22.9964, 261.3486, 637.1485,
     *                 23.0233,  41.6599, 264.6460, 181.6699,
     *                 16.0356,   7.8633,  70.3158, 423.3512, 742.3553,
     *                  0.                                            /
      DATA GCO      /   0.112 ,   0.341 ,   0.809 ,   3.808 ,   6.723 ,
     *                  2.057 ,   3.484 ,   7.210 ,
     *                  2.405 ,   5.133 ,   8.097 ,
     *                  2.084 ,   5.291 ,   8.426 ,
     *                  0.135 ,   0.517 ,   1.606 ,   6.772 ,  12.622 ,
     *                  2.512 ,   4.348 ,   8.253 ,  15.377 ,
     *                  0.132 ,   0.863 ,   3.086 ,  11.789 ,  23.263 ,
     *                  0.                                            /
C
C     data for nickel
C
      DATA ANI      /   7.1268,  12.4486,  11.9953,  10.0546, 114.1658,
     *                391.2064,
     *                 26.3908, 213.8081, 938.7927,
     *                  4.1421,  37.3781,  25.9712, 333.3397, 311.1633,
     *                 33.1031, 184.1854, 136.7072,
     *                 11.1915,   5.4174,  53.6793, 460.6781, 380.0056,
     *                  0.                                            /
      DATA GNI      /   0.026 ,   0.137 ,   0.315 ,   1.778 ,   4.029 ,
     *                  6.621 ,
     *                  2.249 ,   4.042 ,   7.621 ,
     *                  0.191 ,   1.235 ,   3.358 ,   8.429 ,  17.096 ,
     *                  3.472 ,   9.065 ,  16.556 ,
     *                  0.194 ,   1.305 ,   5.813 ,  14.172 ,  26.169 ,
     *                  0.                                            /
C
C     data for copper
C
      DATA ACU      /  11.0549, 238.9423,  10.3077, 126.2990,1073.3876,
     *                 30.0000,  50.0000,  60.0000,
     *                 19.2984,  50.5974, 240.2021,1216.9016,
     *                 48.3048, 583.2011, 320.4931,
     *                  4.0155,  70.3264, 313.1213, 536.5331,
     *                  0.                                            /
      DATA GCU      /   4.212 ,   7.227 ,   1.493 ,   5.859 ,   9.709 ,
     *                  7.081 ,   9.362 ,  10.130 ,
     *                  2.865 ,   8.260 ,  14.431 ,  18.292 ,
     *                  9.650 ,  14.640 ,  24.320 ,
     *                  0.337 ,   8.520 ,  16.925 ,  28.342 ,
     *                  0.                                            /
C
C     data for zinc
C
      DATA AZN      /  15.9880, 484.0042,  18.5863, 123.4134,
     *                  3.0000, 189.0000,
     *                  6.1902,  38.9317, 204.8780,
     *                 10.2588,  89.3771, 370.3640,  30.0000, 128.0000,
     *                 24.6904, 106.7491, 439.5586,
     *                  0.                                            /
      DATA GZN      /   4.546 ,   8.840 ,  10.247 ,  16.620 ,
     *                 11.175 ,  16.321 ,
     *                  6.113 ,  12.964 ,  16.444 ,
     *                  7.926 ,  13.633 ,  24.353 ,  16.286 ,  24.910 ,
     *                 10.291 ,  20.689 ,  32.077 ,
     *                  0.                                            /
C
C
      DATA ICOMP /0/
C
      EQUIVALENCE   ( AHH(1), ALF(  1)),( ALB(1), ALF(  7)),
     *              ( AB (1), ALF( 19)),
     *              ( AC (1), ALF( 30)),( AN (1), ALF( 49)),
     *              ( AO (1), ALF( 79)),( AF (1), ALF(128)),
     *              ( ANN(1), ALF(162)),( ANA(1), ALF(185)),
     *              ( AMG(1), ALF(204)),( AAL(1), ALF(219)),
     *              ( ASI(1), ALF(236)),( AP (1), ALF(259)),
     *              ( AS (1), ALF(278)),( ACL(1), ALF(307)),
     *              ( AAR(1), ALF(335)),( AK (1), ALF(360)),
     *              ( ACA(1), ALF(390)),( ASC(1), ALF(407)),
     *              ( ATI(1), ALF(431)),( AV (1), ALF(464)),
     *              ( ACR(1), ALF(497)),( AMN(1), ALF(526)),
     *              ( AFE(1), ALF(554)),( ACO(1), ALF(589)),
     *              ( ANI(1), ALF(618)),( ACU(1), ALF(641)),
     *              ( AZN(1), ALF(661))
      EQUIVALENCE   ( GHH(1), GAM(  1)),( GLB(1), GAM(  7)),
     *              ( GB (1), GAM( 19)),
     *              ( GC (1), GAM( 30)),( GN (1), GAM( 49)),
     *              ( GO (1), GAM( 79)),( GF (1), GAM(128)),
     *              ( GNN(1), GAM(162)),( GNA(1), GAM(185)),
     *              ( GMG(1), GAM(204)),( GAL(1), GAM(219)),
     *              ( GSI(1), GAM(236)),( GP (1), GAM(259)),
     *              ( GS (1), GAM(278)),( GCL(1), GAM(307)),
     *              ( GAR(1), GAM(335)),( GK (1), GAM(360)),
     *              ( GCA(1), GAM(390)),( GSC(1), GAM(407)),
     *              ( GTI(1), GAM(431)),( GV (1), GAM(464)),
     *              ( GCR(1), GAM(497)),( GMN(1), GAM(526)),
     *              ( GFE(1), GAM(554)),( GCO(1), GAM(589)),
     *              ( GNI(1), GAM(618)),( GCU(1), GAM(641)),
     *              ( GZN(1), GAM(661))
      EQUIVALENCE   ( CH1(1), CHION(  1)),
     *              ( CH2(1), CHION( 67)),
     *              ( CH3(1), CHION(139)),
     *              ( CH4(1), CHION(194)),
     *              ( XL1(1),    XL(  1)),
     *              ( XL2(1),    XL(100))
      EQUIVALENCE   ( IS1(1),  IS(1)),   ( IS2(1),  IS( 54)),
     *              ( IM1(1),  IM(1)),   ( IM2(1),  IM(100)),
     *              (IGP1(1),IGPR(1)),   (IGP2(1),IGPR(100)),
     *              (IG01(1), IG0(1)),   (IG02(1), IG0( 54)),
     *              (II1(1,1),INDEX0(1,1)),(II2(1,1),INDEX0(1,16))
      SAVE ALF,GAM,XL,CHION,INDEX0,INDEXS,INDEXM,IGPR,IG0,ICOMP
C
C     Initialization of auxiliary arrays (executed only once)
C
      IF(ICOMP.NE.0) GO TO 30
      IND=1
      DO 10 K=1,NIONS
         INDEXS(K)=IND
         IND=IND+IS(K)
   10 CONTINUE
      IND=1
      DO 20 K=1,NSS
         INDEXM(K)=IND
         IND=IND+IM(K)
   20 CONTINUE
      ICOMP=1
C
   30 CONTINUE
      MODE=MODPF(IAT)
      IF(MODE.GT.0) GO TO 80
      IF(IAT.EQ.26 .AND. IZI.GE.4) GO TO 170
      IF(IAT.EQ.28 .AND. IZI.GE.4 .AND. IZI.LE.9) GO TO 171
      IF(IZI.LE.0.OR.IZI.GT.5.OR.IAT.LE.0.OR.IAT.GT.30) GO TO 70
      IF(MODE.LT.0) GO TO 70
      I0=INDEX0(IZI,IAT)
      IF(I0.LE.0) GO TO 60
C
C     Traving, Baschek, Holweger formula
C
      QZ=IZI
      THET=5.0404D3/T
      A=31.321*QZ*QZ*THET
      XMAX2=XMAX*XMAX
      QQ=XMAX/4.*(XMAX2+XMAX+SIXTH+A+A*A*HALF/XMAX2)
      QAS1=XMAX*THIRD*(XMAX2+TRHA*XMAX+HALF)
      IS0=INDEXS(I0)
      ISS=IS0+IS(I0)-1
      SU1=0.
      SU2=0.
      SQA=0.
      SQQ=0.
      SQT=0.
      SQ2=0.
      DO 50 K=IS0,ISS
         XXL=XL(K)
         GPR=IGPR(K)
         X=CHION(K)*THET
         EX=0.
         IF(X.LT.30) EX=EXP(-X*2.30258029299405)
         SQQ=SQQ+GPR*EX
         QAS=(QAS1-XXL*THIRD*(XXL*XXL+TRHA*XXL+HALF)+(XMAX-XXL)*
     *       (UN+A*HALF/XXL/XMAX)*A)*GPR*EX
         SQA=SQA+QAS
         SQ2=SQ2+QAS*CHION(K)
         SQT=SQT+GPR*(XMAX-XXL)*(UN+A/XMAX/XXL)*EX
         M0=INDEXM(K)
         M1=M0+IM(K)-1
         AL1=0.
         AL2=0.
         DO 40 M=M0,M1
            XG=GAM(M)*THET
            IF(XG.GT.20.) GO TO 40
            XM=EXP(-XG*2.30258029299405)*ALF(M)
            AL1=AL1+XM
            AL2=AL2+GAM(M)*XM
   40    CONTINUE
         SU1=SU1+AL1
         SU2=SU2+AL2
   50 CONTINUE
      U=IG0(I0)
      U=U+SU1+SQA
      IF(U.LT.0.) U=IG0(I0)
      DUT=(2.302580293*THET*(SU2+SQ2)+QQ*SQQ-A*SQT)/T
      DUN=-QQ*SQQ/ANE
      RETURN
C
C     constant value of partition function for some ions (even if
C     MODPF = 0)
C
   60 U=-I0
      DUT=0.
      DUN=0.
      RETURN
C
C     non-standard, user supplied formula
C
   70 CALL PFSPEC(IAT,IZI,T,ANE,XMAX,U,DUT,DUN)
      RETURN
C
C     Partition functions for Iron  (From Sparks and Fischel)
C
  170 CALL PFFE(IZI,T,ANE,U,DUT,DUN)
      RETURN
C
C     Partition functions for Nickel (from Kurucz predicted levels)
C
  171 CALL PFNI(IZI,T,ANE,U,DUT,DUN)
      RETURN
C
C     Opacity Project value
C
   80 call opfrac(iat,izi,t,ane,u,opfra)
      DUT=0.
      DUN=0.
      RETURN
      END
C
C
C *******************************************************************
C
C
      SUBROUTINE PFSPEC(IAT,IZI,T,ANE,XMAX,U,DUT,DUN)
C     ===============================================
 
C     Non-standard evaluation of the partition function
C     user supplied procedure
C
C     Input:
C      IAT   - atomic number
C      IZI   - ionic charge (=1 for neutrals, =1 for once ionized, etc)
C      T     - temperature
C      ANE   - electron density
C      XMAX  - principal quantum number of the last bound level
C
C     Output:
C      U     - partition function
C      DUT   - derivative dU/dT
C      DUN   - derivative dU/d(ANE)
C
      INCLUDE 'IMPLIC.FOR'
C
c Modified from the ATMOS related programme 5-April-1990
c as an addition to TLUSTY to allow high ionisation states
c of C, N and O
c M.A.Barstow - University of Leicester, Dept of Physics & Astronomy
c
      PARAMETER (MH=100,MHEI=100,MHEII=100,MCI=135,
     +     MCII=157,MCIII=156,MCIV=55,MCV=15,MCVI=100,MNI=228,MNII=122,
     +     MNIII=133,MNIV=73,MNV=51,MNVI=8,MNVII=100,MOI=174,MOII=191,
     +     MOIII=168,MOIV=166,MOV=115,MOVI=52,MOVII=16,MOVIII=100)
      DIMENSION GHYD(MH),SHYD(MH),ENHYD(MH),
     +     GHEL(MH),ENHEL(MH),SHEL(MH),
     +     GCI(MCI),ENCI(MCI),SCI(MCI),
     +     GCII(MCII),ENCII(MCII),SCII(MCII),
     +     GCIII(MCIII),ENCIII(MCIII),SCIII(MCIII),
     +     GCIV(MCIV),ENCIV(MCIV),SCIV(MCIV),
     +     GCV(MCV),ENCV(MCV),SCV(MCV),
     +     GNI(MNI),ENNI(MNI),SNI(MNI),
     +     GNII(MNII),ENNII(MNII),SNII(MNII),
     +     GNIII(MNIII),ENNIII(MNIII),SNIII(MNIII),
     +     GNIV(MNIV),ENNIV(MNIV),SNIV(MNIV),
     +     GNV(MNV),ENNV(MNV),SNV(MNV),
     +     GNVI(MNVI),ENNVI(MNVI),SNVI(MNVI),
     +     GOI(MOI),ENOI(MOI),SOI(MOI),
     +     GOII(MOII),ENOII(MOII),SOII(MOII),
     +     GOIII(MOIII),ENOIII(MOIII),SOIII(MOIII),
     +     GOIV(MOIV),ENOIV(MOIV),SOIV(MOIV),
     +     GOV(MOV),ENOV(MOV),SOV(MOV),
     +     GOVI(MOVI),ENOVI(MOVI),SOVI(MOVI),
     +     GOVII(MOVII),ENOVII(MOVII),SOVII(MOVII)
      INTEGER NHYD(MH),NHEL(MHEI),NCI(MCI),NCII(MCII),
     +     NCIII(MCIII),NCIV(MCIV),NCV(MCV),NNI(MNI),NNII(MNII),
     +     NNIII(MNIII),NNIV(MNIV),NNV(MNV),NNVI(MNVI),NOI(MOI),
     +     NOII(MOII),NOIII(MOIII),NOIV(MOIV),NOV(MOV),NOVI(MOVI),
     +     NOVII(MOVII)
      PARAMETER (HI=13.5878,HEI=24.587,HEII=54.416,CVI=489.84,
     +     NVII=666.83,OVIII=871.12)
      PARAMETER (ZH=1.0,ZHE=2.0,ZC=6.0,ZN=7.0,ZO=8.0)
C                           N***=QUANTUM NO. OF LEVEL
C      DATA FOR IONS        G***=STATISTICAL WEIGHT OF LEVEL
C                           EN***=ENERGY OF LEVEL
C                           S*=SCREENING NO. OF LEVEL
	DATA NHYD/ 1, 2, 3, 4, 5, 6,
     +	 	7, 8, 9,10,11,12,
     +		13,14,15,16,17,18,
     +		19,20,21,22,23,24,
     +		25,26,27,28,29,30,
     +		31,32,33,34,35,36,
     +		37,38,39,40,41,42,
     +		43,44,45,46,47,48,
     +		49,50,51,52,53,54,
     +		55,56,57,58,59,60,
     +		61,62,63,64,65,66,
     +		67,68,69,70,71,72,
     +		73,74,75,76,77,78,
     +		79,80,81,82,83,84,
     +		85,86,87,88,89,90,
     +		91,92,93,94,95,96,
     +		97,98,99, 100 /
	DATA GHYD/ 2.000000000000000, 8.000000000000000, 18.00000000000000,
     +		 32.00000000000000, 50.00000000000000, 72.00000000000000,
     +		 98.00000000000000, 128.0000000000000, 162.0000000000000,
     +		 200.0000000000000, 242.0000000000000, 288.0000000000000,
     +		 338.0000000000000, 392.0000000000000, 450.0000000000000,
     +		 512.0000000000000, 578.0000000000000, 648.0000000000000,
     +		 722.0000000000000, 800.0000000000000, 882.0000000000000,
     +		 968.0000000000000, 1058.000000000000, 1152.000000000000,
     +		 1250.000000000000, 1352.000000000000, 1458.000000000000,
     +		 1568.000000000000, 1682.000000000000, 1800.000000000000,
     +		 1922.000000000000, 2048.000000000000, 2178.000000000000,
     +		 2312.000000000000, 2450.000000000000, 2592.000000000000,
     +		 2738.000000000000, 2888.000000000000, 3042.000000000000,
     +		 3200.000000000000, 3362.000000000000, 3528.000000000000,
     +		 3698.000000000000, 3872.000000000000, 4050.000000000000,
     +		 4232.000000000000, 4418.000000000000, 4608.000000000000,
     +		 4802.000000000000, 5000.000000000000, 5202.000000000000,
     +		 5408.000000000000, 5618.000000000000, 5832.000000000000,
     +		 6050.000000000000, 6272.000000000000, 6498.000000000000,
     +		 6728.000000000000, 6962.000000000000, 7200.000000000000,
     +		 7442.000000000000, 7688.000000000000, 7938.000000000000,
     +		 8192.000000000000, 8450.000000000000, 8712.000000000000,
     +		 8978.000000000000, 9248.000000000000, 9522.000000000000,
     +		 9800.000000000000, 10082.00000000000, 10368.00000000000,
     +		 10658.00000000000, 10952.00000000000, 11250.00000000000,
     +		 11552.00000000000, 11858.00000000000, 12168.00000000000,
     +		 12482.00000000000, 12800.00000000000, 13122.00000000000,
     +		 13448.00000000000, 13778.00000000000, 14112.00000000000,
     +		 14450.00000000000, 14792.00000000000, 15138.00000000000,
     +		 15488.00000000000, 15842.00000000000, 16200.00000000000,
     +		 16562.00000000000, 16928.00000000000, 17298.00000000000,
     +		 17672.00000000000, 18050.00000000000, 18432.00000000000,
     +		 18818.00000000000, 19208.00000000000, 19602.00000000000,
     +		 20000.00000000000/
	DATA ENHYD /0.0000000000000000E+00,10.19085000000000,12.07804444444444,
     +		 12.73856250000000, 13.04428800000000, 13.21036111111111,
     +		 13.31049795918367, 13.37549062500000, 13.42004938271605,
     +		 13.45192200000000, 13.47550413223140, 13.49344027777778,
     +		 13.50739881656805, 13.51847448979592, 13.52740977777778,
     +		 13.53472265625000, 13.54078339100346, 13.54586234567901,
     +		 13.55016066481994, 13.55383050000000, 13.55698866213152,
     +		 13.55972603305785, 13.56211417769376, 13.56421006944444,
     +		 13.56605952000000, 13.56769970414201, 13.56916104252401,
     +		 13.57046862244898, 13.57164328180737, 13.57270244444444,
     +		 13.57366077003122, 13.57453066406250, 13.57532268135905,
     +		 13.57604584775087, 13.57670791836735, 13.57731558641975,
     +		 13.57787465303141, 13.57839016620499, 13.57886653517423,
     +		 13.57930762500000, 13.57971683521713, 13.58009716553288,
     +		 13.58045127095727, 13.58078150826446, 13.58108997530864,
     +		 13.58137854442344, 13.58164889090086, 13.58190251736111,
     +		 13.58214077467722, 13.58236488000000, 13.58257593233372,
     +		 13.58277492603550, 13.58296276254895, 13.58314026063100,
     +		 13.58330816528926, 13.58346715561225, 13.58361785164666,
     +		 13.58376082045184, 13.58389658144211, 13.58402561111111,
     +		 13.58414834721849, 13.58426519250780, 13.58437651801461,
     +		 13.58448266601563, 13.58458395266272, 13.58468067033976,
     +		 13.58477308977501, 13.58486146193772, 13.58494601974375,
     +		 13.58502697959184, 13.58510454274945, 13.58517889660494,
     +		 13.58525021580034, 13.58531866325785, 13.58538439111111,
     +		 13.58544754155125, 13.58550824759656, 13.58556663379356,
     +		 13.58562281685627, 13.58567690625000, 13.58572900472489,
     +		 13.58577920880428, 13.58582760923211, 13.58587429138322,
     +		 13.58591933564014, 13.58596281773932, 13.58600480908971,
     +		 13.58604537706612, 13.58608458527964, 13.58612249382716,
     +		 13.58615915952180, 13.58619463610586, 13.58622897444791,
     +		 13.58626222272522, 13.58629442659280, 13.58632562934028,
     +		 13.58635587203741, 13.58638519366930, 13.58641363126212,
     +		 13.58644122000000/
	DATA SHYD/100*0.0D0/
      DATA NHEL/1,2,2,2,2,2,2,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,
     +        5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,
     +        23,24,25,26,27,
     +		28,29,30,31,32,33,
     +		34,35,36,37,38,39,
     +		40,41,42,43,44,45,
     +		46,47,48,49,50,51,
     +		52,53,54,55,56,57,
     +		58,59,60,61,62,63,
     +		64,65,66,67,68,69,
     +		70,71,72,73,74,75,
     +		76,77,78,79,80,81/
      DATA GHEL/1.0D0,3.0D0,1.0D0,5.0D0,3.0D0,1.0D0,3.0D0,
     +          3.0D0,1.0D0,5.0D0,3.0D0,
     +         1.0D0,15.0D0,5.0D0,3.0D0,3.0D0,1.0D0,9.0D0,
     +         15.0D0,5.0D0,21.0D0,7.0D0,
     +         3.0D0,100.0D0,144.0D0,196.0D0,256.0D0,324.0D0,
     +         400.0D0,484.0D0,
     +         576.0D0,676.0D0,784.0D0,900.0D0,1024.0D0,1156.0D0,
     +         1296.0D0,1444.0D0,1600.0D0,1764.0D0,1936.0D0,
     +         2116.0D0,2304.0D0,2500.0D0,2704.0D0,3136.0D0,
     +		3136.000000000000,3364.000000000000,3600.000000000000,
     +		3844.000000000000,4096.000000000000,4356.000000000000,
     +		4624.000000000000,4900.000000000000,5184.000000000000,
     +		5476.000000000000,5776.000000000000,6084.000000000000,
     +		6400.000000000000,6724.000000000000,7056.000000000000,
     +		7396.000000000000,7744.000000000000,8100.000000000000,
     +		8464.000000000000,8836.000000000000,9216.000000000000,
     +		9604.000000000000,10000.00000000000,10404.00000000000,
     +		10816.00000000000,11236.00000000000,11664.00000000000,
     +		12100.00000000000,12544.00000000000,12996.00000000000,
     +		13456.00000000000,13924.00000000000,14400.00000000000,
     +		14884.00000000000,15376.00000000000,15876.00000000000,
     +		16384.00000000000,16900.00000000000,17424.00000000000,
     +		17956.00000000000,18496.00000000000,19044.00000000000,
     +		19600.00000000000,20164.00000000000,20736.00000000000,
     +		21316.00000000000,21904.00000000000,22500.00000000000,
     +		23104.00000000000,23716.00000000000,24336.00000000000,
     +		24964.00000000000,25600.00000000000,26244.00000000000/
      DATA ENHEL/0.0D0,19.819D0,20.615D0,20.964D0,
     +           20.964D0,20.964D0,21.218D0,
     +           22.718D0,22.920D0,23.007D0,23.007D0,
     +           23.007D0,23.073D0,23.074D0,
     +           23.087D0,23.593D0,23.673D0,23.707D0,
     +           23.736D0,23.736D0,23.737D0,
     +           23.737D0,23.742D0,24.028D0,24.201D0,
     +           24.304D0,24.371D0,24.417D0,
     +           24.449D0,24.473D0,24.491D0,24.506D0,
     +           24.517D0,24.526D0,24.534D0,
     +           24.540D0,24.545D0,24.549D0,24.553D0,
     +           24.556D0,24.559D0,24.562D0,
     +           24.564D0,24.566D0,24.568D0,24.570D0,
     +		24.57131951530612,24.57238228299643,24.57334055555556,
     +		24.57420759625390,24.57499462890625,24.57571120293848,
     +		24.57636548442907,24.57696448979592,24.57751427469136,
     +		24.57802008765522,24.57848649584488,24.57891748849441,
     +		24.57931656250000,24.57968679357525,24.58003089569161,
     +		24.58035127095727,24.58065005165289,24.58092913580247,
     +		24.58119021739130,24.58143481213219,24.58166427951389,
     +		24.58187984173261,24.58208260000000,24.58227354863514,
     +		24.58245358727811,24.58262353150587,24.58278412208505,
     +		24.58293603305785,24.58307987882653,24.58321622037550,
     +		24.58334557074911,24.58346839988509,24.58358513888889,
     +		24.58369618382155,24.58380189906348,24.58390262030738,
     +		24.58399865722656,24.58409029585799,24.58417780073462,
     +		24.58426141679661,24.58434137110727,24.58441787439614,
     +		24.58449112244898,24.58456129736163,24.58462856867284,
     +		24.58469309438919,24.58475502191381,24.58481448888889,
     +		24.58487162396122,24.58492654747850,24.58497937212360,
     +		24.58503020349303,24.58507914062500,24.58512627648224/
      DATA SHEL/0.375D0,0.622D0,0.622D0,0.842D0,
     +          0.842D0,0.842D0,0.842D0,0.747D0,
     +          0.747D0,0.912D0,0.912D0,0.912D0,
     +          0.993D0,0.993D0,0.912D0,0.810D0,
     +          0.810D0,0.937D0,0.995D0,0.995D0,
     +          1.000D0,1.000D0,0.937D0,0.949D0,
     +          0.958D0,75*1.000D0/
	DATA NCI/2,2,2,2,2,2,3,3,3,3,2,2,2,3,3,3,3,3,
     +		3,3,3,3,3,2,3,4,4,4,3,3,3,3,3,3,4,3,
     +		3,3,3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
     +		4,4,4,5,4,4,4,4,4,5,5,5,5,5,5,5,5,5,
     +		5,5,5,5,6,5,5,5,5,5,6,6,6,6,6,6,6,7,
     +		6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,8,8,
     +		8,8,8,8,8,8,8,8,9,9,9,9,9,9,9,10,10,
     +		10,11,11,11,2,3,3,3,2,2/
	DATA GCI/1.0D0,3.0D0,5.0D0,5.0D0,1.0D0,    
     +		5.0D0,1.0D0,3.0D0,5.0D0,3.0D0,    
     +		7.0D0,5.0D0,3.0D0,3.0D0,3.0D0,    
     +		5.0D0,7.0D0,3.0D0,1.0D0,3.0D0,    
     +		5.0D0,5.0D0,1.0D0,9.0D0,5.0D0,    
     +		1.0D0,3.0D0,5.0D0,5.0D0,7.0D0,    
     +		9.0D0,3.0D0,5.0D0,7.0D0,3.0D0,    
     +		3.0D0,3.0D0,5.0D0,3.0D0,1.0D0,    
     +		3.0D0,5.0D0,7.0D0,3.0D0,3.0D0,    
     +		1.0D0,3.0D0,5.0D0,5.0D0,1.0D0,    
     +		5.0D0,5.0D0,7.0D0,9.0D0,3.0D0,    
     +		5.0D0,7.0D0,3.0D0,7.0D0,3.0D0,	 
     +		5.0D0,3.0D0,1.0D0,3.0D0,3.0D0,	 
     +		5.0D0,7.0D0,5.0D0,1.0D0,5.0D0,	 
     +		5.0D0,7.0D0,9.0D0,3.0D0,5.0D0,	 
     +		7.0D0,3.0D0,7.0D0,3.0D0,5.0D0,	 
     +		3.0D0,1.0D0,5.0D0,5.0D0,7.0D0,	 
     +		9.0D0,3.0D0,5.0D0,7.0D0,3.0D0,	 
     +		7.0D0,5.0D0,3.0D0,1.0D0,3.0D0,	 
     +		5.0D0,7.0D0,9.0D0,3.0D0,5.0D0,	 
     +		7.0D0,7.0D0,3.0D0,5.0D0,3.0D0,	 
     +		1.0D0,9.0D0,7.0D0,5.0D0,3.0D0,	 
     +		5.0D0,7.0D0,7.0D0,5.0D0,3.0D0,	 
     +		1.0D0,9.0D0,7.0D0,5.0D0,3.0D0,	 
     +		5.0D0,7.0D0,7.0D0,3.0D0,5.0D0,	 
     +		7.0D0,3.0D0,5.0D0,7.0D0,5.0D0,	 
     +		3.0D0,5.0D0,7.0D0,3.0D0,3.0D0/
	DATA ENCI/0.0D0,2.0333605D-03,5.3933649D-03,1.263870,2.684086,
     +		4.182672,7.480511,7.482891,7.487915,7.684888,
     +		7.946046,7.946620,7.946474,8.537387,8.640516,
     +		8.643146,8.647287,8.771255,8.846707,8.848247,
     +		8.850785,9.002712,9.171972,9.330682,9.631248,
     +		9.683908,9.685375,9.689256,9.695577,9.697620,
     +		9.701885,9.708156,9.708925,9.710041,9.712769,
     +		9.714380,9.761111,9.833419,9.834406,9.834934,
     +		9.940317,9.942698,9.946449,9.988707,10.05592,
     +		10.08144,10.08328,10.08553,10.13833,10.19809,
     +		10.35278,10.38514,10.38514,10.38514,10.39370,
     +		10.39456,10.39580,10.40021,10.40845,10.41874,
     +		10.42750,10.42990,10.42990,10.52043,10.52041,
     +		10.52041,10.53705,10.58840,10.61635,10.67973,
     +		10.70230,10.70328,10.70328,10.70878,10.70878,
     +		10.71184,10.71407,10.71854,10.72362,10.72523,
     +		10.72684,10.72684,10.86509,10.87426,10.87513,
     +		10.87513,10.87997,10.87997,10.88257,10.88533,
     +		10.88679,10.88964,10.89075,10.89075,10.88980,
     +		10.97789,10.97854,10.97854,10.98597,10.98597,
     +		10.98597,10.98808,10.98913,10.98994,10.98994,
     +		10.98994,11.04474,11.04474,11.04487,11.05280,
     +		11.05280,11.05280,11.05392,11.05429,11.05429,
     +		11.05429,11.09049,11.09049,11.09049,11.09843,
     +		11.09843,11.09843,11.09880,11.13129,11.13129,
     +		11.13129,11.15477,11.15477,11.15477,12.13544,
     +		12.83767,12.84024,12.84331,13.11772,14.86312/
	DATA NCII/2,2,2,2,2,2,2,2,2,2,3,3,3,2,3,3,2,2,4,4,4,3,3,3,
     +		4,4,2,2,4,4,5,5,5,3,3,5,5,5,5,6,3,3,3,3,3,3,6,6,
     +		6,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
     +		3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
     +		4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
     +		4,4,4,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
     +		5,5,5,5,5,5,6,6,6,6,6,6,6/
	DATA GCII/2.0D0,4.0D0,2.0D0,4.0D0,6.0D0,
     +		6.0D0,4.0D0,2.0D0,2.0D0,4.0D0,
     +		2.0D0,2.0D0,4.0D0,4.0D0,4.0D0,
     +		6.0D0,6.0D0,4.0D0,2.0D0,2.0D0,
     +		4.0D0,2.0D0,4.0D0,6.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		2.0D0,2.0D0,4.0D0,2.0D0,4.0D0,
     +		4.0D0,6.0D0,6.0D0,8.0D0,2.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,6.0D0,8.0D0,
     +		4.0D0,2.0D0,4.0D0,6.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		10.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		4.0D0,6.0D0,6.0D0,4.0D0,2.0D0,
     +		6.0D0,8.0D0,4.0D0,2.0D0,2.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,4.0D0,6.0D0,4.0D0,
     +		6.0D0,8.0D0,10.0D0,2.0D0,4.0D0,
     +		6.0D0,8.0D0,4.0D0,6.0D0,6.0D0,
     +		4.0D0,2.0D0,6.0D0,8.0D0,4.0D0,
     +		6.0D0,8.0D0,10.0D0,6.0D0,8.0D0,
     +		6.0D0,8.0D0,10.0D0,12.0D0,8.0D0,
     +		10.0D0,8.0D0,6.0D0,4.0D0,2.0D0,
     +		6.0D0,4.0D0,4.0D0,2.0D0,2.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,6.0D0,4.0D0,
     +		2.0D0,6.0D0,8.0D0,4.0D0,6.0D0,
     +		8.0D0,10.0D0,6.0D0,8.0D0,10.0D0,
     +		12.0D0,8.0D0,6.0D0,4.0D0,2.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,6.0D0,
     +		4.0D0,2.0D0/
	DATA ENCII/0.0D0,7.9350658D-03,5.331397,5.334075,5.337658,
     +		9.290338,9.290624,11.96386,13.71590,13.72101,
     +		14.44900,16.33194,16.33332,17.60895,18.04607,
     +		18.04625,18.65519,18.65582,19.49478,20.14995,
     +		20.15068,20.70119,20.70413,20.70971,20.84491,
     +		20.84496,20.92025,20.92256,20.95094,20.95094,
     +		21.49265,21.73314,21.73405,22.09347,22.13075,
     +		22.13075,22.13075,22.18799,22.18799,22.47211,
     +		22.52747,22.52929,22.53239,22.53689,22.56844,
     +		22.57086,22.82136,22.82136,22.85996,22.85996,
     +		22.89870,23.11398,23.11600,23.11878,23.38108,
     +		23.38522,24.12408,24.27024,24.27201,24.27444,
     +		24.27787,24.37010,24.37079,24.37187,24.37315,
     +		24.60198,24.60332,24.65351,24.65617,24.65793,
     +		24.78982,24.79512,25.06741,25.07039,25.98117,
     +		25.98415,25.98986,26.58329,26.58615,26.62689,
     +		26.62867,26.63139,26.63554,26.75178,26.82771,
     +		26.82771,26.83016,26.89454,26.89578,27.22147,
     +		27.22329,27.22585,27.22930,27.29263,27.29263,
     +		27.29378,27.29509,27.35131,27.35294,27.37703,
     +		27.37957,27.38104,27.41188,27.41302,27.41395,
     +		27.41395,27.41395,27.41409,27.46301,27.46301,
     +		27.46810,27.46936,27.47200,27.47561,27.47330,
     +		27.47864,27.48713,27.49096,27.49330,27.49330,
     +		27.48854,27.49412,27.55688,27.56022,27.99752,
     +		27.99752,27.99752,28.25640,28.25640,28.61124,
     +		28.61124,28.61124,28.61124,28.64683,28.64683,
     +		28.64683,28.66803,26.43629,28.66875,28.66875,
     +		28.66875,28.66875,28.70253,28.70253,28.70253,
     +		28.70253,28.70515,28.70515,28.70515,28.70515,
     +		29.31561,29.31561,29.31561,29.31561,29.33557,
     +		29.33557,29.33557/
	DATA NCIII/2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,4,
     +		3,4,4,4,4,3,4,4,4,4,4,4,4,4,3,3,3,4,3,3,3,3,3,3,
     +		3,3,3,3,3,3,5,3,3,3,3,5,5,5,5,3,5,5,5,5,5,5,5,5,
     +		5,5,5,5,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,
     +		7,8,8,8,8,9,9,9,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
     +		4,4,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,6,6,6,6,6,
     +		6,6,6,6,6,6,7,7,7,7,7,7/
	DATA GCIII/1.0D0,1.0D0,3.0D0,5.0D0,3.0D0,
     +		1.0D0,3.0D0,5.0D0,5.0D0,1.0D0,
     +		3.0D0,1.0D0,3.0D0,1.0D0,3.0D0,
     +		5.0D0,3.0D0,5.0D0,7.0D0,5.0D0,
     +		1.0D0,3.0D0,5.0D0,3.0D0,3.0D0,
     +		1.0D0,1.0D0,3.0D0,5.0D0,3.0D0,
     +		3.0D0,5.0D0,7.0D0,5.0D0,7.0D0,
     +		9.0D0,3.0D0,7.0D0,3.0D0,5.0D0,
     +		7.0D0,5.0D0,3.0D0,1.0D0,3.0D0,
     +		5.0D0,5.0D0,5.0D0,5.0D0,7.0D0,
     +		9.0D0,3.0D0,5.0D0,7.0D0,3.0D0,
     +		5.0D0,3.0D0,1.0D0,7.0D0,3.0D0,
     +		1.0D0,3.0D0,5.0D0,1.0D0,3.0D0,
     +		5.0D0,7.0D0,7.0D0,9.0D0,11.0D0,
     +		9.0D0,5.0D0,3.0D0,5.0D0,7.0D0,
     +		9.0D0,7.0D0,3.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,7.0D0,9.0D0,11.0D0,
     +		9.0D0,5.0D0,5.0D0,7.0D0,9.0D0,
     +		7.0D0,3.0D0,3.0D0,3.0D0,5.0D0,
     +		7.0D0,5.0D0,3.0D0,3.0D0,5.0D0,
     +		7.0D0,3.0D0,5.0D0,7.0D0,1.0D0,
     +		3.0D0,5.0D0,3.0D0,3.0D0,5.0D0,
     +		7.0D0,1.0D0,3.0D0,5.0D0,5.0D0,
     +		5.0D0,3.0D0,5.0D0,7.0D0,5.0D0,
     +		3.0D0,1.0D0,7.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,1.0D0,3.0D0,5.0D0,
     +		5.0D0,5.0D0,3.0D0,5.0D0,7.0D0,
     +		5.0D0,3.0D0,1.0D0,3.0D0,5.0D0,
     +		7.0D0,1.0D0,3.0D0,5.0D0,3.0D0,
     +		5.0D0,7.0D0,5.0D0,3.0D0,1.0D0,
     +		3.0D0,5.0D0,7.0D0,1.0D0,3.0D0,5.0D0/
	DATA ENCIII/0.0D0,6.486296,6.489148,6.496191,12.69008,
     +		17.03237,17.03602,17.04185,18.08638,22.62984,
     +		29.52845,30.64541,32.10371,32.19328,32.19396,
     +		32.19555,33.47080,33.45866,33.47146,34.27982,
     +		38.20770,38.21183,38.22034,38.36164,38.43612,
     +		38.64882,39.39549,39.39549,39.39611,39.64054,
     +		39.84380,39.84582,39.84874,39.91699,39.91782,
     +		39.91892,39.97328,40.01022,40.05026,40.05341,
     +		40.05822,40.19756,40.57121,40.86969,40.87231,
     +		40.87686,41.24874,41.30157,41.32848,41.33158,
     +		41.33611,41.85783,41.80309,41.86202,42.14028,
     +		42.16117,42.16444,42.16623,42.32471,42.55869,
     +		42.67342,42.67342,42.67342,42.78661,42.83001,
     +		42.83001,42.83001,42.96405,42.96405,42.96416,
     +		42.96405,42.98029,42.98736,43.03527,43.03550,
     +		43.03579,43.25349,43.98952,44.27370,44.39248,
     +		44.39248,44.39248,44.46592,44.46592,44.46600,
     +		44.47219,44.47673,44.48596,44.48596,44.48596,
     +		44.52591,45.07626,45.24178,45.32720,45.32720,
     +		45.32720,45.38200,45.86543,45.92891,45.92891,
     +		45.92891,46.33929,46.33929,46.33929,46.69749,
     +		46.69749,46.69749,47.25143,47.35238,47.35238,
     +		47.35722,47.64920,47.64920,47.65379,47.81342,
     +		47.83558,48.06245,48.06245,48.06245,48.16114,
     +		48.16114,48.16114,48.20208,50.51542,50.55803,
     +		50.55803,50.55803,50.69428,50.69428,50.69428,
     +		50.77264,50.79460,50.90022,50.90022,50.90022,
     +		50.93829,50.93829,50.93829,52.24497,52.24497,
     +		52.24497,52.31775,52.31775,52.31775,52.43107,
     +		52.43107,52.43107,52.45302,52.45302,52.45302,
     +		53.23251,53.23251,53.23251,53.27802,53.27802,
     +		53.27802/
	DATA NCIV/2,2,2,3,3,3,3,3,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,
     +		6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,8,8,
     +		8,8,8,8,8,8,8/
	DATA GCIV/2.0D0,2.0D0,4.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,6.0D0,8.0D0,
     +		2.0D0,2.0D0,4.0D0,4.0D0,6.0D0,
     +		6.0D0,8.0D0,8.0D0,10.0D0,2.0D0,
     +		2.0D0,4.0D0,4.0D0,6.0D0,6.0D0,
     +		8.0D0,8.0D0,10.0D0,10.0D0,12.0D0,
     +		2.0D0,2.0D0,4.0D0,4.0D0,6.0D0,
     +		6.0D0,8.0D0,8.0D0,10.0D0,10.0D0,
     +		12.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		8.0D0,10.0D0,12.0D0,14.0D0,16.0D0/
	DATA ENCIV/0.0D0,7.995100,8.008378,37.54872,39.68134,
     +		39.68525,40.28040,40.28173,49.76113,50.62434,
     +		50.62599,50.87540,50.87595,50.88784,50.88784,
     +		55.21889,55.65134,55.65221,55.77947,55.77947,
     +		55.78577,55.78578,55.78703,55.78703,58.12002,
     +		58.36774,58.36774,58.44275,58.44275,58.44709,
     +		58.44709,58.44764,58.44764,58.44770,58.44770,
     +		59.84267,60.00038,60.00038,60.04725,60.04725,
     +		60.05156,60.05156,60.05191,60.05191,60.05194,
     +		60.05194,61.05946,61.05946,61.09294,61.09294,
     +		61.09319,61.09319,61.09319,61.09319,61.09319/
	DATA NCV/1,2,2,2,2,2,3,3,3,3,4,5,6,7,8/
	DATA GCV/1.0D0,3.0D0,1.0D0,3.0D0,5.0D0,
     +		3.0D0,3.0D0,5.0D0,7.0D0,3.0D0,
     +		3.0D0,3.0D0,3.0D0,3.0D0,3.0D0/
	DATA ENCV/0.0D0,298.9618,304.4046,304.4030,304.4199,
     +		307.8855,354.2645,354.2645,354.2645,354.5177,
     +		370.9247,378.5349,382.6710,385.1917,386.6807/
	DATA NNI/2,2,2,2,2,3,3,3,3,3,2,2,2,3,3,3,3,3,3,3,3,3,3,3,
     +		3,3,3,3,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
     +		3,3,4,4,4,4,4,4,4,4,4,5,5,5,5,5,4,4,4,4,4,4,4,4,
     +		4,4,4,4,4,4,4,4,4,3,3,3,3,6,6,6,6,6,5,5,5,5,5,5,
     +		5,5,5,5,5,5,5,5,5,5,5,7,7,7,7,7,6,6,6,6,6,6,6,6,
     +		6,6,6,6,6,6,6,6,6,8,8,8,8,8,7,7,7,7,7,7,7,7,7,7,
     +		7,7,7,9,9,9,9,9,8,8,8,8,8,8,8,8,8,8,8,8,8,10,10,10,
     +		10,10,9,9,9,9,9,9,9,9,9,9,9,9,9,11,11,11,11,11,10,
     +		10,10,10,10,10,10,10,10,10,10,10,10,12,12,12,12,12,
     +		11,11,11,11,11,11,11,11,11,11,11,11,11,13,13,12,12,
     +		12,12,12,12,12/
	DATA GNI/4.0D0,6.0D0,4.0D0,4.0D0,2.0D0,
     +		2.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		6.0D0,4.0D0,2.0D0,2.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,2.0D0,4.0D0,
     +		6.0D0,4.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,6.0D0,4.0D0,2.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,10.0D0,6.0D0,
     +		8.0D0,2.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,4.0D0,6.0D0,
     +		2.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		2.0D0,4.0D0,6.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,4.0D0,
     +		6.0D0,8.0D0,10.0D0,2.0D0,4.0D0,
     +		6.0D0,8.0D0,4.0D0,2.0D0,6.0D0,
     +		8.0D0,2.0D0,4.0D0,6.0D0,4.0D0,
     +		6.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		2.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		4.0D0,6.0D0,8.0D0,10.0D0,4.0D0,
     +		2.0D0,6.0D0,8.0D0,2.0D0,4.0D0,
     +		6.0D0,8.0D0,2.0D0,4.0D0,6.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,4.0D0,6.0D0,8.0D0,
     +		10.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		4.0D0,2.0D0,6.0D0,8.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,6.0D0,8.0D0,
     +		4.0D0,2.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,6.0D0,
     +		8.0D0,4.0D0,2.0D0,6.0D0,8.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,2.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,4.0D0,
     +		2.0D0,6.0D0,8.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		2.0D0,4.0D0,6.0D0,4.0D0,2.0D0,
     +		6.0D0,8.0D0,2.0D0,4.0D0,6.0D0,
     +		8.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,2.0D0,4.0D0,
     +		6.0D0,4.0D0,2.0D0,6.0D0,8.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,4.0D0,2.0D0,2.0D0,4.0D0,
     +		6.0D0,4.0D0,6.0D0/
	DATA ENNI/0.0D0,2.383371,2.384363,3.575739,3.575739,
     +		10.32619,10.33038,10.33617,10.67904,10.69042,
     +		10.92429,10.92973,10.93217,11.60284,11.75037,
     +		11.75317,11.75780,11.76412,11.83769,11.83997,
     +		11.84472,11.99580,12.00032,12.00975,12.12207,
     +		12.12649,12.35701,12.35614,12.84713,12.85333,
     +		12.86185,12.91211,12.92268,12.97078,12.97568,
     +		12.97693,12.97929,12.98350,12.98958,12.99502,
     +		13.00392,13.00161,13.00483,13.00074,13.01686,
     +		13.01822,13.01983,13.02095,13.03344,13.03636,
     +		13.20179,13.23674,13.23917,13.24364,13.25041,
     +		13.26429,13.26623,13.27127,13.32189,13.61527,
     +		13.62076,13.62945,13.64202,13.65185,13.66270,
     +		13.66493,13.66914,13.67609,13.66580,13.67249,
     +		13.67410,13.68043,13.66588,13.66872,13.67695,
     +		13.68464,13.67869,13.68191,13.68836,13.69398,
     +		13.69673,13.70310,13.70607,13.92292,13.92614,
     +		13.95653,13.96207,13.97100,13.97749,13.98841,
     +		13.97948,13.98097,13.98543,13.99324,13.98568,
     +		13.98754,13.98803,13.99674,13.98865,13.98865,
     +		13.98865,13.99696,13.99237,13.99473,13.99944,
     +		14.00155,14.00384,14.13620,14.14326,14.15244,
     +		14.15045,14.15455,14.15417,14.15417,14.15417,
     +		14.15417,14.15690,14.15690,14.15690,14.16508,
     +		14.15827,14.16025,14.15864,14.16843,14.16313,
     +		14.17035,14.16645,14.16645,14.16831,14.23464,
     +		14.24468,14.25113,14.25212,14.25212,14.25683,
     +		14.25683,14.25683,14.25683,14.25882,14.25882,
     +		14.26043,14.26043,14.26545,14.27073,14.27109,
     +		14.27109,14.27109,14.36247,14.36247,14.31821,
     +		14.31821,14.31821,14.32329,14.32329,14.32329,
     +		14.32329,14.32403,14.32403,14.32465,14.32465,
     +		14.33234,14.33544,14.33494,14.33494,14.33494,
     +		14.36272,14.36272,14.36433,14.36433,14.36433,
     +		14.36830,14.36830,14.36830,14.36830,14.36854,
     +		14.36854,14.37016,14.37016,14.37896,14.38119,
     +		14.38107,14.38107,14.38107,14.39557,14.39557,
     +		14.39768,14.39768,14.39768,14.40152,14.40152,
     +		14.40202,14.40202,14.40264,14.40264,14.40264,
     +		14.40264,14.41206,14.41206,14.41442,14.41442,
     +		14.41442,14.42012,14.42012,14.42099,14.42099,
     +		14.42099,14.42583,14.42583,14.42682,14.42682,
     +		14.42781,14.42781,14.42781,14.42781,14.43636,
     +		14.43636,14.43698,14.43698,14.43698,14.46253,
     +		14.44021,14.44455,14.44455,14.45434,14.45434,
     +		14.45434,14.45980,14.45980/
	DATA NNII/2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,2,3,3,3,3,2,3,
     +		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,3,4,4,4,
     +		4,4,4,4,4,4,3,3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
     +		4,4,4,4,4,4,4,4,4,4,3,3,3,5,5,5,5,5,5,5,5,5,5,5,
     +		5,5,5,5,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3/
	DATA GNII/1.0D0,3.0D0,5.0D0,5.0D0,1.0D0,
     +		5.0D0,7.0D0,5.0D0,3.0D0,5.0D0,
     +		3.0D0,1.0D0,5.0D0,1.0D0,3.0D0,
     +		5.0D0,3.0D0,3.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,3.0D0,3.0D0,1.0D0,
     +		3.0D0,5.0D0,5.0D0,1.0D0,5.0D0,
     +		7.0D0,9.0D0,5.0D0,3.0D0,5.0D0,
     +		7.0D0,5.0D0,3.0D0,1.0D0,7.0D0,
     +		3.0D0,1.0D0,3.0D0,5.0D0,3.0D0,
     +		3.0D0,3.0D0,5.0D0,7.0D0,1.0D0,
     +		3.0D0,5.0D0,3.0D0,5.0D0,3.0D0,
     +		5.0D0,7.0D0,1.0D0,5.0D0,7.0D0,
     +		9.0D0,5.0D0,3.0D0,5.0D0,7.0D0,
     +		5.0D0,3.0D0,1.0D0,7.0D0,5.0D0,
     +		7.0D0,9.0D0,7.0D0,7.0D0,9.0D0,
     +		11.0D0,3.0D0,9.0D0,7.0D0,5.0D0,
     +		3.0D0,5.0D0,1.0D0,3.0D0,5.0D0,
     +		1.0D0,3.0D0,5.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,5.0D0,7.0D0,9.0D0,
     +		7.0D0,7.0D0,9.0D0,11.0D0,9.0D0,
     +		1.0D0,3.0D0,5.0D0,7.0D0,9.0D0,
     +		3.0D0,5.0D0,7.0D0,5.0D0,3.0D0,
     +		5.0D0,7.0D0,9.0D0,11.0D0,7.0D0,
     +		5.0D0,3.0D0,1.0D0,3.0D0,5.0D0,
     +		7.0D0,9.0D0/
	DATA ENNII/0.0D0,6.0876831D-03,1.6279284D-02,1.898923,4.052723,
     +		5.848106,11.43604,11.43781,11.43801,13.54146,
     +		13.54146,13.54228,17.87734,18.46259,18.46651,
     +		18.48341,18.49722,19.23384,20.40944,20.64636,
     +		20.65389,20.66582,20.67651,20.94027,21.14861,
     +		21.15298,21.16022,21.59986,22.10340,23.12481,
     +		23.13218,23.14229,23.19670,23.23962,23.24260,
     +		23.24636,23.41565,23.42207,23.42555,23.47490,
     +		23.57225,24.36823,24.37465,24.38944,24.53166,
     +		25.06612,25.13369,25.14001,25.15193,25.18946,
     +		25.19245,25.20124,25.23510,25.46049,25.53877,
     +		25.54572,25.55447,25.58160,25.99668,26.00464,
     +		26.01527,26.02787,26.06667,26.06994,26.07548,
     +		26.12440,26.13011,26.13327,26.16475,26.16510,
     +		26.16800,26.16849,26.17391,26.19663,26.19758,
     +		26.20937,26.20252,26.21087,26.21191,26.21252,
     +		26.22134,26.22182,26.25393,26.25770,26.26368,
     +		26.55921,26.56489,26.58065,26.63554,27.36569,
     +		27.36569,27.36569,27.40948,27.40948,27.40999,
     +		27.41783,27.42901,27.42963,27.43824,27.43947,
     +		27.77609,27.77805,27.78169,27.78704,27.79372,
     +		28.01910,28.02209,28.02755,28.54429,30.17253,
     +		30.17448,30.17763,30.18179,30.18682,30.34387,
     +		30.34864,30.35188,30.41607,30.41652,30.41750,
     +		30.41894,30.42068/
	DATA NNIII/2,2,2,2,2,2,2,2,2,2,2,2,2,3,2,2,3,3,3,3,3,3,3,3,
     +		3,4,3,3,3,3,3,3,4,4,3,3,3,3,4,4,4,4,3,3,3,3,3,3,
     +		3,3,3,3,3,5,3,3,3,3,3,3,3,5,5,3,3,5,5,5,5,6,6,6,
     +		6,6,6,4,4,4,3,3,4,4,4,4,4,4,3,3,4,4,4,4,4,4,4,4,
     +		4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
     +		4,4,4,4,4,4,4,3,3,5,5,5,5/
	DATA GNIII/2.0D0,4.0D0,2.0D0,4.0D0,6.0D0,
     +		6.0D0,4.0D0,2.0D0,2.0D0,4.0D0,
     +		4.0D0,6.0D0,4.0D0,2.0D0,2.0D0,
     +		4.0D0,2.0D0,4.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		2.0D0,2.0D0,4.0D0,2.0D0,4.0D0,
     +		6.0D0,8.0D0,2.0D0,4.0D0,4.0D0,
     +		2.0D0,4.0D0,6.0D0,4.0D0,6.0D0,
     +		6.0D0,8.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,10.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,2.0D0,4.0D0,
     +		6.0D0,6.0D0,4.0D0,2.0D0,6.0D0,
     +		8.0D0,4.0D0,6.0D0,4.0D0,2.0D0,
     +		6.0D0,8.0D0,8.0D0,10.0D0,4.0D0,
     +		6.0D0,6.0D0,8.0D0,8.0D0,10.0D0,
     +		2.0D0,4.0D0,6.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,2.0D0,4.0D0,6.0D0,
     +		8.0D0,2.0D0,4.0D0,4.0D0,6.0D0,
     +		4.0D0,2.0D0,4.0D0,6.0D0,4.0D0,
     +		6.0D0,8.0D0,10.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,6.0D0,
     +		4.0D0,2.0D0,6.0D0,8.0D0,4.0D0,
     +		6.0D0,8.0D0,10.0D0,6.0D0,8.0D0,
     +		6.0D0,8.0D0,10.0D0,12.0D0,8.0D0,
     +		10.0D0,8.0D0,6.0D0,4.0D0,2.0D0,
     +		6.0D0,4.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0/
	DATA ENNIII/0.0D0,2.1635452D-02,7.180255,7.098413,7.108480,
     +		12.52548,12.52643,16.24252,18.08651,18.10019,
     +		23.16076,25.17799,25.18006,27.43827,28.56680,
     +		28.56730,30.45896,30.46342,33.13367,33.13441,
     +		35.65022,35.65797,35.67233,36.84229,36.85629,
     +		38.44641,38.32793,38.33453,38.39367,38.39807,
     +		38.40689,38.41771,38.64517,38.64825,38.95919,
     +		39.34056,39.34595,39.35325,39.39646,39.40031,
     +		39.71098,39.71098,39.79651,39.80747,40.55027,
     +		40.94474,40.94909,40.95552,40.96437,41.26192,
     +		41.26358,41.26631,41.26982,41.37555,41.47835,
     +		41.48166,41.68555,41.69232,41.69667,42.12335,
     +		42.13715,42.39634,42.39655,42.48893,42.49769,
     +		42.49625,42.49625,42.54757,42.54757,43.95493,
     +		43.95493,44.00932,44.00932,44.04135,44.04135,
     +		45.69180,45.69957,45.71402,46.28896,46.29317,
     +		46.46321,46.47039,46.71232,46.71811,46.72555,
     +		46.73671,46.81577,46.81788,46.85206,46.86286,
     +		46.92110,47.02857,47.03412,47.04068,47.61238,
     +		47.61238,47.61845,47.62763,47.75000,47.75000,
     +		47.77108,47.77108,49.01428,47.77802,47.88887,
     +		47.88887,47.88887,47.97657,47.97913,47.98245,
     +		47.98245,47.98363,47.98760,48.07270,48.08297,
     +		48.11119,48.11662,48.12305,48.13089,48.12993,
     +		48.14229,48.14024,48.14488,48.15087,48.15427,
     +		48.15307,48.16119,49.16950,49.17073,50.71214,
     +		50.71214,50.71214,50.71214/
	DATA NNIV/2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
     +		3,3,3,3,3,3,4,3,3,3,3,3,4,4,4,3,3,3,3,4,4,4,4,3,
     +		3,3,4,4,4,4,3,4,5,5,5,5,5,5,5,6,6,6,4,4,4,4,5,5,4/
	DATA GNIV/1.0D0,1.0D0,3.0D0,7.0D0,3.0D0,
     +		1.0D0,3.0D0,5.0D0,5.0D0,1.0D0,
     +		3.0D0,1.0D0,1.0D0,3.0D0,5.0D0,
     +		3.0D0,5.0D0,7.0D0,5.0D0,1.0D0,
     +		3.0D0,5.0D0,3.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,3.0D0,1.0D0,3.0D0,
     +		5.0D0,5.0D0,5.0D0,5.0D0,7.0D0,
     +		9.0D0,1.0D0,3.0D0,5.0D0,3.0D0,
     +		5.0D0,7.0D0,7.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,5.0D0,3.0D0,1.0D0,
     +		5.0D0,5.0D0,7.0D0,9.0D0,3.0D0,
     +		7.0D0,3.0D0,3.0D0,5.0D0,7.0D0,
     +		7.0D0,9.0D0,11.0D0,3.0D0,5.0D0,
     +		7.0D0,5.0D0,3.0D0,5.0D0,7.0D0,
     +		3.0D0,5.0D0,7.0D0/
	DATA ENNIV/0.0D0,8.323934,8.331770,8.349648,16.20427,
     +		21.75491,21.76399,21.77946,23.41898,29.18244,
     +		46.76804,50.15470,50.32483,50.32679,50.33118,
     +		52.06988,52.07031,52.07132,53.20933,57.68086,
     +		57.69048,57.71067,58.64906,59.62210,60.05779,
     +		60.05779,60.07403,60.44809,61.27855,61.27855,
     +		61.29070,61.78379,61.95650,61.97423,61.97423,
     +		61.97423,62.44215,62.44215,62.44215,62.67301,
     +		62.67685,62.68218,62.77282,62.86333,63.40415,
     +		63.40415,63.40415,63.41109,63.41767,63.41767,
     +		63.80760,64.05482,64.05569,64.05706,64.39976,
     +		64.70402,68.21900,68.53058,68.53058,68.53058,
     +		68.73986,68.73986,68.73986,71.28416,71.28416,
     +		71.28416,73.28070,73.60580,73.60580,73.61063,
     +		78.63129,78.63129,78.63129/
	DATA NNV/2,2,2,3,3,3,3,3,4,4,4,4,4,5,5,5,5,5,6,6,6,6,6,6,
     +		6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8/
	DATA GNV/2.0D0,2.0D0,4.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,6.0D0,8.0D0,
     +		8.0D0,10.0D0,12.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,6.0D0,8.0D0,
     +		8.0D0,10.0D0,12.0D0,14.0D0,2.0D0,
     +		2.0D0,4.0D0,4.0D0,6.0D0,6.0D0,
     +		8.0D0,8.0D0,10.0D0,12.0D0,14.0D0,16.0D0/
	DATA ENNV/0.0D0,9.976473,10.00851,56.55396,59.23740,
     +		59.24660,60.05890,60.06188,75.17694,76.26962,
     +		76.26962,76.61120,76.61120,83.55153,84.09893,
     +		84.09893,84.27598,84.27598,88.02306,88.33514,
     +		88.33514,88.43854,88.43742,88.44214,88.44214,
     +		88.44313,88.44313,88.44313,90.68689,90.88043,
     +		90.88043,90.94527,90.94527,90.94912,90.94912,
     +		90.94974,90.94974,90.94974,90.94974,92.40136,
     +		92.53167,92.53167,92.57358,92.57358,92.57618,
     +		92.57618,92.57668,92.57668,92.57668,92.57668,
     +		92.57668/
	DATA NNVI/1,2,2,2,2,2,3,4/
	DATA GNVI/1.0D0,3.0D0,1.0D0,3.0D0,5.0D0,3.0D0,3.0D0,3.0D0/
	DATA ENNVI/0.0D0,419.8009,426.2953,426.2965,426.3325,
     +		425.7398,497.9737,521.5830/
	DATA NOI/2,2,2,2,2,3,3,3,3,3,3,3,3,4,4,3,3,3,3,3,3,3,3,3,
     +		4,4,4,4,4,4,3,3,3,5,5,3,4,4,4,4,4,4,4,4,5,5,5,6,
     +		6,5,5,5,5,5,5,5,5,6,6,6,7,7,6,6,6,6,6,6,6,6,8,8,
     +		7,7,7,7,7,7,7,7,9,9,8,8,8,8,8,8,8,8,10,10,9,9,9,9,
     +		9,9,9,9,11,11,10,10,10,10,10,10,10,10,3,3,3,3,3,3,
     +		3,3,3,3,3,3,4,3,3,3,3,3,3,3,3,3,3,3,4,4,4,2,2,2,3,
     +		3,3,3,3,5,4,4,4,4,4,4,4,4,4,4,4,3,6,5,5,5,5,5,5,5,
     +		5,5,5,7,6,6,6,2/
	DATA GOI/5.0D0,3.0D0,1.0D0,5.0D0,1.0D0,
     +		5.0D0,3.0D0,3.0D0,5.0D0,7.0D0,
     +		5.0D0,3.0D0,1.0D0,5.0D0,3.0D0,
     +		9.0D0,7.0D0,5.0D0,5.0D0,3.0D0,
     +		1.0D0,7.0D0,5.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,5.0D0,3.0D0,1.0D0,
     +		7.0D0,5.0D0,3.0D0,5.0D0,3.0D0,
     +		5.0D0,9.0D0,7.0D0,5.0D0,3.0D0,
     +		1.0D0,7.0D0,5.0D0,3.0D0,5.0D0,
     +		3.0D0,1.0D0,5.0D0,3.0D0,9.0D0,
     +		7.0D0,5.0D0,3.0D0,1.0D0,7.0D0,
     +		5.0D0,3.0D0,5.0D0,3.0D0,1.0D0,
     +		5.0D0,3.0D0,9.0D0,7.0D0,5.0D0,
     +		3.0D0,1.0D0,7.0D0,5.0D0,3.0D0,
     +		5.0D0,3.0D0,9.0D0,7.0D0,5.0D0,
     +		3.0D0,1.0D0,7.0D0,5.0D0,3.0D0,
     +		5.0D0,3.0D0,9.0D0,7.0D0,5.0D0,
     +		3.0D0,1.0D0,7.0D0,5.0D0,3.0D0,
     +		5.0D0,3.0D0,9.0D0,7.0D0,5.0D0,
     +		3.0D0,1.0D0,7.0D0,5.0D0,3.0D0,
     +		5.0D0,3.0D0,9.0D0,7.0D0,5.0D0,
     +		3.0D0,1.0D0,7.0D0,5.0D0,3.0D0,
     +		7.0D0,5.0D0,3.0D0,9.0D0,7.0D0,
     +		5.0D0,5.0D0,3.0D0,1.0D0,7.0D0,
     +		3.0D0,5.0D0,5.0D0,5.0D0,3.0D0,
     +		1.0D0,9.0D0,7.0D0,5.0D0,9.0D0,
     +		11.0D0,9.0D0,7.0D0,7.0D0,7.0D0,
     +		5.0D0,3.0D0,5.0D0,3.0D0,1.0D0,
     +		7.0D0,5.0D0,3.0D0,3.0D0,5.0D0,
     +		5.0D0,9.0D0,7.0D0,5.0D0,9.0D0,
     +		11.0D0,9.0D0,7.0D0,7.0D0,5.0D0,
     +		3.0D0,1.0D0,1.0D0,5.0D0,9.0D0,
     +		7.0D0,5.0D0,9.0D0,11.0D0,9.0D0,
     +		7.0D0,5.0D0,3.0D0,1.0D0,5.0D0,
     +		5.0D0,3.0D0,1.0D0,3.0D0/
	DATA ENOI/0.0D0,01.9651687D-02,2.8082693D-02,1.967363,0.4206081,
     +		9.146132,9.521420,10.74028,10.74053,10.74098,
     +		10.98893,10.98886,10.98895,11.83768,11.93056,
     +		12.07869,12.07870,12.07870,12.07872,12.07872,
     +		12.07872,12.08711,12.08711,12.08711,12.28604,
     +		12.28612,12.28627,12.35891,12.35891,12.35891,
     +		12.53927,12.54078,12.54176,12.66092,12.69755,
     +		12.72854,12.75377,12.75377,12.75377,12.75377,
     +		12.75377,12.75911,12.75911,12.75911,12.87829,
     +		12.87829,12.87829,13.02082,13.03891,13.06624,
     +		13.06624,13.06624,13.06624,13.06624,13.06913,
     +		13.06913,13.06913,13.13145,13.13145,13.13145,
     +		13.21004,13.22030,13.23559,13.23559,13.23559,
     +		13.23559,13.23559,13.23740,13.23740,13.23740,
     +		13.32166,13.32807,13.33749,13.33749,13.33749,
     +		13.33749,13.33749,13.33869,13.33869,13.33869,
     +		13.39308,13.39756,13.40353,385.3597,13.40353,
     +		13.40353,13.40353,13.40488,13.40488,13.40488,
     +		13.44262,13.44449,13.44872,13.44872,13.44872,
     +		13.44872,13.44872,13.44966,13.44966,13.44966,
     +		13.47577,13.47812,13.48112,13.48112,13.48112,
     +		13.48112,13.48112,13.48148,13.48148,13.48148,
     +		14.04685,14.04687,14.04730,14.09888,14.09975,
     +		14.10046,14.12320,14.12450,14.12526,14.13382,
     +		14.37218,14.46048,15.22525,15.28698,15.29424,
     +		15.29817,15.40062,15.40062,15.40062,15.40372,
     +		15.40390,15.40622,15.40550,15.41465,15.59420,
     +		15.59514,15.59577,15.65520,15.66431,15.66970,
     +		15.78109,15.78181,15.78222,15.82895,15.94391,
     +		16.01073,16.07676,16.07676,16.07676,16.07836,
     +		16.07844,16.08080,16.08005,16.08545,16.11433,
     +		16.11550,16.11614,16.23505,16.35702,16.35702,
     +		16.35702,16.35702,16.39057,16.39063,16.39308,
     +		16.39308,16.40451,16.40451,16.40451,16.54127,
     +		16.56668,16.56668,16.56668,23.53702/
	DATA NOII/2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,2,3,3,3,3,3,3,3,3,
     +		3,3,3,3,3,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
     +		3,3,3,3,3,3,3,3,3,4,4,4,4,4,3,4,4,4,4,4,4,4,4,3,
     +		3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,3,4,4,4,4,4,4,
     +		4,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,5,5,
     +		5,5,5,4,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
     +		5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,3,3,3,4,4,4,4,
     +		4,4,4,4,4,4,3,3,4,4,4,4,4,4,4,5,5,3,3,3,3,3,4/
	DATA GOII/4.0D0,6.0D0,4.0D0,4.0D0,2.0D0,
     +		6.0D0,4.0D0,2.0D0,6.0D0,4.0D0,
     +		2.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		2.0D0,2.0D0,2.0D0,4.0D0,6.0D0,
     +		8.0D0,6.0D0,4.0D0,2.0D0,4.0D0,
     +		6.0D0,4.0D0,6.0D0,4.0D0,4.0D0,
     +		2.0D0,2.0D0,4.0D0,2.0D0,6.0D0,
     +		8.0D0,6.0D0,4.0D0,4.0D0,6.0D0,
     +		8.0D0,10.0D0,6.0D0,4.0D0,2.0D0,
     +		2.0D0,4.0D0,2.0D0,4.0D0,6.0D0,
     +		8.0D0,6.0D0,8.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		6.0D0,8.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,2.0D0,4.0D0,8.0D0,6.0D0,
     +		10.0D0,8.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,8.0D0,10.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,4.0D0,
     +		6.0D0,4.0D0,2.0D0,4.0D0,2.0D0,
     +		6.0D0,8.0D0,2.0D0,6.0D0,4.0D0,
     +		8.0D0,5.80D0,4.0D0,2.0D0,6.0D0,
     +		8.0D0,10.0D0,12.0D0,8.0D0,10.0D0,
     +		4.0D0,6.0D0,4.0D0,6.0D0,8.0D0,
     +		10.0D0,6.0D0,8.0D0,2.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,6.0D0,4.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,2.0D0,
     +		4.0D0,6.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,6.0D0,8.0D0,6.0D0,4.0D0,
     +		2.0D0,6.0D0,8.0D0,8.0D0,6.0D0,
     +		4.0D0,2.0D0,6.0D0,8.0D0,10.0D0,
     +		12.0D0,8.0D0,10.0D0,4.0D0,6.0D0,
     +		4.0D0,6.0D0,8.0D0,10.0D0,6.0D0,
     +		8.0D0,4.0D0,6.0D0,8.0D0,6.0D0,
     +		8.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		8.0D0,10.0D0,6.0D0,8.0D0,6.0D0,
     +		4.0D0,2.0D0,4.0D0,6.0D0,10.0D0,
     +		12.0D0,2.0D0,4.0D0,4.0D0,6.20D0,
     +		10.0D0,8.0D0,6.0D0,4.0D0,2.0D0,
     +		6.0D0/
	DATA ENOII/0.0D0,3.323850,3.326454,5.017305,5.017491,
     +		14.85813,14.87838,14.88860,20.58005,20.57736,
     +		22.96648,22.97954,23.001876,23.41940,23.44172,
     +		24.26523,25.28586,25.63160,25.63849,25.64984,
     +		25.66529,25.66142,25.66154,25.83188,25.83760,
     +		25.84900,26.22564,26.24928,26.30498,26.35845,
     +		26.37943,26.55392,26.56133,28.12621,28.35835,
     +		28.36128,28.51330,28.51270,28.67733,28.68403,
     +		28.69369,28.70637,28.82200,28.83108,28.83932,
     +		28.82414,28.82992,28.85285,28.85711,28.85729,
     +		28.85808,28.86334,28.88355,28.94193,28.95606,
     +		29.06249,29.06893,29.58618,29.59923,29.61924,
     +		29.79726,29.82051,30.42546,30.47162,30.47763,
     +		30.48836,30.50400,30.74951,30.77135,30.80112,
     +		30.81214,31.02747,31.02747,31.14773,31.14812,
     +		31.31967,31.31982,31.37404,31.37430,31.46620,
     +		31.46649,31.55199,31.55199,31.55199,31.56553,
     +		31.61407,31.61407,31.61407,31.61407,31.61407,
     +		31.62925,31.63375,31.63644,31.63766,31.65117,
     +		31.65364,31.67396,31.69345,31.70178,31.71699,
     +		31.70200,31.71709,31.72948,31.72935,31.70999,
     +		31.71043,31.71889,31.73747,31.71911,31.73823,
     +		31.72081,31.72752,31.75062,31.75112,31.75553,
     +		31.75715,31.75586,31.75803,31.95026,31.96318,
     +		31.98375,32.03889,32.06284,32.14771,32.14780,
     +		32.35511,32.35511,32.36540,32.38251,32.39264,
     +		32.39264,32.40412,32.44667,32.46798,32.88345,
     +		32.88345,32.88345,32.88345,32.90963,32.91418,
     +		32.91418,32.92780,32.92780,32.93536,32.94354,
     +		32.95061,32.96264,32.93858,32.94181,32.95049,
     +		32.97082,32.95073,32.97146,32.96227,32.96227,
     +		32.97119,32.97528,32.97826,32.97999,32.97863,
     +		32.97999,33.19875,33.19968,33.20123,34.06365,
     +		34.06901,34.08607,34.08607,34.17174,34.17174,
     +		34.20029,34.20029,34.20504,34.20504,34.21390,
     +		34.21390,34.21960,34.22819,34.22819,34.23350,
     +		34.23350,34.25269,34.25269,34.48530,34.48530,
     +		36.19083,36.18759,36.19109,36.19123,36.19131,37.05294/
	DATA NOIII/2,2,2,2,2,2,2,3,3,2,2,2,2,2,2,3,3,3,3,2,2,2,3,3,
     +		3,3,3,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
     +		2,3,3,3,4,4,4,4,3,3,3,3,3,3,4,4,4,4,4,3,3,3,4,4,
     +		4,4,4,3,3,3,3,4,4,4,4,3,3,3,4,4,4,4,4,4,4,4,5,5,
     +		5,5,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
     +		5,5,5,5,5,5,5,5,5,3,3,3,6,6,6,6,7,3,3,4,4,4,3,4,
     +		4,4,4,4,4,4,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,5/
	DATA GOIII/1.0D0,3.0D0,5.0D0,5.0D0,1.0D0,
     +		5.0D0,7.0D0,5.0D0,3.0D0,5.0D0,
     +		3.0D0,1.0D0,5.0D0,3.0D0,3.0D0,
     +		1.0D0,3.0D0,5.0D0,3.0D0,5.0D0,
     +		3.0D0,1.0D0,3.0D0,3.0D0,5.0D0,
     +		7.0D0,3.0D0,5.0D0,1.0D0,3.0D0,
     +		5.0D0,5.0D0,1.0D0,5.0D0,7.0D0,
     +		9.0D0,5.0D0,3.0D0,5.0D0,7.0D0,
     +		5.0D0,3.0D0,1.0D0,7.0D0,3.0D0,
     +		3.0D0,5.0D0,7.0D0,1.0D0,1.0D0,
     +		3.0D0,5.0D0,1.0D0,3.0D0,5.0D0,
     +		3.0D0,3.0D0,1.0D0,3.0D0,5.0D0,
     +		7.0D0,9.0D0,3.0D0,3.0D0,5.0D0,
     +		7.0D0,3.0D0,3.0D0,5.0D0,7.0D0,
     +		1.0D0,3.0D0,5.0D0,5.0D0,1.0D0,
     +		3.0D0,5.0D0,7.0D0,5.0D0,5.0D0,
     +		7.0D0,9.0D0,5.0D0,5.0D0,3.0D0,
     +		1.0D0,3.0D0,5.0D0,7.0D0,5.0D0,
     +		3.0D0,1.0D0,7.0D0,3.0D0,1.0D0,
     +		3.0D0,5.0D0,3.0D0,3.0D0,5.0D0,
     +		7.0D0,3.0D0,5.0D0,7.0D0,9.0D0,
     +		11.0D0,1.0D0,3.0D0,5.0D0,7.0D0,
     +		9.0D0,7.0D0,5.0D0,3.0D0,5.0D0,
     +		3.0D0,1.0D0,5.0D0,7.0D0,9.0D0,
     +		5.0D0,7.0D0,9.0D0,5.0D0,3.0D0,
     +		5.0D0,7.0D0,7.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,5.0D0,3.0D0,5.0D0,
     +		7.0D0,7.0D0,7.0D0,5.0D0,3.0D0,
     +		5.0D0,7.0D0,3.0D0,3.0D0,1.0D0,
     +		3.0D0,5.0D0,7.0D0,9.0D0,3.0D0,
     +		5.0D0,7.0D0,3.0D0,5.0D0,7.0D0,
     +		7.0D0,5.0D0,3.0D0,5.0D0,7.0D0,
     +		9.0D0,3.0D0,5.0D0,7.0D0,1.0D0,
     +		3.0D0,5.0D0,3.0D0/
	DATA ENOIII/0.0D0,1.4059945D-02,3.8038719D-02,2.513308,5.354124,
     +		7.477820,14.88140,14.88477,14.88550,17.65325,
     +		17.65339,17.65514,23.19140,24.43587,26.09378,
     +		33.13600,33.15068,33.18253,33.85794,35.18196,
     +		35.20895,35.22094,36.07438,36.43500,36.45190,
     +		36.47919,36.89279,36.98353,37.22392,37.23410,
     +		37.25028,38.01204,38.90675,40.22861,40.25288,
     +		40.27497,40.26230,40.57149,40.57759,40.58673,
     +		40.84922,40.86335,40.87098,41.14086,41.25951,
     +		41.97723,41.99266,42.14902,42.56451,43.39812,
     +		43.41013,43.43237,44.22956,44.24270,44.27655,
     +		44.46952,45.03978,45.31862,45.32294,45.33144,
     +		45.34384,45.35962,45.34443,45.43903,45.45230,
     +		45.47797,45.62070,45.69189,45.69899,45.71153,
     +		45.91510,45.92614,45.93959,45.98626,46.25228,
     +		46.44183,45.21283,46.46955,46.62690,46.78899,
     +		46.78899,46.78899,46.82767,46.91713,46.91867,
     +		46.92080,47.01923,47.02679,47.03461,47.20199,
     +		47.20199,47.20199,47.21141,47.24910,48.62968,
     +		48.62968,48.62968,48.69874,48.86141,48.86587,
     +		48.87442,48.91428,48.91908,48.92621,48.93560,
     +		48.94701,49.36293,49.36248,49.36198,49.36323,
     +		49.37332,49.40500,49.41368,49.41845,49.63815,
     +		49.65178,49.65844,49.76514,49.77709,49.79367,
     +		49.78386,49.78386,49.78386,49.81572,49.78386,
     +		49.78386,49.78386,50.01249,50.03133,50.31391,
     +		50.31750,50.32357,51.41365,51.47638,51.47638,
     +		51.47638,52.44297,52.69355,52.85969,53.12613,
     +		53.14089,53.16110,53.31682,54.18348,54.33549,
     +		54.33549,54.34320,54.35460,54.36977,54.46407,
     +		54.47044,54.48261,54.88958,54.88958,54.88958,
     +		55.81414,55.82281,55.82951,56.14741,56.14741,
     +		56.14741,56.31095,56.31095,56.31095,56.73994,
     +		56.73994,56.73994,58.73808/
	DATA NOIV/2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,
     +		3,3,3,3,3,3,3,3,3,3,3,4,3,3,3,3,3,3,3,3,3,3,3,3,
     +		3,3,4,4,3,3,3,3,3,3,5,3,3,3,3,5,5,5,5,3,4,4,4,3,
     +		3,4,4,6,6,4,4,3,3,3,3,3,3,3,4,4,7,7,4,4,4,4,4,4,
     +		4,4,4,4,4,4,4,4,4,4,3,8,8,4,4,3,3,3,3,3,3,3,3,3,
     +		3,3,3,3,3,5,5,3,3,5,5,3,3,5,5,5,5,5,5,5,5,5,5,5,
     +		3,3,3,3,3,3,3,3,3,6,6,6,6,4,4,3,4,4,7,7,7,7/
	DATA GOIV/2.0D0,4.0D0,2.0D0,4.0D0,6.0D0,
     +		6.0D0,4.0D0,2.0D0,2.0D0,6.0D0,
     +		4.0D0,6.0D0,4.0D0,2.0D0,4.0D0,
     +		2.0D0,2.0D0,4.0D0,2.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,2.0D0,4.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,4.0D0,
     +		2.0D0,4.0D0,6.0D0,4.0D0,6.0D0,
     +		2.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		10.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		4.0D0,6.0D0,6.0D0,4.0D0,2.0D0,
     +		4.0D0,6.0D0,6.0D0,8.0D0,4.0D0,
     +		2.0D0,2.0D0,4.0D0,2.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,4.0D0,6.0D0,
     +		6.0D0,8.0D0,2.0D0,2.0D0,4.0D0,
     +		6.0D0,6.0D0,8.0D0,2.0D0,4.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,4.0D0,
     +		6.0D0,2.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,6.0D0,8.0D0,
     +		2.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		6.0D0,4.0D0,2.0D0,4.0D0,6.0D0,
     +		6.0D0,8.0D0,4.0D0,6.0D0,6.0D0,
     +		8.0D0,2.0D0,6.0D0,8.0D0,4.0D0,
     +		2.0D0,4.0D0,6.0D0,2.0D0,4.0D0,
     +		6.0D0,8.0D0,2.0D0,4.0D0,6.0D0,
     +		6.0D0,4.0D0,4.0D0,6.0D0,8.0D0,
     +		2.0D0,4.0D0,6.0D0,8.0D0,4.0D0,
     +		6.0D0,6.0D0,4.0D0,2.0D0,4.0D0,
     +		6.0D0,8.0D0,6.0D0,4.0D0,2.0D0,
     +		6.0D0,8.0D0,4.0D0,2.0D0,6.0D0,
     +		4.0D0,2.0D0,4.0D0,6.0D0,6.0D0,
     +		8.0D0,4.0D0,2.0D0,2.0D0,4.0D0,
     +		6.0D0,8.0D0,4.0D0,6.0D0,2.0D0,
     +		4.0D0,6.0D0,2.0D0,4.0D0,6.0D0,8.0D0/
	DATA ENOIV/0.0D0,4.7920357D-02,8.824909,8.841201,8.864076,
     +		15.73825,15.73998,20.37910,22.37705,22.40721,
     +		28.67474,31.63571,31.63934,35.83378,35.83476,
     +		44.33902,48.37428,48.38508,54.37857,54.39532,
     +		54.42593,56.14158,56.17444,57.92984,57.94415,
     +		58.03452,58.04428,58.06108,58.08709,58.79609,
     +		59.33789,59.34961,59.36561,59.84372,59.87542,
     +		60.23497,61.10992,61.36131,61.37108,61.38501,
     +		61.40412,61.93150,61.93509,61.94088,61.94888,
     +		62.18008,62.18691,62.46812,62.48219,62.49133,
     +		63.30199,63.30286,63.32506,63.35387,63.75540,
     +		63.77412,64.30924,64.30999,66.87376,67.85857,
     +		67.86167,68.16618,68.17400,68.44416,68.44416,
     +		68.50069,68.50069,68.74507,70.50282,70.51955,
     +		70.55017,70.76975,70.76975,71.12993,71.15609,
     +		71.21387,71.21387,71.31690,71.33785,71.39315,
     +		71.39737,71.48887,71.50672,71.53300,72.12492,
     +		72.12764,72.47591,72.50269,72.88482,72.88482,
     +		73.16019,73.37047,73.37047,73.37047,73.37047,
     +		73.52322,73.52322,73.52322,73.60108,73.61112,
     +		73.64819,73.65725,73.68911,73.71453,73.93237,
     +		73.95444,74.05078,74.06293,74.06293,74.10930,
     +		74.12628,74.40265,74.40438,74.76035,74.76035,
     +		74.76035,74.76035,75.18896,75.18896,75.18896,
     +		76.30446,76.30806,76.44791,77.47625,77.47625,
     +		77.92433,77.92433,78.12258,78.12258,78.19797,
     +		78.21979,78.41159,78.43242,78.59385,78.59385,
     +		78.59385,78.59385,78.63718,78.63718,78.63718,
     +		78.85769,78.88398,78.91572,78.91572,78.96023,
     +		78.97250,78.98019,80.20107,80.20107,80.72665,
     +		80.72900,81.00314,81.01343,81.37509,81.37509,
     +		81.37509,81.37509,81.42716,81.42716,81.83012,
     +		82.88895,82.88895,83.03365,83.03365,83.03365,83.03365/
	DATA NOV/2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
     +		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,
     +		4,4,4,4,4,4,5,5,5,5,5,5,4,4,4,4,4,4,4,4,4,4,4,6,
     +		6,6,6,6,4,4,4,6,4,4,4,4,4,7,7,7,7,7,8,8,8,8,5,5,
     +		5,5,5,5,5,5,5,5,5,5,5,6,6,6,6,6,6,6,6/
	DATA GOV/1.0D0,1.0D0,3.0D0,5.0D0,3.0D0,
     +		1.0D0,3.0D0,5.0D0,5.0D0,1.0D0,
     +		3.0D0,1.0D0,3.0D0,1.0D0,3.0D0,
     +		5.0D0,3.0D0,5.0D0,7.0D0,5.0D0,
     +		1.0D0,3.0D0,5.0D0,3.0D0,3.0D0,
     +		3.0D0,5.0D0,7.0D0,3.0D0,1.0D0,
     +		3.0D0,5.0D0,5.0D0,5.0D0,3.0D0,
     +		5.0D0,7.0D0,1.0D0,5.0D0,3.0D0,
     +		1.0D0,7.0D0,3.0D0,3.0D0,1.0D0,
     +		1.0D0,3.0D0,5.0D0,3.0D0,3.0D0,
     +		5.0D0,7.0D0,5.0D0,7.0D0,3.0D0,
     +		3.0D0,3.0D0,5.0D0,7.0D0,5.0D0,
     +		3.0D0,3.0D0,3.0D0,5.0D0,7.0D0,
     +		3.0D0,1.0D0,3.0D0,5.0D0,5.0D0,
     +		5.0D0,3.0D0,7.0D0,3.0D0,5.0D0,
     +		7.0D0,3.0D0,5.0D0,7.0D0,5.0D0,
     +		5.0D0,3.0D0,1.0D0,7.0D0,3.0D0,
     +		3.0D0,3.0D0,5.0D0,7.0D0,5.0D0,
     +		3.0D0,3.0D0,5.0D0,7.0D0,3.0D0,
     +		3.0D0,5.0D0,7.0D0,1.0D0,3.0D0,
     +		5.0D0,5.0D0,5.0D0,3.0D0,5.0D0,
     +		7.0D0,7.0D0,3.0D0,3.0D0,5.0D0,
     +		7.0D0,1.0D0,3.0D0,5.0D0,5.0D0/
	DATA ENOV/0.0D0,10.18183,10.19878,10.23674,19.68863,
     +		26.48845,26.50776,26.54108,28.73015,35.69651,
     +		67.83862,69.59028,72.01395,72.28146,72.28596,
     +		72.29554,74.50599,74.50733,74.50979,75.95557,
     +		80.97483,80.99497,81.03748,82.38657,83.40436,
     +		83.97941,84.00407,84.04314,84.82139,85.49855,
     +		85.51269,85.53633,86.12596,86.43890,87.33036,
     +		87.33829,87.35107,87.73579,87.80076,87.81837,
     +		87.82866,88.39750,89.17985,89.60004,90.71603,
     +		91.26665,91.26665,91.26888,91.48672,92.04689,
     +		92.04763,92.04937,92.59937,92.97132,98.72523,
     +		99.49233,106.7049,106.7049,106.7049,100.2237,
     +		102.1987,102.8568,103.0377,103.0583,103.0944,
     +		103.1870,103.5465,103.5465,103.5676,103.8792,
     +		103.8829,104.1001,104.2509,104.2990,104.2990,
     +		104.2990,104.3064,104.3181,104.3333,104.4087,
     +		104.5556,104.5689,104.5754,105.0316,105.0733,
     +		106.7358,106.9963,106.9963,106.9963,106.9274,
     +		108.4187,108.5325,108.5325,108.5325,111.4108,
     +		111.5461,111.5461,111.5461,111.7535,111.7535,
     +		111.7535,111.8898,111.9082,112.1444,112.1444,
     +		112.1444,112.3809,115.9379,116.0435,116.0435,
     +		116.0435,116.1501,116.1501,116.1501,116.2166/
	DATA NOVI/2,2,2,3,3,3,3,3,4,4,4,4,4,4,4,5,5,5,5,5,6,6,6,6,
     +		6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,8,8/
	DATA GOVI/2.0D0,2.0D0,4.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,2.0D0,2.0D0,
     +		4.0D0,4.0D0,6.0D0,6.0D0,8.0D0,
     +		2.0D0,2.0D0,4.0D0,4.0D0,6.0D0,
     +		2.0D0,4.0D0,4.0D0,6.0D0,6.0D0,
     +		8.0D0,8.0D0,10.0D0,12.0D0,2.0D0,
     +		2.0D0,4.0D0,4.0D0,6.0D0,6.0D0,
     +		8.0D0,8.0D0,10.0D0,12.0D0,14.0D0,
     +		2.0D0,2.0D0,4.0D0,6.0D0,8.0D0,
     +		8.0D0,10.0D0,12.0D0,14.0D0,16.0D0,4.0D0,6.0D0/
	DATA ENOVI/0.0D0,11.94909,12.01505,79.35559,82.58831,
     +		82.60773,83.64374,83.65008,105.7219,107.0408,
     +		107.0487,107.4805,107.4831,107.5050,107.5062,
     +		117.6237,118.2920,118.2920,118.5122,118.5122,
     +		124.3735,124.3735,124.5034,124.5034,124.5142,
     +		124.5142,124.5156,124.5156,124.5156,127.8017,
     +		128.0311,128.0311,128.1171,128.1171,128.1243,
     +		128.1243,128.1252,128.1252,128.1252,128.1252,
     +		130.2520,130.3984,130.3984,130.4674,130.4674,
     +		130.4680,130.4680,130.4680,130.4680,130.4680,
     +		130.4693,130.4693/
	DATA NOVII/1,2,2,2,2,2,3,3,3,3,3,3,3,4,5,6/
	DATA GOVII/1.0D0,3.0D0,1.0D0,3.0D0,5.0D0,
     +		3.0D0,1.0D0,3.0D0,5.0D0,7.0D0,
     +		5.0D0,3.0D0,3.0D0,3.0D0,3.0D0,3.0D0/
	DATA ENOVII/0.0D0,561.0761,568.6182,568.6255,568.6938,
     +		573.9532,664.1129,664.1129,664.1129,665.1804,
     +		665.1804,665.1804,665.6218,697.8022,712.7239,720.8449/
	DATA SCI/4.179704,4.179868,4.180140,4.284864,4.411317, 
     +		4.556712,4.417538,4.418036,4.419087,4.460873,
     +		5.012059,5.012145,5.012123,4.656621,4.682271,
     +		4.682931,4.683973,4.715525,4.735114,4.735517, 
     +		4.736181,4.776610,4.823287,5.245868,4.960446, 
     +		4.636463,4.637096,4.638772,4.981131,4.981795, 
     +		4.983181,4.985224,4.985476,4.985839,4.648973, 
     +		4.987257,5.002644,5.026932,5.027268,5.027448, 
     +		4.751991,4.753114,4.754886,4.775015,4.807735, 
     +		4.820394,4.821310,4.822433,4.849118,4.880081, 
     +		4.964531,4.983084,4.983084,4.983084,4.988046, 
     +		4.988550,4.989272,4.739796,4.996660,5.002712, 
     +		5.007890,5.009317,5.009317,4.830776,4.830763, 
     +		4.830763,4.843919,4.885500,4.908801,4.963564, 
     +		4.983777,4.984663,4.984663,4.989659,4.989659, 
     +		4.992449,4.793379,4.998577,5.003253,5.004741, 
     +		5.006231,5.006231,4.972326,4.984214,4.985345, 
     +		4.985345,4.991673,4.991673,4.995098,4.831870, 
     +		5.000666,5.004451,5.005935,5.005935,5.004665, 
     +		4.984616,4.985763,4.985763,4.999064,4.999064, 
     +		4.999064,5.002865,5.004758,5.006233,5.006233, 
     +		5.006233,4.984145,4.984145,4.984432,5.002990, 
     +		5.002990,5.002990,5.005628,5.006508,5.006508, 
     +		5.006508,4.983365,4.983365,4.983365,5.006886, 
     +		5.006886,5.006886,5.008002,5.012074,5.012074, 
     +		5.012074,5.014099,5.014099,5.014099,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000/ 
	DATA SCII/3.322208,3.322644,3.633090,3.633257,3.633480,
     +		3.893420,3.893440,4.089184,4.229172,4.229597, 
     +		3.436720,3.692591,3.692789,4.589103,3.953149, 
     +		3.953178,4.702748,4.702820,3.603431,3.770060, 
     +		3.770254,4.440431,4.441056,4.442240,3.961645, 
     +		3.961659,4.991754,4.992091,3.992479,3.992479, 
     +		3.697579,3.795690,3.796068,4.770876,4.780956, 
     +		3.968260,3.968260,3.994325,3.994325,3.754887, 
     +		4.893884,4.894430,4.895358,4.896706,4.906213, 
     +		4.906944,3.971236,3.971236,3.996578,3.996578, 
     +		5.011171,5.086056,5.086787,5.087796,5.188515, 
     +		5.190205,5.591661,5.735427,5.737656,5.740739, 
     +		5.745144,5.937513,5.941305,5.947732,5.956571, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000,6.000000,6.000000,6.000000, 
     +		6.000000,6.000000/
	DATA SCIII/2.247678,2.511178,2.511298,2.511595,2.783334,
     +		2.988424,2.988601,2.988887,3.040348,3.275480,
     +		2.516355,2.624130,2.770250,2.779440,2.779510,
     +		2.779673,2.913505,2.912204,2.913576,3.001503,
     +		3.471912,3.472452,3.473566,2.656192,3.501991,
     +		2.707107,2.843331,2.843331,2.843447,3.667003,
     +		2.928022,2.928408,2.928967,2.942070,2.942230,
     +		2.942442,2.952919,2.960061,3.725865,3.726323,
     +		3.727023,2.996535,3.802981,3.848412,3.848814,
     +		3.849514,3.907525,3.915897,3.920174,3.920667,
     +		3.921389,4.006182,3.997117,4.006877,2.756046,
     +		4.057183,4.057739,4.058045,4.085242,2.876864,
     +		2.910818,2.910818,2.910818,4.166811,2.957773,
     +		2.957773,2.957773,2.998549,2.998549,2.998583,
     +		2.998549,3.003525,3.005696,3.020441,3.020510,
     +		3.020601,3.088544,2.797248,2.916939,2.968366,
     +		2.968366,2.968366,3.000603,3.000603,3.000640,
     +		3.003372,3.005378,3.009464,3.009464,3.009464,
     +		3.027199,2.830505,2.926040,2.976524,2.976524,
     +		2.976524,3.009360,2.932986,2.982088,2.982088,
     +		2.982088,2.986294,2.986294,2.986294,4.828427,
     +		4.828427,4.828427,5.151012,5.224114,5.224114,
     +		5.227788,5.497263,5.497263,5.502663,5.756044,
     +		5.817122,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000,6.000000,6.000000,6.000000,6.000000,
     +		6.000000/
	DATA SCIV/1.644934,1.923884,1.924364,1.778341,1.948965,
     +		1.949284,1.998202,1.998312,1.838941,1.962835,
     +		1.963075,1.999589,1.999669,2.001418,2.001418,
     +		1.874532,1.972046,1.972243,2.001394,2.001394,
     +		2.002842,2.002845,2.003133,2.003133,1.897882,
     +		1.978616,1.978616,2.003384,2.003384,2.004821,
     +		2.004821,2.005002,2.005002,2.005023,2.005023,
     +		1.913888,1.984033,1.984033,2.005113,2.005113,
     +		2.007061,2.007061,2.007217,2.007217,2.007234,
     +		2.007234,1.989961,1.989961,2.009654,2.009654,
     +		2.009800,2.009800,2.009800,2.009800,2.009800/
	DATASCV/0.6309066,0.7688928,0.9242349,0.9241881,0.9246764,
     +		1.026124,1.003322,1.003322,1.003322,1.020118,
     +		1.021842,1.027042,1.033988,1.051912,1.003001/
	DATA SNI/4.931870,5.108953,5.109031,5.204087,5.204087,
     +		5.329969,5.330800,5.331948,5.401419,5.403777,
     +		5.968683,5.969459,5.969806,5.605717,5.641185,
     +		5.641868,5.642995,5.644538,5.662621,5.663186,
     +		5.664362,5.702335,5.703489,5.705897,5.734946,
     +		5.736104,5.797976,5.797737,5.588642,5.591228,
     +		5.594790,5.615995,5.620492,5.980873,5.982464,
     +		5.982872,5.983638,5.985012,5.986994,5.988774,
     +		5.991692,5.990931,5.991989,5.990646,5.995945,
     +		5.996395,5.996926,5.997295,6.001428,6.002394,
     +		5.745163,5.761658,5.762813,5.764937,5.768167,
     +		5.774817,5.775745,5.778173,5.802792,5.696104,
     +		5.699982,5.706142,5.715098,5.722151,5.983984,
     +		5.985277,5.987724,5.991767,5.985780,5.989671,
     +		5.990610,5.994303,5.985830,5.987479,5.992273,
     +		5.996771,5.993288,5.995173,5.998955,6.002261,
     +		6.003886,6.255743,6.257061,6.360914,6.362586,
     +		5.757127,5.763044,5.772635,5.779661,5.791555,
     +		5.984846,5.986194,5.990250,5.997386,5.990476,
     +		5.992170,5.992623,6.000597,5.993189,5.993189,
     +		5.993189,6.000802,5.996590,5.998751,6.003087,
     +		6.005032,6.007153,5.793718,5.804320,5.818227,
     +		5.815205,5.821445,5.989322,5.989322,5.989322,
     +		5.989322,5.992901,5.992901,5.992901,6.003714,
     +		5.994695,5.997311,5.995185,6.008173,6.001116,
     +		6.010741,6.005529,6.005529,6.008008,5.801159,
     +		5.821038,5.833978,5.835981,5.835981,5.989852,
     +		5.989852,5.989852,5.989852,5.993398,5.993398,
     +		5.996287,5.996287,6.005342,6.014956,6.015613,
     +		6.015613,6.015613,5.971639,5.971639,5.850568,
     +		5.850568,5.850568,5.990061,5.990061,5.990061,
     +		5.990061,5.991796,5.991796,5.993244,5.993244,
     +		6.011374,6.018783,6.017594,6.017594,6.017594,
     +		5.858175,5.858175,5.863377,5.863377,5.863377,
     +		5.988659,5.988659,5.988659,5.988659,5.989390,
     +		5.989390,5.994151,5.994151,6.020563,6.027375,
     +		6.026996,6.026996,6.026996,5.866343,5.866343,
     +		5.874646,5.874646,5.874646,5.990859,5.990859,
     +		5.992667,5.992667,5.994933,5.994933,5.994933,
     +		5.994933,6.030020,6.030020,6.038991,6.038991,
     +		6.038991,5.873279,5.873279,5.877365,5.877365,
     +		5.877365,5.992039,5.992039,5.996431,5.996431,
     +		6.000838,6.000838,6.000838,6.000838,6.039686,
     +		6.039686,6.042562,6.042562,6.042562,6.018730,
     +		5.886326,5.994597,5.994597,6.047575,6.047575,
     +		6.047575,6.078406,6.078406/
	DATA SNII/4.048939,4.049242,4.049750,4.145151,4.258360,
     +		4.356432,4.688145,4.688257,4.688270,4.826217,
     +		4.826217,4.826272,5.142618,4.284333,4.284811,
     +		4.286872,4.288557,5.253337,4.532960,4.564950,
     +		4.565974,4.567596,5.379367,4.605227,4.634192,
     +		4.634804,4.635817,4.698180,4.771750,4.928997,
     +		4.930175,4.931792,4.940517,4.947424,4.947905,
     +		4.948512,4.976005,4.977055,4.977624,4.985716,
     +		5.001774,4.517682,4.519204,4.522714,5.167543,
     +		4.688999,4.706267,4.707887,4.710950,4.720615,
     +		4.721386,4.723658,4.732426,4.791679,5.359476,
     +		5.360877,5.362645,4.824183,4.939473,4.941747,
     +		4.944790,4.948400,4.959554,4.960499,4.962098,
     +		4.976268,4.977930,4.978850,4.988034,4.988136,
     +		4.988983,4.989127,4.990716,4.997378,4.997656,
     +		5.001124,4.999108,5.001566,5.001872,5.002052,
     +		5.004650,5.004791,5.510713,5.511550,5.512880,
     +		4.633615,4.635821,4.641956,4.663452,4.970949,
     +		4.970949,4.970949,4.990886,4.990886,4.991119,
     +		4.994713,4.999842,5.000126,5.004091,5.004655,
     +		5.899771,5.900361,5.901458,5.903069,5.905087,
     +		5.975470,5.976437,5.978201,6.162114,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000/
	DATA SNIII/3.264886,3.265738,3.559230,3.555733,3.556163,
     +		3.795859,3.795903,3.971288,4.062202,4.062887,
     +		4.328298,4.441761,4.441879,3.362787,4.644640,
     +		4.644671,3.648880,3.649321,3.924340,3.924418,
     +		4.208216,4.209135,4.210838,4.353292,4.355043,
     +		3.749472,4.546074,4.546964,4.554955,4.555551,
     +		4.556746,4.558211,3.785649,3.786211,4.632736,
     +		4.686664,4.687436,4.688480,3.926233,3.926969,
     +		3.987034,3.987034,4.752837,4.754452,4.866728,
     +		4.928826,4.929522,4.930549,4.931964,4.980142,
     +		4.980413,4.980860,4.981436,3.664742,5.015918,
     +		5.016470,5.050785,5.051935,5.052674,5.126587,
     +		5.129026,3.959079,3.959144,5.192320,5.193925,
     +		3.989434,3.989434,4.005147,4.005147,3.968565,
     +		3.968565,3.992409,3.992409,4.006539,4.006539,
     +		5.571515,5.574721,5.580696,6.132490,6.134099,
     +		5.935632,5.939608,6.083616,6.087341,6.092149,
     +		6.099411,6.364476,6.365573,6.178216,6.185985,
     +		6.229222,6.316157,6.320952,6.326658,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000,7.000000,7.000000,7.000000,
     +		7.000000,7.000000,7.000000/
	DATA SNIV/2.226836,2.490623,2.490878,2.491461,2.755431,
     +		2.952339,2.952669,2.953231,3.013266,3.231892,
     +		2.493613,2.749589,2.762857,2.763010,2.763353,
     +		2.901417,2.901452,2.901533,2.994477,3.382731,
     +		3.383611,3.385459,3.472422,3.564919,3.607152,
     +		3.607152,3.608737,3.645438,3.728391,3.728391,
     +		2.639493,3.779903,3.797702,3.799535,3.799535,
     +		3.799535,2.797721,2.797721,2.797721,3.872625,
     +		3.873032,3.873596,3.883205,2.857107,2.934635,
     +		2.934635,2.934635,3.951730,3.952443,3.952443,
     +		2.993446,3.029914,3.030044,3.030246,4.061023,
     +		3.127314,2.880353,2.950477,2.950477,2.950477,
     +		2.998267,2.998267,2.998267,2.959708,2.959708,
     +		2.959708,4.785085,4.873190,4.873190,4.874527,
     +		7.000000,7.000000,7.000000/
	DATA SNV/1.634565,1.915400,1.916327,1.771111,1.943796,
     +		1.944398,1.997854,1.998051,1.833396,1.959357,
     +		1.959357,1.999384,1.999384,1.870467,1.969524,
     +		1.969524,2.001982,2.001982,1.895972,1.977561,
     +		1.977561,2.004889,2.004593,2.005843,2.005843,
     +		2.006106,2.006106,2.006106,1.914798,1.983841,
     +		1.983841,2.007186,2.007186,2.008574,2.008574,
     +		2.008797,2.008797,2.008797,2.008797,1.929893,
     +		1.990742,1.990742,2.010469,2.010469,2.011696,
     +		2.011696,2.011930,2.011930,2.011930,2.011930,
     +		2.011930/
	DATA SNVI/0.6290283,0.7657156,0.9208641,0.9208946,0.9217644,
     +		0.9074407,1.024314,1.024867/
	DATA SOI/5.998809,6.000254,6.000874,6.149045,6.029965,
     +		6.280362,6.354168,6.620857,6.620917,6.621027,
     +		6.681873,6.681856,6.681878,6.554275,6.592577,
     +		6.991942,6.991948,6.991948,6.991953,6.991953,
     +		6.991953,6.994710,6.994710,6.994710,6.749977,
     +		6.750016,6.750087,6.784759,6.784759,6.784759,
     +		7.156593,7.157186,7.157570,6.676266,6.701954,
     +		7.234456,6.993919,6.993919,6.993919,6.993919,
     +		6.993919,6.997045,6.997045,6.997045,6.836974,
     +		6.836974,6.836974,6.746833,6.766088,6.996467,
     +		6.996467,6.996467,6.996467,6.996467,6.999115,
     +		6.999115,6.999115,6.869720,6.869720,6.869720,
     +		6.793480,6.808908,6.999084,6.999084,6.999084,
     +		6.999084,6.999084,7.001479,7.001479,7.001479,
     +		6.826997,6.839929,7.001803,7.001803,7.001803,
     +		7.001803,7.001803,7.003955,7.003955,7.003955,
     +		6.852827,6.864539,7.004705,8.000000,7.004705,
     +		7.004705,7.004705,7.007905,7.007905,7.007905,
     +		6.877349,6.883498,7.007761,7.007761,7.007761,
     +		7.007761,7.007761,7.010591,7.010591,7.010591,
     +		6.890946,6.900387,7.011453,7.011453,7.011453,
     +		7.011453,7.011453,7.012788,7.012788,7.012788,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000/
	DATA SOII/4.784610,4.940430,4.940555,5.022952,5.022962,
     +		5.557054,5.558274,5.558889,5.930025,5.929834,
     +		5.160761,5.162283,5.164577,5.214052,5.216705,
     +		6.210938,5.445367,5.490556,5.491464,5.492962,
     +		5.495002,5.494491,5.494507,5.517109,5.517870,
     +		5.519391,5.570158,5.573380,5.580988,6.392210,
     +		6.394130,5.615287,5.616316,5.844496,5.880436,
     +		5.880893,5.904250,5.904674,5.930839,5.931911,
     +		5.933457,5.935489,5.954107,5.955576,5.956912,
     +		5.954453,5.955388,5.959104,5.959794,5.959825,
     +		5.959952,5.960805,5.964087,5.973599,5.975908,
     +		5.993385,5.994448,5.442262,5.445265,5.449878,
     +		5.491284,5.496742,6.232405,5.654758,5.656267,
     +		5.658962,5.662895,5.725537,5.731195,5.738926,
     +		5.741796,6.348958,6.348958,6.373241,6.373322,
     +		6.408604,6.408635,6.419951,6.420006,6.439373,
     +		6.439435,5.943564,5.943564,5.943564,5.947441,
     +		5.961402,5.961402,5.961402,5.961402,6.471051,
     +		5.965786,5.967088,5.967867,5.968222,5.972136,
     +		5.972852,5.978758,6.488330,5.986873,5.991322,
     +		5.986938,5.991353,5.994984,5.994948,5.989273,
     +		5.989404,5.991879,5.997332,5.991945,5.997554,
     +		5.992443,5.994410,6.001196,6.001346,6.002642,
     +		6.003121,6.002740,6.003380,5.576062,5.580966,
     +		5.588796,5.609913,5.619139,6.121709,6.121740,
     +		5.734796,5.734796,5.738977,5.745944,5.750079,
     +		5.750079,5.754774,5.772264,5.781077,5.960446,
     +		5.960446,5.960446,5.960446,5.972282,5.974347,
     +		5.974347,5.980535,5.980535,5.983981,5.987715,
     +		5.990945,5.996456,5.985451,5.986923,5.990890,
     +		6.000214,5.991003,6.000511,5.996286,5.996286,
     +		6.000386,6.002267,6.003636,6.004436,6.003808,
     +		6.004436,6.864734,6.865004,6.865458,6.871479,
     +		6.874276,6.883227,6.883227,6.929313,6.929313,
     +		6.945119,6.945119,6.947771,6.947771,7.214550,
     +		7.214550,6.955942,6.960794,6.960794,6.963802,
     +		6.963802,6.974759,6.974759,6.897856,6.897856,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000/
	DATA SOIII/3.980091,3.980605,3.981483,4.073126,4.181011,
     +		4.263698,4.567496,2.851461,2.851508,4.688399,
     +		4.688406,4.688483,4.944256,5.004756,5.087306,
     +		4.201648,4.202927,4.205704,4.265077,5.589531,
     +		5.591178,5.591910,4.466920,4.500863,4.502461,
     +		4.505044,4.544430,5.702087,4.576288,4.577272,
     +		4.578837,4.653335,4.743010,4.880211,4.882788,
     +		4.885133,4.883788,4.916797,4.917453,4.918434,
     +		4.946753,4.948285,4.949112,4.978528,4.991553,
     +		5.071567,5.073311,5.091046,6.092469,5.236801,
     +		5.238239,5.240906,4.450987,4.453166,4.458786,
     +		4.490992,5.440957,5.477274,5.477840,5.478956,
     +		5.480584,5.482658,4.640882,4.657492,4.659830,
     +		4.664354,4.689624,5.526725,5.527675,5.529354,
     +		4.742366,4.744359,4.746791,4.755242,4.803841,
     +		5.629193,5.463434,5.633066,5.655169,4.904211,
     +		4.904211,4.904211,4.911571,5.696494,5.696715,
     +		5.697021,4.948280,4.949739,4.951245,4.983719,
     +		4.983719,4.983719,4.985557,4.992922,4.595488,
     +		4.595488,4.595488,4.614187,5.995187,5.995924,
     +		5.997336,6.003933,6.004729,6.005913,6.007472,
     +		6.009368,6.079757,6.079680,6.079593,6.079808,
     +		6.081549,6.087021,6.088523,6.089349,6.127790,
     +		6.130199,6.131379,6.150372,6.152512,6.155484,
     +		4.922875,4.922875,4.922875,4.932409,4.922875,
     +		4.922875,4.922875,4.991952,4.997716,6.251313,
     +		6.251994,6.253141,4.947119,4.974443,4.974443,
     +		4.974443,5.003925,6.782259,6.828280,6.541492,
     +		6.547457,6.555665,6.965416,7.060263,7.160808,
     +		7.160808,7.166230,7.174317,7.185196,7.256399,
     +		7.261456,7.271212,7.771374,7.771374,7.771374,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000/
	DATA SOIV/3.228562,3.230039,3.508826,3.509360,3.510109,
     +		3.741247,3.741307,3.904661,3.977057,3.978160,
     +		4.214302,4.331145,4.331291,4.503491,4.503533,
     +		3.322590,3.617383,3.618198,4.097019,4.098440,
     +		4.101037,4.249484,4.252384,4.410741,4.412061,
     +		4.420406,4.421309,4.422863,4.425270,4.491520,
     +		4.543003,4.544125,4.545658,4.591770,4.594849,
     +		3.506632,4.717019,4.742457,4.743450,4.744866,
     +		4.746809,4.800908,4.801279,4.801878,4.802707,
     +		4.826727,4.827440,4.856910,4.858391,4.859354,
     +		3.927959,3.928085,4.948471,4.951597,4.995503,
     +		4.997566,5.057139,5.057223,3.602068,5.487784,
     +		5.488193,5.528638,5.529685,3.943577,3.943577,
     +		3.956409,3.956409,5.607411,5.152442,5.155901,
     +		5.162243,5.906104,5.906104,5.285100,5.290775,
     +		3.955026,3.955026,5.325924,5.330538,6.007065,
     +		6.007765,6.023023,6.026014,6.030425,6.132527,
     +		6.133009,5.594399,5.600957,3.969004,3.969004,
     +		5.768017,5.824149,5.824149,5.824149,5.824149,
     +		5.865850,5.865850,5.865850,5.887424,5.890223,
     +		5.900586,5.903125,5.912084,5.919259,5.981793,
     +		5.988238,6.512458,4.040435,4.040435,6.034046,
     +		6.039135,6.592915,6.593322,6.679720,6.679720,
     +		6.679720,6.679720,6.791924,6.791924,6.791924,
     +		7.150804,7.152208,7.208681,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000,
     +		8.000000/
	DATA SOV/2.212299,2.477108,2.477559,2.478570,2.736373,
     +		2.929941,2.930501,2.931468,2.995396,3.204502,
     +		2.480139,2.586177,2.736415,2.753261,2.753545,
     +		2.754149,2.895501,2.895588,2.895747,2.990361,
     +		3.333698,3.335127,3.338143,3.434917,3.509306,
     +		3.551885,3.553720,3.556629,3.614975,3.666382,
     +		3.667461,3.669268,3.714561,3.738797,3.808601,
     +		3.809227,3.810236,3.840735,3.845908,3.847311,
     +		3.848131,3.893723,3.957266,2.655746,2.780047,
     +		2.842480,2.842480,2.842734,2.867645,2.932266,
     +		2.932352,2.932553,2.996815,3.040747,2.722722,
     +		2.858081,4.369750,4.369750,4.369750,2.990545,
     +		4.293693,4.399677,4.429359,4.432752,4.438707,
     +		4.454041,4.514207,4.514207,4.517767,4.570812,
     +		4.571450,2.913395,2.952782,2.965416,2.965416,
     +		2.965416,4.644915,4.646958,4.649636,2.994349,
     +		4.688903,4.691261,4.692408,4.774586,4.782194,
     +		2.928606,3.022013,3.022013,3.022013,2.997125,
     +		2.933282,2.986425,2.986425,2.986425,5.872365,
     +		5.931636,5.931636,5.931636,6.025977,6.025977,
     +		6.025977,6.090481,6.099398,6.217293,6.217293,
     +		6.217293,6.343698,8.000000,8.000000,8.000000,
     +		8.000000,8.000000,8.000000,8.000000,8.000000/
	DATA SOVI/1.626749,1.908750,1.910343,1.765577,1.939605,
     +		1.940666,1.997515,1.997865,1.829541,1.956604,
     +		1.957376,1.999562,1.999822,2.001964,2.002083,
     +		1.867336,1.968342,1.968342,2.001995,2.001995,
     +		1.976059,1.976059,2.004681,2.004681,2.007063,
     +		2.007063,2.007365,2.007365,2.007365,1.914098,
     +		1.982390,1.982390,2.008208,2.008208,2.010369,
     +		2.010369,2.010631,2.010631,2.010631,2.010631,
     +		1.930105,1.987143,1.987143,2.014184,2.014184,
     +		2.014431,2.014431,2.014431,2.014431,2.014431,
     +		2.014965,2.014965/
	DATA SOVII/0.6273875,0.7631111,0.9180546,0.9182081,0.9196253,
     +		1.029738,0.9543552,0.9543552,0.9543552,1.004676,
     +		1.004676,1.004676,1.025589,1.027917,1.034432,
     +		1.045346/
c
      U=0.
      DUT=0.
      DUN=0.
      IF(IAT.EQ.10) THEN
         IF(IZI.EQ.5) U=9.
         IF(IZI.EQ.6) U=6.
         IF(IZI.EQ.7) U=12.
         IF(IZI.EQ.8) U=8.
         IF(IZI.EQ.9) U=1.
         goto 8888
      END IF
c
c	Find index for atom and ion, 10*IAT+IZI
c
        IF(IAT.EQ.26.AND.IZI.GE.6.AND.IZI.LE.9) GO TO 260
	IF(IAT.GT.2.AND.IAT.LT.6) GO TO 9999
	IF(IAT.LT.1.OR.IAT.GT.8) GO TO 9999
	IND=10*IAT+IZI
	IF(IND.EQ.11) GO TO 11
	IF(IND.EQ.21) GO TO 21
	IF(IND.EQ.22) GO TO 22
	IF(IND.EQ.61) GO TO 61
	IF(IND.EQ.62) GO TO 62
	IF(IND.EQ.63) GO TO 63
	IF(IND.EQ.64) GO TO 64
	IF(IND.EQ.65) GO TO 65
	IF(IND.EQ.66) GO TO 66
	IF(IND.EQ.71) GO TO 71
	IF(IND.EQ.72) GO TO 72
	IF(IND.EQ.73) GO TO 73
	IF(IND.EQ.74) GO TO 74
	IF(IND.EQ.75) GO TO 75
	IF(IND.EQ.76) GO TO 76
	IF(IND.EQ.77) GO TO 77
	IF(IND.EQ.81) GO TO 81
	IF(IND.EQ.82) GO TO 82
	IF(IND.EQ.83) GO TO 83
	IF(IND.EQ.84) GO TO 84
	IF(IND.EQ.85) GO TO 85
	IF(IND.EQ.86) GO TO 86
	IF(IND.EQ.87) GO TO 87
	IF(IND.EQ.88) GO TO 88
* 
*	CALCULATING PARTITION FUNCTIONS FOR HYDROGEN
* 
 11	CALL PARTDV(T,ANE,ZH,MH,NHYD,GHYD,ENHYD,SHYD,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR HEI
* 
 21	CALL PARTDV(T,ANE,ZHE,MHEI,NHEL,GHEL,ENHEL,SHEL,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR HEII
* 
 22	CALL PARTDV(T,ANE,ZHE,MHEII,NHYD,GHYD,ENHYD,SHYD,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR CI
* 
 61	CALL PARTDV(T,ANE,ZC,MCI,NCI,GCI,ENCI,SCI,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR CII
* 
 62	CALL PARTDV(T,ANE,ZC,MCII,NCII,GCII,ENCII,SCII,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR CIII
* 
 63	CALL PARTDV(T,ANE,ZC,MCIII,NCIII,GCIII,ENCIII,SCIII,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR CIV
* 
 64	CALL PARTDV(T,ANE,ZC,MCIV,NCIV,GCIV,ENCIV,SCIV,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR CV
* 
 65	CALL PARTDV(T,ANE,ZC,MCV,NCV,GCV,ENCV,SCV,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR CVI
* 
 66	CALL PARTDV(T,ANE,ZC,MH,NHYD,GHYD,ENHYD,SHYD,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR NI
* 
 71	CALL PARTDV(T,ANE,ZN,MNI,NNI,GNI,ENNI,SNI,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR NII
* 
 72	CALL PARTDV(T,ANE,ZN,MNII,NNII,GNII,ENNII,SNII,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR NIII
* 
 73	CALL PARTDV(T,ANE,ZN,MNIII,NNIII,GNIII,ENNIII,SNIII,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR NIV
* 
 74	CALL PARTDV(T,ANE,ZN,MNIV,NNIV,GNIV,ENNIV,SNIV,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR NV
* 
 75	CALL PARTDV(T,ANE,ZN,MNV,NNV,GNV,ENNV,SNV,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR NVI
* 
 76	CALL PARTDV(T,ANE,ZN,MNVI,NNVI,GNVI,ENNVI,SNVI,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR NVII
* 
 77	CALL PARTDV(T,ANE,ZN,MH,NHYD,GHYD,ENHYD,SHYD,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR OI
* 
 81	CALL PARTDV(T,ANE,ZO,MOI,NOI,GOI,ENOI,SOI,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR OII
* 
 82	CALL PARTDV(T,ANE,ZO,MOII,NOII,GOII,ENOII,SOII,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR OIII
* 
 83	CALL PARTDV(T,ANE,ZO,MOIII,NOIII,GOIII,ENOIII,SOIII,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR OIV
* 
 84	CALL PARTDV(T,ANE,ZO,MOIV,NOIV,GOIV,ENOIV,SOIV,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR OV
* 
 85	CALL PARTDV(T,ANE,ZO,MOV,NOV,GOV,ENOV,SOV,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR OVI
* 
 86	CALL PARTDV(T,ANE,ZO,MOVI,NOVI,GOVI,ENOVI,SOVI,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR OVII
* 
 87	CALL PARTDV(T,ANE,ZO,MOVII,NOVII,GOVII,ENOVII,SOVII,U)
	GO TO 8888
* 
*	CALCULATING PARTITION FUNCTIONS FOR OVIII
* 
 88	CALL PARTDV(T,ANE,ZO,MH,NHYD,GHYD,ENHYD,SHYD,U)
	GO TO 8888
*
* 
*	CALCULATING PARTITION FUNCTIONS FOR FE VI - FE IX
* 
 260    CALL PFFE(IZI,T,ANE,U,DUN,DUT)
 8888	CONTINUE
	RETURN
 9999	U=0
	call stop(' INVALID ATOM IN USER SUPPLIED ROUTINE PFSPEC')
	STOP
        END
C
C      ************************************************************
C
C
C
	SUBROUTINE PARTDV(TEMP,DNE,Z,NLEV,NE,GEE,ENRGY,S,U)
C       ===================================================
C
C       routine called by PFSPEC for He, C, N, O

        include 'IMPLIC.FOR'
	DIMENSION GEE(*),ENRGY(*),S(*)
	INTEGER NE(*)
        parameter (cet=11604.8,zero=0.,tthrd=2./3.,un=1.,four=4.)
        parameter (cpt=14.49,emi=65.)
c
	U=zero
	ET=CET/TEMP
	P=(cpt-tthrd*LOG10(DNE))
c 
	DO 10 I=1,NLEV
	   U1=FLOAT(NE(I))
	   ZSTAR=Z-S(I)
	   IF (ZSTAR.GT.zero)THEN
	       W=P+four*(LOG10(ZSTAR)-LOG10(U1))
	    ELSE
	       W=zero
	   ENDIF
	   IF (W.GT.un) W=un
c
           X=ENRGY(I)*ET
	   IF(X.LT.EMI) THEN
		U1=GEE(I)*W*EXP(-X)
	    ELSE
		U1=zero
	   ENDIF
	   U=U+U1
 10	CONTINUE
 	RETURN
	END
C
C ********************************************************************
C
C
C
      subroutine pffe(ion,t,ane,pf,dut,dun)
c     =====================================
c
c     partition functions for Fe IV to Fe IX
c     after Fischel and Sparks, 1971, NASA SP-3066
c
c     Output:  PF   partition function
c              DUT  d(PF)/dT
c              DUN  d(PF)/d(ANE)
c
      include 'IMPLIC.FOR'
c
      dimension tt(50),pn(10),nca(9)
      dimension p4a(22),p4b(10,28), 
     *          p5a(30),p5b(10,20),
     *          p6a(37),p6b(10,13),
     *          p7a(40),p7b(10,10),
     *          p8a(41),p8b(10,9),
     *          p9a(45),p9b(10,5)
c
c
      data nca /3*0,22,30,37,40,41,45/
     *     nne /10/
c
      data tt /
     * 3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,15.,16.,17.,18.,19.,
     * 20.,21.,22.,23.,24.,25.,26.,27.,28.,29.,30.,
     * 32.,34.,36.,38.,40.,42.,44.,46.,48.,
     * 50.,55.,60.,65.,70.,75.,80.,85.,90.,95.,100.,125.,150./
c
      data pn /-2.,-1.,0.,1.,2.,3.,4.,5.,6.,7./
c
      data p4a /
     * 0.778, 0.778, 0.778, 0.779, 0.783, 0.789, 0.801, 0.818,
     * 0.842, 0.871, 0.906, 0.945, 0.987, 1.030, 1.074, 1.117,
     * 1.160, 1.201, 1.242, 1.280, 1.317, 1.353/
c
      data p4b /
     * 1.406,1.393,1.389,7*1.387,
     * 1.464,1.434,1.424,1.421,1.420,5*1.419,
     * 1.546,1.483,1.461,1.454,1.451,1.451,4*1.450,
     * 1.665,1.547,1.503,1.488,1.482,1.481,4*1.480,
     * 1.826,1.636,1.553,1.524,1.514,1.510,4*1.509,
     * 2.024,1.755,1.618,1.564,1.546,1.540,1.538,3*1.537,
     * 2.480,2.087,1.814,1.674,1.619,1.599,1.593,1.591,1.590,1.590,
     * 2.945,2.489,2.105,1.846,1.717,1.667,1.649,1.643,1.641,1.640,
     * 3.379,2.897,2.452,2.089,1.859,1.751,1.710,1.696,1.691,1.689,
     * 3.774,3.283,2.808,2.381,2.054,1.864,1.782,1.751,1.741,1.738,
     * 4.133,3.637,3.150,2.688,2.292,2.015,1.871,1.814,1.793,1.786,
     * 4.460,3.962,3.468,2.989,2.549,2.199,1.984,1.886,1.848,1.835,
     * 4.757,4.258,3.762,3.274,2.809,2.406,2.121,1.972,1.908,1.886,
     * 5.029,4.530,4.032,3.539,3.061,2.624,2.279,2.073,1.976,1.939,
     * 5.279,4.780,4.281,3.785,3.299,2.840,2.450,2.189,2.051,1.996,
     * 5.510,5.010,4.511,4.013,3.522,3.050,2.628,2.318,2.136,2.057,
     * 6.014,5.514,5.014,4.515,4.018,3.530,3.065,2.666,2.381,2.228,
     * 6.435,5.935,5.435,4.936,4.437,3.943,3.460,3.022,2.658,2.422,
     * 6.794,6.294,5.794,5.294,4.794,4.297,3.807,3.343,2.939,2.631,
     * 7.102,6.602,6.102,5.602,5.102,4.604,4.110,3.638,3.194,2.845,
     * 7.370,6.870,6.370,5.870,5.370,4.871,4.375,3.892,3.439,3.052,
     * 7.606,7.106,6.606,6.106,5.605,5.106,4.608,4.125,3.661,3.249,
     * 7.815,7.315,6.814,6.314,5.814,5.314,4.816,4.333,3.851,3.418,
     * 8.001,7.501,7.001,6.500,6.000,5.500,5.001,4.511,4.032,3.586,
     * 8.168,7.668,7.168,6.668,6.168,5.667,5.168,4.680,4.197,3.741,
     * 8.319,7.819,7.319,6.819,6.319,5.818,5.319,4.832,4.347,3.884,
     * 8.900,8.399,7.899,7.399,6.899,6.398,5.898,5.405,4.917,4.431,
     * 9.294,8.794,8.294,7.793,7.293,6.793,6.292,5.799,5.306,4.824/
c
      data p5a /
     * 1.235, 1.276, 1.301, 1.321, 1.339, 1.359, 1.381, 1.405,
     * 1.432, 1.460, 1.489, 1.518, 1.546, 1.574, 1.601, 1.627,
     * 1.652, 1.675, 1.697, 1.718, 1.738, 1.757, 1.775, 1.792,
     * 1.808, 1.823, 1.838, 1.851, 1.877, 1.900/
c
      data p5b /
     * 1.943,1.928,1.923,7*1.921,
     * 2.011,1.964,1.947,1.942,1.941,5*1.940,
     * 2.144,2.025,1.980,1.965,1.960,1.958,4*1.957,
     * 2.361,2.137,2.032,1.993,1.980,1.976,1.975,3*1.974,
     * 2.646,2.315,2.121,2.035,2.004,1.994,1.991,1.990,1.989,1.989,
     * 2.960,2.553,2.260,2.102,2.037,2.015,2.007,2.005,2.004,2.004,
     * 3.274,2.823,2.450,2.205,2.086,2.040,2.025,2.020,2.018,2.018,
     * 3.575,3.101,2.674,2.348,2.158,2.075,2.045,2.036,2.032,2.031,
     * 4.251,3.757,3.275,2.829,2.466,2.234,2.124,2.083,2.069,2.064,
     * 4.822,4.324,3.829,3.346,2.895,2.522,2.278,2.161,2.116,2.100,
     * 5.308,4.808,4.310,3.816,3.334,2.888,2.525,2.297,2.187,2.145,
     * 5.725,5.225,4.726,4.228,3.736,3.260,2.828,2.496,2.294,2.206,
     * 6.088,5.589,5.089,4.590,4.093,3.604,3.139,2.733,2.447,2.291,
     * 6.407,5.907,5.407,4.908,4.409,3.915,3.433,2.988,2.629,2.399,
     * 6.689,6.189,5.689,5.189,4.690,4.193,3.704,3.236,2.832,2.535,
     * 6.940,6.440,5.940,5.440,4.941,4.443,3.949,3.469,3.038,2.687,
     * 7.166,6.666,6.166,5.666,5.166,4.667,4.171,3.684,3.237,2.847,
     * 7.370,6.870,6.369,5.869,5.369,4.870,4.373,3.882,3.417,3.008,
     * 8.150,7.649,7.149,6.649,6.149,5.649,5.149,4.651,4.167,3.700,
     * 8.677,8.177,7.676,7.176,6.676,6.176,5.676,5.176,4.687,4.203/
c
      data p6a /
     * 1.218, 1.273, 1.309, 1.335, 1.358, 1.379, 1.400, 1.421,
     * 1.442, 1.463, 1.484, 1.504, 1.523, 1.542, 1.560, 1.577,
     * 1.594, 1.609, 1.624, 1.638, 1.652, 1.664, 1.677, 1.688,
     * 1.699, 1.709, 1.719, 1.729, 1.746, 1.762, 1.777, 1.790,
     * 1.803, 1.814, 1.825, 1.834, 1.843/
c
      data p6b /
     * 1.862,1.855,1.853,7*1.852,
     * 1.958,1.900,1.880,1.874,1.872,5*1.871,
     * 2.264,2.045,1.944,1.906,1.894,1.890,4*1.888,
     * 2.776,2.386,2.119,1.984,1.930,1.912,1.906,1.904,2*1.903,
     * 3.321,2.856,2.453,2.165,2.012,1.949,1.927,1.920,1.918,1.917,
     * 3.821,3.333,2.868,2.465,2.178,2.025,1.963,1.941,1.934,1.932,
     * 4.266,3.771,3.285,2.825,2.434,2.164,2.027,1.972,1.953,1.947,
     * 4.662,4.164,3.670,3.187,2.739,2.372,2.135,2.022,1.980,1.965,
     * 5.015,4.516,4.019,3.527,3.052,2.624,2.295,2.102,2.019,1.988,
     * 5.332,4.832,4.344,3.838,3.351,2.889,2.493,2.217,2.075,2.017,
     * 5.618,5.118,4.619,4.121,3.628,3.149,2.711,2.364,2.155,2.058,
     * 6.710,6.210,5.710,5.210,4.711,4.213,3.719,3.241,2.807,2.462,
     * 7.446,6.946,6.446,5.946,5.446,4.946,4.447,3.952,3.474,3.022/
c
      data p7a /
     * 1.074,1.130,1.167,1.194,1.215,1.234,1.250,1.266,1.280,1.293,
     * 1.306,1.318,1.329,1.340,1.350,1.360,1.369,1.378,1.386,1.394,
     * 1.401,1.408,1.415,1.421,1.427,1.433,1.439,1.444,1.454,1.463,
     * 1.471,1.479,1.486,1.492,1.498,1.504,1.509,1.514,1.525,1.534/
c
      data p7b /
     * 1.555,1.546,1.544,1.543,6*1.542,
     * 1.617,1.572,1.557,1.552,1.550,1.550,4*1.549,
     * 1.798,1.648,1.587,1.566,1.559,1.557,4*1.556,
     * 2.134,1.832,1.666,1.597,1.573,1.565,1.563,1.562,2*1.561,
     * 2.550,2.138,1.836,1.671,1.602,1.578,1.570,1.568,2*1.567,
     * 2.968,2.504,2.102,1.816,1.665,1.603,1.582,1.575,2*1.572,
     * 3.359,2.875,2.419,2.037,1.779,1.651,1.601,1.584,1.579,1.577,
     * 3.718,3.224,2.745,2.305,1.953,1.736,1.636,1.599,1.586,1.582,
     * 5.097,4.598,4.098,3.601,3.110,2.638,2.217,1.899,1.719,1.643,
     * 6.026,5.526,5.026,4.527,4.028,3.531,3.042,2.576,2.170,1.885/
c
      data p8a /
     * 0.809,0.849,0.875,0.894,0.908,0.918,0.927,0.934,0.939,0.944,
     * 0.948,0.952,0.955,0.958,0.960,0.962,0.964,0.966,0.967,0.969,
     * 0.970,0.971,0.973,0.974,0.975,0.975,0.976,0.977,0.978,0.980,
     * 0.981,0.982,0.983,0.984,0.984,0.985,0.986,0.986,0.987,0.988,
     * 0.989/
c
      data p8b /
     * 0.992,0.991,8*0.990,
     * 1.000,0.994,0.992,7*0.991,
     * 1.032,1.005,0.996,0.993,0.992,5*0.991,
     * 1.129,1.040,1.008,0.997,0.993,5*0.992,
     * 1.335,1.132,1.042,1.009,0.998,0.994,0.993,0.993,2*0.992,
     * 1.640,1.312,1.121,1.038,1.007,0.998,0.994,3*0.993,
     * 1.987,1.573,1.269,1.101,1.030,1.005,0.997,2*0.994,0.993,
     * 3.514,3.017,2.526,2.053,1.628,1.305,1.119,1.039,1.010,1.000,
     * 4.569,4.069,3.569,3.072,2.580,2.103,1.671,1.336,1.136,1.048/
c
      data p9a /39*0.000,0.001,0.002,0.005,0.008,0.014,0.021/
c
      data p9b /
     * 2*0.032,8*0.031,
     * 0.048,0.045,8*0.044,
     * 0.076,0.065,0.061,0.060,6*0.059,
     * 1.128,0.722,0.429,0.271,0.207,0.184,0.177,0.174,2*0.173,
     * 2.696,2.200,1.712,1.249,0.848,0.564,0.415,0.354,0.333,0.327/
c
      parameter (xen=2.302585093,xmil=0.001,xmilen=xmil*xen)
      parameter (xbtz=1.38054d-16)
c
      na=nca(ion)
      nb=50-na
      pne=log10(ane*xbtz*t)
      t0=xmil*t
      j=1
      if(pne.lt.pn(1)) go to 15
      if(pne.gt.pn(nne)) then
        j1=nne
        j2=nne
        goto 16
      endif
      do 10 j=1,nne-1      
        if(pne.ge.pn(j).and.pne.lt.pn(j+1)) go to 15
   10 continue
   15 j1=j
      j2=j1+1
      if(pne.lt.pn(1)) j2=1
   16 do 20 i=1,49
         if(t0.ge.tt(i).and.t0.lt.tt(i+1)) go to 25
   20 continue
   25 i1=i
      i2=i+1
      if(t0.gt.tt(50)) then
        i1=50
        i2=50
      endif
      if(i2.le.na) then
         if(ion.eq.4) then
           px1=p4a(i1)
           px2=p4a(i1)
           py1=p4a(i2)
           py2=p4a(i2)
         else if(ion.eq.5) then
           px1=p5a(i1)
           px2=p5a(i1)
           py1=p5a(i2)
           py2=p5a(i2)
         else if(ion.eq.6) then
           px1=p6a(i1)
           px2=p6a(i1)
           py1=p6a(i2)
           py2=p6a(i2)
         else if(ion.eq.7) then
           px1=p7a(i1)
           px2=p7a(i1)
           py1=p7a(i2)
           py2=p7a(i2)
         else if(ion.eq.8) then
           px1=p8a(i1)
           px2=p8a(i1)
           py1=p8a(i2)
           py2=p8a(i2)
         else if(ion.eq.9) then
           px1=p9a(i1)
           px2=p9a(i1)
           py1=p9a(i2)
           py2=p9a(i2)
         endif
       else if(i1.eq.na) then
         if(ion.eq.4) then
           px1=p4a(i1)
           px2=p4a(i1)
           py1=p4b(j1,i2-na)
           py2=p4b(j2,i2-na)
         else if(ion.eq.5) then
           px1=p5a(i1)
           px2=p5a(i1)
           py1=p5b(j1,i2-na)
           py2=p5b(j2,i2-na)
         else if(ion.eq.6) then
           px1=p6a(i1)
           px2=p6a(i1)
           py1=p6b(j1,i2-na)
           py2=p6b(j2,i2-na)
         else if(ion.eq.7) then
           px1=p7a(i1)
           px2=p7a(i1)
           py1=p7b(j1,i2-na)
           py2=p7b(j2,i2-na)
         else if(ion.eq.8) then
           px1=p8a(i1)
           px2=p8a(i1)
           py1=p8b(j1,i2-na)
           py2=p8b(j2,i2-na)
         else if(ion.eq.9) then
           px1=p9a(i1)
           px2=p9a(i1)
           py1=p9b(j1,i2-na)
           py2=p9b(j2,i2-na)
         endif
      else
         if(ion.eq.4) then
           px1=p4b(j1,i1-na)
           px2=p4b(j2,i1-na)
           py1=p4b(j1,i2-na)
           py2=p4b(j2,i2-na)
         else if(ion.eq.5) then
           px1=p5b(j1,i1-na)
           px2=p5b(j2,i1-na)
           py1=p5b(j1,i2-na)
           py2=p5b(j2,i2-na)
         else if(ion.eq.6) then
           px1=p6b(j1,i1-na)
           px2=p6b(j2,i1-na)
           py1=p6b(j1,i2-na)
           py2=p6b(j2,i2-na)
         else if(ion.eq.7) then
           px1=p7b(j1,i1-na)
           px2=p7b(j2,i1-na)
           py1=p7b(j1,i2-na)
           py2=p7b(j2,i2-na)
         else if(ion.eq.8) then
           px1=p8b(j1,i1-na)
           px2=p8b(j2,i1-na)
           py1=p8b(j1,i2-na)
           py2=p8b(j2,i2-na)
         else if(ion.eq.9) then
           px1=p9b(j1,i1-na)
           px2=p9b(j2,i1-na)
           py1=p9b(j1,i2-na)
           py2=p9b(j2,i2-na)
         endif
      end if
      dlgunx=px2-px1
      px=px1+(pne-pn(j1))*dlgunx
      dlguny=py2-py1
      py=py1+(pne-pn(j1))*dlguny
      delt=tt(i2)-tt(i1)
      if(delt.ne.0.) then
        dlgut=(py-px)/delt
        pf=px+(t0-tt(i1))*dlgut
        dlgun=dlgunx+(t0-tt(i1))/delt*(dlguny-dlgunx)
      else
        dlgut=0.
        pf=px
        dlgun=dlgunx
      endif
      pf=exp(xen*pf)
      dut=xmilen*pf*dlgut
      dun=(dlgun*pf-t*dut)/ane
      return
      end
C
C
C ********************************************************************
C
C
      subroutine pfni(ion,t,ane,pf,dut,dun)
c     =====================================
c
c     partition functions for Ni IV to Ni IX
c
c     this routine interpolates within a grid
c     calculated from all levels predicted by
c     Kurucz (1992), i.e. over 12,000 levels per ion.
c     the partition functions depend only on T !
c     (i.e. no level dissolution with increasing density)
c     TL  27-DEC-1994, 23-JAN-1995
c
c     Output:  PF   partition function
c              DUT  d(PF)/dT
c              DUN  d(PF)/d(ANE)
c
      include 'IMPLIC.FOR'
c
      dimension g0(6)
      dimension p4a(190),p4b(170)
      dimension p5a(190),p5b(170)
      dimension p6a(190),p6b(170)
      dimension p7a(190),p7b(170)
      dimension p8a(190),p8b(170)
      dimension p9a(190),p9b(170)
      parameter (xen=2.302585093,xmil=0.001)
c
      data g0/28.,25.,6.,25.,28.,21./
c
      data p4a/
     .    1.447,1.464,1.482,1.501,1.518,1.535,1.551,1.567,1.582,1.596,
     .    1.610,1.623,1.636,1.648,1.659,1.671,1.681,1.692,1.702,1.711,
     .    1.721,1.730,1.739,1.748,1.757,1.765,1.774,1.782,1.791,1.799,
     .    1.808,1.816,1.824,1.833,1.841,1.850,1.859,1.868,1.877,1.886,
     .    1.895,1.905,1.914,1.924,1.934,1.945,1.955,1.966,1.977,1.989,
     .    2.000,2.012,2.025,2.037,2.050,2.063,2.077,2.091,2.105,2.119,
     .    2.134,2.149,2.164,2.179,2.195,2.211,2.227,2.243,2.260,2.276,
     .    2.293,2.310,2.327,2.344,2.362,2.379,2.397,2.414,2.432,2.449,
     .    2.467,2.484,2.502,2.519,2.537,2.554,2.571,2.588,2.606,2.623,
     .    2.640,2.657,2.674,2.690,2.707,2.723,2.740,2.756,2.772,2.788,
     .    2.804,2.819,2.835,2.850,2.866,2.881,2.896,2.911,2.925,2.940,
     .    2.954,2.969,2.983,2.997,3.010,3.024,3.038,3.051,3.064,3.077,
     .    3.090,3.103,3.116,3.128,3.141,3.153,3.165,3.177,3.189,3.201,
     .    3.213,3.224,3.235,3.247,3.258,3.269,3.280,3.291,3.301,3.312,
     .    3.322,3.332,3.343,3.353,3.363,3.373,3.382,3.392,3.402,3.411,
     .    3.421,3.430,3.439,3.448,3.457,3.466,3.475,3.484,3.492,3.501,
     .    3.509,3.518,3.526,3.534,3.542,3.550,3.558,3.566,3.574,3.582,
     .    3.589,3.597,3.604,3.612,3.619,3.626,3.634,3.641,3.648,3.655,
     .    3.662,3.669,3.676,3.682,3.689,3.696,3.702,3.709,3.715,3.722/
      data p4b/
     .    3.589,3.597,3.604,3.612,3.619,3.626,3.634,3.641,3.648,3.655,
     .    3.662,3.669,3.676,3.682,3.689,3.696,3.702,3.709,3.715,3.722,
     .    3.728,3.734,3.740,3.747,3.753,3.759,3.765,3.771,3.777,3.782,
     .    3.788,3.794,3.800,3.805,3.811,3.816,3.822,3.827,3.833,3.838,
     .    3.843,3.849,3.854,3.859,3.864,3.869,3.874,3.879,3.884,3.889,
     .    3.894,3.899,3.904,3.909,3.913,3.918,3.923,3.927,3.932,3.936,
     .    3.941,3.945,3.950,3.954,3.959,3.963,3.967,3.972,3.976,3.980,
     .    3.984,3.988,3.993,3.997,4.001,4.005,4.009,4.013,4.017,4.021,
     .    4.024,4.028,4.032,4.036,4.040,4.043,4.047,4.051,4.055,4.058,
     .    4.062,4.065,4.069,4.072,4.076,4.079,4.083,4.086,4.090,4.093,
     .    4.097,4.100,4.103,4.107,4.110,4.113,4.116,4.120,4.123,4.126,
     .    4.129,4.132,4.135,4.138,4.141,4.144,4.148,4.151,4.154,4.157,
     .    4.159,4.162,4.165,4.168,4.171,4.174,4.177,4.180,4.182,4.185,
     .    4.188,4.191,4.193,4.196,4.199,4.202,4.204,4.207,4.210,4.212,
     .    4.215,4.217,4.220,4.223,4.225,4.228,4.230,4.233,4.235,4.238,
     .    4.240,4.243,4.245,4.247,4.250,4.252,4.255,4.257,4.259,4.262,
     .    4.264,4.266,4.268,4.271,4.273,4.275,4.278,4.280,4.282,4.284/
      data p5a/
     .    1.398,1.408,1.427,1.446,1.466,1.486,1.506,1.526,1.545,1.564,
     .    1.583,1.601,1.619,1.636,1.652,1.668,1.683,1.698,1.712,1.725,
     .    1.738,1.751,1.763,1.775,1.786,1.797,1.808,1.818,1.828,1.837,
     .    1.846,1.855,1.864,1.873,1.881,1.889,1.897,1.904,1.912,1.919,
     .    1.926,1.933,1.940,1.946,1.953,1.960,1.966,1.972,1.979,1.985,
     .    1.991,1.997,2.003,2.009,2.016,2.022,2.028,2.034,2.040,2.046,
     .    2.052,2.058,2.065,2.071,2.077,2.084,2.090,2.097,2.103,2.110,
     .    2.117,2.124,2.131,2.138,2.145,2.152,2.160,2.167,2.175,2.183,
     .    2.191,2.199,2.207,2.216,2.224,2.233,2.241,2.250,2.259,2.268,
     .    2.278,2.287,2.297,2.306,2.316,2.326,2.336,2.346,2.356,2.367,
     .    2.377,2.387,2.398,2.409,2.419,2.430,2.441,2.452,2.463,2.474,
     .    2.485,2.497,2.508,2.519,2.530,2.542,2.553,2.564,2.576,2.587,
     .    2.599,2.610,2.621,2.633,2.644,2.655,2.667,2.678,2.689,2.701,
     .    2.712,2.723,2.734,2.745,2.757,2.768,2.779,2.790,2.801,2.812,
     .    2.822,2.833,2.844,2.855,2.865,2.876,2.886,2.897,2.907,2.918,
     .    2.928,2.938,2.948,2.958,2.968,2.978,2.988,2.998,3.008,3.018,
     .    3.027,3.037,3.046,3.056,3.065,3.075,3.084,3.093,3.102,3.111,
     .    3.120,3.129,3.138,3.147,3.156,3.164,3.173,3.182,3.190,3.198,
     .    3.207,3.215,3.223,3.232,3.240,3.248,3.256,3.264,3.272,3.279/
      data p5b/
     .    3.120,3.129,3.138,3.147,3.156,3.164,3.173,3.182,3.190,3.198,
     .    3.207,3.215,3.223,3.232,3.240,3.248,3.256,3.264,3.272,3.279,
     .    3.287,3.295,3.303,3.310,3.318,3.325,3.333,3.340,3.347,3.355,
     .    3.362,3.369,3.376,3.383,3.390,3.397,3.404,3.411,3.417,3.424,
     .    3.431,3.438,3.444,3.451,3.457,3.464,3.470,3.476,3.483,3.489,
     .    3.495,3.501,3.507,3.514,3.520,3.526,3.531,3.537,3.543,3.549,
     .    3.555,3.561,3.566,3.572,3.578,3.583,3.589,3.594,3.600,3.605,
     .    3.610,3.616,3.621,3.626,3.632,3.637,3.642,3.647,3.652,3.657,
     .    3.662,3.667,3.672,3.677,3.682,3.687,3.692,3.697,3.701,3.706,
     .    3.711,3.716,3.720,3.725,3.729,3.734,3.738,3.743,3.747,3.752,
     .    3.756,3.761,3.765,3.769,3.774,3.778,3.782,3.786,3.790,3.795,
     .    3.799,3.803,3.807,3.811,3.815,3.819,3.823,3.827,3.831,3.835,
     .    3.839,3.843,3.846,3.850,3.854,3.858,3.862,3.865,3.869,3.873,
     .    3.876,3.880,3.884,3.887,3.891,3.894,3.898,3.901,3.905,3.908,
     .    3.912,3.915,3.918,3.922,3.925,3.929,3.932,3.935,3.939,3.942,
     .    3.945,3.948,3.951,3.955,3.958,3.961,3.964,3.967,3.970,3.974,
     .    3.977,3.980,3.983,3.986,3.989,3.992,3.995,3.998,4.001,4.004/
      data p6a/
     .    0.778,0.804,0.817,0.834,0.854,0.876,0.901,0.928,0.957,0.987,
     .    1.017,1.048,1.079,1.109,1.139,1.169,1.197,1.225,1.253,1.279,
     .    1.304,1.329,1.353,1.376,1.398,1.419,1.440,1.459,1.478,1.497,
     .    1.515,1.532,1.548,1.564,1.580,1.594,1.609,1.623,1.636,1.649,
     .    1.662,1.674,1.686,1.698,1.709,1.720,1.730,1.740,1.750,1.760,
     .    1.769,1.779,1.788,1.796,1.805,1.813,1.821,1.829,1.837,1.845,
     .    1.852,1.860,1.867,1.874,1.881,1.888,1.894,1.901,1.907,1.914,
     .    1.920,1.926,1.932,1.938,1.944,1.950,1.956,1.962,1.968,1.974,
     .    1.979,1.985,1.991,1.996,2.002,2.007,2.013,2.018,2.024,2.029,
     .    2.035,2.041,2.046,2.052,2.057,2.063,2.068,2.074,2.080,2.086,
     .    2.091,2.097,2.103,2.109,2.115,2.121,2.127,2.133,2.139,2.145,
     .    2.152,2.158,2.164,2.171,2.177,2.184,2.190,2.197,2.204,2.211,
     .    2.218,2.225,2.232,2.239,2.246,2.253,2.261,2.268,2.276,2.283,
     .    2.291,2.298,2.306,2.314,2.322,2.330,2.338,2.346,2.354,2.362,
     .    2.370,2.379,2.387,2.395,2.404,2.412,2.420,2.429,2.438,2.446,
     .    2.455,2.463,2.472,2.481,2.489,2.498,2.507,2.516,2.524,2.533,
     .    2.542,2.551,2.560,2.569,2.577,2.586,2.595,2.604,2.613,2.622,
     .    2.631,2.639,2.648,2.657,2.666,2.675,2.683,2.692,2.701,2.710,
     .    2.718,2.727,2.736,2.744,2.753,2.761,2.770,2.779,2.787,2.796/
      data p6b/
     .    2.631,2.639,2.648,2.657,2.666,2.675,2.683,2.692,2.701,2.710,
     .    2.718,2.727,2.736,2.744,2.753,2.761,2.770,2.779,2.787,2.796,
     .    2.804,2.812,2.821,2.829,2.838,2.846,2.854,2.862,2.871,2.879,
     .    2.887,2.895,2.903,2.911,2.919,2.927,2.935,2.943,2.951,2.958,
     .    2.966,2.974,2.982,2.989,2.997,3.005,3.012,3.020,3.027,3.035,
     .    3.042,3.049,3.057,3.064,3.071,3.078,3.086,3.093,3.100,3.107,
     .    3.114,3.121,3.128,3.135,3.141,3.148,3.155,3.162,3.169,3.175,
     .    3.182,3.188,3.195,3.202,3.208,3.214,3.221,3.227,3.234,3.240,
     .    3.246,3.252,3.259,3.265,3.271,3.277,3.283,3.289,3.295,3.301,
     .    3.307,3.313,3.319,3.325,3.330,3.336,3.342,3.348,3.353,3.359,
     .    3.364,3.370,3.376,3.381,3.386,3.392,3.397,3.403,3.408,3.413,
     .    3.419,3.424,3.429,3.434,3.440,3.445,3.450,3.455,3.460,3.465,
     .    3.470,3.475,3.480,3.485,3.490,3.495,3.499,3.504,3.509,3.514,
     .    3.518,3.523,3.528,3.533,3.537,3.542,3.546,3.551,3.555,3.560,
     .    3.564,3.569,3.573,3.578,3.582,3.586,3.591,3.595,3.599,3.604,
     .    3.608,3.612,3.616,3.621,3.625,3.629,3.633,3.637,3.641,3.645,
     .    3.649,3.653,3.657,3.661,3.665,3.669,3.673,3.677,3.681,3.685/
      data p7a/
     .    1.398,1.398,1.398,1.398,1.406,1.425,1.443,1.461,1.480,1.498,
     .    1.516,1.534,1.551,1.568,1.585,1.601,1.616,1.631,1.646,1.660,
     .    1.674,1.687,1.700,1.712,1.724,1.736,1.747,1.758,1.768,1.778,
     .    1.788,1.797,1.806,1.815,1.824,1.832,1.840,1.848,1.855,1.863,
     .    1.870,1.877,1.883,1.890,1.896,1.902,1.908,1.914,1.920,1.925,
     .    1.931,1.936,1.941,1.946,1.951,1.956,1.960,1.965,1.969,1.974,
     .    1.978,1.982,1.986,1.990,1.994,1.998,2.001,2.005,2.009,2.012,
     .    2.016,2.019,2.022,2.026,2.029,2.032,2.035,2.038,2.041,2.044,
     .    2.047,2.050,2.053,2.056,2.059,2.061,2.064,2.067,2.069,2.072,
     .    2.075,2.077,2.080,2.082,2.085,2.088,2.090,2.093,2.095,2.098,
     .    2.100,2.103,2.105,2.107,2.110,2.112,2.115,2.117,2.120,2.122,
     .    2.125,2.127,2.130,2.132,2.135,2.137,2.140,2.142,2.145,2.148,
     .    2.150,2.153,2.155,2.158,2.161,2.163,2.166,2.169,2.172,2.175,
     .    2.178,2.180,2.183,2.186,2.189,2.192,2.195,2.198,2.202,2.205,
     .    2.208,2.211,2.215,2.218,2.221,2.225,2.228,2.232,2.235,2.239,
     .    2.243,2.246,2.250,2.254,2.258,2.261,2.265,2.269,2.273,2.277,
     .    2.282,2.286,2.290,2.294,2.299,2.303,2.307,2.312,2.316,2.321,
     .    2.325,2.330,2.335,2.339,2.344,2.349,2.354,2.359,2.364,2.369,
     .    2.374,2.379,2.384,2.389,2.394,2.399,2.405,2.410,2.415,2.420/
      data p7b/
     .    2.325,2.330,2.335,2.339,2.344,2.349,2.354,2.359,2.364,2.369,
     .    2.374,2.379,2.384,2.389,2.394,2.399,2.405,2.410,2.415,2.420,
     .    2.426,2.431,2.437,2.442,2.448,2.453,2.459,2.464,2.470,2.476,
     .    2.481,2.487,2.493,2.498,2.504,2.510,2.516,2.521,2.527,2.533,
     .    2.539,2.545,2.551,2.556,2.562,2.568,2.574,2.580,2.586,2.592,
     .    2.598,2.604,2.610,2.616,2.622,2.628,2.634,2.640,2.646,2.652,
     .    2.658,2.664,2.670,2.676,2.682,2.687,2.693,2.699,2.705,2.711,
     .    2.717,2.723,2.729,2.735,2.741,2.747,2.753,2.759,2.764,2.770,
     .    2.776,2.782,2.788,2.794,2.799,2.805,2.811,2.817,2.823,2.828,
     .    2.834,2.840,2.846,2.851,2.857,2.863,2.868,2.874,2.879,2.885,
     .    2.891,2.896,2.902,2.907,2.913,2.918,2.924,2.929,2.935,2.940,
     .    2.945,2.951,2.956,2.962,2.967,2.972,2.978,2.983,2.988,2.993,
     .    2.999,3.004,3.009,3.014,3.019,3.025,3.030,3.035,3.040,3.045,
     .    3.050,3.055,3.060,3.065,3.070,3.075,3.080,3.085,3.090,3.095,
     .    3.099,3.104,3.109,3.114,3.119,3.123,3.128,3.133,3.138,3.142,
     .    3.147,3.152,3.156,3.161,3.165,3.170,3.175,3.179,3.184,3.188,
     .    3.193,3.197,3.202,3.206,3.210,3.215,3.219,3.224,3.228,3.232/
      data p8a/
     .    1.447,1.447,1.447,1.447,1.447,1.447,1.459,1.475,1.489,1.504,
     .    1.518,1.531,1.544,1.556,1.568,1.580,1.591,1.602,1.612,1.622,
     .    1.631,1.640,1.649,1.658,1.666,1.674,1.682,1.689,1.696,1.703,
     .    1.710,1.716,1.722,1.728,1.734,1.740,1.745,1.751,1.756,1.761,
     .    1.766,1.770,1.775,1.779,1.784,1.788,1.792,1.796,1.800,1.804,
     .    1.807,1.811,1.814,1.818,1.821,1.824,1.827,1.831,1.834,1.836,
     .    1.839,1.842,1.845,1.848,1.850,1.853,1.855,1.858,1.860,1.863,
     .    1.865,1.867,1.870,1.872,1.874,1.876,1.878,1.880,1.882,1.884,
     .    1.886,1.888,1.890,1.892,1.894,1.896,1.898,1.900,1.902,1.903,
     .    1.905,1.907,1.909,1.911,1.912,1.914,1.916,1.917,1.919,1.921,
     .    1.923,1.924,1.926,1.928,1.929,1.931,1.933,1.934,1.936,1.938,
     .    1.939,1.941,1.943,1.945,1.946,1.948,1.950,1.951,1.953,1.955,
     .    1.957,1.959,1.960,1.962,1.964,1.966,1.968,1.970,1.971,1.973,
     .    1.975,1.977,1.979,1.981,1.983,1.985,1.987,1.989,1.991,1.993,
     .    1.995,1.998,2.000,2.002,2.004,2.006,2.009,2.011,2.013,2.015,
     .    2.018,2.020,2.023,2.025,2.027,2.030,2.032,2.035,2.037,2.040,
     .    2.043,2.045,2.048,2.051,2.053,2.056,2.059,2.062,2.064,2.067,
     .    2.070,2.073,2.076,2.079,2.082,2.085,2.088,2.091,2.094,2.097,
     .    2.100,2.103,2.107,2.110,2.113,2.116,2.120,2.123,2.126,2.130/
      data p8b/
     .    2.070,2.073,2.076,2.079,2.082,2.085,2.088,2.091,2.094,2.097,
     .    2.100,2.103,2.107,2.110,2.113,2.116,2.120,2.123,2.126,2.130,
     .    2.133,2.137,2.140,2.143,2.147,2.151,2.154,2.158,2.161,2.165,
     .    2.168,2.172,2.176,2.180,2.183,2.187,2.191,2.195,2.198,2.202,
     .    2.206,2.210,2.214,2.218,2.222,2.226,2.230,2.233,2.237,2.241,
     .    2.245,2.250,2.254,2.258,2.262,2.266,2.270,2.274,2.278,2.282,
     .    2.286,2.291,2.295,2.299,2.303,2.307,2.312,2.316,2.320,2.324,
     .    2.329,2.333,2.337,2.341,2.346,2.350,2.354,2.359,2.363,2.367,
     .    2.371,2.376,2.380,2.384,2.389,2.393,2.397,2.402,2.406,2.410,
     .    2.415,2.419,2.423,2.428,2.432,2.436,2.441,2.445,2.449,2.454,
     .    2.458,2.462,2.467,2.471,2.475,2.480,2.484,2.488,2.493,2.497,
     .    2.501,2.506,2.510,2.514,2.519,2.523,2.527,2.531,2.536,2.540,
     .    2.544,2.548,2.553,2.557,2.561,2.565,2.570,2.574,2.578,2.582,
     .    2.586,2.591,2.595,2.599,2.603,2.607,2.611,2.616,2.620,2.624,
     .    2.628,2.632,2.636,2.640,2.644,2.648,2.652,2.656,2.661,2.665,
     .    2.669,2.673,2.677,2.681,2.685,2.689,2.693,2.696,2.700,2.704,
     .    2.708,2.712,2.716,2.720,2.724,2.728,2.732,2.736,2.739,2.743/
      data p9a/
     .    1.322,1.322,1.322,1.322,1.322,1.322,1.322,1.322,1.322,1.325,
     .    1.334,1.342,1.351,1.358,1.366,1.373,1.380,1.386,1.392,1.398,
     .    1.404,1.409,1.415,1.420,1.425,1.429,1.434,1.438,1.442,1.446,
     .    1.450,1.454,1.457,1.461,1.464,1.467,1.470,1.473,1.476,1.479,
     .    1.482,1.485,1.487,1.490,1.492,1.495,1.497,1.499,1.501,1.503,
     .    1.505,1.507,1.509,1.511,1.513,1.515,1.517,1.519,1.520,1.522,
     .    1.524,1.525,1.527,1.528,1.530,1.531,1.533,1.534,1.535,1.537,
     .    1.538,1.539,1.541,1.542,1.543,1.545,1.546,1.547,1.548,1.549,
     .    1.551,1.552,1.553,1.554,1.555,1.556,1.558,1.559,1.560,1.561,
     .    1.562,1.563,1.565,1.566,1.567,1.568,1.569,1.570,1.571,1.573,
     .    1.574,1.575,1.576,1.577,1.579,1.580,1.581,1.582,1.584,1.585,
     .    1.586,1.588,1.589,1.590,1.592,1.593,1.594,1.596,1.597,1.599,
     .    1.600,1.602,1.603,1.605,1.606,1.608,1.609,1.611,1.612,1.614,
     .    1.616,1.617,1.619,1.621,1.622,1.624,1.626,1.628,1.630,1.631,
     .    1.633,1.635,1.637,1.639,1.641,1.643,1.645,1.647,1.649,1.651,
     .    1.653,1.655,1.657,1.659,1.661,1.664,1.666,1.668,1.670,1.673,
     .    1.675,1.677,1.679,1.682,1.684,1.686,1.689,1.691,1.694,1.696,
     .    1.699,1.701,1.704,1.706,1.709,1.711,1.714,1.716,1.719,1.722,
     .    1.724,1.727,1.729,1.732,1.735,1.738,1.740,1.743,1.746,1.749/
      data p9b/
     .    1.699,1.701,1.704,1.706,1.709,1.711,1.714,1.716,1.719,1.722,
     .    1.724,1.727,1.729,1.732,1.735,1.738,1.740,1.743,1.746,1.749,
     .    1.751,1.754,1.757,1.760,1.763,1.765,1.768,1.771,1.774,1.777,
     .    1.780,1.783,1.786,1.789,1.792,1.795,1.798,1.801,1.804,1.807,
     .    1.810,1.813,1.816,1.819,1.822,1.825,1.828,1.831,1.834,1.837,
     .    1.840,1.843,1.847,1.850,1.853,1.856,1.859,1.862,1.865,1.869,
     .    1.872,1.875,1.878,1.881,1.884,1.888,1.891,1.894,1.897,1.901,
     .    1.904,1.907,1.910,1.913,1.917,1.920,1.923,1.926,1.930,1.933,
     .    1.936,1.939,1.943,1.946,1.949,1.952,1.956,1.959,1.962,1.965,
     .    1.969,1.972,1.975,1.978,1.982,1.985,1.988,1.992,1.995,1.998,
     .    2.001,2.005,2.008,2.011,2.014,2.018,2.021,2.024,2.027,2.031,
     .    2.034,2.037,2.040,2.044,2.047,2.050,2.053,2.057,2.060,2.063,
     .    2.066,2.070,2.073,2.076,2.079,2.083,2.086,2.089,2.092,2.095,
     .    2.099,2.102,2.105,2.108,2.111,2.115,2.118,2.121,2.124,2.127,
     .    2.131,2.134,2.137,2.140,2.143,2.146,2.149,2.153,2.156,2.159,
     .    2.162,2.165,2.168,2.171,2.175,2.178,2.181,2.184,2.187,2.190,
     .    2.193,2.196,2.199,2.202,2.205,2.208,2.212,2.215,2.218,2.221/
c
      if(t.lt.12000.) then
        pf=g0(ion-3)
        dut=0.
        dun=0.
        return
      endif
c
      it=t/1000
      if(it.ge.350) it=349
      t1=1000.*it
      t2=t1+1000.
      if(ion.eq.4) then
        if(t.le.200000.) then
          xu1=p4a(it-10)
          xu2=p4a(it-9)
        else
          xu1=p4b(it-180)
          xu2=p4b(it-179)
        endif
      else if(ion.eq.5) then
        if(t.le.200000.) then
          xu1=p5a(it-10)
          xu2=p5a(it-9)
        else
          xu1=p5b(it-180)
          xu2=p5b(it-179)
        endif
      else if(ion.eq.6) then
        if(t.le.200000.) then
          xu1=p6a(it-10)
          xu2=p6a(it-9)
        else
          xu1=p6b(it-180)
          xu2=p6b(it-179)
        endif
      else if(ion.eq.7) then
        if(t.le.200000.) then
          xu1=p7a(it-10)
          xu2=p7a(it-9)
        else
          xu1=p7b(it-180)
          xu2=p7b(it-179)
        endif
      else if(ion.eq.8) then
        if(t.le.200000.) then
          xu1=p8a(it-10)
          xu2=p8a(it-9)
        else
          xu1=p8b(it-180)
          xu2=p8b(it-179)
        endif
      else if(ion.eq.9) then
        if(t.le.200000.) then
          xu1=p9a(it-10)
          xu2=p9a(it-9)
        else
          xu1=p9b(it-180)
          xu2=p9b(it-179)
        endif
      endif
c
      dxt=xmil*(xu2-xu1)
      xu=xu1+(t-t1)*dxt
      pf=exp(xen*xu)
      dut=xen*pf*dxt
      dun=0.
      return
      end

c 
c 
c ******************************************************************
c 
c
      subroutine opfrac(iat,ion,t,ane,pf,fra)
c     =======================================
c
      include 'IMPLIC.FOR'
      parameter (mtemp =100,
     *           melec = 60,
     *           mion1 = 30,
     *           mion2 = 32,
     *           mdat  = 17,
     *           mstag = 258)
      parameter (inp=71)
      dimension frac0(mion2),ioo(mion2),idat(mion1)
      dimension gg(mion1,mdat),g0(mion1),z0(mion2)
      dimension uu(mion1,mdat),u0(mion1)
      dimension indxat(mion1,mdat),indxa(mion1)
      dimension u6(6),u7(7),u8(8),u10(10),u11(11)
      dimension u12(12),u13(13),u14(14),u16(16),u18(18),u20(20)
      dimension u24(24),u25(25),u26(26),u28(28)
      common/pfoptb/pfop(mtemp,melec,mstag),pfophm(mtemp,melec),
     *             frac(mtemp,melec,mstag),
     *             frop(mtemp,melec,mstag),itemp(mtemp)
      equivalence (u6(1),uu(1,3)),(u7(1),uu(1,4)),(u8(1),uu(1,5))
      equivalence (u10(1),uu(1,6)),(u11(1),uu(1,7)),(u12(1),uu(1,8))
      equivalence (u13(1),uu(1,9)),(u14(1),uu(1,10)),(u16(1),uu(1,11))
      equivalence (u18(1),uu(1,12)),(u20(1),uu(1,13)),(u24(1),uu(1,14))
      equivalence (u25(1),uu(1,15)),(u26(1),uu(1,16)),(u28(1),uu(1,17))
      character*30 filop
      data idat   / 1, 2, 0, 0, 0, 3, 4, 5, 0, 6,
     *              7, 8, 9,10, 0,11, 0,12, 0,13,
     *              0, 0, 0,14,15,16, 0,17, 0, 0/ 
      data indxat /1,2,28*0,
     *            3,4,5,27*0,
     *            6,7,8,9,10,11,12,23*0,
     *            13,14,15,16,17,18,19,20,22*0,
     *            21,22,23,24,25,26,27,28,29,21*0,
     *            30,31,32,33,34,35,36,37,38,39,40,19*0,
     *            41,42,43,44,45,46,47,48,49,50,51,52,18*0,
     *            53,54,55,56,57,58,59,60,61,62,63,64,65,17*0,
     *            66,67,68,69,70,71,72,73,74,75,76,77,78,79,16*0,
     *            80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,15*0,
     *            95,96,97,98,99,100,101,102,103,104,105,106,107,
     *            108,109,110,111,13*0,
     *            112,113,114,115,116,117,118,119,120,121,122,123,
     *            124,125,126,127,128,129,130,11*0,
     *            131,132,133,134,135,136,137,138,139,140,141,142,
     *            143,144,145,146,147,148,149,150,151,9*0,
     *            152,153,154,155,156,157,158,159,160,161,162,163,
     *            164,165,166,167,168,169,170,171,172,173,174,175,
     *            176,5*0,
     *            177,178,179,180,181,182,183,184,185,186,187,188,
     *            189,190,191,192,193,194,195,196,197,198,199,200,
     *            201,202,4*0,
     *            203,204,205,206,207,208,209,210,211,212,213,214,
     *            215,216,217,218,219,220,221,222,223,224,225,226,
     *            227,228,229,3*0,
     *            230,231,232,233,234,235,236,237,238,239,240,241,
     *            242,243,244,245,246,247,248,249,250,251,252,253,
     *            254,255,256,257,258,0/
      data gg/2.,29*0.,
     *        2.,1.,28*0.,
     *        2.,1.,2.,1.,6.,9.,24*0.,
     *        2.,1.,2.,1.,6.,9.,4.,23*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,22*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,20*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,19*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,18*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,6.,17*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,6.,9.,16*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,6.,9.,4.,9.,14*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,6.,9.,4.,9.,6.,1.,
     *        12*0.,2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,6.,9.,4.,9.,
     *        6.,1.,2.,1.,10*0.,2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,
     *        6.,9.,4.,9.,6.,1.,10.,21.,28.,25.,6.,7.,6*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,6.,9.,4.,9.,
     *           6.,1.,10.,21.,28.,25.,6.,7.,6.,5*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,6.,9.,4.,9.,
     *           6.,1.,10.,21.,28.,25.,6.,25.,30.,25.,4*0.,
     *        2.,1.,2.,1.,6.,9.,4.,9.,6.,1.,2.,1.,6.,9.,4.,9.,
     *           6.,1.,10.,21.,28.,25.,6.,25.,28.,21.,10.,21.,0.,0./
      data uu(1,1)/109.6787/
      data uu(1,2)/198.3108/
      data uu(2,2)/438.9089/
      data u6/90.82,196.665,386.241,520.178,3162.395,3952.061/
      data u7/117.225,238.751,382.704,624.866,789.537,4452.758,5380.089/
      data u8/109.837,283.24,443.086,624.384,918.657,1114.008,5963.135,
     *        7028.393/
      data u10/173.93,330.391,511.8,783.3,1018.,1273.8,1671.792,
     *         1928.462,9645.005,10986.876/
      data u11/41.449,381.395,577.8,797.8,1116.2,1388.5,1681.5,2130.8,
     *         2418.7,11817.061,13297.676/
      data u12/61.671,121.268,646.41,881.1,1139.4,1504.3,1814.3,2144.7,
     *         2645.2,2964.4,14210.261,15829.951/
      data u13/48.278,151.86,229.446,967.8,1239.8,1536.3,1947.3,2295.4,
     *         2663.4,3214.8,3565.6,16825.022,18584.138/
      data u14/65.748,131.838,270.139,364.093,1345.1,1653.9,1988.4,
     *         2445.3,2831.9,3237.8,3839.8,4222.4,19661.693,21560.63/
      data u16/83.558,188.2,280.9,381.541,586.2,710.184,2265.9,2647.4,
     *         3057.7,3606.1,4071.4,4554.3,5255.9,5703.6,26002.663,
     *         28182.535/
      data u18/127.11,222.848,328.6,482.4,605.1,734.04,1002.73,1157.08,
     *         3407.3,3860.9,4347.,4986.6,5533.8,6095.5,6894.2,7404.4,
     *         33237.173,35699.936/
      data u20/49.306,95.752,410.642,542.6,681.6,877.4,1026.,1187.6,
     *         1520.64,1704.047,4774.,5301.,5861.,6595.,7215.,7860.,
     *         8770.,9338.,41366.,44177.41/
      data u24/54.576,132.966,249.7,396.5,560.2,731.02,1291.9,1490.,
     *         1688.,1971.,2184.,2404.,2862.,3098.52,8151.,8850.,
     *         9560.,10480.,11260.,12070.,13180.,13882.,60344.,63675.9/
      data u25/59.959,126.145,271.55,413.,584.,771.1,961.44,1569.,
     *         1789.,2003.,2307.,2536.,2771.,3250.,3509.82,9152.,
     *         9872.,10620.,11590.,12410.,13260.,14420.,15162.,
     *         65660.,69137.4/
      data u26/63.737,130.563,247.22,442.,605.,799.,1008.,1218.38,
     *         1884.,2114.,2341.,2668.,2912.,3163.,3686.,3946.82,
     *         10180.,10985.,11850.,12708.,13620.,14510.,15797.,
     *         16500.,71203.,74829.6/
      data u28/61.6,146.542,283.8,443.,613.5,870.,1070.,1310.,1560.,
     *         1812.,2589.,2840.,3100.,3470.,3740.,4020.,4606.,
     *         4896.2,12430.,13290.,14160.,15280.,16220.,17190.,
     *         18510.,19351.,82984.,86909.4/
c
c     If the routine is called with IAT=0, initialization:
c     read the ionization fractions form the Opacity Project tables,
c     and evaluate partition functions (assuming that the partition
c     function for the highest ion is eqaul to the statistical weight of
c     the ground state.
c     The table of partition functions, in the OP temperatures and
c     electron densities, are stored in the array PFOP
c
C     Read is done in a loop over the OP species
c
      if(iat.gt.0) go to 50
      do 40 iatnum=1,28
      if(idat(iatnum).eq.0) go to 40
c
      g0(iatnum+1)=1.
      indxa(iatnum+1)=indxat(iatnum+1,idat(iatnum))
      do i=1,iatnum
        ig0=iatnum-i+1
        g0(ig0)=gg(i,idat(iatnum))
        indxa(i)=indxat(i,idat(iatnum))
        u0(i)=uu(i,idat(iatnum))*1000.
      enddo
c
c     initializion of partition functions by the statistical weights of
c     the ground state
c
      do i=1,iatnum+1
         indx=indxa(i)
         pf0=g0(i)
         do it=1,mtemp
            do ieind=1,melec
               pfop(it,ieind,indx)=pf0
            enddo
         enddo
      enddo
c
      if(iatnum.eq.1) open(inp,file='ioniz.dat',status='old')
      read(inp,*)
      read(inp,*) it0,it1,itstp
      ntt=(it1-it0)/itstp+1
c
      do 30 it=1,ntt
         read(inp,*) itt,ie0,ie1,iestp
         itemp(it)=itt
         net=(ie1-ie0)/iestp+1
         t=exp(2.3025851*0.025*itt)
         safac0=sqrt(t)*t/2.07d-16
         tkcm=0.69496*t
         do 20 ie=1,net
            read(inp,601) iee,ion0,ion1,
     *                    (ioo(i),frac0(i),i=ion0+2,min(ion1,ion0+3)+2)
            ane=exp(2.3025851*0.25*iee)
            safac=safac0/ane
            nio=ion1-ion0
            if(nio.ge.3) then
               nlin=nio/4
               do ilin=1,nlin
                  read(inp,602) (ioo(i),frac0(i),
     *                 i=ion0+4*ilin+2,min(ion1,ion0+4*ilin+3)+2)
               end do
            end if
            ieind=iee/2
            ion0=ion0+2
            ion1=ion1+2
            do 10 ionn=ion0,ion1
              iation=iatnum+2-ionn
              if(ionn.lt.iatnum+2) then
                 if(ionn.eq.ion0) then
                    z0(ionn)=g0(iation)
                  else
                    z0(ionn)=frac0(ionn)/frac0(ionn-1)*safac*z0(ionn-1)
                    z0(ionn)=z0(ionn)*exp(-u0(iation)/tkcm)
                 endif
                 pfop(it,ieind,indxa(iation))=z0(ionn)
                 frac(it,ieind,indxa(iation))=frac0(ionn)
              else
                 u0hm=6090.5
                 z0hm=frac0(ionn)/frac0(ionn-1)*safac
                 z0hm=z0hm*exp(-u0hm/tkcm)
                 pfophm(it,ieind)=z0hm
              end if
   10       continue
   20    continue
   30 continue
   40 continue
  601 format(3i4,2x,4(i4,1x,e9.3))
  602 format(14x,4(i4,1x,e9.3))
      return
C
C     ----------------------------------------------------------
C
C     If the routine is called with IAT>0, evaluate the partition
C     function of atom IAT, ion ION,
C     for temperature T and electron density ANE
C     Evaluation is done by interpolation from previously computed
C     Opacity Project values
C
   50 continue
c
      xt=log10(t)
      kt0=2*int(20.*xt)
      xne=log10(ane)
      kn0=int(2.*xne)
c
      iatnum=iat
      if(idat(iatnum).eq.0) then
         write(6,600) iatnum
         iatnum=-1
  600    format(' data for element no. ',i3,' do not exist')
         return
      end if
      indx=indxat(ion,idat(iatnum))
      if(kt0.lt.itemp(1)) then
         kt1=1
         write(6,611) t
  611    format(' (FRACOP) Extrapol. in T (low)',f7.0)
         go to 120
      endif
      if(kt0.ge.itemp(ntt)) then
         kt1=ntt-1
         write(6,612) t
  612    format(' (FRACOP) Extrapol. in T (high)',f12.0)
         go to 120
      endif
      do 110 it=1,ntt
         if(kt0.eq.itemp(it)) then
            kt1=it
            go to 120
         endif
  110 continue
  120 continue
      if(kn0.lt.1) then
         kn1=1
         go to 130
      endif
      if(kn0.ge.60) then
         kn1=59
         write(6,614) xne
  614    format(' (FRACOP) Extrapol. in Ne (high)',f9.4)
         go to 130
      endif
      kn1=kn0
  130 continue
      xt1=0.025*itemp(kt1)
      dxt=0.05
      at1=(xt-xt1)/dxt
      xn1=0.5*kn1
      dxn=0.5
      an1=(xne-xn1)/dxn
      x11=pfop(kt1,kn1,indx)
      x21=pfop(kt1+1,kn1,indx)
      x12=pfop(kt1,kn1+1,indx)
      x22=pfop(kt1+1,kn1+1,indx)
      x1221=x11*x21*x12*x22
      if(x1221.eq.0.) then
         xx1=x11+at1*(x21-x11)
         xx2=x12+at1*(x22-x12)
         rrx=xx1+an1*(xx2-xx1)
       else
         x11=log10(x11)
         x21=log10(x21)
         x12=log10(x12)
         x22=log10(x22)
         xx1=x11+at1*(x21-x11)
         xx2=x12+at1*(x22-x12)
         rrx=xx1+an1*(xx2-xx1)
         rrx=exp(2.3025851*rrx)
      endif
      pf=rrx
c
c     the next section - the true ionization fractions - is
C     commented out (not directly needed by TLUSTY)
c
c      x11=frac(kt1,kn1,indx)
c      x21=frac(kt1+1,kn1,indx)
c      x12=frac(kt1,kn1+1,indx)
c      x22=frac(kt1+1,kn1+1,indx)
c      x1221=x11*x21*x12*x22
c      if(x1221.eq.0.) then
c         xx1=x11+at1*(x21-x11)
c         xx2=x12+at1*(x22-x12)
c         rrx=xx1+an1*(xx2-xx1)
c       else
c         x11=log10(x11)
c         x21=log10(x21)
c         x12=log10(x12)
c         x22=log10(x22)
c         xx1=x11+at1*(x21-x11)
c         xx2=x12+at1*(x22-x12)
c         rrx=xx1+an1*(xx2-xx1)
c         rrx=exp(2.3025851*rrx)
c      endif
c      fra=rrx
c
      return
      end
C
C
C ********************************************************************
C
C
      FUNCTION XK2DOP(TAU)
C     ====================
C
C     KERNEL FUNCTION K2 
C     AFTER  HUMMER,  1981, J.Q.S.R.T. 26, 187
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (PI2SQ=2.506628275D0, PISQ=1.772453851D0, UN=1.D0,
     *   A1= -1.117897000D-1,   A2= -1.249099917D-1, 
     *   A3= -9.136358767D-3,   A4= -3.370280896D-4,
     *   B1=  1.566124168D-1,   B2=  9.013261660D-3,   
     *   B3=  1.908481163D-4,   B4= -1.547417750D-7, 
     *   B5= -6.657439727D-9,
     *   C1=  1.915049608D01,   C2=  1.007986843D02,   
     *   C3=  1.295307533D02,   C4= -3.143372468D01,
     *   D1=  1.968910391D01,   D2=  1.102576321D02,   
     *   D3=  1.694911399D02,   D4= -1.669969409D01,  
     *   D5= -3.666448000D01)
      XK2DOP=UN
      IF(TAU.LE.0.) RETURN
      IF(TAU.GT.11.) GO TO 10
      P=UN+TAU*(A1+TAU*(A2+TAU*(A3+TAU*A4)))
      Q=UN+TAU*(B1+TAU*(B2+TAU*(B3+TAU*(B4+TAU*B5))))
      XK2DOP=TAU/PI2SQ*LOG(TAU/PISQ)+P/Q
      RETURN
   10 X=UN/LOG(TAU/PISQ)
      P=UN+X*(C1+X*(C2+X*(C3+X*C4)))
      Q=UN+X*(D1+X*(D2+X*(D3+X*(D4+X*D5))))
      XK2DOP=P/Q/2.D0/TAU/SQRT(LOG(TAU/PISQ))
      RETURN
      END
C
C
C ********************************************************************
C
C
 
 
      SUBROUTINE LTEGR
C     ================
C
C     Driving procedure for computing the initial LTE-grey model
C     atmosphere
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
C
      COMMON ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       DEPTH(MDEPTH),DEPTH0(MDEPTH),TAU(MDEPTH),TAU0(MDEPTH),
     *       TEMP0(MDEPTH),ELEC0(MDEPTH),DENS0(MDEPTH),DM0(MDEPTH)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/GRAYTS/TAUROS(MDEPTH),TAUFLX(MDEPTH)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
C
C ----------------
C Input parameters
C ----------------
C
C     NDEPTH  - number of depth points for evaluating LTE-grey model
C             = 0 - NDEPTH is taken to be ND-1
C     TAUFIR  - Rosseland optical depth in the first depth point
C     TAULAS  - Rosseland optical depth in the last depth point
C     ABROS0  - an estimate of the Rosseland opacity (per gram) at the
C               first depth point
C     TSURF   = 0  - surface temperature and the Hopf function are
C                    evaluated exactly
C             > 0  - value of surface temperature is set to TSURF, and
C                    the Hopf function is assumed to be constant,
C                    corresponding to TSURF
C     ALBAVE  = 0  - wind blanketing is not considered
C             > 0  - wind blanketing is considered; the averaged value
C                    of albedo [precisely the quantity (1+rho)/(1-rho)
C                    in the notation of Hummer (Ap.J. 257, 724, 1982)
C                    see his Eq. (3.1)] is ALBAVE
C     DION0        - initial estimate of the degree of ionization at
C                    the first depth point (=1 for completely ionized;
C                    =1/2 for completely neutral)
C
C --------------------------------------------------------------------
C     IDEPTH  -  mode of determining the mass-depth scale to be used
C                in linearization
C             =  0 - depth scale DM (in g*cm**-2) is evaluated as mass
C                    corresponding to Rosseland optical depths which
C                    are equidistantly spaced in logarithms between
C                    the first point TAUFIR and the last point TAULAS
C                    the last-but-one point is, however, set to
C                    TAULAS-1.
C             =  2 - depth scale DM is evaluated as that corresponding
C                    to input values of Rosseland optical depth -
C                    array TAU0(ID), ID=1,ND
C             =  1 - similar, but now DM is evaluated as mass
C                    corresponding to Rosseland optical depths which
C                    are equidistantly spaced in logarithms between
C                    the first point TAU1 and the last-but-one point
C                    TAU2; the last point is TAUL
C                    (i.e. similar to option 0, now TAU1 and TAUL may
C                    be different from TAUFIR nad TAULAS)
C             =  3 - depth scale DM has already been read in START
C     NCONIT       - number of internal iterations for calculating the
C                    gray model with convection
C             =  0 - and HMIX0>0, then NCONIT is set to 10
C     IPRING       - controls diagnostic output of the LTE-gray model
C                    calculations
C              = 0 - no output
C              = 1 - only final LTE-gray model
C              = 2 - results of all internal iterations
C     IHM     >  0 - negative hydrogen ion considered in particle and
C                    charge conservation in ELDENS
C     IH2     >  0 - hydrogen molecule considered in particle
C                    conservation in ELDENS
C     IH2P    >  0 - ionized hydrogen molecule considered in particle
C                    and charge conservation in ELDENS
C
      READ(IBUFF,*) NDEPTH,TAUFIR,TAULAS,ABROS0,TSURF,ALBAVE,DION0
      IF(NDEPTH.GT.MDEPTH) call stop(' NDEPTH too large in LTEGR')
      READ(IBUFF,*) IDEPTH,NCONIT,IPRING,IHM,IH2,IH2P
      IF(ALBAVE.GT.0.AND.TSURF.EQ.0.) TSURF=(0.433*ALBAVE)**0.25
      HOPF0=0.
      IF(TSURF.NE.0.) HOPF0=4.D0*TSURF**4/3.D0
      T4=TEFF**4
      ANEREL=(DION0-HALF )/DION0
      IF(NCONIT.EQ.0.AND.HMIX0.GT.0.) NCONIT=10
      IF(ANEREL.LT.1.D-3) ANEREL=1.D-3
      LCHC0=LCHC
      LCHC=.TRUE.
      IRSPL0=IRSPLT
      IRSPLT=0
      IF(NDEPTH.EQ.0) NDEPTH=ND-1
      ND0=ND
      ND=NDEPTH
      DO 1 I=1,ND0
    1    DM0(I)=DM(I)
C
C     tau(ross) scale - logarithmically equidistant points between
C     input TAUFIR and TAULAS
C
      DML0=LOG(TAUFIR)
      DLGM=(LOG(TAULAS)-DML0)/(NDEPTH-1)
      DO 10 I=1,NDEPTH
         TAU0(I)=DML0+(I-1)*DLGM
         TAU(I)=EXP(TAU0(I))
         TAUROS(I)=TAU(I)
   10 CONTINUE
C
      DPRAD=1.891204931D-15*T4
      PRAD0=DPRAD/1.732D0
      ABROS=ABROS0
      PLOG1=0.
      PLOG2=0.
      PLOG3=0.
      DPLOG1=0.
      DPLOG2=0.
      IF(IPRING.GT.0) WRITE(6,601)
C
C -------------------------------------------------------------------
C
C     1.part
C     Integration of the hydrostatic equilibrium equation on the
C     tau(ross) scale;
C     basically by a predictor-corrector method
C     (similar to Kurucz's ATLAS code)
C
      DO 50 I=1,NDEPTH
         J=0
         TAUR=TAU(I)
C
C        predictor step
C
         IF(I.EQ.1) PLOG=LOG(GRAV/ABROS*TAUR)
         IF(I.GT.1.AND.I.LE.4) PLOG=PLOG1+DPLOG1
         IF(I.GT.4) PLOG=(3.*PLOG4+8.*DPLOG1-4.*DPLOG2+8.*DPLOG3)/3.
         ERROR=1.
         GO TO 40
C
C        corrector step
C ------ iterate between hydrostatic equilibrium (which determines an
C           increment of the total pressure) and state equations (which
C           determine relevant number densities and then the Rosseland
C           opacity)
C
   30    IF(I.EQ.1) PNEW=LOG(GRAV/ABROS*TAUR)
         IF(I.GT.1.AND.I.LE.4)  PNEW=(PLOG+2.*PLOG1+DPLOG+DPLOG1)/3.
         IF(I.GT.4) PNEW=(126.*PLOG1-14.*PLOG3+9.*PLOG4+42.*DPLOG+
     *                    108.*DPLOG1-54.*DPLOG2+24.*DPLOG3)/121.
         ERROR=ABS(PNEW-PLOG)
         PLOG=PNEW
   40    PTOT=EXP(PLOG)
         P=PTOT-TAUR*DPRAD
         J=J+1
         CALL ROSSOP(I,P,TAUR,HOPF0,T4,T,ANE,ABROS)
         DPLOG=GRAV/ABROS*TAUR/PTOT*DLGM
         IF(ERROR.GT.1.D-4.AND.J.LT.10) GO TO 30
C
C ------ end of the iteration loop;
C        set up necessary quantities for the next depth step of the
C        hydrostatic equilibrium
C
         PLOG4=PLOG3
         PLOG3=PLOG2
         PLOG2=PLOG1
         PLOG1=PLOG
         DPLOG3=DPLOG2
         DPLOG2=DPLOG1
         DPLOG1=DPLOG
         TEMP0(I)=T
         ELEC0(I)=ANE
         AN=P/T/BOLK
         DEPTH(I)=PTOT/GRAV
         DM(I)=DEPTH(I)
         DENS0(I)=WMM(I)*(AN-ANE)
         IF(IPRING.GT.0) WRITE(6,602) I,TAU(I),DEPTH(I),T,
     *                                AN,ANE,P,ABROS
         PTOTAL(I)=PTOT+PRAD0
         PGS(I)=P
   50 CONTINUE
C
C -------------------------------------------------------------------
C
C     2. Second part - taking into account convection
C
      IF(HMIX0.GT.0.) CALL CONTMP
C
C -------------------------------------------------------------------
C
C     3. Third part
C     Interpolation of the computed model to the depth scale which is
C     going to be used in the subsequent - complete-linearization -
C     part of the model atmosphere construction
C
      ND=ND0
C
C     First option - logarithmically equidistant Rosseland opt.depths
C                    the same first and last depth as in the first part
C
      IF(IDEPTH.EQ.0) THEN
         TAU1=TAUFIR
         TAUL=TAULAS
         TAU2=TAULAS-1.
C
C     Second option - logarithmically equidistant Rosseland opt.depths
C                     the first, last-but-one, and last depths are read
C
        ELSE IF(IDEPTH.EQ.1) THEN
          READ(IBUFF,*) TAU1,TAU2,TAUL
      END IF
C
      IF(IDEPTH.LE.1) THEN
         DML0=LOG(TAU1)
         IF(TAUL.GT.0.) THEN
            DLGM=(LOG(TAU2)-DML0)/(ND-2)
            DO 80 I=1,ND-1
   80          TAU0(I)=DML0+(I-1)*DLGM
            TAU0(ND)=LOG(TAUL)
          ELSE
            DLGM=(LOG(TAU2)-DML0)/(ND-1)
            DO 81 I=1,ND
   81          TAU0(I)=DML0+(I-1)*DLGM
         END IF
       ELSE IF(IDEPTH.EQ.2) THEN
C
C     Third option - prescribed set of Rosseland optical depths
C
         READ(IBUFF,*) (TAU0(I),I=1,ND)
         DO 70 I=1,ND
   70       TAU0(I)=LOG(TAU0(I))
       ELSE
C
C     Fourth option - interpolation to the prescribed mass scale DM
C
         DO 60 I=1,ND
            DM(I)=DM0(I)
   60       DM0(I)=LOG(DM(I))
         CALL INTERP(DEPTH0,TEMP0,DM0,TEMP,NDEPTH,ND,2,0,0)
         CALL INTERP(DEPTH0,ELEC0,DM0,ELEC,NDEPTH,ND,2,0,1)
         CALL INTERP(DEPTH0,DENS0,DM0,DENS,NDEPTH,ND,2,0,1)
      END IF
C
C     in the first three options - interpolation from the previous
C     Rosseland opacity scale to the new scale and from the previous
C     mass depth scale to the new one
C
      IF(IDEPTH.LE.2) THEN
         DO 85 I=1,NDEPTH
            TEMP0(I)=TEMP(I)
            ELEC0(I)=ELEC(I)
            DENS0(I)=DENS(I)
            TAU(I)=LOG(TAUROS(I))
            DEPTH0(I)=LOG(DM(I))
   85    CONTINUE
         CALL INTERP(TAU,DEPTH0,TAU0,DM0,NDEPTH,ND,3,0,0)
         CALL INTERP(TAU,TEMP0,TAU0,TEMP,NDEPTH,ND,3,0,0)
         CALL INTERP(TAU,ELEC0,TAU0,ELEC,NDEPTH,ND,3,0,1)
         CALL INTERP(TAU,DENS0,TAU0,DENS,NDEPTH,ND,3,0,1)
         DO 90 I=1,ND
            DM(I)=EXP(DM0(I))
            PTOTAL(I)=DM(I)*GRAV+PRAD0
            PGS(I)=(DENS(I)/WMM(I)+ELEC(I))*BOLK*TEMP(I)
   90    CONTINUE
      END IF
C
C     Recalculation of the populations
C
      DO 100 ID=1,ND
         CALL WNSTOR(ID)
         CALL SABOLF(ID)
         CALL STEQEQ(ID,POP,1)
  100 CONTINUE
      IF(HMIX0.GE.0.) THEN
         WRITE(6,600)
         CALL CONOUT(2,IPRING)
      END IF
      LCHC=LCHC0
      IRSPLT=IRSPL0
  600 FORMAT(1H1,' CONVECTIVE FLUX: AT THE END OF LTEGR'/)
  601 FORMAT(1H1, 'COMPUTED LTE-GREY MODEL'//'    ID    TAU',7X,
     * 'MASS',5X,'TEMP',7X,'N',10X,'NE',9X,'P',9X,'ROSS.OP'/)
  602 FORMAT(1H ,I4,1P2D11.3,0PF8.0,1P4D11.3)
      RETURN
      END
C
C
C ********************************************************************
C
C
 
      SUBROUTINE LTEGR2
C     =================
C
C     Driving procedure for computing the initial LTE-grey model
C     atmosphere
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
C
      COMMON ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       DEPTH(MDEPTH),DEPTH0(MDEPTH),TAU(MDEPTH),TAU0(MDEPTH),
     *       TEMP0(MDEPTH),ELEC0(MDEPTH),DENS0(MDEPTH),DM0(MDEPTH)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/GRAYTS/TAUROS(MDEPTH),TAUFLX(MDEPTH)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      COMMON/LTEGRP/TAUFIR,TAULAS,ABROS0,TSURF,ALBAVE,DION0,
     *              NDGREY,IDGREY
C
C ----------------
C Input parameters
C ----------------
C
C     NDEPTH  - number of depth points for evaluating LTE-grey model
C             = 0 - NDEPTH is taken to be ND-1
C     TAUFIR  - Rosseland optical depth in the first depth point
C     TAULAS  - Rosseland optical depth in the last depth point
C     ABROS0  - an estimate of the Rosseland opacity (per gram) at the
C               first depth point
C     TSURF   = 0  - surface temperature and the Hopf function are
C                    evaluated exactly
C             > 0  - value of surface temperature is set to TSURF, and
C                    the Hopf function is assumed to be constant,
C                    corresponding to TSURF
C     ALBAVE  = 0  - wind blanketing is not considered
C             > 0  - wind blanketing is considered; the averaged value
C                    of albedo [precisely the quantity (1+rho)/(1-rho)
C                    in the notation of Hummer (Ap.J. 257, 724, 1982)
C                    see his Eq. (3.1)] is ALBAVE
C     DION0        - initial estimate of the degree of ionization at
C                    the first depth point (=1 for completely ionized;
C                    =1/2 for completely neutral)
C
C --------------------------------------------------------------------
C     IDEPTH  -  mode of determining the mass-depth scale to be used
C                in linearization
C             =  0 - depth scale DM (in g*cm**-2) is evaluated as mass
C                    corresponding to Rosseland optical depths which
C                    are equidistantly spaced in logarithms between
C                    the first point TAUFIR and the last point TAULAS
C                    the last-but-one point is, however, set to
C                    TAULAS-1.
C             =  2 - depth scale DM is evaluated as that corresponding
C                    to input values of Rosseland optical depth -
C                    array TAU0(ID), ID=1,ND
C             =  1 - similar, but now DM is evaluated as mass
C                    corresponding to Rosseland optical depths which
C                    are equidistantly spaced in logarithms between
C                    the first point TAU1 and the last-but-one point
C                    TAU2; the last point is TAUL
C                    (i.e. similar to option 0, now TAU1 and TAUL may
C                    be different from TAUFIR nad TAULAS)
C             =  3 - depth scale DM has already been read in START
C     NCONIT       - number of internal iterations for calculating the
C                    gray model with convection
C             =  0 - and HMIX0>0, then NCONIT is set to 10
C     IPRING       - controls diagnostic output of the LTE-gray model
C                    calculations
C              = 0 - no output
C              = 1 - only final LTE-gray model
C              = 2 - results of all internal iterations
C     IHM     >  0 - negative hydrogen ion considered in particle and
C                    charge conservation in ELDENS
C     IH2     >  0 - hydrogen molecule considered in particle
C                    conservation in ELDENS
C     IH2P    >  0 - ionized hydrogen molecule considered in particle
C                    and charge conservation in ELDENS
C
      IF(NDGREY.EQ.0) THEN
         NDEPTH=ND
       ELSE
         NDEPTH=NDGREY
      END IF
      IF(NDEPTH.GT.MDEPTH) call stop(' NDEPTH too large in LTEGR')
      IDEPTH=IDGREY
      IF(ALBAVE.GT.0.AND.TSURF.EQ.0.) TSURF=(0.433*ALBAVE)**0.25
      HOPF0=0.
      IF(TSURF.NE.0.) HOPF0=4.D0*TSURF**4/3.D0
      T4=TEFF**4
      ANEREL=(DION0-HALF )/DION0
      IF(NCONIT.EQ.0.AND.HMIX0.GT.0.) NCONIT=10
      IF(ANEREL.LT.1.D-3) ANEREL=1.D-3
      LCHC0=LCHC
      LCHC=.TRUE.
      IRSPL0=IRSPLT
      IRSPLT=0
      IF(NDEPTH.EQ.0) NDEPTH=ND-1
      ND0=ND
      ND=NDEPTH
      DO 1 I=1,ND0
    1    DM0(I)=DM(I)
C
C     tau(ross) scale - logarithmically equidistant points between
C     input TAUFIR and TAULAS
C
      DML0=LOG(TAUFIR)
      DLGM=(LOG(TAULAS)-DML0)/(NDEPTH-1)
      DO 10 I=1,NDEPTH
         TAU0(I)=DML0+(I-1)*DLGM
         TAU(I)=EXP(TAU0(I))
         TAUROS(I)=TAU(I)
   10 CONTINUE
C
      DPRAD=1.891204931D-15*T4
      PRAD0=DPRAD/1.732D0
      ABROS=ABROS0
      PLOG1=0.
      PLOG2=0.
      PLOG3=0.
      DPLOG1=0.
      DPLOG2=0.
      IF(IPRING.GT.0) WRITE(6,601)
C
C -------------------------------------------------------------------
C
C     1.part
C     Integration of the hydrostatic equilibrium equation on the
C     tau(ross) scale;
C     basically by a predictor-corrector method
C     (similar to Kurucz's ATLAS code)
C
      DO 50 I=1,NDEPTH
         J=0
         TAUR=TAU(I)
C
C        predictor step
C
         IF(I.EQ.1) PLOG=LOG(GRAV/ABROS*TAUR)
         IF(I.GT.1.AND.I.LE.4) PLOG=PLOG1+DPLOG1
         IF(I.GT.4) PLOG=(3.*PLOG4+8.*DPLOG1-4.*DPLOG2+8.*DPLOG3)/3.
         ERROR=1.
         GO TO 40
C
C        corrector step
C ------ iterate between hydrostatic equilibrium (which determines an
C           increment of the total pressure) and state equations (which
C           determine relevant number densities and then the Rosseland
C           opacity)
C
   30    IF(I.EQ.1) PNEW=LOG(GRAV/ABROS*TAUR)
         IF(I.GT.1.AND.I.LE.4)  PNEW=(PLOG+2.*PLOG1+DPLOG+DPLOG1)/3.
         IF(I.GT.4) PNEW=(126.*PLOG1-14.*PLOG3+9.*PLOG4+42.*DPLOG+
     *                    108.*DPLOG1-54.*DPLOG2+24.*DPLOG3)/121.
         ERROR=ABS(PNEW-PLOG)
         PLOG=PNEW
   40    PTOT=EXP(PLOG)
         P=PTOT-TAUR*DPRAD
         J=J+1
         CALL ROSSOP(I,P,TAUR,HOPF0,T4,T,ANE,ABROS)
         DPLOG=GRAV/ABROS*TAUR/PTOT*DLGM
         IF(ERROR.GT.1.D-4.AND.J.LT.10) GO TO 30
C
C ------ end of the iteration loop;
C        set up necessary quantities for the next depth step of the
C        hydrostatic equilibrium
C
         PLOG4=PLOG3
         PLOG3=PLOG2
         PLOG2=PLOG1
         PLOG1=PLOG
         DPLOG3=DPLOG2
         DPLOG2=DPLOG1
         DPLOG1=DPLOG
         TEMP0(I)=T
         ELEC0(I)=ANE
         AN=P/T/BOLK
         DEPTH(I)=PTOT/GRAV
         DM(I)=DEPTH(I)
         DENS0(I)=WMM(I)*(AN-ANE)
         IF(IPRING.GT.0) WRITE(6,602) I,TAU(I),DEPTH(I),T,
     *                                AN,ANE,P,ABROS
         PTOTAL(I)=PTOT+PRAD0
         PGS(I)=P
   50 CONTINUE
C
C -------------------------------------------------------------------
C
C     2. Second part - taking into account convection
C
      IF(HMIX0.GT.0.) CALL CONTMP
C
C -------------------------------------------------------------------
C
C     3. Third part
C     Interpolation of the computed model to the depth scale which is
C     going to be used in the subsequent - complete-linearization -
C     part of the model atmosphere construction
C
      ND=ND0
C
C     First option - logarithmically equidistant Rosseland opt.depths
C                    the same first and last depth as in the first part
C
      IF(IDEPTH.EQ.0) THEN
         TAU1=TAUFIR
         TAUL=TAULAS
         TAU2=TAULAS-1.
C
C     Second option - logarithmically equidistant Rosseland opt.depths
C                     the first, last-but-one, and last depths are read
C
        ELSE IF(IDEPTH.EQ.1) THEN
          READ(IBUFF,*) TAU1,TAU2,TAUL
      END IF
C
      IF(IDEPTH.LE.1) THEN
         DML0=LOG(TAU1)
         IF(TAUL.GT.0.) THEN
            DLGM=(LOG(TAU2)-DML0)/(ND-2)
            DO 80 I=1,ND-1
   80          TAU0(I)=DML0+(I-1)*DLGM
            TAU0(ND)=LOG(TAUL)
          ELSE
            DLGM=(LOG(TAU2)-DML0)/(ND-1)
            DO 81 I=1,ND
   81          TAU0(I)=DML0+(I-1)*DLGM
         END IF
       ELSE IF(IDEPTH.EQ.2) THEN
C
C     Third option - prescribed set of Rosseland optical depths
C
         READ(IBUFF,*) (TAU0(I),I=1,ND)
         DO 70 I=1,ND
   70       TAU0(I)=LOG(TAU0(I))
       ELSE
C
C     Fourth option - interpolation to the prescribed mass scale DM
C
         DO 60 I=1,ND
            DM(I)=DM0(I)
   60       DM0(I)=LOG(DM(I))
         CALL INTERP(DEPTH0,TEMP0,DM0,TEMP,NDEPTH,ND,2,0,0)
         CALL INTERP(DEPTH0,ELEC0,DM0,ELEC,NDEPTH,ND,2,0,1)
         CALL INTERP(DEPTH0,DENS0,DM0,DENS,NDEPTH,ND,2,0,1)
      END IF
C
C     in the first three options - interpolation from the previous
C     Rosseland opacity scale to the new scale and from the previous
C     mass depth scale to the new one
C
      IF(IDEPTH.LE.2) THEN
         DO 85 I=1,NDEPTH
            TEMP0(I)=TEMP(I)
            ELEC0(I)=ELEC(I)
            DENS0(I)=DENS(I)
            TAU(I)=LOG(TAUROS(I))
            DEPTH0(I)=LOG(DM(I))
   85    CONTINUE
         CALL INTERP(TAU,DEPTH0,TAU0,DM0,NDEPTH,ND,3,0,0)
         CALL INTERP(TAU,TEMP0,TAU0,TEMP,NDEPTH,ND,3,0,0)
         CALL INTERP(TAU,ELEC0,TAU0,ELEC,NDEPTH,ND,3,0,1)
         CALL INTERP(TAU,DENS0,TAU0,DENS,NDEPTH,ND,3,0,1)
         DO 90 I=1,ND
            DM(I)=EXP(DM0(I))
            PTOTAL(I)=DM(I)*GRAV+PRAD0
            PGS(I)=(DENS(I)/WMM(I)+ELEC(I))*BOLK*TEMP(I)
   90    CONTINUE
      END IF
C
C     Recalculation of the populations
C
      DO 100 ID=1,ND
         CALL WNSTOR(ID)
         CALL SABOLF(ID)
         CALL STEQEQ(ID,POP,1)
  100 CONTINUE
      IF(HMIX0.GE.0.) THEN
         WRITE(6,600)
         CALL CONOUT(2,IPRING)
      END IF
      LCHC=LCHC0
      IRSPLT=IRSPL0
  600 FORMAT(1H1,' CONVECTIVE FLUX: AT THE END OF LTEGR'/)
  601 FORMAT(1H1, 'COMPUTED LTE-GREY MODEL'//'    ID    TAU',7X,
     * 'MASS',5X,'TEMP',7X,'N',10X,'NE',9X,'P',9X,'ROSS.OP'/)
  602 FORMAT(1H ,I4,1P2D11.3,0PF8.0,1P4D11.3)
      RETURN
      END
C
C
C ********************************************************************
C
C

      SUBROUTINE ROSSOP(ID,P,TAUR,HOPF,T4,T,ANE,ABROSS)
C     =================================================
C
C     Auxiliary procedure for LTEGR
C     Evaluation of temperature, electron density, and Rosseland
C     opacity for a given TAUR (Rosseland optical depth) and P (total
C     pressure)
C
C     Input parameters:
C     ID     - depth index
C     P      - total pressure
C     TAUR   - Rosseland optical depth
C     HOPF   - mode of evaluating Hopf function;
C            = 0  -  exact Hopf function
C            > 0  -  constant Hopf function to HOPF
C     T4     = effective temperature ** 4
C
C     Output:
C     T      - temperature
C     ANE    - electron density
C     ABROSS - Rosseland opacity (per gram)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      DIMENSION A(5)
      COMMON/OPMEAN/ABROSD(MDEPTH),ABPLAD(MDEPTH)
      COMMON/EXTOTR/EXTOT
C
      DATA A/0.71044609D0,-0.2830385D0,0.57975839D0,-0.75751038D0,
     *       0.45026781D0/
      SAVE A
C
C     Hopf function
C
      X=HOPF
      IF(X.GT.0.) GO TO 20
      X=A(1)
      IF(TAUR.GT.160.) GO TO 20
      EX=EXP(-TAUR)
      E1=EXPINT(TAUR)
      E=E1
      DO 10 I=1,4
         E=(EX-TAUR*E)/I
   10    X=X+E*A(I+1)
C
C     Temperature
C
   20 T=(0.75*T4*(TAUR+X)+EXTOT)**0.25
C
C     Determination of electron density from the total pressure
C
      AN=P/T/BOLK
      CALL ELDENS(ID,T,AN,ANE,ENRG)
      RHO=WMM(ID)*(AN-ANE)
      DENS(ID)=RHO
C
C     temperature and electron density are transmitted to SABOLF
C     and RATMAT through arrays TEMP and ELEC
C
      TEMP(ID)=T
      ELEC(ID)=ANE
C
C     Corresponding LTE populations
C
      CALL WNSTOR(ID)
      CALL SABOLF(ID)
      CALL STEQEQ(ID,POP,1)
C
C     Finally, evaluation of the Rosseland opacity for the new values
C     of temperature, electron density, and populations
C     (ROSS - Rosseland opacity per 1 cm**3)
C
      CALL OPACF0(ID,NFREQ)
      CALL MEANOP(T,ANE,ABSO,SCAT,OPROS,OPPLA)
      RHO=WMM(ID)*(AN-ANE)
      DENS(ID)=RHO
      ABROSS=OPROS/RHO
      ABROSD(ID)=ABROSS
      ABPLAD(ID)=OPPLA/RHO
      RETURN
      END
C
C
C     ****************************************************************
C
C
 
      SUBROUTINE CONTMP
C     =================
C
C     Auxiliary procedure for LTEGR
C     Determination of temperature in convectively unstable layers
C     This is done by solving the energy balance equation
C     F(rad)+F(conv)=F(mech), which yields a cubic equation for
C     the logarithmic temperature gradient
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      COMMON ESEMAT(MLEVEL,MLEVEL),BESE(MLEVEL),
     *       DEPTH(MDEPTH),DEPTH0(MDEPTH),TAU(MDEPTH),TAU0(MDEPTH),
     *       TEMP0(MDEPTH),ELEC0(MDEPTH),DENS0(MDEPTH),DM0(MDEPTH)
      DIMENSION DELTR(MDEPTH),TEMPR(MDEPTH),ICON0(MDEPTH)
      COMMON/GRAYTS/TAUROS(MDEPTH),TAUFLX(MDEPTH)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/OPMEAN/ABROSD(MDEPTH),ABPLAD(MDEPTH)
      COMMON/HYDMOL/PHMOL(MDEPTH)
      COMMON/CUBCON/A,B,DEL,GRDADB,DELMDE,RHO,FLXTOT
      COMMON/ELDNSP/ANP,AHTOT,AHMOL
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      PARAMETER  (ERRT=1.D-3)
C
C     First, store the temperature(rad) and gradient Delta(rad) -
C     quantities for the  purely raditive equilibrium model
C
      T4=TEFF**4
      FLXTOT=SIG4P*T4
      DPRAD=1.891204931D-15*T4
      PRAD0=DPRAD/1.732D0
      DO 10 ID=1,ND
         TEMPR(ID)=TEMP(ID)
         IF(ID.EQ.1) THEN
            DELTR(ID)=0.
          ELSE
            DELTR(ID)=
     *      (TEMP(ID)-TEMP(ID-1))/(PTOTAL(ID)-PTOTAL(ID-1))*
     *      (PTOTAL(ID)+PTOTAL(ID-1))/(TEMP(ID)+TEMP(ID-1))
         END IF
   10 CONTINUE
      ICONIT=0
C
C     ------------------------------------------------------
C     Global iteration loop for calculating convective model
C     ------------------------------------------------------
C
   20 ICONIT=ICONIT+1
      ICONBE=0
      DO 50 ID=1,ND
         T=TEMP(ID)
         PTOT=PTOTAL(ID)
         PGAS=PGS(ID)
         PTURB=HALF*DENS(ID)*VTURB(ID)*VTURB(ID)
         PRAD=PTOT-PGAS-PTURB
         PRADT(ID)=PRAD
         ICON0(ID)=0
C
         IF(ID.EQ.1) GO TO 40
         J=0
         IF(ICONIT.EQ.1) T=T-TEMPR(ID-1)+TEMP(ID-1)
         TM=TEMP(ID-1)
         IF(T.LT.0.) T=TM
         PTOTM=PTOTAL(ID-1)
         PT0=HALF*(PTOT+PTOTM)
         DELR=DELTR(ID)
C
C        Inner iteration loop for determining temperature in the
C        conectively unstable layers
C
   30    J=J+1
         TOLD=T
         T0=HALF*(T+TM)
         PG0=HALF*(PGAS+PGM)
         PR0=HALF*(PRAD+PRADM)
         IF(ID.GE.ND-2.AND.ICONBE.EQ.0) GO TO 40
         CALL CONVEC(ID,T0,PT0,PG0,PR0,ABROSD(ID),DELR,FLXC,VCON)
         IF(FLXC.EQ.0.) GO TO 40
         ICON0(ID)=1
         ICONBE=1
         CALL CUBIC(DELTA)
         REFF=DELTA/DELR
         IF(REFF.GT.UN) REFF=UN
         IF(REFF.LT.0.) REFF=0.
         PRAD=PRADM+(TAUROS(ID)-TAUROS(ID-1))*DPRAD*REFF
         PRADT(ID)=PRAD
         PGAS=PTOT-PRAD-PTURB
         FAC=DELTA*(PTOT-PTOTM)/(PTOT+PTOTM)
         T=TM*(UN+FAC)/(UN-FAC)
         IF(T.LT.TM) T=TM
         IF(ABS(UN-T/TOLD).GT.ERRT.AND.J.LT.10) GO TO 30
C
C     Store the final quantitites
C
   40    IF(ID.GT.1.AND.ICON0(ID).EQ.0.AND.ICON0(ID-1).EQ.1)
     *      DELTC=DELT0
         DELT0=TEMP(ID)-T
         TEMP(ID)=T
         IF(ICONIT.GT.1.AND.ICON0(ID).EQ.0.AND.ICONBE.EQ.1)
     *      TEMP(ID)=T-DELTC
         PGM=PGAS
         PRADM=PRAD
         PGS(ID)=PGAS
   50 CONTINUE
C
C     Diagnostic outprint
C
      IF(IPRING.EQ.2) THEN
         WRITE(6,600) ICONIT
         CALL CONOUT(1,IPRING)
      END IF
  600 FORMAT(1H1,' CONVECTIVE FLUX: AT CONTMP, ITER=',I2/)
C
C     2. New values of electron density, density, sound spped,
C        and mean opacities and optical depths
C
      ANEREL=ELEC(1)/(DENS(1)/WMM(1)+ELEC(1))
      DO 70 ID=1,ND
         T=TEMP(ID)
         ITINT=0
   60    ITINT=ITINT+1
         AN=PGS(ID)/T/BOLK
         CALL ELDENS(ID,T,AN,ANE,ENRG)
         ELEC(ID)=ANE
         DENS(ID)=WMM(ID)*(AN-ANE)
         PHMOL(ID)=AHMOL
C
C     Corresponding LTE populations
C
         CALL WNSTOR(ID)
         CALL SABOLF(ID)
         CALL STEQEQ(ID,POP,1)
C
C        Evaluation of the Rosseland and Planck mean opacities
C
         CALL OPACF0(ID,NFREQ)
         CALL MEANOP(T,ANE,ABSO,SCAT,OPROS,OPPLA)
         ABROS=OPROS/DENS(ID)
         ABPLA=OPPLA/DENS(ID)
C
C        New values of the Rosseland opacity
C
         PTOLD=PTOTAL(ID)
         IF(ID.EQ.1) THEN
            DM(ID)=TAUROS(ID)/ABROS
            PTOTAL(ID)=DM(ID)*GRAV+PRAD0
          ELSE
            DM(ID)=DM(ID-1)+(TAUROS(ID)-TAUROS(ID-1))/
     *             HALF/(ABROSD(ID-1)+ABROS)
            PTOTAL(ID)=DM(ID)*GRAV+PRAD0
            DELTR(ID)=
     *      (TEMPR(ID)-TEMPR(ID-1))/(PTOTAL(ID)-PTOTAL(ID-1))*
     *      (PTOTAL(ID)+PTOTAL(ID-1))/(TEMPR(ID)+TEMPR(ID-1))
         END IF
C
C        Store the final quantitites
C
         PTURB=HALF*DENS(ID)*VTURB(ID)*VTURB(ID)
         PGS(ID)=PTOTAL(ID)-PRADT(ID)-PTURB
         ABROSD(ID)=ABROS
         ABPLAD(ID)=ABPLA
         IF((PTOTAL(ID)-PTOLD)/PTOLD.LT.1.D-3) GO TO 70
         IF(ITINT.GT.5) THEN
            WRITE(6,601) ID,PTOLD,PTOTAL(ID)
            GO TO 70
          ELSE
            GO TO 60
         END IF
   70 CONTINUE
      IF(ICONIT.LT.NCONIT) GO TO 20
  601 FORMAT(1H0,'SLOW CONVERGENCE OF INTERNAL ITERATIONS IN',
     *       ' CONTMP: ID, PTOT(OLD), PTOT(NEW) ='/I3,1P2D10.2/)
      RETURN
      END
C
C
C     ****************************************************************
C
C
 
      SUBROUTINE ELDENS(ID,T,AN,ANE,ENERG)
C     ====================================
C
C     Evaluation of the electron density and the total hydrogen
C     number density for a given total particle number density
C     and temperature;
C     by solving the set of Saha equations, charge conservation and
C     particle conservation equations (by a Newton-Raphson method)
C
C     Input parameters:
C     T    - temperature
C     AN   - total particle number density
C
C     Output:
C     ANE   - electron density
C     ANP   - proton number density
C     AHTOT - total hydrogen number density
C     AHMOL - relativer number of hydrogen molecules with respect to the
C             total number of hydrogens
C     ENERG - part of the internal energy: excitation and ionization
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ATOMIC.FOR'
      DIMENSION R(3,3),S(3),P(3)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
      COMMON/STATEP/Q,QM,DQT,DQN,DQM,ENER,QREF,DQTR,DQNR
      COMMON/ELDNSP/ANP,AHTOT,AHMOL
C
      QM=0.
      Q2=0.
      QP=0.
      Q=0.
      DQN=0.
      TK=BOLK*T
      THET=5.0404D3/T
C
C     Coefficients entering ionization (dissociation) balance of:
C     atomic hydrogen          - QH;
C     negative hydrogen ion    - QM   (considered only if IHM>0);
C     hydrogen molecule        - QP   (considered only if IH2>0);
C     ion of hydrogen molecule - Q2   (considered only if IH2P>0).
C
      IF(IATREF.EQ.IATH) THEN
      IF(IHM.GT.0)  QM=1.0353D-16/T/SQRT(T)*EXP(8762.9/T)
      IF(IH2P.GT.0) QP=TK*EXP((-11.206998+THET*(2.7942767+THET*
     *             (0.079196803-0.024790744*THET)))*2.30258509299405)
      IF(IH2.GT.0) Q2=TK*EXP((-12.533505+THET*(4.9251644+THET*
     *            (-0.056191273+0.0032687661*THET)))*2.30258509299405)
      QH=EXP((15.38287+1.5*LOG10(T)-13.595*THET)*2.30258509299405)
      END IF
C
C     Initial estimate of the electron density
C
      ANE=AN*ANEREL
      IT=0
C
C     Basic Newton-Raphson loop - solution of the non-linear set
C     for the unknown vector P, consistiong of AH, ANH (neutral
C     hydrogen number density) and ANE.
C
   10 IT=IT+1
C
C     procedure STATE determines Q (and DQN) - the total charge (and its
C     derivative wrt temperature) due to ionization of all atoms which
C     are considered (both explicit and non-explicit), by solving the set
C     of Saha equations for the current values of T and ANE
C
      CALL STATE(1,ID,T,ANE)
C
C     Auxiliary parameters for evaluating the elements of matrix of
C     linearized equations.
C     Note that complexity of the matrix depends on whether the hydrogen
C     molecule is taken into account
C     Treatment of hydrogen ionization-dissociation is based on
C     Mihalas, in Methods in Comput. Phys. 7, p.10 (1967)
C
      IF(IATREF.EQ.IATH) THEN
      G2=QH/ANE
      G3=0.
      G4=0.
      G5=0.
      D=0.
      E=0.
      G3=QM*ANE
      A=UN+G2+G3
      D=G2-G3
      IF(IT.GT.1) GO TO 60
      IF(IH2.EQ.0.AND.IH2P.EQ.0) GO TO 40
      IF(IH2.EQ.0) GO TO 20
      E=G2*QP/Q2
      B=TWO*(UN+E)
      GG=ANE*Q2
      GO TO 30
   20 B=TWO
      E=UN
      GG=G2*ANE*QP
   30 C1=B*(GG*B+A*D)-E*A*A
      C2=A*(TWO*E+B*Q)-D*B
      C3=-E-B*Q
      F1=(SQRT(C2*C2-4.*C1*C3)-C2)*HALF/C1
      FE=F1*D+E*(UN-A*F1)/B+Q
      GO TO 50
   40 F1=UN/A
      FE=D/A+Q
   50 AH=ANE/FE
      ANH=AH*F1
   60 AE=ANH/ANE
      GG=AE*QP
      E=ANH*Q2
      B=ANH*QM
C
C     Matrix of the linearized system R, and the rhs vector S
C
      R(1,1)=YTOT(ID)
      R(1,2)=0.
      R(1,3)=UN
      R(2,1)=-Q
      R(2,2)=-D-TWO*GG
      R(2,3)=UN+B+AE*(G2+GG)-DQN*AH
      R(3,1)=-UN
      R(3,2)=A+4.*(E+GG)
      R(3,3)=B-AE*(G2+TWO*GG)
      S(1)=AN-ANE-YTOT(ID)*AH
      S(2)=ANH*(D+GG)+Q*AH-ANE
      S(3)=AH-ANH*(A+TWO*(E+GG))
C
C     Solution of the linearized equations for the correction vector P
C
      CALL LINEQS(R,S,P,3,3)
C
C     New values of AH, ANH, and ANE
C
      AH=AH+P(1)
      ANH=ANH+P(2)
      DELNE=P(3)
      ANE=ANE+DELNE
C
C     hydrogen is not the reference atom
C
      ELSE
C
C     Matrix of the linearized system R, and the rhs vector S
C
      IF(IT.EQ.1) THEN
         ANE=AN*HALF
         AH=ANE/YTOT(ID)
      END IF
      R(1,1)=YTOT(ID)
      R(1,2)=UN
      R(2,1)=-Q-QREF
      R(2,2)=UN-(DQN+DQNR)*AH
      S(1)=AN-ANE-YTOT(ID)*AH
      S(2)=(Q+QREF)*AH-ANE
C
C     Solution of the linearized equations for the correction vector P
C
      CALL LINEQS(R,S,P,2,3)
      AH=AH+P(1)
      DELNE=P(2)
      ANE=ANE+DELNE
      END IF
C
C     Convergence criterion
C
      IF(ANE.LE.0.) ANE=1.D-3*AN
      IF(ABS(DELNE/ANE).GT.1.D-3.AND.IT.LE.10) GO TO 10
C
C     ANEREL is the exact ratio betwen electron density and total
C     particle density, which is going to be used in the subseguent
C     call of ELDENS
C
      ANEREL=ANE/AN
      AHTOT=AH
      IF(IATREF.EQ.IATH) THEN
      AHMOL=TWO*ANH*(ANH*Q2+ANH/ANE*QP)/AH
      ANP=ANH/ANE*QH
      END IF
C
C     evaluation of ENERG
C
c      ENERG=ENER*AH*TK
      TKEV=8.617D-5*T
      ENERG=13.595/TKEV*ANP*TK
      if(iatref.eq.iathe) ENERG=24.580/TKEV*ANE*TK
      RETURN
      END
C
C
C ***********************************************************************
C
C
 
      SUBROUTINE MEANOP(T,ANE,ABSO,SCAT,OPROS,OPPLA)
C     ==============================================
C
C     Rosseland and Planck mean opacities
C
C     Input parameters:
C      T    - temperature
C      ANE  - electron density
C      ABSO - array of absorption coefficients in all explicit
C             frequency points
C      SCAT - array of scttering coefficients
C     Output:
C      OPROS - Rosseland opacity (per 1 cm**3)
C      OPPLA - Planck mean opacity (per 1 cm**3)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ATOMIC.FOR'
      DIMENSION ABSO(MFREQ),SCAT(MFREQ)
C        
      ABR=0.
      SUMDB=0.
      ABP=0.
      SUMB=0.
      HKT=HK/T
C          
C     1. first part - integration over continuum frequencies
C         
      DO 10 IJ=1,NFREQC
         FR=FREQ(IJ)
         EX=EXP(HKT*FR)
         E1=UN/(EX-UN)
         PLAN=BNUE(IJ)*E1*(W(IJ)-WSUB(IJ))
         DPLAN=PLAN*HKT*FR*EX*E1
         ABR=ABR+DPLAN/ABSO(IJ)
         ABP=ABP+PLAN*(ABSO(IJ)-SCAT(IJ))
         SUMDB=SUMDB+DPLAN
         SUMB=SUMB+PLAN
   10 CONTINUE
C         
C     2. first part - correction from overlapping lines
C            
      DO 50 ITRF=1,NTRANS
         IF(INDEXP(ITRF).EQ.0) GO TO 50
         IF(.NOT.LINE(ITRF).OR.ISUB(ITRF).EQ.0) GO TO 50
         DO 20 IJ=IFR0(ITRF),IFR1(ITRF)
            FR=FREQ(IJ)
            EX=EXP(HKT*FR)
            E1=UN/(EX-UN)
            PLAN=BNUE(IJ)*E1*(W(IJ)-WSUB(IJ))
            DPLAN=PLAN*HKT*FR*EX*E1
            ABR=ABR+DPLAN/ABSO(IJ)
            ABP=ABP+PLAN*(ABSO(IJ)-SCAT(IJ))
            SUMDB=SUMDB+DPLAN
            SUMB=SUMB+PLAN
   20    CONTINUE
C        --- correction (begin)
         IF(ISUB(ITRF).EQ.0.OR.IOVER.GT.0) GO TO 50
         DO 30 IJ=IFRC0(ITRF),IFRC1(ITRF)
            FR=FREQ(IJ)
            EX=EXP(HKT*FR)
            E1=UN/(EX-UN)
            PLAN=-BNUE(IJ)*E1*WCOR(IJ-IFRC0(ITRF)+1,ITRF)
            DPLAN=PLAN*HKT*FR*EX*E1
            ABR=ABR+DPLAN/ABSO(IJ)
            ABP=ABP+PLAN*(ABSO(IJ)-SCAT(IJ))
            SUMDB=SUMDB+DPLAN
            SUMB=SUMB+PLAN
   30    CONTINUE
C        --- correction (end)
   50 CONTINUE
      OPROS=SUMDB/ABR
      OPPLA=ABP/SUMB
      RETURN
      END
 
C
C
C     ****************************************************************
C
C
 
      SUBROUTINE CONVEC(ID,T,PTOT,PG,PRAD,ABROS,DELTA,FLXCNV,VCONV)
C     =============================================================
C
C     Determination of the mixing-lengths convective flux
C
C     Input:  T     - temperature
C             PTOT  - total pressure
C             PG    - gas pressure
C             PRAD  - radiation pressure
C             ABROS - Rosseland opacity (per gram)
C             DELTA - corresponding temperature gradient
C     Output: FLXCNV - convective flux (expressed as H, ie F/4/pi)
C             VCONV  - convective velocity
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      COMMON/CUBCON/A,B,DDEL,GRDADB,DLT,RHO,FLXTOT
C
      VCONV=0.
      FLXCNV=0.
      DLT=0.
      IF(HMIX0.LT.0.) RETURN
 
C     Thermodynamic derivatives
C
      CALL TRMDER(ID,T,PG,PRAD,DEDT,DRDT,DEDPG,DRDPG,RHO)
      DPDPG=1.
      DPDT=4.*PRAD/T
      HEATCV=DEDT-DEDPG*DRDT/DRDPG
      HEATCP=DEDT-DEDPG*DPDT/DPDPG-PTOT/RHO/RHO*(DRDT-DRDPG*DPDT/DPDPG)
      DLRDLT=T/RHO*(DRDT-DRDPG*DPDT/DPDPG)
      GRDADB=-PTOT/RHO/T*DLRDLT/HEATCP
      HSCALE=PTOT/RHO/GRAV
c     IF(HMIX0.EQ.0) GO TO 100
      DDEL=DELTA-GRDADB
C
C     Convective instability criterion
C
      IF(DDEL.LT.0.) GO TO 100
      HMIX=HMIX0
      if(hmix0.eq.0.) hmix=1.
      VCO=HALF *HMIX*SQRT(ABS(HALF *PTOT/RHO*DLRDLT))
      FLCO=HALF *RHO*HEATCP*T*HMIX/12.5664
      TAUE=HMIX*ABROS*RHO*HSCALE
      FAC=TAUE/(UN+HALF *TAUE*TAUE)
C
C     Set up parameters A and B (see Mihalas, Eq. 7-76, 7-79, etc)
C
      B=1.804D-5*T**4*HMIX/FLCO/VCO*FAC
      IF(FLXTOT.GT.0.) A=FLCO*VCO/FLXTOT*DELTA
C
C     Determination of   Delta - Delta(E)
C
      D=B*B/2.D0
      DLT=D+DDEL-B*SQRT(D/2.D0+DDEL)
      IF(DLT.LT.0.) DLT=0.
C
C     Resulting convective velocity VCONV and flux FLXCNV
C
      VCONV=VCO*SQRT(DLT)
      FLXCNV=FLCO*VCONV*DLT
  100 CONTINUE
      RETURN
      END
C
C
C     ***************************************************************
C
C
 
      SUBROUTINE TRMDER(ID,T,PG,PRAD,DEDT,DRDT,DEDPG,DRDPG,RHO)
C     =========================================================
C
C     Thermodynamic derivatives
C     Evaluation similar as in Kurucz's ATLAS
C
C     Input:  T     -  temperature
C             PG    -  gas pressure
C             PRAD  -  radiation pressure
C     Output: DEDT  -  d(energy)/d(T)
C             DRDT  -  d(rho)/d(T)
C             DEDPG -  d(energy)/d(PG)
C             DRDPG -  d(rho)/d(PG)
C             RHO   -  density
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      DIMENSION T1(3),P1(3),RHON(3),ENERGY(3)
C
C   NUMERICAL EVALUATION OF THERMODYNAMIC DERIVATIVES
C
      T1(1)=T*1.001D0
      T1(2)=T
      T1(3)=T
      P1(1)=PG
      P1(2)=PG
      P1(3)=PG*1.001D0
      DO 10 I=1,3
         TT=T1(I)
         TKN=TT*1.38054D-16
         ANT=P1(I)/TKN
         CALL ELDENS(ID,TT,ANT,ANE,ENER)
         RHON(I)=WMM(ID)*(ANT-ANE)
         ENERGY(I)=(1.5D0*P1(I)+ENER+3.D0*PRAD*(TT/T)**4)/RHON(I)
   10 CONTINUE
      DEDT=(ENERGY(1)-ENERGY(2))/T*1.D3
      DRDT=(RHON(1)-RHON(2))/T*1.D3
      DEDPG=(ENERGY(3)-ENERGY(2))/PG*1.D3
      DRDPG=(RHON(3)-RHON(2))/PG*1.D3
      RHO=RHON(2)
      RETURN
      END
C
C
C     ***************************************************************
C
C
 
      SUBROUTINE CUBIC(DELTA)
C     =======================
C
C     Solution of the cubic equation for determination of
C     the true gradient DELTA
C
C     Input:   A,B   - coefficients; transmitted by COMMON/CUBCON
C              DEL   - DELTA(RAD) - DELTA(ADIAB); also transm. by CUBCON
C     Output:  DELTA - true gradient
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      COMMON/CUBCON/A,B,DEL,GRDADB,DELMDE,RHO,FLXTOT
      PARAMETER (THIRD = 0.333333333333333D0)
C
C     first solve the cubic equation
C         A*X**3 + X**2 + B*X = DEL
C     where X = (DELTA - DELTA(ELEM))**(1/2)
C
      AA=THIRD/A
      BB=B/A
      CC=-DEL/A
      P=BB*THIRD-AA*AA
      Q=AA**3-(BB*AA-CC)/2.D0
      D=Q*Q+P*P*P
      IF(D.GT.0.) THEN
         D=SQRT(D)
         IF(D.EQ.ABS(Q)) THEN
            SOL=(2.D0*D)**THIRD-AA
          ELSE
            D1=ABS(D-Q)
            D2=ABS(D+Q)
            SOL=D1/(D-Q)*D1**THIRD-D2/(D+Q)*D2**THIRD-AA
          END IF
       ELSE
         COSF=-Q/SQRT(ABS(P*P*P))
         TANF=SQRT(UN-COSF*COSF)/COSF
         FI=ATAN(TANF)*THIRD
         SOL=2.D0*SQRT(ABS(P))*COS(FI)-AA
      END IF
C
C     if the previous formalism gives an unphysical solution
C     x > DEL, then find the physical solution in the range (0, DEL)
C     by a Newton-Raphson solution of the cubic equation
C
      IF(SOL.GT.DEL.OR.SOL.LT.0.) THEN
	 X0=DEL/B
	 J=0
   10    DELX=(DEL-X0*(B+X0+A*X0*X0))/(3.D0*A*X0*X0+2.D0*X0+B)
	 X0=X0+DELX
	 J=J+1
	 IF(ABS(DELX/X0).GT.1D-6.AND.J.LT.10) GO TO 10
	 SOL=X0
      END IF
C
C     finally, the actual gradient delta
C
      DELTA=GRDADB+B*SOL+SOL*SOL
      RETURN
      END
C
C
C     ***************************************************************
C
C
 
      SUBROUTINE CONOUT(IMOD,IPRIN)
C     =============================
C
C     Diagnostic outprint of temperature gradients, convective flux,
C     and their derivatives
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      COMMON/CUBCON/A,B,DEL,GRDADB,DELMDE,RHO,FLXTOT
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/GRAYTS/TAUROS(MDEPTH),TAUFLX(MDEPTH)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/OPMEAN/ABROSD(MDEPTH),ABPLAD(MDEPTH)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
      COMMON/HYDMOL/PHMOL(MDEPTH)
C
      IF(IPRIN.GT.0) WRITE(6,600)
      ANEREL=ELEC(1)/(DENS(1)/WMM(1)+ELEC(1))
      ICBEG=0
      DO 10 ID=1,ND
         T=TEMP(ID)
         PTOT=PTOTAL(ID)
         PG=PGS(ID)
         PRAD=PTOT-PG-HALF*DENS(ID)*VTURB(ID)**2
         IF(IMOD.EQ.2) THEN
            CALL OPACF0(ID,NFREQ)
            CALL MEANOP(T,ELEC(ID),ABSO,SCAT,OPROS,OPPLA)
            ABROSD(ID)=OPROS/DENS(ID)
         END IF
         FLXTOT=SIG4P*TEFF**4
         IF(ID.EQ.1) THEN
            TAU=DM(ID)*ABROSD(ID)
            FLXCR=0.
            GRDADB=0.
            DELTA(ID)=0.
            FLXC(ID)=0.
            FLXCNV=0.
          ELSE
            TM=TEMP(ID-1)
            TAU=TAUM+HALF*(DM(ID)-DM(ID-1))*(ABROSD(ID)+ABROSD(ID-1))
            PTOTM=PTOTAL(ID-1)
            PGM=PGS(ID-1)
            PRADM=PTOTM-PGM-HALF*DENS(ID-1)*VTURB(ID-1)**2
            T0=HALF*(T+TM)
            PT0=HALF*(PTOT+PTOTM)
            PG0=HALF*(PG+PGM)
            PR0=HALF*(PRAD+PRADM)
            DLT=(T-TM)/(PTOT-PTOTM)*PT0/T0
            DELTA(ID)=DLT
            CALL CONVEC(ID,T0,PT0,PG0,PR0,ABROSD(ID),DLT,FLXCNV,VCON)
            FLXC(ID)=FLXCNV
            IF(ICBEG.EQ.0.AND.FLXC(ID).GT.0..AND.FLXC(ID-1).EQ.0.)
     *         ICBEG=ID
         END IF
	 PRADR=PRAD/PTOT
         IF(IPRIN.GT.0) WRITE(6,601) ID,TAU,T,PTOT,PRADR,
     *                   PHMOL(ID),DELTA(ID),GRDADB,FLXCNV/FLXTOT
         TAUM=TAU
   10 CONTINUE
C
c     if ICONV=1, then:
c     in the convective zone the radiative (+ convective) equilibrium
c     is taken obligatorily in the differential form, i.e. 
c     NDRE is modified to have the value just below the beginning of
c     convection zone, and
c     rediff(id) has to be set to unity, and reint(id) to 0 for id => ndre
c
      IF(ICBEG.GT.3.AND.ICONV.EQ.1) THEN 
         NDRE=ICBEG-1
         DO 20 ID=1,ND
            IF(ID.GE.NDRE) THEN
               REINT(ID)=0.
               REDIF(ID)=1.
             else
               reint(id)=1.
               redif(id)=0.
            END IF
   20    CONTINUE
         WRITE(6,602) NDRE
      END IF
      IF(ICBEG.GT.3.AND.ICONV.GT.1) THEN 
         NDRE=ICBEG-1
         DO 30 ID=1,ND
            IF(ID.GE.NDRE) THEN
               REDIF(ID)=1.
            END IF
   30    CONTINUE
         WRITE(6,602) NDRE
      END IF
c
  600 FORMAT(1H0,'  ID',4X,'TAUR',5X,'TEMP',5X,'PTOT',3X,'PR/PTOT',
     *       3X,'HYD.MOL',4X,'DELTA',2X,'DELTA(AD)',2X,'CON/TOT'//)
  601 FORMAT(1H ,I4,1PD9.2,0PF9.1,1P3D9.2,2X,2D9.2,1X,D9.2)
  602 FORMAT(//' NDRE IS RESET IN CONOUT DUE TO THE EXISTENCE OF'
     *       ,' CONVECTIVE ZONE'/'  NDRE= ' I3/)
      RETURN
      END
C
C
C     ***************************************************************
C
C
 
      FUNCTION ERFC(X)
C     ================
C
C     complementary error function
C     expression from Abramowitz and Stegun, p.299, Eq. 7.1.26
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (P  = 0.3275911D0,
     *           A1 = 0.254829592D0,
     *           A2 =-0.284496736D0,
     *           A3 = 1.421413741D0,
     *           A4 =-1.453152027D0,
     *           A5 = 1.061405429D0,
     *           UN = 1.D0)
      T=UN/(UN+P*X)
      ERFC=0.
      IF(X.GT.13.) RETURN
      ERFC=T*(A1+T*(A2+T*(A3+T*(A4+T*A5))))*EXP(-X*X)
      RETURN
      END
 
C
C
C     ******************************************************************
C
C
 
      SUBROUTINE MATCON(ID)
C     =====================
C
C     Evaluation of the terms in matrices A and B of complete
C     linearization due to convection.
C     Two rows are modified:
C      NRE  - energy balance (i.e. temperature)
C      NDEL - new row corresponding to a new model parameter DELTA
C
C     Model parameter DELTA is defined as dln(T)/dln(P);
C     FCONV is the convective flux
C
C     The equations and corresponding matrix elements are similar
C     to those considered by Grenfell, Astr.Ap. 20, 293 (1972).
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      COMMON/CUBCON/ACNV,BCNV,DEL,GRDADB,DELMDE,RHO,FLXTOT
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/OPMEAN/ABROSD(MDEPTH),ABPLAD(MDEPTH)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
C
      IF(HMIX0.LE.0.) RETURN
      NHE=NFREQE+INHE
      NRE=NFREQE+INRE
      NPC=NFREQE+INPC
      NDEL=NFREQE+INDL
C
C     Upper boundary condition (ID=1)
C
      ANEREL=ELEC(1)/(DENS(1)/WMM(1)+ELEC(1))
      IF(ID.EQ.1) THEN
         DELTA(ID)=0.
         FLXC(ID)=0.
         IF(INDL.GT.0) B(NDEL,NDEL)=UN
        ELSE
C                 
C     Normal depth point 1 < ID < ND
C               
         T=TEMP(ID)
         P=PTOTAL(ID)
         PG=PGS(ID)
         PRAD=P-PG-HALF*DENS(ID)*VTURB(ID)**2
         TM=TEMP(ID-1)
         PM=PTOTAL(ID-1)
         PGM=PGS(ID-1)
         PRADM=PM-PGM-HALF*DENS(ID-1)*VTURB(ID-1)**2
         T0=HALF*(T+TM)
         P0=HALF*(P+PM)
         PG0=HALF*(PG+PGM)
         PR0=HALF*(PRAD+PRADM)
         DLT=(T-TM)/(P-PM)*P0/T0
         DELTA(ID)=DLT
         TT=T*T-TM*TM
         DDT0= DLT/HALF*TM/TT
         DDTM=-DDT0*T/TM
         IF(INDL.GT.0) THEN
            B(NDEL,NDEL)=-UN
            VECL(NDEL)=DELTA(ID)-DLT
c            IF(ID.GE.ICBEG) THEN
            DDP0=0.
            DDPM=0.
            IF(IPRESS.GT.0) THEN
               PP0=P*P-PM*PM
               DDP0=-DLT/HALF*PM/PP0
               DDPM=-DDP0*P/PM
            END IF
C
C           the row of matrix A corresponding to DELTA
C
            A(NDEL,NHE)=BOLK*TM*DDPM
            A(NDEL,NRE)=PGM/TM*DDPM+DDTM
C
C           the row of matrix B corresponding to DELTA
C
            B(NDEL,NHE)=BOLK*T*DDP0
            B(NDEL,NRE)=PG/T*DDP0+DDT0
c            END IF
         END IF
C
C        convective flux and its derivatives
C        (derivativs wrt. T and P(gas) - calculated numerically;
C        derivative wrt. DELTA calculated analytically)
C
         CALL CONVEC(ID,T0,P0,PG0,PR0,ABROSD(ID),DLT,FLXCNV,VCON)
         FLXC(ID)=FLXCNV
         DHCDD=0.
         IF(DELMDE.GT.0.) DHCDD=1.5D0/DELMDE*FLXCNV
         T1=1.001D0*T0
         CALL CONVEC(ID,T1,P0,PG0,PR0,ABROSD(ID),DLT,FLXC1,VCON)
         DHCDT=(FLXC1-FLXCNV)*1.D3/T0*HALF
         DHCDP=0.
         IF(IPRESS.GT.0) THEN
            PG1=1.001*PG0
            CALL CONVEC(ID,T0,P0,PG1,PR0,ABROSD(ID),DLT,FLXC2,VCON)
            DHCDP=(FLXC2-FLXCNV)*1.D3/PG0*HALF
            IF(IPRESS.GT.1) THEN
               P1=1.001*P0
               CALL CONVEC(ID,T0,P1,PG0,PR0,ABROSD(ID),DLT,FLXC3,VCON)
               DHCDPT=(FLXC3-FLXCNV)*1.D3/P0*HALF
               DHCDP=DHCDP+DHCDPT
               DHCDT=DHCDT+DHCDPT*4.*PR0/T0
            END IF
         END IF
         DHCDTM=DHCDT
         IF(INDL.EQ.0) THEN
            DHCDT=DHCDT+DHCDD*DDT0
            DHCDTM=DHCDT+DHCDD*DDTM
         END IF
C
C        additional terms in matrices A and B (the row corresponding to
C        energy balance, i.e. T) due to convection
C
C
C    **  1. differential equation form
C
         if(redif(id).gt.0) then
         IF(ICONV.GT.0) THEN
         A(NRE,NHE)=A(NRE,NHE)+DHCDP*BOLK*TM*redif(id)
         A(NRE,NRE)=A(NRE,NRE)+(DHCDP*PGM/TM+DHCDTM)*redif(id)
         B(NRE,NHE)=B(NRE,NHE)+DHCDP*BOLK*T*redif(id)
         B(NRE,NRE)=B(NRE,NRE)+(DHCDP*PG/T+DHCDT)*redif(id)
         IF(INDL.GT.0) B(NRE,NDEL)=B(NRE,NDEL)+DHCDD*redif(id)
         END IF
         VECL(NRE)=VECL(NRE)-FLXC(ID)*redif(id)
         end if
C
C    **  2. integral equation form
C
         if(reint(id).gt.0.AND.ICONV.GT.2) then
         TP=TEMP(ID+1)
         PTP=PTOTAL(ID+1)
         PGP=PGS(ID+1)
         PRADP=PTP-PGP-HALF*DENS(ID+1)*VTURB(ID+1)**2
         T0=HALF*(T+TP)
         P0=HALF*(P+PTP)
         PG0=HALF*(PG+PGP)
         PR0=HALF*(PRAD+PRADP)
         DLT=(TP-T)/(PTP-P)*P0/T0
         CALL CONVEC(ID,T0,P0,PG0,PR0,ABROSD(ID+1),DLT,FLXCNV,VCON)
         DHCDDP=0.
         IF(DELMDE.GT.0.) DHCDDP=1.5D0/DELMDE*FLXCNV
         T1=1.001D0*T0
         CALL CONVEC(ID,T1,P0,PG0,PR0,ABROSD(ID+1),DLT,FLXC1,VCON)
         DHCDTP=(FLXC1-FLXCNV)*1.D3/T0*HALF
         DHCDPP=0.
         IF(IPRESS.GT.0) THEN
            PG1=1.001*PG0
            CALL CONVEC(ID,T0,P0,PG1,PR0,ABROSD(ID+1),DLT,FLXC2,VCON)
            DHCDPP=(FLXC2-FLXCNV)*1.D3/PG0*HALF
            IF(IPRESS.GT.1) THEN
               P1=1.001*P0
               CALL CONVEC(ID,T0,P1,PG0,PR0,ABROSD(ID+1),DLT,FLXC3,VCON)
               DHCDPT=(FLXC3-FLXCNV)*1.D3/P0*HALF
               DHCDPP=DHCDPP+DHCDPT
               DHCDTP=DHCDTP+DHCDPT*4.*PR0/T0
            END IF
         END IF
C
C        additional terms in matrices A and B (the row corresponding to
C        energy balance, i.e. T) due to convection
C
         DELM=(DM(ID+1)-DM(ID-1))*HALF
         RDELM=DENS(ID)/DELM
         DELHC=WMM(ID)/DELM*(FLXCNV-FLXC(ID))
         IF(ICONV.GT.0) THEN
         A(NRE,NHE)=A(NRE,NHE)-RDELM*DHCDP*BOLK*TM*reint(id)
         A(NRE,NRE)=A(NRE,NRE)-RDELM*(DHCDP*PGM/TM+DHCDT)*reint(id)
         B(NRE,NHE)=B(NRE,NHE)+
     *             (DELHC+RDELM*(DHCDP-DHCDPP)*BOLK*T)*reint(id)
         B(NRE,NRE)=B(NRE,NRE)+
     *              RDELM*((DHCDPP-DHCDP)*PG/T+DHCDTP-DHCDT)*reint(id)
         B(NRE,NPC)=B(NRE,NPC)-DELHC*reint(id)
         B(NRE,NDEL)=B(NRE,NDEL)-RDELM*DHCDD*reint(id)
         C(NRE,NHE)=C(NRE,NHE)+RDELM*DHCDPP*BOLK*TP*reint(id)
         C(NRE,NRE)=C(NRE,NRE)+RDELM*(DHCDPP*PGP/TP+DHCDTP)*reint(id)
         C(NRE,NDEL)=C(NRE,NDEL)+RDELM*DHCDDP*reint(id)
         END IF
         VECL(NRE)=VECL(NRE)-RDELM*(FLXCNV-FLXC(ID))*reint(id)
         end if
      END IF
      RETURN
      END
C
C
C     ******************************************************************
C
C
      SUBROUTINE TEMCOR
C     =================
C
C     Auxiliary procedure for INILAM.
C     Tests wheather the convective flux corresponding to newly
C     determined temperature and logarithmic gradient DELTA
C     is larger than the total flux. If so, the temperature is modified
C     by an iterative procedure for determining new temperature
C     that yields convective flux less than SIG4P*TEFF**4
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ARRAY1.FOR'
      COMMON/CUBCON/ACNV,BCNV,DEL,GRDADB,DELMDE,RHO,FLXTOT
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/OPMEAN/ABROSD(MDEPTH),ABPLAD(MDEPTH)
      COMMON/HYDADD/ANEREL,IHM,IH2,IH2P
C
      IF(ICONV.LE.0.AND.INDL.EQ.0) RETURN
      FLXTOT=SIG4P*TEFF**4
      ANEREL=ELEC(1)/(DENS(1)/WMM(1)+ELEC(1))
      DLTND=DELTA(ND)
      IFNDM1=0
C
      DO 100 ID=2,ND
         T=TEMP(ID)
         P=PTOTAL(ID)
         PG=PGS(ID)
         PRAD=P-PG-HALF*DENS(ID)*VTURB(ID)**2
         TM=TEMP(ID-1)
         PM=PTOTAL(ID-1)
         PGM=PGS(ID-1)
         PRADM=PM-PGM-HALF*DENS(ID-1)*VTURB(ID-1)**2
         IF(ID.EQ.ND.AND.IFNDM1.EQ.1) THEN
            FAC=DLTND*(P-PM)/(P+PM)
            T=TM*(UN+FAC)/(UN-FAC)
         END IF
         KKK=0
   10    KKK=KKK+1
         T0=HALF*(T+TM)
         P0=HALF*(P+PM)
         PG0=HALF*(PG+PGM)
         PR0=HALF*(PRAD+PRADM)
         DLT=(T-TM)/(P-PM)*P0/T0
         DELTA(ID)=DLT
C
C        convective flux and its derivatives
C
         CALL CONVEC(ID,T0,P0,PG0,PR0,ABROSD(ID),DLT,FLXCNV,VCON)
         FLXC(ID)=FLXCNV
         IF(FLXCNV.LT.0.999999*FLXTOT) GO TO 100
C
C        in case that convective flux it too large:
C        iterative procedure for determining new temperature such
C        that yields convective flux less than SIG4P*TEFF**4
C
C        Basically by the Newton-Raphson method;
C        derivative of the convective flux wrt. T calculated
C        numerically; derivative wrt. DELTA calculated analytically
C
         DHCDD=0.
         IF(DELMDE.GT.0.) DHCDD=1.5D0/DELMDE*FLXCNV
         T1=1.001D0*T0
         CALL CONVEC(ID,T1,P0,PG0,PR0,ABROSD(ID),DLT,FLXC1,VCON)
         DHCDT=(FLXC1-FLXCNV)*1.D3/T0*HALF
         TT=T*T-TM*TM
         DDT0= DLT/HALF*TM/TT
         DFLCDT=DHCDT+DHCDD*DDT0
         DELTEM=(FLXTOT-FLXCNV)/DFLCDT
         T1=T+DELTEM
         IF(T1.LT.TM+HALF*(T-TM)) T1=TM+HALF*(T-TM)
         T=T1
         write(14,601) iter,id,kkk,delta(id),temp(id),t,deltem,flxcnv
  601    format(3i4,f10.4,2f12.3,f12.5,1pe15.6)
         TEMP(ID)=T
         IF(ID.EQ.ND-1) IFNDM1=1
         IF(KKK.LE.10.and.abs(deltem/t).gt.1.e-5) GO TO 10
C
C        Determination of electron density from the total pressure
C
         AN=PG/T/BOLK
         CALL ELDENS(ID,T,AN,ANE,ENRG)
         RHO=WMM(ID)*(AN-ANE)
         DENS(ID)=RHO
         ELEC(ID)=ANE
  100 CONTINUE
      RETURN
      END
C
C
C     ******************************************************************
C
C
 
      SUBROUTINE CONCOR
C     =================
C
C     Auxiliary procedure called from INILAM
C     Initialization of the model parameter DELTA immediately
C     after a completed iteration of complete linearization
C
C     DELTA is defined as d(lnT)/dln(P)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      COMMON/HEQAUX/PRD0,IHECOR
C
      IF(INDL.EQ.0) RETURN
      IF(IPRINT.GT.0) WRITE(6,600)
      NDEL=NFREQE+INDL
C
      PRAD0=PRADT(1)-PRD0
      DO 10 ID=1,ND
         PTOTAL(ID)=DM(ID)*GRAV+PRAD0
   10 CONTINUE
C
      DO 20 ID=2,ND
         P=PTOTAL(ID)
         PM=PTOTAL(ID-1)
         DEL1=DELTA(ID)
         TM=TEMP(ID-1)
         T1=TEMP(ID)
         FAC=DEL1*(P-PM)/(P+PM)
         T2=TM*(UN+FAC)/(UN-FAC)
         DEL2=(T1-TM)/(P-PM)/(T1+TM)*(P+PM)
         IF(ITEMP.EQ.1.AND.ID.GE.ICBEG-1) TEMP(ID)=T2
         IF(ITEMP.EQ.2) TEMP(ID)=T2
         IF(IPRINT.GT.0) WRITE(6,601) ID,T1,T2,TEMP(ID),P,DEL2,DEL1
   20 CONTINUE
C
C     check whether the corresponding convective flux is less
C     than total flux; if not, recalculate tempertaure
C
      CALL TEMCOR
C
  600 FORMAT(1H1,'  RECALCULATION OF DELTA AND T'//
     *'    ID T(LINEARIZ) T(DELTA)  TEMP(ID)  P(TOT)',8X,'DEL',
     *     6X,'DEL(LIN)'/)
  601 FORMAT(1H ,I5,3F10.1,1P3D12.3)
      RETURN
      END
C
C
C     ******************************************************************
C
C
 
      SUBROUTINE PZEVAL
C     =================
C
C     Auxiliary procedure called from RESOLV
C     determination of the total and gas pressures, and logarithmic
C     gradient of pressure
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (PCK=4.19168946D-10)
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/MODCON/FLXC(MDEPTH),DELTA(MDEPTH)
      COMMON/PRINTS/IPRINT,IPRING,IPRIND
      COMMON/HEQAUX/PRD0,IHECOR
C
C     total pressure, gas pressure, and logarithmic gradient DELTA
C
      IF(IPRINT.GT.0) WRITE(6,601)
      DO 30 ID=1,ND
         PTURB=HALF*DENS(ID)*VTURB(ID)*VTURB(ID)
         PTOTL0=PGS(ID)+PRADT(ID)+PTURB
         PGS0=(DENS(ID)/WMM(ID)+ELEC(ID))*BOLK*TEMP(ID)
         PTOTL1=GRAV*DM(ID)+PRADT(1)-PRD0
         PGS1=PTOTL1-PTURB-PRADT(ID)
         AAA=3.D0*PRADT(ID)/TEMP(ID)**4/7.5639D-15
         PTOTAL(ID)=PTOTL1
         PGS(ID)=PGS1
         IF(IPRINT.GT.0) WRITE(6,602) ID,PTOTL0,PTOTL1,PGS0,PGS1,
     *                   PRADT(ID),AAA
         IF(ID.EQ.1) GO TO 30
         DELTA(ID)=(TEMP(ID)-TEMP(ID-1))/(TEMP(ID)+TEMP(ID-1))*
     *        (PTOTAL(ID)+PTOTAL(ID-1))/(PTOTAL(ID)-PTOTAL(ID-1))
   30 CONTINUE
      IF(IPRINT.GT.0) WRITE(6,600) ITER-1
      IF(IPRINT.EQ.0.AND.LFIN) WRITE(6,600) ITER-1
      CALL CONOUT(1,IPRINT)
  600 FORMAT(1H1,' CONVECTIVE FLUX: RESOLV; GLOBAL ITERATION =',I2/)
  601 FORMAT(1H1,'  ID   PTOT-SUM  PTOT-MG   PGAS-RHO  PGAS-P    PRAD'
     *       '       A'/)
  602 FORMAT(1H ,I4,1P6D10.3)
      RETURN
      END
C
C
C     ********************************************************************
C
 
      SUBROUTINE STARK0(I,J,XKIJ,WL0,FIJ)
C     ===================================
C
C     Auxiliary procedure for evaluating the approximate Stark profile
C     of hydrogen lines - sets up necessary frequency independent
C     parameters
C
C     Input:  I     - principal quantum number of the lower level
C             J     - principal quantum number of the upper level
C     Output: XKIJ  - coefficients K(i,j) for the Hotzmark profile;
C                     exact up to j=6, asymptotic for higher j
C             WL0   - wavelength of the line i-j
C             FIJ   - Stark f-value for the line i-j
C
      INCLUDE 'IMPLIC.FOR'
      PARAMETER (RYD=9.11763811D2,CXKIJ=5.5D-5)
      PARAMETER (UN=1.,TEN=10.,TWEN=20.,HUND=100.)
      DIMENSION FSTARK(10,4),XKIJT(5,4)
      DATA XKIJT/3.56D-4,5.23D-4,1.09D-3,1.49D-3,2.25D-3,.0125,.0177,
     * .028,.0348,.0493,.124,.171,.223,.261,.342,.683,.866,1.02,1.19,
     * 1.46/
      DATA FSTARK/.1387,.0791,.02126,.01394,.00642,4.814D-3,2.779D-3,
     * 2.216D-3,1.443D-3,1.201D-3,.3921,.1193,.03766,.02209,.01139,
     * 8.036D-3,5.007D-3,3.85D-3,2.658D-3,2.151D-3,.6103,.1506,.04931,
     * .02768,.01485,.01023,6.588D-3,4.996D-3,3.524D-3,2.838D-3,.8163,
     * .1788,.05985,.03189,.01762,.01196,7.825D-3,5.882D-3,4.233D-3,
     * 3.375D-3/
      SAVE XKIJT,FSTARK
C
      II=I*I
      JJ=J*J
      JMIN=J-I
      IF(JMIN.LE.5) THEN
         XKIJ=XKIJT(JMIN,I)
      ELSE
         XKIJ=CXKIJ*(II*JJ)*(II*JJ)/(JJ-II)
      END IF
      IF(JMIN.LE.10) THEN
         FIJ=FSTARK(JMIN,I)
      ELSE
         CFIJ=((TWEN*I+HUND)*J/(I+TEN)/(JJ-II))
         FIJ=FSTARK(10,I)*CFIJ*CFIJ*CFIJ
      END IF
      WL0=RYD/(UN/II-UN/JJ)
      RETURN
      END
C
C
C     ****************************************************************
C
C
 
      FUNCTION STARKA(BETA)
C     =====================
C
C     Approximate expressions for the hydrogen Stark profile
C
C     Input: BETA  - delta lambda in beta units,
C            BETAD - Doppler width in beta units
C            A     - auxiliary parameter
C                    A=1.5*LOG(BETAD)-1.761
C            DIV   - only for A > 1; division point between Doppler
C                    and asymptotic Stark wing, expressed in units
C                    of betad.
C                    DIV = solution of equation
C                    exp(-(beta/betad)**2)/betad/sqrt(pi)=
C                     = 1.5*FAC*beta**-5/2
C                    (ie. the point where Doppler profile is equal to
C                     the asymptotic Holtsmark)
C                    In order to save computer time, the division point
C                    DIV is calculated in advance by routine DIVSTR.
C
      INCLUDE 'IMPLIC.FOR'
      COMMON/STRAUX/BETAD,ADH,DIVH
      PARAMETER (F0=-0.5758228,F1=0.4796232,F2=0.07209481,AL=1.26)
      PARAMETER (SD=0.5641895,SLO=-2.5,THR=3.,BL1=1.14,BL2=11.4)
      PARAMETER (SAC=0.08, UN=1.D0, PISQ1=UN/1.77245385090551D0)
C
C     for a > 1 Doppler core + asymptotic Holtzmark wing with division
C               point DIV
C
      BETAD1=UN/BETAD
      IF(ADH.GT.AL) THEN
         XD=BETA*BETAD1
         IF(XD.LE.DIVH) THEN
            STARKA=EXP(-XD*XD)*BETAD1*PISQ1
          ELSE
            STARKA=THR*EXP(SLO*LOG(BETA))
         END IF
       ELSE
C
C     empirical formula for a < 1
C
         IF(BETA.LE.BL1) THEN
            STARKA=SAC
          ELSE IF(BETA.LT.BL2) THEN
            XL=LOG(BETA)
            FL=(F0*XL+F1)*XL
            STARKA=F2*EXP(FL)
          ELSE
            STARKA=THR*EXP(SLO*LOG(BETA))
         END IF
      END IF
      RETURN
      END
C
C
C *******************************************************************
C
C
 
      SUBROUTINE DIVSTR
C     =================
C
C     Auxiliary procedure for STARKA - determination of the division
C     point between Doppler and asymptotic Stark profiles
C
C     Input:  BETAD - Doppler width in beta units
C     Output: A     - auxiliary parameter
C                     A=1.5*LOG(BETAD)-1.671
C             DIV   - only for A > 1; division point between Doppler
C                     and asymptotic Stark wing, expressed in units
C                     of betad.
C                     DIV = solution of equation
C                     exp(-(beta/betad)**2)/betad/sqrt(pi)=3*beta**-5/2
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      COMMON/STRAUX/BETAD,ADH,DIVH
      PARAMETER (UNQ=1.25,UNH=1.5,TWH=2.5,FO=4.,FI=5.)
      PARAMETER (CA=1.671,BL=5.821,AL=1.26,CX=0.28,DX=0.0001)
C
      ADH=UNH*LOG(BETAD)-CA
      IF(BETAD.LT.BL) RETURN
      IF(ADH.GE.AL) THEN
         X=SQRT(ADH)*(UN+UNQ*LOG(ADH)/(FO*ADH-FI))
      ELSE
         X=SQRT(CX+ADH)
      ENDIF
      DO 10 I=1,5
         X2=X*X
         XN=X*(UN-(X2-TWH*LOG(X)-ADH)/(TWO*X2-TWH))
         IF(ABS(XN-X).LE.DX) GO TO 20
         X=XN
   10 CONTINUE
   20 DIVH=X
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE OPAHST
C     =================
C
C     Auxiliary routine for START
C     sets up necessary parameters for routines OPAHYL and OPHYL1, i.e.
C     for opacity and emissivity in higher hydrogen lines
C     Also sets up Stark parameters for OPAHYL
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ODFPAR.FOR'
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
      COMMON/OPHSTK/XKIJ(2,NLMX),WL0(2,NLMX),FIJ(2,NLMX)
C
      ALLIM1=1450.
      ABLIM1=6650.
      ABLIM2=5000.
      ABLIM3=6500.
C
C     Lyman lines
C
      ILOW=1
      IF(IABS(IOPHL1).EQ.1) IOPHL2=IOPHL2*2
      DO 10 I=1,4
         M1FILE(I,ILOW)=MAX(I,IABS(IOPHL1))
         M2FILE(I,ILOW)=I+1
   10 CONTINUE
      DO 20 I=5,NLMX
         M1FILE(I,ILOW)=MAX(I-1,IABS(IOPHL1))
         M2FILE(I,ILOW)=MIN(I+3,NLMX)
   20 CONTINUE
      M1FILE(NLMX,ILOW)=NLMX+1
      M2FILE(NLMX,ILOW)=NLMX
C
      IF(IABS(IOPHL1).GT.100) THEN
         IOPHL1=MOD(IOPHL1,100)
         ISET=0
   40    CONTINUE
         READ(IBUFF,*,ERR=90) IL1,IU1,IM1,IP1
         ISET=ISET+1
         IF(IL1.LE.0.AND.ISET.EQ.1) THEN
            IL1=1
            IU1=4
            IM1=0
            IP1=1
         END IF
         IF(IL1.LE.0.AND.ISET.EQ.2) THEN
            IL1=5
            IU1=NLMX
            IM1=1
            IP1=3
         END IF
         IUP1=MIN(IU1,NLMX)
         DO 50 I=IL1,IUP1
            M1FILE(I,ILOW)=MAX(I-IM1,IABS(IOPHL1))
            M2FILE(I,ILOW)=MIN(I+IP1,NLMX)
   50    CONTINUE
         IF(IU1.LT.NLMX) GO TO 40
   90 CONTINUE   
      READ(IBUFF,*,ERR=100) ALLIM1
         IF(ALLIM1.LE.0) ALLIM1=1450.
      END IF
      M1FILE(NLMX,ILOW)=NLMX+1
      M2FILE(NLMX,ILOW)=NLMX
C
C     Balmer lines
C
  100 ILOW=2
      IF(IABS(IOPHL2).EQ.1) IOPHL2=IOPHL2*3
      IF(IABS(IOPHL2).EQ.2) IOPHL2=IOPHL2*3/2
      DO 110 I=1,6
         M1FILE(I,ILOW)=MAX(I,IABS(IOPHL2))
         M2FILE(I,ILOW)=I+1
  110 CONTINUE
      DO 120 I=7,NLMX
         M1FILE(I,ILOW)=MAX(I-1,IABS(IOPHL2))
         M2FILE(I,ILOW)=MIN(I+3,NLMX)
  120 CONTINUE
      IF(IABS(IOPHL2).GT.100) THEN
         IOPHL2=MOD(IOPHL2,100)
         ISET=0
  140    CONTINUE
         READ(IBUFF,*,ERR=190) IL1,IU1,IM1,IP1
         ISET=ISET+1
         IF(IL1.LE.0.AND.ISET.EQ.1) THEN
            IL1=1
            IU1=6
            IM1=0
            IP1=1
         END IF
         IF(IL1.LE.0.AND.ISET.EQ.2) THEN
            IL1=7
            IU1=NLMX
            IM1=1
            IP1=3
         END IF
         IUP1=MIN(IU1,NLMX)
         DO 150 I=IL1,IUP1
            M1FILE(I,ILOW)=MAX(I-IM1,IABS(IOPHL2))
            M2FILE(I,ILOW)=MIN(I+IP1,NLMX)
  150    CONTINUE
         IF(IU1.LT.NLMX) GO TO 140
  190    CONTINUE
         READ(IBUFF,*,ERR=200,END=200) ABLIM1,ABLIM2,ABLIM3
         IF(ABLIM1.LE.0) ABLIM1=6650.
         IF(ABLIM2.LE.0) ABLIM2=5000.
         IF(ABLIM3.LE.0) ABLIM3=6500.
      END IF
  200 CONTINUE
      M1FILE(NLMX,ILOW)=NLMX+1
      M2FILE(NLMX,ILOW)=NLMX
c
C    -------------------
C    Stark paramereters
C    -------------------
C
      IF(IOPHL1.NE.0) THEN
         I=1
         I1=MAX(2,IABS(IOPHL1))
         DO 210 J=I1,NLMX
            CALL STARK0(I,J,XKIJ(I,J),WL0(I,J),FIJ(I,J))
  210    CONTINUE
      END IF
      IF(IOPHL2.NE.0) THEN
         I=2
         I2=MAX(3,IABS(IOPHL2))
         DO 220 J=I2,NLMX
            CALL STARK0(I,J,XKIJ(I,J),WL0(I,J),FIJ(I,J))
  220    CONTINUE
      END IF
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE WNSTOR(ID)
C     =====================
C
C     Stores occupation probabilities for hydrogen levels
C     in common WNCOM for further use
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (SIXTH=UN/6.,CCOR=0.09)
      parameter (p1=0.1402,p2=0.1285,p3=un,p4=3.15,p5=4.)
      parameter (tkn=3.01,ckn=5.33333333,cb=8.3d14,f23=-2./3.)
C
      ANE=ELEC(ID)
      A=CCOR*EXP(SIXTH*LOG(ANE))/SQRT(TEMP(ID))
      z=un
      x=exp(p4*log(un+p3*a))
      c1=p1*(x+p5*z*a*a*a)
      c2=p2*x
      beta0=cb*z*z*z*exp(f23*log(ane))
      DO 20 I=1,NLMX
         XN=I
         if(xn.ge.tkn) then
            xkn=un
         else
            xn1=un/(xn+un)
            xkn=ckn*xn*xn1*xn1
         end if
         beta=beta0*xkn*xi2(i)*xi2(i)
         f=(c1*beta*beta*beta)/(un+c2*beta*sqrt(beta))
         WNHINT(I,ID)=f/(un+f)
   20 CONTINUE
C
C     array WOP - occupation probabilities for explicit levels
C
      do 30 ii=1,nlevel
         wop(ii,id)=un
         if(ifwop(ii).le.0) go to 30
         ie=iel(ii)
         nq=nquant(ii)
         if(iz(ie).eq.1) then
            wop(ii,id)=wnhint(nq,id)
          else
            z=iz(ie)
            xn=nq
            wop(ii,id)=wn(xn,a,ane,z)
         end if
   30 continue
      RETURN
      END
C
C
C ********************************************************************
C
C

      function wn(xn,a,ane,z)
c
c     evaluation of the occupation probablities for a hydrogenic ion
c     using eqs (4.26), and (4.39) of Hummer,Mihalas Ap.J. 331, 794, 1988.
c     approximate evaluation of Q(beta) - Hummer
c
c     Input: xn  - real number corresponding to quantum number n
c            a   - correlation parameter
c            ane - electron density
c            z   - ionic charge
c
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      parameter (p1=0.1402,p2=0.1285,p3=un,p4=3.15,p5=4.)
      parameter (tkn=3.01,ckn=5.33333333,cb=8.3d14)
      parameter (f23=-2./3.)
c
c     evaluation of k(n)
c
      if(xn.ge.tkn) then
         xkn=un
       else
         xn1=un/(xn+un)
         xkn=ckn*xn*xn1*xn1
      end if
c
c     evaluation of beta
c
      beta=cb*z*z*z*xkn/(xn*xn*xn*xn)*exp(f23*log(ane))
c
c     approximate expression for Q(beta)
c
      x=exp(p4*log(un+p3*a))
      c1=p1*(x+p5*z*a*a*a)
      c2=p2*x
      f=(c1*beta*beta*beta)/(un+c2*beta*sqrt(beta))
      wn=f/(un+f)
      return
      end
C
C
C ********************************************************************
C
C

      SUBROUTINE DWNFR(MODE,N,FRE,A,ANE,Z,FR,DW)
C     ==========================================
C
C     Auxiliary routine to compute set of dissolved fractions
C     for all frequencies
C      MODE=0  ->  DW=1
C      MODE>0  ->  DW=1-w
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      parameter (p1=0.1402,p2=0.1285,p3=un,p4=3.15,p5=4.)
      parameter (tkn=3.01,ckn=5.33333333,cb=8.3d14,f23=-2./3.)
      PARAMETER (FRH=3.28805D15,SQFRH=5.734152D7)
      DIMENSION FR(N),DW(N)
C
      IF(MODE.EQ.0) THEN
       DO 20 IJ=1,N
         DW(IJ)=UN
   20  CONTINUE
      ELSE
       DO 10 IJ=1,N
         IF(FR(IJ).LT.FRE) THEN
            XN=SQFRH*Z/SQRT(FRE-FR(IJ))
            if(xn.ge.tkn) then
               xkn=un
            else
               xn1=un/(xn+un)
               xkn=ckn*xn*xn1*xn1
            end if
            beta=cb*z*z*z*xkn/(xn*xn*xn*xn)*exp(f23*log(ane))
            x=exp(p4*log(un+p3*a))
            c1=p1*(x+p5*z*a*a*a)
            c2=p2*x
            f=(c1*beta*beta*beta)/(un+c2*beta*sqrt(beta))
            DW(IJ)=UN-f/(un+f)
         ELSE
            DW(IJ)=UN
         END IF
   10  CONTINUE
      END IF
      RETURN
      END
C
C
C ********************************************************************
C
C

      SUBROUTINE ODF1(IMODE,IL,IU,ID,ODF)
C     ===================================
C
C     opacity distribution function for overlapping lines near the series limit
C
C     The lines converge to the edge of the (continuum) transition IL - IU,
C     IL - index of the lower level
C     IU - index of the upper level (usually the ground state of teh next ion)
C     ID - depth index
C
C     Output: ODF - opacity distribution function interpolated to the set of
C                   explicit frequencies
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      PARAMETER (FRH=3.28805D15,CQT=1.284523D12)
      PARAMETER (CCOR=0.09,C00=1.25D-9,CID=0.02654,SIXTH=UN/6.)
      COMMON/ODFSTK/XKIJ(MHOD,NLMX),WL0(MHOD,NLMX),FIJ(MHOD,NLMX)
      COMMON/STRAUX/BETAD,ADH,DIVH
      DIMENSION FRO(MFRO),ODF0(MFRO),ABSO0(MFRO),ODF(MFREQ),SGT(MFRO)
      DIMENSION FROD(MFRO),SGFR(MFRO),IODF(MFRO),IODR(MFRO),DWF(MFRO)
      SAVE FRO,SGFR,ODF0,IODF
C
      I=NQUANT(IL)
      KL=INDODF(IL)
      IELO=IEL(IL)
      N1H=NLAST(IELO)
      NQ1=NQLODF(IL)
      FRE=ENION(IL)/H
      T=TEMP(ID)
      SQT=SQRT(T)
      ANE=ELEC(ID)
      ANES=EXP(SIXTH*LOG(ANE))
c      ACOR=CCOR*ANES/SQT 
      F00=C00*ANES*ANES*ANES*ANES
      DOP0=CQT*SQT
      QZ=IZ(IELO)
C
C     pseudocontinuum opacity (non-zero in all frequencies);
C     formulated through the dissolved fraction
C
      ITR=ITRA(IL,IU)
      NFR0=NFRODF(KL)
      IF(IMODE.EQ.0) THEN
         DO 10 IJ=1,NFR0
            FRO(IJ)=FROS(IJ,KL)
            SGFR(IJ)=SIGK(FRO(IJ),ITR,1)
   10    CONTINUE
      END IF
C        
C        function D(nu) - dissolved fraction
C        
c      CALL DWNFR(1,NFR0,FRE,ACOR,ANE,QZ,FRO,DWF)
      DO 20 IJ=1,NFR0
         ABSO0(IJ)=SGFR(IJ)*DWF(IJ)
   20 CONTINUE
C
C     summation over individual lines
C
      DO 40 J=NQ1,NLMX
         XJ=J
         FXK=F00*XKIJ(KL,J)
         DOP=DOP0/WL0(KL,J)
         DBETA=WL0(KL,J)*WL0(KL,J)/CAS/FXK
         BETAD=DOP*DBETA
         FID=CID*FIJ(KL,J)*DBETA
         CALL DIVSTR
         WPROB=WNHINT(J,ID)
         CALL ODFHST(NFR0,FXK,FID,WPROB,WL0(KL,J),FRO,SGT)
         DO 30 IJ=1,NFR0
   30       ABSO0(IJ)=ABSO0(IJ)+SGT(IJ)
   40 CONTINUE
C
C     opacity distribution function in the internal set of frequencies
C
      IF(IMODE.EQ.0) THEN
         ODF0(1)=ABSO0(1)
         IODF(1)=1
         DO 60 IJ=2,NFR0
            ODF0(IJ)=ABSO0(IJ)
            IODF(IJ)=IJ
            IF(ODF0(IJ).LT.ODF0(IJ-1)) THEN
               AB=ODF0(IJ)
               IJODF=IODF(IJ)
               DO 70 IJ0=1,IJ-1
                  IJ1=IJ-IJ0+1
                  IF(ODF0(IJ1).GE.ODF0(IJ1-1)) GO TO 71
                  ODF0(IJ1)=ODF0(IJ1-1)
                  ODF0(IJ1-1)=AB
                  IODF(IJ1)=IODF(IJ1-1)
                  IODF(IJ1-1)=IJODF
   70          CONTINUE
   71          CONTINUE
            END IF
         if(odf0(ij).gt.0.001) write(6,603) ij,id,odf0(ij)
  603    format(' ij,id,odf0',2i5,1pd10.3)
   60    CONTINUE
      ELSE
         ODF0(1)=ABSO0(IODF(1))
         IODR(1)=IODF(1)
         DO 80 IJ=2,NFR0
            ODF0(IJ)=ABSO0(IODF(IJ))
            IODR(IJ)=IODF(IJ)
            IF(ODF0(IJ).LT.ODF0(IJ-1)) THEN
               AB=ODF0(IJ)
               IJODF=IODR(IJ)
               DO 85 IJ0=1,IJ-1
                  IJ1=IJ-IJ0+1
                  IF(ODF0(IJ1).GE.ODF0(IJ1-1)) GO TO 86
                  ODF0(IJ1)=ODF0(IJ1-1)
                  ODF0(IJ1-1)=AB
                  IODR(IJ1)=IODR(IJ1-1)
                  IODR(IJ1-1)=IJODF
   85          CONTINUE
   86          CONTINUE
            END IF
         if(odf0(ij).gt.0.001) write(6,603) ij,id,odf0(ij)
   80    CONTINUE
         DO 90 IJ=1,NFR0
            IODF(IJ)=IODR(IJ)
   90    CONTINUE
      ENDIF
C
C     Reinitialization of the internal frequencies set
C
      FROD(1)=FRO(1)
      IW=IODF(1)
      IF(IW.GT.1 .AND. IW.LT.NFR0) THEN
         W1=FRO(IW-1)-FRO(IW+1)
      ELSE IF (IW.EQ.1) THEN
         W1=FRO(1)-FRO(2)
      ELSE
         W1=FRO(NFR0-1)-FRO(NFR0)
      ENDIF
      DO 200 IJ=2,NFR0-1
         IW=IODF(IJ)
         IF(IW.GT.1 .AND. IW.LT.NFR0) THEN
            W2=HALF*(FRO(IW-1)-FRO(IW+1))
         ELSE IF (IW.EQ.1) THEN
            W2=HALF*(FRO(1)-FRO(2))
         ELSE
            W2=HALF*(FRO(NFR0-1)-FRO(NFR0))
         ENDIF
         FROD(IJ)=FROD(IJ-1)-HALF*(W1+W2)
         W1=W2
  200 CONTINUE
      IJ=NFR0
      IW=IODF(IJ)
      IF(IW.GT.1 .AND. IW.LT.NFR0) THEN
         W2=FRO(IW-1)-FRO(IW+1)
      ELSE IF (IW.EQ.1) THEN
         W2=FRO(1)-FRO(2)
      ELSE
         W2=FRO(NFR0-1)-FRO(NFR0)
      ENDIF
      FROD(IJ)=FROD(IJ-1)-HALF*(W1+W2)
C
C     Interpolated opacity distribution function to explicit frequencies
C
      DO 150 IJ=2,NFREQ
         IF(FREQ(IJ).GT.FREQ(IJ-1)) RETURN
         ODF(IJ)=0.
         IF(FREQ(IJ).GT.FROD(1).OR.FREQ(IJ).LT.FROD(NFR0)) GO TO 150
         IF(ID.EQ.1) THEN
            IF(FREQ(IJ-1).GT.FROD(1)) I1ODF(IL)=IJ
            I2ODF(IL)=IJ
         END IF
         DO 110 IJ1=2,NFR0
            IJ0=IJ1
            IF(FREQ(IJ).GE.FROD(IJ1)) GO TO 120
  110    CONTINUE
  120    ODF(IJ)=ODF0(IJ0-1)+(ODF0(IJ0)-ODF0(IJ0-1))/
     *           (FROD(IJ0)-FROD(IJ0-1))*(FREQ(IJ)-FROD(IJ0-1))
  150 CONTINUE
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE ODFHST(N,FXK,FID,WP,WL,FR,SG)
C     ========================================
C
C     Auxiliary routine for ODF1 (replaces multiple calls to STARKA)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ODFPAR.FOR'
      COMMON/STRAUX/BETAD,ADH,DIVH
      PARAMETER (F0=-0.5758228,F1=0.4796232,F2=0.07209481,AL=1.26)
      PARAMETER (SD=0.5641895,SLO=-2.5,THR=3.,BL1=1.14,BL2=11.4)
      PARAMETER (SAC=0.08)
      DIMENSION FR(MFRO),SG(MFRO)
C
      BETAD1=UN/BETAD
      FXK1=UN/FXK
      FIDWP=FID*WP
      DO 10 IJ=1,N
         ALAM=CAS/FR(IJ)
         BETA=ABS(ALAM-WL)*FXK1
         XD=BETA*BETAD1
C
C     for a > 1 Doppler core + asymptotic Holtzmark wing with division
C               point DIV
C
      IF(ADH.GT.AL) THEN
         IF(XD.LE.DIVH) THEN
            ST=SD*EXP(-XD*XD)*BETAD1
          ELSE
            ST=THR*EXP(SLO*LOG(BETA))
         END IF
       ELSE
C
C     empirical formula for a < 1
C
         IF(BETA.LE.BL1) THEN
            ST=SAC
          ELSE IF(BETA.LT.BL2) THEN
            XL=LOG(BETA)
            FL=(F0*XL+F1)*XL
            ST=F2*EXP(FL)
          ELSE
            ST=THR*EXP(SLO*LOG(BETA))
         END IF
      END IF
C
      SG(IJ)=ST*FIDWP
   10 CONTINUE
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE HODFIN
C     =================
C
C     Initialization for "hydrogen" opacity distribution functions
C      i.e. higher members of Rydberg series of lines merging to
C      the corresponding continuum
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      COMMON/ODFSTK/XKIJ(MHOD,NLMX),WL0(MHOD,NLMX),FIJ(MHOD,NLMX)
C
      K=0
      NODF=0
      IMERG=0
      DO 10 I=1,NLEVEL
         IF(IFWOP(I).LT.0) THEN
            IF(IEL(I).EQ.IELH) IMERG=NQUANT(I)
         END IF
   10 CONTINUE
      RETURN
      END
C
C
C ********************************************************************
C
C

      SUBROUTINE ODFFR(IL,IU)
C     =======================
C
C     Internal frequencies set for opacity distribution function
C     for overlapping lines near the series limit
C
C     The lines converge to the edge of the (continuum) transition IL - IU,
C     IL - index of the lower level
C     IU - index of the upper level (usually the ground state of the next ion)
C                                     or a mean level in the line ODF formalism 
C
C     Output: FROS - set of internal frequencies
C                    in common ODFFRQ
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      PARAMETER (FRH=3.28805D15,CDOP=2.84511D-7,CDOM=14.)
      PARAMETER (SIX=6.,SEPT=7.)
      DIMENSION FFRO(MFRO)
C
      CH=IZ(IEL(IL))*IZ(IEL(IL))
      FRION=CH*FRH
      FRE=ENION(IL)/H
      NF=1
        NQ1=NQUANT(IU)
        XL2=UN/(NQUANT(IL)*NQUANT(IL))
        XU1=UN/((NQUANT(IU)-1)*(NQUANT(IU)-1))
        XU2=UN/(NQUANT(IU)*NQUANT(IU))
        FRC=FRION*(XL2-XU2)
        FFRO(NF)=HALF*(FRC+FRION*(XL2-XU1))
        KT=ITRA(IL,IU)
        KL=JNDODF(KT)
        DOPO=CDOP*SQRT(TEFF)*FRC
      DOPM=CDOM*DOPO
      FR1=FFRO(1)
C
      DO 10 I=NQ1,NLMX
         II=I*I
         FR2=FRE-FRION/II
         DF=FR2-FR1
         IF(DF.GT.DOPM) THEN
            DO 20 J=1,7
               NF=NF+1
               FFRO(NF)=FR1+J*DOPO
   20       CONTINUE
            DF=FR2-SEPT*DOPO-FFRO(NF)
            NI=DF/SIX/DOPO
            DDF=DF/(NI+1)
            DO 21 J=1,NI
               NF=NF+1
               FFRO(NF)=FR1+SEPT*DOPO+J*DDF
   21       CONTINUE
            DO 22 J=7,0,-1
               NF=NF+1
               FFRO(NF)=FR2-J*DOPO
   22       CONTINUE
            FR1=FR2
         ELSE
            NI=DF/DOPO
            DDF=DF/(NI+1)
            DO 30 J=1,NI
               NF=NF+1
               FFRO(NF)=FR1+J*DDF
   30       CONTINUE
            NF=NF+1
            FFRO(NF)=FR2
            FR1=FR2
         ENDIF
   10 CONTINUE
      NF=NF+1
      FFRO(NF)=FRE*0.999999999
      NFRODF(KL)=NF
      if(nf.gt.mfro) call stop('too many points for hydrogen ODF')
      DO 40 I=1,NF
         FROS(I,KL)=FFRO(NF-I+1)
   40 CONTINUE
C
C   Associated weights
C
      WNUS(1,KL)=HALF*(FROS(1,KL)-FROS(2,KL))
      WNUS(NF,KL)=HALF*(FROS(NF-1,KL)-FROS(NF,KL))
      DO 50 I=2,NF-1
        WNUS(I,KL)=HALF*(FROS(I-1,KL)-FROS(I+1,KL))
   50 CONTINUE
C
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE CHCKSE
C     ==================
C
C     Auxiliary output routine, which enables printing 
C     total rates to check statistical equilibrium at each depth.
C
C     Output: unit 30: <OUT> and <IN> rates, and relative difference, 
C                      for each level.
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (MLEVES=1)
      DIMENSION ROUT(MLEVES,MDEPTH),RIN(MLEVES,MDEPTH)
C
      DO 100 ID=1,ND
         T=TEMP(ID)
         HKT=HK/T
         TK=HKT/H
         ANE=ELEC(ID)
         CALL SABOLF(ID)
         DO 70 IAT=1,NATOM
         N0I=N0A(IAT)
         NKI=NKA(IAT)
         N1I=NKI-1
         DO 80 I=N0I,NKI
            OUT=0.
            XIN=0.
            NKE=NNEXT(IEL(I))
            DO 90 IT=1,NTRANS
               II=ILOW(IT)
               JJ=IUP(IT)
               IF(II.EQ.I) THEN
               J=JJ
               IF(LINE(IT)) THEN
                  AIJ=(COLRAT(IT,ID)*ANE+RRD(IT,ID))*
     *                G(I)/G(J)*WOP(I,ID)*EXP(HKT*FR0(IT))
                ELSE
                  CORR=UN
                  NKE=NNEXT(IEL(I))
                  IF(NKE.NE.J) CORR=G(NKE)/G(J)*
     *                EXP((ENION(NKE)-ENION(J))*TK)
                  AIJ=(COLRAT(IT,ID)*ANE+RRD(IT,ID))*
     *                ANE*SBF(I)*CORR*WOP(I,ID)
               END IF
               AJI=(COLRAT(IT,ID)*ANE+RRU(IT,ID))*WOP(J,ID)
               XIN=XIN+AIJ*POPUL(J,ID)
               OUT=OUT+AJI
               ELSE IF(JJ.EQ.I) THEN
               J=II
               IF(LINE(IT)) THEN
                  AJI=(COLRAT(IT,ID)*ANE+RRD(IT,ID))*
     *                G(J)/G(I)*WOP(J,ID)*EXP(HKT*FR0(IT))
                ELSE
                  CORR=UN
                  NKE=NNEXT(IEL(J))
                  IF(NKE.NE.I) CORR=G(NKE)/G(I)*
     *               EXP((ENION(NKE)-ENION(I))*TK)
                  AJI=(COLRAT(IT,ID)*ANE+RRD(IT,ID))*
     *                  ANE*SBF(J)*CORR*WOP(J,ID)
               END IF
               AIJ=(COLRAT(IT,ID)*ANE+RRU(IT,ID))*WOP(I,ID)
               XIN=XIN+AIJ*POPUL(J,ID)
               OUT=OUT+AJI
               END IF
   90       CONTINUE
            RIN(I,ID)=XIN
            ROUT(I,ID)=OUT*POPUL(I,ID)
   80    CONTINUE
   70    CONTINUE
  100 CONTINUE
      DO 200 I=1,NLEVEL
         IF(RIN(I,ND).EQ.0.) GO TO 200
         WRITE(16,300) I
         DO 210 ID=1,ND
            DEL=(RIN(I,ID)-ROUT(I,ID))/RIN(I,ID)
            WRITE(16,310) I,ID,RIN(I,ID),ROUT(I,ID),DEL,popul(i,id)
  210    CONTINUE
  200 CONTINUE
  300 FORMAT('1 Level:',I5///)
  310 FORMAT(2I5,1P3E16.7,2x,e16.7)
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE ACCEL2
C     =================
C
C     Acceleration of convergence (from Auer 1987, in Numerical
C     Radiative Transfer p. 101)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ITERAT.FOR'
      COMMON/FILES/PSY0(MTOT,MDEPTH),PSY1(MTOT,MDEPTH),
     * PSY2(MTOT,MDEPTH),PSY3(MTOT,MDEPTH)
C
      IF(NITER.LT.IACC .OR. ITER.LT.IACC0) RETURN
      ipng=1
      if(iacd.gt.0) ipng=mod((iter-iacc),iacd)
      if(.not.lac2) then
      IPT=MOD(ITER,3)
      IPT0=MOD(IACC,3)
      IPT1=MOD((IACC+1),3)
      IPT2=MOD((IACC+2),3)
      IF(ITER.EQ.IACC0) THEN
         DO 11 ID=1,ND
            DO 11 IX=1,NN
               PSY3(IX,ID)=PSY0(IX,ID)
   11    CONTINUE
      ELSE IF(IPT.EQ.IPT1) THEN
         DO 12 ID=1,ND
            DO 12 IX=1,NN
               PSY2(IX,ID)=PSY0(IX,ID)
   12    CONTINUE
      ELSE IF(IPT.EQ.IPT2) THEN
         DO 13 ID=1,ND
            DO 13 IX=1,NN
               PSY1(IX,ID)=PSY0(IX,ID)
   13    CONTINUE
      ENDIF
      else if (ipng.ne.0) then
         DO 21 ID=1,ND
            DO 21 IX=1,NN
               PSY3(IX,ID)=PSY2(IX,ID)
   21    CONTINUE
         DO 22 ID=1,ND
            DO 22 IX=1,NN
               PSY2(IX,ID)=PSY1(IX,ID)
   22    CONTINUE
         DO 23 ID=1,ND
            DO 23 IX=1,NN
               PSY1(IX,ID)=PSY0(IX,ID)
   23    CONTINUE
         RETURN
      end if

      IF(ITER.LT.IACC) RETURN
C
      A1=0.
      B1=0.
      B2=0.
      C1=0.
      C2=0.
      DO 50 ID=1,ND
         DO 51 IX=1,NN
            IF(PSY0(IX,ID).NE.0.) WT=1./ABS(PSY0(IX,ID))
            D0=PSY0(IX,ID)-PSY1(IX,ID)
            D1=D0-PSY1(IX,ID)+PSY2(IX,ID)
            D2=D0-PSY2(IX,ID)+PSY3(IX,ID)
            A1=A1+WT*D1*D1
            B1=B1+WT*D1*D2
            B2=B2+WT*D2*D2
            C1=C1+WT*D0*D1
            C2=C2+WT*D0*D2
   51    CONTINUE
   50 CONTINUE
         AB=B2*A1-B1*B1
         IF(AB.EQ.0.) THEN
            WRITE(6,601) ITER,AB
            WRITE(10,601) ITER,AB
            IACC=IACC+IACD
            IACC0=IACC-3
            RETURN
         ENDIF
         A=(B2*C1-B1*C2)/AB
         B=(A1*C2-B1*C1)/AB
      DO 53 ID=1,ND
         DO 52 IX=1,NN
            PSY0(IX,ID)=(1.-A-B)*PSY0(IX,ID)+A*PSY1(IX,ID)+
     *                   B*PSY2(IX,ID)
   52    CONTINUE
   53 CONTINUE
      WRITE(6,600) ITER
      WRITE(10,600) ITER
      LAC2=.TRUE.
      LRES2=.FALSE.
c
c     call RESOLV after evaluating the accelerated estimate
c
      CALL RESOLV
C     CALL TIMING(1,ITER)
      LRES2=.TRUE.
      RETURN
  600 FORMAT(/,' **** ACCEL2, ITER=',I4,/)
  601 FORMAT(/,' **** ACCEL2, ITER=',I4,'   AB = ',F7.3,/)
      END
C
C
C ********************************************************************
C
C

      SUBROUTINE ACCELP
C     =================
C
C     Acceleration of convergence for populations
C     (from Auer 1987, in Numerical Radiative Transfer p. 101)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ITERAT.FOR'
      COMMON/POPULS/POPUL1(MLEVEL,MDEPTH),
     *              POPUL2(MLEVEL,MDEPTH),POPUL3(MLEVEL,MDEPTH)
C
      IF(NLAMBD.LT.IACCP.OR. ILAM.LT.IACC0P) RETURN
      ipng=1
      if(iacdp.gt.0) ipng=mod((ILAM-IACCP),IACDP)
      if(.not.lac2p) then
      IPT=MOD(ILAM,3)
      IPT0=MOD(IACCP,3)
      IPT1=MOD((IACCP+1),3)
      IPT2=MOD((IACCP+2),3)
      IF(ILAM.EQ.IACC0P) THEN
         DO 11 ID=1,ND
            DO 11 IX=1,NLEVEL
               POPUL3(IX,ID)=POPUL(IX,ID)
   11    CONTINUE
      ELSE IF(IPT.EQ.IPT1) THEN
         DO 12 ID=1,ND
            DO 12 IX=1,NLEVEL
               POPUL2(IX,ID)=POPUL(IX,ID)
   12    CONTINUE
      ELSE IF(IPT.EQ.IPT2) THEN
         DO 13 ID=1,ND
            DO 13 IX=1,NLEVEL
               POPUL1(IX,ID)=POPUL(IX,ID)
   13    CONTINUE
      ENDIF
      else if (ipng.ne.0) then
         DO 21 ID=1,ND
            DO 21 IX=1,NLEVEL
               POPUL3(IX,ID)=POPUL2(IX,ID)
   21    CONTINUE
         DO 22 ID=1,ND
            DO 22 IX=1,NLEVEL
               POPUL2(IX,ID)=POPUL1(IX,ID)
   22    CONTINUE
         DO 23 ID=1,ND
            DO 23 IX=1,NLEVEL
               POPUL1(IX,ID)=POPUL(IX,ID)
   23    CONTINUE
         RETURN
      end if

      IF(ILAM.LT.IACCP) RETURN
C
      A1=0.
      B1=0.
      B2=0.
      C1=0.
      C2=0.
      DO 50 ID=1,ND
         DO 51 IX=1,NLEVEL
            IF(POPUL(IX,ID).NE.0.) WT=1./ABS(POPUL(IX,ID))
            D0=POPUL(IX,ID)-POPUL1(IX,ID)
            D1=D0-POPUL1(IX,ID)+POPUL2(IX,ID)
            D2=D0-POPUL2(IX,ID)+POPUL3(IX,ID)
            A1=A1+WT*D1*D1
            B1=B1+WT*D1*D2
            B2=B2+WT*D2*D2
            C1=C1+WT*D0*D1
            C2=C2+WT*D0*D2
   51    CONTINUE
   50 CONTINUE
         AB=B2*A1-B1*B1
         IF(AB.EQ.0.) THEN
            WRITE(6,601) ILAM,AB
            WRITE(10,601) ILAM,AB
            IACCP=IACCP+IACDP
            IACC0P=IACCP-3
            RETURN
         ENDIF
         A=(B2*C1-B1*C2)/AB
         B=(A1*C2-B1*C1)/AB
      DO 53 ID=1,ND
         DO 52 IX=1,NLEVEL
            POPUL(IX,ID)=(1.-A-B)*POPUL(IX,ID)+A*POPUL1(IX,ID)+
     *                   B*POPUL2(IX,ID)
   52    CONTINUE
   53 CONTINUE
      WRITE(6,600) ILAM
      WRITE(10,600) ILAM
      LAC2P=.TRUE.
c
c     call RESOLV after evaluating the accelerated estimate
c
C     CALL RESOLV
c      CALL TIMING(1,ITER)
      RETURN
  600 FORMAT(/,' **** ACCELP, ITER=',I4,/)
  601 FORMAT(/,' **** ACCELP, ITER=',I4,'   AB = ',F7.3,/)
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE TIMING(MOD,ITER)
C     ===========================
C
C     Timing procedure (call machine dependent routine!!)
C
      CHARACTER ROUT*20
      SAVE T0
C
      TIME=SECOND()
      DT=TIME-T0
      T0=TIME
      IF(MOD.EQ.1) THEN
         IP=ITER-1
         ROUT='     FORMAL SOLUTION'
      ELSE IF(MOD.EQ.2) THEN
         IP=ITER
         ROUT='       LINEARIZATION'
      ENDIF
      WRITE(69,600) IP,MOD,TIME,DT,ROUT
  600 FORMAT(2I4,2F11.2,2X,A20)
      RETURN
      END
C
C
C ********************************************************************
C
C
      function second()
c
c this machine-dependent function returns elapsed CPU time in seconds.
c This version is for the DEC 5000/200 running Ultrix
c
      dimension dummy(2)
      second=etime(dummy)
      return
      end
C
C
C ********************************************************************
C
C
c
      subroutine stop(text)
c
c     stops the program and writes a text
c
      character*(*) text
      write(6,10) text
      write(10,10) text
   10 format(1x,a)
      stop
      end
C
C
C ********************************************************************
C
C
      SUBROUTINE ODFSET
C     =================
C
C     Initialization of line ODF's
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      CHARACTER*30 FIDATA(MION),FIODF1(MION),FIODF2(MION),FIBFCS(MION)
      COMMON/STFCR/IDODF(MDODF),OFR(MFODF),OW(MFODF),OWSUB(MFODF),
     *          ODFL0(MDODF,MFODF),ODF2(MDEPTH),IFTRA(MTRANS),NDODF
      COMMON/IONFIL/FIDATA,FIODF1,FIODF2,FIBFCS
      DIMENSION DML(MDEPTH)
C
      IDSTD=ND*2/3
      NLASTE=NFREQ
      ITR0=0
      DO 5 ID=1,ND
         IF(DM(ID).GT.0) THEN
            DML(ID)=LOG(DM(ID))
          ELSE
            DML(ID)=ID
         END IF
    5 CONTINUE
C
      DO 500 ION=1,NION
         IND=INODF1(ION)
         IF(IND.LE.0) GO TO 500
         IND2=INODF2(ION)
         IF(FIODF1(ION).NE.' ') OPEN(IND,FILE=FIODF1(ION),STATUS='OLD')
         IF(FIODF2(ION).NE.' ') OPEN(IND2,FILE=FIODF2(ION),STATUS='OLD')
         READ(IND,*,END=500) NDODF
         IF(NDODF.GT.MDODF) CALL STOP('too many depths for an ODF')
         READ(IND,*) (IDODF(ID),ID=1,NDODF)
         IREC=0
   10    CONTINUE
         READ(IND,*,END=500) II,JJ,FR,NFRO,FAV
         IF(NFRO.GT.MFODF) CALL STOP('too many frequencies for an ODF')
         DO 20 IJ=1,NFRO
            READ(IND,*) OFR(IJ),OW(IJ),OWSUB(IJ)
   20    CONTINUE
         IND2=INODF2(ION)
         READ(IND2,*) ((ODFL0(ID,IF),ID=1,NDODF),IF=1,NFRO)
         IREC=IREC+1
C
         N0=NFIRST(ION)-1
         I=II+N0
         J=JJ+N0
         IF(J.GT.NLAST(ION)) GO TO 10
         IF(I.GE.NLAST(ION)) GO TO 500
         ITR=ITRA(I,J)
         IF(ITR.EQ.ITR0) THEN
            ITR1=0
            IF(IF1.EQ.1) THEN
               IFIJ=0
               DO 30 IT=1,NTRANS
                  IF(ILOW(IT).NE.I.OR.IUP(IT).NE.J) GO TO 30
                  IF(IT.EQ.ITR) GO TO 30
                  IFIJ=IFIJ+1
                  IFTRA(IT)=IFIJ
   30          CONTINUE
               IF1=0
            END IF
            DO 40 IT=1,NTRANS
               IF(IFTRA(IT).GT.0) THEN
                  ITR1=IT
                  GO TO 50
               END IF
   40       CONTINUE
   50       CONTINUE
            IF(ITR1.EQ.0) THEN
               WRITE(6,601) ITR,N0,II,JJ
               STOP
  601          FORMAT(' CONFLICT IN ODF INPUT; ITR=',4I5)
            END IF
            ITR=ITR1
            IFTRA(ITR)=0
            OSC0(ITR)=FAV
          ELSE
            ITR0=ITR
            IF1=1
            OSC0(ITR)=FAV
         END IF
C
         MODE=IABS(INDEXP(ITR))
         IF(MODE.EQ.3.OR.MODE.EQ.4) THEN
c            FR0(ITR)=FR
            LCOMP(ITR)=.FALSE.
            INTMOD(ITR)=5
         END IF
         IFRQ0=IFR0(ITR)
         IFRQ1=IFR1(ITR)
        IF(OFR(1).GE.OFR(NFRO)) THEN
         IF(MODE.EQ.3) THEN
            IFR0(ITR)=NLASTE+1
            IFR1(ITR)=NLASTE+NFRO
            DO 110 IJ=1,NFRO
               FREQ(IJ+NLASTE)=OFR(IJ)
               W(IJ+NLASTE)=OW(IJ)
               WSUB(IJ+NLASTE)=OWSUB(IJ)
  110       CONTINUE
            IF(NDODF.EQ.1) THEN
               DO 120 ID=1,ND
                  DO 120 IJ=1,NFRO
                     PRFLIN(IJ+NLASTE,ID)=ODFL0(1,IJ)
  120          CONTINUE
             ELSE
               DO 160 ID=1,ND
                  ID1=1
                  DO 130 IDO=1,NDODF-1
                     IF(ID.GE.IDODF(IDO).AND.ID.LE.IDODF(IDO+1)) THEN
                        ID1=IDO
                        ID2=IDO+1
                        GO TO 140
                     END IF
  130             CONTINUE                           
  140             CONTINUE 
                  IF(ID2.GT.NDODF) ID2=NDODF
                  IF(ID1.EQ.ID2) THEN
                     A1=1.
                     A2=0.
                   ELSE
C                    X=FLOAT(IDODF(ID2)-IDODF(ID1))
C                    A1=FLOAT(IDODF(ID2)-ID)/X                          
C                    A2=FLOAT(ID-IDODF(ID1))/X 
                     X=DML(IDODF(ID2))-DML(IDODF(ID1))
                     A1=(DML(IDODF(ID2))-DML(ID))/X                          
                     A2=UN-A1
                  END IF
                  DO 150 IJ=1,NFRO
                     IF(ODFL0(ID1,IJ).LE.0.OR.ODFL0(ID2,IJ).LE.0) THEN
                        PRFLIN(IJ+NLASTE,ID)=0.
                      ELSE
                        X=A1*LOG(ODFL0(ID1,IJ))+A2*LOG(ODFL0(ID2,IJ))
                        PRFLIN(IJ+NLASTE,ID)=EXP(X)
                     END IF
  150             CONTINUE                         
  160          CONTINUE 
            END IF                        
C
            IF(IPROF(ITR).EQ.0) THEN
               DO 170 ID=1,ND
                  PRFLIN(IFR1(ITR),ID)=0.
  170          CONTINUE
            END IF
            DO 180 IJ=1,NFRO
               PROF(IJ+NLASTE)=PRFLIN(IJ+NLASTE,IDSTD)
  180       CONTINUE 
C
            NLASTE=IFR1(ITR)
C        ELSE IF(INDEXP(ITR).EQ.4) THEN
         END IF
        ELSE
         IF(MODE.EQ.3) THEN
            IFR0(ITR)=NLASTE+1
            IFR1(ITR)=NLASTE+NFRO
            DO 210 IJ=1,NFRO
               FREQ(IJ+NLASTE)=OFR(NFRO-IJ+1)
               W(IJ+NLASTE)=OW(NFRO-IJ+1)
               WSUB(IJ+NLASTE)=OWSUB(NFRO-IJ+1)
  210       CONTINUE
            IF(NDODF.EQ.1) THEN
               DO 220 ID=1,ND
                  DO 220 IJ=1,NFRO
                     PRFLIN(IJ+NLASTE,ID)=ODFL0(1,NFRO-IJ+1)
  220          CONTINUE
             ELSE
               DO 260 ID=1,ND
                  ID1=1
                  DO 230 IDO=1,NDODF-1
                     IF(ID.GE.IDODF(IDO).AND.ID.LE.IDODF(IDO+1)) THEN
                        ID1=IDO
                        ID2=IDO+1
                        GO TO 240
                     END IF
  230             CONTINUE                           
  240             CONTINUE 
                  IF(ID2.GT.NDODF) ID2=NDODF
                  IF(ID1.EQ.ID2) THEN
                     A1=1.
                     A2=0.
                   ELSE
c                    X=FLOAT(IDODF(ID2)-IDODF(ID1))
c                    A1=FLOAT(IDODF(ID2)-ID)/X                          
c                    A2=FLOAT(ID-IDODF(ID1))/X 
                     X=DML(IDODF(ID2))-DML(IDODF(ID1))
                     A1=(DML(IDODF(ID2))-DML(ID))/X                          
                     A2=UN-A1
                  END IF
                  DO 250 IJ=1,NFRO
		     IJ0=NFRO-IJ+1
                     IF(ODFL0(ID1,IJ0).LE.0.OR.ODFL0(ID2,IJ0).LE.0) THEN
                        PRFLIN(IJ+NLASTE,ID)=0.
                      ELSE
                        X=A1*LOG(ODFL0(ID1,IJ0))+A2*LOG(ODFL0(ID2,IJ0))
                        PRFLIN(IJ+NLASTE,ID)=EXP(X)
                     END IF
  250             CONTINUE                         
  260          CONTINUE 
            END IF                        
C
            IF(IPROF(ITR).EQ.0) THEN
               DO 270 ID=1,ND
                  PRFLIN(IFR0(ITR),ID)=0.
  270          CONTINUE
            END IF
            DO 280 IJ=1,NFRO
               PROF(IJ+NLASTE)=PRFLIN(IJ+NLASTE,IDSTD)
  280       CONTINUE 
C
            NLASTE=IFR1(ITR)
C        ELSE IF(INDEXP(ITR).EQ.4) THEN
         END IF
        END IF
         IF(NLASTE.GT.MFREQ)
     *      CALL STOP(' too many frequencies (ODFSET)')
         IF(INDEXP(ITR).NE.0) THEN
           CALL IJALIS(ITR,IFRQ0,IFRQ1)
           IF(IOVER.EQ.0) CALL CORRQW(ITR)
         ENDIF
         GO TO 10
  500 CONTINUE
C
      NFREQ=NLASTE
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE ODFHYS(DOPO)
C     =======================
C
C     Initialization of line ODF's for hydrogen
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      PARAMETER (CCM=UN/2.997925D10,THIRD=UN/3.,FRH=3.28805D15)
      COMMON/ODFSTK/XKIJ(MHOD,NLMX),WL0(MHOD,NLMX),FIJ(MHOD,NLMX)
      DIMENSION FFRO(MFREQL)
C
      NLASTE=NFREQ
      DO 100 ITR=1,NTRANS
        JND=JNDODF(ITR)
        MODE=IABS(INDEXP(ITR))
        IF(JND.LE.0 .OR. MODE.NE.2) GO TO 100
        LCOMP(ITR)=.FALSE.
        INTMOD(ITR)=6
        I=ILOW(ITR)
        J=IUP(ITR)
        NQLODF(I)=IABS(IPROF(ITR))
        IF(NQLODF(I).EQ.0) NQLODF(I)=NQUANT(J)
        XJ2A=HALF*(XI2(NQUANT(J))+XI2(NQUANT(J)-1))
C
C     set up explicit frequencies & weights
C
        NFRO=0
        DO 10 IFQ=1,4
          NFRO=NFRO+KDO(IFQ,JND)
   10   CONTINUE
        NFRO=NFRO-2
        FRION=FRH*IZ(IEL(I))*IZ(IEL(I))
        FRA=FRION*(XI2(NQUANT(I))-XJ2A)
c        FR0(ITR)=FRA
        DOPI=DOPO*FRA*CCM
          IFRQ0=IFR0(ITR)
          IFRQ1=IFR1(ITR)
          IFR0(ITR)=NLASTE+1
          IFR1(ITR)=NLASTE+NFRO
          I1ODF(I)=IFR0(ITR)
          I2ODF(I)=IFR1(ITR)-1
          FFRO(1)=0.99999999*FRA
          FFRO(2)=FRA
          IJ00=1
          DO 30 IK=1,3
            DO 20 IJ=2,KDO(IK,JND)
              IJQ=IJ00+IJ
              FFRO(IJQ)=FFRO(IJQ-1)+XDO(IK,JND)*DOPI
   20       CONTINUE
            IJ00=IJ00+KDO(IK,JND)-1
   30     CONTINUE
          IJ00=IJ00+1
          FFRO(NFRO)=0.99999999*FRION*XI2(NQUANT(I))
          do while (ffro(ij00).ge.ffro(nfro))
             xdo(3,jnd)=0.75*xdo(3,jnd)
             ij00=ij00-kdo(3,jnd)
             do ij=2,kdo(3,jnd)
                ijq=ij00+ij
                ffro(ijq)=ffro(ijq-1)+xdo(3,jnd)*dopi
             enddo
             ij00=ij00+kdo(3,jnd)
          enddo
          TIDO=(FFRO(NFRO)-FFRO(IJ00))/FLOAT(KDO(4,JND)-1)
          DO 40 IJ=1,KDO(4,JND)-2
            IJQ=NFRO-IJ
            FFRO(IJQ)=FFRO(NFRO)-FLOAT(IJ)*TIDO
   40     CONTINUE
          DO 140 IJ=1,NFRO
             FREQ(NLASTE+IJ)=FFRO(NFRO-IJ+1)
  140     CONTINUE
          W(NLASTE+NFRO)=HALF*(FREQ(NLASTE+NFRO-1)-FREQ(NLASTE+NFRO))
          W(NLASTE+NFRO-1)=W(NLASTE+NFRO)
          DO 50 IJ=2,NFRO-2,2
            TIDO=(FREQ(NLASTE+IJ)-FREQ(NLASTE+IJ+1))*THIRD
            W(NLASTE+IJ-1)=W(NLASTE+IJ-1)+TIDO
            W(NLASTE+IJ)=W(NLASTE+IJ)+4.*TIDO
            W(NLASTE+IJ+1)=W(NLASTE+IJ+1)+TIDO
   50     CONTINUE
          DO 60 IJ=1,NFRO
            WSUB(NLASTE+IJ)=0.
   60     CONTINUE
          WSUB(NLASTE+NFRO)=HALF*(FREQ(NLASTE+NFRO)-FREQ(NLASTE+1))
          WSUB(NLASTE+1)=WSUB(NLASTE+NFRO)
          NLASTE=IFR1(ITR)
C
C     set up internal frequencies & Stark parameters
C
        CALL ODFFR(I,J)
        OSC0(ITR)=0.
        IS=NQUANT(I)
        DO 70 K=NQUANT(J),NLMX
          CALL STARK0(IS,K,XKIJ(JND,K),WL0(JND,K),FIJ(JND,K))
          OSC0(ITR)=OSC0(ITR)+FIJ(JND,K)
   70   CONTINUE
        IF(INDEXP(ITR).NE.0) THEN
          CALL IJALIS(ITR,IFRQ0,IFRQ1)
          IF(IOVER.EQ.0) CALL CORRQW(ITR)
        ENDIF
  100 CONTINUE
C
      NFREQ=NLASTE
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE ODFMER
C     =================
C
C     Opacity distribution function for superlines to merged states
C     (calculated only if DT/T>=CHTL, kept constant after).
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      PARAMETER (CHTL=1.D-3)
      COMMON/ODFCHT/CHANT(MDEPTH)
C
      DO 10 ITR=1,NTRANS
         IF(.NOT.LINE(ITR).OR.IABS(INDEXP(ITR)).NE.2) GO TO 10
         DO ID=1,ND
            IF(INIT.EQ.1 .OR. ABS(CHANT(ID)).GE.CHTL)
     *         CALL ODFHYD(ID,ITR)
         END DO
   10 CONTINUE
      RETURN
      END
C
C
C
C ********************************************************************
C
C
      SUBROUTINE ODFHYD(ID,ITR)
C     =========================
C
C     Line ODF's for hydrogen line series
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      PARAMETER (CDOP=TWO*BOLK/HMASS)
      PARAMETER (CA=2.997925D18,CCM=CA/1.D8,FRH=3.28805D15)
      PARAMETER (RYDEL=911.764,TTW=2./3.)
      PARAMETER (C00=1.25D-9,CID=0.02654)
      COMMON/ODFSTK/XKIJ(MHOD,NLMX),WL0(MHOD,NLMX),FIJ(MHOD,NLMX)
      COMMON/STRAUX/BETAD,ADH,DIVH
      DIMENSION SIG(MFRO),SGT(MFRO),ODF(MFRO),IODF(MFRO)
      DIMENSION YNUS(MFRO)
C
      JO=JNDODF(ITR)
      NF=NFRODF(JO)
      DO 10 IJ=1,NF
         IODF(IJ)=0
         SIG(IJ)=0.
         ODF(IJ)=0.
   10 CONTINUE
C
      II=ILOW(ITR)
      JJ=IUP(ITR)
      ANES=EXP(TTW*LOG(ELEC(ID)))
      F00=C00*ANES
      FRA=FRH*(XI2(NQUANT(II))-XI2(NQUANT(JJ)))
      DOPO=FRA/CCM*SQRT(CDOP*TEMP(ID)+VTB*VTB)
      DO 30 J=NQLODF(II),NLMX
         WL=RYDEL/(XI2(NQUANT(II))-XI2(J))
         FXK=F00*XKIJ(JO,J)
         DBETA=WL*WL/CA/FXK
         BETAD=DBETA*DOPO
         FID=CID*FIJ(JO,J)*DBETA
         CALL DIVSTR
         WPROB=WNHINT(J,ID)
         CALL ODFHST(NF,FXK,FID,WPROB,WL,FROS(1,JO),SGT)
         DO 20 IJ=1,NF
           SIG(IJ)=SIG(IJ)+SGT(IJ)
   20    CONTINUE
   30 CONTINUE
C
      CALL INDEXX(NF,SIG,IODF)
      DO 40 IJ=1,NF
         ODF(IJ)=SIG(IODF(IJ))
   40 CONTINUE
      I0=IFR0(ITR)
      I1=IFR1(ITR)
      IF(IABS(INDEXP(ITR)).EQ.2) YNUS(1)=FREQ(I0)
      IW1=IODF(1)
      DO 50 IJ=2,NF
         IW2=IODF(IJ)
         IF(IJ.GT.2 .AND. IJ.LT.NF) THEN
            YNUS(IJ)=YNUS(IJ-1)-HALF*(WNUS(IW1,JO)+WNUS(IW2,JO))
          ELSE IF (IJ.EQ.2) THEN
            YNUS(IJ)=YNUS(IJ-1)-HALF*(TWO*WNUS(IW1,JO)+WNUS(IW2,JO))
          ELSE IF (IJ.EQ.NF) THEN
            YNUS(IJ)=YNUS(IJ-1)-HALF*(WNUS(IW1,JO)+TWO*WNUS(IW2,JO))
         ENDIF
         IW1=IW2
   50 CONTINUE
C
      IF(IABS(INDEXP(ITR)).EQ.2) THEN
        PRFLIN(I1,ID)=1.D-35
        DO 80 IJQ=I0,I1-1
           DO 60 IJ=2,NF
              JI=IJ
              IF(YNUS(IJ).LE.FREQ(IJQ)) GO TO 70
   60      CONTINUE
   70      PRFLIN(IJQ,ID)=ODF(JI-1)+(ODF(JI)-ODF(JI-1))*
     *       (FREQ(IJQ)-YNUS(JI-1))/(YNUS(JI)-YNUS(JI-1))
   80   CONTINUE
      END IF
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE INDEXX(N,ARRIN,INDX)
C     ===============================
C
C     Sorting routine
C
      INCLUDE 'IMPLIC.FOR'
      DIMENSION ARRIN(N),INDX(N)
      DO 11 J=1,N
        INDX(J)=J
11    CONTINUE
      M=N/2+1
      IR=N
10    CONTINUE
        IF(M.GT.1)THEN
          M=M-1
          INDXT=INDX(M)
          Q=ARRIN(INDXT)
        ELSE
          INDXT=INDX(IR)
          Q=ARRIN(INDXT)
          INDX(IR)=INDX(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            INDX(1)=INDXT
            RETURN
          ENDIF
        ENDIF
        I=M
        J=M+M
20      IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(ARRIN(INDX(J)).LT.ARRIN(INDX(J+1)))J=J+1
          ENDIF
          IF(Q.LT.ARRIN(INDX(J)))THEN
            INDX(I)=INDX(J)
            I=J
            J=J+J
          ELSE
            J=IR+1
          ENDIF
        GO TO 20
        ENDIF
        INDX(I)=INDXT
      GO TO 10
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE SIGAVE(IBF)
C     ======================
C
C     Read bound-free cross-sections for averaged levels
C     from the unit INSA (given by IFANCY), with increasing frequencies
C     It assumes that all continuum transitions for a given ion are
C     given in a successive order in the data
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      PARAMETER (HCCM=H*2.997925D10)
      PARAMETER (TX=2.30258509299405,BAM=1.D-18)
      DIMENSION IBF(MLEVEL),XIFE(6)
      DIMENSION FRINSG(MFREQ),CRIN(MFREQ),JKF(MFREQ)
      CHARACTER*30 FIDATA(MION),FIODF1(MION),FIODF2(MION),FIBFCS(MION)
      COMMON/IONFIL/FIDATA,FIODF1,FIODF2,FIBFCS
C
      DATA XIFE/63480.,130563.,247220.,442000.,605000.,799000./
C
      NFREQB=NFREQ
      IF(IBFINT.GT.0) NFREQB=NFREQC
      ITR=0
   10 ITR=ITR+1
      IF(ITR.GT.NTRANS) RETURN
      IC=ITRA(IUP(ITR),ILOW(ITR))
      INSA=IBF(IC)
      IF(INSA.LT.50 .OR. INSA.GT.100) GO TO 10
      IE=IEL(ILOW(ITR))
      ITR=ITR-1
      NL1=NFIRST(IE)
      NL2=NLAST(IE)
      IF(FIBFCS(IE).NE.' ') THEN
         INSA=INBFCS(IE)
         OPEN(INSA,FILE=FIBFCS(IE),STATUS='OLD')
      END IF
      READ(INSA,*,END=500,ERR=500) IERR,IZRR,NLRR
      DO 100 I=NL1,NL2
        ITR=ITR+1
        IF(INDEXP(ITR).EQ.0) GO TO 100
        IC=ITRA(IUP(ITR),ILOW(ITR))
        READ(INSA,*) INL,ECMR,GDUM,NFIS
        IF(IERR.NE.26) GO TO 20
        ECMR=XIFE(IZRR)-ECMR
        DE=ABS((ENION(ILOW(ITR))-HCCM*ECMR)/ENION(ILOW(ITR)))
        IF(DE.GT.1.D-2)  THEN
          PRINT *,INSA,IE,ITR,I
          CALL STOP(' Incorrect energy level in SIGAVE')
        ENDIF
   20   DO 30 IJ=1,NFIS
          JI=NFIS-IJ+1
          READ(INSA,*,END=500,ERR=500) FRINSG(JI),CRIN(JI)
   30   CONTINUE
        DO 50 IJ=1,NFREQB
          JK=0
          DO 35 IK=1,NFIS
              IF(FREQ(IJ).GT.FRINSG(IK)) THEN
                 JK=IK
                 GO TO 40
              ENDIF
   35     CONTINUE
          JK=NFIS
   40     IF(JK.EQ.1) JK=2
          JKF(IJ)=JK
   50   CONTINUE
        DO 60 IJ=1,NFREQB
          JK=JKF(IJ)
	  IF(CRIN(JK-1).EQ.0. .OR. CRIN(JK).EQ.0.) THEN
            BFCS (IC,IJ)=CRIN(JK)+(FREQ(IJ)-FRINSG(JK))/
     *        (FRINSG(JK-1)-FRINSG(JK))*(CRIN(JK-1)-CRIN(JK))
          ELSE
            XF1=LOG10(FRINSG(JK-1))
            XF2=LOG10(FRINSG(JK))
            YS1=LOG10(CRIN(JK-1))
            YS2=LOG10(CRIN(JK))
            XXF=LOG10(FREQ(IJ))
            YYF=(XXF-XF2)/(XF1-XF2)*(YS1-YS2)+YS2
            BFCS (IC,IJ)=EXP(TX*YYF)
          ENDIF
          BFCS (IC,IJ)=BAM*BFCS (IC,IJ)
   60   CONTINUE 
  100 CONTINUE
      GO TO 10
  500 CALL STOP(' ERROR IN DATA FILE FOR BF SIG OF AVERAGED LEVELS')
      RETURN
      END
C
C
C *********************************************************************
C
c
      subroutine readbf
c     =================
c
c     auxiliary subroutine for enabling reading of input data with
c     comments
c
c     lines beginning with ! or * are understood as comments
c
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      character*80 buff
   10 continue
      read(5,501,end=20) buff
      if(buff(1:1).eq.'!'.or.buff(1:1).eq.'*') go to 10
      write(ibuff,501) buff
      go to 10
  501 format(a)
   20 continue
      rewind 15
      return 
      end
C
C
C     *******************************************************************
C
C

      SUBROUTINE CORRQW(ITR)
C     ======================
C
C     The routine for evaluating the so-called "subtraction weights" 
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (THR=3.D0, FOUR=4.D0, THIRD=UN/THR)
C
      DO 1 IJC=1,MFRCOR
         WCOR(IJC,ITR)=0.
         WCHCOR(IJC,ITR)=0.
    1 CONTINUE
      IF(ISUB(ITR).EQ.0.OR.IOVER.NE.0) RETURN
C
      I1=IFR0(ITR)
      I2=IFR1(ITR)
      FRL=FR0(ITR)
      IF(FRL.EQ.0.) FRL=UN
      FR1=FREQ(I1)
      FR2=FREQ(I2)
      INXA=IABS(INDEXP(ITR))
C
C     largest and smallest frequency within the line
C
      FRMX=FR1
      FRMN=FR1
      DO 10 IJ=I1,I2
         IF(FREQ(IJ).GT.FRMX) FRMX=FREQ(IJ)
         IF(FREQ(IJ).LT.FRMN) FRMN=FREQ(IJ)
   10 CONTINUE
C
C     correction for the case the line is represented by a half profile
C
      IF(ABS(FRMX-FRL)/FRL.LT.1.E-8.AND.INXA.EQ.1) THEN
         FRMX=FRL+(FRMX-FRMN)
      END IF
      IF(FRMX.GT.FREQ(1)) CALL STOP('LINE FREQUENCY GREATER THAN MAX')
C 
C     determination of the position of the line in the set of continuum
C     frequency points;
C     i.e. determination of for which continuum frequency points the
C     quadrature weight will have to be modified by the presence of
C     line ITR
C
      DO 20 IJ=1,NFREQC
         IF(FRMX.LE.FREQ(IJ)) IMAX=IJ
         IF(FRMN.LE.FREQ(IJ)) THEN
            IMIN=IJ
          ELSE
            GO TO 30
         END IF
   20 CONTINUE
      RETURN
   30 CONTINUE
C
C     two cases where the line may be located:
C
C     1. the immediately larger continuum frequency is not an endpoint of a
C        partial frequency integration - it must be a midpoint of 
C        a Simpson integration
C
      ISIMP=0
      IMX0=IMAX
      IF(WCH(IMAX).EQ.0.) THEN 
         ISIMP=1
         IF(W(IMAX).GT.W(IMAX+1)-WCH(IMAX+1)) IMX0=IMAX-1
       ELSE      
C
C     2. the immediately larger continuum frequency is an endpoint of a
C        partial frequency integration.
C
         IF(WCH(IMAX+1).EQ.0..AND.IMAX+1.LT.NFREQC) ISIMP=1
      END IF
      IFRC0(ITR)=IMX0
      II0=IMX0-1
C
C     Now, if the whole line is located within this interval, the
C     corection weights will be modified only for this interval.
C     If not, one evaluates a correction weights for an integral from
C     the maximum frequency within the line till the end of the current
C     interval (the low-frequency side of this inteval)
C
      IMX1=IMX0+1
      IF(ISIMP.EQ.1) IMX1=IMX0+2
      IF(FRMN.GE.FREQ(IMX1)) THEN
         HH=FRMX-FRMN
         IEND=1
       ELSE
         HH=FRMX-FREQ(IMX1)
         IEND=0
      END IF
      X=FRMX
      X1=FREQ(IMX0)
      H0=FREQ(IMX0)-FREQ(IMX0+1)
C
C      a. Simpson integration interval
C
      IF(ISIMP.EQ.1) THEN
         C1=X*X+X*HH+HH*HH*THIRD
         C2=X+HH*HALF
         C0=HH/(TWO*H0*H0)
         WC0=C0*(C1-C2*(TWO*X1+THR*H0)+X1*X1+THR*X1*H0+TWO*H0*H0)
         WC1=C0*(-TWO*C1+FOUR*C2*(X1+H0)-TWO*(X1*X1+TWO*X1*H0))
         WC2=C0*(C1-C2*(TWO*X1+H0)+X1*X1+X1*H0)
         WCOR(IMX0-II0,ITR)=WCOR(IMX0-II0,ITR)+WC0
         WCOR(IMX0+1-II0,ITR)=WCOR(IMX0+1-II0,ITR)+WC1
         WCOR(IMX0+2-II0,ITR)=WCOR(IMX0+2-II0,ITR)+WC2
         WCHCOR(IMX0-II0,ITR)=WCHCOR(IMX0-II0,ITR)+WC0
         IMN0=IMX0+2
       ELSE
C                
C      b. trapezoidal integration interval
C          
         C0=HH/H0
         C1=X+HH*HALF   
         WCOR(IMX0-II0,ITR)= WCOR(IMX0-II0,ITR)+C0*(H0+X1-C1)
         WCOR(IMX0+1-II0,ITR)=WCOR(IMX0+1-II0,ITR)+C0*(C1-X1)
         WCHCOR(IMX0-II0,ITR)=WCHCOR(IMX0-II0,ITR)+WCOR(IMX0-II0,ITR)
         IMN0=IMX0+1
      END IF
C
      IF(IEND.EQ.1) GO TO 100
C
C     If the line extends to another interval, repeat the above procedure
C     of finding what type of interval is it (Simpson or trapezoidal),
C     construct correction weights, and look whether the line extenDS to still
C     another inteval. 
C     This procedure is repeated until the low-frequency end of the line
C     is reached.
C
   50 ISIMP=0
      IMX0=IMX1
      IF(WCH(IMX0+1).EQ.0.) ISIMP=1
C
      IMX1=IMX0+1
      IF(ISIMP.EQ.1) IMX1=IMX0+2
      IF(FRMN.GE.FREQ(IMX1)) THEN
         HH=FREQ(IMX0)-FRMN
         IEND=1
       ELSE
         HH=FREQ(IMX0)-FREQ(IMX1)
         IEND=0
      END IF
      X=FREQ(IMX0)
      X1=X
      H0=FREQ(IMX0)-FREQ(IMX0+1)
C
C      a. Simpson integration interval
C
      IF(ISIMP.EQ.1) THEN
         C1=X*X+X*HH+HH*HH*THIRD
         C2=X+HH*HALF
         C0=HH/(TWO*H0*H0)
         WC0=C0*(C1-C2*(TWO*X1+THR*H0)+X1*X1+THR*X1*H0+TWO*H0*H0)
         WC1=C0*(-TWO*C1+FOUR*C2*(X1+H0)-TWO*(X1*X1+TWO*X1*H0))
         WC2=C0*(C1-C2*(TWO*X1+H0)+X1*X1+X1*H0)
         WCOR(IMX0-II0,ITR)=WCOR(IMX0-II0,ITR)+WC0
         WCOR(IMX0+1-II0,ITR)=WCOR(IMX0+1-II0,ITR)+WC1
         WCOR(IMX0+2-II0,ITR)=WCOR(IMX0+2-II0,ITR)+WC2
         WCHCOR(IMX0-II0,ITR)=WCHCOR(IMX0-II0,ITR)+WC0
         IMN0=IMX0+2
       ELSE
C                
C      b. trapezoidal integration interval
C               
         C0=HH/H0
         C1=X+HH*HALF 
         WC0=C0*(H0+X1-C1)
         WC1=C0*(C1-X1) 
         WCOR(IMX0-II0,ITR)=WCOR(IMX0-II0,ITR)+WC0
         WCOR(IMX0+1-II0,ITR)=WCOR(IMX0+1-II0,ITR)+WC1
         WCHCOR(IMX0-II0,ITR)=WCHCOR(IMX0-II0,ITR)+WC0
         IMN0=IMX0+1
      END IF
C
      IF(IEND.EQ.0) GO TO 50
  100 CONTINUE
      IFRC1(ITR)=IMN0
      IF(IFRC1(ITR)-IFRC0(ITR).GE.MFRCOR) THEN
         WRITE(6,*) 'ITR= ',ITR
         WRITE(10,*) 'ITR= ',ITR
         CALL STOP(' TOO MANY CORRECTION WEIGTHS')
      END IF
      sum=0.
      do 101 i=ifrc0(itr),ifrc1(itr)
  101    sum=sum+wcor(i-ifrc0(itr)+1,itr)
      sumw=0.
      do 102 i=ifr0(itr),ifr1(itr)
  102    sumW=sumW+w(i)
      RETURN
      END
C
C
C     *******************************************************************
C
C
   
      SUBROUTINE CORRWM
C     =================
C
C     The routine for management of various flags for treating 
C     frequency points; in particular those connected to the so-called 
C     "subtraction weights" (in the non-overlapping mode only)
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (T15=1.D-15)
C
      NFREQE=0
      DO 10 IJ=1,NFREQ
         IJEX(IJ)=0.
         LSKIP(IJ)=.FALSE.
         IF(IJALI(IJ).NE.0) GO TO 10
         NFREQE=NFREQE+1
         IJEX(IJ)=NFREQE
         IJFR(NFREQE)=IJ
   10 CONTINUE
C
      IF(IBFINT.LE.0) THEN
	 DO IJ=1,NFREQ
	    IJBF(IJ)=IJ
	    AIJBF(IJ)=UN
         END DO
       ELSE
         DO IJ=1,NFREQC
	    IJBF(IJ)=IJ
	    AIJBF(IJ)=UN
         END DO
	 IF(NFREQ.GT.NFREQC) THEN
	 DO IJ=NFREQC+1,NFREQ
	    FR=FREQ(IJ)
	    IJ0=1
	    DO IJT=1,NFREQC
	       IF(FREQ(IJT).LE.FR) THEN
		  IJ0=IJT
		  GO TO 12
               END IF
            END DO
   12       IJ1=IJ0-1
	    A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))
	    IJBF(IJ)=IJ1
	    AIJBF(IJ)=A1
         END DO
         END IF
      END IF
C
c      do ij=1,nfreq
c	 write(6,601) ijbf(ij),aijbf(ij)
c      end do
c  601 format(i6,f10.5)
C
      if(nfreqe.gt.mfrex) call stop('nfreqe too large')
      DO 20 IJE=1,NFREQE
         WDC0(IJE)=0.
         WCTOTE(IJE)=0.
   20 CONTINUE
C
      DO 100 ITR=1,NTRANS
         IF(.NOT.LINE(ITR)) GO TO 100
C
C        first set up array LSKIP(IJ), which has values
C        TRUE - if the radiation at frequency point IJ does not contribute
C               radiation pressure (ie. this point belongs to a transition
C               for which the user required the radiation pressure to be
C               skipped - IABS(INDEXP) chosen as 9 or 19)
C        FALSE - normal calculation of radiation pressure
C
         INX=IABS(INDEXP(ITR))
         IF(INX.EQ.9.OR.INX.GE.19) THEN
            DO 25 IJ=IFR0(ITR),IFR1(ITR)
               LSKIP(IJ)=.TRUE.
   25       CONTINUE
         END IF
C
         IF(ISUB(ITR).EQ.0) THEN
            DO 30 IJ=IFRC0(ITR),IFRC1(ITR)
               WCOR(IJ-IFRC0(ITR)+1,ITR)=0.
   30       CONTINUE
          ELSE IF(ISUB(ITR).GT.0) THEN
            DO 40 IJ=IFR0(ITR),IFR1(ITR)
               WSUB(IJ)=0.
   40       CONTINUE
            DO 50 IJ=IFRC0(ITR),IFRC1(ITR)
               IJE=IJEX(IJ)
               IF(IJE.LE.0) GO TO 50
               WCTOTE(IJE)=WCTOTE(IJE)+WCOR(IJ-IFRC0(ITR)+1,ITR)
   50       CONTINUE
            INX=IABS(INDEXP(ITR))
            IF(INX.EQ.9.OR.INX.GE.19) THEN
               DO 70 IJ=IFRC0(ITR),IFRC1(ITR)
                  IJE=IJEX(IJ)
                  WDC0(IJE)=WDC0(IJE)+WCOR(IJ-IFRC0(ITR)+1,ITR)
   70          CONTINUE
            END IF
          ELSE
            DO 80 IJ=IFR0(ITR),IFR1(ITR)
               WSUB(IJ)=0.
   80       CONTINUE
         END IF
         IF(IOVER.GT.0) THEN
            DO 90 IJ=IFR0(ITR),IFR1(ITR)
               WSUB(IJ)=0.
   90       CONTINUE
         END IF
  100 CONTINUE
C            
      WRITE(6,609)
      DO 110 IJ=1,NFREQ
         FR15=FREQ(IJ)*T15
         W0E(IJ)=(W(IJ)-WSUB(IJ))*PI4H/FREQ(IJ)
         W0EC(IJ)=W(IJ)
         WCHCE(IJ)=WCH(IJ)
         BNUE(IJ)=BN*FR15*FR15*FR15
         IF(IJALI(IJ).NE.0) GO TO 110
         WRITE(6,610) IJ,FREQ(IJ),W(IJ),PROF(IJ),
     *                WSUB(IJ)
  110 CONTINUE
C
      DO 140 IJ=1,NFREQ
         WC(IJ)=W(IJ)-WSUB(IJ)
C        IF(IJ.LE.NFREQC.AND.IJALI(IJ).LE.0) WC(IJ)=0.
         IF(IJALI(IJ).LE.0) WC(IJ)=0.
  140 CONTINUE
C
C     *** correction (begin)
           DO 190 ITRF=1,NTRANS
              IF(LINEXP(ITRF)) GO TO 190
              IF(ISUB(ITRF).NE.0) THEN             
              DO 170 IJ=IFRC0(ITRF),IFRC1(ITRF)
                 W0EC(IJ)=W0EC(IJ)-WCOR(IJ-IFRC0(ITRF)+1,ITRF)
                 WCHCE(IJ)=WCHCE(IJ)-WCHCOR(IJ-IFRC0(ITRF)+1,ITRF)
  170         CONTINUE
              END IF
              IF(ISUB(ITRF).LT.0) THEN             
              DO 180 IJ=IFRC0(ITRF),IFRC1(ITRF)
                 WC(IJ)=WC(IJ)-WCOR(IJ-IFRC0(ITRF)+1,ITRF)
  180         CONTINUE
              END IF
  190       CONTINUE
      DO 200 IJ=1,NFREQ
         PIFQ=PI4H/FREQ(IJ)
         W0EC(IJ)=(W0EC(IJ)-WSUB(IJ))*PIFQ
         WCHCE(IJ)=WCHCE(IJ)*PIFQ
  200  CONTINUE
C      *** correction (end)
C
      IF(IOVER.GT.0) THEN
      DO 210 IJ=1,NFREQ
         WC(IJ)=W(IJ)
         WSUB(IJ)=0.
         IF(IJALI(IJ).LE.0) WC(IJ)=0.
  210  CONTINUE
      DO 220 IJ=1,NFREQE
         WCTOTE(IJ)=0.
         WDC0(IJ)=0.
  220 CONTINUE
      END IF
  609 FORMAT(1H0//' FREQUENCY POINTS AND WEIGHTS - EXPLICIT'/
     *            ' ---------------------------------------'//
     * '    IJ',7X,'FREQ',13X,'WEIGHT',11X,'PROF',10X,
     * 'WSUB'/)
  610 FORMAT(1H ,I5,1P2D17.8,D15.5,D17.8)
      RETURN 
      END
C
C
C     *******************************************************************
C
C
   
      SUBROUTINE IJALIS(ITR,IFRQ0,IFRQ1)
C     ==================================
C
C     auxiliary routine - sets up the necessary flags for ALI treatment
C      of individual transitions (in the fully hybrid CL/ALI scheme)
C
C     Output:
C
C     IJALI(IJ) = 0 - frequency point IJ is an explicit point
C               = 1 - frequency point IJ is an ALI point
C
C     LEXP(ITR) = T - at least one point within transition ITR is explicit
C     LEXP(ITR) = F - no point within transition ITR is explicit
C     LALI(ITR) = T - at least one point within transition ITR is ALI
C     LALI(ITR) = F - no point within transition ITR is ALI
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
C
      INDXP=INDEXP(ITR)
      I0=IFR0(ITR)
      I1=IFR1(ITR)
      NF=I1-I0+1
      DO 10 IJ=I0,I1
         IF(INDXP.GT.0) THEN
            IJALI(IJ)=0
          ELSE IF(INDXP.LT.0) THEN
            IJALI(IJ)=1
         END IF
   10 CONTINUE
C
C     primarily explicit transitions
C
      IF(INDXP.GT.0) THEN
         LEXP(ITR)=.TRUE.
         LALI(ITR)=.FALSE.
         ISUB(ITR)=1
         IF(IFRQ0.GT.0) THEN
            LALI(ITR)=.TRUE.
            ISUB(ITR)=-1
            IF(IFRQ1.EQ.0.OR.IFRQ1.GT.NF) IFRQ1=NF
            DO 20 I=IFRQ0,IFRQ1
                IJALI(I0+I-1)=1
   20       CONTINUE
          ELSE IF(IFRQ0.LT.0) THEN
            LALI(ITR)=.TRUE.
            READ(57,*) (IJALI(IJ),IJ=I0,I1),ISUB(ITR)
         END IF
         IF(IFRQ0.EQ.1.AND.IFRQ1.EQ.NF) LEXP(ITR)=.FALSE.
       ELSE IF(INDXP.LT.0) THEN
C
C      primarily ALI transitions
C
         LALI(ITR)=.TRUE.
         LEXP(ITR)=.FALSE.
         ISUB(ITR)=-1
         IF(IFRQ0.GT.0) THEN
            LEXP(ITR)=.TRUE.
            IF(IFRQ1.EQ.0.OR.IFRQ1.GT.NF) IFRQ1=NF
            DO 30 I=IFRQ0,IFRQ1
                IJALI(I0+I-1)=0
   30       CONTINUE
          ELSE IF(IFRQ0.LT.0) THEN
            LEXP(ITR)=.TRUE.
            READ(57,*) (IJALI(IJ),IJ=I0,I1),ISUB(ITR)
         END IF
         IF(IFRQ0.EQ.1.AND.IFRQ1.EQ.NF) LALI(ITR)=.FALSE.
      END IF
      IF(NFFIX.GT.0) THEN
         DO 40 IJ=I0,I1
            IJALI(IJ)=1
   40    CONTINUE
         LALI(ITR)=.TRUE.
         LEXP(ITR)=.FALSE.
         ISUB(ITR)=-1
      END IF
      IF(IFSUB.GT.0) ISUB(ITR)=IFSUB-2
      IF(ISUB(ITR).LT.0) LALI(ITR)=.TRUE.
      RETURN
      END
C
C
C     *******************************************************************
C
C
   
      SUBROUTINE LEVSET
C     =================
C
C     sets up level parameters IIEXP and IIFOR which control the
C     treatment of levels
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
C
C     1. case - treatment of levels determined by IMODL
C
      IF(IFLEV.EQ.0) THEN
      DO 10 I=1,NLEVEL
         IIEXP(I)=0
         IIFOR(I)=0
   10 CONTINUE
      IIE=0
      IIF=0
      DO 20 I=1,NLEVEL
         IF(IIFIX(IATM(I)).EQ.1) GO TO 20
         IF(IMODL(I).EQ.0) THEN
            IIE=IIE+1
            IIF=IIF+1
            IIEXP(I)=IIE
            IIFOR(I)=IIF
            INDLEV(IIEXP(I))=I
          ELSE IF(IMODL(I).GT.0) THEN
            IIF=IIF+1
            IIFOR(I)=IIF
            IF(ILTLEV(I).GE.1) THEN
               IIE=IIE+1
               IIEXP(I)=IIE
            END IF
            IF(I.EQ.NFIRST(IEL(I)).OR.I.EQ.NNEXT(IEL(I))) THEN
               IIE=IIE+1
               IIEXP(I)=IIE
            END IF
         END IF
   20 CONTINUE
      NLVEXP=IIE
      if(nlvexp.gt.mlvexp) call stop('nlvexp too large')
      NLVFOR=IIF
      DO 30 I=1,NLEVEL
         IF(IMODL(I).EQ.1.OR.IMODL(I).GE.3) THEN
            IIEXP(I)=-NNEXT(IEL(I))
          ELSE IF(IMODL(I).EQ.-1.OR.IMODL(I).LE.-3) THEN
            IIEXP(I)=-NNEXT(IEL(I))
            IIFOR(I)=-NNEXT(IEL(I))
         END IF
         IF(IMODL(I).EQ.6) THEN
            IIEXP(I)=0
          ELSE IF(IMODL(I).EQ.-6) THEN
            IIEXP(I)=0
            IIFOR(I)=0
         END IF
         DO ID=1,ND
            ILTREF(I,ID)=NNEXT(IEL(I))
         END DO      
   30 CONTINUE
C
C     2. case - treatment of levels automatic - all levels with ILK=0
C               in updated LTE mode
C
      ELSE
      IIF=0
      DO 110 I=1,NLEVEL
         IF(IIFIX(IATM(I)).EQ.1) GO TO 110
         IMODL(I)=5
c        IF(ILK(I).GT.0) THEN
         IF(I.EQ.NFIRST(IEL(I)).OR.I.EQ.NNEXT(IEL(I))) THEN
            IIF=IIF+1
            IIFOR(I)=IIF
         END IF
  110 CONTINUE
      NLVEXP=IIF
      if(nlvexp.gt.mlvexp) call stop('nlvexp too large')
      NLVFOR=IIF
      DO 120 I=1,NLEVEL
         IF(I.NE.NFIRST(IEL(I)).AND.I.NE.NNEXT(IEL(I)))
     *   IIFOR(I)=-NNEXT(IEL(I))
  120 CONTINUE
      DO 130 I=1,NLEVEL
         IIEXP(I)=IIFOR(I)
         IF(IIEXP(I).GT.0) INDLEV(IIEXP(I))=I
  130 CONTINUE
      if(.not.lte) then
      do 140 i=1,nlevel
         iifor(i)=i
  140 continue
      nlvfor=nlevel
      end if
c      
      END IF
C     
C     initialize b-factors
C
      DO 200 I=1,NLEVEL
         DO 200 ID=1,ND
            BFAC(I,ID)=UN
  200 CONTINUE
      RETURN
      END
C
C
C     *******************************************************************
C
C

      SUBROUTINE DWNFR0(ID)
C     =====================
C
C     Auxiliary quantities for dissolved fractions
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (SIXTH=UN/6.,CCOR=0.09)
      parameter (p1=0.1402,p2=0.1285,p3=un,p4=3.15,p5=4.)
      parameter (f23=-2./3.)
C
      ANE=ELEC(ID)
      ELEC23(ID)=EXP(F23*LOG(ANE))
      ANES=EXP(SIXTH*LOG(ANE))
      ACOR(ID)=CCOR*ANES/SQT1(ID)
      X=EXP(P4*LOG(UN+P3*ACOR(ID)))
      DWC2(ID)=P2*X
      A3=ACOR(ID)*ACOR(ID)*ACOR(ID)
      DO 10 IZZ=1,MZZ
         Z3(IZZ)=IZZ*IZZ*IZZ
         DWC1(IZZ,ID)=P1*(X+P5*IZZ*A3)
   10 CONTINUE
      RETURN
      END
C
C
C ********************************************************************
C
C
      SUBROUTINE DWNFR1(FR,FR0,ID,IZZ,DW1)
C     ====================================
C
C     dissolved fraction for frequency FR
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (TKN=3.01,CKN=5.33333333,CB=8.3d14)
      PARAMETER (SQFRH=5.734152D7)
C
      IF(FR.LT.FR0) THEN
         XN=SQFRH*IZZ/SQRT(FR0-FR)
         if(xn.ge.tkn) then
            xkn=un
          else
            xn1=un/(xn+un)
            xkn=ckn*xn*xn1*xn1
         end if
         BETA=CB*Z3(IZZ)*XKN/(XN*XN*XN*XN)*ELEC23(ID)
         BETA3=BETA*BETA*BETA
         BETA32=SQRT(BETA3)
         F=(DWC1(IZZ,ID)*BETA3)/(UN+DWC2(ID)*BETA32)
         DW1=UN-F/(UN+F)
       ELSE
         DW1=UN
      END IF
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE SGMER0
C     =================
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (FRH=3.28805E15, PH2=2.815D29*2., EHB=157802.77355)
      DIMENSION FREDG(NLMX),S(NLMX),SUM(NLMX),SUD(NLMX)
C
      IMER=0
      DO 100 II=1,NLEVEL
         IF(IFWOP(II).GE.0) GO TO 100
         IMER=IMER+1
         IMRG(II)=IMER
         IIMER(IMER)=II
         IE=IEL(II)
         CH=IZ(IE)*IZ(IE)
         FRCH(IMER)=FRH*CH
         SGM0(IMER)=PH2*CH*CH
         II0=NQUANT(II-1)+1
         DO 90 ID=1,ND
            EX=EHB*CH*TEMP1(ID)
c            SGEXT1(IMER,ID)=EX*TEMP1(ID)
            DO 10 I=II0,NLMX
               FREDG(I)=FRCH(IMER)*XI2(I)
               EXI=EXP(EX*XI2(I))
               S(I)=EXI*WNHINT(I,ID)*XI3(I)
               SUM(I)=0.
c               SUD(I)=0.
   10       CONTINUE
            SUM(NLMX)=S(NLMX)
            SUD(NLMX)=S(NLMX)*XI2(NLMX)
            DO 20 I=NLMX-1,II0,-1
               SUM(I)=SUM(I+1)+S(I)
   20       CONTINUE
            DO 30 I=1,II0-1
               SUM(I)=SUM(II0)
   30       CONTINUE
c            DO 40 I=NLMX-1,II0,-1
c               SUD(I)=SUD(I+1)+S(I)*XI2(I)
c   40       CONTINUE
c            DO 50 I=1,II0-1
c               SUD(I)=SUD(II0)
c   50       CONTINUE
            SGEM=SGM0(IMER)/GMER(IMER,ID)
c            SGEMD=SGEM*SGEXT1(IMER,ID)
            DO 60 I=1,NLMX
               SGMSUM(I,IMER,ID)=SUM(I)*SGEM
c               SGMSUD(I,IMER,ID)=SUD(I)*SGEMD
   60       CONTINUE
   90    CONTINUE
  100 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE SGMER1(FRINV,FR3INV,IMER,ID,SGME1)
C     =============================================
C
C     photoionization cross-section for a merged level
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
C
      ISU=INT(SQRT(FRCH(IMER)*FRINV))+1
      SGME1=SGMSUM(ISU,IMER,ID)*FR3INV
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE SGMERD(FRINV,FR3INV,IMER,ID,SGME1,DSGME1)
C     ====================================================
C
C     photoionization cross-section for a merged level
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
C
      ISU=INT(SQRT(FRCH(IMER)*FRINV))+1
      SGME1=SGMSUM(ISU,IMER,ID)*FR3INV
      DSGME1=-SGMSUD(ISU,IMER,ID)*FR3INV
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE TDPINI
C     =================
C
C     initialization of only temperature dependent quantities
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (CFF1=1.3727D-25,CFF2=4.3748D-10,CFF3=2.5993D-7)
      PARAMETER (SGFF0 = 3.694D8)
C
C     temperature-dependent quantities
C
      DO 10 ID=1,ND
         T=TEMP(ID)
         T1=UN/T
         HKT1(ID)=HK*T1
         HKT21(ID)=HKT1(ID)*T1
         TK1(ID)=HKT1(ID)/H
         SQT1(ID)=SQRT(T)
         TEMP1(ID)=T1
         CALL GFREE0(ID)
         EMEL1(ID)=UN
   10 CONTINUE
C
C     delta m (for evaluation of optical depths)
C
      DO 20 ID=1,ND-1
         DELDM(ID)=HALF*(DM(ID+1)-DM(ID))
   20 CONTINUE
      DEDM1=DM(1)/DENS(1)
      RETURN
      END
C
C
C     ****************************************************************
C
C
C
      SUBROUTINE OPAINI(IMOD)
C     =======================
C
C     initialization of only depth-dependent quantities
C     for evaluation of opacities
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      COMMON/UPSUMS/DUSUMT(MION),DUSUMN(MION)
      DIMENSION PRF(MFREQL)
      PARAMETER (CFF1=1.3727D-25,CFF2=4.3748D-10,CFF3=2.5993D-7)
      PARAMETER (SIXTH=UN/6.,CCOR=0.09,T32=1.5D0)
      PARAMETER (SGFF0 = 3.694D8)
      DATA ICOMP /0/
C
      DO 100 ID=1,ND
         WMT=WMM(ID)*YTOT(ID)
         T=TEMP(ID)
         ANE=ELEC(ID)
         ELEC1(ID)=UN/ANE
         DENS1(ID)=UN/DENS(ID)
         ELSCAT(ID)=ANE*SIGE
         CALL DWNFR0(ID)
         CALL WNSTOR(ID)
         CALL SABOLF(ID)
         CALL REFLEV(ID,IMOD)
         DO II=1,NLEVEL
            POPINV(II,ID)=0.
            IF(POPUL(II,ID).NE.0.) POPINV(II,ID)=UN/POPUL(II,ID)
         ENDDO
         DO II=1,NLEVEL
            IF(IIEXP(II).LE.0) THEN
               IE=ILTREF(II,ID)
               PT(II,ID)=POPUL(II,ID)*DSBPST(II,ID)
               PN(II,ID)=POPUL(II,ID)*DSBPSN(II,ID)
               PP(II,ID)=POPUL(II,ID)*POPINV(IE,ID)
            END IF
         ENDDO
         DO ION=1,NION
            USUMS(ION,ID)=USUM(ION)
            DUSMT(ION,ID)=DUSUMT(ION)
            DUSMN(ION,ID)=DUSUMN(ION)
         ENDDO
c
c        quantities for the bound-free opacity
c
         DO 10 IBF=1,NTRANC
            ITR=ITRBF(IBF)
            II=ILOW(ITR)
            JJ=IUP(ITR)
            IT=ITRA(JJ,II)
            IE=IEL(II)
            NKE=NNEXT(IE)
            CORR=UN
            IF(NKE.NE.JJ) CORR=G(NKE)/G(JJ)*
     *         EXP((ENION(NKE)-ENION(JJ))*TK1(ID))
            ABTRA(ITR,ID)=POPUL(II,ID)
            EMTRA(ITR,ID)=POPUL(JJ,ID)*ANE*SBF(II)*WOP(II,ID)*CORR
            DEMLT(ITR,ID)=-(T32+FR0(ITR)*HKT1(ID))/TEMP(ID)
   10    CONTINUE
c
c        quantities for the free-free opacity
c
         IF(IELHM.GT.0) THEN
            CFFN(ID)=POPUL(NFIRST(IELH),ID)*ANE
            CFFT(ID)=CFF2-CFF3/T
         END IF
         SGFF=SGFF0/SQT1(ID)*ANE
         DO 20 ION=1,NION
            SFF2(ION,ID)=EXP(FF(ION)*HKT1(ID))
            SFF3(ION,ID)=POPUL(NNEXT(ION),ID)*CHARG2(ION)*SGFF
            DSFF(ION,ID)=(FF(ION)*HKT1(ID)+HALF)*TEMP1(ID)
   20    CONTINUE
  100 CONTINUE
      CALL SGMER0
C
C     initialization of the line opacity
C
      LASER=ITER.GT.ITLAS
      DO 200 ITR=1,NTRANS
         IF(.NOT.LINE(ITR)) GO TO 200
         II=ILOW(ITR)
         JJ=IUP(ITR)
         IF(INTMOD(ITR).NE.0 .AND. ICOMP.EQ.0) THEN
           INDXA=IABS(INDEXP(ITR))
           IJL0=IFR0(ITR)
           IJL1=IFR1(ITR)
           IF(INDXA.LT.2.OR.INDXA.GT.4) THEN
             DO 150 ID=1,ND
               CALL LINPRO(ITR,ID,PRF)
               DO 110 IJ=IJL0,IJL1
                  PRFLIN(IJ,ID)=PRF(IJ-IJL0+1)
  110          CONTINUE
  150        CONTINUE
           ENDIF
         END IF
         GG=G(II)/G(JJ)
         DO 160 ID=1,ND
            IF(IFWOP(JJ).GE.0) THEN
               PI=POPUL(II,ID)*WOP(JJ,ID)
               PJ=GG*POPUL(JJ,ID)*WOP(II,ID)
             ELSE
               PI=POPUL(II,ID)
               PJ=G(II)/GMER(IMRG(JJ),ID)*POPUL(JJ,ID)*WOP(II,ID)
            END IF
            ABTRA(ITR,ID)=PI
            EMTRA(ITR,ID)=PJ*EXP(FR0(ITR)*HKT1(ID))
            DEMLT(ITR,ID)=-FR0(ITR)*HKT21(ID)
            IF(PI.LT.PJ.AND.LASER) THEN
               ABTRA(ITR,ID)=0.
               EMTRA(ITR,ID)=0.
               DEMLT(ITR,ID)=0.
            END IF
  160    CONTINUE
  200 CONTINUE
      ICOMP=1
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE TRAINI
C     =================
C
C     initialization of depth-independent quantities
C     for evaluation of opacities
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
C
C     bound-free transitions
C
      NCDW=0
      DO 10 IBF=1,NTRANC
         ITR=ITRBF(IBF)
         MODW=IABS(INDEXP(ITR))
         IF(MODW.NE.5.AND.MODW.NE.15) GO TO 10
         NCDW=NCDW+1
         MCDW(ITR)=NCDW
         ITRCDW(NCDW)=ITR
   10 CONTINUE
C
C     bound-bound transitions
C
      DO 20 IJ=1,NFREQ
         NLINES(IJ)=0
   20 CONTINUE
C
      DO 100 ITR=1,NTRANS
         IF(LINEXP(ITR)) GOTO 100
         DO 30 IJ=IFR0(ITR),IFR1(ITR)
            IJLIN(IJ)=ITR
   30    CONTINUE
  100 CONTINUE
      RETURN
      END

C
C
C     ****************************************************************
C
C
      SUBROUTINE RTEDF1(IJ)
C     =====================
C
C     Solution of the radiative transfer equation - for one frequency 
C     for the known source function.
C     Determination of the radiation field and variable Eddington
C     factors.
C
C     The numerical method used:
c     Discontinuous Finite Element method
c     Castor, Dykema, Klein, 1992, ApJ  387, 561
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (NMU=3)
      DIMENSION DT(MDEPTH),ST0(MDEPTH),RDK(MDEPTH),
     *          dtau(mdepth),rip(mdepth),rim(mdepth),
     *          riin(mdepth),riup(mdepth),u(mdepth),
     *          aip(mdepth),aim(mdepth),al(mdepth),
     *          aiin(mdepth),aiup(mdepth),
     *          AMU(NMU),WTMU(NMU)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
C
C     angle points (AMU) and angular integration weights (WTMU)
C
      DATA AMU/.887298334620742D0,.5D0,.112701665379258D0/,
     * WTMU/.277777777777778D0,.444444444444444D0,.277777777777778D0/
C
      FR=FREQ(IJ)
C
C     total source function
C  
      DO 20 ID=1,ND
         ST0(ID)=(EMIS1(ID)+SCAT1(ID)*EMEL1(ID)*RAD(IJ,ID))/
     *           ABSO1(ID)
         RAD1(ID)=0.
         RDK(ID)=0.
         ALI1(ID)=0.
   20 CONTINUE
      AH=0.
C
C     optical depth scale
C
      DO 30 ID=1,ND-1
         DT(ID)=DELDM(ID)*(ABSO1(ID+1)*DENS1(ID+1)+
     *                     ABSO1(ID)*DENS1(ID))
   30 CONTINUE
C
      U0=0.
      QQ0=0.
      US0=0.
      TAUMIN=ABSO1(1)*DEDM1
C
C     Allowance for wind blanketing
C
      ALB1=0.
      IF(IWINBL.GT.0) ALB1=TWO*ALBE(IJ)/(UN+ALBE(IJ))
C
C     quantities for the lower boundary condition
C
      FR15=FR*1.D-15
      BNU=BN*FR15*FR15*FR15
      PLAND=BNU/(EXP(HK*FR/TEMP(ND))-UN)
      DPLAN=BNU/(EXP(HK*FR/TEMP(ND-1))-UN)
      DPLAN=(PLAND-DPLAN)/DT(ND-1)
c
c     loop over angle poits
c
      DO 100 I=1,NMU
         AMU2=AMU(I)*AMU(I)*WTMU(I)
         do id=1,nd-1
            dtau(id)=dt(id)/amu(i)
         enddo
c               
c        incoming intensity 
c   
         ID=1
         P0=0.
         EX=UN
C
C        allowance for non-zero optical depth at the first depth point
C
         rim(id)=EXTRAD(IJ)
         IF(IWINBL.EQ.0) THEN
            TAMM=TAUMIN/AMU(I)
            EX=EXP(-TAMM)
            P0=UN-EX
            QQ0=QQ0+P0*AMU(I)*WTMU(I)
            U0=U0+EX*WTMU(I)
            US0=US0+P0/TAMM*WTMU(I)
            rip(id)=st0(id)*p0
         END IF
c               
c          incoming intensity 
c   
            id=1
            dt0=dtau(id)
            dtaup1=dt0+un
            dtau2=dt0*dt0
            bb=two*dtaup1
            cc=dt0*dtaup1
            aa=dtau2+bb
            rim(id)=(aa*rip(id)-cc*st0(id)+dt0*st0(id+1))/bb
            aip(id)=0.
            aim(id)=-cc/bb
            do id=1,nd-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               rim(id+1)=(two*rim(id)+dt0*st0(id)+cc*st0(id+1))*aa
               rip(id)=(bb*rim(id)+cc*st0(id)-dt0*st0(id+1))*aa
               aim(id+1)=cc*aa
               aip(id)=(cc+bb*aim(id))*aa
            enddo
            do id=2,nd-1
               dtt=un/(dtau(id-1)+dtau(id))
               riin(id)=(rim(id)*dtau(id)+rip(id)*dtau(id-1))*dtt
               aiin(id)=(aim(id)*dtau(id)+aip(id)*dtau(id-1))*dtt
            enddo
            riin(1)=rip(1)
            riin(nd)=rim(nd)
            aiin(1)=aip(1)
            aiin(nd)=aim(nd)
C
c           outgoing intensity
c 
            rip(nd)=PLAND+AMU(I)*DPLAN
            id=nd-1
            dt0=dtau(id)
            dtaup1=dt0+un
            dtau2=dt0*dt0
            bb=two*dtaup1
            cc=dt0*dtaup1
            aa=dtau2+bb
            rim(id+1)=(aa*rip(id+1)-cc*st0(id+1)+dt0*st0(id))/bb
            aip(id)=0.
            aim(id+1)=-cc/bb
            do id=nd-1,1,-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               rim(id)=(two*rim(id+1)+dt0*st0(id+1)+cc*st0(id))*aa
               rip(id+1)=(bb*rim(id+1)+cc*st0(id+1)-dt0*st0(id))*aa
               aim(id)=cc*aa
               aip(id+1)=(cc+bb*aim(id+1))*aa
            enddo
            do id=2,nd-1
               dtt=un/(dtau(id-1)+dtau(id))
               riup(id)=(rim(id)*dtau(id-1)+rip(id)*dtau(id))*dtt
               aiup(id)=(aim(id)*dtau(id-1)+aip(id)*dtau(id))*dtt
            enddo
            riup(1)=rim(1)
            riup(nd)=rip(nd)
            aiup(1)=aim(1)
            aiup(nd)=aip(nd)
c
c           final symmetrized (Feautrier) intensity -- (riin+riup)/2
c
            do id=1,nd
               u(id)=(riin(id)+riup(id))*half
               al(id)=(aiin(id)+aiup(id))*half
            enddo
c
         DO 70 ID=1,ND
            RAD1(ID)=RAD1(ID)+WTMU(I)*U(ID)
            RDK(ID)=RDK(ID)+AMU2*U(ID)
            ALI1(ID)=ALI1(ID)+WTMU(I)*AL(ID)
   70    CONTINUE
         AH=AH+AMU(I)*WTMU(I)*U(1)
c
c     end of the loop over angle points
c
  100 CONTINUE
C
C     solution of the transfer equation
C     Variables:
C     RAD1    - mean intensity
C     FAK1    - Eddington factor f(K) = K/J
C     FH      - the "surface" Eddington factor
C     ALI1    - diagonal element of the lambda operator
C
      DO 110 ID=1,ND
         RAD(IJ,ID)=RAD1(ID)
         FAK1(ID)=RDK(ID)/RAD1(ID)
  110 CONTINUE
      ALI1(ND)=RAD1(ND)/ST0(ND)
      ALI1(ND-1)=RAD1(ND-1)/ST0(ND-1)
      FLUX(IJ)=AH
      FH(IJ)=AH/RAD1(1)-HALF*ALB1
C
C     store quantities for explicit (linearized) frequencies
C
      IF(IJEX(IJ).LE.0) RETURN
      IJE=IJEX(IJ)
      DO ID=1,ND
         RADEX(IJE,ID)=RAD1(ID)
         FAKEX(IJE,ID)=FAK1(ID)
      END DO
      Q0(IJE)=QQ0
      UU0(IJE)=U0
      RETURN
      END

C
C
C     ****************************************************************
C
C
      SUBROUTINE RTEDF2(IJ)
C     =====================
C
C     Solution of the radiative transfer equation - for one frequency 
C     for the known source function.
C     Determination of the radiation field and variable Eddington
C     factors.
C     Analogous to RTEDF1, but slightly different formulation 
C     of the boundary conditions
C
C     The numerical method used:
c     Discontinuous Finite Element method
c     Castor, Dykema, Klein, 1992, ApJ 387, 561.
C
C     different formulation of the boundary conditions
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (NMU=3)
      DIMENSION DT(MDEPTH),ST0(MDEPTH),RDK(MDEPTH),
     *          dtau(mdepth),rip(mdepth),rim(mdepth),
     *          riin(mdepth),riup(mdepth),u(mdepth),
     *          aip(mdepth),aim(mdepth),al(mdepth),
     *          aiin(mdepth),aiup(mdepth),
     *          AMU(NMU),WTMU(NMU)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
C
C     angle points (AMU) and angular integration weights (WTMU)
C
      DATA AMU/.887298334620742D0,.5D0,.112701665379258D0/,
     * WTMU/.277777777777778D0,.444444444444444D0,.277777777777778D0/
C
      FR=FREQ(IJ)
C
C     total source function
C  
      DO 20 ID=1,ND
         ST0(ID)=(EMIS1(ID)+SCAT1(ID)*EMEL1(ID)*RAD(IJ,ID))/
     *           ABSO1(ID)
         RAD1(ID)=0.
         RDK(ID)=0.
         ALI1(ID)=0.
   20 CONTINUE
      AH=0.
C
C     optical depth scale
C
      DO 30 ID=1,ND-1
         DT(ID)=DELDM(ID)*(ABSO1(ID+1)*DENS1(ID+1)+
     *                     ABSO1(ID)*DENS1(ID))
   30 CONTINUE
C
      U0=0.
      QQ0=0.
      US0=0.
      TAUMIN=ABSO1(1)*DEDM1
C
C     Allowance for wind blanketing
C
      ALB1=0.
      IF(IWINBL.GT.0) ALB1=TWO*ALBE(IJ)/(UN+ALBE(IJ))
C
C     quantities for the lower boundary condition
C
      FR15=FR*1.D-15
      BNU=BN*FR15*FR15*FR15
      PLAND=BNU/(EXP(HK*FR/TEMP(ND))-UN)
      DPLAN=BNU/(EXP(HK*FR/TEMP(ND-1))-UN)
      DPLAN=(PLAND-DPLAN)/DT(ND-1)
c
c     loop over angle poits
c
      DO 100 I=1,NMU
         AMU2=AMU(I)*AMU(I)*WTMU(I)
         do id=1,nd-1
            dtau(id)=dt(id)/amu(i)
         enddo
c               
c        incoming intensity 
c   
         ID=1
         P0=0.
         EX=UN
C
C        allowance for non-zero optical depth at the first depth point
C
         rim(id)=EXTRAD(IJ)
         IF(IWINBL.EQ.0) THEN
            TAMM=TAUMIN/AMU(I)
            EX=EXP(-TAMM)
            P0=UN-EX
            QQ0=QQ0+P0*AMU(I)*WTMU(I)
            U0=U0+EX*WTMU(I)
            US0=US0+P0/TAMM*WTMU(I)
            rim(id)=st0(id)*p0
         END IF
c               
c          incoming intensity 
c   
            aim(id)=0.
            do id=1,nd-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               rim(id+1)=(two*rim(id)+dt0*st0(id)+cc*st0(id+1))*aa
               rip(id)=(bb*rim(id)+cc*st0(id)-dt0*st0(id+1))*aa
               aim(id+1)=cc*aa
               aip(id)=(cc+bb*aim(id))*aa
            enddo
            do id=2,nd-1
               dtt=un/(dtau(id-1)+dtau(id))
               riin(id)=(rim(id)*dtau(id)+rip(id)*dtau(id-1))*dtt
               aiin(id)=(aim(id)*dtau(id)+aip(id)*dtau(id-1))*dtt
            enddo
            riin(1)=rim(1)
            riin(nd)=rim(nd)
            aiin(1)=aim(1)
            aiin(nd)=aim(nd)
C
c           outgoing intensity
c 
            rim(nd)=PLAND+AMU(I)*DPLAN
            do id=nd-1,1,-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               rim(id)=(two*rim(id+1)+dt0*st0(id+1)+cc*st0(id))*aa
               rip(id+1)=(bb*rim(id+1)+cc*st0(id+1)-dt0*st0(id))*aa
               aim(id)=cc*aa
               aip(id+1)=(cc+bb*aim(id+1))*aa
            enddo
            do id=2,nd-1
               dtt=un/(dtau(id-1)+dtau(id))
               riup(id)=(rim(id)*dtau(id-1)+rip(id)*dtau(id))*dtt
               aiup(id)=(aim(id)*dtau(id-1)+aip(id)*dtau(id))*dtt
            enddo
            riup(1)=rim(1)
            riup(nd)=rim(nd)
            aiup(1)=aim(1)
            aiup(nd)=aim(nd)
c
c           final symmetrized (Feautrier) intensity -- (riin+riup)/2
c
            do id=1,nd
               u(id)=(riin(id)+riup(id))*half
               al(id)=(aiin(id)+aiup(id))*half
            enddo
c
         DO 70 ID=1,ND
            RAD1(ID)=RAD1(ID)+WTMU(I)*U(ID)
            RDK(ID)=RDK(ID)+AMU2*U(ID)
            ALI1(ID)=ALI1(ID)+WTMU(I)*AL(ID)
   70    CONTINUE
         AH=AH+AMU(I)*WTMU(I)*U(1)
c
c     end of the loop over angle points
c
  100 CONTINUE
C
C     solution of the transfer equation
C     Variables:
C     RAD1    - mean intensity
C     FAK1    - Eddington factor f(K) = K/J
C     FH      - the "surface" Eddington factor
C     ALI1    - diagonal element of the lambda operator
C
      DO 110 ID=1,ND
         RAD(IJ,ID)=RAD1(ID)
         FAK1(ID)=RDK(ID)/RAD1(ID)
  110 CONTINUE
      ALI1(ND)=RAD1(ND)/ST0(ND)
      ALI1(ND-1)=RAD1(ND-1)/ST0(ND-1)
      FLUX(IJ)=AH
      FH(IJ)=AH/RAD1(1)-HALF*ALB1
C
C     store quantities for explicit (linearized) frequencies
C
      IF(IJEX(IJ).LE.0) RETURN
      IJE=IJEX(IJ)
      DO ID=1,ND
         RADEX(IJE,ID)=RAD1(ID)
         FAKEX(IJE,ID)=FAK1(ID)
      END DO
      Q0(IJE)=QQ0
      UU0(IJE)=U0
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE RTEDF3(IJ)
C     =====================
C
C     Solution of the radiative transfer equation - for one frequency 
C     for the known source function.
C     Determination of the radiation field and variable Eddington
C     factors.
C     Analogous to RTEDF1, but slightly different formulation 
C     of the boundary conditions
C
C     The numerical method used:
c     Discontinuous Finite Element method
c     Castor, Dykema, Klein, 1992, ApJ 387, 561.
C
C     different formulation of the boundary conditions
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (NMU=3)
      DIMENSION DT(MDEPTH),RDK(MDEPTH),
     *          ST0(MDEPTH),SA0(MDEPTH),SS0(MDEPTH),
     *          dtau(mdepth),rip(mdepth),rim(mdepth),
     *          riin(mdepth),riup(mdepth),u(mdepth),
     *          aip(mdepth),aim(mdepth),al(mdepth),
     *          aiin(mdepth),aiup(mdepth),
     *          AMU(NMU),WTMU(NMU)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
      COMMON/DFEALI/DJMAX,NTRALI
C
C     angle points (AMU) and angular integration weights (WTMU)
C
      DATA AMU/.887298334620742D0,.5D0,.112701665379258D0/,
     * WTMU/.277777777777778D0,.444444444444444D0,.277777777777778D0/
C
      FR=FREQ(IJ)
C
C     optical depth scale
C
      DO ID=1,ND-1
         DT(ID)=DELDM(ID)*(ABSO1(ID+1)*DENS1(ID+1)+
     *                     ABSO1(ID)*DENS1(ID))
         SA0(ID)=EMIS1(ID)/ABSO1(ID)
         SS0(ID)=-SCAT1(ID)/ABSO1(ID)
      END DO
      SA0(ND)=EMIS1(ND)/ABSO1(ND)
      SS0(ND)=-SCAT1(ND)/ABSO1(ND)
C
      TAUMIN=ABSO1(1)*DEDM1
C
C     Allowance for wind blanketing
C
      ALB1=0.
      IF(IWINBL.GT.0) ALB1=TWO*ALBE(IJ)/(UN+ALBE(IJ))
C
C     quantities for the lower boundary condition
C
      FR15=FR*1.D-15
      BNU=BN*FR15*FR15*FR15
      PLAND=BNU/(EXP(HK*FR/TEMP(ND))-UN)
      DPLAN=BNU/(EXP(HK*FR/TEMP(ND-1))-UN)
      DPLAN=(PLAND-DPLAN)/DT(ND-1)
C
C     global ALI loop for treating electron scattering
C
      itrali=0
   10 itrali=itrali+1
C
C     total source function
C  
      DO 20 ID=1,ND
         ST0(ID)=SA0(ID)-SS0(ID)*RAD(IJ,ID)
         RAD1(ID)=0.
         RDK(ID)=0.
         ALI1(ID)=0.
   20 CONTINUE
      AH=0.
      U0=0.
      QQ0=0.
      US0=0.
c
c     loop over angle poits
c
      DO 100 I=1,NMU
         AMU2=AMU(I)*AMU(I)*WTMU(I)
         do id=1,nd-1
            dtau(id)=dt(id)/amu(i)
         enddo
c               
c        incoming intensity 
c   
         ID=1
         P0=0.
         EX=UN
C
C        allowance for non-zero optical depth at the first depth point
C
         rim(id)=EXTRAD(IJ)
         IF(IWINBL.EQ.0) THEN
            TAMM=TAUMIN/AMU(I)
            EX=EXP(-TAMM)
            P0=UN-EX
            QQ0=QQ0+P0*AMU(I)*WTMU(I)
            U0=U0+EX*WTMU(I)
            US0=US0+P0/TAMM*WTMU(I)
            rim(id)=st0(id)*p0
         END IF
c               
c          incoming intensity 
c   
            aim(id)=0.
            do id=1,nd-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               rim(id+1)=(two*rim(id)+dt0*st0(id)+cc*st0(id+1))*aa
               rip(id)=(bb*rim(id)+cc*st0(id)-dt0*st0(id+1))*aa
               aim(id+1)=cc*aa
               aip(id)=(cc+bb*aim(id))*aa
            enddo
            do id=2,nd-1
               dtt=un/(dtau(id-1)+dtau(id))
               riin(id)=(rim(id)*dtau(id)+rip(id)*dtau(id-1))*dtt
               aiin(id)=(aim(id)*dtau(id)+aip(id)*dtau(id-1))*dtt
            enddo
            riin(1)=rim(1)
            riin(nd)=rim(nd)
            aiin(1)=aim(1)
            aiin(nd)=aim(nd)
C
c           outgoing intensity
c 
            rim(nd)=PLAND+AMU(I)*DPLAN
            do id=nd-1,1,-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               rim(id)=(two*rim(id+1)+dt0*st0(id+1)+cc*st0(id))*aa
               rip(id+1)=(bb*rim(id+1)+cc*st0(id+1)-dt0*st0(id))*aa
               aim(id)=cc*aa
               aip(id+1)=(cc+bb*aim(id+1))*aa
            enddo
            do id=2,nd-1
               dtt=un/(dtau(id-1)+dtau(id))
               riup(id)=(rim(id)*dtau(id-1)+rip(id)*dtau(id))*dtt
               aiup(id)=(aim(id)*dtau(id-1)+aip(id)*dtau(id))*dtt
            enddo
            riup(1)=rim(1)
            riup(nd)=rim(nd)
            aiup(1)=aim(1)
            aiup(nd)=aim(nd)
c
c           final symmetrized (Feautrier) intensity -- (riin+riup)/2
c
            do id=1,nd
               u(id)=(riin(id)+riup(id))*half
               al(id)=(aiin(id)+aiup(id))*half
            enddo
c
         DO 70 ID=1,ND
            RAD1(ID)=RAD1(ID)+WTMU(I)*U(ID)
            RDK(ID)=RDK(ID)+AMU2*U(ID)
            ALI1(ID)=ALI1(ID)+WTMU(I)*AL(ID)
   70    CONTINUE
         AH=AH+AMU(I)*WTMU(I)*U(1)
c
c     end of the loop over angle points
c
  100 CONTINUE
C
C     solution of the transfer equation
C     Variables:
C     RAD1    - mean intensity
C     FAK1    - Eddington factor f(K) = K/J
C     FH      - the "surface" Eddington factor
C     ALI1    - diagonal element of the lambda operator
C
      IF(IBC.EQ.0) THEN
         ALI1(ND)=RAD1(ND)/ST0(ND)
         ALI1(ND-1)=RAD1(ND-1)/ST0(ND-1)
      END IF
C
      DJTOT=0.
      DO 110 ID=1,ND
         DELTAJ=(RAD1(ID)-RAD(IJ,ID))/(UN+SS0(ID)*ALI1(ID))
         RAD(IJ,ID)=RAD(IJ,ID)+DELTAJ
         DJTOT=MAX(DJTOT,ABS(DELTAJ/RAD(IJ,ID)))
  110 CONTINUE
c      write(6,600) IJ,ITRALI,DJTOT
c  600 format(' ij,iter,djmax',2i4,1pe10.3)
      IF(DJTOT.GT.DJMAX.AND.ITRALI.LE.NTRALI) GO TO 10
C
C     end of ALI loop for electron scattering
C
      DO 120 ID=1,ND
         RAD1(ID)=RAD(IJ,ID)
         FAK1(ID)=RDK(ID)/RAD(IJ,ID)
  120 CONTINUE
      FLUX(IJ)=AH
      FH(IJ)=AH/RAD1(1)-HALF*ALB1
C
C     store quantities for explicit (linearized) frequencies
C
      IF(IJEX(IJ).LE.0) RETURN
      IJE=IJEX(IJ)
      DO ID=1,ND
         RADEX(IJE,ID)=RAD1(ID)
         FAKEX(IJE,ID)=FAK1(ID)
      END DO
      Q0(IJE)=QQ0
      UU0(IJE)=U0
      RETURN
      END

C
C
C     ****************************************************************
C
C
      SUBROUTINE RTEDF4(IJ)
C     =====================
C
C     Solution of the radiative transfer equation - for one frequency 
C     for the known source function.
C     Determination of the radiation field and variable Eddington
C     factors.
C     Analogous to RTEDF1, but slightly different formulation 
C     of the boundary conditions
C
C     The numerical method used:
c     Discontinuous Finite Element method
c     Castor, Dykema, Klein, 1992, ApJ 387, 561.
C
C     different formulation of the boundary conditions
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (NMU=3)
      PARAMETER (SIXTH=UN/6.D0,
     *           THIRD=UN/3.D0,
     *           TWOTHR=TWO/3.D0)
      DIMENSION DT(MDEPTH),RDK(MDEPTH),FKK(MDEPTH),
     *          ST0(MDEPTH),SA0(MDEPTH),SS0(MDEPTH),
     *          dtau(mdepth),rip(mdepth),rim(mdepth),
     *          riin(mdepth),riup(mdepth),u(mdepth),
     *          aip(mdepth),aim(mdepth),al0(mdepth),
     *          aiin(mdepth),aiup(mdepth),
     *          AMU(NMU),WTMU(NMU),
     *          FFPD(NMU,NMU),ali0(mdepth),
     *          AAA(MDEPTH),BBB(MDEPTH),CCC(MDEPTH),EEE(MDEPTH),
     *          ZZZ(MDEPTH),ALRH(MDEPTH),ALRM(MDEPTH),ALRP(MDEPTH),
     *          DDD(MDEPTH),AANU(MDEPTH)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
      COMMON/DFEALI/DJMAX,NTRALI
C
C     angle points (AMU) and angular integration weights (WTMU)
C
      DATA AMU/.887298334620742D0,.5D0,.112701665379258D0/,
     * WTMU/.277777777777778D0,.444444444444444D0,.277777777777778D0/
C
      FR=FREQ(IJ)
C
C     optical depth scale
C
      DO ID=1,ND-1
         DT(ID)=DELDM(ID)*(ABSO1(ID+1)*DENS1(ID+1)+
     *                     ABSO1(ID)*DENS1(ID))
         SA0(ID)=EMIS1(ID)/ABSO1(ID)
         SS0(ID)=-SCAT1(ID)/ABSO1(ID)
      END DO
      SA0(ND)=EMIS1(ND)/ABSO1(ND)
      SS0(ND)=-SCAT1(ND)/ABSO1(ND)
C
      TAUMIN=ABSO1(1)*DEDM1
C
C     Allowance for wind blanketing
C
      ALB1=0.
      IF(IWINBL.GT.0) ALB1=TWO*ALBE(IJ)/(UN+ALBE(IJ))
C
C     quantities for the lower boundary condition
C
      FR15=FR*1.D-15
      BNU=BN*FR15*FR15*FR15
      PLAND=BNU/(EXP(HK*FR/TEMP(ND))-UN)
      DPLAN=BNU/(EXP(HK*FR/TEMP(ND-1))-UN)
      DPLAN=(PLAND-DPLAN)/DT(ND-1)
C
C     global ALI loop for treating electron scattering
C
      itrali=0
   10 itrali=itrali+1
C
C     total source function
C  
      DO 20 ID=1,ND
         ST0(ID)=SA0(ID)-SS0(ID)*RAD(IJ,ID)
         RAD1(ID)=0.
         RDK(ID)=0.
         ALI1(ID)=0.
   20 CONTINUE
      AH=0.
      U0=0.
      QQ0=0.
      US0=0.
c
c     loop over angle poits
c
      DO 100 I=1,NMU
         AMU2=AMU(I)*AMU(I)*WTMU(I)
         do id=1,nd-1
            dtau(id)=dt(id)/amu(i)
         enddo
c               
c        incoming intensity 
c   
         ID=1
         P0=0.
         EX=UN
C
C        allowance for non-zero optical depth at the first depth point
C
         rim(id)=EXTRAD(IJ)
         IF(IWINBL.EQ.0) THEN
            TAMM=TAUMIN/AMU(I)
            EX=EXP(-TAMM)
            P0=UN-EX
            QQ0=QQ0+P0*AMU(I)*WTMU(I)
            U0=U0+EX*WTMU(I)
            US0=US0+P0/TAMM*WTMU(I)
            rim(id)=st0(id)*p0
         END IF
c               
c          incoming intensity 
c   
            aim(id)=0.
            do id=1,nd-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               rim(id+1)=(two*rim(id)+dt0*st0(id)+cc*st0(id+1))*aa
               rip(id)=(bb*rim(id)+cc*st0(id)-dt0*st0(id+1))*aa
               aim(id+1)=cc*aa
               aip(id)=(cc+bb*aim(id))*aa
            enddo
            do id=2,nd-1
               dtt=un/(dtau(id-1)+dtau(id))
               riin(id)=(rim(id)*dtau(id)+rip(id)*dtau(id-1))*dtt
               aiin(id)=(aim(id)*dtau(id)+aip(id)*dtau(id-1))*dtt
            enddo
            riin(1)=rim(1)
            riin(nd)=rim(nd)
            aiin(1)=aim(1)
            aiin(nd)=aim(nd)
C
c           outgoing intensity
c 
            rim(nd)=PLAND+AMU(I)*DPLAN
            do id=nd-1,1,-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               rim(id)=(two*rim(id+1)+dt0*st0(id+1)+cc*st0(id))*aa
               rip(id+1)=(bb*rim(id+1)+cc*st0(id+1)-dt0*st0(id))*aa
               aim(id)=cc*aa
               aip(id+1)=(cc+bb*aim(id+1))*aa
            enddo
            do id=2,nd-1
               dtt=un/(dtau(id-1)+dtau(id))
               riup(id)=(rim(id)*dtau(id-1)+rip(id)*dtau(id))*dtt
               aiup(id)=(aim(id)*dtau(id-1)+aip(id)*dtau(id))*dtt
            enddo
            riup(1)=rim(1)
            riup(nd)=rim(nd)
            aiup(1)=aim(1)
            aiup(nd)=aim(nd)
c
c           final symmetrized (Feautrier) intensity -- (riin+riup)/2
c
            do id=1,nd
               u(id)=(riin(id)+riup(id))*half
               al0(id)=(aiin(id)+aiup(id))*half
            enddo
c
         DO 70 ID=1,ND
            RAD1(ID)=RAD1(ID)+WTMU(I)*U(ID)
            RDK(ID)=RDK(ID)+AMU2*U(ID)
            ALI1(ID)=ALI1(ID)+WTMU(I)*AL0(ID)
   70    CONTINUE
         AH=AH+AMU(I)*WTMU(I)*U(1)
c
c     end of the loop over angle points
c
  100 CONTINUE
C
C     solution of the transfer equation
C     Variables:
C     RAD1    - mean intensity
C     FAK1    - Eddington factor f(K) = K/J
C     FH      - the "surface" Eddington factor
C     ALI1    - diagonal element of the lambda operator
C
      IF(IBC.EQ.0) THEN
         ALI1(ND)=RAD1(ND)/ST0(ND)
         ALI1(ND-1)=RAD1(ND-1)/ST0(ND-1)
      END IF
C
      DJTOT=0.
      DO 110 ID=1,ND
         DELTAJ=(RAD1(ID)-RAD(IJ,ID))/(UN+SS0(ID)*ALI1(ID))
         RAD(IJ,ID)=RAD(IJ,ID)+DELTAJ
         DJTOT=MAX(DJTOT,ABS(DELTAJ/RAD(IJ,ID)))
  110 CONTINUE
      IF(DJTOT.GT.DJMAX.AND.ITRALI.LE.NTRALI) GO TO 10
C
C     end of ALI loop for electron scattering
C
      DO 120 ID=1,ND
         RAD1(ID)=RAD(IJ,ID)
         FAK1(ID)=RDK(ID)/RAD(IJ,ID)
         FKK(ID)=FAK1(ID)
  120 CONTINUE
      FLUX(IJ)=AH
      FH(IJ)=AH/RAD1(1)-HALF*ALB1
      FH0=FH(IJ)
C
C ********************
C
C     Again solution of the transfer equation, now with Eddington
C     FKK and FH determined above, to insure strict consistency of the
C     radiation field and Eddington factors
C
C     Upper boundary condition
C
      ID=1
      DTP1=DT(ID)
      IF(MOD(ISPLIN,3).EQ.0) THEN
         B=DTP1*HALF
         C=0.
       ELSE
         B=DTP1*THIRD
         C=B*HALF
      END IF
      BQ=UN/(B+QQ0)
      CQ=C*BQ
      BBB(ID)=(FKK(ID)/DTP1+FH0+B)*BQ+SS0(ID)
      CCC(ID)=(FKK(ID+1)/DTP1)*BQ-CQ*(UN+SS0(ID+1))
      ZZZ(ID)=UN/BBB(ID)
      VLL=SA0(ID)+CQ*SA0(ID+1)
      IF(IWINBL.LT.0) VLL=VLL+EXTRAD(IJ)/DTP1
      AANU(ID)=VLL*ZZZ(ID)
      DDD(ID)=CCC(ID)*ZZZ(ID)
      IF(ISPLIN.GT.2) FFF=BBB(ID)/CCC(ID)-UN
C
C     Normal depth point
C
      DO 280 ID=2,ND-1
         DTM1=DTP1
         DTP1=DT(ID)
         DT0=TWO/(DTP1+DTM1)
         AL=UN/DTM1*DT0
         GA=UN/DTP1*DT0
         IF(MOD(ISPLIN,3).EQ.0) THEN
            A=0.
            C=0.
          ELSE IF(ISPLIN.EQ.1) THEN
            A=DTM1*DT0*SIXTH
            C=DTP1*DT0*SIXTH
          ELSE
            A=(UN-HALF*DTP1*DTP1*AL)*SIXTH
            C=(UN-HALF*DTM1*DTM1*GA)*SIXTH
         END IF
         AAA(ID)=AL*FKK(ID-1)-A*(UN+SS0(ID-1))
         CCC(ID)=GA*FKK(ID+1)-C*(UN+SS0(ID+1))
         BBB(ID)=(AL+GA)*FKK(ID)+(UN-A-C)*(UN+SS0(ID))
         VLL=A*SA0(ID-1)+C*SA0(ID+1)+(UN-A-C)*SA0(ID)
         AANU(ID)=VLL+AAA(ID)*AANU(ID-1)
         IF(ISPLIN.LE.2) THEN
            ZZZ(ID)=UN/(BBB(ID)-AAA(ID)*DDD(ID-1))
            DDD(ID)=CCC(ID)*ZZZ(ID)
            AANU(ID)=AANU(ID)*ZZZ(ID)
         ELSE
            SUM=-AAA(ID)+BBB(ID)-CCC(ID)
            FFF=(SUM+AAA(ID)*FFF*DDD(ID-1))/CCC(ID)
            DDD(ID)=UN/(UN+FFF)
            AANU(ID)=AANU(ID)*DDD(ID)/CCC(ID)
         ENDIF
  280 CONTINUE
C
C     Lower boundary condition
C
      ID=ND
      IF(IBC.EQ.0) THEN
         BBB(ID)=FKK(ID)/DTP1+HALF
         AAA(ID)=FKK(ID-1)/DTP1
         VLL=HALF*PLAND+THIRD*DPLAN
       ELSE
         B=UN/DTP1
         A=TWO*B*B
c        BBB(ID)=UN+SS0(ID)+B*TWO*FHD(IJ)+A*FKK(ID)
         BBB(ID)=UN+SS0(ID)+B+A*FKK(ID)
         AAA(ID)=A*FKK(ID-1)
         VLL=SA0(ID)+B*(PLAND+TWOTHR*DPLAN)
      END IF
      EEE(ND)=AAA(ID)/BBB(ID)
      ZZZ(ID)=UN/(BBB(ID)-AAA(ID)*DDD(ID-1))
      RAD1(ID)=(VLL+AAA(ID)*AANU(ID-1))*ZZZ(ID)
      FAK1(ID)=FKK(ND)
      ALRH(ID)=ZZZ(ID)
C
C     Backsolution
C
      DO 290 ID=ND-1,1,-1
         EEE(ID)=AAA(ID)/(BBB(ID)-CCC(ID)*EEE(ID+1))
         RAD1(ID)=AANU(ID)+DDD(ID)*RAD1(ID+1)
         FAK1(ID)=FKK(ID)
         ALRH(ID)=ZZZ(ID)/(UN-DDD(ID)*EEE(ID+1))
         ALRM(ID)=0
         ALRP(ID)=0
  290 CONTINUE
      FLUX(IJ)=FH(IJ)*RAD1(1)
c
C        evaluate approximate Lambda operator
C
C        a) Rybicki-Hummer Lambda^star operator (diagonal)
C           (for JALI = 1)
C
         DO 301 ID=1,ND
            ALIM1(ID)=0.
            ALIP1(ID)=0.
  301    CONTINUE
         IF(JALI.EQ.1) THEN
         DO 310 ID=1,ND 
            ALI1(ID)=ALRH(ID)
  310    CONTINUE
c
         IF(IBC.EQ.0) THEN
            ali1(nd-1)=rad1(nd-1)/sa0(nd-1)
            ali1(nd)=rad1(nd)/sa0(nd)
         END IF
C
C        for IFALI>5:
C        tridiagonal Rybicki-Hummer operator (off-diagonal terms)
C
         IF(IFALI.GE.6) THEN
         ALIP1(1)=ALRH(2)*DDD(1)
         DO 340 ID=2,ND-1 
            ALIM1(ID)=ALRH(ID-1)*EEE(ID)
            ALIP1(ID)=ALRH(ID+1)*DDD(ID)
  340    CONTINUE
         ALIM1(ND)=ALRH(ND-1)*EEE(ND)
         IF(IBC.EQ.0) THEN
            ALIM1(nd)=0.
            ALIM1(nd-1)=0.
            ALIP1(nd)=0.
            ALIP1(nd-1)=0.
         END IF
         END IF
c
C        b) diagonal Olson-Kunasz Lambda^star operator, 
C           (for JALI = 2)
C
         ELSE IF(JALI.EQ.2) THEN
         DO 360 ID=1,ND-1
            ALI0(ID)=0.
            DO 350 I=1,NMU
               DIV=DT(ID)/AMU(I)
               ALI0(ID)=ALI0(ID)+(UN-EXP(-DIV))/DIV*WTMU(I)
  350       CONTINUE
  360    CONTINUE
         DO 370 ID=2,ND-1
            ALI1(ID)=UN-HALF*(ALI0(ID)+ALI0(ID-1))
  370    CONTINUE
         ALI1(1)=UN-HALF*(ALI0(1)+US0)
         ALI1(ND)=UN-ALI0(ND-1)
         ali1(nd-1)=rad1(nd-1)/sa0(nd-1)
         ali1(nd)=rad1(nd)/sa0(nd)
	 END IF
C          
C        correction for scattering
C  
         IF(ILMCOR.NE.0) THEN     
         DO 380 ID=1,ND
             ALI1(ID)=ALI1(ID)*(UN+SS0(ID))
  380    CONTINUE
         END IF
C
C     store quantities for explicit (linearized) frequencies
C
      IF(IJEX(IJ).LE.0) RETURN
      IJE=IJEX(IJ)
      DO ID=1,ND
         RADEX(IJE,ID)=RAD1(ID)
         FAKEX(IJE,ID)=FAK1(ID)
      END DO
      Q0(IJE)=QQ0
      UU0(IJE)=U0
      RETURN
      END


C
C
C     ****************************************************************
C
C
      SUBROUTINE RTEDF5(IJ)
C     =====================
C
C     Solution of the radiative transfer equation - for one frequency 
C     for the known source function.
C     Determination of the radiation field and variable Eddington
C     factors.
C     Analogous to RTEDF1, but slightly different formulation 
C     of the boundary conditions
C
C     The numerical method used:
c     Discontinuous Finite Element method
c     Castor, Dykema, Klein, 1992, ApJ 387, 561.
C
C     different formulation of the boundary conditions
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (NMU=3)
      PARAMETER (SIXTH=UN/6.D0,
     *           THIRD=UN/3.D0,
     *           TWOTHR=TWO/3.D0,
     *           three=3.d0, 
     *           quart=0.25d0)
      DIMENSION DT(MDEPTH),RDK(MDEPTH),FKK(MDEPTH),
     *          ST0(MDEPTH),SA0(MDEPTH),SS0(MDEPTH),
     *          dtau(mdepth),rip(mdepth),rim(mdepth),
     *          riin(mdepth),riup(mdepth),u(mdepth),
     *          aip(mdepth),aim(mdepth),al0(mdepth),
     *          aiin(mdepth),aiup(mdepth),
     *          chip0(mdepth),chim0(mdepth),ddm0(mdepth),
     *          chip(mdepth),chim(mdepth),
     *          etap(mdepth),etam(mdepth),
     *          AMU(NMU),WTMU(NMU),
     *          FFPD(NMU,NMU),ali0(mdepth),
     *          AAA(MDEPTH),BBB(MDEPTH),CCC(MDEPTH),EEE(MDEPTH),
     *          ZZZ(MDEPTH),ALRH(MDEPTH),ALRM(MDEPTH),ALRP(MDEPTH),
     *          DDD(MDEPTH),AANU(MDEPTH)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
      COMMON/DFEALI/DJMAX,NTRALI
C
C     angle points (AMU) and angular integration weights (WTMU)
C
      DATA AMU/.887298334620742D0,.5D0,.112701665379258D0/,
     * WTMU/.277777777777778D0,.444444444444444D0,.277777777777778D0/
C
      FR=FREQ(IJ)
C
C     optical depth scale
C
      DO ID=1,ND-1
         DT(ID)=DELDM(ID)*(ABSO1(ID+1)*DENS1(ID+1)+
     *                     ABSO1(ID)*DENS1(ID))
         SA0(ID)=EMIS1(ID)/ABSO1(ID)
         SS0(ID)=-SCAT1(ID)/ABSO1(ID)
         ddm0(id)=(dm(id+1)-dm(id))
         chip0(id)=(abso1(id)*dens1(id)+three*
     *              abso1(id+1)*dens1(id+1))*quart*ddm0(id)
         chim0(id)=(abso1(id)*dens1(id)*three+
     *              abso1(id+1)*dens1(id+1))*quart*ddm0(id)
      END DO
      SA0(ND)=EMIS1(ND)/ABSO1(ND)
      SS0(ND)=-SCAT1(ND)/ABSO1(ND)
C
      TAUMIN=ABSO1(1)*DEDM1
C
C     Allowance for wind blanketing
C
      ALB1=0.
      IF(IWINBL.GT.0) ALB1=TWO*ALBE(IJ)/(UN+ALBE(IJ))
C
C     quantities for the lower boundary condition
C
      FR15=FR*1.D-15
      BNU=BN*FR15*FR15*FR15
      PLAND=BNU/(EXP(HK*FR/TEMP(ND))-UN)
      DPLAN=BNU/(EXP(HK*FR/TEMP(ND-1))-UN)
      DPLAN=(PLAND-DPLAN)/DT(ND-1)
C
C     global ALI loop for treating electron scattering
C
      itrali=0
   10 itrali=itrali+1
C
C     total source function
C  
      DO 20 ID=1,ND
         ST0(ID)=SA0(ID)-SS0(ID)*RAD(IJ,ID)
         RAD1(ID)=0.
         RDK(ID)=0.
         ALI1(ID)=0.
   20 CONTINUE
      AH=0.
      U0=0.
      QQ0=0.
      US0=0.
c
c     loop over angle poits
c
      DO 100 I=1,NMU
         AMU2=AMU(I)*AMU(I)*WTMU(I)
         do id=1,nd-1
            dtau(id)=dt(id)/amu(i)
            chip(id)=un+chip0(id)/amu(i)
            chim(id)=un+chim0(id)/amu(i)
            etap(id)=emis1(id+1)*dens1(id+1)/amu(i)*ddm0(id)
            etam(id)=emis1(id)*dens1(id)/amu(i)*ddm0(id)
         enddo
c               
c        incoming intensity 
c   
         ID=1
         P0=0.
         EX=UN
C
C        allowance for non-zero optical depth at the first depth point
C
         rim(id)=EXTRAD(IJ)
         IF(IWINBL.EQ.0) THEN
            TAMM=TAUMIN/AMU(I)
            EX=EXP(-TAMM)
            P0=UN-EX
            QQ0=QQ0+P0*AMU(I)*WTMU(I)
            U0=U0+EX*WTMU(I)
            US0=US0+P0/TAMM*WTMU(I)
            rim(id)=st0(id)*p0
         END IF
c               
c          incoming intensity 
c   
            aim(id)=0.
            do id=1,nd-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               aam=un/(un+chim(id)*chip(id))
               rim(id+1)=(two*rim(id)+etap(id)*chim(id)+etam(id))*aam
               rip(id)=(two*rim(id)*chim(id)+etam(id)*chip(id)-
     *                  etap(id))*aam
               aim(id+1)=bb*aa
               aip(id)=(cc+bb*aim(id))*aa
            enddo
            do id=2,nd-1
c               riin(id)=(rim(id)*dtau(id)+rip(id)*dtau(id-1))*dtt
               dtt=un/(dtau(id-1)+dtau(id))
               dtm=un/(ddm0(id-1)+ddm0(id))
               riin(id)=(rim(id)*ddm0(id)+rip(id)*ddm0(id-1))*dtm
               aiin(id)=(aim(id)*dtau(id)+aip(id)*dtau(id-1))*dtt
            enddo
            riin(1)=rim(1)
            riin(nd)=rim(nd)
            aiin(1)=aim(1)
            aiin(nd)=aim(nd)
C              
c           outgoing intensity
c               
            rim(nd)=PLAND+AMU(I)*DPLAN
            do id=nd-1,1,-1
               dt0=dtau(id)
               dtaup1=dt0+un
               dtau2=dt0*dt0
               bb=two*dtaup1
               cc=dt0*dtaup1
               aa=un/(dtau2+bb)
               aam=un/(un+chim(id)*chip(id))
               rim(id)=(two*rim(id+1)+etam(id)*chip(id)+etap(id))*aam
               rip(id+1)=(two*rim(id+1)*chip(id)+etap(id)*chim(id)-
     *                    etam(id))*aam
               aim(id)=cc*aa
               aip(id+1)=(cc+bb*aim(id+1))*aa
            enddo
            do id=2,nd-1
c               riup(id)=(rim(id)*dtau(id-1)+rip(id)*dtau(id))*dtt
               dtt=un/(dtau(id-1)+dtau(id))
               dtm=un/(ddm0(id-1)+ddm0(id))
               riup(id)=(rim(id)*ddm0(id-1)+rip(id)*ddm0(id))*dtm
               aiup(id)=(aim(id)*dtau(id-1)+aip(id)*dtau(id))*dtt
            enddo
            riup(1)=rim(1)
            riup(nd)=rim(nd)
            aiup(1)=aim(1)
            aiup(nd)=aim(nd)
c                
c           final symmetrized (Feautrier) intensity -- (riin+riup)/2
c                
            do id=1,nd
               u(id)=(riin(id)+riup(id))*half
               al0(id)=(aiin(id)+aiup(id))*half
            enddo
c             
         DO 70 ID=1,ND
            RAD1(ID)=RAD1(ID)+WTMU(I)*U(ID)
            RDK(ID)=RDK(ID)+AMU2*U(ID)
            ALI1(ID)=ALI1(ID)+WTMU(I)*AL0(ID)
   70    CONTINUE
         AH=AH+AMU(I)*WTMU(I)*U(1)
c
c     end of the loop over angle points
c
  100 CONTINUE
C
C     solution of the transfer equation
C     Variables:
C     RAD1    - mean intensity
C     FAK1    - Eddington factor f(K) = K/J
C     FH      - the "surface" Eddington factor
C     ALI1    - diagonal element of the lambda operator
C
      IF(IBC.EQ.0) THEN
         ALI1(ND)=RAD1(ND)/ST0(ND)
         ALI1(ND-1)=RAD1(ND-1)/ST0(ND-1)
      END IF
C
      DJTOT=0.
      DO 110 ID=1,ND
         DELTAJ=(RAD1(ID)-RAD(IJ,ID))/(UN+SS0(ID)*ALI1(ID))
         RAD(IJ,ID)=RAD(IJ,ID)+DELTAJ
         DJTOT=MAX(DJTOT,ABS(DELTAJ/RAD(IJ,ID)))
  110 CONTINUE
      IF(DJTOT.GT.DJMAX.AND.ITRALI.LE.NTRALI) GO TO 10
C
C     end of ALI loop for electron scattering
C
      DO 120 ID=1,ND
         RAD1(ID)=RAD(IJ,ID)
         FAK1(ID)=RDK(ID)/RAD(IJ,ID)
         FKK(ID)=FAK1(ID)
  120 CONTINUE
      FLUX(IJ)=AH
      FH(IJ)=AH/RAD1(1)-HALF*ALB1
      FH0=FH(IJ)
C
C     store quantities for explicit (linearized) frequencies
C
      IF(IJEX(IJ).LE.0) RETURN
      IJE=IJEX(IJ)
      DO ID=1,ND
         RADEX(IJE,ID)=RAD1(ID)
         FAKEX(IJE,ID)=FAK1(ID)
      END DO
      Q0(IJE)=QQ0
      UU0(IJE)=U0
      RETURN
      END
C
C
C     ****************************************************************
C
C

      SUBROUTINE RTEFR1(IJ)
C     =====================
C
C     Solution of the radiative transfer equation - for one frequency
C     - for the known source function.
C     Determination of the radiation field and variable Eddington
C     factors.
C
C     The numerical method used:
C      for ISPLIN = 0  -  the ordinary Feautrier scheme
C                 = 1  -  the spline collocation method
C                 = 2  -  Hermitian fourth-order method
C                 = 3  -  improved Feautrier scheme
C                         (Rybicki & Hummer 1991, A&A 245, 171.)
C
C     In all cases, the overall matrix system is solved by the standard
C     Gaussian elimination, analogous to that described in SOLVE
C     (auxiliary matrix D is called ALF in SOLVE; auxiliary vector ANU
C     is called BET in SOLVE)
C
C      U0     - derivative of Q0 wrt taumin;
C               for "fixed" frequencies, U0 has the meaning of
C               absorption coefficient * second moment H
C               ( a quantity needed for lower boundary condition of the
C               hydrostatic equilibrium equation, specifically for
C               accounting for an effect of fixed-option transitions)
C
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ALIPAR.FOR'
      INCLUDE 'ITERAT.FOR'
      PARAMETER (NMU=3)
      PARAMETER (PCK=4.19168946D-10)
      PARAMETER (SIXTH=UN/6.D0,
     *           THIRD=UN/3.D0,
     *           TWOTHR=TWO/3.D0)
      DIMENSION AANU(MDEPTH),DDD(MDEPTH),DT(MDEPTH),FKK(MDEPTH),
     *          RDD(MDEPTH),ST0(MDEPTH),SS0(MDEPTH),AB0(MDEPTH),
     *          AA(NMU,NMU),BB(NMU,NMU),CC(NMU,NMU),VL(NMU),
     *          AMU(NMU),WTMU(NMU),FFD(NMU,NMU),FF0D(NMU,NMU),
     *          FFPD(NMU,NMU),ali0(mdepth),
     *          AAA(MDEPTH),BBB(MDEPTH),CCC(MDEPTH),EEE(MDEPTH),
     *          ZZZ(MDEPTH),ALRH(MDEPTH),ALRM(MDEPTH),ALRP(MDEPTH),
     *          D(NMU,NMU,MDEPTH),ANU(NMU,MDEPTH),scor(mdepth)
      COMMON/WINDBL/ALBE(MFREQ),IWINBL
      COMMON/PRESSR/PTOTAL(MDEPTH),PGS(MDEPTH),PRADT(MDEPTH)
      COMMON/HEQAUX/PRD0,IHECOR
C
C     angle points (AMU) and angular integration weights (WTMU)
C
      DATA AMU/.887298334620742D0,.5D0,.112701665379258D0/,
     * WTMU/.277777777777778D0,.444444444444444D0,.277777777777778D0/
C
      IF(IJ.EQ.1) THEN
         PRD0=0.
         DO ID=1,ND
            PRADT(ID)=0.
         END DO
      END IF
C
      WW=W(IJ)-WSUB(IJ)
      ISPL=ISPLIN
      IF(ISPLIN.GE.5) THEN
         ISPLIN=ISPL-5
         IF(IJALI(IJ).GT.0) THEN
            IF(IRTE.EQ.0) THEN
               CALL RTEDF1(IJ)
             ELSE IF(IRTE.EQ.1) THEN
               CALL RTEDF2(IJ)
             ELSE IF(IRTE.EQ.2) THEN
               CALL RTEDF3(IJ)
             ELSE IF(IRTE.EQ.3) THEN
               CALL RTEDF4(IJ)
             ELSE IF(IRTE.EQ.4) THEN
               CALL RTEDF5(IJ)
            END IF
            ISPLIN=ISPL
            DO ID=1,ND
               PRADT(ID)=PRADT(ID)+RAD1(ID)*FAK1(ID)*WW
            END DO
            PRD0=PRD0+ABSO1(1)*RAD1(1)*FH(IJ)*WW
            IF(IJ.EQ.NFREQ) THEN
               DO ID=1,ND
                  PRADT(ID)=PRADT(ID)*PCK
               END DO
               PRD0=PRD0/DENS1(1)*DM(1)*PCK
            END IF
            RETURN
         END IF
      END IF
C
      FR=FREQ(IJ)
C
C     total source function
C  
      AH=0.
      DO 20 ID=1,ND
         AB0(ID)=ABSO1(ID)
         ST0(ID)=EMIS1(ID)/AB0(ID)
         RAD1(ID)=0.
         ALI1(ID)=0.
   20 CONTINUE
C
C     non-coherent electron scattering by lambda iteration
C  
      IF(NELSC.LE.0) THEN
         DO 22 ID=1,ND
            SS0(ID)=-SCAT1(ID)/AB0(ID)
   22    CONTINUE
       ELSE
         DO 24 ID=1,ND
            ST0(ID)=ST0(ID)+SCAT1(ID)*EMEL1(ID)*RAD1(ID)/AB0(ID)
            SS0(ID)=0.
   24    CONTINUE
      END IF
C
C     optical depth scale
C
      DO 30 ID=1,ND-1
         DT(ID)=DELDM(ID)*(ABSO1(ID+1)*DENS1(ID+1)+
     *                     ABSO1(ID)*DENS1(ID))
   30 CONTINUE
C
      U0=0.
      QQ0=0.
      US0=0.
      TAUMIN=ABSO1(1)*DEDM1
C
      ALB1=0.
C
C     Allowance for wind blanketing
C
      IF(IWINBL.GT.0) ALB1=TWO*ALBE(IJ)/(UN+ALBE(IJ))
C
C ************** Forward elimination
C
C     Upper boundary condition
C
      ID=1
      DTP1=DT(1)
      P0=0.
      EX=UN
      IF(MOD(ISPLIN,3).EQ.0) THEN
         B=DTP1*HALF
         C=0.
       ELSE
         B=DTP1*THIRD
         C=B*HALF
      END IF
      QQ0=0.
      US0=0.
      DO 60 I=1,NMU
         IF(IWINBL.EQ.0) THEN
C
C     allowance for non-zero optical depth at the first depth point
C
            TAMM=TAUMIN/AMU(I)
            EX=EXP(-TAMM)
            P0=UN-EX
            QQ0=QQ0+P0*AMU(I)*WTMU(I)
            U0=U0+EX*WTMU(I)
            US0=US0+P0/TAMM*WTMU(I)
         END IF
C
         BI=B/AMU(I)
         CI=C/AMU(I)
         VL(I)=(BI+P0)*ST0(ID)+CI*ST0(ID+1)
         IF(IWINBL.LT.0) VL(I)=VL(I)+EXTRAD(IJ)
         DO 40 J=1,NMU
            BB(I,J)=SS0(ID)*WTMU(J)*(BI+P0)-ALB1*WTMU(J)
   40       CC(I,J)=-CI*SS0(ID+1)*WTMU(J)
         BB(I,I)=BB(I,I)+AMU(I)/DTP1+UN+BI
         CC(I,I)=CC(I,I)+AMU(I)/DTP1-CI
         ANU(I,ID)=0.
   60 CONTINUE
C
      IF(ISPLIN.LE.2) THEN
C
C     In-lining MINV3
C        CALL MINV3(BB)
          BB(2,1)=BB(2,1)/BB(1,1)
          BB(2,2)=BB(2,2)-BB(2,1)*BB(1,2)
          BB(2,3)=BB(2,3)-BB(2,1)*BB(1,3)
          BB(3,1)=BB(3,1)/BB(1,1)
          BB(3,2)=(BB(3,2)-BB(3,1)*BB(1,2))/BB(2,2)
          BB(3,3)=BB(3,3)-BB(3,1)*BB(1,3)-BB(3,2)*BB(2,3)
          BB(3,2)=-BB(3,2)
          BB(3,1)=-BB(3,1)-BB(3,2)*BB(2,1)
          BB(2,1)=-BB(2,1)
          BB(3,3)=UN/BB(3,3)
          BB(2,3)=-BB(2,3)*BB(3,3)/BB(2,2)
          BB(2,2)=UN/BB(2,2)
          BB(1,3)=-(BB(1,2)*BB(2,3)+BB(1,3)*BB(3,3))/BB(1,1)
          BB(1,2)=-BB(1,2)*BB(2,2)/BB(1,1)
          BB(1,1)=UN/BB(1,1)
          BB(1,1)=BB(1,1)+BB(1,2)*BB(2,1)+BB(1,3)*BB(3,1)
          BB(1,2)=BB(1,2)+BB(1,3)*BB(3,2)
          BB(2,1)=BB(2,2)*BB(2,1)+BB(2,3)*BB(3,1)
          BB(2,2)=BB(2,2)+BB(2,3)*BB(3,2)
          BB(3,1)=BB(3,3)*BB(3,1)
          BB(3,2)=BB(3,3)*BB(3,2)
C
         DO 70 I=1,NMU
            DO 70 J=1,NMU
               D(I,J,ID)=0.
               DO 71 K=1,NMU
   71             D(I,J,ID)=D(I,J,ID)+BB(I,K)*CC(K,J)
               ANU(I,1)=ANU(I,1)+BB(I,J)*VL(J)
   70    CONTINUE
      ELSE
         DO 80 I=1,NMU
            DO 81 J=1,NMU
               FF0D(I,J)=BB(I,J)/CC(I,I)
   81       CONTINUE
            FF0D(I,I)=FF0D(I,I)-UN
   80    CONTINUE
C
C     In-lining MINV3
C        CALL MINV3(BB)
          BB(2,1)=BB(2,1)/BB(1,1)
          BB(2,2)=BB(2,2)-BB(2,1)*BB(1,2)
          BB(2,3)=BB(2,3)-BB(2,1)*BB(1,3)
          BB(3,1)=BB(3,1)/BB(1,1)
          BB(3,2)=(BB(3,2)-BB(3,1)*BB(1,2))/BB(2,2)
          BB(3,3)=BB(3,3)-BB(3,1)*BB(1,3)-BB(3,2)*BB(2,3)
          BB(3,2)=-BB(3,2)
          BB(3,1)=-BB(3,1)-BB(3,2)*BB(2,1)
          BB(2,1)=-BB(2,1)
          BB(3,3)=UN/BB(3,3)
          BB(2,3)=-BB(2,3)*BB(3,3)/BB(2,2)
          BB(2,2)=UN/BB(2,2)
          BB(1,3)=-(BB(1,2)*BB(2,3)+BB(1,3)*BB(3,3))/BB(1,1)
          BB(1,2)=-BB(1,2)*BB(2,2)/BB(1,1)
          BB(1,1)=UN/BB(1,1)
          BB(1,1)=BB(1,1)+BB(1,2)*BB(2,1)+BB(1,3)*BB(3,1)
          BB(1,2)=BB(1,2)+BB(1,3)*BB(3,2)
          BB(2,1)=BB(2,2)*BB(2,1)+BB(2,3)*BB(3,1)
          BB(2,2)=BB(2,2)+BB(2,3)*BB(3,2)
          BB(3,1)=BB(3,3)*BB(3,1)
          BB(3,2)=BB(3,3)*BB(3,2)
C
         DO 82 I=1,NMU
            ANU(I,ID)=0.
            DO 82 J=1,NMU
               D(I,J,ID)=BB(I,J)*CC(J,J)
               ANU(I,ID)=ANU(I,ID)+BB(I,J)*VL(J)
   82    CONTINUE
      ENDIF
C
C     Normal depth points  1 < ID < ND
C
      DO 200 ID=2,ND-1
         DTM1=DTP1
         DTP1=DT(ID)
         DT0=TWO/(DTM1+DTP1)
         AL=UN/DTM1*DT0
         GA=UN/DTP1*DT0
         BE=AL+GA
         IF(MOD(ISPLIN,3).EQ.0) THEN
            A=0.
            C=0.
          ELSE IF(ISPLIN.EQ.1) THEN
            A=DTM1*DT0*SIXTH
            C=DTP1*DT0*SIXTH
          ELSE
            A=(UN-HALF*AL*DTP1*DTP1)*SIXTH
            C=(UN-HALF*GA*DTM1*DTM1)*SIXTH
         END IF
         B=UN-A-C
         VL0=A*ST0(ID-1)+B*ST0(ID)+C*ST0(ID+1)
         DO 110 I=1,NMU
            DO 110 J=1,NMU
               AA(I,J)=-A*SS0(ID-1)*WTMU(J)
               CC(I,J)=-C*SS0(ID+1)*WTMU(J)
               BB(I,J)=B*SS0(ID)*WTMU(J)
  110    CONTINUE
         DO 120 I=1,NMU
            VL(I)=VL0
            DIV=AMU(I)*AMU(I)
            AA(I,I)=AA(I,I)+DIV*AL-A
            CC(I,I)=CC(I,I)+DIV*GA-C
            BB(I,I)=BB(I,I)+DIV*BE+B
  120    CONTINUE
         DO 130 I=1,NMU
            DO 130 J=1,NMU
               VL(I)=VL(I)+AA(I,J)*ANU(J,ID-1)
  130    CONTINUE
         IF(ISPLIN.LE.2) THEN
            DO 140 I=1,NMU
               DO 140 J=1,NMU
                  S=0.
                  DO 141 K=1,NMU
  141                S=S+AA(I,K)*D(K,J,ID-1)
                  BB(I,J)=BB(I,J)-S
  140       CONTINUE
C
C     In-lining MINV3
C           CALL MINV3(BB)
          BB(2,1)=BB(2,1)/BB(1,1)
          BB(2,2)=BB(2,2)-BB(2,1)*BB(1,2)
          BB(2,3)=BB(2,3)-BB(2,1)*BB(1,3)
          BB(3,1)=BB(3,1)/BB(1,1)
          BB(3,2)=(BB(3,2)-BB(3,1)*BB(1,2))/BB(2,2)
          BB(3,3)=BB(3,3)-BB(3,1)*BB(1,3)-BB(3,2)*BB(2,3)
          BB(3,2)=-BB(3,2)
          BB(3,1)=-BB(3,1)-BB(3,2)*BB(2,1)
          BB(2,1)=-BB(2,1)
          BB(3,3)=UN/BB(3,3)
          BB(2,3)=-BB(2,3)*BB(3,3)/BB(2,2)
          BB(2,2)=UN/BB(2,2)
          BB(1,3)=-(BB(1,2)*BB(2,3)+BB(1,3)*BB(3,3))/BB(1,1)
          BB(1,2)=-BB(1,2)*BB(2,2)/BB(1,1)
          BB(1,1)=UN/BB(1,1)
          BB(1,1)=BB(1,1)+BB(1,2)*BB(2,1)+BB(1,3)*BB(3,1)
          BB(1,2)=BB(1,2)+BB(1,3)*BB(3,2)
          BB(2,1)=BB(2,2)*BB(2,1)+BB(2,3)*BB(3,1)
          BB(2,2)=BB(2,2)+BB(2,3)*BB(3,2)
          BB(3,1)=BB(3,3)*BB(3,1)
          BB(3,2)=BB(3,3)*BB(3,2)
C
            DO 150 I=1,NMU
               DO 150 J=1,NMU
                  D(I,J,ID)=0.
                  DO 151 K=1,NMU
  151                D(I,J,ID)=D(I,J,ID)+BB(I,K)*CC(K,J)
  150       CONTINUE
         ELSE
            DO 160 I=1,NMU
               BB(I,I)=-AA(I,I)+BB(I,I)-CC(I,I)
               DO 160 J=1,NMU
                  FFPD(I,J)=AA(I,I)*FF0D(I,J)
  160       CONTINUE
            DO 162 I=1,NMU
               DO 162 J=1,NMU
                  S=0.
                  DO 163 K=1,NMU
  163                S=S+FFPD(I,K)*D(K,J,ID-1)
                  FFD(I,J)=(BB(I,J)+S)/CC(I,I)
  162       CONTINUE
            DO 164 I=1,NMU
               DO 165 J=1,NMU
  165             FF0D(I,J)=FFD(I,J)
               FFD(I,I)=FFD(I,I)+UN
  164       CONTINUE
C
C     In-lining MINV3
C           CALL MINV3(FFD)
          FFD(2,1)=FFD(2,1)/FFD(1,1)
          FFD(2,2)=FFD(2,2)-FFD(2,1)*FFD(1,2)
          FFD(2,3)=FFD(2,3)-FFD(2,1)*FFD(1,3)
          FFD(3,1)=FFD(3,1)/FFD(1,1)
          FFD(3,2)=(FFD(3,2)-FFD(3,1)*FFD(1,2))/FFD(2,2)
          FFD(3,3)=FFD(3,3)-FFD(3,1)*FFD(1,3)-FFD(3,2)*FFD(2,3)
          FFD(3,2)=-FFD(3,2)
          FFD(3,1)=-FFD(3,1)-FFD(3,2)*FFD(2,1)
          FFD(2,1)=-FFD(2,1)
          FFD(3,3)=UN/FFD(3,3)
          FFD(2,3)=-FFD(2,3)*FFD(3,3)/FFD(2,2)
          FFD(2,2)=UN/FFD(2,2)
          FFD(1,3)=-(FFD(1,2)*FFD(2,3)+FFD(1,3)*FFD(3,3))/FFD(1,1)
          FFD(1,2)=-FFD(1,2)*FFD(2,2)/FFD(1,1)
          FFD(1,1)=UN/FFD(1,1)
          FFD(1,1)=FFD(1,1)+FFD(1,2)*FFD(2,1)+FFD(1,3)*FFD(3,1)
          FFD(1,2)=FFD(1,2)+FFD(1,3)*FFD(3,2)
          FFD(2,1)=FFD(2,2)*FFD(2,1)+FFD(2,3)*FFD(3,1)
          FFD(2,2)=FFD(2,2)+FFD(2,3)*FFD(3,2)
          FFD(3,1)=FFD(3,3)*FFD(3,1)
          FFD(3,2)=FFD(3,3)*FFD(3,2)
C
            DO 170 I=1,NMU
               DO 170 J=1,NMU
                  D(I,J,ID)=FFD(I,J)
                  BB(I,J)=FFD(I,J)/CC(J,J)
  170       CONTINUE
         ENDIF
         DO 180 I=1,NMU
            ANU(I,ID)=0.
            DO 180 J=1,NMU
               ANU(I,ID)=ANU(I,ID)+BB(I,J)*VL(J)
  180    CONTINUE
  200 CONTINUE
C
C     Lower boundary condition
C
      ID=ND
      FR15=FR*1.D-15
      BNU=BN*FR15*FR15*FR15
      PLAND=BNU/(EXP(HK*FR/TEMP(ND ))-UN)
      DPLAN=BNU/(EXP(HK*FR/TEMP(ND-1))-UN)
      DPLAN=(PLAND-DPLAN)/DT(ND-1)
      IF(IBC.EQ.0) THEN
         DO 220 I=1,NMU
            AA(I,I)=AMU(I)/DTP1
            VL(I)=PLAND+AMU(I)*DPLAN+AA(I,I)*ANU(I,ID-1)
            DO 210 J=1,NMU
  210          BB(I,J)=-AA(I,I)*D(I,J,ID-1)
            BB(I,I)=BB(I,I)+AA(I,I)+UN
  220    CONTINUE
       ELSE
         DO 224 I=1,NMU
            A=AMU(I)/DTP1
            B=HALF/A
            AA(I,I)=A
            VL(I)=B*ST0(ID)+PLAND+AMU(I)*DPLAN+AA(I,I)*ANU(I,ID-1)
            DO 222 J=1,NMU
  222          BB(I,J)=B*SS0(ID)*WTMU(J)-AA(I,I)*D(I,J,ID-1)
            BB(I,I)=BB(I,I)+A+B+UN
  224    CONTINUE
      END IF
C
C     In-lining MINV3
C        CALL MINV3(BB)
          BB(2,1)=BB(2,1)/BB(1,1)
          BB(2,2)=BB(2,2)-BB(2,1)*BB(1,2)
          BB(2,3)=BB(2,3)-BB(2,1)*BB(1,3)
          BB(3,1)=BB(3,1)/BB(1,1)
          BB(3,2)=(BB(3,2)-BB(3,1)*BB(1,2))/BB(2,2)
          BB(3,3)=BB(3,3)-BB(3,1)*BB(1,3)-BB(3,2)*BB(2,3)
          BB(3,2)=-BB(3,2)
          BB(3,1)=-BB(3,1)-BB(3,2)*BB(2,1)
          BB(2,1)=-BB(2,1)
          BB(3,3)=UN/BB(3,3)
          BB(2,3)=-BB(2,3)*BB(3,3)/BB(2,2)
          BB(2,2)=UN/BB(2,2)
          BB(1,3)=-(BB(1,2)*BB(2,3)+BB(1,3)*BB(3,3))/BB(1,1)
          BB(1,2)=-BB(1,2)*BB(2,2)/BB(1,1)
          BB(1,1)=UN/BB(1,1)
          BB(1,1)=BB(1,1)+BB(1,2)*BB(2,1)+BB(1,3)*BB(3,1)
          BB(1,2)=BB(1,2)+BB(1,3)*BB(3,2)
          BB(2,1)=BB(2,2)*BB(2,1)+BB(2,3)*BB(3,1)
          BB(2,2)=BB(2,2)+BB(2,3)*BB(3,2)
          BB(3,1)=BB(3,3)*BB(3,1)
          BB(3,2)=BB(3,3)*BB(3,2)
C
      DO 230 I=1,NMU
         ANU(I,ID)=0.
         DO 230 J=1,NMU
            D(I,J,ID)=0.
            ANU(I,ID)=ANU(I,ID)+BB(I,J)*VL(J)
  230 CONTINUE
C
C ***************** Backsolution
C
      FKK(ND)=THIRD
      AJ=0.
      AH=0.
      AK=0.
      DO 235 I=1,NMU
         RMU=WTMU(I)*ANU(I,ID)
         AJ=AJ+RMU
         AH=AH+RMU*AMU(I)
         AK=AK+RMU*AMU(I)*AMU(I)
  235 CONTINUE
      RDD(ID)=AJ
      IF(IBC.EQ.0) THEN
         FKK(ND)=THIRD
       ELSE
         FKK(ID)=AK/AJ
         FHD(IJ)=AH/AJ
      END IF
C
      DO 260 ID=ND-1,1,-1
         DO 240 I=1,NMU
            DO 240 J=1,NMU
               ANU(I,ID)=ANU(I,ID)+D(I,J,ID)*ANU(J,ID+1)
  240    CONTINUE
         AJ=0.
         AK=0.
         DO 250 I=1,NMU
            RMU=WTMU(I)*ANU(I,ID)
            AJ=AJ+RMU
            AK=AK+RMU*AMU(I)*AMU(I)
  250    CONTINUE
C
C         solution of the transfer equation
C         Variables:
C          ANU     - Feautrier intensity
C          RDD     - mean intensity
C          FKK     - Eddington factor f(K) = K/J
C
         FKK(ID)=AK/AJ
         RDD(ID)=AJ
  260 CONTINUE
C
C     the "surface" Eddington factor fH
C
      AH=0.
      DO 270 I=1,NMU
  270    AH=AH+WTMU(I)*AMU(I)*ANU(I,1)
      FH0=AH/AJ-HALF*ALB1
      FH(IJ)=FH0
C
C ********************
C
C     Again solution of the transfer equation, now with Eddington
C     FKK and FH determined above, to insure strict consistency of the
C     radiation field and Eddington factors
C
      if(ilmcor.eq.2) then
         do id=1,nd
            scor(id)=un/(un+ss0(id))
         end do
      end if
C
C     Upper boundary condition
C
      ID=1
      DTP1=DT(ID)
      IF(MOD(ISPLIN,3).EQ.0) THEN
         B=DTP1*HALF
         C=0.
       ELSE
         B=DTP1*THIRD
         C=B*HALF
      END IF
      BQ=UN/(B+QQ0)
      CQ=C*BQ
      BBB(ID)=(FKK(ID)/DTP1+FH0+B)*BQ+SS0(ID)
      CCC(ID)=(FKK(ID+1)/DTP1)*BQ-CQ*(UN+SS0(ID+1))
      VLL=ST0(ID)+CQ*ST0(ID+1)
      IF(IWINBL.LT.0) VLL=VLL+EXTRAD(IJ)/DTP1
      if(ilmcor.eq.2) then
         bbb(id)=bbb(id)*scor(id)
         ccc(id)=ccc(id)*scor(id)
         vll=vll*scor(id)
      end if
      ZZZ(ID)=UN/BBB(ID)
      AANU(ID)=VLL*ZZZ(ID)
      DDD(ID)=CCC(ID)*ZZZ(ID)
      IF(ISPLIN.GT.2) FFF=BBB(ID)/CCC(ID)-UN
C
C     Normal depth point
C
      DO 280 ID=2,ND-1
         DTM1=DTP1
         DTP1=DT(ID)
         DT0=TWO/(DTP1+DTM1)
         AL=UN/DTM1*DT0
         GA=UN/DTP1*DT0
         IF(MOD(ISPLIN,3).EQ.0) THEN
            A=0.
            C=0.
          ELSE IF(ISPLIN.EQ.1) THEN
            A=DTM1*DT0*SIXTH
            C=DTP1*DT0*SIXTH
          ELSE
            A=(UN-HALF*DTP1*DTP1*AL)*SIXTH
            C=(UN-HALF*DTM1*DTM1*GA)*SIXTH
         END IF
         AAA(ID)=AL*FKK(ID-1)-A*(UN+SS0(ID-1))
         CCC(ID)=GA*FKK(ID+1)-C*(UN+SS0(ID+1))
         BBB(ID)=(AL+GA)*FKK(ID)+(UN-A-C)*(UN+SS0(ID))
         VLL=A*ST0(ID-1)+C*ST0(ID+1)+(UN-A-C)*ST0(ID)
         if(ilmcor.eq.2) then
            aaa(id)=aaa(id)*scor(id)
            bbb(id)=bbb(id)*scor(id)
            ccc(id)=ccc(id)*scor(id)
            vll=vll*scor(id)
         end if
         AANU(ID)=VLL+AAA(ID)*AANU(ID-1)
         IF(ISPLIN.LE.2) THEN
            ZZZ(ID)=UN/(BBB(ID)-AAA(ID)*DDD(ID-1))
            DDD(ID)=CCC(ID)*ZZZ(ID)
            AANU(ID)=AANU(ID)*ZZZ(ID)
         ELSE
            SUM=-AAA(ID)+BBB(ID)-CCC(ID)
            FFF=(SUM+AAA(ID)*FFF*DDD(ID-1))/CCC(ID)
            DDD(ID)=UN/(UN+FFF)
            AANU(ID)=AANU(ID)*DDD(ID)/CCC(ID)
         ENDIF
  280 CONTINUE
C
C     Lower boundary condition
C
      ID=ND
      IF(IBC.EQ.0) THEN
         BBB(ID)=FKK(ID)/DTP1+HALF
         AAA(ID)=FKK(ID-1)/DTP1
         VLL=HALF*PLAND+THIRD*DPLAN
       ELSE
         B=UN/DTP1
         A=TWO*B*B
         BBB(ID)=UN+SS0(ID)+B*TWO*FHD(IJ)+A*FKK(ID)
         AAA(ID)=A*FKK(ID-1)
         VLL=ST0(ID)+B*(PLAND+TWOTHR*DPLAN)
      END IF
      if(ilmcor.eq.2) then
         aaa(id)=aaa(id)*scor(id)
         bbb(id)=bbb(id)*scor(id)
         vll=vll*scor(id)
      end if
      EEE(ND)=AAA(ID)/BBB(ID)
      ZZZ(ID)=UN/(BBB(ID)-AAA(ID)*DDD(ID-1))
      RAD1(ID)=(VLL+AAA(ID)*AANU(ID-1))*ZZZ(ID)
      FAK1(ID)=FKK(ND)
      ALRH(ID)=ZZZ(ID)
C
C     Backsolution
C
      DO 290 ID=ND-1,1,-1
         EEE(ID)=AAA(ID)/(BBB(ID)-CCC(ID)*EEE(ID+1))
         RAD1(ID)=AANU(ID)+DDD(ID)*RAD1(ID+1)
         FAK1(ID)=FKK(ID)
         ALRH(ID)=ZZZ(ID)/(UN-DDD(ID)*EEE(ID+1))
c        ALRH(ID)=UN/(BBB(ID)-CCC(ID)*EEE(ID+1))/(UN-EEE(ID)*DDD(ID-1))
         ALRM(ID)=0
         ALRP(ID)=0
  290 CONTINUE
      FLUX(IJ)=FH(IJ)*RAD1(1)
c
C        evaluate approximate Lambda operator
C
C        a) Rybicki-Hummer Lambda^star operator (diagonal)
C           (for JALI = 1)
C
         DO 301 ID=1,ND
            ALIM1(ID)=0.
            ALIP1(ID)=0.
  301    CONTINUE
         IF(JALI.EQ.1) THEN
         DO 310 ID=1,ND 
            ALI1(ID)=ALRH(ID)
  310    CONTINUE
c
         IF(IBC.EQ.0) THEN
            ali1(nd-1)=rad1(nd-1)/st0(nd-1)
            ali1(nd)=rad1(nd)/st0(nd)
         END IF
C
C        for IFALI>5:
C        tridiagonal Rybicki-Hummer operator (off-diagonal terms)
C
         IF(IFALI.GE.6) THEN
         ALIP1(1)=ALRH(2)*DDD(1)
         DO 340 ID=2,ND-1 
            ALIM1(ID)=ALRH(ID-1)*EEE(ID)
            ALIP1(ID)=ALRH(ID+1)*DDD(ID)
  340    CONTINUE
         ALIM1(ND)=ALRH(ND-1)*EEE(ND)
         IF(IBC.EQ.0) THEN
            ALIM1(nd)=0.
            ALIM1(nd-1)=0.
            ALIP1(nd)=0.
            ALIP1(nd-1)=0.
         END IF
         END IF
c
C        b) diagonal Olson-Kunasz Lambda^star operator, 
C           (for JALI = 2)
C
         ELSE IF(JALI.EQ.2) THEN
         DO 360 ID=1,ND-1
            ALI0(ID)=0.
            DO 350 I=1,NMU
               DIV=DT(ID)/AMU(I)
               ALI0(ID)=ALI0(ID)+(UN-EXP(-DIV))/DIV*WTMU(I)
  350       CONTINUE
  360    CONTINUE
         DO 370 ID=2,ND-1
            ALI1(ID)=UN-HALF*(ALI0(ID)+ALI0(ID-1))
  370    CONTINUE
         ALI1(1)=UN-HALF*(ALI0(1)+US0)
         ALI1(ND)=UN-ALI0(ND-1)
         ali1(nd-1)=rad1(nd-1)/st0(nd-1)
         ali1(nd)=rad1(nd)/st0(nd)
	 END IF
C          
C        correction for scattering
C  
         IF(ILMCOR.EQ.1) THEN     
         DO 380 ID=1,ND
             ALI1(ID)=ALI1(ID)*(UN+SS0(ID))
             ALIM1(ID)=ALIM1(ID)*(UN+SS0(ID))
             ALIP1(ID)=ALIP1(ID)*(UN+SS0(ID))
  380    CONTINUE
         END IF
C
  390 CONTINUE
      ISPLIN=ISPL
C
C     radiation pressure
C
      DO ID=1,ND
         PRADT(ID)=PRADT(ID)+RAD1(ID)*FAK1(ID)*WW
      END DO
      PRD0=PRD0+ABSO1(1)*RAD1(1)*FH(IJ)*WW
      IF(IJ.EQ.NFREQ) THEN
         DO ID=1,ND
            PRADT(ID)=PRADT(ID)*PCK
         END DO
         PRD0=PRD0/DENS1(1)*DM(1)*PCK
      END IF
c
      if(chmax.ge.1.01e-3.and.chmax.le.1.03e-3) then
      tauij=taumin
      do id=1,nd
      if(id.gt.1) tauij=tauij+dt(id-1)
      write(97,697) ij,id,tauij,rad1(id),st0(id)/(un+ss0(id)),st0(id),
     *              un+ss0(id),ali1(id)
      end do
  697 format(2i4,1p6e12.4)
      end if
C
C     store quantities for explicit (linearized) frequencies
C
      IF(IJEX(IJ).LE.0) RETURN
      IJE=IJEX(IJ)
      DO ID=1,ND
         RADEX(IJE,ID)=RAD1(ID)
         FAKEX(IJE,ID)=FAK1(ID)
      END DO
      Q0(IJE)=QQ0
      UU0(IJE)=U0
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE OPACF0(ID,NFRQ)
C     ==========================
C
C     Absorption, emission, and scattering coefficients
C     at depth ID
C
C     Input: ID   opacity and emissivity is calculated for the
C                 depth point ID
C     Output: ABSO -  array of absorption coefficient
C             EMIS -  array of emission coefficient
C             SCAT -  array of scattering coefficient 
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      PARAMETER (FRH=3.28805E15, PH2=2.815D29*2., EHB=157802.77355)
      PARAMETER (CFF1=1.3727D-25,CFF2=4.3748D-10,CFF3=2.5993D-7)
      PARAMETER (SGFF0 = 3.694D8)
      DIMENSION FREDG(NLMX),S(NLMX),SUM(NLMX),PRF(MFREQL)
      COMMON/OPACAD/ABAD,EMAD,SCAD,DAT,DAN,DET,DEN,DST,DSN,DDN(MLEVEL)
      COMMON/OPCKEY/NCON,IRSCT,IOPHMI,IOPH2P,IOPHE1,IOPHE2,
     *              IOPHL1,IOPHL2,IPHE2C
C
C     initialize
c
C        this part is analogous to TDPINI - for one depth only
C
         T=TEMP(ID)
         T1=UN/T
         HKT1(ID)=HK*T1
         HKT21(ID)=HKT1(ID)*T1
         TK1(ID)=HKT1(ID)/H
         SQT1(ID)=SQRT(T)
         TEMP1(ID)=T1
         CALL GFREE0(ID)
         EMEL1(ID)=UN
         LASER=ITER.GT.ITLAS
c
C        this part is analogous to OPAINI - for one depth only
C
         ANE=ELEC(ID)
         ELEC1(ID)=UN/ANE
         DENS1(ID)=UN/DENS(ID)
         ELSCAT(ID)=ANE*SIGE
         CALL DWNFR0(ID)
         CALL WNSTOR(ID)
         CALL SABOLF(ID)
c
c        quantities for the bound-free opacity
c
         DO 10 IBF=1,NTRANC
            ITR=ITRBF(IBF)
            II=ILOW(ITR)
            JJ=IUP(ITR)
            IT=ITRA(JJ,II)
            IE=IEL(II)
            NKE=NNEXT(IE)
            CORR=UN
            IF(NKE.NE.JJ) CORR=G(NKE)/G(JJ)*
     *         EXP((ENION(NKE)-ENION(JJ))*TK1(ID))
            ABTRA(ITR,ID)=POPUL(II,ID)
            EMTRA(ITR,ID)=POPUL(JJ,ID)*ANE*SBF(II)*WOP(II,ID)*CORR
   10    CONTINUE
c
c        quantities for the free-free opacity
c
         IF(IELHM.GT.0) THEN
            CFFN(ID)=POPUL(NFIRST(IELH),ID)*ANE
            CFFT(ID)=CFF2-CFF3/T
         END IF
         SGFF=SGFF0/SQT1(ID)*ANE
         DO 20 ION=1,NION
            SFF2(ION,ID)=EXP(FF(ION)*HKT1(ID))
            SFF3(ION,ID)=POPUL(NNEXT(ION),ID)*CHARG2(ION)*SGFF
   20    CONTINUE
c
C        this part is analogous to SGMER0 - for one depth only
C
      IMER=0
      DO 25 II=1,NLEVEL
         IF(IFWOP(II).GE.0) GO TO 25
         IMER=IMER+1
         IMRG(II)=IMER
         IIMER(IMER)=II
         IE=IEL(II)
         CH=IZ(IE)*IZ(IE)
         FRCH(IMER)=FRH*CH
         SGM0(IMER)=PH2*CH*CH
         II0=NQUANT(II-1)+1
            EX=EHB*CH*TEMP1(ID)
            DO 21 I=II0,NLMX
               FREDG(I)=FRCH(IMER)*XI2(I)
               EXI=EXP(EX*XI2(I))
               S(I)=EXI*WNHINT(I,ID)*XI3(I)
               SUM(I)=0.
   21       CONTINUE
            SUM(NLMX)=S(NLMX)
            DO 22 I=NLMX-1,II0,-1
               SUM(I)=SUM(I+1)+S(I)
   22       CONTINUE
            DO 23 I=1,II0-1
               SUM(I)=SUM(II0)
   23       CONTINUE
            SGEM=SGM0(IMER)/GMER(IMER,ID)
            DO 24 I=1,NLMX
               SGMSUM(I,IMER,ID)=SUM(I)*SGEM
   24       CONTINUE
   25 CONTINUE
C
C     initialization of the line opacity
C
      IF(NFRQ.GT.NFREQC) THEN
      DO 29 ITR=1,NTRANS
         IF(.NOT.LINE(ITR)) GO TO 29
         IF(INTMOD(ITR).EQ.0) GO TO 29
         INDXA=IABS(INDEXP(ITR))
         IJL0=IFR0(ITR)
         IJL1=IFR1(ITR)
         II=ILOW(ITR)
         JJ=IUP(ITR)
         IF(INDXA.LT.2.OR.INDXA.GT.4) THEN
         CALL LINPRO(ITR,ID,PRF)
         DO 28 IJ=IJL0,IJL1
            PRFLIN(IJ,ID)=PRF(IJ-IJL0+1)
   28    CONTINUE
         ENDIF
   29 CONTINUE
      GG=G(II)/G(JJ)
      IF(IFWOP(JJ).GE.0) THEN
         PI=POPUL(II,ID)*WOP(JJ,ID)
         PJ=POPUL(JJ,ID)*WOP(II,ID)*GG*EXP(FR0(ITR)*HKT1(ID))
       ELSE
         PI=POPUL(II,ID)
         PJ=POPUL(JJ,ID)*WOP(II,ID)*
     *      G(II)/GMER(IMRG(JJ),ID)*EXP(FR0(ITR)*HKT1(ID))
      END IF
      ABTRA(ITR,ID)=PI
      EMTRA(ITR,ID)=PJ
      IF(PI.LT.PJ.AND.LASER) THEN
         ABTRA(ITR,ID)=0.
         EMTRA(ITR,ID)=0.
      END IF
      END IF
C
C     ---------------------------------------------------------
C     
C     loop over frequency points
C
      DO 200 IJ=1,NFRQ
         ABSO(IJ)=ELSCAT(ID)
         EMIS(IJ)=0.
         SCAT(IJ)=ELSCAT(ID)
C
C     basic frequency- and depth-dependent quantities
C
         FR=FREQ(IJ)
         FRINV=UN/FR
         FR3INV=FRINV*FRINV*FRINV
         XKF(ID)=EXP(-HKT1(ID)*FR)
         XKF1(ID)=UN-XKF(ID)
         XKFB(ID)=XKF(ID)*BNUE(IJ)
C
C ********  1. bound-free contribution
C
         DO 30 IBF=1,NTRANC
            ITR=ITRBF(IBF)
            II=ILOW(ITR)
            JJ=IUP(ITR)
            SG=CROSS(IBF,IJ)
            IF(IFWOP(II).LT.0) THEN
               IMER=IMRG(II)
               CALL SGMER1(FRINV,FR3INV,IMER,ID,SGME1)
               SGMG(IMER,ID)=SGME1
               SG=SGME1
            END IF
            IF(SG.LE.0.) GO TO 30
            IF(MCDW(ITR).GT.0) THEN
               IZZ=IZ(IEL(II))
               CALL DWNFR1(FR,FR0(ITR),ID,IZZ,DW1)
               DWF1(MCDW(ITR),ID)=DW1
               SG=SG*DW1
            END IF
            EMISBF=SG*EMTRA(ITR,ID)
            ABSO(IJ)=ABSO(IJ)+SG*ABTRA(ITR,ID)
            EMIS(IJ)=EMIS(IJ)+EMISBF
   30    CONTINUE
C
C ******** 2. free-free contribution
C
         DO 40 ION=1,NION
            IT=ITRA(NNEXT(ION),NNEXT(ION))
C
C           hydrogenic with Gaunt factor = 1
C
            IF(IT.EQ.1) THEN
               SF1=SFF3(ION,ID)*FR3INV
               SF2=SFF2(ION,ID)
               IF(FR.LT.FF(ION)) SF2=UN/XKF(ID)
               ABSOFF=SF1*SF2
               ABSO(IJ)=ABSO(IJ)+ABSOFF
               EMIS(IJ)=EMIS(IJ)+ABSOFF
C
C            hydrogenic with exact Gaunt factor 
C
             ELSE IF(IT.EQ.2) THEN
               SF1=SFF3(ION,ID)*FR3INV
               SF2=SFF2(ION,ID)
               IF(FR.LT.FF(ION)) SF2=UN/XKF(ID)
               CALL GFREE1(ID,FR,CHARG2(ION),GFR)
               SF2=SF2-UN+GFR
               ABSOFF=SF1*SF2
               ABSO(IJ)=ABSO(IJ)+ABSOFF
               EMIS(IJ)=EMIS(IJ)+ABSOFF
C
C            H minus free-free opacity
C
             ELSE IF(IT.EQ.3) THEN
               ABSOFF=(CFF1+CFFT(ID)*FRINV)*CFFN(ID)*FRINV
               ABSO(IJ)=ABSO(IJ)+ABSOFF
               EMIS(IJ)=EMIS(IJ)+ABSOFF
C
C            special evaluation of the cross-section
C
             ELSE IF(IT.LT.0) THEN
               ABSOFF=FFCROS(ION,IT,TEMP(ID),FR)*
     *                POPUL(NNEXT(ION),ID)*ELEC(ID)
               ABSO(IJ)=ABSO(IJ)+ABSOFF
               EMIS(IJ)=EMIS(IJ)+ABSOFF
            END IF
   40    CONTINUE
C
C     ********  3. - additional continuum opacity (OPADD)
C
         IF(IOPADD.NE.0) THEN
            CALL OPADD(0,ICALL,IJ,ID)
            IF(ICALL.EQ.1) ICALL=0
            ABSO(IJ)=ABSO(IJ)+ABAD
            EMIS(IJ)=EMIS(IJ)+EMAD
            SCAT(IJ)=SCAT(IJ)+SCAD
         END IF
C
C ********  4. - opacity and emissivity in lines
C
         IF(IJLIN(IJ).GT.0) THEN
C
C        the "primary" line at the given frequency
C
            ITR=IJLIN(IJ)
            SG=PRFLIN(IJ,ID)
            ABSO(IJ)=ABSO(IJ)+SG*ABTRA(ITR,ID)
            EMIS(IJ)=EMIS(IJ)+SG*EMTRA(ITR,ID)
         ENDIF
         IF(NLINES(IJ).LE.0) GO TO 110
C
C        the "overlapping" lines at the given frequency
C
         DO 100 ILINT=1,NLINES(IJ)
            ITR=ITRLIN(ILINT,IJ)
            IJ0=IFR0(ITR)
            DO 60 IJT=IJ0,IFR1(ITR)
               IF(FREQ(IJT).LE.FR) THEN
                  IJ0=IJT
                  GO TO 70
               END IF
   60       CONTINUE
   70       IJ1=IJ0-1
            X=UN/(FREQ(IJ1)-FREQ(IJ0))
            A1=(FR-FREQ(IJ0))*X
            X=UN/(FREQ(IJ1)-FREQ(IJ0))
            A2=(FREQ(IJ1)-FR)*X
            SG=A1*PRFLIN(IJ1,ID)+A2*PRFLIN(IJ0,ID)
            ABSO(IJ)=ABSO(IJ)+SG*ABTRA(ITR,ID)
            EMIS(IJ)=EMIS(IJ)+SG*EMTRA(ITR,ID)
  100    CONTINUE
  110    CONTINUE
C
C        ----------------------------
C        total opacity and emissivity
C        ----------------------------
C
         ABSO(IJ)=ABSO(IJ)-EMIS(IJ)*XKF(ID)
         EMIS(IJ)=EMIS(IJ)*XKFB(ID)
  200 CONTINUE
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE SRTFRQ
C     =================
C
C     Sort the frequency sets, and assign to each frequency
C     a list of contributing transitions
C     Select final frequency set.
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      PARAMETER (SIXTH=UN/6.,FTH=4./3.,V0X=4.D-4,VCX=10.*V0X)
      DIMENSION SGZ(MTRANS),SX(500)
C
C     Sort frequencies and assign primary line
C
      CALL INDEXX(NFREQ,FREQ,NLINES)
      DO 10 IJ=1,NFREQ
         KIJ(NLINES(IJ))=NFREQ-IJ+1
   10 CONTINUE
      DO 20 IT=1,NTRANS
         IF(LINEXP(IT)) GOTO 20
         KFR0(IT)=KIJ(IFR0(IT))
         KFR1(IT)=KIJ(IFR1(IT))
         DO 25 IJ=IFR0(IT),IFR1(IT)
            IJLIN(IJ)=IT
   25    CONTINUE
   20 CONTINUE
      DO 30 IJ=1,NFREQ
         JIK(KIJ(IJ))=IJ
   30 CONTINUE
      JK1=JIK(1)
      IF(IJLIN(JK1).NE.0)
     *  CALL STOP(' Largest freq. is a line freq. - (SRTFRQ)')
      JK1=JIK(NFREQ)
      IF(IJLIN(JK1).NE.0)
     *  CALL STOP(' Smallest freq. is a line freq. - (SRTFRQ)')
C
C     lines or ODFs associated with each frequency
C
      NLIMAX=0
      DO 40 IJ=1,NFREQ
         NLINES(IJ)=0
         DO 50 IT=1,NTRANS
            IF(LINEXP(IT)) GOTO 50
            IF(KIJ(IJ).LT.KFR0(IT)) GOTO 50
            IF(KIJ(IJ).GT.KFR1(IT)) GOTO 50
            IF(IJLIN(IJ).EQ.IT) GOTO 50
            NLINES(IJ)=NLINES(IJ)+1
            IF(NLINES(IJ).GT.MITJ)
     *        CALL STOP('Too many overlapping trans., MITJ - (SRTFRQ)')
            ITRLIN(NLINES(IJ),IJ)=IT
   50    CONTINUE
         IF(NLINES(IJ).GT.NLIMAX) NLIMAX=NLINES(IJ)
   40 CONTINUE
      WRITE(6,611) NLIMAX
  611 FORMAT(' MAXIMUM NUMBER OF OVERLAPPING TRANSITIONS:  ',I3/)
C
C     Select final set of frequencies:
C        IJX = 1 : included frequency
C        IJX =-1 : rejected frequency
C        IJX = 0 : used for rates, but no contribution of primary
C                   transition to opacity
C
      NPPX=NFREQ-NFREQC
      DO 310 IT=1,NTRANS
         IF(LINEXP(IT)) GOTO 310
         IF(ABS(INDEXP(IT)).NE.3) GOTO 310
         IF(PROF(IFR0(IT)+1).GT.PROF(IFR1(IT)-1)) THEN
           DO 315 IJ=IFR0(IT)+5,IFR1(IT)-1
              IJX(IJ)=-1
              NPPX=NPPX-1
  315      CONTINUE
         ELSE
           DO 316 IJ=IFR0(IT)+1,IFR1(IT)-5
              IJX(IJ)=-1
              NPPX=NPPX-1
  316      CONTINUE
         ENDIF
  310 CONTINUE
      ISX=0
      DO 320 IJ=1,NFREQ
        ISX=ISX-1
        IF(ISX.GT.0) GOTO 320
        IJP=JIK(IJ)
        DX0=0.
        IF(IJX(IJP).EQ.1) GOTO 320
        IF(PROF(IJP).EQ.0.) GOTO 320
        DX0=V0X*FREQ(IJP)
        DNUX=ABS(FREQ(JIK(IJ-1))-FREQ(IJP))
        IF(DNUX.GT.DX0) THEN
          IJX(IJP)=1
          NPPX=NPPX+1
        ELSE
          NPX=0
          DO WHILE (DNUX.LT.DX0 .AND. IJX(JIK(IJ+NPX)).EQ.-1)
            ITRX=IJLIN(JIK(IJ+NPX))
            PSX0=PROF(IFR0(ITRX+1))
            IF(PSX0.GT.0.) THEN
              SX0=PROF(JIK(IJ+NPX))/PSX0
              SX(NPX+1)=PROF(JIK(IJ+NPX))/PROF(IJP)*SX0
            ELSE
              SX(NPX+1)=0.
            ENDIF
            NPX=NPX+1
            DNUX=ABS(FREQ(JIK(IJ-1))-FREQ(JIK(IJ+NPX)))
          END DO
          IF(NPX.EQ.1) THEN
            IJX(IJP)=1
            NPPX=NPPX+1
          ELSE
            SXX=-1.
            DO 325 IPX=1,NPX
              IF(SX(IPX).GT.SXX) THEN
                SXX=SX(IPX)
                ISX=IPX
              ENDIF
  325       CONTINUE
            IJX(JIK(IJ+ISX))=1
            NPPX=NPPX+1
          ENDIF
        ENDIF
  320 CONTINUE
      DO 330 IJ=1,NFREQ
        IJP=JIK(IJ)
        IF(IJP.GT.NFREQC) GOTO 330
        IF(IJX(IJP).EQ.1) THEN
          NPPX=NPPX+1
          GOTO 330
        ENDIF
        DX0=VCX*FREQ(IJP)
        NIXA=0
        DO WHILE (IJX(JIK(IJ-NIXA)).NE.1)
           NIXA=NIXA+1
        ENDDO
        NIXB=0
        DO WHILE (IJX(JIK(IJ+NIXB)).NE.1)
           NIXB=NIXB+1
        ENDDO
        DNUXA=ABS(FREQ(JIK(IJ-NIXA))-FREQ(IJP))
        DNUXB=ABS(FREQ(JIK(IJ+NIXB))-FREQ(IJP))
        IF(DNUXA.GT.DX0 .AND. DNUXB.GT.DX0) THEN
          IJX(IJP)=1
          NPPX=NPPX+1
        ELSE
          IJX(IJP)=-1
        ENDIF
  330 CONTINUE
c
c     correction
c
      do ij=1,nfreqc
         ijx(ij)=1
      end do
      do ije=1,nfreqe
         ijx(ijfr(ij))=1
      end do
C
C     weights
C
      DO 100 IJ=1,NFREQ
         W(IJ)=0.
         KJ0=KIJ(IJ)
         IF(IJX(JIK(KJ0)).EQ.-1) GOTO 100
         IF(KJ0.GE.2 .AND. KJ0.LT.NFREQ) THEN
           IK1=KJ0-1
           DO WHILE (IJX(JIK(IK1)).EQ.-1)
             IK1=IK1-1
           ENDDO
           IK2=KJ0+1
           DO WHILE (IJX(JIK(IK2)).EQ.-1)
             IK2=IK2+1
           ENDDO
           W(IJ)=HALF*ABS(FREQ(JIK(IK1))-FREQ(JIK(IK2)))
         ELSE IF(KJ0.EQ.1) THEN
           W(IJ)=HALF*ABS(FREQ(JIK(KJ0))-FREQ(JIK(KJ0+1)))
         ELSE IF(KJ0.EQ.NFREQ) THEN
           W(IJ)=HALF*ABS(FREQ(JIK(KJ0-1))-FREQ(JIK(KJ0)))
         ENDIF
  100 CONTINUE
C
C     Correction for Simpson weights
C
      JK1=JIK(1)
      DO 120 IJ=2,NFREQ,2
         JK2=JIK(IJ)
         JK3=JIK(IJ+1)
         IF(IJLIN(JK2).NE.0 .OR. IJLIN(JK3).NE.0) GOTO 130
         IF(WCH(JK2).NE.0.) GOTO 130
         W(JK1)=W(JK1)-SIXTH*W(JK2)
         W(JK3)=W(JK3)-SIXTH*W(JK2)
         W(JK2)=W(JK2)*FTH
         JK1=JK3
  120 CONTINUE
  130 JK1=JIK(NFREQ)
      DO 140 IJ=NFREQ-1,1,-2
         JK2=JIK(IJ)
         JK3=JIK(IJ-1)
         IF(IJLIN(JK2).NE.0 .OR. IJLIN(JK3).NE.0) GOTO 150
         IF(WCH(JK2).NE.0.) GOTO 150
         W(JK1)=W(JK1)-SIXTH*W(JK2)
         W(JK3)=W(JK3)-SIXTH*W(JK2)
         W(JK2)=W(JK2)*FTH
         JK1=JK3
  140 CONTINUE
  150 CONTINUE
C
C     check accuracy of weights for integration
C
      Z0=0.
      Z1=0.
      Z2=0.
      ZH=0.
      T1=TEFF
      T2=TWO*TEFF
      T3=HALF*TEFF
      X1=HK/T1
      X2=HK/T2
      X3=HK/T3
      DO 200 IJ=1,NFREQ
         Z0=Z0+W(IJ)
         X15=FREQ(IJ)*1.D-15
         BNZ=BN*X15*X15*X15
         Z1=Z1+W(IJ)*BNZ/(EXP(FREQ(IJ)*X1)-1)
         Z2=Z2+W(IJ)*BNZ/(EXP(FREQ(IJ)*X2)-1)
         ZH=ZH+W(IJ)*BNZ/(EXP(FREQ(IJ)*X3)-1)
  200 CONTINUE
      T1S=SQRT(SQRT(0.25*Z1/SIG4P))
      T1ER=T1S/T1-UN
      T2S=SQRT(SQRT(0.25*Z2/SIG4P))
      T2ER=T2S/T2-UN
      T3S=SQRT(SQRT(0.25*ZH/SIG4P))
      T3ER=T3S/T3-UN
      JK1=JIK(1)
      JK2=JIK(NFREQ)
      Z00=FREQ(JK1)-FREQ(JK2)
      WRITE(6,601) FREQ(JK1),FREQ(JK2),Z00,Z0,T3,T3ER,T1,T1ER,T2,T2ER
  601 FORMAT(/' ACCURACY OF INTEGRATIONS:',/,
     * ' Interval:',1p4e16.8,/,
     * 15x,' Planck functions:',9x,0pf12.0,4x,1pe12.4,/,
     * 42x,0pf12.0,4x,1pe12.4,/,42x,0pf12.0,4x,1pe12.4,/)
      WRITE(6,602) NFREQ,NPPX
  602 FORMAT(' TOTAL NUMBER OF FREQUENCIES:',I8,/,
     *       ' SELECTED FREQUENCIES:       ',I8)
C
C     check integrations for rates
C
c     DO 400 IJ=1,NFREQ
c       IF(IJX(IJ).EQ.-1) GOTO 400
c       FR=FREQ(IJ)
c       ITR=IJLIN(IJ)
c       IF(ITR.GT.0) THEN
c         SGZ(ITR)=SGZ(ITR)+W(IJ)*PROF(IJ)
c       ENDIF
c       IF(NLINES(IJ).LE.0) GO TO 400
c       DO 410 ILINT=1,NLINES(IJ)
c         ITR=ITRLIN(ILINT,IJ)
c         IJ0=IFR0(ITR)
c         DO 420 IJT=IJ0,IFR1(ITR)
c           IF(FREQ(IJT).LE.FR) THEN
c             IJ0=IJT
c             GO TO 421
c           END IF
c 420     CONTINUE
c 421     IJ1=IJ0-1
c         A1=(FR-FREQ(IJ0))/(FREQ(IJ1)-FREQ(IJ0))
c         A2=UN-A1
c         SGW=A1*PROF(IJ1)+A2*PROF(IJ0)
c         SGZ(ITR)=SGZ(ITR)+W(IJ)*SGW
c 410   CONTINUE
c 400 CONTINUE
c     DO 500 ITR=1,NTRANS
c       rzal=0.
c       IF(ABS(INDEXP(ITR)).NE.3) GOTO 501
c       rzz=0.
c       DO 520 IJT=IFR0(ITR)+1,IFR1(ITR)-1
c         rzz=rzz+prof(ijt)*abs(freq(ijt-1)-freq(ijt+1))
c 520   CONTINUE
c       rzz=rzz+prof(ifr0(itr))*abs(freq(ifr0(itr))-freq(ifr0(itr)+1))
c       rzz=rzz+prof(ifr1(itr))*abs(freq(ifr1(itr)-1)-freq(ifr1(itr)))
c       rzal=two*sgz(itr)/rzz
c 500 CONTINUE
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE INIFRC(IALIEX)
C     =========================
C
C     Setup continuum frequencies, including frequencies around
C       ionization limits
C          IALIEX=0 :  setup frequencies, all ALI
C          IALIEX=1 :  change IJALI for explicit frequencies
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      DIMENSION FRLEV(MLEVEL),IENS(MLEVEL),IJFL(MLEVEL)
      PARAMETER (THIRD=UN/3.,FTH=4./3.)
C
      SAVE IJFL
C
      DFTAIL=HALF
      NFTAIL=21
      IF(IALIEX.EQ.1) THEN
c 
        if(nffix.lt.0) then
           do ij=1,nfreqc
              ijali(ij)=0
           end do
           return
        end if
c
        IF(NFFIX.EQ.2) RETURN
        DO 10 IT=1,NTRANS
          IF(LINE(IT)) GOTO 10
          IF(IFC1(IT).EQ.0) GOTO 10
          IJFL0=IJFL(ILOW(IT))+1
          IF(IFC1(IT).LT.100) THEN
            DO 11 IJ=IFC0(IT),IFC1(IT)
              IJFLS=IJFL0-IJ
              IF(IJFLS.GE.1) THEN
                IJALI(IJFLS)=0
                IJX(IJFLS)=1
              ENDIF
   11       CONTINUE
          ELSE
            DO 12 IJ=IJFL0,1,-1
              IJALI(IJ)=0
              IJX(IJ)=1
   12       CONTINUE
          ENDIF
   10   CONTINUE
        RETURN
      ENDIF
C
      NEND=NFTAIL
      DIVEND=DFTAIL
      NJC=NFREQC/5
      DNX=UN-UN/FLOAT(NJC)
      CALL INDEXX(NLEVEL,ENION,IENS)
      DO 50 IL=1,NLEVEL
         ILS=IENS(NLEVEL-IL+1)
         FRLEV(IL)=ENION(ILS)/H
         IJFL(IL)=0
   50 CONTINUE
      NFREQC=NEND+1
      IF(FRCMAX.LE.0.) THEN
         FREQ(1)=6.25D11*TEFF
      ELSE
         FREQ(1)=FRCMAX
      ENDIF
      FREQ(NEND)=1.000001*FRLEV(1)
      FREQ(NEND+1)=0.999999*FRLEV(1)
      NEND1=NEND/2+1
      XEND=UN/FLOAT(NEND1-1)
      FREQ(NEND1)=FREQ(1)-(UN-DIVEND)*(FREQ(1)-FREQ(NEND))
      IJX(NEND+1)=1
C
C     high-frequency tail of the spectrum - a two-part Simpson integration
C
C     1st part - from the highest frequency FRCMAX to a division freq.
C      
      IJX(1)=1
      IJX(NEND1)=1
      D121=XEND*(FREQ(1)-FREQ(NEND1))
      DO IJ=2,NEND1-1
         FREQ(IJ)=FREQ(IJ-1)-D121
         IJX(IJ)=2
      END DO
      D121=THIRD*(FREQ(1)-FREQ(2))
      DO IJ=2,NEND1-1,2
         W(IJ)=4.*D121
         W(IJ-1)=W(IJ-1)+D121
         W(IJ+1)=W(IJ+1)+D121
         WCH(IJ)=0.
      END DO
C
C     2nd part - from the division freq to the first discontinuity
C      
      IJX(NEND)=1
      IJX(NEND+1)=1
      D121=XEND*(FREQ(NEND1)-FREQ(NEND))
      DO IJ=NEND1+1,NEND-1
         FREQ(IJ)=FREQ(IJ-1)-D121
         IJX(IJ)=2
      END DO
      D121=THIRD*(FREQ(NEND1)-FREQ(NEND1+1))
      DO IJ=NEND1+1,NEND-1,2
         W(IJ)=4.*D121
         W(IJ-1)=W(IJ-1)+D121
         W(IJ+1)=W(IJ+1)+D121
         WCH(IJ)=0.
      END DO
C 
C     the 1st discontinuity - the one with the highest frequency 
C     
      HAEND=HALF*(FREQ(NEND)-FREQ(NEND+1))
      XEND=UN/FLOAT(NEND-1)
      W(NEND)=W(NEND)+HAEND
      W(NEND+1)=W(NEND+1)+HAEND
      WCH(1)=0.
      WCH(NEND)=HAEND
      ILS=IENS(NLEVEL)
      IJFL(ILS)=NEND
      IL0=NLEVEL
      IF(FRCMIN.LE.0.) FRCMIN=1.D12
      FRCLST=FRLEV(IL0)
      DO WHILE(FRCLST.LT.FRCMIN)
         IF(FRLEV(IL0).GT.0.) WRITE(10,600) IL0,FRLEV(IL0)
  600    FORMAT(' Edge at frequency smaller than 1.d12',i5,e12.4)
         IL0=IL0-1
         FRCLST=FRLEV(IL0)
      END DO
      IL0=2
C
C     the following discontinuity - with lower an lower frequency 
C     
  100 FRC0=DNX*FREQ(NFREQC)
      IF(FRC0.LT.FRCLST) THEN
        NFREQC=NFREQC+2
        FREQ(NFREQC-1)=1.000001*FRCLST
        FREQ(NFREQC)=0.999999*FRCLST
        IJX(NFREQC-1)=1
        IJX(NFREQC)=1
        W(NFREQC)=W(NFREQC)+HALF*(FREQ(NFREQC-1)-FREQ(NFREQC))
        W(NFREQC-1)=W(NFREQC-1)+HALF*(FREQ(NFREQC-2)-FREQ(NFREQC))
        W(NFREQC-2)=W(NFREQC-2)+HALF*(FREQ(NFREQC-2)-FREQ(NFREQC-1))
        WCH(NFREQC-1)=HALF*(FREQ(NFREQC-1)-FREQ(NFREQC))
        WCH(NFREQC-2)=HALF*(FREQ(NFREQC-2)-FREQ(NFREQC-1))
        ILS=IENS(NLEVEL-IL0+1)
        IJFL(ILS)=NFREQC-1
        IF(IL0.LT.NLEVEL) THEN
          DO 110 IL=IL0+1,NLEVEL
            ILS=IENS(NLEVEL-IL+1)
            IJFL(ILS)=NFREQC-1
            IF(FRLEV(IL).LT.FRCMIN) IJFL(ILS)=0
  110     CONTINUE 
        ENDIF
        D121=XEND*(FREQ(NFREQC)-FRCMIN)
        DO 120 IJ=NFREQC+1,NFREQC+NEND-1
          FREQ(IJ)=FREQ(IJ-1)-D121
          IJX(IJ)=2
          WCH(IJ)=0.
  120   CONTINUE
        IJX(NFREQC+NEND-1)=1
        DO 130 IJ=NFREQC+1,NFREQC+NEND-2,2
           W(IJ)=FTH*D121
           W(IJ-1)=W(IJ-1)+THIRD*D121
           W(IJ+1)=W(IJ+1)+THIRD*D121
  130   CONTINUE
        WCH(NFREQC)=THIRD*D121
        NFREQC=NFREQC+NEND-1
        GOTO 200
      ENDIF
      DF0=FRLEV(IL0)+0.1*(FREQ(NFREQC)-FRC0)
      FRTL=1.000001*FRLEV(IL0)
      IF(FRC0.GT.DF0) THEN
        NFREQC=NFREQC+1
        FREQ(NFREQC)=FRC0
        IJX(NFREQC)=2
        W(NFREQC)=W(NFREQC)+HALF*(FREQ(NFREQC-1)-FREQ(NFREQC))
        W(NFREQC-1)=W(NFREQC-1)+HALF*(FREQ(NFREQC-1)-FREQ(NFREQC))
        WCH(NFREQC-1)=HALF*(FREQ(NFREQC-1)-FREQ(NFREQC))
      ELSE IF(FRTL.LT.FREQ(NFREQC)) THEN
        NFREQC=NFREQC+2
        FREQ(NFREQC-1)=FRTL
        FREQ(NFREQC)=0.999999*FRLEV(IL0)
        IJX(NFREQC-1)=1
        IJX(NFREQC)=1
        W(NFREQC)=W(NFREQC)+HALF*(FREQ(NFREQC-1)-FREQ(NFREQC))
        W(NFREQC-1)=W(NFREQC-1)+HALF*(FREQ(NFREQC-2)-FREQ(NFREQC))
        W(NFREQC-2)=W(NFREQC-2)+HALF*(FREQ(NFREQC-2)-FREQ(NFREQC-1))
        WCH(NFREQC-1)=HALF*(FREQ(NFREQC-1)-FREQ(NFREQC))
        WCH(NFREQC-2)=HALF*(FREQ(NFREQC-2)-FREQ(NFREQC-1))
        ILS=IENS(NLEVEL-IL0+1)
        IJFL(ILS)=NFREQC-1
        IL0=IL0+1
      ELSE
        ILS=IENS(NLEVEL-IL0+1)
        IJFL(ILS)=NFREQC-1
        IL0=IL0+1
      ENDIF
      GOTO 100
C
  200 SUMWC=0.
      DO 300 IJ=1,NFREQC
        IJALI(IJ)=1
  300 CONTINUE
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE INIFRD(IALIEX)
C     =========================
C
C     Setup continuum frequencies, including frequencies around
C       ionization limits
C          IALIEX=0 :  setup frequencies, all ALI
C          IALIEX=1 :  change IJALI for explicit frequencies
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      DIMENSION FRLEV(MLEVEL),IENS(MLEVEL),IJFL(MLEVEL),IJXCO(MFREQC),
     *          FREQCO(MFREQC),WCO(MFREQC),WCHCO(MFREQC)
      PARAMETER (THIRD=UN/3.,FTH=4./3.)
C
      SAVE IJFL
C
      IF(IALIEX.EQ.1) THEN
c 
        if(nffix.lt.0) then
           do ij=1,nfreqc
              ijali(ij)=0
           end do
           return
        end if
c
        IF(NFFIX.EQ.2) RETURN
        DO 10 IT=1,NTRANS
          IF(LINE(IT)) GOTO 10
          IF(IFC1(IT).EQ.0) GOTO 10
          IJFL0=IJFL(ILOW(IT))+1
          IF(IFC1(IT).LT.100) THEN
            DO 11 IJ=IFC0(IT),IFC1(IT)
              IJFLS=IJFL0-IJ
              IF(IJFLS.GE.1) THEN
                IJALI(IJFLS)=0
                IJX(IJFLS)=1
              ENDIF
   11       CONTINUE
          ELSE
            DO 12 IJ=IJFL0,1,-1
              IJALI(IJ)=0
              IJX(IJ)=1
   12       CONTINUE
          ENDIF
   10   CONTINUE
        RETURN
      ENDIF
C
      NEND=NFTAIL
      DIVEND=DFTAIL
      NJC=NFREQC/5
      DNX=UN-UN/FLOAT(NJC)
      CALL INDEXX(NLEVEL,ENION,IENS)
      DO 50 IL=1,NLEVEL
         ILS=IENS(NLEVEL-IL+1)
         FRLEV(IL)=ENION(ILS)/H
         IJFL(IL)=0
   50 CONTINUE
      NFREQC=NEND+1
      IF(FRCMAX.LE.0.) THEN
         FREQCO(1)=6.25D11*TEFF
      ELSE
         FREQCO(1)=FRCMAX
      ENDIF
      FREQCO(NEND)=1.000001*FRLEV(1)
      FREQCO(NEND+1)=0.999999*FRLEV(1)
      NEND1=NEND/2+1
      XEND=UN/FLOAT(NEND1-1)
      FREQCO(NEND1)=FREQCO(1)-(UN-DIVEND)*(FREQCO(1)-FREQCO(NEND))
      IJXCO(NEND+1)=1
C
C     high-frequency tail of the spectrum - a two-part Simpson integration
C
C     1st part - from the highest frequency FRCMAX to a division freq.
C      
      IJXCO(1)=1
      IJXCO(NEND1)=1
      D121=XEND*(FREQCO(1)-FREQCO(NEND1))
      DO IJ=2,NEND1-1
         FREQCO(IJ)=FREQCO(IJ-1)-D121
         IJXCO(IJ)=2
      END DO
      D121=THIRD*(FREQCO(1)-FREQCO(2))
      DO IJ=2,NEND1-1,2
         WCO(IJ)=4.*D121
         WCO(IJ-1)=WCO(IJ-1)+D121
         WCO(IJ+1)=WCO(IJ+1)+D121
         WCHCO(IJ)=0.
      END DO
C
C     2nd part - from the division freq to the first discontinuity
C      
      IJXCO(NEND)=1
      IJXCO(NEND+1)=1
      D121=XEND*(FREQCO(NEND1)-FREQCO(NEND))
      DO IJ=NEND1+1,NEND-1
         FREQCO(IJ)=FREQCO(IJ-1)-D121
         IJXCO(IJ)=2
      END DO
      D121=THIRD*(FREQCO(NEND1)-FREQCO(NEND1+1))
      DO IJ=NEND1+1,NEND-1,2
         WCO(IJ)=4.*D121
         WCO(IJ-1)=WCO(IJ-1)+D121
         WCO(IJ+1)=WCO(IJ+1)+D121
         WCHCO(IJ)=0.
      END DO
C 
C     the 1st discontinuity - the one with the highest frequency 
C     
      HAEND=HALF*(FREQCO(NEND)-FREQCO(NEND+1))
      XEND=UN/FLOAT(NEND-1)
      WCO(NEND)=WCO(NEND)+HAEND
      WCO(NEND+1)=WCO(NEND+1)+HAEND
      WCHCO(1)=0.
      WCHCO(NEND)=HAEND
      ILS=IENS(NLEVEL)
      IJFL(ILS)=NEND
      IL0=NLEVEL
      IF(FRCMIN.LE.0.) FRCMIN=1.D12
      FRCLST=FRLEV(IL0)
      DO WHILE(FRCLST.LT.FRCMIN)
         IF(FRLEV(IL0).GT.0.) WRITE(10,600) IL0,FRLEV(IL0)
  600    FORMAT(' Edge at frequency smaller than 1.d12',i5,e12.4)
         IL0=IL0-1
         FRCLST=FRLEV(IL0)
      END DO
      IL0=2
C
  100 FRC0=DNX*FREQCO(NFREQC)
      IF(FRC0.LT.FRCLST) THEN
        NFREQC=NFREQC+2
        FREQCO(NFREQC-1)=1.000001*FRCLST
        FREQCO(NFREQC)=0.999999*FRCLST
        IJXCO(NFREQC-1)=1
        IJXCO(NFREQC)=1
        WCO(NFREQC)=WCO(NFREQC)+HALF*(FREQCO(NFREQC-1)-FREQCO(NFREQC))
        WCO(NFREQC-1)=WCO(NFREQC-1)+
     *                HALF*(FREQCO(NFREQC-2)-FREQCO(NFREQC))
        WCO(NFREQC-2)=WCO(NFREQC-2)+
     *                HALF*(FREQCO(NFREQC-2)-FREQCO(NFREQC-1))
        WCHCO(NFREQC-1)=HALF*(FREQCO(NFREQC-1)-FREQCO(NFREQC))
        WCHCO(NFREQC-2)=HALF*(FREQCO(NFREQC-2)-FREQCO(NFREQC-1))
        ILS=IENS(NLEVEL-IL0+1)
        IJFL(ILS)=NFREQC-1
        IF(IL0.LT.NLEVEL) THEN
          DO 110 IL=IL0+1,NLEVEL
            ILS=IENS(NLEVEL-IL+1)
            IJFL(ILS)=NFREQC-1
            IF(FRLEV(IL).LT.FRCMIN) IJFL(ILS)=0
  110     CONTINUE 
        ENDIF
        D121=XEND*(FREQCO(NFREQC)-FRCMIN)
        DO 120 IJ=NFREQC+1,NFREQC+NEND-1
          FREQCO(IJ)=FREQCO(IJ-1)-D121
          IJXCO(IJ)=2
          WCHCO(IJ)=0.
  120   CONTINUE
        IJXCO(NFREQC+NEND-1)=1
        DO 130 IJ=NFREQC+1,NFREQC+NEND-2,2
           WCO(IJ)=FTH*D121
           WCO(IJ-1)=WCO(IJ-1)+THIRD*D121
           WCO(IJ+1)=WCO(IJ+1)+THIRD*D121
  130   CONTINUE
        WCHCO(NFREQC)=THIRD*D121
        NFREQC=NFREQC+NEND-1
        GO TO 200
      ENDIF
      DF0=FRLEV(IL0)+0.1*(FREQCO(NFREQC)-FRC0)
      FRTL=1.000001*FRLEV(IL0)
      IF(FRC0.GT.DF0) THEN
        NFREQC=NFREQC+1
        FREQCO(NFREQC)=FRC0
        IJXCO(NFREQC)=2
        WCO(NFREQC)=WCO(NFREQC)+HALF*(FREQCO(NFREQC-1)-FREQCO(NFREQC))
        WCO(NFREQC-1)=WCO(NFREQC-1)+
     *                HALF*(FREQCO(NFREQC-1)-FREQCO(NFREQC))
        WCHCO(NFREQC-1)=HALF*(FREQCO(NFREQC-1)-FREQCO(NFREQC))
      ELSE IF(FRTL.LT.FREQCO(NFREQC)) THEN
        NFREQC=NFREQC+2
        FREQCO(NFREQC-1)=FRTL
        FREQCO(NFREQC)=0.999999*FRLEV(IL0)
        IJXCO(NFREQC-1)=1
        IJXCO(NFREQC)=1
        WCO(NFREQC)=WCO(NFREQC)+HALF*(FREQCO(NFREQC-1)-FREQCO(NFREQC))
        WCO(NFREQC-1)=WCO(NFREQC-1)+
     *                HALF*(FREQCO(NFREQC-2)-FREQCO(NFREQC))
        WCO(NFREQC-2)=WCO(NFREQC-2)+
     *                HALF*(FREQCO(NFREQC-2)-FREQCO(NFREQC-1))
        WCHCO(NFREQC-1)=HALF*(FREQCO(NFREQC-1)-FREQCO(NFREQC))
        WCHCO(NFREQC-2)=HALF*(FREQCO(NFREQC-2)-FREQCO(NFREQC-1))
        ILS=IENS(NLEVEL-IL0+1)
        IJFL(ILS)=NFREQC-1
        IL0=IL0+1
      ELSE
        ILS=IENS(NLEVEL-IL0+1)
        IJFL(ILS)=NFREQC-1
        IL0=IL0+1
      ENDIF
      GO TO 100
C       
  200 SUMWC=0.
      DO 300 IJ=1,NFREQC
        IJALI(IJ)=1
  300 CONTINUE
C
      IF(NFREAD.GT.0) THEN
      DO IJ=NFREQ,1,-1 
         FREQ(IJ+NFREQC)=FREQ(IJ)
         W(IJ+NFREQC)=W(IJ)
         PROF(IJ+NFREQC)=PROF(IJ)    
         WSUB(IJ+NFREQC)=WSUB(IJ)
         IJALI(IJ+NFREQC)=IJALI(IJ)
      END DO
      DO IJ=1,NFREQC
         FREQ(IJ)=FREQCO(IJ)
         W(IJ)=WCO(IJ)
         WCH(IJ)=WCHCO(IJ)
         IJX(IJ)=IJXCO(IJ)
         PROF(IJ)=0.
         WSUB(IJ)=0.
      END DO
      DO 320 ITR=1,NTRANS
         IF(.NOT.LINE(ITR).OR.INDEXP(ITR).EQ.0) GO TO 320
         IF(IFR0(ITR).GT.0) IFR0(ITR)=IFR0(ITR)+NFREQC
         IF(IFR1(ITR).GT.0) IFR1(ITR)=IFR1(ITR)+NFREQC
  320 CONTINUE
      NFREQ=NFREQ+NFREQC
      END IF
C
C     determination of the first (IFR0) and the last (IFR1) frequency
C     for each explicit continuum - only if they were not already read
C
      DO 340 ITR=1,NTRANS
         IF(LINE(ITR)) GO TO 340
            IF(IFR0(ITR).LE.0) IFR0(ITR)=1
            IF(IFR1(ITR).GT.0) GO TO 340
            IF1=0
            FR01=FR0(ITR)
            MODE=INDEXP(ITR)
            IF(IABS(MODE).EQ.5.OR.IABS(MODE).EQ.15) FR01=FR0PC(ITR)
            DO 330 IJ=1,NFREQC
               IF(FREQ(IJ).GE.FR01) IF1=IJ
  330       CONTINUE
            IFR1(ITR)=IF1
  340 CONTINUE
C
      RETURN
      END
C
C
C     ****************************************************************
C
C
      FUNCTION CROSS(IBF,IJ)
C     ======================
C
C     Evaluation of the photoionization cross-section 
C     IBF - index ot the b-f transition
C     IJ  - frequency index
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'MODELQ.FOR'
C
      IJ0=IJBF(IJ)
      A1=AIJBF(IJ)
      CROSS=A1*BFCS(IBF,IJ0)+(UN-A1)*BFCS(IBF,IJ0+1)
      RETURN
      END
C
C
C     ****************************************************************
C
C
      SUBROUTINE RADPRE(IJ)
C     =====================
C
C     radiation pressure at the surface for the individual transitions
C
      INCLUDE 'IMPLIC.FOR'
      INCLUDE 'BASICS.FOR'
      INCLUDE 'ATOMIC.FOR'
      INCLUDE 'MODELQ.FOR'
      INCLUDE 'ODFPAR.FOR'
      INCLUDE 'ALIPAR.FOR'
      dimension gradln(mtrans)
C
C
C     for IJ > 0 - contribution from the frequency IJ to the 
C                  radiation pressure integral
C
      id=1
      IF(IJ.GT.0) THEN
         IF(IJ.EQ.1) THEN
            do itr=1,ntrans
               gradln(itr)=0.
            end do
         END IF
         FLUXW=W(IJ)*FH(IJ)*RAD1(ID)
         IF(IJLIN(IJ).GT.0) THEN
            ITR=IJLIN(IJ)
            SG=PRFLIN(IJ,ID)
            ABSO1(ID)=ABSO1(ID)+SG*ABTRA(ITR,ID)
            EMIS1(ID)=EMIS1(ID)+SG*EMTRA(ITR,ID)
            ABSO1(ID)=ABSO1(ID)-EMIS1(ID)*XKF(ID)
            EMIS1(ID)=EMIS1(ID)*XKFB(ID)
            gradln(itr)=gradln(itr)+fluxw*abso1(id)
         ENDIF
       ELSE
         grcum=0.
         do itr=1,ntrans
            grd=gradln(itr)/dens(id)
            if(iabs(indexp(itr)).ne.9) grdcum=grdcum+grd
            gr=0.
            grc=0.
            IF(GRD.GT.0.) GR=LOG10(GRD*4.1916825D-10)
            IF(GRDCUM.GT.0.) GRC=LOG10(GRDCUM*4.1916825D-10)
            write(83,601) itr,ilow(itr),iup(itr),gr,grc
  601       format(3i6,1p2e15.3)
         end do
      END IF
      RETURN
      END
